package com.cosylab.application;

import static java.lang.annotation.ElementType.TYPE;

import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * 
 * <code>AutoScalable</code> indicates that the class that is annotated with it can be automatically scaled. The
 * annotation itself does not provide any scaling capabilities it is just a hint to the toolkit.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(value = TYPE)
@Inherited
public @interface AutoScalable {
    
    /**
     * @return true if the layout in this container is fixed or false if it is managed by the layout manager
     */
    boolean fixedLayout() default false;
    
    /**
     * @return true if the fonts in this container are statically defined or false if they are defined by the look
     *          and feel
     */
    boolean staticFonts() default false;
    
    /**
     * @return the name of the method in the class that can be invoked to scale the internals not visible to the 
     *         outside world. If the name is an empty string it is ignored. The method is expected to receive the
     *         scaling factor as a float parameter.
     */
    String scaleMethodName() default "";
    
    /**
     * @return true if child components are added or removed in runtime or false otherwise
     */
    boolean childComponentsChangeInRuntime() default false;
}
