/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.application.state;

/**
 * This interface has to be implemented by the components which will exchange
 * its state with the <code>StateKeeper</code>
 *
 * @author dvitas
 */
public interface StateOriginator
{
	/**
	 * Will be called by the <code>StateKeeper</code> in the process of the
	 * application state saving. The component which implements this interface
	 * will tipycally use <code>StateFactory</code> to create a
	 * <code>State</code> object and fill it with needed values. For example a
	 * component can use this snippet.
	 * <pre>
	 * <code>
	 * public State getState() {
	 * 	State s = StateFactory.createState();
	 * 	s.putDouble("Min", getMinimum());
	 * 	s.putDouble("Max", getMaximum());
	 * 	s.putString("Format", getFormat());
	 * 	s.putString("Units", getUnits());
	 * 	return s;
	 * }
	 * </code>
	 * </pre>
	 *
	 * @return state object
	 */
	public State getState();

	/**
	 * Will be called by the <code>StateKeepr</code> when it reads
	 * <code>State</code> object which belongs to this component.
	 *
	 * @param state object previously created by this component
	 */
	public void setState(State state);
}

/* __oOo__ */
