/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.application.state.impl;

import com.cosylab.application.state.State;
import com.cosylab.application.state.StateStorage;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * This class is an abstract class that can be extended by the
 * <code>StateStorage</code> implementators.
 *
 * @author dvitas
 */
abstract public class DefaultStateStorage implements StateStorage
{
	protected ArrayList<State> states = new ArrayList<>();
	protected StateStorage delegate = null;

	/**
			 *
			 */
	public DefaultStateStorage()
	{
		super();
	}

	/**
	 * Creates a new DefaultStateStorage object.
	 *
	 * @param other DOCUMENT ME!
	 */
	public DefaultStateStorage(StateStorage other)
	{
		super();
		delegate = other;

		//states.addAll(other.getStates());
	}

	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateStorage#add(com.cosylab.application.state.State)
	 */
	public void add(State state)
	{
		states.add(state);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateStorage#add(java.util.ArrayList)
	 */
	public void addAll(List<State> states)
	{
		this.states.addAll(states);
	}

	protected InputStream getInputStream(String filePath, String fileName)
	{
		// if file exists
		File file = new File(filePath + File.separatorChar + fileName);

		if(!file.exists() || !file.canRead()) {
			return null;
		}

		BufferedInputStream bis;

		try {
			bis = new BufferedInputStream(new FileInputStream(file));
		} catch(FileNotFoundException e) {
			return null;
		}

		return bis;
	}

	protected OutputStream getOutputStream(String filePath, String fileName)
		throws IOException
	{
		// prepare the file object
		File file = new File(filePath + File.separatorChar + fileName);

		// make path to the file if it doesn't exists jet
		if(!file.getParentFile().exists()) {
			file.getParentFile().mkdirs();
		}

		// create output stream
		return new BufferedOutputStream(new FileOutputStream(file));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateStorage#load(java.io.InputStream)
	 */
	abstract public void load(String filePath, String applicationName)
		throws IOException;

	/**
	 * DOCUMENT ME!
	 *
	 * @param filePath DOCUMENT ME!
	 * @param applicationName DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	abstract public void store(String filePath, String applicationName)
		throws IOException;

	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateStorage#getStates()
	 */
	public List<State> getStates()
	{
		return states;
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateStorage#contains(com.cosylab.application.state.State)
	 */
	public boolean contains(State st) {
		return states.contains(st);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateStorage#iterator()
	 */
	public Iterator<State> iterator() {
		return states.iterator();
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateStorage#remove(com.cosylab.application.state.State)
	 */
	public void remove(State st) {
		states.remove(st);
	}
	
}

/* __oOo__ */
