package com.cosylab.cosydist.ant;

import java.io.File;
import java.util.Hashtable;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Delete;
import org.apache.tools.ant.taskdefs.Expand;
import org.apache.tools.ant.taskdefs.Jar;
import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.taskdefs.Manifest;
import org.apache.tools.ant.taskdefs.ManifestException;
import org.apache.tools.ant.taskdefs.Mkdir;
import org.apache.tools.ant.taskdefs.Jar.FilesetManifestConfig;
import org.apache.tools.ant.taskdefs.Manifest.Attribute;
import org.apache.tools.ant.taskdefs.Zip.Duplicate;
import org.apache.tools.ant.taskdefs.Zip.WhenEmpty;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

public class AntTasksHelper extends Task {

	// public static String[] findDependencies(Project p) throws IOException {
	// String home = p.getBaseDir().getPath();
	// File f = new File(home + p.getProperty("file.separator") + ".classpath");
	//
	// Vector<String> dependencies = new Vector<String>();
	// FileReader fr = new FileReader(f);
	// BufferedReader br = new BufferedReader(fr);
	// String s = "";
	// while (br.ready()) {
	// s = br.readLine();
	// if (s.indexOf("kind=\"lib\"") > -1 && s.indexOf("jar") > -1) {
	// int beginIndex = s.indexOf("path=\"") + 6;
	// int endIndex = s.indexOf("\"", beginIndex + 4);
	// if (beginIndex == -1 || endIndex == -1)
	// System.out.println(beginIndex + ", " + endIndex + ", " + s);
	// else {
	// dependencies.add(s.substring(beginIndex, endIndex));
	// }
	// }
	// }
	// String[] str = new String[dependencies.size()];
	// return dependencies.toArray(str);
	// }

	// /**
	// * Builds a jar file that contains all source files and classes of the
	// * current project, together with all the dependencies. The jar file will
	// be
	// * placed into &quot;target&quot; folder<br />
	// * Additional properties can be added:<br />
	// * <ul>
	// * <li>&lt;property name=&quot;version&quot; value=&quot;1.0&quot;
	// /&gt;</li>
	// * <li>&lt;property name=&quot;jar.name&quot;
	// * value=&quot;myProjectsJar&quot; /&gt;</li>
	// * </ul>
	// * If you add the &quot;jar.name&quot; property, the default name
	// * (${ant.project.name}-${version}.jar) will be replaced by value of that
	// * property. Custom jar name must be written without .jar ending.
	// *
	// * @param p
	// * current project
	// */
	// public static String buildJarFile(Project p) {
	// return buildJarFile(p, "target" + p.getProperty("file.separator")
	// + generateJarName(p) + ".jar");
	// }

	/**
	 * Builds a jar file that contains all source files and classes of the
	 * current project, together with all the dependencies. You can specify the
	 * name of the jar file and the location, where should it be found in the
	 * String target.<br />
	 * Additional properties can be added:<br />
	 * <ul>
	 * <li>&lt;property name=&quot;version&quot; value=&quot;1.0&quot; /&gt;</li>
	 * <li>&lt;property name=&quot;jar.name&quot;
	 * value=&quot;myProjectsJar&quot; /&gt;</li>
	 * </ul>
	 * If you add the &quot;jar.name&quot; property, the default name
	 * (${ant.project.name}-${version}.jar) will be replaced by value of that
	 * property. Custom jar name must be written without .jar ending.
	 * @param tempDir 
	 * @param jarFile 
	 */
	public static void buildJarFile(Project p, String compress, String keepcompression, 
			String encoding, String filesonly, String defaultexcludes, String manifestString, 
			String filesetmanifest, 
			String update, String whenmanifestonly, String duplicate, String index, String manifestencoding, 
			String roundup, String level, String mainclass, String version, Manifest manifest, File tempDir, File jarFile) {
		// creates jar file
		Jar j = new Jar();
		j.setTaskName("jar");
		j.setProject(p);
		
		j.setDestFile(jarFile);
		
		j.setBasedir(tempDir);
		
		if (compress != null && compress.length() != 0){
			j.setCompress(toBoolean(compress));
		}
		
		if (keepcompression != null && keepcompression.length() != 0) {
			j.setKeepCompression(toBoolean(keepcompression));
		}
		
		if (encoding != null && encoding.length() != 0) {
			j.setEncoding(encoding);
		}
		
		if (filesonly != null && filesonly.length() != 0) {
			j.setFilesonly(AntTasksHelper.toBoolean(filesonly));
		}
		
		if (manifest != null){
			try {
				j.addConfiguredManifest(manifest);
			} catch (ManifestException e) {
				e.printStackTrace();
			}
		}
		else if (manifestString != null && manifestString.length() != 0){
			j.setManifest(new File(manifestString));
		}
		else {
			// creates manifest file with (if given) main-class property (defines,
			// which class's main method should be called by default, for example
			// main-class=my.first.package.HelloWorld runs main method of the
			// HelloWorld class in my.first.package package)
			try {
				if (manifest == null) {
					manifest = new Manifest();
					Attribute mainClassAttribute = new Attribute("Main-Class", mainclass), 
						usernameAttribute = new Attribute("Built-By", p.getProperty("user.name")), 
						versionAttribute = new Attribute("Build-Version", (version != null
									&& !version.equals("") ? version : p
									.getProperty("version")));

					if (mainclass != null && mainclass.length() != 0){
						manifest.addConfiguredAttribute(mainClassAttribute);
					}
					manifest.addConfiguredAttribute(usernameAttribute);
					manifest.addConfiguredAttribute(versionAttribute);
				}
				j.addConfiguredManifest(manifest);
			} catch (ManifestException e) {
				e.printStackTrace();
			}
		}
		
		if (filesetmanifest != null && filesetmanifest.length() != 0){
			FilesetManifestConfig fmc = new FilesetManifestConfig();
			fmc.setValue(filesetmanifest);
			j.setFilesetmanifest(fmc);	
		}
		
		if (update != null && update.length() != 0) {
			j.setUpdate(AntTasksHelper.toBoolean(update));
		}
		
		if (whenmanifestonly != null && whenmanifestonly.length() != 0) {
			WhenEmpty we = new WhenEmpty();
			we.setValue(whenmanifestonly);
			j.setWhenmanifestonly(we);
		}
		
		if (duplicate != null && duplicate.length() != 0) {
			Duplicate d = new Duplicate();
			d.setValue(duplicate);
			j.setDuplicate(d);
		}
		
		if (index != null && index.length() != 0) {
			j.setIndex(AntTasksHelper.toBoolean(index));
		}
		
		if (manifestencoding != null && manifestencoding.length() != 0) {
			j.setManifestEncoding(manifestencoding);
		}
		
		if (roundup != null && roundup.length() != 0) {
			j.setRoundUp(AntTasksHelper.toBoolean(roundup));
		}
		
		if (level != null && level.length() != 0) {
			j.setLevel(Integer.parseInt(level));
		}
		
		if (defaultexcludes != null && defaultexcludes.length() != 0) {
			j.setDefaultexcludes(AntTasksHelper.toBoolean(defaultexcludes));
		}
		// TODO: insert missing properties
		
		
		j.execute();
	}

	/**
	 * Extracts all the dependencies to the temporary folder.<br />
	 * Multiple dependencies can be defined at the same time, separated with
	 * ":", ";" or ",".<br />
	 * <b>Example:</b><br />
	 * <ul>
	 * <li>&lt;property name=&quot;dependencies&quot;
	 * value=&quot;lib/dependency1.jar&quot; /&gt;</li>
	 * <li>&lt;property name=&quot;dependencies&quot;
	 * value=&quot;lib/dependency1.jar: lib/dependency2.jar&quot; /&gt;</li>
	 * </ul>
	 * 
	 * @param p
	 *            current project
	 * @param classpathref
	 * @param tempDir 
	 * @param jarAssamblerJarName 
	 */
	public static void extractDependencies(Project p, String classpath,
			String classpathref, File tempDir, String jarAssamblerJarName) {
		Expand e = new Expand();
		e.setProject(p);
		e.setTaskName("unjar");
		createDirectory(p, tempDir);
		e.setDest(tempDir);
		if (classpath != null && !classpath.equals("")) {
			//System.out.println("CLASSPATH");
			String[] cpath = classpath.split(p.getProperty("path.separator"));
			for (int i = 0; i < cpath.length; i++) {
				e.setSrc(new File(cpath[i]));
				e.execute();
			}
		} else if (classpathref != null && !classpathref.equals("")) {
			Path path = (Path) p.getReference(classpathref);
			String[] cpath = path.toString().split(p.getProperty("path.separator"));
			for (int i = 0; i < cpath.length; i++) {
				File f= new File(cpath[i]);
				//System.out.println(f);
				if (f.isDirectory()) {
					//System.out.println("CLASSPATHREF, no .jar " + cpath[i]);
					FileSet fs = new FileSet();
					fs.setExcludes(jarAssamblerJarName);
					fs.setIncludes("*.jar");
					fs.setDir(new File(cpath[i]));
					String[] files = fs.getDirectoryScanner(p)
							.getIncludedFiles();
					for (int j = 0; j < files.length; j++) {
						e.setSrc(new File(fs.getDir().getPath()
								+ p.getProperty("file.separator") + files[j]));
						e.execute();
					}
				} else {
					if (cpath[i].endsWith(".jar")) {
						e.setSrc(f);
						e.execute();
					} else {
						Copy c = new Copy();
						c.setProject(p);
						c.setTodir(tempDir);
						c.setFile(f);
						c.execute();
					}
				}
			}
		}

		// String[] dependencies = null;
		// try {
		// dependencies = findDependencies(p);
		// } catch (IOException e) {
		// }
		//
		// String depend = p.getProperty("dependencies");
		// if (depend != null && !depend.equals("")) {
		// String delimeter = ":";
		// if (depend.indexOf(';') > -1)
		// delimeter = ";";
		// else if (depend.indexOf(',') > -1)
		// delimeter = ",";
		//
		// dependencies = depend.split(delimeter);
		// }
		// UnJar uj = new UnJar();
		// uj.setProject(p);
		// uj.setDest(new File(buildTemp(p)));
		// for (int i = 0; i < dependencies.length; i++) {
		// uj.setSrc(new File(dependencies[i].trim()));
		// uj.execute();
		// }
	}

	/**
	 * Deletes temporary files.
	 * 
	 * @param p
	 *            current project
	 * @param tempDir 
	 */
	public static void deleteTemporaryFiles(Project p, File tempDir) {
		Delete d = new Delete();
		d.setProject(p);
		d.setTaskName("delete");
		d.setDir(tempDir);
		d.execute();
	}

	/**
	 * Copies all the *.java files from places, defined with property named src,
	 * to temporary folder. Multiple places can be defined together, separated
	 * with ";" or ":" (operating system's default path separator).<br />
	 * <b>Example:</b><br />
	 * <ul>
	 * <li>&lt;property name=&quot;src&quot; value=&quot;src/java&quot; /&gt;</li>
	 * <li>&lt;property name=&quot;src&quot; value=&quot;src/java,
	 * src/test&quot; /&gt;</li>
	 * </ul>
	 * <br />
	 * Temporary folder can be set as &lt;property name=&quot;temp.folder&quot;
	 * value=&quot;folder/temp&quot; /&gt;. If not set, the default folder
	 * &quot;target/jarExtract&quot; is used.
	 * 
	 * @param p
	 *            current project
	 * @param tempDir 
	 */
	public static void copyJavaFiles(Project p, FileSet src, String srcdir, String pathSeparator, File tempDir) {
		Copy c = new Copy();
		c.setProject(p);
		c.setTodir(tempDir);
		if (src != null){
			c.addFileset(src);
		}
		else {
			String[] srcList = srcdir.split(pathSeparator);
			FileSet fs = new FileSet();
			fs.setIncludes("**/*.java");
			for (int i = 0; i < srcList.length; i++) {
				fs.setDir(new File(srcList[i].trim()));
				c.addFileset(fs);
			}
		}
		c.execute();
	}
	
	public static void copyNonJavaFiles(Project p, FileSet src, String srcdir, String pathSeparator, String includes, String excludes, File tempDir, Object jarAssamblerJarName) {
		Copy c = new Copy();
		c.setProject(p);
		c.setTodir(tempDir);
		if (src != null){
			c.addFileset(src);
		}
		else {
			String[] srcList = srcdir.split(pathSeparator);
			FileSet fs = new FileSet();
			fs.setIncludes(includes);
			StringBuilder sb = new StringBuilder("**/*.java **/*.class **/");
			sb.append(jarAssamblerJarName);
			sb.append(" ");
			sb.append(excludes);
			fs.setExcludes(sb.toString());
			
			for (int i = 0; i < srcList.length; i++) {
				fs.setDir(new File(srcList[i].trim()));
				c.addFileset(fs);
			}
		}
		c.execute();
	}

	/**
	 * Compiles all java files from source folders and puts classes into the
	 * temporary folder.<br />
	 * Multiple source folders can be defined at the same time, separated with
	 * "," (comma).<br />
	 * <b>Example:</b><br />
	 * <ul>
	 * <li>&lt;property name=&quot;src&quot; value=&quot;src/java&quot; /&gt;</li>
	 * <li>&lt;property name=&quot;src&quot; value=&quot;src/java,
	 * src/test&quot; /&gt;</li>
	 * </ul>
	 * 
	 * @param p
	 *            current project
	 */
//	public static void compileJavaFiles(Project p) {
//		compileJavaFiles(p, null, null, null, null, null, null);
//	}

	/**
	 * @param p
	 * @param includes
	 *            pattern that presents files that should be included in compile
	 * @param excludes
	 *            pattern that presents files that should be excluded from
	 *            compile
	 * @param mainclass
	 * @param compress
	 * @param jardir
	 * @param jarprefix
	 * @param source
	 * @param debug
	 * @param classpathref
	 * @param classpath
	 * @param srcdir
	 */
	/*public static void compileJavaFiles(Project p, String includes,
			String excludes, String srcdir, String classpath,
			String classpathref, String debug) {
		Javac jc = new Javac();
		jc.setProject(p);
		jc.setDestdir(buildTemp(p));
		jc.setTaskName("javac");
		if (includes != null && !includes.equals(""))
			jc.setIncludes(includes);
		if (excludes != null && !excludes.equals(""))
			jc.setExcludes(excludes);
		if (srcdir != null && !srcdir.equals(""))
			jc.setSrcdir(new Path(p, srcdir));
		if (classpath != null && !classpath.equals(""))
			jc.setClasspath(new Path(p, classpath));
		if (classpathref != null && !classpathref.equals("")) {
			Reference r = new Reference(p, classpathref);
			String[] ref = r.getReferencedObject().toString().split(p.getProperty("path.separator"));
			StringBuilder reference = new StringBuilder();
			for (int i = 0; i < ref.length; i++) {
				if (!ref[i].endsWith(".jar")) {
					FileSet fs = new FileSet();
					fs.setDir(new File(ref[i]));
					fs.setExcludes(projectJarName);
					fs.setIncludes("*.jar:*.java:*.class");
					String[] includedFiles = fs.getDirectoryScanner(p)
							.getIncludedFiles();
					if (includedFiles.length == 0)
						continue;
					StringBuilder refer = new StringBuilder(includedFiles[0]);
					for (int j = 1; j < includedFiles.length; j++) {
						refer.append(p.getProperty("path.separator"));
						refer.append(includedFiles[j]);
					}
					if (reference.length() == 0){
						reference.append(refer);
					}
					else{
						reference.append(p.getProperty("path.separator"));
						reference.append(refer);
					}
				} else {
					if (reference.length() == 0){
						reference.append(ref[i]);
					}
					else{
						reference.append(p.getProperty("path.separator"));
						reference.append(ref[i]);
					}
				}
			}
			System.out.println("reference: " + reference.toString());
			jc.setClasspath(new Path(p, reference.toString()));
		}
		if (debug != null && !debug.equals(""))
			jc.setDebug(debug.equalsIgnoreCase("true")
					|| debug.equalsIgnoreCase("yes")
					|| debug.equalsIgnoreCase("on") ? true : false);

		jc.execute();
	}*/

	// private static String[] findSrc(Project p) throws IOException{
	// String home = p.getBaseDir().getPath();
	// File f = new File(home + p.getProperty("file.separator") + ".classpath");
	//
	// Vector<String> srcs = new Vector<String>();
	// FileReader fr = new FileReader(f);
	// BufferedReader br = new BufferedReader(fr);
	// String s = "";
	// while (br.ready()) {
	// s = br.readLine();
	// if (s.indexOf("kind=\"src\"") > -1) {
	// int beginIndex = s.indexOf("path=\"") + 6;
	// int endIndex = s.indexOf("\"", beginIndex);
	// if (beginIndex == -1 || endIndex == -1)
	// System.out.println(beginIndex + ", " + endIndex + ", " + s);
	// else {
	// srcs.add(s.substring(beginIndex, endIndex));
	// }
	// }
	// }
	// String[] str = new String[srcs.size()];
	// return srcs.toArray(str);
	// }

//	public static void buildZipFile(Project p, String target) {
//		Zip z = new Zip();
//		z.setProject(p);
//		z.setDestFile(new File(target));
//		z.setBasedir(new File(buildTemp(p)));
//		z.execute();
//	}

//	public static void buildTarFile(Project p, String target) {
//		Tar z = new Tar();
//		z.setProject(p);
//		z.setDestFile(new File(target));
//		z.setBasedir(new File(buildTemp(p)));
//		z.execute();
//	}

	public static void createDirectory(Project p, File directory) {
		Mkdir mkdir = new Mkdir();
		mkdir.setTaskName("mkdir");
		mkdir.setProject(p);
		mkdir.setDir(directory);
		mkdir.execute();
	}
	
	public static Path ReferenceToPath(Project p, String referenceString, String pathSeparator, String PROJECT_JAR_NAME){
		Reference r = new Reference(p, referenceString);
		String[] ref = r.getReferencedObject().toString().split(pathSeparator);
		StringBuilder reference = new StringBuilder();
		File f = null;
		for (int i = 0; i < ref.length; i++) {
			f = new File(ref[i]);
			if (f.isDirectory()) {
				FileSet fs = new FileSet();
				fs.setDir(new File(ref[i]));
				fs.setExcludes(PROJECT_JAR_NAME);
				fs.setIncludes("*.jar:*.java:*.class");
				String[] includedFiles = fs.getDirectoryScanner(p).getIncludedFiles();
				if (includedFiles.length == 0){
					continue;
				}
				StringBuilder refer = new StringBuilder(includedFiles[0]);
				for (int j = 1; j < includedFiles.length; j++){
					refer.append(pathSeparator);
					refer.append(includedFiles[j]);
				}
				if (reference.length() == 0){
					reference.append(refer);
				}
				else{
					reference.append(pathSeparator);
					reference.append(refer);
				}
			} else {
				if (reference.length() == 0){
					reference.append(ref[i]);
				}
				else{
					reference.append(pathSeparator);
					reference.append(ref[i]);
				}
			}
		}
		return new Path(p, reference.toString());
	}
	
	public static boolean toBoolean(String s){
		return s.equalsIgnoreCase("true")
			|| s.equalsIgnoreCase("yes")
			|| s.equalsIgnoreCase("on") ? true : false;
	}

	public static void compileJavaFiles(Project p, String srcdir, String includes, String includesfile, 
			String excludes, String excludesfile, String classpathString, String sourcepath, 
			String bootclasspath, String classpathref, String sourcepathref, 
			String bootclasspathref, String extdirs, String encoding, String nowarn, 
			String debug, String debuglevel, String optimize, String deprecation, String target,
			String verbose, String depend, String includeAntRuntime, String includeJavaRuntime, 
			String fork, String executable, String memoryInitialSize, String memoryMaximumSize, 
			String failonerror, String source, String compiler, String listfiles, String tempdir,
			FileSet src, Path classpath,
			String pathSeparator, String PROJECT_JAR_NAME, File tempDir) {
		
		Javac jc = new Javac();
		jc.setProject(p);
		jc.setDestdir(tempDir);
		jc.setTaskName("javac");
		if (includes != null && !includes.equals("")){
			jc.setIncludes(includes);
		}
		if (excludes != null && !excludes.equals("")){
			jc.setExcludes(excludes);
		}
		
		// get source folders from <src path="..."/> tags
		if (src != null){
			jc.setSrcdir(new Path(p, src.getDir(p).toString()));
			String[] excludesList = src.mergeExcludes(p);
			if (excludesList != null && excludesList.length > 0){
				StringBuilder excl = new StringBuilder(excludesList[0]);
				for (int i = 1; i < excludesList.length; i++) {
					excl.append(" ");
					excl.append(excludesList[i]);
				}
				jc.setExcludes(excl.toString());
			}
			String[] includesList = src.mergeIncludes(p);
			if (includesList != null && includesList.length > 0){
				StringBuilder incl = new StringBuilder(includesList[0]);
				for (int i = 1; i < includesList.length; i++) {
					incl.append(" ");
					incl.append(includesList[i]);
				}
				jc.setExcludes(incl.toString());
			}
		}
		// set source directory with <jar_assambler srcdir="..." />
		else if(srcdir != null && !srcdir.equals("")) {
			jc.setSrcdir(new Path(p, srcdir));
		}
		
		if (sourcepath != null){
			jc.setSourcepath(new Path(p, sourcepath));
		}
		
		// set classpath either with <jar_assambler classpath="..." />
		if (classpath != null) {
			jc.setClasspath(classpath);
		}
		// or with <classpath><pathelement path="${classpath}"/></classpath>
		else if (classpathString != null && !classpathString.equals("")) {
			jc.setClasspath(new Path(p, classpathString));
		}
		// or with <jar_assambler classpathref="..." />
		else if (classpathref != null && !classpathref.equals("")) {
			jc.setClasspath(AntTasksHelper.ReferenceToPath(p, classpathref, pathSeparator, PROJECT_JAR_NAME));
		}
		
		if (excludesfile != null){
			jc.setExcludesfile(new File(excludesfile));
		}
		
		if (includesfile != null){
			jc.setExcludesfile(new File(includesfile));
		}
		
		if (target != null){
			jc.setTarget(target);
		}
		
		if (bootclasspath != null){
			jc.setBootclasspath(new Path(p, bootclasspath));
		}
		
		if (sourcepathref != null){
			jc.setSourcepath(AntTasksHelper.ReferenceToPath(p, sourcepathref, pathSeparator, PROJECT_JAR_NAME));
		}
		
		if (bootclasspathref != null){
			jc.setBootclasspath(AntTasksHelper.ReferenceToPath(p, bootclasspathref, pathSeparator, PROJECT_JAR_NAME));
		}
		
		if (extdirs != null){
			jc.setExtdirs(new Path(p, extdirs));
		}
		
		if (encoding != null){
			jc.setEncoding(encoding);
		}
		
		if (nowarn != null){
			jc.setNowarn(AntTasksHelper.toBoolean(nowarn));
		}
		
		if (debuglevel != null){
			jc.setDebugLevel(debuglevel);
		}
		
		if (optimize != null) {
			jc.setOptimize(AntTasksHelper.toBoolean(optimize));
		}
		
		if (deprecation != null) {
			jc.setDeprecation(AntTasksHelper.toBoolean(deprecation));

		}
		
		if (verbose != null) {
			jc.setVerbose(AntTasksHelper.toBoolean(verbose));
		}
		
		if (depend != null) {
			jc.setDepend(AntTasksHelper.toBoolean(depend));
		}
		
		if (includeAntRuntime != null) {
			jc.setIncludeantruntime(AntTasksHelper.toBoolean(includeAntRuntime));
		}
		
		if (includeJavaRuntime != null) {
			jc.setIncludejavaruntime(AntTasksHelper.toBoolean(includeJavaRuntime));
		}
		
		if (fork != null) {
			jc.setFork(AntTasksHelper.toBoolean(fork));
		}
		
		if (executable != null) {
			jc.setExecutable(executable);
		}
		
		if (memoryInitialSize != null) {
			jc.setMemoryInitialSize(memoryInitialSize);
		}
		
		if (memoryMaximumSize != null) {
			jc.setMemoryMaximumSize(memoryMaximumSize);
		}
		
		if (failonerror != null) {
			jc.setFailonerror(AntTasksHelper.toBoolean(failonerror));
		}
		
		if (source != null) {
			jc.setSource(source);
		}
		
		if (compiler != null) {
			jc.setCompiler(compiler);
		}
		
		if (listfiles != null){
			jc.setListfiles(AntTasksHelper.toBoolean(listfiles));
		}
		
		if (tempdir != null) {
			jc.setTempdir(new File(tempdir));
		}
		
		if (debug != null && !debug.equals("")){
			jc.setDebug(AntTasksHelper.toBoolean(debug));
		}
		
		
		jc.execute();
	}
	
	public static String[][] prepareStringReplacementProperties(Project p){
		Hashtable<String, String> props = p.getProperties();
		
		Object[] keysObject = props.keySet().toArray();
		
		String[] keys = new String[keysObject.length];
		
		for (int i = 0; i < keys.length; i++) {
			keys[i] = keysObject[i].toString();
		}
		String[][] replacementTable = new String[keys.length][2];
		
		StringBuilder sb;
		String start = "\\$\\{";
		String end="\\}";
		
		for (int i = 0; i < keys.length; i++) {
			sb = new StringBuilder(start);
			sb.append(keys[i]);
			sb.append(end);
			replacementTable[i][0] = sb.toString();
			replacementTable[i][1] = props.get(keys[i]);
			//System.out.println(">> '"+keys[i]+"' '"+sb.toString()+"' '"+props.get(keys[i])+"'");
		}
		return replacementTable;
	}
	
	public static String insertPropertiesIntoString(String str, String[][] replacement, Project p){
		return insertPropertiesIntoString(str, replacement);
	}
	
	public static String insertPropertiesIntoString(String s, String[][] replacement){
		
		if (s==null || s.length()==0) {
			return s;
		}
		
		for (int i = 0; i < replacement.length; i++) {
			s= s.replaceAll(replacement[i][0], replacement[i][1]);
		}
		return s;
	}
	
	public static File toJarFile(Project p, String jardir, String jarname, String jarprefix, String version) {
		StringBuilder sb= new StringBuilder(128);

		if (jardir != null && !jardir.equals("")){
			sb.append(jardir);
			sb.append(p.getProperty("file.separator"));
		}
		
		printJarName(p, jarname, jarprefix, version, sb);
		
		sb.append(".jar");
		
		return new File(sb.toString());
	}

	/**
	 * Generates temporary folder path.
	 * 
	 * @param p
	 *            current project
	 * @param jarprefix 
	 * @param version 
	 */
	public static File toTempFolder(Project p, String jarname, String jarprefix, String version) {
		String temp = p.getProperty("temp.folder");
		StringBuilder sb= new StringBuilder(128);
		
		if (temp == null || temp.equals("")) {
	
			sb.append("target");
			sb.append(p.getProperty("file.separator"));
			
			printJarName(p, jarname, jarprefix, version, sb);
			
		} else {
			sb.append(temp);
		}
	
		return new File(sb.toString());
	}

	/**
	 * @param p
	 * @param jarprefix
	 * @param version
	 * @param sb
	 */
	private static void printJarName(Project p, String jarname, String jarprefix,
			String version, StringBuilder sb) {

		if (jarname != null && jarname.length()>0){
			sb.append(jarname);
		} else {
			
			if (jarprefix!=null && jarprefix.length()>0) {
				sb.append(jarprefix);
			} else if (p.getProperty("ant.project.name") != null 
					&& !p.getProperty("ant.project.name").equals("")){
					
				sb.append(p.getProperty("ant.project.name"));

			} else {
				sb.append(p.getName());
			}
			
			String s= version;
			if (s == null || s.length()==0) {
				s= p.getProperty("version");
			}
			if (s != null && s.length()>0) {
				sb.append("-");
				sb.append(s);
			}
		}
		
	}
}
