package com.cosylab.cosydist.ant;

import java.io.File;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Manifest;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;

/**
 * Generates jar file that includes all the dependencies, provided in the same
 * project.
 * 
 * @author Rok Koncina
 * 
 */
public class JarAssambler extends Task {
	private final static String PROJECT_JAR_NAME = "cosydist.jar";

	// settings, inherited from javac
	private String srcdir = null, jardir = null, includes = null,
			includesfile = null, excludes = null, excludesfile = null,
			classpathString = null, sourcepath = null, bootclasspath = null,
			classpathref = null, sourcepathref = null, bootclasspathref = null,
			extdirs = null, encoding = null, nowarn = null, debug = null,
			debuglevel = null, optimize = null, deprecation = null,
			target = null, verbose = null, depend = null,
			includeAntRuntime = null, includeJavaRuntime = null, fork = null,
			executable = null, memoryInitialSize = null,
			memoryMaximumSize = null, failonerror = null, source = null,
			compiler = null, listfiles = null, tempdir = null;

	// packaging settings
	private String jarprefix = null, compress = null, keepcompression = null,
			filesonly = null, defaultexcludes = null, manifestString = null,
			filesetmanifest = null, update = null, whenmanifestonly = null,
			duplicate = null, index = null, manifestencoding = null,
			roundup = null, level = null, mainclass = null,
			jarname = null;

	// other settings
	private String pathSeparator = null, version = null;
	private Manifest manifest = null;
	private FileSet src = null;
	private Path classpath = null;
	private String[][] properties = null;
	private String includesources = null;
	
	
	@Override
	public void execute() {
		prepare();
		
		File tempDir= AntTasksHelper.toTempFolder(getProject(), jarname, jarprefix, version);
		File jarFile= AntTasksHelper.toJarFile(getProject(), jardir, jarname, jarprefix, version);

		
		AntTasksHelper.extractDependencies(getProject(), classpathString,
				classpathref, tempDir, PROJECT_JAR_NAME);

		AntTasksHelper.compileJavaFiles(getProject(), srcdir, includes,
				includesfile, excludes, excludesfile, classpathString,
				sourcepath, bootclasspath, classpathref, sourcepathref,
				bootclasspathref, extdirs, encoding, nowarn, debug, debuglevel,
				optimize, deprecation, target, verbose, depend,
				includeAntRuntime, includeJavaRuntime, fork, executable,
				memoryInitialSize, memoryMaximumSize, failonerror, source,
				compiler, listfiles, tempdir, src, classpath, pathSeparator,
				PROJECT_JAR_NAME,tempDir);

		AntTasksHelper.copyNonJavaFiles(getProject(), src, srcdir, pathSeparator, includes, excludes,tempDir, PROJECT_JAR_NAME);

		if ("true".equalsIgnoreCase(getProject().getProperty(
				"include.java.files"))) {
			AntTasksHelper.copyJavaFiles(getProject(), src, srcdir, pathSeparator,tempDir);
		}

		AntTasksHelper.buildJarFile(getProject(), compress,
				keepcompression, encoding, filesonly, defaultexcludes,
				manifestString, filesetmanifest, update, whenmanifestonly,
				duplicate, index, manifestencoding, roundup, level, mainclass,
				version, manifest, tempDir, jarFile);

		if ("false".equalsIgnoreCase(getProject().getProperty("temp.delete"))) {
			log("Temporary files are not deleted.");
		}
		else {
			AntTasksHelper.deleteTemporaryFiles(getProject(),tempDir);
		}
	}
	
	private void prepare(){
		super.execute();
		if (pathSeparator == null){
			pathSeparator = getProject().getProperty("path.separator");
		}
		if (srcdir == null){
			srcdir = getProject().getProperty("src");
		}
		if (srcdir == null) {
			srcdir = getProject().getProperty("basedir");
		}
		if (tempdir != null && !tempdir.equals("")){
			getProject().setProperty("temp.folder", tempdir);
		}
		
	}
	
	public String[][] getProperties() {
		if (properties==null) {
			properties= AntTasksHelper.prepareStringReplacementProperties(getProject());
		}
		return properties;
	}

	// SETTERS AND GETTERS

	public void setTarget(String target) {
		this.target = AntTasksHelper.insertPropertiesIntoString(target, getProperties(), getProject());
	}

	public void setIncludes(String s) {
		this.includes = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setExcludes(String s) {
		this.excludes = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setSrcdir(String s) {
		this.srcdir = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setClasspath(String s) {
		pathSeparator = getProject().getProperty("path.separator");
		this.classpathString = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject()).replaceAll(";", pathSeparator).replaceAll(":",
				pathSeparator);
	}

	public void setClasspathref(String s) {
		this.classpathref = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setDebug(String s) {
		this.debug = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setSource(String s) {
		this.source = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setJarprefix(String s) {
		this.jarprefix = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setJardir(String s) {
		this.jardir = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setDestdir(String s) {
		this.jardir = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setCompress(String s) {
		this.compress = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setMainclass(String s) {
		this.mainclass = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	public void setVersion(String s) {
		this.version = AntTasksHelper.insertPropertiesIntoString(s, getProperties(), getProject());
	}

	// NESTED ELEMENTS

	public Manifest createManifest() {
		Manifest m = new Manifest();
		return m;
	}

	public void addManifest(Manifest m) {
		this.manifest = m;
	}

	public void addConfiguredManifest(Manifest m) {
		this.manifest = m;
	}

	// Path <-> FileSet
	public FileSet createSrc() {
		return new FileSet();
	}

	public void addSrc(FileSet src) {
		this.src = src;
	}

	public void addConfiguredSrc(FileSet src) {
		this.src = src;
	}

	//

	public Path createClasspath() {
		return new Path(getProject());
	}

	public void addClasspath(Path classpath) {
		this.classpath = classpath;
	}

	public void addConfiguredClasspath(Path classpath) {
		this.classpath = classpath;
	}

	public void setIncludesfile(String includesfile) {
		this.includesfile = AntTasksHelper.insertPropertiesIntoString(includesfile, getProperties(), getProject());
	}

	public void setExcludesfile(String excludesfile) {
		this.excludesfile = AntTasksHelper.insertPropertiesIntoString(excludesfile, getProperties(), getProject());
	}

	public void setBootclasspath(String bootclasspath) {
		this.bootclasspath = AntTasksHelper.insertPropertiesIntoString(bootclasspath, getProperties(), getProject());
	}

	public void setSourcepath(String sourcepath) {
		this.sourcepath = AntTasksHelper.insertPropertiesIntoString(sourcepath, getProperties(), getProject());
	}

	public void setSourcepathref(String sourcepathref) {
		this.sourcepathref = AntTasksHelper.insertPropertiesIntoString(sourcepathref, getProperties(), getProject());
	}

	public void setBootclasspathref(String bootclasspathref) {
		this.bootclasspathref = AntTasksHelper.insertPropertiesIntoString(bootclasspathref, getProperties(), getProject());
	}

	public void setExtdirs(String extdirs) {
		this.extdirs = AntTasksHelper.insertPropertiesIntoString(extdirs, getProperties(), getProject());
	}

	public void setEncoding(String encoding) {
		this.encoding = AntTasksHelper.insertPropertiesIntoString(encoding, getProperties(), getProject());
	}

	public void setNowarn(String nowarn) {
		this.nowarn = AntTasksHelper.insertPropertiesIntoString(nowarn, getProperties(), getProject());
	}

	public void setDebugLevel(String debuglevel) {
		this.debuglevel = AntTasksHelper.insertPropertiesIntoString(debuglevel, getProperties(), getProject());
	}

	public void setOptimize(String optimize) {
		this.optimize = AntTasksHelper.insertPropertiesIntoString(optimize, getProperties(), getProject());
	}

	public void setDeprecation(String deprecation) {
		this.deprecation = AntTasksHelper.insertPropertiesIntoString(deprecation, getProperties(), getProject());
	}

	public void setVerbose(String verbose) {
		this.verbose = AntTasksHelper.insertPropertiesIntoString(verbose, getProperties(), getProject());
	}

	public void setDepend(String depend) {
		this.depend = AntTasksHelper.insertPropertiesIntoString(depend, getProperties(), getProject());
	}

	public void setIncludeantruntime(String includeAntRuntime) {
		this.includeAntRuntime = AntTasksHelper.insertPropertiesIntoString(includeAntRuntime, getProperties(), getProject());
	}

	public void setIncludejavaruntime(String includeJavaRuntime) {
		this.includeJavaRuntime = AntTasksHelper.insertPropertiesIntoString(includeJavaRuntime, getProperties(), getProject());
	}

	public void setFork(String fork) {
		this.fork = AntTasksHelper.insertPropertiesIntoString(fork, getProperties(), getProject());
	}

	public void setExecutable(String executable) {
		this.executable = AntTasksHelper.insertPropertiesIntoString(executable, getProperties(), getProject());
	}

	public void setMemoryInitialSize(String memoryInitialSize) {
		this.memoryInitialSize = AntTasksHelper.insertPropertiesIntoString(memoryInitialSize, getProperties(), getProject());
	}

	public void setMemoryMaximumSize(String memoryMaximumSize) {
		this.memoryMaximumSize = AntTasksHelper.insertPropertiesIntoString(memoryMaximumSize, getProperties(), getProject());
	}

	public void setFailonerror(String failonerror) {
		this.failonerror = AntTasksHelper.insertPropertiesIntoString(failonerror, getProperties(), getProject());
	}

	public void setCompiler(String compiler) {
		this.compiler = AntTasksHelper.insertPropertiesIntoString(compiler, getProperties(), getProject());
	}

	public void setListfiles(String listfiles) {
		this.listfiles = AntTasksHelper.insertPropertiesIntoString(listfiles, getProperties(), getProject());
	}

	public void setManifest(String manifest) {
		this.manifestString = AntTasksHelper.insertPropertiesIntoString(manifest, getProperties(), getProject());
	}

	public void setIndex(String index) {
		this.index = AntTasksHelper.insertPropertiesIntoString(index, getProperties(), getProject());
	}

	public void setTempdir(String tempdir) {
		this.tempdir = AntTasksHelper.insertPropertiesIntoString(tempdir, getProperties(), getProject());
	}

	public void setKeepCompression(String keepcompression) {
		this.keepcompression = AntTasksHelper.insertPropertiesIntoString(keepcompression, getProperties(), getProject());
	}

	public void setFilesonly(String filesonly) {
		this.filesonly = AntTasksHelper.insertPropertiesIntoString(filesonly, getProperties(), getProject());
	}

	public void setDefaultexcludes(String defaultexcludes) {
		this.defaultexcludes = AntTasksHelper.insertPropertiesIntoString(defaultexcludes, getProperties(), getProject());
	}

	public void setFilesetmanifest(String filesetmanifest) {
		this.filesetmanifest = AntTasksHelper.insertPropertiesIntoString(filesetmanifest, getProperties(), getProject());
	}

	public void setUpdate(String update) {
		this.update = AntTasksHelper.insertPropertiesIntoString(update, getProperties(), getProject());
	}

	public void setWhenmanifestonly(String whenmanifestonly) {
		this.whenmanifestonly = AntTasksHelper.insertPropertiesIntoString(whenmanifestonly, getProperties(), getProject());
	}

	public void setDuplicate(String duplicate) {
		this.duplicate = AntTasksHelper.insertPropertiesIntoString(duplicate, getProperties(), getProject());
	}

	public void setManifestEncoding(String manifestencoding) {
		this.manifestencoding = AntTasksHelper.insertPropertiesIntoString(manifestencoding, getProperties(), getProject());
	}

	public void setRoundUp(String roundup) {
		this.roundup = AntTasksHelper.insertPropertiesIntoString(roundup, getProperties(), getProject());
	}

	public void setLevel(String level) {
		this.level = AntTasksHelper.insertPropertiesIntoString(level, getProperties(), getProject());
	}

	public void setPathSeparator(String pathSeparator) {
		this.pathSeparator = AntTasksHelper.insertPropertiesIntoString(pathSeparator, getProperties(), getProject());
	}
	
	public void setJarname(String jarname){
		this.jarname = AntTasksHelper.insertPropertiesIntoString(jarname, getProperties(), getProject());
	}

	public void setIncludesources(String includesources) {
		this.includesources = AntTasksHelper.insertPropertiesIntoString(includesources, getProperties(), getProject());
	}
}
