package com.cosylab.cosydist.maven;

import java.io.File;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.cosylab.cosydist.LicenseHeaderExecutor;
import com.cosylab.cosydist.Logger;


/**
 * This plugin checks all the .java files in a project if they have a proper
 * license at the beginning of the file.
 * 
 * @goal add-license
 */
public class HeadCheck extends AbstractMojo {

	/**
	 * Path to source directory.
	 * 
	 * @parameter expression="${project.build.sourceDirectory}"
	 */
	private String sourceDirectory = null;

	/**
	 * Path to license.
	 * 
	 * @parameter
	 */
	private File licenseFile = null;
	
	/** 
	 * @parameter
	 */
	private boolean showChangedFiles = true;
	
	/** 
	 * @parameter
	 */
	private boolean showSkippedFiles = false;
	
	/** 
	 * @parameter
	 */
	private boolean showNonJavaFiles = false;

	/*
	 * Executes plugin
	 * 
	 * @see org.apache.maven.plugin.AbstractMojo#execute()
	 */
	public void execute() throws MojoExecutionException {
		if (sourceDirectory == null || sourceDirectory.equals("")) {
			getLog().info("Nothing to do - no source directory defined.");
			return;
		}
		if (licenseFile == null) {
			getLog().info("Nothing to do - no license file defined.");
			return;
		}
		LicenseHeaderExecutor.attachLicense(sourceDirectory.split(" "), licenseFile, new Logger(), showChangedFiles, showSkippedFiles, showNonJavaFiles);
	}
}
