/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.events;

import java.util.EventObject;


/**
 * Notifys listeners that user set action was performed on number displayer.
 *
 * @author Ales Pucelj
 * @version $$id$$
 */
public class SetEvent extends EventObject
{
	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private final Number value;

	/**
	 * Constructor for SliderSetEvent.
	 *
	 * @param source Object
	 * @param newValue double
	 */
	public SetEvent(Object source, double newValue)
	{
		super(source);
		value = new Double(newValue);
	}

	/**
	 * Constructor for SliderSetEvent.
	 *
	 * @param source Object
	 * @param newValue Object
	 *
	 * @throws NullPointerException DOCUMENT ME!
	 */
	public SetEvent(Object source, Number newValue)
	{
		super(source);

		if (newValue == null) {
			throw new NullPointerException("newValue == null");
		}

		value = newValue;
	}

	/**
	 * Returns value which was set by user.
	 *
	 * @return Number
	 */
	public Number getValue()
	{
		return value;
	}

	/**
	 * Returns set value as double.
	 *
	 * @return double
	 */
	public double getDoubleValue()
	{
		return value.doubleValue();
	}

	/**
	 * Returns set value as long.
	 *
	 * @return double
	 */
	public long getLongValue()
	{
		return value.longValue();
	}
}

/* __oOo__ */
