/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;

/**
 * <code>IconPropertyEditor</code> is an implementation of the 
 * <code>java.beans.PropertyEditor</code> which enables setting of an icon.
 * @author Jaka Bobnar, Cosylab
 *
 */
public class IconPropertyEditor extends PropertyEditorSupport implements PropertyEditor {

	private IconEditor editor;
	 
	/**
	 * Creates new instance of SelectorPropertyEditor.
	 */
	public IconPropertyEditor() {
		super();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
    public Component getCustomEditor() {
		if (editor == null) {
			editor = new IconEditor();
			editor.setIcon((String)getValue());
			editor.addPropertyChangeListener(IconEditor.ICON,new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					setValueSilently(editor.getIconPath());
				}
			});
			editor.setSize(250,80);
		}
		
		return editor;
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
    public String getJavaInitializationString() {
		return "\"" + getAsText() + "\"";
//		if (getValue() == null) return "null";
//		String name = getValue().toString(); 
//		name = name.replace('\\','/');
//		int index = name.indexOf("!/");
//		if (index > 0) {
//			name = name.substring(index+2);
//		}
//		return "com.cosylab.gui.components.util.IconHelper.createIcon(\"" + name + "\")";
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
    public boolean supportsCustomEditor() {
	    return true;
    }
	
	private void setValueSilently(Object value) {
		super.setValue(value);
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
	 */
	@Override
    public void setValue(Object value) {
		super.setValue(value);
		if (editor!=null) {
			editor.setIcon((String)value);
		}
	}
	
}


/* __oOo__ */
