/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.adapters;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.beans.PropertyVetoException;

import com.cosylab.gui.displayers.ConvertibleDisplayer;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ConverterCustomizerPropertyEditor extends PropertyEditorSupport
		implements PropertyEditor {

	ConverterCustomizer customizer = null;

	public ConverterCustomizerPropertyEditor() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
    public Component getCustomEditor() {
		if (customizer == null) {
			customizer = new ConverterCustomizer();
			customizer.addPropertyChangeListener(ConvertibleDisplayer.CONVERTER_PROPERTY,
					new PropertyChangeListener() {
						public void propertyChange(PropertyChangeEvent e) {
							setValueSilently(customizer.getConverter());
						}
					});
		}
		return customizer;
	}

	private void setValueSilently(Converter value) {
		super.setValue(value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
	 */
	@Override
    public void setValue(Object value) {
		super.setValue(value);
		if (customizer != null) {
			customizer.setConverter((Converter) value);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#setAsText(java.lang.String)
	 */
	@Override
    public void setAsText(String text) {
		String[] values = text.split(";");
		Converter[] converters = null;
		if (values != null && values.length > 0) {
			converters = new Converter[values.length];
			for (int i = 0; i < values.length; i++) {
				converters[i] = ConverterUtilities.getConverterFromString(values[i]);
			}
		} else {
			converters = new Converter[]{ConverterUtilities.getConverterFromString(text)};
		}
		try {
			setValue(new ConverterChain(converters));
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
    public String getJavaInitializationString() {
		if (getValue() != null && getValue() instanceof Converter) {
			return ConverterUtilities.getInitializationString((Converter)getValue());
		} else {
			return "";
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
    public boolean supportsCustomEditor() {
		return true;
	}
}
