/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.adapters;

import java.util.Objects;

/**
 * <code>IdentityConverter</code> is ismplest possible conversion, it delegates same data value to consumers. 
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: IdentityConverter.java,v 1.12 2008-04-22 12:31:02 jbobnar Exp $
 * 
 * @since Feb 15, 2004.
 */
public class IdentityConverter extends SimpleConverterSupport {

	private static final long serialVersionUID = 583866778614153371L;
	public static final String SHORT_NAME = "Identity";
	/**
	 * Creates new identity converter.
	 * 
	 */
	public IdentityConverter() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.adapters.SimpleConverterSupport#transform(double)
	 */
	@Override
    protected double transform(double value) {
		return value;
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.adapters.SimpleConverterSupport#inverseTransform(double)
	 */
	@Override
    protected double inverseTransform(double value) {
		return value;
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.adapters.DataConsumerDispatcher#getName()
	 */
	@Override
    public String getName(){
		return SHORT_NAME;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
    public String toString(){
		return SHORT_NAME;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof IdentityConverter) return true;
		return false;
	}
	
	@Override
	public int hashCode() {
	    return Objects.hash(SHORT_NAME);
	}
}
