/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.cosylab.gui.components.util.PaintHelper;


/**
 * A simple resizable button identified by an arrow icon.
 *
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 *
 * @see com.cosylab.gui.components.SimpleButton
 */
public class ArrowButton extends SimpleButton
{
    private static final long serialVersionUID = -2014851617632168689L;

    public static final int PREFERRED_SIZE = 20;

    /** Arrow icon pointing upwards. */
	public static int UP = 0;

	/** Arrow icon pointing downwards. */
	public static int DOWN = 1;

	/** Arrow icon pointing to the left. */
	public static int LEFT = 2;

	/** Arrow icon pointing to the right. */
	public static int RIGHT = 3;
	
	public static enum Orientation {
		UP, DOWN, LEFT, RIGHT
	}
	
	private Insets insets;
	@Deprecated
	private int orientation;
	private Orientation orientationMode;

	/**
	 * Constructor for ArrowButton. Sets the orientation of the arrow icon.
	 *
	 * @param newOrientation int
	 * @deprecated 
	 */
	@Deprecated
	public ArrowButton(int newOrientation)
	{
		this(Orientation.values()[newOrientation]);
	}
	
	public ArrowButton(Orientation newOrientation)
	{
		super();

		orientation = newOrientation.ordinal();
		orientationMode = newOrientation;
		setMinimumSize(new Dimension(10, 10));
		setPreferredSize(new Dimension(PREFERRED_SIZE, PREFERRED_SIZE));
		insets = null;
		setActionMode(CHAIN_ACTION_MODE);
	}

	/**
	 * Sets the insets of the arrow icon.
	 *
	 * @param newInsets Insets
	 */
	public void setArrowInsets(Insets newInsets)
	{
		Insets oldInsets = insets;
		insets = newInsets;
		firePropertyChange("arrowInsets", oldInsets, newInsets);

		if (insets != oldInsets) {
			repaint();
		}
	}

	/**
	 * Returns the insets of the arrow icon.
	 *
	 * @return Insets
	 */
	public Insets getArrowInsets()
	{
		return insets;
	}

	/**
	 * This method has been overriden to implement arrow icon painting.
	 *
	 * @param g Graphics
	 */
	@Override
	public void paintComponent(Graphics g)
	{
		((Graphics2D)g).addRenderingHints(PaintHelper.getAntialiasingHints());
		super.paintComponent(g);
		
		if (!isVisible()) return;
		int maxHeight = getSize().height;
		int maxWidth = getSize().width;
		int top;
		int bottom;
		int left;
		int right;

		if (insets == null) {
			top = Math.min(maxWidth, maxHeight) / 4;
			bottom = Math.min(maxWidth, maxHeight) / 4;
			left = Math.min(maxWidth, maxHeight) / 4;
			right = Math.min(maxWidth, maxHeight) / 4;
		} else {
			top = insets.top;
			bottom = insets.bottom;
			left = insets.left;
			right = insets.right;
		}

		g.setColor(getForeground());
		int[] xs = null;
		int[] ys = null;
		if (orientationMode == Orientation.DOWN) {
			xs = new int[]{ left, maxWidth / 2, maxWidth - right };
			ys = new int[]{ top, maxHeight - bottom, top };
		} else if (orientationMode == Orientation.UP) {
			xs = new int[]{ left, maxWidth / 2, maxWidth - right };
			ys = new int[]{ maxHeight - bottom, top, maxHeight - bottom };
		} else if (orientationMode == Orientation.RIGHT) {
			ys = new int[]{ top, maxHeight / 2, maxHeight - bottom };
			xs = new int[]{ left, maxWidth - right, left };
		} else if (orientationMode == Orientation.LEFT) {
			ys = new int[]{ top, maxHeight / 2, maxHeight - bottom };
			xs = new int[]{ maxWidth - right, left, maxWidth - right };
		}
		
		g.fillPolygon(xs, ys, 3);
	}

	/**
	 * Returns the orientation of the arrow icon.
	 *
	 * @return int
	 * @deprecated use {@link #getOrientationMode()}
	 */
	@Deprecated
	public int getOrientation()
	{
		return orientation;
	}
	
	/**
	 * Sets the orientation of the arrow icon.
	 *
	 * @param newOrientation int
	 * @deprecated use {@link #setOrientationMode(Orientation)}
	 */
	@Deprecated
	public void setOrientation(int newOrientation)
	{
		int oldValue = this.orientation;
		setOrientationMode(Orientation.values()[newOrientation]);
		firePropertyChange("orientation", oldValue, orientationMode);
	}
	
	/**
	 * Returns the orientation mode.
	 * 
	 * @return the orientataion mode
	 */
	public Orientation getOrientationMode() {
		return orientationMode;
	}
	
	/**
	 * Sets the orientation mode.
	 * 
	 * @param orientationMode the orientation mode
	 */
	public void setOrientationMode(Orientation orientationMode) {
		if (this.orientationMode == orientationMode) return;
		Orientation oldValue = this.orientationMode;
		this.orientationMode = orientationMode;
		this.orientation = orientationMode.ordinal();
		firePropertyChange("orientationMode", oldValue, orientationMode);
		repaint();
	}

	/**
	 * Run test applet.
	 *
	 * @param args command line parameters
	 */
    public static void main(String[] args)
	{
		JApplet applet = new JApplet() {

			private static final long serialVersionUID = 1L;
			    @Override
				public void init()
				{
					Container cp = this.getContentPane();
					JPanel panel = new JPanel();
					panel.setLayout(new GridLayout(1, 0));
					panel.add(new ArrowButton(ArrowButton.Orientation.UP));
					panel.add(new ArrowButton(ArrowButton.Orientation.DOWN));
					panel.add(new ArrowButton(ArrowButton.Orientation.LEFT));
					panel.add(new ArrowButton(ArrowButton.Orientation.RIGHT));
					cp.add(panel);
				}
			};

		JFrame frame = new JFrame("Arrow Button Demo");
		frame.getContentPane().add(applet);
		frame.setSize(300, 400);
		frame.addWindowListener(new WindowAdapter() {
		    @Override
				public void windowClosing(WindowEvent e)
				{
					System.exit(0);
				}
			});
		applet.init();
		applet.start();
		frame.setVisible(true);
	}
}

/* __oOo__ */
