/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;

/**
 * <code>DialKnobCustomizer</code> is a customizer for <code>DialKnob</code>
 * which allows customization of its most common properties.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: DialKnobCustomizer.java,v 1.18 2008-04-22 12:28:40 jbobnar Exp $
 * @see DialKnob
 */
public class DialKnobCustomizer extends AbstractCustomizerPanel {
	private static final long serialVersionUID = 1L;
	
	/** Title of the basic visual tab. */
	public static final String VISUAL_BASIC = "Visual";

	/** Title of the value display tab. */
	public static final String VALUE_DISPLAY = "Value Display";
	
	/** Title of the auto synchronization tab. */
	public static final String AUTO_SYNCHRONIZATION = "Auto synchronization";
	
	/** Title of the value policy tab. */
	public static final String VALUE_POLICY = "Value Display/Value Range Policy";

	/** Title of the advance value display tab. */
	public static final String VALUE_DISPLAY_ADVANCED = "Value Display/Advanced";
	
	/** Supported aspects  */
	public static String[] ASPECTS = {
		VISUAL_BASIC, VALUE_DISPLAY, VALUE_DISPLAY_ADVANCED, VALUE_POLICY
	};

	/** Tag for titleVisible property */
	public static final String TITLE_VISIBLE = "titleVisible";

	/** Tag for title property */
	public static final String TITLE = "title";

	/** Tag for units property */
	public static final String UNITS = "units";

	/** Tag for unitsVisible property */
	public static final String UNITS_VISIBLE = "unitsVisible";

	/** Tag for format property */
	public static final String FORMAT = "format";

	/** Tag for graphMax property */
	public static final String GRAPH_MAX = "graphMax";

	/** Tag for graphMin property */
	public static final String GRAPH_MIN = "graphMin";

	/** Tag for maximum property */
	public static final String MAXIMUM = "maximum";

	/** Tag for minimum property */
	public static final String MINIMUM = "minimum";
	
	/** Tag for editable property */
	public static final String EDITABLE = "editable";

	/** Tag for resizable property */
	public static final String RESIZABLE = "resizable";

	/** Tag for enhanced property */
	public static final String ENHANCED = "enhanced";

	/** Tag for tiltingEnabled property */
	public static final String TILTING_ENABLED = "tiltingEnabled";
	
	public static final String ROTATION_TYPE = "rotationType";
	
	public static final String ENDLESS_SENSITIVITY = "endlessRotationTypeSensitivity";

	/** Names of the value display properties */
	public static String[] VALUE_DISPLAY_PROPERTIES = {
		MINIMUM, MAXIMUM, UNITS, UNITS_VISIBLE, FORMAT
	};

	/** Names of basic visual properties */
	public static String[] VISUAL_BASIC_PROPERTIES = {
		TITLE, TITLE_VISIBLE, RESIZABLE, ENHANCED, TILTING_ENABLED
	};

	/** Names of advanced properties */
	public static String[] ADVANCED_PROPERTIES = { EDITABLE, ROTATION_TYPE, ENDLESS_SENSITIVITY };	
	
	/**
	 * 
	 * <code>SyncPanel</code> enables configuring auto synchronization of the DialKnob.
	 *
	 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
	 * @version $Id: DialKnobCustomizer.java,v 1.18 2008-04-22 12:28:40 jbobnar Exp $
	 * 
	 * @since VERSION
	 */
	private class SyncPanel extends JPanel implements Customizer {
		private static final long serialVersionUID = 1L;
		private DialKnob displayer;
		private boolean initialized = false;
		private JPanel displayPanel;
		private NumberField autoSyncDelay;
		private JLabel millis;
		private JLabel syncDelay;
		private JRadioButton syncManual;
		private JRadioButton syncAuto;
		
		public SyncPanel(){
			if (displayer != null){
				initialize();
			}
		}
		
		private void initialize()
		{
			if (initialized) {
				return;
			}

			setLayout(new BorderLayout());

			add(getDisplayPanel(), BorderLayout.CENTER);
			updateEditor();
			initialized = true;

			setSize(300, 450);
		}
		
		private void updateEditor() {
			syncAuto.setSelected(displayer.isAutoSynchronize());
			syncManual.setSelected(!displayer.isAutoSynchronize());
			autoSyncDelay.setLongValue(displayer.getAutoSynchronizeDelay());
			autoSyncDelay.setEnabled(displayer.isAutoSynchronize());
		}
		
		private JPanel getDisplayPanel() {
			if (displayPanel == null) {
				displayPanel = new JPanel();
			}
			displayPanel.setBorder(new GroupBoxBorder("Synchronization configuration :"));
			displayPanel.setLayout(new GridBagLayout());
						
			syncManual = new JRadioButton("Manual synchronization");
			syncManual.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					autoSyncDelay.setEnabled(false);
					applySettings();
				}
				
				
			});
			syncAuto = new JRadioButton("Automatic synchronization");
			syncAuto.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					autoSyncDelay.setEnabled(true);	
					applySettings();
				}
				
				
			});
			ButtonGroup syncModeButtonGroup = new ButtonGroup();
			syncModeButtonGroup.add(syncManual);
			syncModeButtonGroup.add(syncAuto);
						
			displayPanel.add(syncManual, new GridBagConstraints(0,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			displayPanel.add(syncAuto, new GridBagConstraints(0,1,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			
			syncDelay = new JLabel("Automatic synchronization delay:");
			displayPanel.add(syncDelay, new GridBagConstraints(0,2,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,21,0,0), 1,1));
			
			JPanel fieldPanel = new JPanel();
			fieldPanel.setLayout(new GridBagLayout());
			autoSyncDelay = new NumberField();
			autoSyncDelay.setNumberType(long.class);
			autoSyncDelay.addCaretListener(new CaretListener(){

				public void caretUpdate(CaretEvent e) {
					applySettings();
				}
				
			});
			autoSyncDelay.setPreferredSize(new Dimension(70, 20));
			fieldPanel.add(autoSyncDelay, new GridBagConstraints(0,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			
			millis = new JLabel("milliseconds");
			fieldPanel.add(millis, new GridBagConstraints(1,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,10,0,0), 1,1));
			
			displayPanel.add(fieldPanel, new GridBagConstraints(0,3,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(5,21,0,0), 1,1));
			return displayPanel;
		}

		public void applySettings() {
			boolean auto = syncAuto.isSelected();
			displayer.setAutoSynchronize(auto);
			if (syncAuto.isSelected()) {
				displayer.setAutoSynchronizeDelay(autoSyncDelay.getLongValue());
				firePropertyChange("autoSynchronizeDelay", -1, autoSyncDelay.getLongValue());
			}
			firePropertyChange("autoSynchronize", !auto, auto);
			
		}

		public void setObject(Object arg0) {
			displayer = (DialKnob) arg0;
			initialize();
		}
		
	}
	
	public DialKnobCustomizer() {
		addCustomizerTable(VISUAL_BASIC, VISUAL_BASIC_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY, VALUE_DISPLAY_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY_ADVANCED, ADVANCED_PROPERTIES);
		addCustomizer(VALUE_POLICY, new ValuePolicyCustomizer());
		addCustomizer(AUTO_SYNCHRONIZATION, new SyncPanel());
		setSize(423, 184);
	}

}
