/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;

import com.cosylab.application.state.State;
import com.cosylab.gui.components.ledder.BitDescriptor;
import com.cosylab.gui.components.ledder.DefaultBitDescriptor;


/**
 * <code>LabelledLedder</code> ...  DOCUMENT ME!
 *
 * @author <a href="mailto:anze.zupanc@cosylab.com">Anze Zupanc</a>
 * @version $Id: LabelledLedder.java,v 1.24 2008-04-22 12:28:40 jbobnar Exp $
 *
 * @since Jul 20, 2004.
 */
public class LabelledLedder extends AbstractDisplayerPanel
{

	//private BitmaskField bitmask=null;
	private static final long serialVersionUID = 1L;

	/**
	 * DOCUMENT ME!
	 *
	 * @param args DOCUMENT ME!
	 */
	public static void main(String[] args)
	{
		LabelledLedder ll = new LabelledLedder("Labelled Ledder") ;
		DefaultBitDescriptor lm = new DefaultBitDescriptor();
		ll.setBitDescriptor(lm);
		
		int ledCount = 7;

		String[] descText =
			{
				"1On",
				"2Sum Failure",
				"3DC Overcurrent",
				"4Remote",
				"5Phase Failure",
				"6External Interlock",
				"7Not Ready"};

		long c = 0;
		long m = 1;

		for (int i = 0; i < ledCount; i++) {
			Color con = Color.GREEN;
			Color cof = Color.RED;
			String cde;

			if (i < descText.length) {
				cde = descText[i % descText.length];
			} else {
				int nChars = (int) (Math.random() * 12) + 2;
				cde = "";
				cde += (char) (int) (Math.random() * 25 + 65);
				for (int j = 0; j < nChars; j++) {
					cde += (char) (int) (Math.random() * 25 + 97);
				}
			}
			c += m;
			m *= 2;
			
			lm.add(cde, con, cof);
		}
		
		ll.setMask(c);

		

		JFrame frame = new JFrame("Ledder Testing Applet");
		frame.getContentPane().setLayout(new BorderLayout(10, 10));
		frame.getContentPane().add(ll);
		frame.setSize(500, 200);
		frame.addWindowListener(new WindowAdapter() {
				@Override
                public void windowClosing(WindowEvent e)
				{
					System.exit(0);
				}
			});
		ll.setEditable(true);
		frame.setVisible(true);
		try {
            Thread.sleep(2000);
            ll.setBits(10);
            Thread.sleep(2000);
            ll.getLedder().setBits(15);
            Thread.sleep(2000);
            System.out.println(ll.getLedder().getBits());
        } catch (InterruptedException e1) {
            e1.printStackTrace();
        }
	
	}

	private Ledder ledder;
	
	/**
	 * Creates a new LabelledLedder object.
	 */
	public LabelledLedder()
	{
		this(null);
	}

	/**
	 * Creates a new LabelledLedder object.
	 *
	 * @param title DOCUMENT ME!
	 */
	public LabelledLedder(String title)
	{
		super(title, true,Layout.DYNAMIC,true,true,true,1,Integer.MAX_VALUE);
		
	}

	/**
	 * DOCUMENT ME!
	 *TODO prej je bil bitmask().getBits... vracal je 0!
	 * @return long
	 */
	public long getBits()
	{
		return getLedder().getBits();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return int
	 */
	
	protected Ledder getLedder()
	{
		if (ledder == null) {
			ledder = new Ledder();
			//			ledder.addPropertyChangeListener(new PropertyChangeListener() {
			//					/**
			//					 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
			//					 */
			//					public void propertyChange(PropertyChangeEvent evt)
			//					{
			//						firePropertyChange(evt.getPropertyName(),
			//						    evt.getOldValue(), evt.getNewValue());
			//					}
			//				});
			//ledder.addMouseListener(getPopupManager().getMouseHook());
			
			
			
			ledder.addContainerListener(new ContainerListener() {
				public void componentRemoved(ContainerEvent e) {
					e.getChild().removeMouseListener(getPopupManager().getMouseHook());
				}
				public void componentAdded(ContainerEvent e) {
					e.getChild().addMouseListener(getPopupManager().getMouseHook());
				}
			});
		
		}

		return ledder;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return long
	 */
	public long getMask()
	{
		return getLedder().getMask();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return BitDescriptor
	 */
	public BitDescriptor getBitDescriptor()
	{
		return getLedder().getModel();
	}


	/**
	 * Overriden to implement additional features saving.
	 *
	 * @return state of the component.
	 */
	@Override
    public State getState()
	{
		State state = super.getState();

		return state;
	}

	
	/**
	 * Returns value displaying GUI component: the <code>Wheelswitch</code>.
	 *
	 * @see AbstractDisplayerPanel#getValueComponent()
	 */
	@Override
    protected JComponent getValueComponent()
	{
		return getLedder();
	}

	/**
	 * Overriden to implement additional functionality.
	 *
	 * @see com.cosylab.gui.components.AbstractDisplayerPanel#setResizable(boolean)
	 */
	@Override
    protected void internalSetResizable()
	{
		super.internalSetResizable();
		getLedder().revalidate();
	}

	/**
	 * Returns whether the ledder can be edited  by the user.
	 *
	 * @return boolean
	 */
	public boolean isEditable()
	{
		return getLedder().isEditable();
	}
	
	public void setEditable(boolean editable){
	    getLedder().setEditable(editable);
	}
	/**
	 * If the reverseLedsOrder is <code>true<code> then the leds will be displayed
	 * in reverse order.
	 * 
	 * @return reverseLedsOrder boolean 
	 */
	public boolean isReverseLedsOrder() {
		return getLedder().isReverseLedsOrder();
	}
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.AbstractDisplayerPanel#layoutDisplayer()
	 */
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param value long
	 */
	public void setBits(long value)
	{
		getLedder().setBits(value);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param value long
	 */
	public void setMask(long value)
	{
		getLedder().setMask(value);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param model BitDescriptor
	 */
	public void setBitDescriptor(BitDescriptor model)
	{
		getLedder().setModel(model);
	}

	/**
	 * If the reverseLedsOrder is <code>true<code> then the leds will be displayed
	 * in reverse order.
	 * 
	 * @param b boolean
	 */
	public void setReverseLedsOrder(boolean b) {
		getLedder().setReverseLedsOrder(b);
	}
	
	/**
	 * Overriden to implement additional features loading.
	 *
	 * @param state to be loaded.
	 */
	@Override
    public void setState(State state)
	{
		super.setState(state);

	}
	
	
	@Override
    public void setEnabled(boolean enabled){
	    getTitleComponent().setEnabled(enabled);
	}
	

}

/* __oOo__ */
