/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import com.cosylab.gui.components.customizer.Editor;

/**
 * <code>LabelledNumberFieldCustomizer</code> is a customizer for 
 * <code>LabelledNumberField</code> which allows customization of its most
 * common properties.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: LabelledNumberFieldCustomizer.java,v 1.14 2008-04-22 12:28:40 jbobnar Exp $
 * @see LabelledNumberField
 *
 */
public class LabelledNumberFieldCustomizer extends
		AbstractNumericDisplayerPanelCustomizer {
	private static final long serialVersionUID = 1L;

	/** Aspect for bounds. */
	public static final String VALUE_DISPLAY_BOUNDS = "Value Display/Bounds";
	public static final String[] VALUE_DISPLAY_BOUNDS_PROPERTIES = {"minimumValue","maximumValue"};
	
	
	public static class WPanel extends JPanel implements Customizer
	{
		private static final long serialVersionUID = 1L;
		private LabelledNumberField displayer = null;
		NumberField min;
		NumberField max;

		/**
		 * Creates a new WPanel object.
		 */
		public WPanel()
		{
			super();

			setLayout(new GridBagLayout());

			min = new NumberField();
			min.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent arg0) {
					applySettings();
				}
				
			});
			min.setNumberType(Double.class);
			min.setPreferredSize(new Dimension(100, 21));
			min.setToolTipText("Minimal allowed value");
			if (displayer != null) min.setValue(displayer.getMinimumValue());
			add(new JLabel("Minimum"),
			    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 11, 4, 4), 0, 0));
			add(min,
			    new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			max = new NumberField();
			max.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent arg0) {
					applySettings();
				}
				
			});
			max.setNumberType(Double.class);
			max.setPreferredSize(new Dimension(100, 21));
			max.setToolTipText("Maximal allowed value");
			if (displayer != null) max.setValue(displayer.getMaximumValue());
			ActionListener l = new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					applySettings();
				}
			};
			max.addActionListener(l);
			min.addActionListener(l);
			
			add(new JLabel("Maximum"),
			    new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 11, 4, 4), 0, 0));
			add(max,
			    new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));
		}

		public void setObject(Object arg0) {
			initialize((LabelledNumberField)arg0);
		}

		private void initialize(LabelledNumberField newDisplayer)
		{
			if (newDisplayer == null) {
				return;
			}
			
			displayer = newDisplayer;
			displayer.addPropertyChangeListener("maximumValue", new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					max.setValue(displayer.getMaximumValue());
				}
			});
			displayer.addPropertyChangeListener("minimumValue", new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					min.setValue(displayer.getMinimumValue());
				}
			});
			max.setNumberType(displayer.getNumberType());
			max.setValue(displayer.getMaximumValue());
			min.setNumberType(displayer.getNumberType());
			min.setValue(displayer.getMinimumValue());
		}

		/**
		 * @see Editor#applySettings()
		 */
		public void applySettings()
		{
			if (displayer == null) {
				return;
			}

			displayer.setMaximumValue(max.getValue());
			displayer.setMinimumValue(min.getValue());
			initialize(displayer);
			firePropertyChange("maximumValue", null, max.getValue());
			firePropertyChange("minimumValue", null, min.getValue());
		}

	}

	public LabelledNumberFieldCustomizer() {
//		addCustomizerTable(VALUE_DISPLAY_BOUNDS, new String[]{"minimumValue","maximumValue"});
		addCustomizer(VALUE_DISPLAY_BOUNDS, new WPanel());
		setSize(445, 185);
	}
}
