/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

/**
 * Customizer for <code>LabelledWheelswitch</code> allows setting the most
 * common properties of the bean.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: LabelledWheelswitchCustomizer.java,v 1.10 2008-04-22 12:28:40 jbobnar Exp $
 * @see LabelledWheelswitch
 */
public class LabelledWheelswitchCustomizer extends
		AbstractNumericDisplayerPanelCustomizer {
	private static final long serialVersionUID = 1L;

	public static String[] ASPECTS = {
		VISUAL_BASIC, VISUAL_LAYOUT, VALUE_DISPLAY, VALUE_DISPLAY_ADVANCED
	};

	/** Tag for enhanced property. */
	public static final String ENHANCED = "enhanced";

	/** Tag for animated property. */
	public static final String ANIMATED = "animated";

	/** Tag for editable property. */
	public static final String EDITABLE = "editable";

	/** Tag for columns property. */
	public static final String COLUMNS = "columns";

	/** Tag for minimum property. */
	public static final String GRAPH_MIN = "minimum";

	/** Tag for maximum property. */
	public static final String GRAPH_MAX = "maximum";

	/** Tag for tiltingEnabled property. */
	public static final String TILTING_ENABLED = "tiltingEnabled";

	/** Names of basic visual properties. */
	public static final String[] VISUAL_BASIC_PROPERTIES = {
		TITLE, TITLE_VISIBLE, UNITS_SHOWN_WITH_TITLE_VISIBLE, MIN_TITLE_FONT_SIZE, MAX_TITLE_FONT_SIZE, RESIZABLE, ENHANCED, TILTING_ENABLED, ANIMATED
	};

	/** Names for advanced value display properties. */
	public static String[] VALUE_DISPLAY_ADVANCED_PROPERTIES = {
		EDITABLE
	};

	/** Names for the value display properties. */
	public static String[] VALUE_DISPLAY_PROPERTIES = {
		GRAPH_MIN, GRAPH_MAX, BOUNDS_VISIBLE, UNITS, UNITS_VISIBLE, FORMAT
	};
	
	public LabelledWheelswitchCustomizer(){
		addCustomizerTable(VALUE_DISPLAY, VALUE_DISPLAY_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY_ADVANCED, VALUE_DISPLAY_ADVANCED_PROPERTIES);
		setSize(452, 205);
	}
}
