/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.Customizer;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.range2.RescalingValuePolicy;
import com.cosylab.gui.components.range2.ShiftValuePolicy;
import com.cosylab.gui.components.range2.TrimValuePolicy;

public class PiperCustomizer extends AbstractCustomizerPanel {
	private static final long serialVersionUID = 1L;
	public static final String VALUE_DISPLAY = "Value Display";
	public static final String VALUE_POLICY = "Value Range Policy";
	private static final String UNITS = "units";
	private static final String UNITS_VISIBLE = "unitsVisible";
	private static final String FORMAT = "format";
	private static final String GRAPH_MIN = "minimum";
	private static final String GRAPH_MAX = "maximum";
	private static final String ENHANCED = "enhanced";
	private static final String STRETCH = "stretch";
	private static final String TILTING_ENABLED = "tiltingEnabled";
	private static final String RESIZABLE = "resizable";
	public static final String TITLE = "title";
	public static final String[] VISUAL_BASIC_PROPERTIES = {TITLE, RESIZABLE, ENHANCED,  TILTING_ENABLED, STRETCH };
	public static String[] VALUE_DISPLAY_PROPERTIES = {
		GRAPH_MIN, GRAPH_MAX, UNITS,UNITS_VISIBLE, FORMAT
	};

	/** Supported aspects. */
	public static String[] ASPECTS = { AbstractDisplayerPanelCustomizer.VISUAL_BASIC, VALUE_DISPLAY, VALUE_POLICY };
	
	/*
	 * Auxilary Editor implementation for customizing DisplayerPanel layout.
	 */
	private class WPanel extends JPanel implements Customizer, ItemListener
	{
		private static final long serialVersionUID = 1L;
		private Piper displayer;
		private JRadioButton trimButton;
		private JRadioButton rescalingButton;
		private JRadioButton shiftButton;
		private ButtonGroup bg;

		/**
		 * Creates a new WPanel object.
		 */
		public WPanel()
		{
			bg = new ButtonGroup();

			trimButton = new JRadioButton("Trim Value Range");
			trimButton.addItemListener(this);
			rescalingButton = new JRadioButton("Rescale Value Range");
			rescalingButton.addItemListener(this);
			shiftButton = new JRadioButton("Shift Value Range");
			shiftButton.addItemListener(this);

			bg.add(trimButton);
			bg.add(rescalingButton);
			bg.add(shiftButton);

			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			add(trimButton);
			add(rescalingButton);
			add(shiftButton);
		}


		private void initialize()
		{
			if (displayer == null) {
				return;
			}

			if (displayer.getValuePolicy() instanceof RescalingValuePolicy) {
				rescalingButton.setSelected(true);
			}
			if (displayer.getValuePolicy() instanceof TrimValuePolicy) {
				trimButton.setSelected(true);
			}
			if (displayer.getValuePolicy() instanceof ShiftValuePolicy) {
				shiftButton.setSelected(true);
			}
		}

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				if (shiftButton.isSelected()) {
					displayer.setValuePolicy(new ShiftValuePolicy());
				} else if (rescalingButton.isSelected()) {
					displayer.setValuePolicy(new RescalingValuePolicy());
				} else if (trimButton.isSelected()) {
					displayer.setValuePolicy(new TrimValuePolicy());
				} 
			}
		}

		/*
		 * (non-Javadoc)
		 * @see java.beans.Customizer#setObject(java.lang.Object)
		 */
		public void setObject(Object bean) {
			displayer = (Piper) bean;
			initialize();
		}
	}

	public PiperCustomizer() {
		addCustomizerTable(AbstractDisplayerPanelCustomizer.VISUAL_BASIC, VISUAL_BASIC_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY, VALUE_DISPLAY_PROPERTIES);
		addCustomizer(VALUE_POLICY, new WPanel());
		setSize(331, 185);
	}
}
