/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.util.CosyUIElements;
import com.cosylab.gui.components.util.FontHelper;
import com.cosylab.gui.components.util.PopupManageable;
import com.cosylab.gui.components.util.PopupManager;


/**
 * Class specialized for entering the time. All fields are validated and allow
 * only entry of valid time. The interface to application is provided through
 * <code>java.util.Date</code> class. After creation, the component will be set
 * to current time. 
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class SimpleTimeSelector extends JPanel implements PopupManageable 
{
	private static final long serialVersionUID = 1L;
	private JLabel separator1;
	private JLabel separator2;
	private JLabel hourLabel;
	private JLabel minuteLabel;
	private JLabel secondsLabel;
	private Calendar calendar;
	private NumberField hourChooser;
	private NumberField minuteChooser;
	private NumberField secondChooser;
	
	private PopupManager popupManager;
	private AbstractCustomizerPanel customizer;

	/**
	 * Listens for changes to fields and updates selection accordingly.
	 */
	private class NumberChangeListener implements PropertyChangeListener
	{
		/**
		 * Listens for changes to fields and updates selection accordingly.
		 *
		 * @param e PropertyChangeEvent
		 *
		 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
		 */
		public void propertyChange(PropertyChangeEvent e)
		{
			SimpleTimeSelector owner = SimpleTimeSelector.this;
			Date old = owner.getTime();
			if (e.getPropertyName().equals("value")) {
				int newValue = ((Number)e.getNewValue()).intValue();

				if (e.getSource() == owner.hourChooser) {
					owner.setHour(newValue);
					firePropertyChange("time",old,owner.getTime());
				}

				if (e.getSource() == owner.minuteChooser) {
					owner.setMinute(newValue);
					firePropertyChange("time",old,owner.getTime());
				}

				if (e.getSource() == owner.secondChooser) {
					owner.setSeconds(newValue);
					firePropertyChange("time",old,owner.getTime());
				}
			}
		}
	}

	/**
	 * SimpleTimeSelector constructor comment.
	 */
	public SimpleTimeSelector()
	{
		super(new GridBagLayout());

		calendar = new GregorianCalendar();

		createComponents();

		updateDisplay();
	}

	/*
	 * Creates and layouts the components that represent this panel.
	 */
	protected void createComponents()
	{
		setBorder(new CompoundBorder(CosyUIElements.getPlainBorder(true),
		        new EmptyBorder(3, 3, 3, 3)));
		
		NumberChangeListener cl = new NumberChangeListener();

		hourLabel = new JLabel("Hours");
		setComponentFont(hourLabel);
		add(hourLabel, new GridBagConstraints(0,0,2,1,0,0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(0,0,2,0),0,0));

		hourChooser = new NumberField(new Long(0), "%2d");
		hourChooser.addPropertyChangeListener(cl);
		add(hourChooser, new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(0,0,0,0),0,0));
		hourChooser.setMinimum(new Long(0));
		hourChooser.setMaximum(new Long(23));
        hourChooser.setColumns(2);

		separator1 = new JLabel(":");
		add(separator1, new GridBagConstraints(1,1,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(0,1,0,1),0,0));

		minuteLabel = new JLabel("Minutes");
		setComponentFont(minuteLabel);
		add(minuteLabel, new GridBagConstraints(2,0,2,1,0,0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(0,0,2,0),0,0));

		minuteChooser = new NumberField(new Long(0), "%2d");
		minuteChooser.addPropertyChangeListener(cl);
		add(minuteChooser, new GridBagConstraints(2,1,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(0,0,0,0),0,0));
		minuteChooser.setMinimum(new Long(0));
		minuteChooser.setMaximum(new Long(59));
        minuteChooser.setColumns(2);

		separator2 = new JLabel(":");
		add(separator2, new GridBagConstraints(3,1,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(0,1,0,1),0,0));

		secondsLabel = new JLabel("Seconds");
		setComponentFont(secondsLabel);
		add(secondsLabel, new GridBagConstraints(4,0,1,1,0,0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(0,0,2,0),0,0));

		secondChooser = new NumberField(new Long(0), "%2d");
		secondChooser.addPropertyChangeListener(cl);
		add(secondChooser, new GridBagConstraints(4,1,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(0,0,0,0),0,0));
		secondChooser.setMinimum(new Long(0));
		secondChooser.setMaximum(new Long(59));
		secondChooser.setColumns(2);
	}
	

	/**
	 * Internal helper routine to adjust the component font.
	 *
	 * @param c JComponent
	 */
	private final void setComponentFont(JComponent c)
	{
		c.setFont(FontHelper.getFontWithStyle(Font.PLAIN, c.getFont()));
	}

//	protected GridBagConstraints createConstraints(int x, int y, double ratio,
//	    int top, int bottom)
//	{
//		GridBagConstraints constraints = new GridBagConstraints();
//		constraints.gridx = x;
//		constraints.gridy = y;
//		constraints.weightx = ratio;
//		constraints.weighty = 0.0;
//		constraints.fill = GridBagConstraints.HORIZONTAL;
//
//		//	constraints.fill = GridBagConstraints.NONE;
//		constraints.insets = new java.awt.Insets(top, x==0?4:2, bottom, x==4?4:2);
//		constraints.anchor = GridBagConstraints.WEST;
//
//		return constraints;
//	}
	
	/**
	 * Creates a customizer for this component.
	 * 
	 * @return costomizer
	 */
	public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
		}

		return customizer;
	}
		
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.util.PopupManageable#getPopupManager()
	 */
	public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = new PopupManager(this, true);
			popupManager.addAction(new AbstractAction("Settings") {
				private static final long serialVersionUID = 1L;
				public void actionPerformed(ActionEvent e)
				{
					getCustomizer().showDialog();
				}

			});
		}

		return popupManager;
	}

	/**
	 * Returns the currently selected hour. The value returned is between 0 and
	 * 23 inclusively.
	 *
	 * @return int
	 */
	public int getHour()
	{
		return calendar.get(Calendar.HOUR_OF_DAY);
	}

	/**
	 * Returns the currently selected minute. The value is in range 0..59
	 * inclusively.
	 *
	 * @return int
	 */
	public int getMinute()
	{
		return calendar.get(Calendar.MINUTE);
	}

	/**
	 * Returns the currently selected seconds. The value is in range 0..59
	 * inclusively.
	 *
	 * @return int
	 */
	public int getSeconds()
	{
		return calendar.get(Calendar.SECOND);
	}

	/**
	 * Returns the currently selected time. Only hour, minute and second
	 * portions of the result are defined.
	 *
	 * @return java.util.Date
	 */
	public Date getTime()
	{
		return calendar.getTime();
	}

	/**
	 * Sets the enabled state of this component.
	 *
	 * @param how boolean
	 */
	@Override
	public void setEnabled(boolean how)
	{
		hourChooser.setEnabled(how);
		minuteChooser.setEnabled(how);
		secondChooser.setEnabled(how);
	}
	
	
	/**
	 * Returns true if hours label is enabled.
	 * 
	 * @param hour
	 */
	public boolean getHourStatus(){
		return hourLabel.isEnabled();
	}
	
	/**
	 * Enables/disables the hour label.
	 * 
	 * @param status
	 */
	public void setHourStatus(boolean status){
		hourLabel.setEnabled(status);	
	}

	/**
	 * Sets the hour. The value will be cropped to the 0..23 range.
	 *
	 * @param hour int
	 */
	public void setHour(int hour)
	{
		if (hour == getHour()) {
			return;
		}

		hour = Math.max(hour, 0);
		hour = Math.min(hour, 23);

		calendar.set(Calendar.HOUR_OF_DAY, hour);

		hourChooser.setLongValue(hour);
	}
    
    /**
     * Sets the visibility of the labels above the text fields.
     * The labels are visible by default. 
     *  
     * @param visible
     */
    public void setLabelsVisible(boolean visible) {
        hourLabel.setVisible(visible);
        minuteLabel.setVisible(visible);
        secondsLabel.setVisible(visible);
    }
    
	/**
	 * Sets the minutes of the current time. New value will be cropped to the
	 * 0..59 range.
	 *
	 * @param minute int
	 */
	public void setMinute(int minute)
	{
		if (minute == getMinute()) {
			return;
		}

		minute = Math.max(minute, 0);
		minute = Math.min(minute, 59);

		calendar.set(Calendar.MINUTE, minute);

		minuteChooser.setLongValue(minute);
	}

	/**
	 * Sets the seconds of the current time. New value will be cropped to the
	 * 0..59 range.
	 *
	 * @param seconds int
	 */
	public void setSeconds(int seconds)
	{
		if (seconds == getSeconds()) {
			return;
		}

		seconds = Math.max(seconds, 0);
		seconds = Math.min(seconds, 59);

		calendar.set(Calendar.SECOND, seconds);

		secondChooser.setLongValue(seconds);
	}

	/**
	 * Sets the time to display. Only the hour, minute and second portion of
	 * the date will be used.
	 *
	 * @param date Date
	 */
	public void setTime(Date date)
	{
		calendar.setTime(date);
		updateDisplay();
	}

	/**
	 * Updates the displayed values after manually setting the time.
	 */
	protected void updateDisplay()
	{
		hourChooser.setLongValue(getHour());
		minuteChooser.setLongValue(getMinute());
		secondChooser.setLongValue(getSeconds());
	}

	/**
	 * Sets the time zone.
	 *
	 * @param zone
	 */
	public void setTimeZone(TimeZone zone)
	{
		calendar.setTimeZone(zone);
	}

	/**
	 * Returns currently used time zone
	 *
	 * @return currently used time zone
	 */
	public TimeZone getTimeZone()
	{
		return calendar.getTimeZone();
	}
	

	public static void main(String[] args)
	{
		JFrame dialog = new JFrame();
		//dialog.setModal(true);
		dialog.setSize(200, 100);
		dialog.getContentPane().setLayout(new BorderLayout());

		SimpleTimeSelector dc = new SimpleTimeSelector();
//        dc.setBorder(new LineBorder(Color.RED));
        //dc.setLabelsVisible(false);

		dialog.getContentPane().add(dc, BorderLayout.CENTER);
		dialog.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		//dialog.show();
		dialog.setVisible(true);

		//System.exit(0);
	}

	
}

/* __oOo__ */
