/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;

/**
 * </code>SliderCustomizer</code> is a customizer for <code>Slider</code>
 * which enables customization of the most common slider's properties.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: SliderCustomizer.java,v 1.18 2008-09-09 09:09:09 jbobnar Exp $
 * @see Slider
 */
public class SliderCustomizer extends AbstractCustomizerPanel {
	private static final long serialVersionUID = 1L;

	/** Name of the value display tab. */
	public static final String VALUE_DISPLAY = "Value Display";

	/** Name of the advanced value display tab. */
	public static final String VALUE_DISPLAY_ADVANCED = "Value Display/Advanced";

	/** Name of the step configuration tab. */
	public static final String STEP_CONFIGURATION = "Step configuration";
	
	/** Name of the synchronization tab. */
	public static final String AUTO_SYNCHRONIZATION = "Auto synchronization";
	
	/** Name of the basic visual properties tab. */
	public static final String VISUAL_BASIC = "Visual";
	
	/** Aspects */
	public static String[] ASPECTS = {
			VISUAL_BASIC, VALUE_DISPLAY, VALUE_DISPLAY_ADVANCED,
			STEP_CONFIGURATION, AUTO_SYNCHRONIZATION
		};
	
	/** Tag for the title property. */
	public static final String TITLE = "title";
	
	/** Tag for the title visible property. */
	public static final String TITLE_VISIBLE = "titleVisible";
	
	/** Tag for the units property. */
	public static final String UNITS = "units";

	/** Tag for unitsVisible property. */
	public static final String UNITS_VISIBLE = "unitsVisible";

	/** Tag for format property. */
	@Deprecated
	public static final String FORMAT = "format";
	
	/** Tag for ticks format property. */
	public static final String TICKS_FORMAT = "ticksFormat";
	
	/** Tag for field format property. */
	public static final String FIELD_FORMAT = "fieldFormat";
	
	/** Tag for maximum property. */
	public static final String GRAPH_MAX = "maximum";

	/** Tag for minimum property. */
	public static final String GRAPH_MIN = "minimum";

	/** Tag for editable property. */
	public static final String EDITABLE = "editable";

	/** Names of the value display properties. */
	public static String[] VALUE_DISPLAY_PROPERTIES = {
			GRAPH_MIN, GRAPH_MAX, UNITS, UNITS_VISIBLE, TICKS_FORMAT, FIELD_FORMAT
		};

	/** Names of the basic visual properties. */
	public static String[] VISUAL_BASIC_PROPERTIES = {TITLE, TITLE_VISIBLE};
	
	/** Names of the advanced properties. */
	public static String[] ADVANCED_PROPERTIES = { EDITABLE , "snapToTicks", 
		"continuousControlVisible", "continuousModeEnabled", "continuousModeRepeatTime", 
		"continuousModeDelayTime", "setMode"};

	private class WPanel extends JPanel implements Customizer
	{
		private static final long serialVersionUID = 1L;
		private static final String TOOL_TIP2 = "When selected , slow step will increment/decrement on value(0-100 reprisenting procents ! )";
		private static final String TOOL_TIP1 = "User defines own step values !";
		private boolean initialized = false;
		private Slider displayer;

		// GUI elements
		private JPanel displayPanel;
		private JLabel beforeInput;
		private JButton resetSteps;

		//Bit
		private NumberField bitInputValue;
		private JRadioButton useAbsulutBitValue;
		private JRadioButton useRelativeBitValue;

		//Small
		private NumberField smallInputValue;
		private JRadioButton useAbsulutSmallValue;
		private JRadioButton useRelativeSmallValue;

		//Large
		private NumberField largeInputValue;
		private JRadioButton useAbsulutLargeValue;
		private JRadioButton useRelativeLargeValue;
		
		public WPanel(){
			if (displayer != null){
				initialize();
			}
		}
		
		private void initialize()
		{
			if (initialized) {
				return;
			}

			setLayout(new BorderLayout());

			add(getDisplayPanel(), BorderLayout.CENTER);
			updateEditor();
			initialized = true;

			setSize(300, 450);
		}

		private JPanel getDisplayPanel()
		{
			if (displayPanel == null) {
				displayPanel = new JPanel();
			}

			displayPanel.setBorder(new GroupBoxBorder("Step configuration :"));
			displayPanel.setLayout(new GridBagLayout());

			//Bit
			useAbsulutBitValue = new JRadioButton(" Use absulute bit step");
			useAbsulutBitValue.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						if (useAbsulutBitValue.isSelected() == true) {
							useRelativeBitValue.setSelected(false);
						} else {
							useRelativeBitValue.setSelected(true);
						}
						applySettings();
					}
				});
			useAbsulutBitValue.setToolTipText(TOOL_TIP1);
			useRelativeBitValue = new JRadioButton(" Use relative bit step");
			useRelativeBitValue.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						if (useRelativeBitValue.isSelected() == true) {
							useAbsulutBitValue.setSelected(false);
						} else {
							useAbsulutBitValue.setSelected(true);
						}
						applySettings();
					}
				});
			useRelativeBitValue.setToolTipText(
			    "When selected , bit step will increment/decrement on value(0-100 reprisenting procents ! )");
			beforeInput = new JLabel("Value : ");

			displayPanel.add(useAbsulutBitValue,
			    new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(useRelativeBitValue,
			    new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(beforeInput,
			    new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(getBitField(),
			    new GridBagConstraints(0, 2, 1, 1, 4.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 50, 4, 4), 0, 0));

			//small
			useAbsulutSmallValue = new JRadioButton(" Use absulute small step ");
			useAbsulutSmallValue.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						if (useAbsulutSmallValue.isSelected() == true) {
							useRelativeSmallValue.setSelected(false);
						} else {
							useRelativeSmallValue.setSelected(true);
						}
						applySettings();
					}
				});
			useAbsulutSmallValue.setToolTipText(TOOL_TIP1);
			useRelativeSmallValue = new JRadioButton(" Use relative small step");
			useRelativeSmallValue.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						if (useRelativeSmallValue.isSelected() == true) {
							useAbsulutSmallValue.setSelected(false);
						} else {
							useAbsulutSmallValue.setSelected(true);
						}
						applySettings();
					}
				});
			useRelativeSmallValue.setToolTipText(TOOL_TIP2);

			displayPanel.add(useAbsulutSmallValue,
			    new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(useRelativeSmallValue,
			    new GridBagConstraints(0, 5, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(beforeInput,
			    new GridBagConstraints(0, 6, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(getSmallField(),
			    new GridBagConstraints(0, 6, 1, 1, 4.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 50, 4, 4), 0, 0));

			//Large
			useAbsulutLargeValue = new JRadioButton(" Use absulute fast step");
			useAbsulutLargeValue.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						if (useAbsulutLargeValue.isSelected() == true) {
							useRelativeLargeValue.setSelected(false);
						} else {
							useRelativeLargeValue.setSelected(true);
						}
						applySettings();
					}
				});
			useAbsulutLargeValue.setToolTipText(TOOL_TIP1);
			useRelativeLargeValue = new JRadioButton(" Use relative fast step");
			useRelativeLargeValue.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						if (useRelativeLargeValue.isSelected() == true) {
							useAbsulutLargeValue.setSelected(false);
						} else {
							useAbsulutLargeValue.setSelected(true);
						}
						applySettings();
					}
				});
			useRelativeLargeValue.setToolTipText(
			    "When selected , fast step will increment/decrement on value(0-100 reprisenting procents ! )");

			displayPanel.add(useAbsulutLargeValue,
			    new GridBagConstraints(0, 7, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(useRelativeLargeValue,
			    new GridBagConstraints(0, 8, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(beforeInput,
			    new GridBagConstraints(0, 9, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			displayPanel.add(getLargeField(),
			    new GridBagConstraints(0, 9, 1, 1, 4.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 50, 4, 4), 0, 0));

			//reset...
			resetSteps = new JButton("RESET STEPS");
			resetSteps.setToolTipText("Resets all steps to default value...");
			resetSteps.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						displayer.resetSteps();
						updateEditor();
					}
				});
			displayPanel.add(resetSteps,
			    new GridBagConstraints(0, 12, 1, 1, 4.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 50, 4, 4), 0, 0));

			// sets listeners to NumberFields
			bitInputValue.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e) {
					applySettings();
				}
			});
			smallInputValue.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e) {
					applySettings();
				}
			});
			largeInputValue.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e) {
					applySettings();
				}
			});
			
			return displayPanel;
		}

		private void updateEditor()
		{
			if (displayer == null) return;
			double oldValue = bitInputValue.getDoubleValue();
			bitInputValue.setValue(new Double(displayer.getBitStep()));
			useAbsulutBitValue.setSelected(!displayer.isBitStepRelative());
			useRelativeBitValue.setSelected(displayer.isBitStepRelative());
			boolean bitStepRelative = useAbsulutBitValue.isSelected();
			firePropertyChange("bitStepRelative", !bitStepRelative, bitStepRelative);
			firePropertyChange("bitStep", oldValue, bitInputValue.getDoubleValue());

			oldValue = smallInputValue.getDoubleValue();
			smallInputValue.setValue(new Double(displayer.getSmallStep()));
			useAbsulutSmallValue.setSelected(!displayer.isSmallStepRelative());
			useRelativeSmallValue.setSelected(displayer.isSmallStepRelative());
			boolean smallStepRelative = useRelativeSmallValue.isSelected();
			firePropertyChange("smallStepRelative", !smallStepRelative, smallStepRelative);
			firePropertyChange("smallStep", oldValue, smallInputValue.getDoubleValue());
			
			oldValue = largeInputValue.getDoubleValue();
			largeInputValue.setValue(new Double(displayer.getLargeStep()));
			useAbsulutLargeValue.setSelected(!displayer.isLargeStepRelative());
			useRelativeLargeValue.setSelected(displayer.isLargeStepRelative());
			boolean  largeStepRelative = useRelativeLargeValue.isSelected();
			firePropertyChange("largeStepRelative", !largeStepRelative, largeStepRelative);
			firePropertyChange("largeStep", oldValue, largeInputValue.getDoubleValue());
		}

		private NumberField getBitField()
		{
			if (bitInputValue == null) {
				bitInputValue = new NumberField();
				bitInputValue.setNumberType(Double.class);
				bitInputValue.setPreferredSize(new Dimension(100, 20));
				bitInputValue.setToolTipText("Bit step");
				
				if (displayer == null){
					System.out.println("displayer = null");
				}else bitInputValue.setValue(new Double(displayer.getBitStep()));
			}

			return bitInputValue;
		}

		private NumberField getSmallField()
		{
			if (smallInputValue == null) {
				smallInputValue = new NumberField();
				smallInputValue.setNumberType(Double.class);
				smallInputValue.setPreferredSize(new Dimension(100, 20));
				smallInputValue.setToolTipText("Slow step");
				if (displayer != null) smallInputValue.setValue(new Double(displayer.getSmallStep()));
				
			}

			return smallInputValue;
		}

		private NumberField getLargeField()
		{
			if (largeInputValue == null) {
				largeInputValue = new NumberField();
				largeInputValue.setNumberType(Double.class);
				largeInputValue.setPreferredSize(new Dimension(100, 20));
				largeInputValue.setToolTipText("Fast step");
				if (displayer != null) largeInputValue.setValue(new Double(displayer.getLargeStep()));
				
			}

			return largeInputValue;
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see com.cosylab.gui.components.customizer.Editor#applySetting()
		 */
		public void applySettings()
		{
			//double value = 0;

			//apply all setting for steps...
			boolean bitStepRelative = useRelativeBitValue.isSelected();
			displayer.setBitStepRelative(bitStepRelative);
			firePropertyChange("bitStepRelative", !bitStepRelative, bitStepRelative);

			double oldValue = displayer.getBitStep();
			displayer.setBitStep(bitInputValue.getDoubleValue());
			firePropertyChange("bitStep", oldValue, bitInputValue.getDoubleValue());
			
			boolean smallStepRelative = useRelativeSmallValue.isSelected();
			displayer.setSmallStepRelative(smallStepRelative);
			firePropertyChange("smallStepRelative", !smallStepRelative, smallStepRelative);
			
			oldValue = displayer.getSmallStep();
			displayer.setSmallStep(smallInputValue.getDoubleValue());
			firePropertyChange("smallStep", oldValue, smallInputValue.getDoubleValue());
			
			boolean  largeStepRelative = useRelativeLargeValue.isSelected();
			displayer.setLargeStepRelative(largeStepRelative);
			firePropertyChange("largeStepRelative", !largeStepRelative, largeStepRelative);
			
			oldValue = displayer.getLargeStep();
			displayer.setLargeStep(largeInputValue.getDoubleValue());
			firePropertyChange("largeStep", oldValue, largeInputValue.getDoubleValue());
			updateEditor();
		}

		public void setObject(Object arg0) {
			displayer = (Slider) arg0;
			initialize();
		}
	}
	
	/**
	 * 
	 * <code>SyncPanel</code> enables configuring auto synchronization of the slider.
	 *
	 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
	 * @version $Id: SliderCustomizer.java,v 1.18 2008-09-09 09:09:09 jbobnar Exp $
	 * 
	 * @since VERSION
	 */
	private class SyncPanel extends JPanel implements Customizer {
		private static final long serialVersionUID = 1L;
		private Slider displayer;
		private boolean initialized = false;
		private JPanel displayPanel;
		private NumberField autoSyncDelay;
		private JLabel millis;
		private JLabel syncDelay;
		private JRadioButton syncManual;
		private JRadioButton syncAuto;
		
		public SyncPanel(){
			if (displayer != null){
				initialize();
			}
		}
		
		private void initialize()
		{
			if (initialized) {
				return;
			}

			setLayout(new BorderLayout());

			add(getDisplayPanel(), BorderLayout.CENTER);
			updateEditor();
			initialized = true;

			setSize(300, 450);
		}
		
		private void updateEditor() {
			syncAuto.setSelected(displayer.isAutoSynchronize());
			syncManual.setSelected(!displayer.isAutoSynchronize());
			autoSyncDelay.setLongValue(displayer.getAutoSynchronizeDelay());
			autoSyncDelay.setEnabled(displayer.isAutoSynchronize());
		}
		
		private JPanel getDisplayPanel() {
			if (displayPanel == null) {
				displayPanel = new JPanel();
			}
			displayPanel.setBorder(new GroupBoxBorder("Synchronization configuration :"));
			displayPanel.setLayout(new GridBagLayout());
						
			syncManual = new JRadioButton("Manual synchronization");
			syncManual.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					autoSyncDelay.setEnabled(false);
					applySettings();
				}
				
				
			});
			syncAuto = new JRadioButton("Automatic synchronization");
			syncAuto.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					autoSyncDelay.setEnabled(true);	
					applySettings();
				}
				
				
			});
			ButtonGroup syncModeButtonGroup = new ButtonGroup();
			syncModeButtonGroup.add(syncManual);
			syncModeButtonGroup.add(syncAuto);
						
			displayPanel.add(syncManual, new GridBagConstraints(0,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			displayPanel.add(syncAuto, new GridBagConstraints(0,1,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			
			syncDelay = new JLabel("Automatic synchronization delay:");
			displayPanel.add(syncDelay, new GridBagConstraints(0,2,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,21,0,0), 1,1));
			
			JPanel fieldPanel = new JPanel();
			fieldPanel.setLayout(new GridBagLayout());
			autoSyncDelay = new NumberField();
			autoSyncDelay.setNumberType(long.class);
			autoSyncDelay.addCaretListener(new CaretListener(){

				public void caretUpdate(CaretEvent e) {
					applySettings();
				}
				
			});
			autoSyncDelay.setPreferredSize(new Dimension(70, 20));
			fieldPanel.add(autoSyncDelay, new GridBagConstraints(0,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			
			millis = new JLabel("milliseconds");
			fieldPanel.add(millis, new GridBagConstraints(1,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,10,0,0), 1,1));
			
			displayPanel.add(fieldPanel, new GridBagConstraints(0,3,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(5,21,0,0), 1,1));
			return displayPanel;
		}
	

		public void applySettings() {
			boolean auto = syncAuto.isSelected();
			displayer.setAutoSynchronize(auto);
			if (syncAuto.isSelected()) {
				displayer.setAutoSynchronizeDelay(autoSyncDelay.getLongValue());
				firePropertyChange("autoSynchronizeDelay", -1, autoSyncDelay.getLongValue());
			}
			firePropertyChange("autoSynchronize", !auto, auto);
			
		}

		public void setObject(Object arg0) {
			displayer = (Slider) arg0;
			initialize();
		}
		
	}
	
	public SliderCustomizer() {
		super();
		addCustomizerTable(VALUE_DISPLAY_ADVANCED,ADVANCED_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY, VALUE_DISPLAY_PROPERTIES);
		addCustomizerTable(VISUAL_BASIC,VISUAL_BASIC_PROPERTIES);
		addCustomizer(STEP_CONFIGURATION, new WPanel());
		addCustomizer(AUTO_SYNCHRONIZATION, new SyncPanel());
		setSize(452, 399);
	}
}
