/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.customizer;

import java.util.ArrayList;

import javax.swing.JComponent;


/**
 * <code>EditorChooser</code> is convenience implementation, which trys to find
 * one active editor form list of available editorList.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$ EditorChooser.java,v 1.3 2003/07/19 11:52:10 jkamenik Exp $$
 *
 * @since Jul 15, 2003.
 */
public class EditorChooser implements Editor
{
	private ArrayList<Editor> editorList = new ArrayList<>();
	private Editor[] editors;
	private Editor activeEditor;

	/**
	 * Default constructor.
	 */
	public EditorChooser()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.customizer.Editor#getEditor(java.lang.Object, java.lang.String)
	 */
	public JComponent getEditorComponent(Object displayer, String aspect)
		throws IllegalArgumentException
	{
		if (displayer == null) {
			throw new NullPointerException("displayer==null");
		}

		//		if (activeEditor != null) {
		//			throw new IllegalStateException(
		//			    "Can not produce start new session, while old is not finished.");
		//		}
		// by jkamenik <not wanted behavior>
		Editor[] ed = getEditors();

		for (int i = 0; i < ed.length; i++) {
			if (ed[i].canEdit(displayer, aspect)) {
				return (activeEditor = ed[i]).getEditorComponent(displayer,
				    aspect);
			}
		}

		throw new IllegalArgumentException("Editor for displayer '" + displayer
		    + "' in aspect of '" + aspect + "' not found from "
		    + editorList.size() + " of avaliable editors.");
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.customizer.Editor#canEdit(java.lang.Object, java.lang.String)
	 */
	public boolean canEdit(Object displayer, String aspect)
	{
		Editor[] ed = getEditors();

		for (int i = 0; i < ed.length; i++) {
			if (ed[i].canEdit(displayer, aspect)) {
				return true;
			}
		}

		return false;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.customizer.Editor#applySetting()
	 */
	public void applySettings()
	{
		Editor[] ed = getEditors();

		for (int i = 0; i < ed.length; i++) {
			ed[i].applySettings();
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.customizer.Editor#revertSettings()
	 */
	public void revertSettings()
	{
		Editor[] ed = getEditors();

		for (int i = 0; i < ed.length; i++) {
			ed[i].revertSettings();
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.customizer.Editor#stopEditing()
	 */
	public void stopEditing()
	{
		Editor[] ed = getEditors();

		for (int i = 0; i < ed.length; i++) {
			ed[i].stopEditing();
		}

		activeEditor = null;
	}

	/**
	 * Returns active editor.
	 *
	 * @return returns active editor
	 */
	public Editor getActiveEditor()
	{
		return activeEditor;
	}

	/**
	 * Returns list of all registered editors.
	 *
	 * @return all registered editors
	 */
	public synchronized Editor[] getEditors()
	{
		if (editors == null) {
			editors = new Editor[editorList.size()];
			editorList.toArray(editors);
		}

		return editors;
	}

	/**
	 * Adds editor to the list of available editorList.
	 *
	 * @param editor new available editor
	 */
	public synchronized void addEditor(Editor editor)
	{
		editorList.add(editor);
		editors = null;
	}

	/**
	 * Removes provided editor from this list.
	 *
	 * @param editor to beremoved
	 */
	public synchronized void removeEditor(Editor editor)
	{
		editorList.remove(editor);
		editors = null;
	}
}

/* __oOo__ */
