/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.customizer;

import java.io.File;

import javax.swing.JFileChooser;

/**
 * <code>ImageFilter</code> is a file filter for common image files.
 * This filter can be used by a {@link JFileChooser} and will filter out
 * all but <i>png</i>, <i>jpg</i> and <i>jpeg</i> files.
 *
 */
public abstract class ImageFilter extends javax.swing.filechooser.FileFilter
{
  /**
    This is the one of the methods that is declared in
    the abstract class
   */
  @Override
public boolean accept(File f)
  {
    //if it is a directory -- we want to show it so return true.
    if (f.isDirectory())
      return true;
  
    //get the extension of the file

    String extension = getExtension(f);
    if ((extension.equals("png")) || (extension.equals("jpeg"))||
    		(extension.equals("jpg")))
       return true;

    //default -- fall through. False is return on all
    //occasions except:
    //a) the file is a directory
    //b) the file's extension is what we are looking for.
    return false;
  }
    
  /**
    Again, this is declared in the abstract class

    The description of this filter
   */
  @Override
public String getDescription()
  {
      return "Picture files";
  }

  /**
    Method to get the extension of the file, in lowercase
   */
  protected String getExtension(File f)
  {
    String s = f.getName();
    int i = s.lastIndexOf('.');
    if (i > 0 &&  i < s.length() - 1)
      return s.substring(i+1).toLowerCase();
    return "";
  }
}
