/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.gauger;

import java.awt.geom.Point2D;


/**
 * Defines polar transform for angles larger than 180 degrees, but less than
 * 360. Minimum will be at lower left and maximum at lower right. 
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class PolarFullTransform extends PolarTransform
{
	private static final int MAXIMUM_HEIGHT = 60;
	private static final int MINIMUM_HEIGHT = 35;
	private static final double RELATIVE_OFFSET = 0.9;
	private static final double RELATIVE_SCALE_SIZE_COMPARED_TO_SIZE = 0.15;

	/*
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	@Override
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		super.setParameters(w, h, marginX, marginY, tickOffset);

		double size = (Math.min(w, h * 1.171573411) * RELATIVE_OFFSET);

		R = size / 2.0;
		centre = new Point2D.Double(0.5 * w, 0.5 * h + (R * 0.17));

		h = (int)(h * RELATIVE_SCALE_SIZE_COMPARED_TO_SIZE);
		h = Math.min(h, MAXIMUM_HEIGHT);
		h = Math.max(h, MINIMUM_HEIGHT);
		height = h;

		r = R - h;

		alpha = 225.0;
		span = 270.0;

		setLabelPosition((int)centre.getX(), (int)centre.getY());

		//		this.tickOffset = PolarPoint.degToRad(tickOffset) / r;
		this.tickOffset = Math.toDegrees(tickOffset) / r;

		ArcSegment segment = new ArcSegment(centre, r, R, alpha, span);
		addSegment(segment);
	}
}

/* __oOo__ */
