/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.gauger;

import java.awt.Graphics;
import java.awt.geom.Point2D;


/**
 * Base class for polar transforms. All polar transforms will be rendered using
 * scale in the shape of arc.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public abstract class PolarTransform extends ScaleTransform
{
	protected Point2D centre;
	protected double R;
	protected double r;
	protected double alpha;
	protected double span;
	protected int height;
	protected double tickOffset;

	/*
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	@Override
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		setPolar(true);
	}

	/*
	 * @see ScaleTransform#mapUVtoXY(PointDouble, Point)
	 */
	@Override
	public void mapUVtoXY(Point2D scaleSpace, Point2D cartesianSpace)
	{
		double phi = alpha
			- (tickOffset + scaleSpace.getX() * (span - 2.0 * tickOffset));

		PolarPoint.polarToCartesian(cartesianSpace,
		    r + scaleSpace.getY() * height, phi);

		cartesianSpace.setLocation(cartesianSpace.getX() + centre.getX(),
		    cartesianSpace.getY() + centre.getY());
	}

	/*
	 * @see ScaleTransform#maxXYtoUV(Point, PointDouble)
	 */
	public void mapXYtoUV(Point2D cartesianSpace, Point2D scaleScape)
	{
		// not implemented
	}

	/*
	 * @see ScaleTransform#scaleWidth(double)
	 */
	@Override
	public double scaleWidth(double v)
	{
		return Math.abs(((r + v * height) * Math.PI * (span - 2 * tickOffset) / 180.0));
	}

	/*
	 * @see ScaleTransform#scaleHeight(double)
	 */
	@Override
	public double scaleHeight(double u)
	{
		return height;
	}

	/**
	 * @see com.cosylab.gui.components.gauger.ScaleTransform#measureTick(Graphics,
	 *      double, String)
	 */
	@Override
	public int measureTick(Graphics g, double x, String text)
	{
		return g.getFontMetrics().stringWidth(text);
	}
}

/* __oOo__ */
