/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.gauger;

import java.awt.geom.Point2D;


/**
 * Defines vertical polar transform. This transform will define gauger with 
 * arc scale with origin at left and arc spanning to the right. Minimum will
 * be at top and maximum at bottom.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class PolarVerticalTransform extends PolarTransform
{
	private static final int MAXIMUM_HEIGHT = 90;
	private static final int MINIMUM_HEIGHT = 35;
	private static final double RELATIVE_X_OFFSET = 0.1;
	private static final double RELATIVE_Y_OFFSET = 0.05;
	private static final double RELATIVE_SCALE_SIZE_COMPARED_TO_WIDTH = 0.3;

	/*
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	@Override
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		super.setParameters(w, h, marginX, marginY, tickOffset);

		double heightDouble = h * (1 - RELATIVE_Y_OFFSET);
		double widthDouble = (w - MAXIMUM_HEIGHT / 2) * (1 - RELATIVE_X_OFFSET);

		height = (int)(widthDouble * RELATIVE_SCALE_SIZE_COMPARED_TO_WIDTH);
		height = Math.min(height, MAXIMUM_HEIGHT);
		height = Math.max(height, MINIMUM_HEIGHT);

		double RDouble = 0.5 * widthDouble * (1
			+ Math.pow(0.5 * heightDouble / widthDouble, 2));

		double phi = (Math.asin(0.5 * heightDouble / RDouble) * 180 / Math.PI);

		R = RDouble;
		r = R - height;

		double cw = R
			+ (RELATIVE_X_OFFSET * widthDouble / 2 + MAXIMUM_HEIGHT / 6);

		centre = new Point2D.Double((w - cw), h / 2);
		alpha = phi;
		span = 2 * phi;

		this.tickOffset = ((tickOffset * 180.0 / Math.PI) / r);

		setLabelPosition(3 * w / 8, (int)centre.getY());

		ArcSegment segment = new ArcSegment(centre, r, R, alpha, span);
		addSegment(segment);
	}
}

/* __oOo__ */
