/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.introspection;

import java.util.HashMap;
import java.util.Map;

import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;


/**
 * <code>DefaultPropertiesTable</code> displays various properties in 
 * a two column table.
 * 
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class DefaultPropertiesTable extends JTable {

	private static final long serialVersionUID = 1L;
	private Map<Class, TableCellEditor> editors=new HashMap<>();
	private Map<Class, TableCellRenderer> renderers=new HashMap<>();

	/**
	 * ParametersTable constructor comment.
	 */
	public DefaultPropertiesTable() {
		super();
		initialize();
	}
	
	
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		setName("ParametersTable");
		PropertiesTableModel parametersModel = new PropertiesTableModel();
		setModel(parametersModel);
		createDefaultColumnsFromModel();
		setTableHeader(new JTableHeader(getColumnModel()));
		getTableHeader().setVisible(true);
		setRowHeight(20);
		/* Ike commented out. This looks like workaroud, seting value when editing is cancled, ccccc
		 addPropertyChangeListener("tableCellEditor",new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt.getNewValue()==null && evt.getOldValue()!=null) {
					Thread.dumpStack();
					System.out.println(evt);
					TableCellEditor tce=(TableCellEditor)evt.getOldValue();
					if (!tce.stopCellEditing()) {
						tce.cancelCellEditing();
					}
					setValueAt(tce.getCellEditorValue(),getEditingRow(),getEditingColumn());
				}
			}
		});*/
		/* Ike: another workaround from Miha, let's try without it
		addFocusListener(new FocusAdapter(){
			public void focusLost(FocusEvent e) {
				super.focusLost(e);
				if (e.getOppositeComponent()==null || !SwingUtilities.isDescendingFrom(e.getOppositeComponent(),DefaultPropertiesTable.this)){
					if (isEditing()) {
						if (!getCellEditor().stopCellEditing()) {
							getCellEditor().cancelCellEditing();
						}
					}
				}
			}
		});*/
//		setSurrendersFocusOnKeystroke(true);
		setColumnSelectionAllowed(false);
		setRowSelectionAllowed(true);
		putClientProperty("terminateEditOnFocusLost",Boolean.TRUE);
	}
	/**
	 * @param row int
	 * @param column int
	 * @return TableCellEditor
	 * @see JTable#getCellEditor(int, int)
	 */
	@Override
    public TableCellEditor getCellEditor(int row, int column) {
		TableCellEditor ed=null;
		if (getModel() instanceof PropertiesTableModel) {
			PropertiesTableModel ptm=(PropertiesTableModel)getModel();
			if (column==ptm.valCol) {
				ed=retrieveCellEditor(ptm,row);
			}
		}
		if (ed==null) {
			ed=super.getCellEditor(row, column);
		}
		return ed;
	}
	
	private TableCellEditor retrieveCellEditor(PropertiesTableModel model,int row) {
		
		Class dataType=model.getTypeForRow(row);
		if (!editors.containsKey(dataType)) {
			TableCellEditor edt=EditorsHelper.getTableCellEditorForClass(dataType);
			if (edt==null) {
				return null;
			}
			editors.put(dataType,edt);
		}
		return editors.get(dataType);
	}
	
	/**
	 * @param row int
	 * @param column int
	 * @return TableCellRenderer
	 * @see JTable#getCellRenderer(int, int)
	 */
	@Override
    public TableCellRenderer getCellRenderer(int row, int column) {
		if (getModel() instanceof PropertiesTableModel){
			PropertiesTableModel ptm=(PropertiesTableModel)getModel();
			if (column==ptm.valCol) {
				TableCellRenderer tcr= retrieveCellRenderer(ptm,row);
				if (tcr!=null) {
					return tcr;
				}
			}
		}
		return super.getCellRenderer(row, column);
	}
	
	private TableCellRenderer retrieveCellRenderer(PropertiesTableModel model, int row) {
				Class dataType=model.getTypeForRow(row);
				if (!renderers.containsKey(dataType)) {
					TableCellRenderer rnd=EditorsHelper.getTableCellRendererForClass(dataType);
					if (rnd==null) {
						return null;
					}
					renderers.put(dataType,rnd);
				}
				return renderers.get(dataType);
	}
	
}
