/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.introspection;

import java.awt.Color;
import java.awt.Font;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import com.cosylab.gui.components.util.ColorHelper;
import com.cosylab.gui.property.editors.BooleanEditor;
import com.cosylab.gui.property.editors.ClassEditor;
import com.cosylab.gui.property.editors.ColorEditor;
import com.cosylab.gui.property.editors.DoubleEditor;
import com.cosylab.gui.property.editors.FloatEditor;
import com.cosylab.gui.property.editors.FontEditor;
import com.cosylab.gui.property.editors.IntegerEditor;
import com.cosylab.gui.property.editors.LongEditor;
import com.cosylab.gui.property.editors.NumberEditor;
import com.cosylab.gui.property.editors.PropertyEditor;
import com.cosylab.gui.property.editors.ShortEditor;
import com.cosylab.gui.property.editors.StringEditor;
import com.cosylab.introspection.ClassIntrospector;
import com.cosylab.introspection.DataFormatter;

/**
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class EditorsHelper {
	public static final Border EMPTY_BORDER=new EmptyBorder(1,1,1,1);
	public static final Border FOCUS_BORDER=new LineBorder(ColorHelper.getFocus(),1);

	/**
	 * @param theClass Class
	 * @return JTextField
	 */
	public static PropertyEditor getPropertyEditorForClass(Class theClass){
		if (theClass.equals(String.class)) { 
			return new StringEditor();
		}
		if ((theClass.equals(Boolean.class)) || (theClass.equals(Boolean.TYPE))) {
			BooleanEditor be=new BooleanEditor();
			be.setFocusPainted(false);
			be.setOpaque(true);
			be.setBorderPainted(true);
			return be;
		}
		NumberEditor nf=null;
		if ((theClass.equals(Double.class)) || (theClass.equals(Double.TYPE))) {
			nf= new DoubleEditor();
		}
		if ((theClass.equals(Float.class)) || (theClass.equals(Float.TYPE))) {
			nf= new FloatEditor();
		}
		if ((theClass.equals(Short.class)) || (theClass.equals(Short.TYPE))) {
			nf= new ShortEditor();
		}
		if ((theClass.equals(Integer.class)) || (theClass.equals(Integer.TYPE))) {
			nf= new IntegerEditor();
		}
		if ((theClass.equals(Long.class)) || (theClass.equals(Long.TYPE))) {
			nf= new LongEditor();
		}
		if (nf!=null) {
			nf.setBorder(null);
			return nf;
		}
		if (theClass.equals(Class.class)) {
			ClassEditor ce=new ClassEditor();
			ce.setApplyMode(ClassEditor.APPLY_NEVER);
			return ce;
		}
		if (theClass.equals(Color.class)) {
			return new ColorEditor();
		}
		if (theClass.equals(Font.class)) {
			return new FontEditor();
		}
		return null;
	}

	/**
	 * @param theClass Class
	 * @return TableCellEditor
	 */
	public static TableCellEditor getTableCellEditorForClass(Class theClass){
		return getTableCellEditorForClass(theClass, 0);
	}
	/**
	 * @param theClass Class
	 * @return TableCellEditor
	 */
	public static TableCellEditor getTableCellEditorForClass(Class theClass, int clickCountToStart){
		TableCellEditor ret=null;
        if (theClass==null) {
        	DefaultCellEditor dce= new DefaultCellEditor(new JTextField());
        	dce.setClickCountToStart(clickCountToStart);
        	ret=dce;
        }
		if (ClassIntrospector.isEnum(theClass)) {
			JComboBox b=new JComboBox();
			b.setModel(new DefaultComboBoxModel(DataFormatter.toStringArray(ClassIntrospector.getEnumFields(theClass), false, false)));
			DefaultCellEditor dce= new DefaultCellEditor(b);
			dce.setClickCountToStart(clickCountToStart);
        	ret=dce;
		} else {
			PropertyEditor editor=getPropertyEditorForClass(theClass);
			if (editor!=null) {
				ret=new PropertyCellEditorRenderer(editor,clickCountToStart);
				if (editor instanceof JComponent) {
					((JComponent)editor).setBorder(FOCUS_BORDER);
				}
			}
		}
		if (ret instanceof JComponent) {
			((JComponent)ret).setBorder(FOCUS_BORDER);
		}
		return ret;
		

	}
	/**
	 * @param theClass Class
	 * @return TableCellRenderer
	 */
	public static TableCellRenderer getTableCellRendererForClass(Class theClass){
		TableCellRenderer ret=null;
/*		if (ClassIntrospector.isEnum(theClass)) {
			JComboBox b=new JComboBox();
			b.setModel(new DefaultComboBoxModel(DataFormatter.toStringArray(ClassIntrospector.getEnumFields(theClass), false, false)));
			ret = new Default(b);
		}
		else*/
		PropertyEditor editor = getPropertyEditorForClass(theClass);
		if (editor!=null) {
			ret=new PropertyCellEditorRenderer(editor);
			if (editor instanceof JComponent) {
				((JComponent)editor).setBorder(EMPTY_BORDER);
			}
		}
		else {
			ret=new DefaultTableCellRenderer();
		}
		if (ret instanceof JComponent) {
			((JComponent)ret).setBorder(EMPTY_BORDER);
		}
		return ret;
		

	}
//	/**
//	 * @param theClass Class
//	 * @return JComponent
//	 */
	/* Ike: noone uses it
	 public static JComponent getComponentForClass(Class theClass){
		if (ClassIntrospector.isEnum(theClass)) {
			return new JComboBox();
		}
		return (JComponent)getPropertyEditorForClass(theClass);
	}*/

	public static Object convert(Object value, Class type) {
		if (type.isAssignableFrom(value.getClass())) {
			return value;
		}
		if (value instanceof Number) {
			 if (type==double.class || type==Double.class) {
			 	return new Double(((Number)value).doubleValue());
			 }
			if (type==float.class || type==Float.class) {
			   return new Float(((Number)value).floatValue());
			}
			if (type==int.class || type==Integer.class) {
			   return new Integer(((Number)value).intValue());
			}
			if (type==long.class || type==Long.class) {
			   return new Long(((Number)value).longValue());
			}
			if (type==short.class || type==Short.class) {
			   return new Short(((Number)value).shortValue());
			}
		}
		if (value instanceof Boolean) {
			if (type==boolean.class || type==Boolean.class) {
			   return value;
			}
		}	
		if (value instanceof String) {
			if (type==String.class) {
				return value;
			}
			if (type==double.class || type==Double.class) {
			   return new Double((String)value);
			}
			if (type==float.class || type==Float.class) {
			   return new Float((String)value);
			}
			if (type==int.class || type==Integer.class) {
			   return new Integer((String)value);
			}
			if (type==short.class || type==Short.class) {
			   return new Short((String)value);
			}
			if (type==long.class || type==Long.class) {
			   return new Long((String)value);
			}
			if (type==boolean.class || type==Boolean.class) {
			   return new Boolean((String)value);
			}
			if (type==Number.class) {
				return new Double((String)value);
			}
			if (type.isEnum()) {
				Object[] enums = type.getEnumConstants();
				for (Object obj : enums) {
					if (((Enum)obj).name().equalsIgnoreCase((String)value)) {
						return obj;
					}
				}				
			}
		}	
			
		throw new UnsupportedOperationException("Unable to convert Object of type '" + value.getClass() + "' to specified type '" + type +"'.");
	}

}
