/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.introspection;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.EventObject;

import javax.swing.AbstractCellEditor;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import com.cosylab.gui.property.editors.PropertyEditor;


/**
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc </a>
 * @version $id$
 */
public class PropertyCellEditorRenderer extends AbstractCellEditor
																	implements
																	TableCellEditor,
																	TableCellRenderer {
	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    private PropertyEditor			editor				= null;

	private JComponent				editorComp			= null;

	private Color					prevBack			= null;

	private Color					prevFore			= null;

	private Object					tempVal;

	private PropertyChangeListener	pcl					= null;

	private int						clickCountToStart	= 0;

	/**
	 * Constructor for PropertyCellEditor.
	 */
	public PropertyCellEditorRenderer(PropertyEditor pe, int clickCountToStart) {
		this(pe);
		this.clickCountToStart = clickCountToStart;
	}

	public PropertyCellEditorRenderer(PropertyEditor pe) {
		super();
		if (pe == null) {
			throw new NullPointerException("PropertyEditor");
		}
		this.editor = pe;
		if (editor instanceof JComponent) {
			this.editorComp = (JComponent)editor;
			prevFore = editorComp.getForeground();
			prevBack = editorComp.getBackground();
			editorComp.addKeyListener(new KeyAdapter() {
				@Override
                public void keyPressed(KeyEvent e) {
					if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
						//System.out.println(e);
						cancelCellEditing();
					}
				}
			});
		}
	}

	/**
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(JTable,
	 *      Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable table, Object value,
													boolean isSelected,
													int row, int column) {
		editor.setPropertyValue(value);
		tempVal = value;
		editorComp.setBackground(table.getBackground());
		editorComp.setForeground(table.getForeground());
		editorComp.setBorder(EditorsHelper.FOCUS_BORDER);
		if (pcl==null) {
			pcl = new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (PropertyEditor.PROPERTY_VALUE_NAME.equals(evt.getPropertyName())) {
						stopCellEditing();
					}
				}
			};
			editor.addPropertyChangeListener(pcl);
		}
		editorComp.revalidate();
		editorComp.repaint();
		return editorComp;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.CellEditor#isCellEditable(java.util.EventObject)
	 */
	@Override
    public boolean isCellEditable(EventObject anEvent) {
		if (anEvent instanceof MouseEvent) {
			return ((MouseEvent)anEvent).getClickCount() >= clickCountToStart;
		}
		return true;
	}

	/**
	 * @see javax.swing.CellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue() {
		tempVal = null;
		return editor.getPropertyValue();
	}

	/**
	 * @see javax.swing.CellEditor#stopCellEditing()
	 */
	@Override
    public boolean stopCellEditing() {
		tempVal = null;
		super.stopCellEditing();
		if (pcl != null) {
			editor.removePropertyChangeListener(pcl);
			pcl=null;
		}
		return true;
	}

	/**
	 * @see javax.swing.CellEditor#cancelCellEditing()
	 */
	@Override
    public void cancelCellEditing() {
		editor.setPropertyValue(tempVal);
		tempVal = null;
		super.cancelCellEditing();
	}

	public Component getTableCellRendererComponent(JTable table, Object value,
													boolean isSelected,
													boolean hasFocus, int row,
													int column) {
		editor.setPropertyValue(value);
		if (isSelected) {
			if (!table.isEditing()) {
				editorComp.setBackground(table.getSelectionBackground());
				editorComp.setForeground(table.getSelectionForeground());
			} else {
				editorComp.setBackground(prevBack);
				editorComp.setForeground(prevFore);
			}
		} else {
			editorComp.setBackground(table.getBackground());
			editorComp.setForeground(table.getForeground());
		}
		if (hasFocus) {
			editorComp.setBorder(EditorsHelper.FOCUS_BORDER);
		} else {
			editorComp.setBorder(EditorsHelper.EMPTY_BORDER);
		}
		editorComp.doLayout();
		editorComp.revalidate();
		editorComp.repaint();
		return editorComp;
	}

}
