/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.ledder;
import java.awt.Color;

/**
 * Interface representing the led model. This interface provides minimum
 * requirements neccessary to uniquely define the model of leds. In this
 * definition the model is immutable, although this is not a requirement. 
 * Implementations should provide methods for modifying the contents during
 * runtime and fire appropriate events.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public interface BitDescriptor {

	/**
	 * Returns description for the led at specified index or null if
	 * such led does not exist.
	 * 
	 * @param index int
	 * @return String
	 */
	String getDescription(int index);

	/**
	 * Returns color for the led at specified index or null if
	 * such led does not exist. This is the color that will be displayed when 
	 * the led indicates ON state.
	 * @param index int
	 * @return Color
	 */
	Color getColorWhenOn(int index);

	/**
	 * Returns color for the led at specified index or null if such led does not
	 * exist. This is the color that will be displayed when the led indicates 
	 * OFF state.
	 * 
	 * @param index int
	 * @return Color
	 */
	Color getColorWhenOff(int index);
}
