/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.ledder;
import java.awt.Color;
import java.util.HashMap;

import javax.swing.Icon;

/**
 * Creates LED icon for given color.
 *  
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class LedIconFactory {

	private static HashMap<Color,Icon> cache;
	
	
	/**
	 * Returns led icon from cache. New instance will be created if it does not
	 * exist already.
	 * 
	 * @param color Color of led.
	 * @return Instance of led icon.
	 */
	private static final Icon getCached(Color color) {
		if (cache == null) {
			cache = new HashMap<>();
		}
		Icon ic = cache.get(color);
		if (ic == null) {
			ic = new CustomLedIcon(color);
			cache.put(color, ic);
		}
		return ic;
	}

	/**
	 * Creates the icon from <code>java.awt.Color</code>. See the class overview
	 * for details.
	 * 
	 * @param color color of the icon.
	 * @return Icon always an instance of an icon 
	 */
	public static Icon createIcon(Color color) {
		return getCached(color);
	}

}
