/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.numberfield;

import com.cosylab.logging.DebugLogger;

import com.cosylab.util.PrintfFormat;
import com.cosylab.util.StringUtilities;

import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * <code>DefaultNumberDescriptor</code> is a default implementaion of the 
 * NumberDescriptor interface.
 *
 * @author $Author: jbobnar $
 * @version $Revision: 1.8 $
 */
public class DefaultNumberDescriptor implements NumberDescriptor
{
	/**
	 * @see NumberDescriptor
	 */
	public final static String NAME = "Default Visualization";

	/**
	 * @see NumberDescriptor
	 */
	public final static String DESCRIPTION = "Prints value to string according to ANSI format string.";
	private final Logger logger = DebugLogger.getLogger("DND", Level.OFF);
	private String format;
	private Class<? extends Number> type = Double.class;
	private PrintfFormat formatter;

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#parseNumber(java.lang.String)
	 */
	public Number parseNumber(String number)
	{
		try {
			if (type == Long.class) {
				if ("-".equals(number.trim())) {
					return new Long(0);
				}

				logger.fine("Long '" + number + "'");

				double d = StringUtilities.parseDouble(number);

				if (Double.isNaN(d)) {
					return null;
				}

				return new Long((long)d);
			}

			double newValue = StringUtilities.parseDouble(number);

			if (Double.isInfinite(Math.abs(newValue))) {
				logger.fine("Infinite '" + number + "'");

				return null;
			}

			if (Double.isNaN(newValue)) {
				logger.fine("NaN '" + number + "'");

				return null;
			}

			logger.fine("Dobule '" + number + "' -> " + newValue);

			return new Double(newValue);
		} catch (Exception e) {
			// ignored
			logger.fine("Exception " + e);
		}

		logger.fine("Plain failed '" + number + "'");

		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#toString(java.lang.Number)
	 */
	public String printString(Number number)
	{
		String s = formatter.sprintf(number).replace(',', '.');
		logger.fine("'" + number + "' -> '" + s + "'");

		return s;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#toEditString(java.lang.Number)
	 */
	public String printEditString(Number number)
	{
		String s = printString(number).trim();

		if (s.charAt(0) == '+') {
			s = s.substring(1);
		}

		logger.fine("'" + number + "' -> '" + s + "'");
		return s;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#getFormat()
	 */
	public String getFormat()
	{
		return format;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#setFormat(java.lang.String)
	 */
	public void setFormat(String format)
	{
		logger.info("newFormat '" + format + "'");
		this.format = format;
		formatter = new PrintfFormat(format);
	}

	/**
	 * Supported types are <code>Double.class</code> and
	 * <code>Long.class</code>.
	 *
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#setNumberType(java.lang.Class)
	 */
	public void setNumberType(Class<? extends Number> newClass)
	{
		logger.info("newClass '" + newClass + "'");

		if (newClass != Double.class && newClass != Long.class) {
			throw new IllegalArgumentException("Type '" + newClass
			    + "' not supported.");
		}

		type = newClass;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#getNumberType()
	 */
	public Class<? extends Number> getNumberType()
	{
		return type;
	}
}

/* __oOo__ */
