/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.numberfield;

/**
 * <p>
 * Implementation of this interface provides conversion of a
 * <code>Number</code> to <code>String</code> and back. This interface acts as
 * kind of interpretation of a <code>Number</code> with bijective
 * transformation to string.
 * </p>
 * 
 * <p>
 * This interfaces provides more flexible visualization of number as string
 * than is available with standard ANSI C kind of formatting. For example:
 * display and edit double value in hh:mm:ss format (interpret it as degrees).
 * </p>
 * 
 * <p>
 * It is advised for the descriptor implementation to define two final static String fields: <code>NAME</code> and <code>DESCRIPTION</code>. First should be short descriptive name and second should be short description. These strings are used in GUI choosers for NumberField visualization. 

 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public interface NumberDescriptor
{
	/**
	 * Parses string and creates <code>Number</code>. If string is not
	 * presentation of a number and can not be parsed, then null must be
	 * returned.
	 *
	 * @param number a string representing a number
	 *
	 * @return instance of <code>Number</code> or <code>null</code> if string
	 *         can not be parsed
	 */
	public Number parseNumber(String number);

	/**
	 * Converts provided <code>Number</code> to a string, which is used for
	 * displaying the value.
	 *
	 * @param number a number to be transformed to string
	 *
	 * @return a <code>String</code> representing a number.
	 */
	public String printString(Number number);

	/**
	 * Converts provided <code>Number</code> to a string, which is displayed
	 * when NumberField enters in to edit mode and wants to edit value. Edit
	 * string is usually stripped from unnecessary characters and prepared for
	 * easyer user editing.
	 *
	 * @param number a number to be transformed to string
	 *
	 * @return a <code>String</code> representation a number prepared for
	 *         editing.
	 */
	public String printEditString(Number number);

	/**
	 * Returns the format
	 *
	 * @return Format
	 */
	public String getFormat();

	/**
	 * Set the Number format
	 *
	 * @param format
	 */
	public void setFormat(String format);

	/**
	 * Sets number type to descriptor, ame as used on number field.
	 *
	 * @param newClass number type
	 */
	public void setNumberType(Class<? extends Number> newClass);

	/**
	 * Returns last set number type.
	 *
	 * @return last set number type
	 */
	public Class<? extends Number> getNumberType();

}

/* __oOo__ */
