/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.range2;

import com.cosylab.util.PrintfFormat;

/**
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @version $Id: DefaultTickParameters.java,v 1.4 2008-04-22 12:28:40 jbobnar Exp $
 *
 */
public class DefaultTickParameters implements TickParameters {
	
	private int width;
	private PrintfFormat formatter;

	/**
	 * Default constructor for DefaultTickCallback (tick width is 50, format is %3.2f).
	 */
	public DefaultTickParameters()
	{
		this(50, "%3.2f");
	}

	/**
	 * Constructs a new DefaultTickParameters object with the given tickWidth and 
	 * string format used to format values.
	 * 
	 * @param tickWidth
	 * @param format
	 */
	public DefaultTickParameters(int tickWidth, String format)
	{
		width = Math.max(tickWidth, 0);
		formatter = new PrintfFormat(format);
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.range2.TickParameters#measureTick(double, java.lang.String)
	 */
	public int measureTick(double position, String text)
	{
		return width;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.range2.TickParameters#formatNumber(double)
	 */
	public String formatNumber(double value) {
		return formatter.sprintf(value);
	}
}
