/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.range2;

/**
 * Helper class for the Ticks class. Tick contains information about individual
 * tick that is rendered on the scale.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version 1.0
 */
public final class Tick
{
	/** DOCUMENT ME! */
	public static final int TICKS_OFFSET_PIXELS = 20;

	// Value in range [0..1] indicating the position of the tick on the scale

	/** DOCUMENT ME! */
	public double proportional = 0;

	/** DOCUMENT ME! */
	public double absolute = 0;

	// If true, the tick will be rendered longer

	/** DOCUMENT ME! */
	public boolean major = false;

	// Text to be displayed for this tick.
	// Text is only displayed if major == true.

	/** DOCUMENT ME! */
	public String text = null;

	/**
	 * Constructs new Tick
	 *
	 * @param proportional
	 * @param major
	 */
	public Tick(double absolute, double proportional, boolean major)
	{
		this.absolute= absolute;
		this.proportional = proportional;
		this.major = major;
		this.text = null;
	}

	/**
	 * Constructs new Tick
	 *
	 * @param proportional
	 * @param major
	 * @param text
	 */
	public Tick(double absolute, double proportional, boolean major, String text)
	{
		this.absolute= absolute;
		this.proportional = proportional;
		this.major = major;
		this.text = text;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("Tick = { text='");
		sb.append(text);
		sb.append("' major='");
		sb.append(major);
		sb.append("' prop='");
		sb.append(proportional);
		sb.append("' abs='");
		sb.append(absolute);
		sb.append("' }");

		return sb.toString();
	}
}

/* __oOo__ */
