/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.range2;

/**
 * Ensures that value will never be outside the bounds. Bounds will never be
 * adjusted, only the value will be trimmed to minimum and maximum. If values
 * are changed, value will be adjusted accordingly.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public final class TrimValuePolicy extends RangedValuePolicy
{
	
	/**
	 * Trims value to bounds.
	 */
	public static RangedValue trim(RangedValue params)
	{
		double value = params.getValue();
		value = Math.max(value, params.getMinimum());
		value = Math.min(value, params.getMaximum());
		return new RangedValue(params.getMinimum(), params.getMaximum(), value);
	}

	/**
	 * Trims value to bounds.
	 *
	 * @see com.cosylab.gui.components.range.RangedValuePolicy#validate(RangedValueHolder)
	 */
	@Override
	public RangedValue validate(RangedValue params)
	{
		params = trim(params);
		return validatePeerPolicy(params);
	}
}

/* __oOo__ */
