/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.slider;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import com.cosylab.gui.components.ResizableTextLabel;
import com.cosylab.gui.components.Slider;
import com.cosylab.gui.components.util.ColorHelper;

/**
 * Part of slider UI, displays title and any other general information. This
 * class is intended to be extended.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class InfoBar extends JPanel {
	
	private static final long serialVersionUID = 1L;

	private ResizableTextLabel deviceName;

	private JButton sync;
	private JButton set;
	private JButton store;
	private JButton restore;

	private Slider slider;

	private JLabel stored;
	private JCheckBox continuousMode;

	private boolean storedValueLabelVisible = true;

	/**
	 * Creates a new InfoBar object.
	 * 
	 * @param slider
	 */
	public InfoBar(Slider slider) {
		super();
		this.slider = slider;
		initialize();
	}

	private void initialize() {
		setLayout(new GridBagLayout());
		setOpaque(false);

		add(getTitleComponent(), new GridBagConstraints(0, 0, 1, 1, 1, 0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(0, 4, 0, 0), 0, 0));

		add(getSetButton(), new GridBagConstraints(1, 0, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 1, 0, 0), 0, 0));
		getSetButton().setVisible(false);

		add(getContinuousMode(), new GridBagConstraints(2, 0, 1, 1, 1, 0,
				GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,
						4, 0, 4), 0, 0));
		add(getStoreButton(), new GridBagConstraints(3, 0, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 1, 0, 0), 0, 0));
		add(getStoredValueLabel(), new GridBagConstraints(4, 0, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 4, 0, 3), 0, 0));
		add(getRestoreButton(), new GridBagConstraints(5, 0, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						0, 1, 0, 0), 0, 0));

		add(getSyncButton(), new GridBagConstraints(6, 0, 1, 1, 0, 0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						1, 0, 4), 0, 0));
	}

	private ResizableTextLabel getTitleComponent() {
		if (deviceName == null) {
			deviceName = new ResizableTextLabel("");
			deviceName.setOpaque(false);
			deviceName.setResizable(false);
			deviceName.setHorizontalAlignment(SwingConstants.LEFT);
		}

		return deviceName;
	}

	private JButton getSyncButton() {
		if (sync == null) {
			sync = new JButton("Sync");
			sync.setMargin(new Insets(0, 1, 0, 1));
			Font font = sync.getFont().deriveFont(Font.PLAIN);
			sync.setFont(font.deriveFont(font.getSize() * 0.8f));
			sync.setOpaque(true);
			sync.setBackground(ColorHelper.getCosyControl());
			sync.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					slider.synchronize();
				}
			});
			sync.setToolTipText("Synchronize Value with Readback");
		}

		return sync;
	}

	private JButton getStoreButton() {
		if (store == null) {
			store = new JButton("Store");
			store.setMargin(new Insets(0, 1, 0, 1));
			Font font = store.getFont().deriveFont(Font.PLAIN);
			store.setFont(font.deriveFont(font.getSize() * 0.8f));
			store.setOpaque(true);
			store.setBackground(ColorHelper.getCosyControl());
			store.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					slider.storeValue();
				}
			});
			store.setToolTipText("Store Current Readback Value");

		}

		return store;
	}

	private JButton getRestoreButton() {
		if (restore == null) {
			restore = new JButton("Restore");
			restore.setMargin(new Insets(0, 1, 0, 1));
			Font font = restore.getFont().deriveFont(Font.PLAIN);
			restore.setFont(font.deriveFont(font.getSize() * 0.8f));
			restore.setOpaque(true);
			restore.setBackground(ColorHelper.getCosyControl());
			restore.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					slider.restoreValue();
				}
			});
			restore.setToolTipText("Set value to: <no value>");

		}

		return restore;
	}

	private JLabel getStoredValueLabel() {
		if (stored == null) {
			stored = new JLabel("<no value>");
			stored.setFont(stored.getFont().deriveFont(Font.PLAIN));
			stored.setOpaque(true);
			// stored.setBackground(ColorHelper.getCosyControl());
			stored.setToolTipText("<no value>");
		}

		return stored;
	}

	private JButton getSetButton() {
		if (set == null) {
			set = new JButton("Set");
			set.setMargin(new Insets(0, 1, 0, 1));
			Font font = set.getFont().deriveFont(Font.PLAIN);
            set.setFont(font.deriveFont(font.getSize() * 0.8f));
			set.setOpaque(true);
			set.setBackground(ColorHelper.getCosyControl());
			set.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					slider.setManual();
				}
			});
			set.setToolTipText("Set Value");
		}

		return set;
	}

	/**
	 * Sets the visibility of the set button.
	 * 
	 * @param visible
	 */
	public void setSetButtonVisible(boolean visible) {
		getSetButton().setVisible(visible);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#setBackground(java.awt.Color)
	 */
	@Override
	public void setBackground(Color bg) {
		super.setBackground(bg);
		getSyncButton().setBackground(bg);
		getStoreButton().setBackground(bg);
		getRestoreButton().setBackground(bg);
		getSetButton().setBackground(bg);
		getTitleComponent().setBackground(bg);
		getStoredValueLabel().setBackground(bg);
		getContinuousMode().setBackground(bg);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#setForeground(java.awt.Color)
	 */
	@Override
	public void setForeground(Color fg) {
		super.setForeground(fg);
		getSyncButton().setForeground(fg);
		getStoreButton().setForeground(fg);
		getSetButton().setForeground(fg);
		getRestoreButton().setForeground(fg);
		getTitleComponent().setForeground(fg);
		getStoredValueLabel().setForeground(fg);
		getContinuousMode().setForeground(fg);
	}

	/**
	 * Sets the title for slider.
	 * 
	 * @param title
	 *            of slider.
	 */
	public void setTitle(String title) {
		getTitleComponent().setText(title);
	}

	/**
	 * Returns title of slider.
	 * 
	 * @return current title of slider.
	 */
	public String getTitle() {
		return getTitleComponent().getText();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#setFont(java.awt.Font)
	 */
	@Override
	public void setFont(Font font) {
		super.setFont(font);
		getTitleComponent().setFont(font);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Component#getFont()
	 */
	@Override
	public Font getFont() {
		return getTitleComponent().getFont();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		getSyncButton().setEnabled(enabled);
		getSetButton().setEnabled(enabled);
		getContinuousMode().setEnabled(enabled);
		getRestoreButton().setEnabled(enabled);
		getStoreButton().setEnabled(enabled);
		getStoredValueLabel().setEnabled(enabled);
	}

	/**
	 * Sets the store value as tooltip to the Restore button.
	 * 
	 * @param value
	 */
	public void setStoredValue(String value) {
		if (value == null) {
			getRestoreButton().setToolTipText("Set value to: <no value>");
			getStoredValueLabel().setText("<no value>");
			getStoredValueLabel().setToolTipText("<no value>");
		} else {
			getRestoreButton().setToolTipText("Set value to: " + value);
			getStoredValueLabel().setText(value);
			getStoredValueLabel().setToolTipText(value);
		}
	}

	/**
	 * Sets the title of the restore button.
	 * 
	 * @param title
	 */
	public void setRestoreButtonTitle(String title) {
		getRestoreButton().setText(title);
	}

	/**
	 * Returns the title of the restore button;
	 * 
	 * @return
	 */
	public String getRestoreButtonTitle() {
		return getRestoreButton().getText();
	}

	/**
	 * Sets the visibility of the store/restore button.
	 * 
	 * @param button
	 *            the button
	 * @param visible
	 *            true if button should be visible
	 */
	public void setStorageButtonsVisible(boolean visible) {
		getStoreButton().setVisible(visible);
		getRestoreButton().setVisible(visible);
		getStoredValueLabel().setVisible(
				storedValueLabelVisible && getStoreButton().isVisible());
	}

	/**
	 * Sets the visibility of the storeed value label.
	 * 
	 * @param button
	 *            the button
	 * @param visible
	 *            true if button should be visible
	 */
	public void setStoredValueLabelVisible(boolean visible) {
		storedValueLabelVisible = visible;
		getStoredValueLabel().setVisible(
				storedValueLabelVisible && getStoreButton().isVisible());
	}

	/**
	 * Returns the visibility of the store/restore buttons.
	 * 
	 * @param button
	 * @return visibility of the button
	 */
	public boolean isStorageButtonsVisible() {
		return getStoreButton().isVisible();
	}

	/**
	 * Sets the visibility of the sync button.
	 * 
	 * @param button
	 *            the button
	 * @param visible
	 *            true if button should be visible
	 */
	public void setSyncButtonVisible(boolean visible) {
		getSyncButton().setVisible(visible);
	}

	/**
	 * Returns the visibility of the sync button.
	 * 
	 * @param button
	 * @return visibility of the button
	 */
	public boolean isSyncButtonVisible() {
		return getSyncButton().isVisible();
	}

	/**
	 * Returns true if the stored value label is visible.
	 * 
	 * @return true if label is visible or false otherwise
	 */
	public boolean isStoredValueLabelVisible() {
		return storedValueLabelVisible;
	}

	/**
	 * Returns the continuousMode combobox
	 * 
	 * @param continuousMode
	 */
	private JCheckBox getContinuousMode() {
		if (continuousMode == null) {
			continuousMode = new JCheckBox("Continuous");
			Font font = continuousMode.getFont().deriveFont(Font.PLAIN);
			continuousMode.setFont(font.deriveFont(font.getSize() * 0.8f));
			continuousMode.setOpaque(false);			
			continuousMode.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					if(continuousMode.isSelected()){
						slider.setContinuousModeEnabled(true);
					} else {
						slider.setContinuousModeEnabled(false);
					}
				}
			});
		}
		return continuousMode;
	}
	
	/**
	 * Sets the continuous mode. If true the continuous mode is enabled otherwise
	 * it is false.
	 * 
	 * @see Slider#setContinuousModeEnabled(boolean)
	 * @param b true if enabled
	 */
	public void setContinuousMode(boolean b) {
		if (b!=getContinuousMode().isSelected()) {
			getContinuousMode().setSelected(b);
		}
	}
	
	/**
	 * Sets the visibility of the continuous mode button.
	 * 
	 * @param b true if visible or false otherwise
	 */
	public void setContinuousModeVisible(boolean b) {
		if (b!=getContinuousMode().isVisible()) {
			getContinuousMode().setVisible(b);
		}
	}
	
	/**
	 * Returns true if continuous mode is turned on.
	 * 
	 * @return true if continuous mode or false otherwise
	 */
	public boolean isContinuousMode() {
		return getContinuousMode().isSelected();
	}

	/**
	 * Returns true if the continuous mode button is visible.
	 * 
	 * @return true if continuous mode button is visible
	 */
	public boolean isContinuousModeVisible() {
		return getContinuousMode().isVisible();
	}
	
}

/* __oOo__ */
