/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table;

import com.cosylab.gui.components.table.cells.TableCell;


/**
 * Default implementation of the ObjectTableEngine.
 * 
 */
public class DefaultObjectTableEngine implements ObjectTableEngine
{

	public DefaultObjectTableEngine()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRowFactory#getTableRow(java.lang.Object)
	 */
	public TableRow getTableRow(Object key)
	{
		TableCell[] tc = getTableCells(key);

		TableRowArray rta = new TableRowArray();

		for (int i = 0; i < tc.length; i++) {
			rta.addValue(tc[i]);
		}

		return rta;
		
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRowFactory#releaseTableRow(java.lang.Object, com.cosylab.gui.components.table.TableRow)
	 */
	public void releaseTableRow(Object key, TableRow row)
	{
		// not used
	}

	/**
	 * Override this method to convert key object to own array of
	 * <code>TableCell</code> instances. Default implementation creates for
	 * each object own <code>TableCell</code> and returns it in an array.
	 *
	 * @param key key object
	 *
	 * @return array of table cells
	 */
	protected TableCell[] getTableCells(Object key)
	{
		if (key instanceof Object[]) {
			Object[] o = (Object[])key;

			TableCell[] tc = new TableCell[o.length];

			for (int i = 0; i < o.length; i++) {
				tc[i] = new TableCell(null, o[i], o[i], null);
			}
		}

		return new TableCell[]{ new TableCell(null, key, key, null) };
	}
	
	/**
	 * Dummy implementation: casts row element to TableCell and sets value 
	 * @see com.cosylab.gui.components.table.ObjectTableEngine#setValue(Object, Object, TableRow, int)
	 */
	public void setValue(Object value, Object key, TableRow row, int col) {
		TableCell r= (TableCell)row.getValueAt(col);
		r.setValue(value);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.ObjectTableEngine#getColumnName(int)
	 */
	public String getColumnName(int i) {
		return null;
	}
}

/* __oOo__ */
