/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;


/**
 * <code>TableModelRows</code> is implementation of <code>TableModel</code>,
 * which accepts data in form of <code>TableRowModel</code>, which cvan be set
 * only as construction parameter.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableModelRows.java,v 1.6 2008-04-22 12:28:40 jbobnar Exp $
 *
 * @since May 3, 2004.
 */
public class TableModelRows extends AbstractTableModel
	implements TableRowModelListener
{
	private static final long serialVersionUID = 1L;
	private TableRowModel rowModel;

	/**
	 * Creates a new TableModelRows object.
	 *
	 * @param model the row mode, which is used as data source
	 */
	public TableModelRows(TableRowModel model)
	{
		this();
		rowModel = model;
		model.addTableRowModelListener(this);
	}

	protected TableModelRows()
	{
		super();
	}

	/*
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount()
	{
		return rowModel.getColumnCount();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount()
	{
		return rowModel.getRowCount();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		TableRow r = rowModel.getRowAt(rowIndex);

		if (r != null) {
			return r.getValueAt(columnIndex);
		}

		return null;
	}

	/**
	 * Returns row model wrapped inside thia table model.
	 *
	 * @return the row model
	 */
	public TableRowModel getRowModel()
	{
		return rowModel;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRowModelListener#tableChanged(com.cosylab.gui.components.table.TableRowModelEvent)
	 */
	public void tableChanged(TableRowModelEvent event)
	{
		int first = event.getRowIndex();
		int last = first;
		
		if (first==TableRowModelEvent.HEADER_ROW) {
			fireTableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
			return;
		}

		for (int i = 1; i < event.getRows().length; i++) {
			int j = rowModel.getRowIndex(event.getRows()[i]);

			if (j < first) {
				first = j;
			} else if (j > last) {
				last = j;
			}
		}

		if (first == -1) {
			first = 0;
			last = getRowCount();
		}

		fireTableChanged(new TableModelEvent(this, first, last,
		        event.getValue(), event.getType()));
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
    public String getColumnName(int column)
	{
		return rowModel.getColumnName(column);
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
	 */
	@Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		TableRow r = rowModel.getRowAt(rowIndex);

		if (r == null) {
			return false;
		}

		return r.isValueSettable(columnIndex);
	}
}

/* __oOo__ */
