/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table;

/**
 * <code>TableRow</code> presents a row in a table.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableRow.java,v 1.6 2008-04-22 12:28:40 jbobnar Exp $
 *
 * @since May 3, 2004.
 */
public interface TableRow
{
	/**
	 * Returns value at column index.
	 *
	 * @param column the column index
	 *
	 * @return value at column index
	 */
	public Object getValueAt(int column);

	/**
	 * Returns the number values int this row.
	 *
	 * @return the number values int this row
	 */
	public int getValueCount();

	/**
	 * This method is called when row is added to model. When row is remove
	 * from the model, this method is called with parameter <code>null</code>.
	 * 
	 * <p>
	 * Row can call on parent model method
	 * <code>fireRowUpdate(TableRow,int)</code> when want to notify model
	 * listeners (specially the JTable) about data changes.
	 * </p>
	 *
	 * @param model the model to which row was added, or <code>null</code>,
	 *        when row is removed form model
	 *
	 * @see TableRowModel#fireRowUpdate(TableRow, int)
	 */
	public void setParentModel(TableRowModel model);

	/**
	 * The parent model which contains this row.
	 *
	 * @return the parent model which contains this row.
	 */
	public TableRowModel getParentModel();

	/**
	 * Return <code>true</code> if value at specified column index is editable.
	 *
	 * @param index the column index
	 *
	 * @return <code>true</code> if value at the specified column index is
	 *         editable
	 */
	public boolean isValueSettable(int index);

	/**
	 * Compares this table row with the specified table row for order.  Returns
	 * a negative integer, zero, or a positive integer as this row is less
	 * than, equal to, or greater than the specified row.
	 *
	 * @param row a row, to which it is compared
	 * @param valueIndex the index of value, which is compared in row
	 *
	 * @return negative integer, zero, or a positive integer as this row is
	 *         less than, equal to, or greater than the specified row.
	 */
	public int compareTo(TableRow row, int valueIndex);
}

/* __oOo__ */
