/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * <code>TableRowArray</code> is support  implementation of
 * <code>TableRow</code> build around <code>ArrayList</code>.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableRowArray.java,v 1.7 2008-04-22 12:28:40 jbobnar Exp $
 *
 * @since May 5, 2004.
 */
public class TableRowArray implements TableRow
{
	protected List<Object> values;
	protected TableRowModel parentModel;

	/**
	 * Creates new instance.
	 */
	public TableRowArray()
	{
		super();
		values = new ArrayList<>();
	}

	/**
	 * Creates new instance.
	 *
	 * @param l values as array
	 */
	public TableRowArray(Object[] l)
	{
		super();
		values = Arrays.asList(l);
	}

	/**
	 * Creates new instance.
	 *
	 * @param l values as List
	 */
	public TableRowArray(List<Object> l)
	{
		super();
		values = l;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#getValueAt(int)
	 */
	public Object getValueAt(int column)
	{
		return values.get(column);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#getValueCount()
	 */
	public int getValueCount()
	{
		return values.size();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#setParentModel(com.cosylab.gui.components.table.TableRowModel)
	 */
	public void setParentModel(TableRowModel model)
	{
		parentModel = model;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#getParentModel()
	 */
	public TableRowModel getParentModel()
	{
		return parentModel;
	}

	/**
	 * Adds new value to the end of this row.
	 *
	 * @param value the value added to the end of this row
	 */
	public void addValue(Object value)
	{
		values.add(value);
	}

	/**
	 * Inserts new value to this row at specified position.
	 *
	 * @param index where to insert value
	 * @param value new inserted value
	 */
	public void addValue(int index, Object value)
	{
		values.add(index, value);
	}

	/**
	 * Replaces value at specified index.
	 *
	 * @param index where value is set
	 * @param newValue new value
	 *
	 * @return old value at index
	 */
	public Object setValue(int index, Object newValue)
	{
		return values.set(index, newValue);
	}

	/*
	 * Fires update event for entire row.
	 */
	protected void fireUpdate()
	{
		fireUpdate(TableRowModelEvent.ALL_VALUES);
	}

	/*
	 * Fires update event for value in row.
	 */
	protected void fireUpdate(int index)
	{
		if (parentModel != null) {
			parentModel.fireRowUpdate(this, index);
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#isEditable(int)
	 */
	public boolean isValueSettable(int index)
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#compateTo(com.cosylab.gui.components.table.TableRow, int)
	 */
	public int compareTo(TableRow row, int valueIndex)
	{
		Object o1 = getValueAt(valueIndex);
		Object o2 = row.getValueAt(valueIndex);

		if (o1 instanceof Comparable) {
			return ((Comparable)o1).compareTo(o2);
		}

		return 0;
	}
}

/* __oOo__ */
