/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.cells;

/**
 * BooleanCell encapsulates boolean value.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class BooleanCell extends TableCell
{
	/**
		 *
		 */
	public BooleanCell()
	{
		super();
	}

	/**
	 * Constructs a new BooleanCell.
	 *
	 * @param identifier
	 * @param dataSource
	 * @param value
	 * @param commands
	 */
	public BooleanCell(String identifier, Object dataSource, boolean value,
	    Command[] commands)
	{
		super(identifier, dataSource, new Boolean(value), commands);
	}

	/**
	 * Returns the boolean value displayed inside the cell.
	 *
	 * @return the boolean value
	 */
	public boolean getBooleanValue()
	{
		return value instanceof Boolean ? ((Boolean)value).booleanValue() : false;
	}

	/**
	 * Sets the boolean value to the cell.
	 *
	 * @param value new boolean value
	 */
	public void setBooleanValue(boolean value)
	{
		setValue(new Boolean(value));
	}
}

/* __oOo__ */
