/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.numberfield.LongDocument;
import com.cosylab.gui.components.table.cells.LongCell;

import com.cosylab.util.StringUtilities;

import java.awt.Component;

import javax.swing.JTable;


/**
 * Edits LongCell value.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class LongCellEditor extends JTextFieldCellEditor
{
	private static final long serialVersionUID = 1L;
	private LongDocument document;
	private boolean active;

	/**
	 * Creates a new DoubleCellEditor object.
	 */
	public LongCellEditor()
	{
		setDocument(document = new LongDocument());
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	@Override
    public Component getTableCellEditorComponent(JTable table, Object cell,
	    boolean isSelected, int row, int column)
	{
		if (cell instanceof LongCell) {
			active = !((LongCell)cell).isSuspended();

			document.setMaxValue(((LongCell)cell).getMaximum());
			document.setMinValue(((LongCell)cell).getMinimum());

			if (active) {
				setText(Long.toString(((LongCell)cell).getLongValue()));
			} else {
				setText("");
			}

			selectAll();
		}

		return super.getTableCellEditorComponent(table, cell, isSelected, row,
		    column);
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue()
	{
		return document.getValue();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#isValueValid()
	 */
	@Override
    protected boolean isValueValid()
	{
		return StringUtilities.isTrueNumber(getText());
	}
}

/* __oOo__ */
