/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.LongEnumCell;

import java.awt.Component;

import javax.swing.JTable;


/**
 * Default table renderer for LongEnumCell. Displays enum state description.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class LongEnumCellRenderer extends DefaultTableCellRenderer
{
	private static final long serialVersionUID = 1L;

	/**
	 * Constructs a new LongEnumCellRenderer.
	 *
	 */
	public LongEnumCellRenderer()
	{
		super();
	}

	/**
	 * Constructs a new LongEnumCellRenderer.
	 * 
	 * @param decorateBorder if true decorated border will be used
	 * @param decorateBackground if true decorated background will be used
	 */
	public LongEnumCellRenderer(boolean decorateBorder,
	    boolean decorateBackground)
	{
		super(decorateBorder, decorateBackground);
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
    public Component getTableCellRendererComponent(JTable table, Object cell,
	    boolean isSelected, boolean hasFocus, int row, int column)
	{
		super.getTableCellRendererComponent(table, cell, isSelected, hasFocus,
		    row, column);

		if (cell instanceof LongEnumCell) {
			LongEnumCell property = (LongEnumCell)cell;

			if (property.isSuspended()) {
				setText("");
			} else {
				long l = property.getLongValue();
				String s = property.getStateDescription(l);

				if (s == null) {
					StringBuffer sb = new StringBuffer(8);
					sb.append('<');
					sb.append(l);
					sb.append('>');
					s = sb.toString();
				}

				setText(s);
			}
		}

		return this;
	}
}

/* __oOo__ */
