/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.table.renderers;

import java.awt.Color;

import javax.swing.border.LineBorder;


/**
 * Border displayed whenever timeout is detected for individual cell.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class TimeoutBorder extends LineBorder
{
	private static final long serialVersionUID = 1L;
	private Color disabledColor = Color.white;
	protected boolean enabled;
	private Color color;

	/**
	 * TimeoutBorder constructor comment.
	 */
	public TimeoutBorder()
	{
		super(new java.awt.Color(255, 90, 200), 2);
		color = lineColor;
	}

	/**
	 * Returns color of this border when cell is disabled.
	 *
	 * @return Color of this cell when disabled.
	 */
	public Color getDisabledColor()
	{
		return disabledColor;
	}

	/**
	 * Returns whether this border is enabled.
	 *
	 * @return true if enabled.
	 */
	public boolean isEnabled()
	{
		return enabled;
	}

	/**
	 * Sets the color when this border is disabled.
	 *
	 * @param newDisabledColor new disabled color.
	 */
	public void setDisabledColor(Color newDisabledColor)
	{
		disabledColor = newDisabledColor;
	}

	/**
	 * Sets whether this border is enabled.
	 *
	 * @param newEnabled boolean
	 */
	public void setEnabled(boolean newEnabled)
	{
		enabled = newEnabled;
		lineColor = enabled ? color : disabledColor;
	}
}

/* __oOo__ */
