/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.util;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;

/**
 * This is a convenience class with static methods used to position graphic components 
 * on screen.
 * 
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */

public class ComponentPositioner {
	
	/**
	 * Centers a window in the middle of the screen. Screen size is queried from the 
	 * AWT Toolkit. The window is centered by calling its <code>setLocation</code> function.
	 * 
	 * @param w the window to be centered
	 */
	public static void centerOnScreen(Window w)
	{
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension size = w.getSize();
		w.setLocation(screenSize.width / 2 - size.width / 2, screenSize.height / 2 - size.height / 2);
	}
}

