/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.util;

import java.awt.Color;

import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;


/**
 * A color and font theme used for MetalLookAndFeel.
 *
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class CosyMetalTheme extends MetalTheme {
    private static final ColorUIResource primary1 = new ColorUIResource(102,
            102, 153);
    private static final ColorUIResource primary2 = new ColorUIResource(153,
            153, 204);
    private static final ColorUIResource primary3 = new ColorUIResource(204,
            204, 255);
    private static final ColorUIResource secondary1 = new ColorUIResource(102,
            102, 102);
    private static final ColorUIResource secondary2 = new ColorUIResource(153,
            153, 153);
    private static final ColorUIResource secondary3 = new ColorUIResource(204,
            204, 204);

    /*
      private static final ColorUIResource primary1=new ColorUIResource(SystemColor.inactiveCaption);
            private static final ColorUIResource primary2=new ColorUIResource(SystemColor.activeCaption);
            private static final ColorUIResource primary3=new ColorUIResource(SystemColor.info);

            private static final ColorUIResource secondary1=new ColorUIResource(SystemColor.controlDkShadow);
            private static final ColorUIResource secondary2=new ColorUIResource(SystemColor.controlShadow);
            private static final ColorUIResource secondary3=new ColorUIResource(SystemColor.control);
    */
    private static final ColorUIResource white = new ColorUIResource(Color.white);
    private static final ColorUIResource black = new ColorUIResource(Color.black);
    private static final FontUIResource controlFont = MetalLookAndFeel.getControlTextFont();
    private static final FontUIResource menuFont = MetalLookAndFeel.getMenuTextFont();
    private static final FontUIResource subFont = MetalLookAndFeel.getSubTextFont();
    private static final FontUIResource systemFont = MetalLookAndFeel.getSystemTextFont();
    private static final FontUIResource windowFont = MetalLookAndFeel.getWindowTitleFont();
    private static final FontUIResource userFont = MetalLookAndFeel.getUserTextFont();

    /**
     * @see MetalTheme#getName()
     */
    @Override
    public String getName() {
        return "CosyBeans Metal Theme";
    }

    /**
     * @see MetalTheme#getPrimary1()
     */
    @Override
    protected ColorUIResource getPrimary1() {
        return primary1;
    }

    /**
     * @see MetalTheme#getPrimary2()
     */
    @Override
    protected ColorUIResource getPrimary2() {
        return primary2;
    }

    /**
     * @see MetalTheme#getPrimary3()
     */
    @Override
    protected ColorUIResource getPrimary3() {
        return primary3;
    }

    /**
     * @see MetalTheme#getSecondary1()
     */
    @Override
    protected ColorUIResource getSecondary1() {
        return secondary1;
    }

    /**
     * @see MetalTheme#getSecondary2()
     */
    @Override
    protected ColorUIResource getSecondary2() {
        return secondary2;
    }

    /**
     * @see MetalTheme#getSecondary3()
     */
    @Override
    protected ColorUIResource getSecondary3() {
        return secondary3;
    }

    /**
     * @see MetalTheme#getControlTextFont()
     */
    @Override
    public FontUIResource getControlTextFont() {
        return controlFont;
    }

    /**
     * @see MetalTheme#getSystemTextFont()
     */
    @Override
    public FontUIResource getSystemTextFont() {
        return systemFont;
    }

    /**
     * @see MetalTheme#getUserTextFont()
     */
    @Override
    public FontUIResource getUserTextFont() {
        return userFont;
    }

    /**
     * @see MetalTheme#getMenuTextFont()
     */
    @Override
    public FontUIResource getMenuTextFont() {
        return menuFont;
    }

    /**
     * @see MetalTheme#getWindowTitleFont()
     */
    @Override
    public FontUIResource getWindowTitleFont() {
        return windowFont;
    }

    /**
     * @see MetalTheme#getSubTextFont()
     */
    @Override
    public FontUIResource getSubTextFont() {
        return subFont;
    }

    /**
     * @see javax.swing.plaf.metal.MetalTheme#getBlack()
     */
    @Override
    protected ColorUIResource getBlack() {
        return black;
    }

    /**
     * @see javax.swing.plaf.metal.MetalTheme#getWhite()
     */
    @Override
    protected ColorUIResource getWhite() {
        return white;
    }
}
