/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.util;


import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.TransferHandler;

import com.cosylab.gui.components.customizer.ImageFilter;

/**
 * @author tomo
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class ScreenCapturer {

	private Component component = null;

	/**
	 * String that holds path(full path: example "C:\something\gauger.png") ,
	 * where to create image
	 */
	private String path;

	/**
	 * ScreenCapturer supportable formats
	 */
	private String[] formats = { "png", "jpeg", "jpg" };

	/**
	 * A varuble that holds JFileChooser-s action(like what button was
	 * pressed...)
	 */
	private int retval;

	/**
	 * Creates new object
	 * 
	 * @param com
	 *            Component
	 */
	public ScreenCapturer(Component com) {
		if (com != null) {
			component = com;
		} else {
			//TODO throw NullPointer...
		}
	}

	/**
	 * Creates new object
	 * 
	 * @param com
	 *            Component to paint
	 * @param p
	 *            String path , where to save file
	 */
	public ScreenCapturer(Component com, String p) {

		component = com;
		path = p;
	}

	/**
	 * Creates new object
	 *  
	 */
	public ScreenCapturer() {
	}

	/**
	 * Shows JFileChooser, sets filters for supportable file types , and saves
	 * screen image
	 *  
	 */
	public void showScreenDialog() {

		JFileChooser chooser = new JFileChooser();
		ImageFilter pngFilter = new ImageFilter() {
			@Override
            public boolean accept(File f) {
				if (f.isDirectory())
					return true;
				String extension = getExtension(f);
				if (extension.equals("png"))
					return true;
				return false;
			}

			@Override
            public String getDescription() {
				return "png";
			}
		};

		ImageFilter jpegFilter = new ImageFilter() {
			@Override
            public boolean accept(File f) {
				if (f.isDirectory())
					return true;
				String extension = getExtension(f);
				if (extension.equals("jpeg"))
					return true;
				return false;
			}

			@Override
            public String getDescription() {
				return "jpeg";
			}
		};

		ImageFilter jpgFilter = new ImageFilter() {
			@Override
            public boolean accept(File f) {
				if (f.isDirectory())
					return true;
				String extension = getExtension(f);
				if (extension.equals("jpg"))
					return true;
				return false;
			}

			@Override
            public String getDescription() {
				return "jpg";
			}
		};

		chooser.setFileFilter(jpgFilter);
		chooser.addChoosableFileFilter(jpgFilter);
		chooser.setFileFilter(jpegFilter);
		chooser.addChoosableFileFilter(jpegFilter);
		chooser.setFileFilter(pngFilter);
		chooser.addChoosableFileFilter(pngFilter);

		retval = chooser.showSaveDialog(component);
		if (retval == JFileChooser.APPROVE_OPTION) {
			path = chooser.getSelectedFile().getAbsolutePath();
			if (chooser.getFileFilter() == jpegFilter) {
				path += ".jpeg";
			} else if (chooser.getFileFilter() == jpgFilter) {
				path += ".jpg";
			} else {
				path += ".png";
			}

			saveImage();

		} else {
			return;
		}

	}

	/**
	 * returns file type
	 * 
	 * @return String
	 */
	public String getFormat() {
		String ext[] = path.split("\\.");

		if (ext[ext.length - 1] == formats[2]
				|| ext[ext.length - 1] == formats[1]) {

			return ext[ext.length - 1];
		} else
			return formats[0];

	}

	/**
	 * returns file path
	 * 
	 * @return String
	 */
	public String getFile() {
		return path;
	}

	/**
	 * set the file path
	 * 
	 * @param path
	 *            String
	 */
	public void setFile(String path) {
		this.path = path;
	}

	/**
	 * Saves image to file ...
	 *  
	 */
	public void saveImage() {
		if (component != null) {

			int w = component.getWidth();
			int h = component.getHeight();
			BufferedImage bi = new BufferedImage(w, h,
					BufferedImage.TYPE_INT_RGB);
			Graphics2D g2 = bi.createGraphics();
			component.paint(g2);

			g2.dispose();
			try {
			  ImageIO.write(bi, getFormat(), new File(path));
				
			} catch (Exception ioe) {
				System.out.println("IOE: " + ioe.getMessage());
			}
		}
	}
	
	public void saveImageToSysClipBoard(Component componentToCopy){	     
	    if (componentToCopy != null) {
			int w = componentToCopy.getWidth();
			int h = componentToCopy.getHeight();
			BufferedImage bi = new BufferedImage(w, h,
					BufferedImage.TYPE_INT_RGB);
			Graphics2D g2 = bi.createGraphics();
			componentToCopy.paint(g2);

			g2.dispose();
			try {
			    Toolkit kit = Toolkit.getDefaultToolkit();
			    final Clipboard clipboard = kit.getSystemClipboard();

		        Icon icon = new ImageIcon(bi);
		        final JLabel label = new JLabel(icon);
		        label.setTransferHandler(new ImageClipboardHelper());

			    TransferHandler transferHandler = label.getTransferHandler();
                transferHandler.exportToClipboard(label, clipboard,
                    TransferHandler.COPY);
			    
				
				
			} catch (Exception ioe) {
				System.out.println("IOE: " + ioe.getMessage());
			}
		}
	}
	
	

}
