/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.util;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JSplitPane;


/**
 * An action that, when associated with a JSplitPane,
 * changes the orientation from horizontal to vertical
 * and vice versa.
 *
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class SplitPaneOrientationAction extends AbstractAction {
   
	private static final long serialVersionUID = 1L;
	private JSplitPane split;

    /**
     * Creates a new action, associated with the <code>pane</code> to
     * change its orientation.
     */
    public SplitPaneOrientationAction(JSplitPane pane) {
        super();
        split = pane;
        putValue(AbstractAction.SHORT_DESCRIPTION, "Change Orientation");
        putValue(AbstractAction.LONG_DESCRIPTION,
            "Change orientation of the split pane");
        putValue(AbstractAction.NAME, "Change Orientation");
        putValue(AbstractAction.SMALL_ICON,
            IconHelper.createIcon(
                "icons/window/ChangeOrientation16.gif"));
    }

    /**
     * Called when the action is performed. Switches the orientation.
     */
    public void actionPerformed(ActionEvent e) {
        if (split.getOrientation() == JSplitPane.HORIZONTAL_SPLIT) {
            split.setOrientation(JSplitPane.VERTICAL_SPLIT);
        } else {
            split.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        }
    }
}
