/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.components.wheelswitch;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.LayoutManager;

import com.cosylab.gui.components.Wheelswitch;

/**
 * Implementation of <code>java.awt.LayoutManager</code> to be used for
 * the layout of the <code>Wheelswitch</code> component.
 * 
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class WheelswitchLayout implements LayoutManager {
  
	/**
	 * Constructor for WheelswitchLayout
	 */
	public WheelswitchLayout() {
		super();
	}
	
	/**
	 * Empty implementation.
	 * 
	 * @see LayoutManager#addLayoutComponent(String, Component)
	 */
	public void addLayoutComponent(String name, Component comp) {
	}

	/**
	 * Empty implementation.
	 * 
	 * @see LayoutManager#removeLayoutComponent(Component)
	 */
	public void removeLayoutComponent(Component comp) {
	}

	/**
	 * @see LayoutManager#preferredLayoutSize(Container)
	 */
	public Dimension preferredLayoutSize(Container parent) {
		return parent.getPreferredSize();
	}

	/**
	 * @see LayoutManager#minimumLayoutSize(Container)
	 */
	public Dimension minimumLayoutSize(Container parent) {
		return parent.getMinimumSize();
	}

	/**
	 * @see LayoutManager#layoutContainer(Container)
	 */
	public void layoutContainer(Container parent) {
		Wheelswitch wswitch = (Wheelswitch)parent;

		int numberOfComponents = wswitch.getComponentCount();
		if (numberOfComponents<2) return;
		
		int height = wswitch.getHeight();
		int width = wswitch.getWidth();

        if (width==0 || height==0) return;

		int preferredWidth = wswitch.getPreferredSize().width;
		int preferredHeight = wswitch.getPreferredSize().height;

		double widthRatio = 1.*width/preferredWidth;
		double heightRatio = 1.*height/preferredHeight;

		double ratio = Math.min(widthRatio,heightRatio);

		int x = (int)Math.ceil((width-preferredWidth*ratio)/2.);
		int y = 0;
		int compWidth = 0;
		int compHeight = 0;
		Component comp = null;
		for (int i=numberOfComponents-1; i>=0; i--) {
			comp = wswitch.getComponent(i);
			compHeight = (int)(comp.getPreferredSize().height*ratio);
			compWidth = (int)(comp.getPreferredSize().width*ratio);
			y = (int)((height-compHeight)/2.);
			x += compWidth;
			comp.setBounds(width-x,y,compWidth,compHeight);
		}
	}
}
