/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.displayers;

import java.util.Map;


/**
 * <p>
 * <code>DataConsumer</code> interface defines receiver for udates of
 * 
 * <ul>
 * <li>
 * dynamic value change
 * </li>
 * <li>
 * dynamic value quality (or state) change
 * </li>
 * <li>
 * dynamic value characteristics
 * </li>
 * </ul>
 * </p>
 * 
 * <p>
 * Combunation of daata consumer and data source is data pipe. It could be used
 * for data riltering or editing.
 * </p>
 * 
 * <p>
 * By default all data consumers are synchronous: they returns method call when
 * update was successfull of fail with exception.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: DataConsumer.java,v 1.4 2008-04-22 12:31:02 jbobnar Exp $
 *
 * @since Nov 24, 2003.
 */
public interface DataConsumer
{
	/**
	 * <p>
	 * Returns data consumer for requested data consumer type. If this data
	 * consumer does not support required data consumer, <code>null</code> is
	 * returned.
	 * </p>
	 * 
	 * <p>
	 * If this data cosnumer does not support specific data consumer type, then
	 * it means, that it is not desirable to cast  data of that type to any by
	 * this consumer supported data consumer types.
	 * </p>
	 *
	 * @param type Class of specific data consumer  (e.g. DoubleConsumer)
	 *
	 * @return implementation of required data consumer or <code>null</code> if
	 *         not supported
	 */
	public <D extends DataConsumer> D getDataConsumer(Class<D> type);

	/**
	 * Returns the implementation of default consumer type supported by this
	 * data consumer.
	 *
	 * @return the implementation of default consumer type supported by this
	 *         data consumer
	 */
	public DataConsumer getDefaultDataConsumer();

	/**
	 * Push data state (quality) update change for dynamic value. Usually
	 * called by data source.
	 *
	 * @param state new data state(quality) descriptor
	 */
	public void updateDataState(DataState state);

	/**
	 * Sets new dynamic value characteristics to this data consumer. This is
	 * usually called only at initializaation, before first data state or
	 * value is updated. Usually carries characteristics such as value minimum
	 * or maximum. Name and meaning of atributes is defined with contract
	 * betwean data consumer and source implementation.
	 *
	 * @param characteristics the <code>Map</code> with attri
	 */
	public void setCharacteristics(Map<String,Object> characteristics);

	/**
	 * Returns name of this data consumer.
	 *
	 * @return the name of this data consumer
	 */
	public String getName();

	/**
	 * Resturns array with names of supported characteristics. Data source can
	 * use this method to optimize setting characteristics to this data
	 * consumer. If arrray of length 0 is return, then consumer does not need
	 * or support any characteristic. If <code>null</code> is returned, than
	 * consumer expect all existing characteristics from data source.
	 *
	 * @return array with names of supported characteristics
	 */
	public String[] getSupportedCharacteristics();

	/**
	 * Returns array of supported data consumer types, which can be used as
	 * parameter and returnned with <code>getDataCosnumer(Class)</code>
	 * method.
	 *
	 * @return array of supported data consumer types
	 */
	public Class<DataConsumer>[] getSupportedConsumerTypes();
}

/* __oOo__ */
