/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.displayers;

import java.beans.PropertyVetoException;


/**
 * <code>DataSource</code>  is interface for generic data source. Data source
 * source dispatches dynamic value updates to registered data consumers.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: DataSource.java,v 1.4 2008-04-22 12:31:02 jbobnar Exp $
 *
 * @see com.cosylab.gui.displayers2.DataConsumer
 * @since Nov 24, 2003.
 */
public interface DataSource {
    /**
     * Registers data consumer at this data source. This data source can denay
     * consumer by throwing <code>PropertyVetoException</code> if consumer
     * does not provide necessary <code>DataConsumer</code> type.
     *
     * @param consumer the data consumer to be regitered at this data source
     *
     * @see DataConsumer#getDataConsumer(Class)
     */
    public void addConsumer(DataConsumer consumer) throws PropertyVetoException;

    /**
     * Deregisters data consumer from this data source.
     *
     * @param consumer data consumer to be deregistered
     */
    public void removeConsumer(DataConsumer consumer);

    /**
     * Returns array of registered data consumer.
     *
     * @return array of registered data consumer
     */
    public DataConsumer[] getConsumers();

    /**
     * Returns the array of consumer types, which are acceptable by this data
     * source.
     *
     * @return the array of acceptable data source types
     */
    public Class<DataConsumer>[] getAcceptableConsumerTypes();

    /**
     * Removes all consumers from data source and releases all resources
     * and bindings allocated inside datasource to the consumers.
     *
     */
    public void removeAllConsumers();
}


/* __oOo__ */
