/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.displayers;

import java.awt.Color;

import com.cosylab.util.BitCondition;

/**
 * DefaultConditionDescriptor take cares of Condition Colors , it defines
 * default colors for Conditions , sets Conditions other Colors.
 *
 * @author tomo
 */
public class DefaultConditionDescriptor implements ConditionDescriptor
{
	/** Default ERROR color. */
	public static final Color DEFAULT_ERROR = Color.RED;

	/** Defaut UNUSED color. */
	public static final Color DEFAULT_UNUSED = Color.GRAY;

	/** Default WARNING color. */
	public static final Color DEFAULT_WARNING = Color.YELLOW;

	/** Default OK color. */
	public static final Color DEFAULT_OK = Color.GREEN;
	private static BitCondition[] conditions = null;
	private static Color[] colors = null;
	private static boolean overRide = false;

	/**
	 * Simple constructor...
	 */
	public DefaultConditionDescriptor()
	{
		conditions=BitCondition.values();

		colors = new Color[conditions.length];

		for (int i = 0; i < conditions.length; i++) {
			colors[i] = Color.BLACK;
		}

		setDefaultColors();
	}

	/**
	 * Sets the default colors for conditions
	 */
	public void setDefaultColors()
	{
		for (int i = 0; i < conditions.length; i++) {
			if (conditions[i] == BitCondition.UNUSED) {
				colors[i] = DEFAULT_UNUSED;
			} else if (conditions[i] == BitCondition.OK) {
				colors[i] = DEFAULT_OK;
			} else if (conditions[i] == BitCondition.WARNING) {
				colors[i] = DEFAULT_WARNING;
			} else if (conditions[i] == BitCondition.ERROR) {
				colors[i] = DEFAULT_ERROR;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.cosylab.gui.displayers.ConditionDescriptor#toColor(com.cosylab.datatypes.Condition)
	 */
	public Color toColor(BitCondition condition)
	{
		Color c = null;

		for (int i = 0; i < conditions.length; i++) {
			if (condition == conditions[i]) {
				c = colors[i];
			}
		}

		return c;
	}

	private void checkIfColorAllreadyUsed(BitCondition condition, Color color)
		throws IllegalArgumentException
	{
		for (int i = 0; i < conditions.length; i++) {
			if (colors[i] == color && condition != conditions[i]) {
				if (overRide == false) {
					throw new IllegalArgumentException();
				}
			}
		}
	}

	/**
	 * Changes Conditions old Color to new Color , if new Color is allready
	 * used by other Condition then it throws IllegalArgumentException(). When
	 * changing colors , don't forget to update component...
	 *
	 * @param condition to change color
	 * @param color to what color
	 */
	public void putConditionToColor(BitCondition condition, Color color)
	{
		checkIfColorAllreadyUsed(condition, color);

		for (int i = 0; i < conditions.length; i++) {
			if (condition == conditions[i]) {
				colors[i] = color;
			}
		}
	}

	/**
	 * If in override mode , then color can be used on more then one Condition
	 * , else olny for one (if Color is allready used by other condition , it
	 * will throw IllegalArgumentException).
	 *
	 * @param value
	 */
	public void setOverRideOn(boolean value)
	{
		overRide = value;
	}

	/**
	 * Tells if DefaultConditionDescriptor is in overRide mode
	 *
	 * @return
	 */
	public boolean isOverRideOn()
	{
		return overRide;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.cosylab.gui.displayers.ConditionDescriptor#toCondition(java.awt.Color)
	 */
	public BitCondition toCondition(Color color)
	{
		BitCondition c = null;

		for (int i = 0; i < conditions.length; i++) {
			if (color == colors[i]) {
				c = conditions[i];
			}
		}

		return c;
	}

	/**
	 * Clears all colors
	 */
	public void clear()
	{
		colors = null;
	}
}

/* __oOo__ */
