/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.displayers;

import javax.swing.Icon;

import com.cosylab.gui.ValueIconPair;

/**
 * <code>IconDisplayer</code> can display long values (similar to LongDisplayer) 
 * but instead of showing the number value it shows different icons for each of
 * the received values. 
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public interface IconDisplayer extends Displayer, LongConsumer {

	/**
	 * Sets the value displayed by the displayer. This method may be invoked
	 * only by the adapter delivering a new value from the modeling layer.
	 *
	 * @param value new value to be displayed
	 */
	public void setValue(long value);

	/**
	 * Gets the value displayed by the displayer. This method can be invoked by
	 * any object and will probably most often be invoked by the GUI rendition
	 * code.
	 *
	 * @return double currently displayed value
	 */
	public long getValue();
	
	/**
	 * Sets the default icon for this displayer. If there is no icon assigned to a specific
	 * value, a default icon is shown when that value arrives.
	 * 
	 * @param icon
	 */
	public void setDefaultIcon(Icon icon);
	
	/**
	 * Returns the default icon.
	 * 
	 * @return
	 */
	public Icon getDefaultIcon();
	
	/**
	 * Returns an array of all icons that were associated with values.
	 * 
	 * @return
	 */
	public ValueIconPair[] getIcons();
	
	/**
	 * Associates icons with values. This method overrides all previously loaded icons.
	 * 
	 * @param icons
	 */
	public void setIcons(ValueIconPair[] pairs);
	
	/**
	 * Associates icons with values. This methods adds icons to previously
	 * loded inventory and overrides only those values that were loaded before and now
	 * is a different values associated with them. 
	 * 
	 * @param icons
	 */
	public void addIcons(ValueIconPair[] icons);
	
	/**
	 * Associates icon with value. This methods adds icon to previously
	 * loded inventory. 
	 * 
	 * @param icons
	 */
	public void addIcon(ValueIconPair pair); 
	
	/**
	 * Removes icons.
	 * 
	 * @param icons
	 */
	public void removeIcon(ValueIconPair pair);
	
	/**
	 * Removes icons.
	 * 
	 * @param icons
	 */
	public void removeIcons(ValueIconPair[] pairs);
}
