/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans.
 *
 * CosyBeans is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.displayers;

/**
 * <code>MultipleDataConsumer</code> is data consumer, which is able to process
 * at the same time several dynamic data sources by allocation own simple data
 * consumer for each source.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: MultipleDataConsumer.java,v 1.4 2008-04-22 12:31:02 jbobnar Exp $
 *
 * @since Nov 24, 2003.
 */
public interface MultipleDataConsumer
{
	/**
	 * <p>
	 * Returns data consumer of specified data consumer type and for specified
	 * string name. The organization of data is indicated with name (list,
	 * table, tree organization). Multiple consumer might interprete this by
	 * it's own preferences or abilities. The name of returned consuer must
	 * equals to provided name.
	 * </p>
	 * 
	 * <p>
	 * Example. Names: <tt>PS1/current, PS1/readback, PS2/current,
	 * PS2/readback</tt> migth table displayer interprete as two rows (PS1 and
	 * PS2) wtih two columns (current and readback). Tree displayer migth
	 * interprete this names as two branches with two subbranches.
	 * </p>
	 * 
	 * <p>
	 * Note that name is not fixed consumer property. Some displayers migth
	 * expect for name to be same as DISPLAY_NAME and is changed if display
	 * name is changed.
	 * </p>
	 *
	 * @param name organized data name
	 * @param type required data soncumer type
	 *
	 * @return newly allocated consumer for required name and type
	 */
	public <D extends DataConsumer> D getConsumer(String name, Class<D> type);

	/**
	 * Requests frm this multiple consumer to release all resources connectet
	 * to provided single consumer. Provided consumer must be created by this
	 * multiple consumer.
	 *
	 * @param consumer the consumer to be released
	 */
	public void releaseConsumer(DataConsumer consumer);

	/**
	 * Returns all data consumers created by this multiple data consumer.
	 *
	 * @return all data consumers created by this multiple data consumer
	 */
	public DataConsumer[] getConsumers();

	/**
	 * Returns array of supported data consumer types, which can be used as
	 * parameter and returnned  with <code>getConsumer(String,Class)</code>
	 * method.
	 *
	 * @return array of supported data consumer types
	 */
	public Class<DataConsumer>[] getSupportedConsumerTypes();
}

/* __oOo__ */
