/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.property.editors;

import java.awt.Component;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;

import com.cosylab.gui.components.SimpleButton;
import com.cosylab.gui.components.SimpleComboBox;
import com.cosylab.gui.components.util.CosyUIElements;
import com.cosylab.gui.components.util.FontHelper;

/**
 * A property editor for selecting Fonts using a combo box.
 * 
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class FontEditor extends SimpleComboBox implements PropertyEditor {
	/**
     * 
     */
    private static final long serialVersionUID = -4135974794618993061L;

    private class FontEditorUI extends SimpleComboBox.DropComboBoxUI {
		/**
		 * @see com.cosylab.gui.components.SimpleComboBox.DropComboBoxUI#getDropButton()
		 */
		@Override
        protected SimpleButton getDropButton() {
			return extendButton;
		}
	}

	private class FontRenderer extends DefaultListCellRenderer {
		/**
         * 
         */
        private static final long serialVersionUID = -5651585245832224280L;

        /**
		 * Returns a component displaying the font's name by using font itself.
		 * 
		 * @param list
		 *            containing values to be rendered
		 * @param value
		 *            to be rednered (Font)
		 * @param index
		 *            of value
		 * @param isSelected
		 *            true if renderer is selected
		 * @param cellHasFocus
		 *            tue if renderer has focus
		 * 
		 * @return a label displaying the Font
		 */
		@Override
        public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			String name = (String) value;
			super.getListCellRendererComponent(list, value, index, isSelected,
					cellHasFocus);

			// if (isSelected || !FontEditor.this.hasFocus())
			// FontEditor.this.setFont(Font.decode(name));
			setFont(Font.decode(name));

			if (!isPopupVisible()) {
				setBackground(FontEditor.this.getBackground());
			}

			return this;
		}
	}

	/** Both drop-down selection and button for the dialog. */
	public static final int MODE_BOTH = 0;

	/** Only the drop-down selection */
	public static final int MODE_COMBO = 1;

	private Font value = null;

	private SimpleButton extendButton = null;

	private String[] fonts;

	private int mode = -1;

	/**
	 * Creates a new FontEditor object.
	 */
	public FontEditor() {
		super();
		fonts = GraphicsEnvironment.getLocalGraphicsEnvironment()
				.getAvailableFontFamilyNames();
		setModel(new DefaultComboBoxModel(fonts));
		setRenderer(new FontRenderer());
		setEditable(false);
		initialize();
		setPropertyValue(FontHelper.getDefaultFont().getFamily());
	}

	/**
	 * @see com.cosylab.gui.property.editors.PropertyEditor#setDescription(java.lang.String)
	 */
	public void setDescription(String description) {
	}

	/**
	 * @see com.cosylab.gui.property.editors.PropertyEditor#getDescription()
	 */
	public String getDescription() {
		return null;
	}

	/**
	 * Sets the ComboBox mode. Overriden to implement FontEditorUI UI.
	 * 
	 * @param mode
	 *            to be set.
	 */
	public void setMode(int mode) {
		if (mode != this.mode) {
			int old = this.mode;
			this.mode = mode;

			if (mode == MODE_BOTH) {
				setUI(new FontEditorUI());
			} else if (mode == MODE_COMBO) {
				setStyle(Style.DROP_DOWN_STYLE);
			}

			firePropertyChange("mode", old, mode);
		}
	}

	/**
	 * @see com.cosylab.gui.property.editors.PropertyEditor#setPropertyValue(java.lang.Object)
	 */
	public boolean setPropertyValue(Object value) {
		if (value instanceof Font) {
			Font daNew = (Font) value;
			Font old = this.value;
			this.value = daNew;
			setFont(daNew);
			setSelectedItem(((Font) value).getFamily());
			extendButton.repaint();
			firePropertyChange(PROPERTY_VALUE_NAME, old, daNew);

			return true;
		}

		return false;
	}

	/**
	 * Returns the selected font.
	 * 
	 * @return the selected font.
	 */
	public Object getPropertyValue() {
		return value;
	}

	private void initialize() {
		setBorder(CosyUIElements.getPlainBorder(false));
		addActionListener(new ActionListener() {
			/**
			 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
			 */
			public void actionPerformed(ActionEvent e) {
				setPropertyValue(Font.decode((String) getSelectedItem()));
			}
		});
		initializeComboListeners();
		extendButton = new SimpleButton("...");
		extendButton.setBorder(CosyUIElements.getPlainBorder(true));
		extendButton.setPressedBorder(null);
		extendButton.setActionMode(SimpleButton.FAST_ACTION_MODE);
		extendButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// TODO implement fontChooser dialog
				// Font c = FontChooser.showDialog(FontEditor.this, "Choose the
				// desired font", value);
				// setPropertyValue(c);
			}
		});
		setMode(MODE_BOTH);
	}

	private void initializeComboListeners() {
	}
}

/* __oOo__ */
