/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of CosyBeans-Common.
 *
 * CosyBeans-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CosyBeans-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CosyBeans-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.gui.property.editors;

import java.awt.Dimension;

import javax.swing.JTextField;

import com.cosylab.gui.components.numberfield.AbstractNumberDocument;
import com.cosylab.gui.components.numberfield.DoubleDocument;

/**
 * description
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a> 
 * @version $id$
 */
public class NumberEditor extends JTextField implements PropertyEditor {
    /**
     * 
     */
    private static final long serialVersionUID = 594794111625657826L;
    private boolean allowNull=false;
    /**
     * 
     */
	public NumberEditor() {
		this(new DoubleDocument());
		setValue(new Double(0));
	}

	public NumberEditor(AbstractNumberDocument doc) {
		super();
		setDocument(doc);
		setPreferredSize(new Dimension(120, 20));
	}
	
	/**
	 * This implementation return null, to indicate the editor is unable to display 
	 * the description.
	 * 
	 * @see PropertyEditor#getDescription()
	 */
	public String getDescription() {
		return null;
	}

	/**
	 * @see PropertyEditor#getEditor()
	 */
	public PropertyEditor getEditor() {
		return this;
	}

	/**
	 * @see PropertyEditor#getValue()
	 */
	public Object getPropertyValue() {
		return getValue();
	}

	/**
	 * @see PropertyEditor#setValue(Object)
	 */
	public boolean setPropertyValue(Object value) {
		if (value instanceof Number) {
			setValue((Number)value);
			return true;
		} else if (value==null && allowNull) {
            setValue(null);
		    return true;
        }
		return false;
	}

	/**
	 * @see PropertyEditor#setDescription(String)
	 */
	public void setDescription(String description) {
		// we can't handle this
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.NumberField#setValue(java.lang.Number)
	 */
	public void setValue(Number newValue) {
		Number oldValue=getValue();
		setText(newValue==null?"":String.valueOf(newValue));
		revalidate();
		if (newValue!=oldValue && (newValue==null || !newValue.equals(oldValue))) {
			firePropertyChange(PROPERTY_VALUE_NAME,oldValue,newValue);
		}
	}
    
    public void setMaxValue(Number maxValue) {
        ((AbstractNumberDocument)getDocument()).setMaxValue(maxValue);
    }
    
    public void setMinValue(Number minValue) {
        ((AbstractNumberDocument)getDocument()).setMinValue(minValue);
    }

	/**
	 * @return
	 */
	public Number getValue() {
		Number ret=((AbstractNumberDocument)getDocument()).getValue();
		if (ret==null && !allowNull) {
			ret=new Double(0);
		}
		return ret;
	}
    public boolean isNullAllowed() {
        return allowNull;
    }
    public void setNullAllowed(boolean allow) {
        this.allowNull=allow;
    }

}
