/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.logging;

import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;


/**
 * <code>DebugLogger</code> is wrapper around <code>Logger</code> class, which
 * produces logger with preset arbitrary logging level and handler
 * <code>ConsoleLogHandler</code>.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: DebugLogger.java,v 1.3 2008-04-22 12:26:29 jbobnar Exp $
 *
 * @since Jun 27, 2004.
 */
public class DebugLogger extends Logger
{
	//private static boolean init = false;

	/**
	 * Returns logger with specified name and initial logging level. If level
	 * is OFF , then instance of <code>NullLogger</code> is return, which
	 * statically sends all logs to dev/null.
	 *
	 * @param name a name for the logger
	 * @param level initial level of logger
	 *
	 * @return a logger, <code>NullLogger</code> if level is OFF
	 *
	 * @see Logger#getLogger(java.lang.String)
	 */
	public synchronized static Logger getLogger(String name, Level level)
	{
		// TODO workaround becouse maven junit tester fails to load handler

		/*if (!init) {
		    // to make sure class loaded, because of strange problems with JUnit class loader
		    //ConsoleLogHandler clh = new ConsoleLogHandler();
		    //clh.close();
		    ConsoleLogHandler.initializeConsoleLogging();
		    init = true;
		}*/
		if (Level.OFF.equals(level)) {
			return NullLogger.getNullLogger();
		}

		// TODO workaround becouse maven junit tester fails to load handler
		Logger l = LogManager.getLogManager().getLogger(name);

		if (l == null) {
			l = new DebugLogger(name, null);
			LogManager.getLogManager().addLogger(l);
			l.setUseParentHandlers(false);
			l.setLevel(level);
			l.addHandler(new ConsoleLogHandler());
		}

		return l;
	}

	protected DebugLogger(String name, String resourceBundleName)
	{
		super(name, resourceBundleName);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param args DOCUMENT ME!
	 */
	public static void main(String[] args)
	{
		try {
			Logger l = getLogger("dummy", Level.ALL);

			l.info("Test");

			l.finest("Finest Test");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}

/* __oOo__ */
