/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

/**
 * This is a convenience implementation of <code>SchedulerTask</code>
 * interface.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public abstract class AbstractSchedulerTask implements SchedulerTask
{
	/** Flag indicating if this task is scheduled to be canceled. */
	private boolean canceled;

	/** Interval between executions. */
	private long interval;

	/** Name of this task, can be null. */
	private String name;

	/**
	 * Creates a new AbstractSchedulerTask object.
	 *
	 * @param taskName DOCUMENT ME!
	 * @param taskInterval DOCUMENT ME!
	 */
	public AbstractSchedulerTask(String taskName, long taskInterval)
	{
		name = taskName;
		interval = taskInterval;
	}

	/**
	 * Mark this task to be canceled.
	 *
	 * @see com.cosylab.util.SchedulerTask#cancel()
	 */
	public void cancel()
	{
		canceled = true;
	}

	/**
	 * Returns whether <code>cancel()</code> has been called.
	 *
	 * @see com.cosylab.util.SchedulerTask#isCanceled()
	 */
	public boolean isCanceled()
	{
		return canceled;
	}

	/**
	 * Set new name for this task.
	 *
	 * @param taskName Name of task.
	 */
	public void setTaskName(String taskName)
	{
		name = taskName;
	}

	/**
	 * @see com.cosylab.util.SchedulerTask#getTaskName()
	 */
	public String getTaskName()
	{
		return name;
	}

	/**
	 * Returns interval between executions. If negative, indicates this task
	 * should execute immediatelly.
	 *
	 * @see com.cosylab.util.SchedulerTask#getInterval()
	 */
	public long getInterval()
	{
		return interval;
	}
}

/* __oOo__ */
