/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.util.BitSet;


/**
 * <code>BitSetUtilities</code> containes usefull clases to manipulate with
 * <code>BitSet</code>.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: BitSetUtilities.java,v 1.4 2008-04-22 12:26:29 jbobnar Exp $
 *
 * @since Dec 22, 2003.
 */
public final class BitSetUtilities
{
	/**
	 * Hidden.
	 */
	private BitSetUtilities()
	{
		super();
	}

	/**
	 * Converts <code>long</code> value to <code>BitSet</code>.
	 *
	 * @param value the long value
	 *
	 * @return the <code>BitSet</code> corresponding to the value
	 */
	public static BitSet forLong(long value)
	{
		BitSet bs = new BitSet();

		int i = 0;
		
		if (value < 0) {
			value = value & 0x7FFFFFFFFFFFFFFFL;
		}
		
		while (value > 0) {
			bs.set(i++, (value & 1) > 0);
			value = value >> 1;
		}

		return bs;
	}

	/**
	 * Converts <code>BitSet</code> to <code>long</code> value if possible.
	 *
	 * @param value the <code>BitSet</code> object
	 *
	 * @return long representatnion of the bit set
	 *
	 * @throws NullPointerException DOCUMENT ME!
	 */
	public static long toLong(BitSet value)
	{
		if (value == null) {
			throw new NullPointerException("value");
		}

		long longValue = 0;

		// see BitSet.length() javadoc to understand "bitValue.length()-1" 
		for (int i = Math.min(value.length() - 1, 63); i >= 0; i--) {
			longValue <<= 1;

			if (value.get(i)) {
				longValue++;
			}
		}

		return longValue;
	}

	/**
	 * Returns bit pattern by ignoring all zeros in the mask. If mask is less
	 * than 0, return value is equal to bits parameter.
	 * 
	 * <p>
	 * Example: <br>
	 * <code> mask = XX10101 bits = XX10010 getCompactValue() = XXXXX100
	 * </code>
	 * </p>
	 *
	 * @param bits bit pattern as long value
	 * @param mask mask which tells, which bits in pattern are visible, if less
	 *        than 0, maks is ignored
	 *
	 * @return long maksed representation of the bit pattern.
	 */
	public static long getMaskedValue(long bits, long mask)
	{
		if (mask < 0) {
			return bits;
		}

		long vi = 0;
		long newBits = 0;

		while (mask > 0) {
			if ((mask & 1) == 1) {
				newBits += ((bits & 1) << vi++);
			}

			mask >>= 1;
			bits >>= 1;
		}

		return newBits;
	}
}

/* __oOo__ */
