/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.util.Enumeration;


/**
 * This class defines enumeration with no elements. Its intended use is as
 * return value when method returns enumeration that has no elements. This is
 * a singleton implementation and instance of this class is obtained using
 * <code>EmptyEnumeration.getInstance()</code> method.
 * 
 * <p>
 * This class should serve as a convenience implementation of an interface.
 * </p>
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public final class EmptyEnumeration implements Enumeration
{
	private static final Enumeration INSTANCE = new EmptyEnumeration();

	/**
	 * Private singleton constructor.
	 */
	private EmptyEnumeration()
	{
		super();
	}

	/**
	 * Returns singleton instance of this class.
	 *
	 * @return Instance of Enumeration.
	 */
	public static final Enumeration getInstance()
	{
		return INSTANCE;
	}

	/**
	 * Returns false.
	 *
	 * @see java.util.Enumeration#hasMoreElements()
	 */
	public boolean hasMoreElements()
	{
		return false;
	}

	/**
	 * Returns null.
	 *
	 * @see java.util.Enumeration#nextElement()
	 */
	public Object nextElement()
	{
		return null;
	}
}

/* __oOo__ */
