/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;

/**
 * A support class that acts as a <code>java.beans.PropertyChangeSupport</code> with the ability
 * to be <b>extended</b> by classes that need PropertyChange events and an internal 
 * map-like store for property values which automatically fires <code>PropertyChangeEvent</code>s
 * 
 * @author Miha Kadunc (<a href="mailto:miha.kadunc@cosylab.com">miha.kadunc@cosylab.com</a>
 */
public class PropertyChangeHelper {
	private PropertyChangeSupport pcSupport;
	private HashMap clientProperties;
	
	public PropertyChangeHelper() {
		super();
		pcSupport=new PropertyChangeSupport(this);
	}
	
	public PropertyChangeHelper(Object source) {
		super();
		pcSupport=new PropertyChangeSupport(source);
	}
	
	public void setClientProperty(String propertyName, Object propertyValue) {
		if (clientProperties==null) {
			clientProperties=new HashMap();
		}
		Object old=clientProperties.get(propertyName);
		clientProperties.put(propertyName,propertyValue);
		firePropertyChange(propertyName,old,propertyValue);
	}
	
	public Object getClientProperty(String propertyName) {
		if (clientProperties==null) {
			return null;
		} else {
			return clientProperties.get(propertyName);
		}
	}
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		pcSupport.addPropertyChangeListener(listener);
	}
	public void addPropertyChangeListener(String propertyName,
											PropertyChangeListener listener) {
		pcSupport.addPropertyChangeListener(propertyName, listener);
	}
	public void firePropertyChange(PropertyChangeEvent evt) {
		pcSupport.firePropertyChange(evt);
	}
	public void firePropertyChange(String propertyName, boolean oldValue,
									boolean newValue) {
		pcSupport.firePropertyChange(propertyName, oldValue, newValue);
	}
	public void firePropertyChange(String propertyName, int oldValue,
									int newValue) {
		pcSupport.firePropertyChange(propertyName, oldValue, newValue);
	}
	public void firePropertyChange(String propertyName, Object oldValue,
									Object newValue) {
		pcSupport.firePropertyChange(propertyName, oldValue, newValue);
	}
	public PropertyChangeListener[] getPropertyChangeListeners() {
		return pcSupport.getPropertyChangeListeners();
	}
	public PropertyChangeListener[] getPropertyChangeListeners(
																String propertyName) {
		return pcSupport.getPropertyChangeListeners(propertyName);
	}
	@Override
    public int hashCode() {
		return pcSupport.hashCode();
	}
	public boolean hasListeners(String propertyName) {
		return pcSupport.hasListeners(propertyName);
	}
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		pcSupport.removePropertyChangeListener(listener);
	}
	public void removePropertyChangeListener(String propertyName,
												PropertyChangeListener listener) {
		pcSupport.removePropertyChangeListener(propertyName, listener);
	}
	@Override
    public String toString() {
		return pcSupport.toString();
	}
}
