/*
 * Copyright (c) 2003-2008 by Cosylab d. d.
 *
 * This file is part of Java-Common.
 *
 * Java-Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java-Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Java-Common.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.cosylab.util;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;


/**
 * A default implementation of thread pool which delegates to  Java concurrent
 * library. It creates worker threads with <code>Thread.NORM_PRIORITY -
 * 2</code>. A maximum number of workers can be active concurrently,
 * specifiable by the <code>maximumPoolSize</code> property. If
 * <code>execute()</code> is called and no worker is free, the call will block
 * until  one of the workers terminate. Use this service to execute short
 * tasks. In this way excessive construction of new thread instances can be
 * prevented, speeding up the  application.
 *
 * @author Gasper Tkacik
 * @version $id$
 *
 *	@deprecated This implementation has been replaced by java.util.concurent package 
 */
@Deprecated
public class ThreadPool extends ThreadPoolExecutor
{
	/**
	 * Creates threads belonging to the "ThreadPoolWorker" group, with name
	 * "DefaultThreadPoolWorker" and priority of normal less by 2.
	 */
	static private class ThreadFactoryImpl implements ThreadFactory
	{
		private int priority = Thread.NORM_PRIORITY - 2;
		private ThreadGroup group = null;

		/**
		 * Creates a new ThreadFactoryImpl object.
		 *
		 * @param priority priority for new worker threads
		 */
		public ThreadFactoryImpl(int priority)
		{
			this.priority = priority;
			group= new ThreadGroup("DefaultThreadPoolWorkerGroup");
		}

		/**
		 * Creates a new ThreadFactoryImpl object.
		 */
		public ThreadFactoryImpl()
		{
			group= new ThreadGroup("DefaultThreadPoolWorkerGroup");
		}

		/**
		 * Creates a new thread instance.
		 *
		 * @param command the command that should be executed by the thread
		 *
		 * @return new worker thread
		 */
		public Thread newThread(Runnable command)
		{
			Thread retVal = new Thread(group, command, "DefaultThreadPoolWorker");
			retVal.setPriority(priority);

			return retVal;
		}
	}

	/**
	 * Constructs a executor with 25 active threads.
	 */
	public ThreadPool()
	{
		super(25,25,1000,TimeUnit.MILLISECONDS,new LinkedBlockingQueue<Runnable>(),new ThreadFactoryImpl());
	}

	/**
	 * Creates a new ThreadPool object.
	 *
	 * @param priority the priority of worker threads.
	 */
	public ThreadPool(int priority)
	{
		super(25,25,1000,TimeUnit.MILLISECONDS,new LinkedBlockingQueue<Runnable>(),new ThreadFactoryImpl(priority));
	}

	/**
	 * Returns the thread group to which all worker threads of this service
	 * belong. This will return non-<code>null</code> after this component has
	 * been  initialized.
	 *
	 * @return the group of all worker threads
	 */
	public ThreadGroup getGroup()
	{
		return ((ThreadFactoryImpl)getThreadFactory()).group;
	}

	/**
	 * Releases the thread pool. First attempts to wait until the tasks have
	 * executed. New tasks are not accepted. Then, if after the period
	 * <code>WAIT_FOR_SHUTDOWN</code> the threads do not complete, the pool is
	 * forcibly terminated.
	 */
	public void destroy()
	{
		shutdown();
	}

	/**
	 * Returns a short summary of this instance.
	 *
	 * @return internal state of this
	 */
	@Override
    public String toString()
	{
		StringBuffer sbuff = new StringBuffer(200);
		sbuff.append("ThreadPool = { currentSize=");
		sbuff.append(getPoolSize());
		sbuff.append(" maxSize=");
		sbuff.append(getMaximumPoolSize());
		sbuff.append(" coreSize=");
		sbuff.append(getCorePoolSize());
		sbuff.append(" } ");

		return new String(sbuff);
	}
}

/* __oOo__ */
