package de.desy.acop.about;

import java.io.InputStream;
import java.util.Map;
import java.util.Properties;

/**
 * 
 * <code>ApplicationInfo</code> provides a set of information about the application
 * and its keeper.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class ApplicationInfo {
	
	public static final String CONTACT_NAME = "contact.name";
	public static final String CONTACT_MAIL = "contact.mail";
	public static final String CONTACT_PHONE = "contact.phone";
	public static final String RELEASE_DATE = "app.release.date";
	public static final String RELEASE_VERSION = "app.release.version";
	public static final String APPLICATION = "app.name";
	
	private String version;
	private String date;
	private String responsible;
	private String email;
	private String phone;
	private String application;
			
	/**
	 * Constructs the application info. User should provide the properties,
	 * which initializes the fields of this info.
	 * 
	 * @param properties the properties containing key-value pairs
	 * @param propertiesResources the resources from which to additionally load properties
	 */
	public ApplicationInfo(Map<String, String> properties, String... propertiesResources) {
		initialize(properties, propertiesResources);
	}
	
	private void initialize(Map<String, String> properties, String... propertiesResources) {
		Properties pr = System.getProperties();
		if (propertiesResources != null) {
    		for (int i = 0; i < propertiesResources.length; i++) {
        		try (InputStream is = getClass().getClassLoader().getResourceAsStream(propertiesResources[i])) {
        			pr.load(is);
        		} catch (Exception e) {
        			//ignore the file is missing
        		} 
    		}
		}
		
		responsible = pr.getProperty(CONTACT_NAME,properties.get(CONTACT_NAME));
		email = pr.getProperty(CONTACT_MAIL,properties.get(CONTACT_MAIL));
		phone = pr.getProperty(CONTACT_PHONE,properties.get(CONTACT_PHONE));
		application = pr.getProperty(APPLICATION,properties.get(APPLICATION));
		date = pr.getProperty(RELEASE_DATE,properties.get(RELEASE_DATE));
		version = pr.getProperty(RELEASE_VERSION,properties.get(RELEASE_VERSION));
		
	}

	/**
	 * Returns the application version.
	 * 
	 * @return the version
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * Returns the release date of the application.
	 * 
	 * @return the release date
	 */
	public String getDate() {
		return date;
	}

	/**
	 * Returns the name of the application keeper.
	 * 
	 * @return the application keeper
	 */
	public String getResponsible() {
		return responsible;
	}

	/**
	 * Returns the email of the application keeper.
	 * 
	 * @return the emailof the keeper
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * Returns the phone number of the application keeper.
	 * 
	 * @return the phone number
	 */
	public String getPhone() {
		return phone;
	}
	
	/**
	 * Returns the name of the application.
	 * 
	 * @return the name
	 */
	public String getApplication() {
		return application;
	}
}
