/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart;
import java.awt.Point;
import java.awt.event.MouseEvent;

public class AcopEvent extends java.util.EventObject
{
  private static final long serialVersionUID = 8604284866638112625L;
  protected int event_type;             // event type
  protected int link_id, link_error;    // receive event
  protected int histo_index;            // histo index
  protected Point mouse_point;          // mouse position for mouse event
  protected double mouseUserX, mouseUserY; // converted user mouse position for mouse event
  protected double zoomX1, zoomY1, zoomX2, zoomY2;        // converted zoom rectangle
  protected double secondaryYMin, secondaryYMax;
  private boolean consumed = false;

  // total AcopEventType
  public final static int ACOP_MOUSE_MOVE_EVENT = 1;
  public final static int ACOP_MOUSE_CLICK_EVENT = 2;
  public final static int ACOP_MOUSE_ZOOM_EVENT = 3;
  public final static int ACOP_MOUSE_ENTER_EVENT = 4;
  public final static int ACOP_MOUSE_EXIT_EVENT = 5;
  public final static int ACOP_MOUSE_DCLICK_EVENT = 6;
  public final static int ACOP_MOUSE_PRESSED_EVENT = 7;
  public final static int ACOP_MOUSE_DRAG_EVENT = 8;
  //public final static int ACOP_RECEIVE_EVENT = 10;
  private final MouseEvent trigger;
  
  // the constructor
  public AcopEvent(Object source, MouseEvent trigger, int eventId, int linkId, int error)
  {
    // pass the source object to the superclass
    super(source);
    
    // save data
    event_type = eventId;
    link_id = linkId;
    link_error = error;
    this.trigger = trigger;
  }

  // the constructor
  public AcopEvent(Object source, MouseEvent trigger, int eventId, int histIndex, int x, int y, double userx, double usery)
  {
    // pass the source object to the superclass
    super(source);
    
    event_type = eventId;

    // save data
    histo_index = histIndex;
    mouse_point = new Point(x,y);
    mouseUserX = userx;
    mouseUserY = usery;
    this.trigger = trigger;
  }

  // the constructor
  public AcopEvent(Object source, MouseEvent trigger, 
		  int eventId, double xMin, double yMin, double xMax, double yMax, double secondaryYMin, double secondaryYMax)
  {
    // pass the source object to the superclass
    super(source);
    
    event_type = eventId;

    // save data
    zoomX1 = xMin;
    zoomY1 = yMin;
    zoomX2 = xMax;
    zoomY2 = yMax;
    this.secondaryYMax = secondaryYMax;
    this.secondaryYMin = secondaryYMin;
    this.trigger = trigger;
  }

  public int getEventType()
  {
    return event_type;
  }
  
  public int getLinkId()
  {
    return link_id;
  }
  
  public int getLinkError()
  {
    return link_error;
  }
  
  public int getHistoIndex()
  {
    return histo_index;
  }
  
  public Point getMousePoint()
  {
    return mouse_point;
  }
  
  public double getMousePositionX()
  {
    return mouseUserX;
  }
  
  public double getMousePositionY()
  {
    return mouseUserY;
  }
  
  public double getZoomXMin()
  {
    return zoomX1;
  }  
  
  public double getZoomYMin()
  {
    return zoomY1;
  }  
  
  public double getZoomXMax()
  {
    return zoomX2;
  }  
  
  public double getZoomYMax()
  {
    return zoomY2;
  }  
  
  public double getSecondaryYMax() 
  {
	return secondaryYMax;
  }
  
  public double getSecondaryYMin() 
  {
	return secondaryYMin;
  }
  
  public MouseEvent getTrigger() {
    return trigger;
  } 
    
  public boolean isConsumed() {
    return consumed;
  }
    
  public void consume() {
    consumed = true;
  }
}
