package de.desy.acop.chart;

public class AcopFFT
{
  private int data_size, half_size, ary_size, reverseFFT;
	private float q_wert;
	private float[]	q_array, f_array, adr0_r, adr0_i, adr1_r, adr1_i;

  public AcopFFT(){};

  public int FFT(int n_points, double[] x_array, double[] outf_array, double[] maxPerCut,  int specktrum)
  { int i,j, oddFlag;
	  int[] error = new int[1];
       
    if ( n_points < 2 || n_points > AcopConst.max_points ) return -1;
      
    if ( q_array == null || n_points > ary_size )
    {
        q_array = new float[n_points];
        f_array = new float[n_points];
        adr0_r = new float[n_points*2];
        adr0_i = new float[n_points*2];
        adr1_r = new float[n_points*2];
        adr1_i = new float[n_points*2];
        ary_size = n_points;
    }
	  else for (i=0; i<n_points; i++)
    {
        adr0_r[i] = 0;
        adr0_i[i] = 0;
        adr1_r[i] = 0;
        adr1_i[i] = 0;
    }
    for (j=0; j<n_points; j++) q_array[j] = (float)x_array[j];   /* input data points    */
    half_size = n_points / 2;
    oddFlag = n_points % 2;
      
	  // Fast FT if possible
    data_size = n_points ;
    reverseFFT = specktrum == AcopConst.FFT_RI_INVERSE ? 1 : 0;
    c_fourier ( data_size, 0) ;     // FFT first call = 0 (have to) */


    // 0x1: spektrum + 0; 0x2: phase + 0; 0x3: spek + phase; else: real + imaginary part
    if ( specktrum < AcopConst.FFT_SP || specktrum==AcopConst.FFT_REAL || specktrum==AcopConst.FFT_IMAGINARY )
      for (i=half_size; i<n_points; i++)
      {
        f_array[i] = 0;
        outf_array[i] = 0;
      }
      if ( oddFlag != 0 ) // last point
      {
        f_array[n_points-1]=0;
        f_array[n_points-1]=0;
      }

    for (j=0; j<=half_size; j++)	// first half => FFT Power spectrum
	  {								// second half => FFT phase (-pi -> pi)
	    if ( (specktrum == AcopConst.FFT_SPEKTRUM) || (specktrum == AcopConst.FFT_SP) )
	    {
	      f_array[j] = (float)Math.sqrt(adr0_r[j]*adr0_r[j]+adr0_i[j]*adr0_i[j]);
		    if ( specktrum == AcopConst.FFT_SP && j>0 && j+half_size<n_points ) f_array[j+half_size] = (float)Math.atan2(adr0_i[j],adr0_r[j]);
	    }
	    else if ( specktrum == AcopConst.FFT_PHASE ) f_array[j] = (float)Math.atan2(adr0_i[j],adr0_r[j]);
      else if ( specktrum == AcopConst.FFT_RI )
      {
        f_array[j] = adr0_r[j];
        if ( j>0 && j+half_size<n_points ) f_array[j+half_size] = adr0_i[j];
      }
      else if ( specktrum == AcopConst.FFT_REAL ) f_array[j] = adr0_r[j];
      else if ( specktrum == AcopConst.FFT_IMAGINARY ) f_array[j] = adr0_i[j];
      else
      {
        f_array[j] = adr0_r[j];
        if ( j>0 && j+half_size<n_points ) f_array[j+half_size] = adr0_r[j+half_size];
      }
	  }
    for (j=0; j<n_points; j++)
    {
      if ( (specktrum < AcopConst.FFT_SP || specktrum==AcopConst.FFT_REAL || specktrum==AcopConst.FFT_IMAGINARY) && j > half_size )break;
      outf_array[j] = (double)f_array[j];
    } 
       
	  maxPerCut[0] = max_find ( half_size, error, maxPerCut[1], maxPerCut[2]);   // find Q value
	  
	  return 0;
  }
    
  private double max_find(int size, int[] error, double percent, double cut)
  { int j, i_max ;
	  double r_max, r_cut, i_weight ;

	  error[0] = 0 ;
    r_max =  f_array[0] ;       /* find maximum position i_max  */
    i_max = 0;
	  for (j=0; j<=size; j++)  if ( f_array[j] > r_max)
	  {
	  	r_max =  f_array[j] ;
		  i_max = j ;
	  }
	  r_cut = r_max * percent ;       /* above this wert, take into account   */
	  r_max = 0.0;                            // r_max = -1000.0 ;
	  i_weight = 0.0 ;
	  for (j=0; j<=size; j++)         /* make weight to all high points       */
	    if ( f_array[j] > r_cut) 
	  {
  		r_max +=  f_array[j] ;
	  	i_weight +=  f_array[j] * j ;
	  }
	  if ( r_max < AcopConst.zero_check)
	  {
	    i_weight = i_max;
	    error[0] |= 0x1;
    }
    else i_weight /= r_max ;
	
	  if ( Math.abs(i_weight - i_max) > cut ) error[0] |= 0x2 ;
	  
	  return (i_weight) ;                                             /* return Q*freqency    */
  }

  private void c_fourier(int now_data, int n_call)
  { int i, j, flag;
	  int off_set ;                      /* memory off set       */
	  int h_now = now_data/2 ;
	  int r_base_adr_index, r_base_adr_flag;
	  float f_sin, f_cos, temp_r, temp_i, tmp ;

    // if even number, do seperation 
    if ( now_data % 2 == 0 && now_data > 2 )       /* seperate even and odd points */
	  {
	  	c_fourier (h_now, n_call) ;                     /* call even    */
		  c_fourier (h_now, n_call + data_size/now_data) ;    /* odd  */
      off_set = now_data;     /* destination offset position */
      r_base_adr_flag = 0;
      r_base_adr_index = 0;
	  	if ( data_size > now_data)         /* if not first call    */
		  {
		    if (n_call >=  half_size / now_data ) r_base_adr_flag = 1; /* choose memory odd*/
        r_base_adr_index += off_set;
		  }
		  off_set = now_data / 2 ;           /* lower step offset, get source*/

		  f_cos = (float)1.0 ;
		  f_sin = (float)0.0 ;
	  	for (i=0; i<h_now; i++)               /* calculate f1k, f2k   */
	  	{
		    if (i != 0 )
		    {
			    f_sin = (float) Math.PI * i / h_now ;                        /* shift angle  */
          if ( reverseFFT != 0 ) f_sin = -f_sin;
			    f_cos = (float) Math.cos ( (double) f_sin) ;
			    f_sin = (float) Math.sin ( (double) f_sin) ;
		    }
		    temp_r =  adr1_r [i+ off_set] * f_cos + adr1_i [i+ off_set] * f_sin ;
		    temp_i = -adr1_r [i+ off_set] * f_sin + adr1_i [i+ off_set] * f_cos ;

        if ( r_base_adr_flag == 0 )
        {
          adr0_r [i+r_base_adr_index] = (adr0_r [i+off_set] + temp_r) / 2 ;  /* first half f(k)       */
          adr0_i [i+r_base_adr_index] = (adr0_i [i+ off_set] + temp_i) /2 ;
          if ( reverseFFT != 0 )
          {
            adr0_r [i+r_base_adr_index] *= 2;
            adr0_i [i+r_base_adr_index] *= 2;
          }
        }
        else
        {
          adr1_r [i+r_base_adr_index] = (adr0_r [i+off_set] + temp_r) /2 ;  /* first half f(k)       */
          adr1_i [i+r_base_adr_index] = (adr0_i [i+ off_set] + temp_i) /2 ;
          if ( reverseFFT != 0 )
          {
            adr1_r [i+r_base_adr_index] *= 2;
            adr1_i [i+r_base_adr_index] *= 2;
          }
        }
        if ( data_size != now_data || reverseFFT != 0 )  // not first call, do f(n+k)
        {
          if ( r_base_adr_flag == 0 )
          {
            adr0_r [i+r_base_adr_index + h_now ] =  (adr0_r [i+off_set] - temp_r) / 2 ; // f(n+k)
            if ( reverseFFT != 0 ) adr0_r [i+r_base_adr_index + h_now ] *= 2;
          }
          else
          {
            adr1_r [i+r_base_adr_index + h_now ] =  (adr0_r [i+off_set] - temp_r) / 2 ; // f(n+k)
            if ( reverseFFT != 0 ) adr1_r [i+r_base_adr_index + h_now ] *= 2;
          }

          if ( data_size != now_data) // no imaginay for first call
          {
            if ( r_base_adr_flag == 0 )
            {
              adr0_i [i+r_base_adr_index + h_now ] =  (adr0_i [i+off_set] - temp_i) / 2 ;
              if ( reverseFFT != 0 ) adr0_i [i+r_base_adr_index + h_now ] *= 2;
            }
            else
            {
              adr1_i [i+r_base_adr_index + h_now ] =  (adr0_i [i+off_set] - temp_i) / 2 ;
              if ( reverseFFT != 0 ) adr1_i [i+r_base_adr_index + h_now ] *= 2;
            }
          }
        }
		  }

      if ( data_size == now_data)    /*  if first call, do middle point   */
      {
        adr0_r[half_size] = 0;   // init to 0
        adr0_i[half_size] = 0;

        for (i=0; i<data_size; i++)
        {
          if ( reverseFFT != 0 && i>h_now) j=data_size -i;
          else j=i;
          if ( i%2 == 0 ) adr0_r[half_size] += q_array[j];
          else adr0_r[half_size] -= q_array[j];
        }
        if ( reverseFFT == 0 ) adr0_r[half_size] /= data_size;
      }

	  }
	  else
    {                        /* only last two points, or odd points */
      if ( n_call * now_data * 2 < data_size ) r_base_adr_flag = 0;
      else r_base_adr_flag = 1;
      r_base_adr_index = 0;
       if ( data_size != now_data ) r_base_adr_index = now_data; // add offset

       if ( data_size != now_data && now_data == 2 ) // after seperation to 2 points
       {
         if ( reverseFFT == 0 )
         {
           if ( r_base_adr_flag == 0 )
           {
             adr0_r[r_base_adr_index] = (float) ( q_array[n_call]+ q_array[n_call + half_size]) / 2;
             adr0_r[r_base_adr_index + 1] = (float) ( q_array[n_call]- q_array[n_call + half_size]) / 2;
           }
           else
           {
             adr1_r[r_base_adr_index] = (float) ( q_array[n_call]+ q_array[n_call + half_size]) / 2;
             adr1_r[r_base_adr_index + 1] = (float) ( q_array[n_call]- q_array[n_call + half_size]) / 2;
           }
         }
         else // reverseFFT, get imaginary part
         {
           i = n_call+ half_size;
           if ( i>half_size )  // second half
           {
             i = data_size - i ;
             flag=1;
           }
           else flag=0;

           if ( r_base_adr_flag == 0 )
           {
             adr0_r[r_base_adr_index] = (float) ( q_array[n_call]+ q_array[i]);
             adr0_r[r_base_adr_index + 1] = (float) ( q_array[n_call]- q_array[i]);
           }
           else
           {
             adr1_r[r_base_adr_index] = (float) ( q_array[n_call]+ q_array[i]);
             adr1_r[r_base_adr_index + 1] = (float) ( q_array[n_call]- q_array[i]);
           }
           
           if ( i + half_size < data_size )
           {
             if(flag==0)
             {
               if ( r_base_adr_flag == 0 )
               {
                 adr0_i[r_base_adr_index] = (float) q_array[i+ half_size] ;
                 adr0_i[r_base_adr_index + 1] = (float) -q_array[i+ half_size] ;
               }
               else
               {
                 adr1_i[r_base_adr_index] = (float) q_array[i+ half_size] ;
                 adr1_i[r_base_adr_index + 1] = (float) -q_array[i+ half_size] ;
               }
             }
             else
             {
               if ( r_base_adr_flag == 0 )
               {
                 adr0_i[r_base_adr_index] = (float) -q_array[i+ half_size] ;
                 adr0_i[r_base_adr_index + 1] = (float) q_array[i+ half_size] ;
               }
               else
               {
                 adr1_i[r_base_adr_index] = (float) -q_array[i+ half_size] ;
                 adr1_i[r_base_adr_index + 1] = (float) q_array[i+ half_size] ;
               }
             }
           }

           if ( n_call != 0 ) // first entry no imaginary
           {
             if ( r_base_adr_flag == 0 )
             {
               adr0_i[r_base_adr_index] += q_array[n_call+half_size] ;
               adr0_i[r_base_adr_index+1] += q_array[n_call+half_size] ;
             }
             else
             {
               adr1_i[r_base_adr_index] += q_array[n_call+half_size] ;
               adr1_i[r_base_adr_index+1] += q_array[n_call+half_size] ;
             }
           }
         }
       }
       else
       {
         if ( data_size == now_data )
         {
           if ( reverseFFT != 0 ) h_now = data_size;
           else h_now += 1 ;  // complete normal FT (odd number)
         }
         else h_now = now_data;       // odd data

         f_cos = (float)1.0 ; //init
         f_sin = (float)0.0 ;
         for (i=0; i<h_now; i++)  // odd number data, half of whole data
         {
           if ( r_base_adr_flag == 0 )
           {
             adr0_r[r_base_adr_index+i] = 0;   // init
             adr0_i[r_base_adr_index+i] = 0;
           }
           else
           {
             adr1_r[r_base_adr_index+i] = 0;   // init
             adr1_i[r_base_adr_index+i] = 0;
           }
           for (j=0; j<now_data; j++) //FT
           {
             if (j==0)  //init
             {
               f_cos = (float)1.0 ;
               f_sin = (float)0.0 ;
             }
             else if ( i != 0 && j != 0 ) //get sin and cos
             {
               f_sin = (float) (-2 * Math.PI * i * j / now_data) ; /* shift angle  */
               if ( reverseFFT != 0 ) f_sin = -f_sin;
               f_cos = (float) Math.cos ( (double) f_sin) ;
               f_sin = (float) Math.sin ( (double) f_sin) ;
             }
             off_set = n_call + data_size / now_data * j;  // offset
             flag=0;
             if ( reverseFFT != 0 && off_set > half_size )
             {
               off_set = data_size - off_set ;
               flag=1;
             }
             if (i==0 || j==0)
             {
               if ( r_base_adr_flag == 0 )
                 adr0_r[r_base_adr_index+i] += (float)(q_array[off_set]);   // init
               else
                 adr1_r[r_base_adr_index+i] += (float)(q_array[off_set]);   // init               
             }
             else
             {
               if ( r_base_adr_flag == 0 )
                 adr0_r[r_base_adr_index+i] += (float)(q_array[off_set] * f_cos );
               else
                 adr1_r[r_base_adr_index+i] += (float)(q_array[off_set] * f_cos );
             }
             if ( ( reverseFFT == 0 && now_data != 2 ) || (reverseFFT != 0 && data_size != now_data) )
             {
               if ( r_base_adr_flag == 0 )
                 adr0_i[r_base_adr_index+i] += (float)(q_array[off_set] * f_sin );
               else
                 adr1_i[r_base_adr_index+i] += (float)(q_array[off_set] * f_sin );
             }
             if ( reverseFFT == 0 ) continue;
             if ( off_set == 0 || (off_set == half_size && data_size %2 == 0) )continue;
             //imaginary has different sign
             if (flag==0)
             {
               if ( r_base_adr_flag == 0 )
                 adr0_r[r_base_adr_index+i] -= (float)(q_array[off_set + half_size] * f_sin );
               else
                 adr1_r[r_base_adr_index+i] -= (float)(q_array[off_set + half_size] * f_sin );
             }
             else
             {
               if ( r_base_adr_flag == 0 )
                 adr0_r[r_base_adr_index+i] += (float)(q_array[off_set + half_size] * f_sin );
               else
                 adr1_r[r_base_adr_index+i] += (float)(q_array[off_set + half_size] * f_sin );
             }
             if (data_size != now_data)
             {
               if (flag==0)
               {
                 if ( r_base_adr_flag == 0 )
                   adr0_i[r_base_adr_index+i] += (float)(q_array[off_set + half_size] * f_cos );
                 else
                   adr1_i[r_base_adr_index+i] += (float)(q_array[off_set + half_size] * f_cos );
               }
               else
               {
                 if ( r_base_adr_flag == 0 )
                   adr0_i[r_base_adr_index+i] -= (float)(q_array[off_set + half_size] * f_cos );
                 else
                   adr1_i[r_base_adr_index+i] -= (float)(q_array[off_set + half_size] * f_cos );
               }
             }
           }
           if (reverseFFT == 0)
           {
             if ( r_base_adr_flag == 0 )
               adr0_r[r_base_adr_index+i] /= now_data;
             else
               adr1_r[r_base_adr_index+i] /= now_data;
             if (now_data!=2)
             {
               if ( r_base_adr_flag == 0 )
                 adr0_i[r_base_adr_index+i] /= now_data;
               else
                 adr1_i[r_base_adr_index+i] /= now_data;
             }
           }
         }
       }
     }

  }

}