/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.util.Arrays;

public class AcopText
{   
	protected  boolean	textON,			// on or off
			outside,
			scalable;					// go with coordinates
	protected  int	xcenter,					// x input: left(0), center(1) or right(2)
			ycenter,					// y input: bottom(0), center(1) or top(2)
      moveable,
      rotationDegree,   // 0 - 365degree
			prePrint,					// textprint prio ondraw!
			width,
			height,
			length;						// char width and string length
  protected  double	text_x, text_y,		// user units??
			conv_x, conv_y,             // starting print position
      lastTextx, lastTexty;
  protected  double	xsize, ysize;		// user display rectangle
	protected  double	scaleFactor;
  protected  int	saveHeight, saveWidth, lfHeight, lfWidth, lfAscent, lfDescent;
	protected  int		dev_x, dev_y;		// device units??
  protected  java.awt.Color   textColor;
  protected  java.lang.String[]	textstr;
	protected  java.awt.Font	textFont, inputFont;
	protected  Rectangle	textRect;
  protected  int rescaleFlag;

  protected  double rotateGrad;
  protected  int    offsetX, offsetY;   // device units??
  protected  double sin, cos;
  protected  double  xOffset,yOffset;      // for text area computation
  protected  double  heightSin, heightCos, lengthSin, lengthCos;
 
  private Acop acop;
    
  public AcopText(Acop acop)
	{
    this.acop = acop;
	  
	  textON=false;
	  text_x=0;
	  text_y=0;
	  length=0;
	  prePrint=0;
	  outside=true;
	  scalable=false;
	  moveable=0;
	  xcenter=0;
	  ycenter=0;
    rescaleFlag = 1;
      
    textRect = new Rectangle();
		//{{INIT_CONTROLS
		//}}
	}
	//{{DECLARE_CONTROLS
	protected java.lang.String getPrintedText(double[] leftBottom, double[] size, double[] position)
	{
    if ( position != null )
    {
      position[0] = text_x;
	    if ( position.length > 1 ) position[1] = text_y;
	  }
    if ( size != null )
    {
      size[0] = textRect.width * acop.sca[0].dispSize / acop.aFrame.dxwidth;
      if ( size.length > 1 ) size[1] = textRect.height * acop.sca[1].dispSize / acop.aFrame.histRect.height;
    }
    if ( leftBottom != null )
    {
      leftBottom[0] = (textRect.x-acop.aFrame.histRect.x) * acop.sca[0].dispSize / acop.aFrame.dxwidth + acop.sca[0].min;
      if ( leftBottom.length > 1 ) leftBottom[1] = (textRect.height-textRect.y+acop.aFrame.histRect.y) * acop.sca[1].dispSize / acop.aFrame.histRect.height + acop.sca[1].min;
    }
    StringBuilder sb = new StringBuilder();
    for (String s : textstr) {
    	sb.append(s);
    	sb.append("\n");
    }
    	return sb.toString();
	}
        
  protected void printUserText(java.awt.Graphics2D a)
  { boolean nullFlag;
  //by jbobnar: Do not print the text if acop is not visible
  if (!acop.isShowing()) return;
    if ( acop.textFlag == false || textON == false || outside == true ) return;
    if ( rescaleFlag != 0 ) rescaleText(rescaleFlag);
	  if ( a == null )
	  {
	    a = (Graphics2D)acop.getGraphics();
      a.clipRect(acop.histRoundRect.x, acop.histRoundRect.y, acop.histRoundRect.width, acop.histRoundRect.height);
      a.translate(acop.aFrame.histRect.x, acop.aFrame.histRect.y);
	    nullFlag = true;
	  }
	  else nullFlag = false;
    
//    a.clipRect(textRect.x, textRect.y, textRect.width, textRect.height);
    	  
    a.setFont(textFont);
    a.setColor(textColor);
    if ( rotationDegree != 0 )
    {
      AffineTransform oldTransform = a.getTransform();
      a.translate(dev_x, dev_y);
      a.rotate(-rotateGrad);
      int yText = offsetY - lfDescent;
      int textHeight = a.getFontMetrics().getHeight() + 1;
      for (int i = 0; i < textstr.length; i++) {
    	  a.drawString(textstr[i], offsetX, yText + i*textHeight);  
      }
      a.setTransform(oldTransform);
    } else {
    	int offset = 0;
    	if (acop.printTextLine) {
    		Color c = a.getColor();
    		a.setPaintMode();
    		a.setColor(Color.DARK_GRAY);
            a.drawLine(dev_x+offsetX,0,dev_x+offsetX,acop.histRoundRect.height);
            a.setColor(c);
            offset = 2;
    	}
    	int yText = dev_y+offsetY - lfDescent;
        int textHeight = a.getFontMetrics().getHeight() + 1;
        for (int i = 0; i < textstr.length; i++) {
      	  a.drawString(textstr[i], dev_x+offsetX+offset, yText + i*textHeight);  
        }
        
    }
     
    if ( nullFlag )
    {
      a.translate(-acop.aFrame.histRect.x, -acop.aFrame.histRect.y);
      a.dispose();
	  }
  }

  protected int printText(java.lang.String Text, double Xposition, double Yposition, java.awt.Color c, java.awt.Font font)
  { int resca, lineFit;      
    int move, localxcenter, localycenter, localRotation;
    boolean localscalable;
      
    lineFit = Text.indexOf(13);
      
    move = acop.printTextStickyAttribute & AcopConst.TEXT_MOVE;
    if ( acop.printTextStickyAttribute == 4 ) move = AcopConst.TEXT_MOVE;
	  localxcenter = acop.printTextLocation % 3;
	  if ( acop.printTextLocation < 3 ) localycenter = 0;
	  else if ( acop.printTextLocation < 6 ) localycenter = 1;
	  else localycenter = 2;
    localRotation = acop.printTextRotation;
    
    localscalable = move == AcopConst.TEXT_MOVE ? true : false;
    if ( acop.printTextStickyAttribute == 4 ) localscalable = false;

    moveable = 0;
    scalable = false;

    if ( textON == true )		// get rid of old text
    {
    	textON = false;
    	if ( outside == false ) acop.repaint( textRect.x, textRect.y, textRect.width, textRect.height);
    }
      
    // check if location or ration have been changed?
    resca = (localxcenter != xcenter || localycenter != ycenter) ? 1 : 0 ;
    if ( Math.abs(Xposition - text_x) > AcopConst.zero_check ) resca++;
	  text_x = Xposition;
    if ( Math.abs(Yposition - text_y) > AcopConst.zero_check ) resca++;
	  text_y = Yposition;
    if ( localRotation != rotationDegree ) resca++;
 
    String[] newText = Text.split("\n");
    if ( lineFit > 0 ) {		// if text + linefeed, then remove linefeed
    	Text = Text.substring(0,lineFit-1);
  	  lineFit = 1;
    } else {
    	lineFit = 0;
    }

    rotationDegree = localRotation;
    rotateGrad = rotationDegree * Math.PI / 180;
    if ( rotationDegree == 0 )
    {
      cos = 1;
      sin = 0;
    }
    else
    {
      cos = Math.cos(rotateGrad);
      sin = Math.sin(rotateGrad);
    }
    
    textColor = c;
    textFont = font;
    inputFont = font;
    textstr = newText;
    xcenter = localxcenter;
    ycenter = localycenter;
    if ( xcenter == 0 ) xOffset = 0;  // set rotation calculation parameters
    else if ( xcenter == 1 ) xOffset = 0.5;
    else xOffset = 1;
    if ( ycenter == 0 ) yOffset = 1;
    else if ( ycenter == 1 ) yOffset = 0.5;
    else yOffset = 0;
    
    textON = true;
    if ( resca != 0 || rescaleFlag != 0 )
    {
    	length = Text.length();
    	rescaleText(AcopConst.NEW_PRINT_WINDOW);
    }
    else if ( length != Text.length() )
    {
	    length = Text.length();
      setTextAttribute(true,conv_x);
    }
    lastTextx = text_x;       //store last text coordinates
    if ( lineFit != 0 )
    {
      lastTexty = acop.sca[1].dispSize * lfHeight / acop.aFrame.histRect.height;
      if ( acop.sca[1].log != 0 ) lastTexty = Math.pow ( 10.0, lastTexty );
      lastTexty = text_y - lastTexty;
    } else lastTexty = text_y;

    acop.textFlag = true;
    moveable = move;
    scalable = localscalable;
      
    if ( scalable )	// store origin display area
    {
    	xsize = acop.sca[0].dispSize;
    	ysize = acop.sca[1].dispSize;
      scaleFactor = 1;
	    saveHeight = lfHeight;
    	saveWidth = lfWidth;
    }

    if ( acop.drawTicks )
    {
      acop.repaint();
      acop.drawTicks = false;
    }
    else  printUserText(null);

//      if ( de.desy.acop.chart.disp[0] == null && scalecall < 2 )
//      {
//    	InvalidateControl(NULL);		// draw ticks and grid...if no histo at all
//    	scalecall = 2;
//      }
    return ( 0 );
  }

  protected void refreshText(java.lang.String Text)
  { int scaled = 0;

  String[] newText = Text != null ? Text.split("\n") : null;
    if ( textON )    // get rid of old text
    {
      if ( Arrays.equals(textstr,newText) && rescaleFlag == 0 ) return;  // already on
    	if ( outside == false )
    	{
    	  textON = false;
        acop.repaint( textRect.x, textRect.y, textRect.width, textRect.height);
    	}
    }
    else
    {
      if ( rescaleFlag != 0 && (moveable != 0  || scalable) )
      {
        textON = true;
    	  length = Text.length();
        rescaleText(0);
	      scaled = 1;
      }
	    acop.textFlag = true;
    }
    textON = true;
    textstr = newText;
    if ( scaled == 0 && length != Text.length() )
    {
	    length = Text.length();
      setTextAttribute(true,conv_x);
    }
    printUserText(null);
  }

  protected void setTextRotationParameters()
  {   
    Graphics2D a = (Graphics2D)acop.getGraphics();
    if (a == null) return; // currently not displayable!! 2010-9-29
    java.awt.Font tempFont = a.getFont();
    a.setFont(textFont);
    FontMetrics fMetrix = a.getFontMetrics();
    a.setFont(tempFont);
    a.dispose();
    
    int length = 0;
    for (String s : textstr) {
    	length = Math.max(length,fMetrix.stringWidth(s));
    }
    lfWidth = length;
    if (acop.printTextLine) {
    	lfWidth += 2;
    }
    lfHeight = fMetrix.getHeight()  + fMetrix.getAscent() + fMetrix.getDescent();
    lfHeight *= textstr.length;
    lfAscent = fMetrix.getAscent();
    lfDescent = fMetrix.getDescent();
    
    heightSin = lfHeight * sin;
    heightCos = lfHeight * cos;
    lengthSin = lfWidth * sin;
    lengthCos = lfWidth * cos;
    
    if ( xcenter == 0 ) offsetX = 0;
    else if ( xcenter == 1 ) offsetX = - lfWidth /2 ;
    else offsetX = -lfWidth;

    if ( ycenter == 0 ) offsetY = 0;
    else if ( ycenter == 1 ) offsetY = lfHeight /2 ;
    else offsetY = lfHeight;
  }

  protected void setTextAttribute(boolean newText, double xInput)
  { double xtemp = xInput;     // can be conv_x or input after wrapText()
    double ytemp = conv_y;
    int x,y;
    int tmp;
    int bottomLeft_cx, bottomLeft_cy;
    int topRight_cx, topRight_cy, bottomRight_cx, bottomRight_cy;
 
    if ( newText  ) setTextRotationParameters();

    // rescale to device units
    dev_x = (int)((xtemp - acop.sca[0].min) / acop.sca[0].dispSize * acop.aFrame.dxwidth + acop.aFrame.xleft + 0.01);
    dev_y = (int)(acop.aFrame.histRect.height - ( (ytemp - acop.sca[1].min) / acop.sca[1].dispSize * acop.aFrame.histRect.height ));
    
    y = (int) (dev_y - yOffset * heightCos + xOffset * lengthSin) + acop.aFrame.histRect.y;// + lfDescent;
    x = (int) (dev_x - yOffset * heightSin - xOffset * lengthCos) + acop.aFrame.histRect.x ;

    bottomLeft_cx = (int)(x + heightSin);
    bottomLeft_cy = (int)(y + heightCos);

    topRight_cx = (int)(x + lengthCos);
    topRight_cy = (int)(y - lengthSin);

    bottomRight_cx = (int)(x + heightSin + lengthCos);
    bottomRight_cy = (int)(y + heightCos - lengthSin);

    tmp = x;
    if ( tmp > bottomLeft_cx ) tmp = bottomLeft_cx;
    if ( tmp > topRight_cx ) tmp = topRight_cx;
    if ( tmp > bottomRight_cx ) tmp = bottomRight_cx;
    textRect.x = tmp;

    tmp = x;
    if ( tmp < bottomLeft_cx ) tmp = bottomLeft_cx;
    if ( tmp < topRight_cx ) tmp = topRight_cx;
    if ( tmp < bottomRight_cx ) tmp = bottomRight_cx;
    textRect.width = tmp - textRect.x;

    tmp = y;
    if ( tmp > bottomLeft_cy ) tmp = bottomLeft_cy;
    if ( tmp > topRight_cy ) tmp = topRight_cy;
    if ( tmp > bottomRight_cy ) tmp = bottomRight_cy;
    textRect.y = tmp;

    tmp = y;
    if ( tmp < bottomLeft_cy ) tmp = bottomLeft_cy;
    if ( tmp < topRight_cy ) tmp = topRight_cy;
    if ( tmp < bottomRight_cy ) tmp = bottomRight_cy;
    textRect.height = tmp - textRect.y;

    outside = textRect.intersects( acop.histRoundRect ) ? false : true;
  }

  protected void scaleFont()
  { double x,y;
      
    // relative to original size
    x = acop.sca[0].dispSize / xsize;
	  y = acop.sca[1].dispSize / ysize;
    if ( Math.abs(scaleFactor - y) <= AcopConst.zero_check ||
      Math.abs(scaleFactor - x) <= AcopConst.zero_check ) return;
    if ( x > 1 && y > 1 )
	  {
	    if ( x < y ) y = x;
    }
    else if ( x > y ) y = x;

    scaleFactor = y;    // store new factor for next comparison
	  textFont = new Font(inputFont.getName(), inputFont.getStyle(), (int)(inputFont.getSize() / y));

    lfHeight = (int)(saveHeight / y);
	  lfWidth = (int)(saveWidth / x);
  }

  protected void adjustXY(int flag)
  { int xflag = flag & 0x1;
	  int yflag = flag & 0x2;
	  
	  if ( xflag != 0 )
      conv_x = acop.sca[0].log != 0 ? Math.log( AcopConst.log_check(text_x) )/Math.log(10) : text_x;
	  if ( yflag != 0 )
      conv_y = acop.sca[1].log != 0 ? Math.log( AcopConst.log_check(text_y) )/Math.log(10) : text_y;
      
    if ( xflag != 0 || yflag != 0 ) setTextRotationParameters();
  }

  protected double wrapText()
  { int i;
    double x = conv_x;
    AcopHisto h = acop.histo[acop.m_histstart];
      
	  i = h.wraphigh < 0 ? -(h.wraphigh + 1) : h.wraphigh;
    if ( h.wraphigh < 0 )
    {
  		if ( x >= h.x[i] )
	   	{
		    if ( h.xaryFlag == 1 ) x += (acop.wrapxmax - acop.wrapxmin);
		    else x -= (acop.wrapxmax - acop.wrapxmin);
  		}
    }
    else if (x < h.x[i])
    {
  	 	if ( h.xaryFlag == 1 ) x -= (acop.wrapxmax - acop.wrapxmin);
	   	else x += (acop.wrapxmax - acop.wrapxmin);
	  }
	  return x;
  }
        
  protected void rescaleText(int rescaleTextFlag)
  { double x;
      
    // resize, framewidth and printtext(3); Zoom (2); move(1)
    if ( textON == false )
    {
      rescaleFlag = rescaleTextFlag;  // store flag
      return;
    }
      
    // load flag
    if ( rescaleTextFlag == 0 )
    {
      if ( rescaleFlag == 0 ) return;
      rescaleTextFlag = rescaleFlag;
      return;
    }
      
    // new font
    if ( rescaleTextFlag == AcopConst.NEW_ZOOM_WINDOW && scalable ) scaleFont();
    // new device window
    if ( rescaleTextFlag == AcopConst.NEW_PRINT_WINDOW ) adjustXY(AcopConst.TEXT_MOVE);
    else if ( rescaleTextFlag != AcopConst.NEW_MOVE_WINDOW && moveable != 0 ) adjustXY(moveable);
    // wrap
    if ( ( rescaleTextFlag == AcopConst.NEW_PRINT_WINDOW || (moveable & 0x1) != 0) &&
                acop.wrapAround && acop.histo[acop.m_histstart] != null && acop.sca[0].log == 0 && acop.histo[acop.m_histstart].wraphigh != 0 ) x = wrapText();
    else x = conv_x;
    // map
    if ( rescaleTextFlag == AcopConst.NEW_PRINT_WINDOW || (moveable & 0x1) != 0 ) setTextAttribute(false, x);
      
    rescaleFlag = 0 ;     // reset flag
  }
	//}}
}

