/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;
import java.text.Format;
import javax.swing.JFormattedTextField;
import de.desy.acop.chart.Acop;

public class AcopJFormattedTextField extends JFormattedTextField
{
  private static final long serialVersionUID = 1048925056801267192L;
  Acop acopBean;
  String proper;
  int dataType; // integer, Double, string
  boolean checkRange;
  double lowCheck, highCheck;
  Class<?> c;
  Object oldValue, newValue;
  public AcopJFormattedTextField()
  {
    super();
  }
  public AcopJFormattedTextField(Object value)
  {
    super(value);
  }
  public AcopJFormattedTextField(Format format)
  {
    super(format);
  }
  public AcopJFormattedTextField(AbstractFormatter formatter)
  {
    super(formatter);
  }
  public AcopJFormattedTextField(AbstractFormatterFactory factory)
  {
    super(factory);
  }
  public AcopJFormattedTextField(AbstractFormatterFactory factory, Object currentValue)
  {
    super(factory, currentValue);
  }
  public void setAcopBean(Acop Bean)
  {
    acopBean = Bean;
    try
    {
      Method m = acopBean.getClass().getMethod("get" + proper);
      oldValue = m.invoke(acopBean);

//      System.out.println("set Text by acopBean: " + oldValue.toString());
      
      setValue(oldValue); // set text value
      setText(oldValue.toString());
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }
  public AcopJFormattedTextField(String text, String property, Object value)
  {
    super(value);
    initAcopJFormattedTextField(text, property, value, false, 0, 0);
  }
  public AcopJFormattedTextField(String text, String property, Object value, boolean check, double low,
      double high)
  {
    super(value);
    initAcopJFormattedTextField(text, property, value, check, low, high);
  }
  public void initAcopJFormattedTextField(String text, String property, Object value, boolean check,
      double low, double high)
  {
    proper = property;
    oldValue = value;
    checkRange = check;
    this.lowCheck = low;
    this.highCheck = high;
    setName(proper);
    if (value.equals(int.class) || value.equals(short.class))
    {
      c = int.class;
      dataType = 0;
    }
    else if (value.equals(float.class) || value.equals(double.class))
    {
      c = double.class;
      dataType = 1;
    }
    else
    {
      c = String.class;
      dataType = 2;
    }
//    System.out.println("text data Type: " + dataType);
    this.addPropertyChangeListener(new PropertyChangeListener()
    {
      public void propertyChange(PropertyChangeEvent evt)
      {
        String name = evt.getPropertyName();
        if ( name.equals(proper)) {
        	return;
        }
        Object source = evt.getSource();
        newValue = ((JFormattedTextField) source).getValue();
        // amount = ((Number)amountField.getValue()).doubleValue();
        // paymentField.setValue(new Double(payment));
//        System.out.println("text changed: " + proper + "   oldValue: " + oldValue + "   value: " + newValue); 
        if (checkRange)
        {
          double conValue = Double.valueOf(newValue.toString());
          if (conValue < lowCheck || conValue > highCheck)
          {
            setValue(oldValue); // set text value
            setText(oldValue.toString());
          }
        }
        if (acopBean != null && !newValue.equals(oldValue))
        {
          try
          {
            Method m = acopBean.getClass().getMethod("set" + proper, c);
            m.invoke(acopBean, newValue);
//            System.out.println("fire===> text changed: " + proper + "   oldValue: " + oldValue + "   value: "
//                + newValue); 
            firePropertyChange(proper, oldValue, newValue);
            // firePropertyChange(null, null, null);
            oldValue = newValue;
          }
          catch (Exception ex)
          {
            ex.printStackTrace();
          }
        }
        // }
      }
    });
  }
}
