/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.beans.Customizer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import de.desy.acop.chart.Acop;

public class AxisCustomizerPanel extends JPanel implements Customizer
{
	
  private static final long serialVersionUID = -6983060404484262793L;
  public static final String[] AXIS_PROPERTIES = {
	    "secondaryYTickText","secondaryYFineTicks","secondaryYTicks","secondaryYBestScale","secondaryYLogOn",
	    "secondaryYAxisOn","secondaryYMax","secondaryYMin","secondaryYAxisLabel","YTickText","YFineTicks",
	    "YTicks","leftYLabel","YBestScale","YScale","YShift","YMax","YMin","YAxisLabel","XTickText","XFineTicks",
	    "XTicks","leftTimeLabel","bottomXLabel","XBestScale","XScale","XShift","XMax","XMin","XAxisLabel"
	};

  private JLabel xAxisLabel = null;
  private JLabel xLabel = null;
  private AcopJFormattedTextField xLabelText;
  private JLabel xMinLabel, xMaxLabel, xShiftLabel, xScaleLabel;
  private AcopJFormattedTextField xMinText, xMaxText, xShiftText, xScaleText;
  private AcopJRadioButton xBestScale, xBottomLabel, xLeftTime;
  private JLabel xTickLabel = null;
  private JLabel xTicksLabel, xFTicksLabel;
  private AcopJFormattedTextField xTickText, xFTickText;
  private AcopJRadioButton xTickTextButton;
  
  private JLabel yAxisLabel = null;
  private JLabel yLabel = null;
  private AcopJFormattedTextField yLabelText;
  private JLabel yMinLabel, yMaxLabel, yShiftLabel, yScaleLabel;
  private AcopJFormattedTextField yMinText, yMaxText, yShiftText, yScaleText;
  private AcopJRadioButton yBestScale, yLeftLabel;
  private JLabel yTickLabel = null;
  private JLabel yTicksLabel, yFTicksLabel;
  private AcopJFormattedTextField yTickText, yFTickText;
  private AcopJRadioButton yTickTextButton;
  
  private JLabel y2AxisLabel = null;
  private JLabel y2Label = null;
  private AcopJFormattedTextField y2LabelText;
  private JLabel y2MinLabel, y2MaxLabel;
  private AcopJFormattedTextField y2MinText, y2MaxText;
  private AcopJRadioButton y2AxisOn, y2LogOn;
  private AcopJRadioButton y2BestScale;
  private JLabel y2TickLabel = null;
  private JLabel y2TicksLabel, y2FTicksLabel;
  private AcopJFormattedTextField y2TickText, y2FTickText;
  private AcopJRadioButton y2TickTextButton;
  
  public AxisCustomizerPanel(LayoutManager layout, boolean isDoubleBuffered)
  {
    super(layout, isDoubleBuffered);
    initialize();
  }
  public AxisCustomizerPanel(LayoutManager layout)
  {
    super(layout);
    initialize();
  }
  public AxisCustomizerPanel(boolean isDoubleBuffered)
  {
    super(isDoubleBuffered);
    initialize();
  }
  public AxisCustomizerPanel()
  {
    super();
    initialize();
  }
  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize()
  {
    setLayout(new GridBagLayout());
    if (xAxisLabel == null)
    {
      xAxisLabel = new JLabel("-- X-Axis --");
    }
    if (xLabel == null)
    {
      xLabel = new JLabel("Label:");
    }
    if (xMinLabel == null)
    {
      xMinLabel = new JLabel("Min:");
    }
    if (xMaxLabel == null)
    {
      xMaxLabel = new JLabel("Max:");
    }
    if (xShiftLabel == null)
    {
      xShiftLabel = new JLabel("Shift:");
    }
    if (xScaleLabel == null)
    {
      xScaleLabel = new JLabel("Scale:");
    }
    if (xTickLabel == null)
    {
      xTickLabel = new JLabel("-- Ticks --");
    }
    if (xTicksLabel == null)
    {
      xTicksLabel = new JLabel("C:");
    }
    if (xFTicksLabel == null)
    {
      xFTicksLabel = new JLabel("F:");
    }

    if (yAxisLabel == null)
    {
      yAxisLabel = new JLabel("-- Y-Axis --");
    }
    if (yLabel == null)
    {
      yLabel = new JLabel("Label:");
    }
    if (yMinLabel == null)
    {
      yMinLabel = new JLabel("Min:");
    }
    if (yMaxLabel == null)
    {
      yMaxLabel = new JLabel("Max:");
    }
    if (yShiftLabel == null)
    {
      yShiftLabel = new JLabel("Shift:");
    }
    if (yScaleLabel == null)
    {
      yScaleLabel = new JLabel("Scale:");
    }
    if (yTickLabel == null)
    {
      yTickLabel = new JLabel("-- Ticks --");
    }
    if (yTicksLabel == null)
    {
      yTicksLabel = new JLabel("C:");
    }
    if (yFTicksLabel == null)
    {
      yFTicksLabel = new JLabel("F:");
    }
    
    if (y2AxisLabel == null)
    {
      y2AxisLabel = new JLabel("-- Secondary Y-Axis --");
    }
    if (y2Label == null)
    {
      y2Label = new JLabel("Label:");
    }
    if (y2MinLabel == null)
    {
      y2MinLabel = new JLabel("Min:");
    }
    if (y2MaxLabel == null)
    {
      y2MaxLabel = new JLabel("Max:");
    }
    if (y2TickLabel == null)
    {
      y2TickLabel = new JLabel("-- Ticks --");
    }
    if (y2TicksLabel == null)
    {
      y2TicksLabel = new JLabel("C:");
    }
    if (y2FTicksLabel == null)
    {
      y2FTicksLabel = new JLabel("F:");
    }
    
    add(xAxisLabel, new GridBagConstraints(0, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(xLabel, new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xLabelText = new AcopJFormattedTextField("xAxis", "XAxisLabel", String.class);
    add(xLabelText, new GridBagConstraints(2, 1, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    add(xMinLabel, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xMinText = new AcopJFormattedTextField("1", "XMin", double.class);
    add(xMinText, new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(xMaxLabel, new GridBagConstraints(2, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xMaxText = new AcopJFormattedTextField("1", "XMax", double.class);
    add(xMaxText, new GridBagConstraints(3, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(xShiftLabel, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xShiftText = new AcopJFormattedTextField("1", "XShift", double.class);
    add(xShiftText, new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(xScaleLabel, new GridBagConstraints(2, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xScaleText = new AcopJFormattedTextField("1", "XScale", double.class);
    add(xScaleText, new GridBagConstraints(3, 3, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    xBestScale = new AcopJRadioButton("Best Scale", "XBestScale", "xBestScale");
    add(xBestScale, new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xBottomLabel = new AcopJRadioButton("Bottom X Label", "BottomXLabel", "bottomXLabel");
    add(xBottomLabel, new GridBagConstraints(2, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xLeftTime = new AcopJRadioButton("LeftTimeLabel", "LeftTimeLabel", "leftTimeLabel");
    add(xLeftTime, new GridBagConstraints(3, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    add(xTickLabel, new GridBagConstraints(0, 6, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));

    add(xTicksLabel, new GridBagConstraints(0, 7, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xTickText = new AcopJFormattedTextField("-1", "XTicks", int.class);
    add(xTickText, new GridBagConstraints(1, 7, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(xFTicksLabel, new GridBagConstraints(2, 7, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    xFTickText = new AcopJFormattedTextField("-1", "XFineTicks", int.class);
    add(xFTickText, new GridBagConstraints(3, 7, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    xTickTextButton = new AcopJRadioButton("TickText", "XTickText", "xTickText");
    add(xTickTextButton, new GridBagConstraints(1, 8, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    
    
    add(yAxisLabel, new GridBagConstraints(4, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(yLabel, new GridBagConstraints(4, 1, 2, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yLabelText = new AcopJFormattedTextField("yAxis", "YAxisLabel", String.class);
    add(yLabelText, new GridBagConstraints(6, 1, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(yMinLabel, new GridBagConstraints(4, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yMinText = new AcopJFormattedTextField("1", "YMin", double.class);
    add(yMinText, new GridBagConstraints(5, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(yMaxLabel, new GridBagConstraints(6, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yMaxText = new AcopJFormattedTextField("1", "YMax", double.class);
    add(yMaxText, new GridBagConstraints(7, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(yShiftLabel, new GridBagConstraints(4, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yShiftText = new AcopJFormattedTextField("1", "YShift", double.class);
    add(yShiftText, new GridBagConstraints(5, 3, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(yScaleLabel, new GridBagConstraints(6, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yScaleText = new AcopJFormattedTextField("1", "YScale", double.class);
    add(yScaleText, new GridBagConstraints(7, 3, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    yBestScale = new AcopJRadioButton("Best Scale", "YBestScale", "yBestScale");
    add(yBestScale, new GridBagConstraints(5, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yLeftLabel = new AcopJRadioButton("Left Y Label", "LeftYLabel", "leftYLabel");
    add(yLeftLabel, new GridBagConstraints(7, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    add(yTickLabel, new GridBagConstraints(4, 6, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));

    
    add(yTicksLabel, new GridBagConstraints(4, 7, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yTickText = new AcopJFormattedTextField("-1", "YTicks", int.class);
    add(yTickText, new GridBagConstraints(5, 7, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(yFTicksLabel, new GridBagConstraints(6, 7, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    yFTickText = new AcopJFormattedTextField("-1", "YFineTicks", int.class);
    add(yFTickText, new GridBagConstraints(7, 7, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    yTickTextButton = new AcopJRadioButton("TickText", "YTickText", "yTickText");
    add(yTickTextButton, new GridBagConstraints(5, 8, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));

    
    add(y2AxisLabel, new GridBagConstraints(8, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(y2Label, new GridBagConstraints(8, 1, 2, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    y2LabelText = new AcopJFormattedTextField("secondaryYAxis", "SecondaryYAxisLabel", String.class);
    add(y2LabelText, new GridBagConstraints(10, 1, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    add(y2MinLabel, new GridBagConstraints(8, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    y2MinText = new AcopJFormattedTextField("1", "SecondaryYMin", double.class);
    add(y2MinText, new GridBagConstraints(9, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(y2MaxLabel, new GridBagConstraints(10, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    y2MaxText = new AcopJFormattedTextField("1", "SecondaryYMax", double.class);
    add(y2MaxText, new GridBagConstraints(11, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    y2AxisOn = new AcopJRadioButton("Axis On", "SecondaryYAxisOn", "secondaryYAxisOn");
    add(y2AxisOn, new GridBagConstraints(9, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    y2LogOn = new AcopJRadioButton("Log On", "SecondaryYLogOn", "secondaryYAxisOn");
    add(y2LogOn, new GridBagConstraints(11, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));

    y2BestScale = new AcopJRadioButton("Best Scale", "SecondaryYBestScale", "secondaryYBestScale");
    add(y2BestScale, new GridBagConstraints(9, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    add(y2TickLabel, new GridBagConstraints(8, 6, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));

    add(y2TicksLabel, new GridBagConstraints(8, 7, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    y2TickText = new AcopJFormattedTextField("-1", "SecondaryYTicks", int.class);
    add(y2TickText, new GridBagConstraints(9, 7, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    add(y2FTicksLabel, new GridBagConstraints(10, 7, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    y2FTickText = new AcopJFormattedTextField("-1", "SecondaryYFineTicks", int.class);
    add(y2FTickText, new GridBagConstraints(11, 7, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    y2TickTextButton = new AcopJRadioButton("TickText", "SecondaryYTickText", "secondaryYTickText");
    add(y2TickTextButton, new GridBagConstraints(9, 8, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));

    this.setSize(300, 200);
  }
  public void setAcopBean(Acop acopBean)
  {
    xLabelText.setAcopBean(acopBean);
    xMinText.setAcopBean(acopBean);
    xMaxText.setAcopBean(acopBean);
    xShiftText.setAcopBean(acopBean);
    xScaleText.setAcopBean(acopBean);
    xBestScale.setAcopBean(acopBean);
    xBottomLabel.setAcopBean(acopBean);
    xLeftTime.setAcopBean(acopBean);
    xTickText.setAcopBean(acopBean);
    xFTickText.setAcopBean(acopBean);
    xTickTextButton.setAcopBean(acopBean);
    
    yLabelText.setAcopBean(acopBean);
    yMinText.setAcopBean(acopBean);
    yMaxText.setAcopBean(acopBean);
    yShiftText.setAcopBean(acopBean);
    yScaleText.setAcopBean(acopBean);
    yBestScale.setAcopBean(acopBean);
    yLeftLabel.setAcopBean(acopBean);
    yTickText.setAcopBean(acopBean);
    yFTickText.setAcopBean(acopBean);
    yTickTextButton.setAcopBean(acopBean);

    y2LabelText.setAcopBean(acopBean);
    y2MinText.setAcopBean(acopBean);
    y2MaxText.setAcopBean(acopBean);
    y2BestScale.setAcopBean(acopBean);
    y2AxisOn.setAcopBean(acopBean);
    y2LogOn.setAcopBean(acopBean);
    y2TickText.setAcopBean(acopBean);
    y2FTickText.setAcopBean(acopBean);
    y2TickTextButton.setAcopBean(acopBean);
  }
  public void setObject(Object bean) {
	setAcopBean((Acop)bean);
		
  }
}
