/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.beans.Customizer;
import java.lang.reflect.Method;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import de.desy.acop.chart.Acop;

/**
 * @author wu
 */
public class ColorCustomizerPanel extends JPanel implements Customizer
{
  private static final long serialVersionUID = -9095097441423816805L;
  public static final String[] COLOR_PROPERTIES = {
		"background", "cursorMarkerColor", "errorColor", "foreground", "frameForeColor", "gridColor", "keepColor", "leadingEdgeColor", "markerColor", "secondaryYTickLabelColor", "tagColor", "XTickLabelColor", "YTickLabelColor"
	};
	
  class ColorCellRenderer extends DefaultListCellRenderer
  {
    private static final long serialVersionUID = 1299162777225798050L;
    Color[] colors = new Color[AccessColor.values().length];
    int factor = 64; // for background color
    ColorCellRenderer()
    {
      super();
    }
    @Override
    public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected,
        boolean cellHasFocus)
    {
      Component c = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
      if (colors[index] == null)
      { // initialize color[] array
        if (acopBean != null)
        {
          try
          {
            Method m = acopBean.getClass().getMethod("get" + value.toString());
            colors[index] = (Color) m.invoke(acopBean);
//            System.out.println("init CellColor: " + index); 
          }
          catch (Exception ex)
          {
            ex.printStackTrace();
          }
        }
        else
        {
          colors[index] = new Color((int) (Integer.MAX_VALUE * Math.random()));
//          System.out.println("init random CellColor: " + index); 
        }
      }
      c.setBackground(colors[index]);
      // c.setForeground(new Color(255-colors[index].getRed(),
      // 255-colors[index].getGreen(),255-colors[index].getBlue()));
      c.setForeground(new Color(255 - (255 - colors[index].getRed()) / factor, 255
          - (255 - colors[index].getGreen()) / factor, 255 - (255 - colors[index].getBlue()) / factor));
      // c.setForeground(new Color(~(c.getBackground().getRGB())));
//      System.out.println("setCellColor: " + index + "  " + colors[index]); 
      return c;
    }
  }
  private ColorCellRenderer cellRender;
  private AccessColor accessColor;
  private java.awt.Color selectColor, currentColor;
  private JList<AccessColor> listColor = null; // @jve:decl-index=0:visual-constraint="207,112"
  private Acop acopBean;

  public ColorCustomizerPanel()
  {
    super();
    initialize();
  }
  private void initialize()
  {
    setLayout(new GridBagLayout());
    add(new JLabel("Color Property:"), new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    JScrollPane sc = new JScrollPane(getListColor());
    sc.setMinimumSize(sc.getPreferredSize());
    add(sc, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.VERTICAL, new Insets(4, 4, 4, 4), 0, 0));
  }
  private JList<AccessColor> getListColor()
  {
    if (listColor == null)
    {
      listColor = new JList<>(AccessColor.values());
      listColor.addListSelectionListener(new ListSelectionListener()
      {
        public void valueChanged(ListSelectionEvent e)
        {
          System.out.println("ListCOlor select index: " + listColor.getSelectedIndex()); 
          // if ( accessColor.values() != listColor.getSelectedValue() )
          // {
          accessColor = listColor.getSelectedValue();
          currentColor = cellRender.colors[listColor.getSelectedIndex()];
          /*
           * try { Method m = acopBean.getClass().getMethod("get" +
           * accessColor.toString(),null); currentColor =
           * (Color)m.invoke(acopBean, null); } catch (Exception ex) {
           * currentColor =
           * (java.awt.Color)cellRender.colors[listColor.getSelectedIndex()];
           * System.out.println("exception set color: " + "currentColor: "
           * +currentColor); 
           * itemStateChanged() ex.printStackTrace(); }
           */
//          System.out.println(accessColor.toString() + "currentColor: " + currentColor); 
          selectColor = JColorChooser.showDialog(ColorCustomizerPanel.this, accessColor.toString(),
              currentColor);
//          System.out.println("selectColor: " + selectColor + "currentColor: " + currentColor); 
          if (selectColor != null && selectColor != currentColor)
          {
            try
            {
              Method m = acopBean.getClass().getMethod("set" + accessColor.toString(),
                  Color.class);
              m.invoke(acopBean, selectColor);
              firePropertyChange(accessColor.toString(), currentColor, selectColor);
            }
            catch (Exception ex)
            {
              ex.printStackTrace();
            }
            // set cellRende display color
            cellRender.colors[listColor.getSelectedIndex()] = selectColor;
//            System.out.println("setAccessColor" + listColor.getSelectedValue()); 
          }
//          System.out.println("itemStateChanged()"); 
          // listColor.setSelectedIndex(-1);
          // }
        }
      });
      // init cell render
      cellRender = new ColorCellRenderer();
      listColor.setCellRenderer(cellRender);
    }
    return listColor;
  }
  /**
   * @return Returns the acopBean.
   */
  public Acop getAcopBean()
  {
    return acopBean;
  }
  /**
   * @param acopBean
   *          The acopBean to set.
   */
  public void setAcopBean(Acop acopBean)
  {
    this.acopBean = acopBean;
    accessColor = AccessColor.Background; // init to first element of
                                          // AccessColor enum
    // re-init cell render after loading acop bean
    cellRender = new ColorCellRenderer();
    listColor.setCellRenderer(cellRender);
  }
  public void setObject(Object bean) {
	setAcopBean((Acop)bean);
		
  }
}
