/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.beans.Customizer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import de.desy.acop.chart.Acop;
import de.desy.acop.chart.AcopCaptionLocationEnum;
import de.desy.acop.chart.AcopConst;
import de.desy.acop.chart.AcopTextPositionEnum;
import de.desy.acop.chart.AcopTextStickyAttributeEnum;

public class FrameCustomizerPanel extends JPanel implements Customizer
{
  private static final long serialVersionUID = 6756733178775494377L;
  public static final String[] FRAME_PROPERTIES = {
	    "reuseText", "textDepth", "printTextStickyAttribute", "printTextLocation", 
	    "frameTopOffset", "frameBottomOffset", "frameLeftOffset", "frameRightOffset",
	    "frameWidth", "captionLocation", "caption"
	};

  private JLabel frameLabel = null;
  private JLabel captionLabel = null;
  private JLabel textLabel = null;
  private AcopJFormattedTextField captionText;
  private JLabel captionLocationLabel = null;
  private AcopJComBox<AcopCaptionLocationEnum> captionLocationBox;  
  private JLabel widthLabel;
  private AcopJFormattedTextField widthText;
  private JLabel widthOffsetLabel = null;
  private JLabel topLabel, bottomLabel, leftLabel, rightLabel;
  private AcopJFormattedTextField frameTopText, frameBottomText, frameLeftText, frameRightText;
  
  private JLabel textPositionLabel = null;
  private AcopJComBox<AcopTextPositionEnum> textPositionBox;
  private JLabel stickyParamLabel = null;
  private AcopJComBox<AcopTextStickyAttributeEnum> stickyParamBox;
  private JLabel textBufferLabel;
  private AcopJFormattedTextField textDepthText;
  private AcopJRadioButton reuseText;
   
  public FrameCustomizerPanel(LayoutManager layout, boolean isDoubleBuffered)
  {
    super(layout, isDoubleBuffered);
    initialize();
  }
  public FrameCustomizerPanel(LayoutManager layout)
  {
    super(layout);
    initialize();
  }
  public FrameCustomizerPanel(boolean isDoubleBuffered)
  {
    super(isDoubleBuffered);
    initialize();
  }
  public FrameCustomizerPanel()
  {
    super();
    initialize();
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize()
  {
    setLayout(new GridBagLayout());
    if (frameLabel == null)
    {
      frameLabel = new JLabel("-- Frame Parameters --");
    }
    if (captionLabel == null)
    {
      captionLabel = new JLabel("Caption:");
    }
    if (captionLocationLabel == null)
    {
      captionLocationLabel = new JLabel("Caption Location:");
    }
    if (widthLabel == null)
    {
      widthLabel = new JLabel("Width:");
    }

    if (textLabel == null)
    {
      textLabel = new JLabel("-- Text Parameters --");
    }
    if (widthOffsetLabel == null)
    {
      widthOffsetLabel = new JLabel("-- Width Offset --");
    }
    if (topLabel == null)
    {
      topLabel = new JLabel("Top:");
    }
    if (bottomLabel == null)
    {
      bottomLabel = new JLabel("Bottom:");
    }
    if (leftLabel == null)
    {
      leftLabel = new JLabel("Left:");
    }
    if (rightLabel == null)
    {
      rightLabel = new JLabel("Right:");
    }
    if (textPositionLabel == null)
    {
      textPositionLabel = new JLabel("Position:");
    }
    if (stickyParamLabel == null)
    {
      stickyParamLabel = new JLabel("StickyAttribute:");
    }
    if (textBufferLabel == null)
    {
      textBufferLabel = new JLabel("#Text Buffers:");
    }

    add(frameLabel, new GridBagConstraints(0, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));

    add(captionLabel, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    captionText = new AcopJFormattedTextField("Test", "Caption", String.class);
    add(captionText, new GridBagConstraints(1, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    add(captionLocationLabel, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    captionLocationBox = new AcopJComBox<>("CaptionLocation", AcopCaptionLocationEnum.values());
    add(captionLocationBox, new GridBagConstraints(1, 2, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
    add(widthLabel, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    widthText = new AcopJFormattedTextField("10", "FrameWidth", int.class);
    add(widthText, new GridBagConstraints(1, 3, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(widthOffsetLabel, new GridBagConstraints(0, 4, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));

    add(topLabel, new GridBagConstraints(0, 5, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    frameTopText = new AcopJFormattedTextField("1", "FrameTopOffset", int.class);
    add(frameTopText, new GridBagConstraints(1, 5, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
    add(bottomLabel, new GridBagConstraints(2, 5, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    frameBottomText = new AcopJFormattedTextField("1", "FrameBottomOffset", int.class);
    add(frameBottomText, new GridBagConstraints(3, 5, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(leftLabel, new GridBagConstraints(0, 6, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    frameLeftText = new AcopJFormattedTextField("1", "FrameLeftOffset", int.class);
    add(frameLeftText, new GridBagConstraints(1, 6, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
   
    add(rightLabel, new GridBagConstraints(2, 6, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    frameRightText = new AcopJFormattedTextField("1", "FrameRightOffset", int.class);
    add(frameRightText, new GridBagConstraints(3, 6, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    add(textLabel, new GridBagConstraints(4, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(textPositionLabel, new GridBagConstraints(4, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    textPositionBox = new AcopJComBox<>("PrintTextLocation", AcopTextPositionEnum.values());
    add(textPositionBox, new GridBagConstraints(5, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    add(stickyParamLabel, new GridBagConstraints(4, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    stickyParamBox = new AcopJComBox<>("PrintTextStickyAttribute", AcopTextStickyAttributeEnum.values());
    add(stickyParamBox, new GridBagConstraints(5, 2, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(textBufferLabel, new GridBagConstraints(4, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    textDepthText = new AcopJFormattedTextField("1", "TextDepth", int.class, true, 1,
        AcopConst.MAX_DISPLAY);
    add(textDepthText, new GridBagConstraints(5, 3, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    reuseText = new AcopJRadioButton("Reuse Text", "ReuseText", "reuseText");
    add(reuseText, new GridBagConstraints(4, 4, 2, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));

    this.setSize(300, 200);
  }
  public void setAcopBean(Acop acopBean)
  {
    captionText.setAcopBean(acopBean);
    captionLocationBox.setAcopBean(acopBean);
    widthText.setAcopBean(acopBean);
    frameTopText.setAcopBean(acopBean);
    frameBottomText.setAcopBean(acopBean);
    frameLeftText.setAcopBean(acopBean);
    frameRightText.setAcopBean(acopBean);
    textPositionBox.setAcopBean(acopBean);
    stickyParamBox.setAcopBean(acopBean);
    reuseText.setAcopBean(acopBean);
    textDepthText.setAcopBean(acopBean);
  }
  
  public void setObject(Object bean) {
	setAcopBean((Acop)bean);
		
  }
}
