/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.beans.Customizer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import de.desy.acop.chart.Acop;
import de.desy.acop.chart.AcopDrawStyleEnum;
import de.desy.acop.chart.AcopGraphStyleEnum;

public class GraphCustomizerPanel extends JPanel implements Customizer 
{
  private static final long serialVersionUID = 7075733109842622938L;
  public static final String[] GRAPH_PROPERTIES = {
		"YGrid","XGrid","gridWidth","gridStyle","drawWidth","drawStyle","graphStyle"
	};
  private JLabel drawParametersLabel = null;
  private JLabel graphLabel = null;
  private AcopJComBox<AcopGraphStyleEnum> graphStyleBox;
  private JLabel drawStyleLabel = null;
  private AcopJComBox<AcopDrawStyleEnum> drawStyleBox;
  private JLabel drawWidthLabel = null;
  private AcopJFormattedTextField drawWidthText;

  private JLabel gridParametersLabel = null;
  private JLabel gridStyleLabel = null;
  private AcopJComBox<AcopDrawStyleEnum> gridStyleBox;
  private JLabel gridWidthLabel = null;
  private AcopJFormattedTextField gridWidthText;
  private AcopJRadioButton xgrid, ygrid;
  
  public GraphCustomizerPanel(LayoutManager layout, boolean isDoubleBuffered)
  {
    super(layout, isDoubleBuffered);
    initialize();
  }
  public GraphCustomizerPanel(LayoutManager layout)
  {
    super(layout);
    initialize();
  }
  public GraphCustomizerPanel(boolean isDoubleBuffered)
  {
    super(isDoubleBuffered);
    initialize();
  }
  public GraphCustomizerPanel()
  {
    super();
    initialize();
  }
  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize()
  {
    setLayout(new GridBagLayout());
    if (drawParametersLabel == null)
    {
      drawParametersLabel = new JLabel("-- Draw Parameters --");
    }
    if (graphLabel == null)
    {
      graphLabel = new JLabel("Graph:");
    }
    if (drawStyleLabel == null)
    {
      drawStyleLabel = new JLabel("Style:");
    }
    if (drawWidthLabel == null)
    {
      drawWidthLabel = new JLabel("Width:");
    }

    if (gridParametersLabel == null)
    {
      gridParametersLabel = new JLabel("-- Grid Parameters --");
    }
    if (gridStyleLabel == null)
    {
      gridStyleLabel = new JLabel("Style:");
    }
    if (gridWidthLabel == null)
    {
      gridWidthLabel = new JLabel("Width:");
    }
    
    add(drawParametersLabel, new GridBagConstraints(0, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));

    add(graphLabel, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    graphStyleBox = new AcopJComBox<>("GraphStyle", AcopGraphStyleEnum.values());
    add(graphStyleBox, new GridBagConstraints(1, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(drawStyleLabel, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    drawStyleBox = new AcopJComBox<>("DrawStyle", AcopDrawStyleEnum.values());
    add(drawStyleBox, new GridBagConstraints(1, 2, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(drawWidthLabel, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    drawWidthText = new AcopJFormattedTextField("10", "DrawWidth", int.class);
    add(drawWidthText, new GridBagConstraints(1, 3, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(gridParametersLabel, new GridBagConstraints(4, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(gridStyleLabel, new GridBagConstraints(4, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    gridStyleBox = new AcopJComBox<>("GridStyle", AcopDrawStyleEnum.values());
    add(gridStyleBox, new GridBagConstraints(5, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(gridWidthLabel, new GridBagConstraints(4, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    gridWidthText = new AcopJFormattedTextField("1", "GridWidth", int.class);
    add(gridWidthText, new GridBagConstraints(5, 2, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    xgrid = new AcopJRadioButton("X-Grid", "XGrid", "xGrid");
    add(xgrid, new GridBagConstraints(4, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    ygrid = new AcopJRadioButton("Y-Grid", "YGrid", "yGrid");
    add(ygrid, new GridBagConstraints(5, 3, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    
    
    this.setSize(396, 218);
  }
  public void setAcopBean(Acop acopBean)
  {
    graphStyleBox.setAcopBean(acopBean);
    drawStyleBox.setAcopBean(acopBean);
    drawWidthText.setAcopBean(acopBean);
    gridStyleBox.setAcopBean(acopBean);
    gridWidthText.setAcopBean(acopBean);
    xgrid.setAcopBean(acopBean);
    ygrid.setAcopBean(acopBean);
  }
  
  public void setObject(Object bean) {
	setAcopBean((Acop)bean);
		
  }
  
}  //  @jve:decl-index=0:visual-constraint="118,10"
