/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.chart.customizer;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.beans.Customizer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import de.desy.acop.chart.Acop;
import de.desy.acop.chart.AcopHistogramPositionEnum;
import de.desy.acop.chart.AcopLeadingEdgeMotionEnum;
import de.desy.acop.chart.AcopZoomingStyleEnum;

public class SpecialCustomizerPanel extends JPanel implements Customizer
{
  private static final long serialVersionUID = 3287108130234117974L;
  public static final String[] SPECIAL_PROPERTIES = {
		"leadingEdgeGapSize","leadingEdgeMode", "histogramPosition", "zooming"
	};
 
  private JLabel zoomingParametersLabel = null;
  private JLabel zoomingStyleLabel = null;
  private AcopJComBox<AcopZoomingStyleEnum> zoomingStyleBox;
  private JLabel histogramParametersLabel = null;
  private JLabel positionLabel = null;
  private AcopJComBox<AcopHistogramPositionEnum> positionBox;

  private JLabel leadingEdgeLabel = null;
  private JLabel motionLabel = null;
  private AcopJComBox<AcopLeadingEdgeMotionEnum> motionBox;
  private JLabel gapSizeLabel = null;
  private AcopJFormattedTextField gapSizeText;

  public SpecialCustomizerPanel(LayoutManager layout, boolean isDoubleBuffered)
  {
    super(layout, isDoubleBuffered);
    initialize();
  }
  public SpecialCustomizerPanel(LayoutManager layout)
  {
    super(layout);
    initialize();
  }
  public SpecialCustomizerPanel(boolean isDoubleBuffered)
  {
    super(isDoubleBuffered);
    initialize();
  }
  public SpecialCustomizerPanel()
  {
    super();
    initialize();
  }
  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize()
  {
    setLayout(new GridBagLayout());
    if (zoomingParametersLabel == null)
    {
      zoomingParametersLabel = new JLabel("-- Zooming Parameters --");
    }
    if (zoomingStyleLabel == null)
    {
      zoomingStyleLabel = new JLabel("Style:");
    }
    if (histogramParametersLabel == null)
    {
      histogramParametersLabel = new JLabel("-- Histogram Parameters --");
    }
    if (positionLabel == null)
    {
      positionLabel = new JLabel("Position:");
    }

    if (leadingEdgeLabel == null)
    {
      leadingEdgeLabel = new JLabel("-- Leading Edge --");
    }
    if (motionLabel == null)
    {
      motionLabel = new JLabel("Motion:");
    }
    if (gapSizeLabel == null)
    {
      gapSizeLabel = new JLabel("Gap Size:");
    }
    
    add(zoomingParametersLabel, new GridBagConstraints(0, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(zoomingStyleLabel, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    zoomingStyleBox = new AcopJComBox<>("Zooming", AcopZoomingStyleEnum.values());
    add(zoomingStyleBox, new GridBagConstraints(1, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    add(histogramParametersLabel, new GridBagConstraints(0, 3, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(positionLabel, new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    positionBox = new AcopJComBox<>("HistogramPosition", AcopHistogramPositionEnum.values());
    add(positionBox, new GridBagConstraints(1, 4, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    
    add(leadingEdgeLabel, new GridBagConstraints(4, 0, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.CENTER, new Insets(4, 4, 4, 4), 0, 0));
    add(motionLabel, new GridBagConstraints(4, 1, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    motionBox = new AcopJComBox<>("LeadingEdgeMode", AcopLeadingEdgeMotionEnum.values());
    add(motionBox, new GridBagConstraints(5, 1, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

    add(gapSizeLabel, new GridBagConstraints(4, 2, 1, 1, 1.0, 1.0, GridBagConstraints.EAST,
        GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
    gapSizeText = new AcopJFormattedTextField("1", "LeadingEdgeGapSize", int.class);
    add(gapSizeText, new GridBagConstraints(5, 2, 3, 1, 1.0, 1.0, GridBagConstraints.CENTER,
        GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
    
    this.setSize(300, 200);
  }
  public void setAcopBean(Acop acopBean)
  {
    zoomingStyleBox.setAcopBean(acopBean);
    positionBox.setAcopBean(acopBean);
    motionBox.setAcopBean(acopBean);
    gapSizeText.setAcopBean(acopBean);
  }
  
  public void setObject(Object bean) {
	setAcopBean((Acop)bean);
		
  }
}
