/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers;

import de.desy.acop.displayers.chart.AcopTrendChartSettingsCustomizer;
import de.desy.acop.displayers.selector.TrendChartConnectionCustomizer;

/**
 * <code>AcopTrendChartCustomizer</code> is a customizer for 
 * <code>AcopTrendChart</code> and includes {@link TrendChartConnectionCustomizer}
 * and {@link AcopTrendChartSettingsCustomizer}.
 * 
 * @author Tilen Kusterle, Cosylab
 * @see AcopTrendChart
 */
public class AcopTrendChartCustomizer extends AcopChartReorgCustomizer {

	private static final long serialVersionUID = 1L;
	private TrendChartConnectionCustomizer connectionCustomizer;

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopChart2Customizer#addChartConnectionCustomizer()
	 */
	@Override
	protected void addCustomizers() {
		addCustomizer("Connections", getConnectionCustomizer());
		addCustomizer("Settings", new AcopTrendChartSettingsCustomizer());
	}
	
	/**
	 * Method returns the connection customizer with preselected parameters.
	 * 
	 * @param parameters
	 * @return
	 */
	public TrendChartConnectionCustomizer getConnectionCustomizer() {
		if (connectionCustomizer == null) {
			connectionCustomizer = new TrendChartConnectionCustomizer();
		}
		return connectionCustomizer;
	}

}
