/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.chart;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import com.cosylab.gui.components.NumberField;

import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.displayers.AcopChartReorg;

/**
 * <code>AcopChartReorgSettingsCustomizer</code> is a customizer which
 * allows customization of the common chart options of the 
 * <code>AcopChartReorg</code>.
 * 
 * @author Tilen Kusterle, Cosylab
 * @see AcopChartReorg
 */
public class AcopChartReorgSettingsCustomizer extends JPanel implements Customizer {

	private static final long serialVersionUID = 1L;
	
	protected static final String NO_STYLE = "No Style";
	
	protected AcopChartReorg acopChartReorg;
	
	protected JPanel scalePanel;
	private JButton autoscaleXButton;
	private JButton autoscaleYButton;
	private JCheckBox autoscaleBox;
	private JCheckBox logBox;
	private NumberField xRangeMinField;
	private NumberField xRangeMaxField;
	private NumberField yRangeMinField;
	private NumberField yRangeMaxField;
	protected JPanel stylePanel;
	private JCheckBox axisTraceBox;
	private JCheckBox chubbyLinesBox;
	private JLabel displayModeLabel;
	private JComboBox<Object> displayModeCombo;
	
	/**
	 * Constructs new AcopChartReorgSettingsCustomizer.
	 *
	 */
	public AcopChartReorgSettingsCustomizer() {
		initialize();
	}
	
	protected void initialize() {
		setLayout(new GridBagLayout());
		
		add(getScalePanel(),new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(1,1,1,1),0,0));
		add(getStylePanel(),new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(1,1,1,1),0,0));
		
	}
	
	protected JPanel getScalePanel() {
		if (scalePanel == null) {
			scalePanel = new JPanel();
			scalePanel.setBorder(new TitledBorder("Scale"));
			scalePanel.setLayout(new GridBagLayout());
			
			scalePanel.add(getAutoscaleBox(), new GridBagConstraints(0,0,2,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(getLogBox(), new GridBagConstraints(2,0,2,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(new JLabel("Horizontal Min:"), new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(getXRangeMinField(), new GridBagConstraints(1,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(new JLabel("Horizontal Max:"), new GridBagConstraints(2,1,1,1,0,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(getXRangeMaxField(), new GridBagConstraints(3,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(getAutoscaleXButton(), new GridBagConstraints(4,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(new JLabel("Vertical Min:"), new GridBagConstraints(0,2,1,1,0,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(getYRangeMinField(), new GridBagConstraints(1,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(new JLabel("Vertical Max:"), new GridBagConstraints(2,2,1,1,0,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(getYRangeMaxField(), new GridBagConstraints(3,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			scalePanel.add(getAutoscaleYButton(), new GridBagConstraints(4,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		}

		return scalePanel;
	}
	
	protected JButton getAutoscaleXButton() {
		if (autoscaleXButton == null) {
			autoscaleXButton = new JButton("Autoscale Horizontal");
			autoscaleXButton.setToolTipText("Autoscale Horizontal Axis");
			autoscaleXButton.setPreferredSize(new Dimension(180, 25));
			autoscaleXButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) acopChartReorg.autoScaleXOnce();					
				}});
		}

		return autoscaleXButton;
	}
	
	protected JButton getAutoscaleYButton() {
		if (autoscaleYButton == null) {
			autoscaleYButton = new JButton("Autoscale Vertical");
			autoscaleYButton.setToolTipText("Autoscale Vertical Axis");
			autoscaleYButton.setPreferredSize(new Dimension(180, 25));
			autoscaleYButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) acopChartReorg.autoScaleYOnce();
				}});
		}

		return autoscaleYButton;
	}
	
	protected JCheckBox getAutoscaleBox() {
		if (autoscaleBox == null) {
			autoscaleBox = new JCheckBox("Automatic Vertical Scaling");
			autoscaleBox.setToolTipText("Toggles Automatic Scaling of Vertical Axis");
			autoscaleBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (acopChartReorg != null) {
						acopChartReorg.setYAutoScale(autoscaleBox.isSelected());					
					}
				}
			});
		}

		return autoscaleBox;
	}
	
	protected JCheckBox getLogBox() {
		if (logBox == null) {
			logBox = new JCheckBox("LOG Vertical Scale");
			logBox.setToolTipText("Toggles Lin / Log Mode for Vertical Axis");
			logBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) acopChartReorg.setYLogScale(logBox.isSelected());
				}});
		}

		return logBox;
	}
	
	protected NumberField getXRangeMinField() {
		if (xRangeMinField == null) {
			xRangeMinField = new NumberField();
			xRangeMinField.setColumns(6);
			xRangeMinField.setNumberType(Double.class);
			xRangeMinField.addPropertyChangeListener(NumberField.VALUE, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					Number number = (Number) evt.getNewValue();
					if (acopChartReorg != null) acopChartReorg.setXRangeMin(number.doubleValue());
				}});
		}

		return xRangeMinField;
	}
	
	protected NumberField getXRangeMaxField() {
		if (xRangeMaxField == null) {
			xRangeMaxField = new NumberField();
			xRangeMaxField.setColumns(6);
			xRangeMaxField.setNumberType(Double.class);
			xRangeMaxField.addPropertyChangeListener(NumberField.VALUE, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					Number number = (Number) evt.getNewValue();
					if (acopChartReorg != null) {
						acopChartReorg.setXRangeMax(number.doubleValue());
					}
				}});
		}

		return xRangeMaxField;
	}
	
	protected NumberField getYRangeMinField() {
		if (yRangeMinField == null) {
			yRangeMinField = new NumberField();
			yRangeMinField.setColumns(6);
			yRangeMinField.setNumberType(Double.class);
			yRangeMinField.addPropertyChangeListener(NumberField.VALUE, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					Number number = (Number) evt.getNewValue();
					if (acopChartReorg != null && !acopChartReorg.isYAutoScale()) {
						acopChartReorg.setYRangeMin(number.doubleValue());
					}
				}});
		}

		return yRangeMinField;
	}
	
	protected NumberField getYRangeMaxField() {
		if (yRangeMaxField == null) {
			yRangeMaxField = new NumberField();
			yRangeMaxField.setColumns(6);
			yRangeMaxField.setNumberType(Double.class);
			yRangeMaxField.addPropertyChangeListener(NumberField.VALUE, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					Number number = (Number) evt.getNewValue();
					if (acopChartReorg != null && !acopChartReorg.isYAutoScale()){
						acopChartReorg.setYRangeMax(number.doubleValue());
					}
				}});
		}

		return yRangeMaxField;
	}
	
	protected JPanel getStylePanel() {
		if (stylePanel == null) {
			stylePanel = new JPanel();
			stylePanel.setBorder(new TitledBorder("Style"));
			stylePanel.setLayout(new GridBagLayout());
			
			JPanel lineStylePanel = new JPanel();
			lineStylePanel.setLayout(new GridBagLayout());
			lineStylePanel.add(getDisplayModeLabel(),new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			lineStylePanel.add(getDisplayModeCombo(),new GridBagConstraints(1,0,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			
			
			stylePanel.add(getAxisTraceBox(),new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			stylePanel.add(getChubbyLinesBox(),new GridBagConstraints(1,0,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			stylePanel.add(lineStylePanel,new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			
		}

		return stylePanel;
	}
	
	protected JCheckBox getAxisTraceBox() {
		if (axisTraceBox == null) {
			axisTraceBox = new JCheckBox("Axis Trace Visible");
			axisTraceBox.setToolTipText("Toggles Axis Trace On / Off");
			axisTraceBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) acopChartReorg.setAxisTraceVisible(axisTraceBox.isSelected());
				}});
		}

		return axisTraceBox;
	}
	
	protected JCheckBox getChubbyLinesBox() {
		if (chubbyLinesBox == null) {
			chubbyLinesBox = new JCheckBox("Chubby Lines");
			chubbyLinesBox.setToolTipText("Toggles Normal / Chubby Lines");
			chubbyLinesBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) acopChartReorg.setChubbyLines(chubbyLinesBox.isSelected());
				}});
		}

		return chubbyLinesBox;
	}
	
	protected JLabel getDisplayModeLabel() {
		if (displayModeLabel == null) {
			displayModeLabel = new JLabel("Display Mode");
		}
		return displayModeLabel;
	}
	
	protected JComboBox<Object> getDisplayModeCombo() {
		if (displayModeCombo == null) {
			displayModeCombo = new JComboBox<>();
			displayModeCombo.setToolTipText("Sets Chart Display Mode");
			displayModeCombo.addItem(NO_STYLE);
			
			AcopDisplayMode[] m= AcopDisplayMode.values();
			for (int i = 0; i < m.length; i++) {
				displayModeCombo.addItem(m[i]);
			}

			displayModeCombo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) {
						Object selected = displayModeCombo.getSelectedItem();
						if (selected instanceof AcopDisplayMode) {
							acopChartReorg.setDisplayMode((AcopDisplayMode) selected);
						}
						else acopChartReorg.setDisplayMode(null);
					}
				}});
		}

		return displayModeCombo;
	}

	/* (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		acopChartReorg = (AcopChartReorg) bean;
		addListener();
	}
	
	protected void addListener() {
		if (acopChartReorg == null) return;
		acopChartReorg.addPropertyChangeListener(new PropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent evt) {
				String propertyName = evt.getPropertyName();
				if (propertyName.equals(AcopChartReorg.LINE_STYLE)) {
					Object style = evt.getNewValue();
					if (style == null) getDisplayModeCombo().setSelectedItem(NO_STYLE);
					else getDisplayModeCombo().setSelectedItem(style);
				}
				else if (propertyName.equals(AcopChartReorg.CHUBBY_LINES)) {
					getChubbyLinesBox().setSelected((Boolean) evt.getNewValue());
				}
				else if (propertyName.equals(AcopChartReorg.AXIS_TRACE_VISIBLE)) {
					getAxisTraceBox().setSelected((Boolean) evt.getNewValue());
				}
				else if (propertyName.equals(AcopChartReorg.Y_AUTOSCALE)) {
					getAutoscaleBox().setSelected((Boolean) evt.getNewValue());
				}
				else if (propertyName.equals(AcopChartReorg.Y_LOG_SCALE)) {
					getLogBox().setSelected((Boolean) evt.getNewValue());
				}
				else if (propertyName.equals(AcopChartReorg.X_RANGE_MAX)) {
					getXRangeMaxField().setValue((Double) evt.getNewValue());
				}
				else if (propertyName.equals(AcopChartReorg.X_RANGE_MIN)) {
					getXRangeMinField().setValue((Double) evt.getNewValue());
				}
				else if (propertyName.equals(AcopChartReorg.Y_RANGE_MAX)) {
					getYRangeMaxField().setValue((Double) evt.getNewValue());
				}
				else if (propertyName.equals(AcopChartReorg.Y_RANGE_MIN)) {
					getYRangeMinField().setValue((Double) evt.getNewValue());
				}
				
			}});
	}

}
