/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.chart;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.beans.PropertyVetoException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.util.DisplayerParametersFlavor;
import com.cosylab.gui.util.MultipleDisplayerParametersFlavor;

import de.desy.acop.displayers.AcopChartReorg;
import de.desy.acop.displayers.AcopTrendChart;
import de.desy.acop.displayers.tools.AcopChartTransferHandler;
import de.desy.acop.displayers.tools.AcopDisplayerParametersFlavor;
import de.desy.acop.displayers.tools.AcopGraphHistoryParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.displayers.tools.AcopGraphParametersFlavor;
import de.desy.acop.displayers.tools.ConnectionParametersFlavor;
import de.desy.acop.transport.ConnectionParameters;


/**
 * <code>AcopChartReorgTransferHandler</code> is an extension of the
 * <code>TransferHandler</code> adapted to <code>AcopChartReorg</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopChartReorg
 */
public class AcopChartReorgTransferHandler extends AcopChartTransferHandler {

	private static final long serialVersionUID = -6825714785450126489L;
	
	private TrendSelectorDialog trendSelectorDialog;

	public AcopChartReorgTransferHandler(AcopChartReorg comp) {
		super(comp, true, true, new DataFlavor[] {AcopGraphParametersFlavor.FLAVOR, 
				AcopDisplayerParametersFlavor.FLAVOR,	MultipleDisplayerParametersFlavor.FLAVOR,
				DisplayerParametersFlavor.FLAVOR,ConnectionParametersFlavor.FLAVOR,
				DataFlavor.stringFlavor}, true);

	}
	
	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopTransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopGraphParametersFlavor.FLAVOR)) {
				Object drop= t.getTransferData(AcopGraphParametersFlavor.FLAVOR);
				
				if (!(drop instanceof AcopGraphParameters[])) {
					return false;
				}
				
				AcopGraphParameters[] p = (AcopGraphParameters[])drop;
				for (int i = 0; i < p.length; i++) {
					if (displayer instanceof AcopTrendChart && !(p[i] instanceof AcopGraphHistoryParameters && ((AcopGraphHistoryParameters)p[i]).getHistoryParameters() != null)) {
						HistoryParameters hp = getTrendSelectorDialog().chooseHistoryParameters(p[i].getConnectionParameters());
						p[i] = new AcopGraphHistoryParameters(p[i], hp);
					}
					((AcopChartReorg)displayer).addDisplayerParameters(p[i]);
				}
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (super.importData(comp,t)) {
			return true;
		}
		
		return false;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopChartTransferHandler#setParametersToDisplayer(java.lang.Object, com.cosylab.gui.displayers.DisplayerParameters)
	 */
	@Override
	public boolean setParametersToDisplayer(Object disp, DisplayerParameters parameters) throws PropertyVetoException {
		try {
			if (displayer instanceof AcopTrendChart && !(parameters instanceof AcopGraphHistoryParameters && ((AcopGraphHistoryParameters)parameters).getHistoryParameters() != null)) {
				ConnectionParameters cp = ((AcopGraphParameters)parameters).getConnectionParameters();
				HistoryParameters hp = getTrendSelectorDialog().chooseHistoryParameters(cp.deriveWithDeviceProperty(cp.getDeviceProperty() +".HIST"));
				parameters = new AcopGraphHistoryParameters((AcopGraphParameters)parameters, hp);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return super.setParametersToDisplayer(disp, parameters);
	}
	
	private TrendSelectorDialog getTrendSelectorDialog() {
		if (trendSelectorDialog == null) {
			trendSelectorDialog = new TrendSelectorDialog(displayer);
		}

		return trendSelectorDialog;
	}

}
