/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.chart;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.chart.AcopGraphStyleEnum;
import de.desy.acop.displayers.AcopChartReorg;
import de.desy.acop.displayers.AcopTrendChart;
import de.desy.acop.displayers.tools.RecentTimespanSelector;

/**
 * <code>AcopTrendChartSettingsCustomizer</code> is a customizer which allows
 * timespan and style setting of the <code>AcopTrendChart</code>.
 * 
 * @author Tilen Kusterle, Cosylab
 * @see AcopTrendChart
 */
public class AcopTrendChartSettingsCustomizer extends JPanel implements Customizer {

	private static final long serialVersionUID = 1L;
		
	protected AcopTrendChart acopChartReorg;
	
	private JButton autoscaleXButton;
	private JButton autoscaleYButton;
	protected JPanel stylePanel;
	private JCheckBox axisTraceBox;
	private ScalePanel scalePanel;
	private ChartOptionsPanel chartOptionsPanel;
	private JCheckBox allDataBox;
	private RecentTimespanSelector selector;
	private JPanel timespanPanel;
	
	/**
	 * Constructs a new AcopTrendCHartSettingsCustomizer.
	 *
	 */
	public AcopTrendChartSettingsCustomizer() {
		initialize();
	}
	
	protected void initialize() {
		setLayout(new GridBagLayout());
		
		add(getScalePanel(),new GridBagConstraints(0,0,1,1,0,1,GridBagConstraints.NORTHWEST,GridBagConstraints.VERTICAL,new Insets(1,1,1,1),0,0));
		add(getTimespanPanel(),new GridBagConstraints(1,0,1,1,1,1,GridBagConstraints.NORTHWEST,GridBagConstraints.VERTICAL,new Insets(1,1,1,1),0,0));
		add(getStylePanel(),new GridBagConstraints(0,1,2,1,0,1,GridBagConstraints.NORTHWEST,GridBagConstraints.VERTICAL,new Insets(1,1,1,1),0,0));
		
	}
	
	protected JPanel getTimespanPanel() {
		if (timespanPanel == null) {
			timespanPanel = new JPanel(new GridBagLayout());
			timespanPanel.add(getAllDataBox(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
			timespanPanel.add(getSelector(), new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(1,1,1,1),0,0));
			timespanPanel.setBorder(BorderFactory.createTitledBorder("Timespan"));
		}
		return timespanPanel;
	}
	
	protected RecentTimespanSelector getSelector() {
		if (selector == null) {
			selector = new RecentTimespanSelector();
			selector.addPropertyChangeListener(RecentTimespanSelector.TIME_SPAN, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (acopChartReorg == null) return;
					acopChartReorg.setTimeSpan((Double) evt.getNewValue());
				}});
		}
		return selector;
	}
	
	protected ScalePanel getScalePanel() {
		if (scalePanel == null) {
			scalePanel = new ScalePanel();
			scalePanel.setDefaultMax(150);
			scalePanel.setDefaultMin(0);
			scalePanel.setMax(150);
			scalePanel.setMin(0);
			scalePanel.setRelativeScaleMode(true);
			AcopGraphStyleEnum[] styles = new AcopGraphStyleEnum[]{AcopGraphStyleEnum.TimeLin, AcopGraphStyleEnum.TimeLog};
			scalePanel.setAvailableStyles(styles);
			scalePanel.setStyleComboRenderer(new StyleComboRenderer());
			scalePanel.addPropertyChangeListener(new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					if (acopChartReorg == null) return;
					String propertyName = evt.getPropertyName();
					if ("autoScale".equals(propertyName)) {
						scalePanel.setMinMaxEnabled(!scalePanel.isAutoScale());
						acopChartReorg.setYAutoScale(scalePanel.isAutoScale());
					} else if ("max".equals(propertyName)) {
						acopChartReorg.setYRangeMax(scalePanel.getMax());
					} else if ("min".equals(propertyName)) {
						acopChartReorg.setYRangeMin(scalePanel.getMin());
					} else if ("style".equals(propertyName)) {
						acopChartReorg.setYLogScale(scalePanel.getStyle() == AcopGraphStyleEnum.TimeLog);
					}
				}
			});		
			scalePanel.addActionListener(e -> {
                if (e.getActionCommand() == ScalePanel.AUTO_SCALE_ONCE_CMD) {
                    acopChartReorg.autoScale(false,false,true,true);
                }
            });
			scalePanel.setBorder(BorderFactory.createTitledBorder("Scale"));
		}

		return scalePanel;
	}
	
	protected ChartOptionsPanel getChartOptionsPanel() {
		if (chartOptionsPanel == null) {
			chartOptionsPanel = new ChartOptionsPanel();
			chartOptionsPanel.setBorder(null);
			chartOptionsPanel.addPropertyChangeListener(new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					if (acopChartReorg == null) return;
					String propertyName = evt.getPropertyName();
					if ("useBestScale".equals(propertyName)) {
						acopChartReorg.setBestScale(chartOptionsPanel.isUseBestScale());
					} else if ("showGrid".equals(propertyName)) {
						acopChartReorg.setShowGrid(chartOptionsPanel.isShowGrid());
					} else if ("lineDisplayMode".equals(propertyName)) {
						if (chartOptionsPanel.getLineDisplayMode() != null) {
							acopChartReorg.setDisplayMode(chartOptionsPanel.getLineDisplayMode());
						}
					} else if ("cursorMarkerVisible".equals(propertyName)) {			
						acopChartReorg.setChubbyLines(chartOptionsPanel.isCursorMarkerVisible());
					} 
				}
			});
			chartOptionsPanel.setShowGrid(true);
			chartOptionsPanel.setUseBestScale(true);
			chartOptionsPanel.setCursorMarkerVisibleView(false);
			
		}
		return chartOptionsPanel;
	}
	
	protected JButton getAutoscaleXButton() {
		if (autoscaleXButton == null) {
			autoscaleXButton = new JButton("Autoscale Horizontal");
			autoscaleXButton.setToolTipText("Autoscale Horizontal Axis");
			autoscaleXButton.setPreferredSize(new Dimension(180, 25));
			autoscaleXButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) {
						acopChartReorg.autoScaleXOnce();					
					}
				}});
		}

		return autoscaleXButton;
	}
	
	protected JButton getAutoscaleYButton() {
		if (autoscaleYButton == null) {
			autoscaleYButton = new JButton("Autoscale Vertical");
			autoscaleYButton.setToolTipText("Autoscale Vertical Axis");
			autoscaleYButton.setPreferredSize(new Dimension(180, 25));
			autoscaleYButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) {
						acopChartReorg.autoScaleYOnce();
					}
				}});
		}

		return autoscaleYButton;
	}
		
	protected JPanel getStylePanel() {
		if (stylePanel == null) {
			stylePanel = new JPanel();
			stylePanel.setBorder(new TitledBorder("Style"));
			stylePanel.setLayout(new GridBagLayout());
			stylePanel.add(getAxisTraceBox(),new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(0,4,0,4),0,0));
			stylePanel.add(getChartOptionsPanel(), new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
		}

		return stylePanel;
	}
	
	protected JCheckBox getAxisTraceBox() {
		if (axisTraceBox == null) {
			axisTraceBox = new JCheckBox("Axis Trace Visible");
			axisTraceBox.setToolTipText("Toggles Axis Trace On / Off");
			axisTraceBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) {
						acopChartReorg.setAxisTraceVisible(axisTraceBox.isSelected());
					}
				}});
		}

		return axisTraceBox;
	}
	
	/* (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		acopChartReorg = (AcopTrendChart) bean;
		addListener();
		getAllDataBox().setSelected(acopChartReorg.isAutoScaleToLiveData());
		selector.setEnabled(false);
	}
	
	protected void addListener() {
		if (acopChartReorg == null) return;
		acopChartReorg.addPropertyChangeListener(new PropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent evt) {
				String propertyName = evt.getPropertyName();
				if (propertyName.equals(AcopChartReorg.LINE_STYLE)) {
					Object style = evt.getNewValue();
					if (style == null) getChartOptionsPanel().setLineDisplayMode(AcopDisplayMode.PolyLine);
					else getChartOptionsPanel().setLineDisplayMode((AcopDisplayMode) style);
				} else if (propertyName.equals(AcopChartReorg.AXIS_TRACE_VISIBLE)) {
					getAxisTraceBox().setSelected((Boolean) evt.getNewValue());
				} else if (propertyName.equals(AcopChartReorg.Y_AUTOSCALE)) {
					getScalePanel().setAutoScale((Boolean) evt.getNewValue());
				} else if (propertyName.equals(AcopChartReorg.Y_LOG_SCALE)) {
					getScalePanel().setStyle((Boolean) evt.getNewValue() ? AcopGraphStyleEnum.TimeLog : AcopGraphStyleEnum.TimeLin);
				} else if (propertyName.equals(AcopChartReorg.Y_RANGE_MAX)) {
					getScalePanel().setMax((Double) evt.getNewValue(), false);
				} else if (propertyName.equals(AcopChartReorg.Y_RANGE_MIN)) {
					getScalePanel().setMin((Double) evt.getNewValue(), false);
				} else if (propertyName.equals("printText")) {
					getChartOptionsPanel().setTextSelected((Boolean)evt.getNewValue());
				} else if (propertyName.equals("bestScale")) {
					getChartOptionsPanel().setUseBestScale((Boolean)evt.getNewValue());
				} else if (propertyName.equals("showGrid")) {
					getChartOptionsPanel().setShowGrid((Boolean)evt.getNewValue());
				} else if (propertyName.equals(AcopChartReorg.X_RANGE_MIN) || propertyName.equals(AcopChartReorg.X_RANGE_MAX)) {
					selector.setTimeSpan(acopChartReorg.getXRangeMax()-acopChartReorg.getXRangeMin());
				}
				
			}});
	}
	
	private JCheckBox getAllDataBox() {
		if (allDataBox == null) {
			allDataBox = new JCheckBox("Autoscale to Live Data");
			allDataBox.setToolTipText("Adjusts displayed time span to include new live data");
			allDataBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopChartReorg != null) {
						acopChartReorg.setAutoScaleToLiveData(allDataBox.isSelected());
					}
					selector.setEnabled(!allDataBox.isSelected());
				}});
		}

		return allDataBox;
	}

}
