/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.chart;

import java.beans.PropertyVetoException;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.gui.displayers.MinimalDataConsumer;
import com.cosylab.util.CommonException;

/**
 * @author Tilen Kusterle, Cosylab
 *
 */
class HistoryDataConverter {
	
	private HistoryDataSource historyDataSource;
	private TransformedDataConsumer transformedDataConsumer;
	private Converter converter;
	private double[] trensformedData;
	
	private class HistoryDataSource extends DataSourceSupport {
		
		private static final long serialVersionUID = 1L;
		private DoubleSeqConsumer doubleSeqConsumer;
		
		@SuppressWarnings("unchecked")
		public HistoryDataSource() {
			super(new Class[]{DoubleSeqConsumer.class});
		}
		
		public void setConverter(Converter c) {
			if (converter != null) converter.removeAllConsumers();
			converter = c;
			if (converter == null || !(converter instanceof DoubleSeqConsumer)) {
				resetConverter();
				return;
			}
			try {
				doubleSeqConsumer = converter.getDataConsumer(DoubleSeqConsumer.class);
				converter.addConsumer(transformedDataConsumer);
				addConsumer(converter);
			} catch (PropertyVetoException e) {
				resetConverter();
				e.printStackTrace();
			}
		}
		
		public void resetConverter() {
			converter = null;
			try {
				doubleSeqConsumer = transformedDataConsumer;
				addConsumer(transformedDataConsumer);
			} catch (PropertyVetoException e) {
				e.printStackTrace();
			}
		}
		
		public void transformData(long timestamp, double[] data) {
			try {
				doubleSeqConsumer.updateValue(timestamp, data);
			} catch (CommonException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	private class TransformedDataConsumer extends MinimalDataConsumer implements DoubleSeqConsumer {
		@Override
        public void updateValue(long timestamp, double[] value) throws CommonException {
			trensformedData = value;
		}
	}
	
	public HistoryDataConverter() {
		historyDataSource = new HistoryDataSource();
		transformedDataConsumer = new TransformedDataConsumer();
		historyDataSource.resetConverter();
	}
	
	/**
	 * Clones and uses the cloned corverter for converting history data. The converter
	 * passed as argument must be an instance of </code>DoubleSeqConsumer</code>,
	 * otherwise no converter is used. 
	 * @param c the </code>Converter</code> to be cloned and used for converting history
	 * data
	 */
	public void useConverter(Converter c) {
		if (c == null) {
			historyDataSource.resetConverter();
			return;
		}
		try {
			historyDataSource.setConverter((Converter) c.clone());
		} catch (CloneNotSupportedException e) {
			historyDataSource.resetConverter();
			e.printStackTrace();
		}
	}
	
	/**
	 * Converts history data by using the </code>Converter</code> set by 
	 * </code>useConverter</code> method.
	 * @param inputData raw history data
	 * @return converted history data
	 */
	public double[] convertHistoryData(double[] inputData) {
		historyDataSource.transformData(System.currentTimeMillis(), inputData);
		return trensformedData;
	}
	
}
