/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.chart;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

import de.desy.acop.displayers.AcopChartReorg;
import de.desy.acop.displayers.AcopTrendChart;
import de.desy.acop.displayers.tools.RecentTimespanSelector;

/**
 * <code>RecentTimespanCustomizer</code> is a customizer which allows
 * selecting a timespan from now until some time in the past using the
 * {@link RecentTimespanSelector}.
 * 
 * @author Tilen Kusterle, Cosylab
 *
 */
public class RecentTimespanCustomizer extends JPanel implements Customizer {

	private static final long serialVersionUID = 1L;
	
	private AcopTrendChart acopTrendChart;
	private JCheckBox allDataBox;
	private RecentTimespanSelector selector;
	
	/**
	 * Constructs a new RecentTimespanCustomizer.
	 *
	 */
	public RecentTimespanCustomizer() {
		setLayout(new GridBagLayout());
		add(getAllDataBox(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		selector = new RecentTimespanSelector();
		add(selector, new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(1,1,1,1),0,0));
		
	}

	/* (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		acopTrendChart = (AcopTrendChart) bean;
		acopTrendChart.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				String propertyName = evt.getPropertyName();
				if (propertyName.equals(AcopChartReorg.X_RANGE_MIN) || propertyName.equals(AcopChartReorg.X_RANGE_MAX)) {
					selector.setTimeSpan(acopTrendChart.getXRangeMax()-acopTrendChart.getXRangeMin());
				}
			}});
		selector.addPropertyChangeListener(RecentTimespanSelector.TIME_SPAN, new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				acopTrendChart.setTimeSpan((Double) evt.getNewValue());
			}});
		getAllDataBox().setSelected(acopTrendChart.isAutoScaleToLiveData());
		selector.setEnabled(false);

	}
	
	private JCheckBox getAllDataBox() {
		if (allDataBox == null) {
			allDataBox = new JCheckBox("Autoscale to Live Data");
			allDataBox.setToolTipText("Adjusts displayed time span to include new live data");
			allDataBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (acopTrendChart != null) acopTrendChart.setAutoScaleToLiveData(allDataBox.isSelected());
					selector.setEnabled(!allDataBox.isSelected());
				}});
		}

		return allDataBox;
	}

}
