/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.chart;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.util.GUIUtilities;

import de.desy.acop.displayers.selector.AcopComboBoxModel;


/**
 * <code>TrendSelector</code> is a visual component which enables selecting
 * <code>HistoryParameters</code>. The component shows input fields
 * which allow one to enter a desired connection point which will be used
 * for history connections. 
 * 
 * @author Tilen Kusterle, Cosylab
 */
public class TrendSelector extends JPanel {

	public static void main(String[] args) {
		RunnerHelper.runComponent(new TrendSelector(), 440,400);
	}
	private static final long serialVersionUID = 1L;
	private static final String SUBSYSTEM = "HIST";
	
	private JLabel subsystemLabel;
	private JLabel contextLabel;
	private JLabel serverLabel;
	private JLabel propertyLabel;
	private JLabel deviceLabel;
	private JLabel indexLabel;
	
	private JLabel subsystemValueLabel;
	private JTextField contextField;
	private JTextField serverField;
	private JTextField propertyField;
	private JTextField deviceField;
	private JComboBox<Integer> indexCombo;
	
	private JCheckBox enableHistoryBox;
	private boolean historyEnabled = false;
	
	private IndexModel indexModel;
	
	private class IndexModel extends AcopComboBoxModel<Integer> {

		private static final long serialVersionUID = 1L;
		public IndexModel() {
			super(Integer.class);
		}
		
		public void setMaxIndex(int maxIndex) {
			removeAllElements();
			Integer[] values = new Integer[maxIndex+1];
			for (int i = 0; i <= maxIndex; i++) {
				values[i] = i;
			}
			setElements(values);
			if (values.length > 0) {
				setSelectedItem(values[0]);
			}
		}
	}
	
	/**
	 * Constructs new TrendSelector.
	 *
	 */
	public TrendSelector() {
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());
		
		add(getEnableHistoryBox(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0));
		add(getSubsystemLabel(), new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0));
		add(getSubsystemValueLabel(), new GridBagConstraints(1,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));
		add(getContextLabel(), new GridBagConstraints(0,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0));
		add(getContextField(), new GridBagConstraints(1,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));
		add(getServerLabel(), new GridBagConstraints(0,3,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0));
		add(getServerField(), new GridBagConstraints(1,3,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));
		add(getDeviceLabel(), new GridBagConstraints(0,4,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0));
		add(getDeviceField(), new GridBagConstraints(1,4,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));
		add(getPropertyLabel(), new GridBagConstraints(0,5,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0));
		add(getPropertyField(), new GridBagConstraints(1,5,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));
		add(getIndexLabel(), new GridBagConstraints(0,6,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,10,1,1),0,0));
		add(getIndexCombo(), new GridBagConstraints(1,6,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.HORIZONTAL,new Insets(1,1,1,10),0,0));
		
//		enableHistoryChoosing(false);
//		getEnableHistoryBox().setSelected(false);
		
	}
	
	private JCheckBox getEnableHistoryBox() {
		if (enableHistoryBox == null) {
			enableHistoryBox = new JCheckBox("History Enabled");
			enableHistoryBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					setHistoryEnabled(enableHistoryBox.isSelected());
				}});
			enableHistoryBox.setSelected(true);
		}

		return enableHistoryBox;
	}
	
	private JLabel getSubsystemLabel() {
		if (subsystemLabel == null) {
			subsystemLabel = new JLabel("Subsystem");
			GUIUtilities.setFixedSize(subsystemLabel,new Dimension(120,21));
		}

		return subsystemLabel;
	}
	
	private JLabel getSubsystemValueLabel() {
		if (subsystemValueLabel == null) {
			subsystemValueLabel = new JLabel(SUBSYSTEM);
			GUIUtilities.setFixedSize(subsystemValueLabel,new Dimension(120,21));
		}

		return subsystemValueLabel;
	}
	
	private JLabel getContextLabel() {
		if (contextLabel == null) {
			contextLabel = new JLabel("Context");
			GUIUtilities.setFixedSize(contextLabel,new Dimension(120,21));
		}

		return contextLabel;
	}
	
	private JTextField getContextField() {
		if (contextField == null) {
			contextField = new JTextField();
			GUIUtilities.setFixedSize(contextField,new Dimension(120,21));
		}

		return contextField;
	}
	
	private JLabel getServerLabel() {
		if (serverLabel == null) {
			serverLabel = new JLabel("Server");
			GUIUtilities.setFixedSize(serverLabel,new Dimension(120,21));
		}

		return serverLabel;
	}
	
	private JTextField getServerField() {
		if (serverField == null) {
			serverField = new JTextField();
			GUIUtilities.setFixedSize(serverField,new Dimension(120,21));
		}

		return serverField;
	}
	
	private JLabel getPropertyLabel() {
		if (propertyLabel == null) {
			propertyLabel = new JLabel("Property");
			GUIUtilities.setFixedSize(propertyLabel,new Dimension(120,21));
		}

		return propertyLabel;
	}
	
	private JTextField getPropertyField() {
		if (propertyField == null) {
			propertyField = new JTextField();
			GUIUtilities.setFixedSize(propertyField,new Dimension(120,21));
		}

		return propertyField;
	}
	
	private JLabel getDeviceLabel() {
		if (deviceLabel == null) {
			deviceLabel = new JLabel("Device");
			GUIUtilities.setFixedSize(deviceLabel,new Dimension(120,21));
		}

		return deviceLabel;
	}
	
	private JTextField getDeviceField() {
		if (deviceField == null) {
			deviceField = new JTextField();
			GUIUtilities.setFixedSize(deviceField,new Dimension(120,21));
		}

		return deviceField;
	}
	
	private JLabel getIndexLabel() {
		if (indexLabel == null) {
			indexLabel = new JLabel("Index");
			GUIUtilities.setFixedSize(indexLabel,new Dimension(120,21));
		}

		return indexLabel;
	}
	
	private JComboBox<Integer> getIndexCombo() {
		if (indexCombo == null) {
			indexCombo = new JComboBox<>();
			indexCombo.setModel(getIndexModel());
			GUIUtilities.setFixedSize(indexCombo,new Dimension(120,21));
		}

		return indexCombo;
	}
	
	private IndexModel getIndexModel() {
		if (indexModel == null) {
			indexModel = new IndexModel();
			indexModel.setElements(new Integer[]{Integer.valueOf(0)});
		}

		return indexModel;
	}
	
	private void enableHistoryChoosing(boolean enable) {
		getContextField().setEnabled(enable);
		getServerField().setEnabled(enable);
		getDeviceField().setEnabled(enable);
		getPropertyField().setEnabled(enable);
	}
	
	/**
	 * Returns the selected HistoryParameters.
	 * 
	 * @return the parameters
	 */
	public HistoryParameters getHistoryParameters() {
		if (!enableHistoryBox.isSelected()) return null;
		return new HistoryParameters(
				getContextField().getText(),
				getServerField().getText(),
				getDeviceField().getText(),
				getPropertyField().getText(),
				(Integer) getIndexCombo().getSelectedItem());
	}
	
	/**
	 * Sets the HistoryParameters.
	 * 
	 * @param parameters 
	 */
	public void setHistoryParameters(HistoryParameters parameters) {
		if (parameters == null) {
			setValues(null, null, null, null);
		}
		else setValues(
				parameters.getHistoryContext(),
				parameters.getHistoryServer(),
				parameters.getHistoryDevice(),
				parameters.getHistoryProperty());
	}
	
	/**
	 * Sets the given values to the entry fields.
	 * 
	 * @param context
	 * @param server
	 * @param device
	 * @param property
	 */
	public void setHistoryParameters(String context, String server, String device, String property) {
		setValues(context, server, device, property);
		getIndexModel().setMaxIndex(0);
		
	}
	
	private void setValues(String context, String server, String device, String property) {
		getContextField().setText(context == null ? "" : context);
		getServerField().setText(server == null ? "" : server);
		getDeviceField().setText(device == null ? "" : device);
		getPropertyField().setText(property == null ? "" : property);
	}
	
	/**
	 * Sets the selected index.
	 * 
	 * @param index
	 */
	public void setSelectedIndex(int index) {
		getIndexCombo().setSelectedItem(Integer.valueOf(index));
	}
	
	/**
	 * Sets the maximum value for the index selector.
	 * 
	 * @param max new maximum value
	 */
	public void setIndexMax(final int max) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				getIndexModel().setMaxIndex(max);
			}
		});	
	}
	
	/**
	 * Returns the selected index.
	 * 
	 * @return the index
	 */
	public int getSelectedIndex() {
		return getIndexCombo().getSelectedIndex();
	}

	/**
	 * Returns true if history selection is enabled.
	 * 
	 * @return the historyEnabled
	 */
	public boolean isHistoryEnabled() {
		return historyEnabled;
	}

	/**
	 * Enables/disables history selection.
	 * 
	 * @param historyEnabled the historyEnabled to set
	 */
	public void setHistoryEnabled(boolean historyEnabled) {
		this.historyEnabled = historyEnabled;
		getEnableHistoryBox().setSelected(historyEnabled);
		enableHistoryChoosing(historyEnabled);
	}
	
	
}
