/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.chart;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import de.desy.acop.displayers.selector.SelectorUtilities;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>TrendSelectorDialog</code> can be used to to select <code>HistoryParameters</code>.
 * User can supply <code>ConnectionParameters</code> which represent the default
 * selected parameters.
 * 
 * @author Tilen Kusterle, Cosylab
 *
 */
public class TrendSelectorDialog {
	
	private JDialog dialog;
	private TrendSelector trendSelector;
	private JButton applyButton;
	
	private boolean apply = false;
	
	public TrendSelectorDialog(Component owner) {
		dialog = new JDialog(JOptionPane.getFrameForComponent(owner), "Trend Settings", true);
		dialog.setSize(300, 400);
		JPanel contentPane = new JPanel(new GridBagLayout());
		trendSelector = new TrendSelector();
		applyButton = new JButton("Apply");
		applyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				apply = true;
				dialog.setVisible(false);
			}});
		contentPane.add(trendSelector, new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(1,1,1,1),0,0));
		contentPane.add(applyButton, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		dialog.setContentPane(contentPane);
	}
	
	/**
	 * Opens a selector dialog and sets the connection parameters as default
	 * selected history parameters.
	 *  
	 * @param cp default parameters
	 * @return selected history parameters
	 */
	public HistoryParameters chooseHistoryParameters(ConnectionParameters cp) {
		apply = false;
		trendSelector.setHistoryParameters(cp.getDeviceContext(), cp.getDeviceGroup(), cp.getDeviceName(), cp.getDeviceProperty());
		if (SelectorUtilities.isConnectionParametersValid(cp)) {
			trendSelector.setIndexMax(SelectorUtilities.getSequenceLength(cp)-1);
		}
		dialog.setLocationRelativeTo(dialog.getOwner());
		dialog.setVisible(true);
		if (apply) return trendSelector.getHistoryParameters();
		else return null;
	}

}
