/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.info;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import com.cosylab.gui.components.introspection.PropertiesTableModel;

import de.desy.acop.displayers.tools.AcopInfoDialog;

/**
 * <code>CharacteristicsPanel</code> shows characteristics of the 
 * channel specified by the ConnectionParameters. This panel can be used in
 * combination with <code>AcopInfoDialog</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopInfoDialog
 */
public class CharacteristicsPanel extends JPanel {
	
	private static final long serialVersionUID = 5967883467749353832L;
	private Map<String, Object> characteristics;
	private JTable table;
	private PropertiesTableModel model;
	private JScrollPane tableScroll;
	
	/**
	 * Constructs new CharacteristicsPanel.
	 *
	 */
	public CharacteristicsPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new BorderLayout());
		model = new PropertiesTableModel() {
			private static final long serialVersionUID = -8633216620068900661L;

			/*
			 * (non-Javadoc)
			 * @see com.cosylab.gui.components.introspection.PropertiesTableModel#getValueAt(int, int)
			 */
			@Override
			public synchronized Object getValueAt(int rowIndex, int columnIndex) {
				Object obj = super.getValueAt(rowIndex, columnIndex);
				if (obj instanceof Object[]) {
					return Arrays.toString((Object[])obj);
				}
				return obj;
			}
			
			/*
			 * (non-Javadoc)
			 * @see com.cosylab.gui.components.introspection.PropertiesTableModel#getColumnName(int)
			 */
			@Override
			public String getColumnName(int columnIndex)
			{	
				String name = super.getColumnName(columnIndex);
				
				if (name.equals("property")) {
					return "Characteristic";
				} else if (name.equals("value")) {
					return "Value";
				} else {
					return name;
				}
			}
		};
		model.setEditable(false);
		table = new JTable();
		table.setModel(model);
		table.setEnabled(false);
		tableScroll = new JScrollPane(table);
		this.add(tableScroll, BorderLayout.CENTER);
	}
	
	/**
	 * Sets the characteristics which will be shown in this panel.
	 * 
	 * @param ch map of all characteristics
	 */
	public void setCharacteristics(Map<String, Object> ch) {
		this.characteristics = ch;
		if (characteristics == null) {
			model.setPropertyNames(new String[]{});
			model.setPropertyTypes(new Class[]{});
			model.setPropertyValues(new Object[]{});
		} else {
			Iterator<String> it = characteristics.keySet().iterator();
			String key;
			Object object;
			ArrayList<String> names = new ArrayList<>();
			ArrayList<Class<?>> types = new ArrayList<>();
			ArrayList<Object> values = new ArrayList<>();
			while (it.hasNext()) {
				key = it.next();
				object = characteristics.get(key);
				if (object == null) continue;
				names.add(key);
				types.add(object.getClass());
				values.add(object);
			}
			int size = names.size();
			model.setPropertyNames(names.toArray(new String[size]));
			model.setPropertyTypes(types.toArray(new Class[size]));
			model.setPropertyValues(values.toArray(new Object[size]));
		}
		model.fireTableDataChanged();
	}
}
