/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.info;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.cosylab.gui.displayers.CommonDisplayer;
import com.cosylab.gui.util.GUIUtilities;

import de.desy.acop.displayers.tools.AcopInfoDialog;
import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.acop.transport.adapters.AdapterFactoryService;

/**
 * <code>ConnectionPanel</code> shows the connection points as specified
 * by the ConnectionParameter. This panel can be used in combination with
 * <code>AcopInfoDialog</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopInfoDialog
 */
public class ConnectionPanel extends JPanel {
	private static final long serialVersionUID = -8289192882139945420L;
	private JLabel protocolLabel;
	private JLabel protocol;
	private JLabel contextLabel;
	private JLabel context;
	private JLabel serverLabel;
	private JLabel server;
	private JLabel deviceLabel;
	private JLabel device;
	private JLabel propertyLabel;
	private JLabel property;
	private JLabel accessRateLabel;
	private JLabel accessRate;
	private JLabel accessModeLabel;
	private JLabel accessMode;
	private JLabel sizeLabel;
	private JLabel size;
	private JLabel infoLabel;
	private ConnectionParameters connectionParameters;
	
	/**
	 * Constructs new ConnectionPanel.
	 *
	 */
	public ConnectionPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new GridBagLayout());
		
		protocolLabel = new JLabel("Protocol:");
		GUIUtilities.setFixedSize(protocolLabel,new Dimension(120,21));
		add(protocolLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(11,11,1,1),0,0));
		
		protocol = new JLabel("N/A");
		GUIUtilities.setFixedSize(protocol,new Dimension(120,21));
		add(protocol, new GridBagConstraints(1,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(11,1,1,11),0,0));
					
		contextLabel = new JLabel("Context:");
		GUIUtilities.setFixedSize(contextLabel,new Dimension(120,21));
		add(contextLabel, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		
		context = new JLabel("N/A");
		GUIUtilities.setFixedSize(context,new Dimension(120,21));
		add(context, new GridBagConstraints(1,1,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,11),0,0));
		
		serverLabel = new JLabel("Server:");
		GUIUtilities.setFixedSize(serverLabel,new Dimension(120,21));
		add(serverLabel, new GridBagConstraints(0,2,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		
		server = new JLabel("N/A");
		GUIUtilities.setFixedSize(server,new Dimension(120,21));
		add(server, new GridBagConstraints(1,2,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,11),0,0));
		
		deviceLabel = new JLabel("Device:");
		GUIUtilities.setFixedSize(deviceLabel,new Dimension(120,21));
		add(deviceLabel, new GridBagConstraints(0,3,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		
		device = new JLabel("N/A");
		GUIUtilities.setFixedSize(device,new Dimension(120,21));
		add(device, new GridBagConstraints(1,3,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,11),0,0));
		
		propertyLabel = new JLabel("Property:");
		GUIUtilities.setFixedSize(propertyLabel,new Dimension(120,21));
		add(propertyLabel, new GridBagConstraints(0,4,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		
		property = new JLabel("N/A");
		GUIUtilities.setFixedSize(property,new Dimension(120,21));
		add(property, new GridBagConstraints(1,4,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,11),0,0));
		
		sizeLabel = new JLabel("Size:");
		GUIUtilities.setFixedSize(sizeLabel,new Dimension(120,21));
		add(sizeLabel, new GridBagConstraints(0,5,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		
		size = new JLabel("N/A");
		GUIUtilities.setFixedSize(size,new Dimension(120,21));
		add(size, new GridBagConstraints(1,5,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,11),0,0));

		accessRateLabel = new JLabel("Access Rate:");
		GUIUtilities.setFixedSize(accessRateLabel,new Dimension(120,21));
		add(accessRateLabel, new GridBagConstraints(0,6,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		
		accessRate = new JLabel("N/A");
		GUIUtilities.setFixedSize(accessRate,new Dimension(120,21));
		add(accessRate, new GridBagConstraints(1,6,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,11),0,0));
		
		accessModeLabel = new JLabel("Access Mode:");
		GUIUtilities.setFixedSize(accessModeLabel,new Dimension(120,21));
		add(accessModeLabel, new GridBagConstraints(0,7,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,11,1,1),0,0));
		
		accessMode = new JLabel("N/A");
		GUIUtilities.setFixedSize(accessMode,new Dimension(120,21));
		add(accessMode, new GridBagConstraints(1,7,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,11),0,0));
		
		infoLabel = new JLabel();
		GUIUtilities.setFixedSize(infoLabel,new Dimension(240,21));
		add(infoLabel, new GridBagConstraints(0,8,2,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4,11,11,11),0,0));
		
	}
	
	/**
	 * Sets new remote connection point to this panel.
	 * 
	 * @param p the parameters that specify the connection point
	 */
	public void setConnectionParameters(ConnectionParameters p) {
		this.connectionParameters = p;
		
		if (this.connectionParameters != null) {
			protocol.setText(connectionParameters.getAccessProtocol());
			context.setText(connectionParameters.getDeviceContext());
			server.setText(connectionParameters.getDeviceGroup());
			device.setText(connectionParameters.getDeviceName());
			property.setText(connectionParameters.getDeviceProperty());
			size.setText(String.valueOf(connectionParameters.getPropertySize()));
			accessRate.setText(String.valueOf(connectionParameters.getAccessRate()));
			accessMode.setText(String.valueOf(connectionParameters.getAccessMode()));
			infoLabel.setText(getInfo(connectionParameters));
		} else {
			protocol.setText("N/A");
			context.setText("N/A");
			server.setText("N/A");
			device.setText("N/A");
			property.setText("N/A");
			size.setText("N/A");
			accessRate.setText("N/A");
			accessMode.setText("N/A");
		}
	}
	
	/**
	 * Returns the ConnectionParameters associated with this panel.
	 * 
	 * @return the remote connection point
	 */
	public ConnectionParameters getConnectionParameters() {
		return connectionParameters;
	}
	
	private String getInfo(ConnectionParameters param) {
		try {
			return (String) AdapterFactoryService.getInstance().getAdapterFactory().getCharacteristics(param).get(CommonDisplayer.C_DESCRIPTION);
		} catch (ConnectionFailed e) {
			e.printStackTrace();
		}
		return "N/A";
	}
}
