/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;

import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>AcopTableParametersDialog</code> enables generation of 
 * <code>AcopTableParameters</code>. It uses the <code>TableCustomizerPanel</code>
 * to customize the parameters.
 * 
 * @author Jaka Bobnar, Cosylab
 * @see AcopTableParameters
 * @see TableCustomizerPanel
 *
 */
public class AcopTableParametersDialog extends AcopDisplayerParametersDialog<AcopTableParameters> {

	private static final long serialVersionUID = 4031445693248397678L;
	private TableCustomizerPanel tableCustomizerPanel;
	
	/**
	 * Constructs a new AcopTableParametersDialog.
	 * 
	 * @param comp thw owner of the dialog
	 */
	public AcopTableParametersDialog(Component comp) {
		super(comp);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#generateParameters()
	 */
	@Override
	protected AcopTableParameters generateParameters() {
	    return  new AcopTableParameters(getDisplayerParameters().getConnectionParameters(),
	    		getTableCustomizerPanel().getType(), getConverterPanel().getConverter(), getTableCustomizerPanel().getName());

	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#getCustomizerPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopTableParameters> getCustomizerPanel() {
		return getTableCustomizerPanel();
	}
	
	private TableCustomizerPanel getTableCustomizerPanel()
	{
		if (tableCustomizerPanel == null) {
			tableCustomizerPanel = new TableCustomizerPanel();
			tableCustomizerPanel.setParameters(null);
		}
		return tableCustomizerPanel;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#showChartDialog(java.awt.Component, de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	public AcopTableParameters showDialog(Component component, AcopDisplayerParameters adp) {
		this.getTableCustomizerPanel().setType(null);
		this.getTableCustomizerPanel().setName(null);
		return super.showDialog(component, adp);
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#showChartDialog(java.awt.Component, de.desy.acop.transport.ConnectionParameters)
	 */
	@Override
	public AcopTableParameters showDialog(Component component, ConnectionParameters cp) {
		this.getTableCustomizerPanel().setType(null);
		this.getTableCustomizerPanel().setName(null);
		return super.showDialog(component, cp);
	}
	
}
