/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import com.cosylab.gui.adapters.Converter;

import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;

/**
 * <code>ChartConnectionCustomizer</code> is a connection customizer 
 * for multiple displayers that handle <code>AcopGraphParameters</code> (eg. AcopChart).
 * 
 * @author Jaka Bobnar, Cosylab
 * @see AcopGraphParameters
 * @see MultipleAcopDisplayer
 *
 */
public class ChartConnectionCustomizer extends MultipleConnectionCustomizer<AcopGraphParameters>{
	
	private static final long serialVersionUID = 8873830017332674294L;
	private GraphCustomizerPanel graphCustomizerPanel;
	private SelectorGUI selector;
	
	protected class AcopGraphCellRenderer extends DefaultListCellRenderer {
		
		private static final long serialVersionUID = 7937185460891053942L;

		public AcopGraphCellRenderer() {
			super();
			setOpaque(true);
		}
		
		/*
		 * (non-Javadoc)
		 * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
		 */
		@Override
		public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			Component c = super.getListCellRendererComponent(list, value, index, isSelected,
					cellHasFocus);
			
			if (value instanceof AcopGraphParameters) {
				c.setForeground(((AcopGraphParameters)value).getColor());
				((JLabel)c).setText(String.valueOf(((AcopGraphParameters)value).getConnectionParameters()) + (((AcopGraphParameters)value).getConverter() == null ? "" : "; Converter"));
			}
			return c;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getListRenderer()
	 */
	@Override
	protected ListCellRenderer<Object> getListRenderer() {
		return new AcopGraphCellRenderer();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSelectorGUI()
	 */
	@Override
	protected SelectorGUI getSelectorGUI() {
		if (selector == null) {
			selector = super.getSelectorGUI();
			selector.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					if (e.isSelectionComplete() && SelectorUtilities.isChannel(selector.getConnectionParameters())) {
						getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.SimpleHistogram.ordinal());
					} else {
						getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.PolyLine.ordinal());
					}
				}
			});
		}
		
	    return super.getSelectorGUI();
    }
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#generateParameters()
	 */
	@Override
	protected AcopGraphParameters generateParameters() {
		Converter c=null;
		try {
			c= getConverterPanel().getConverter();
			if (c!=null) {
				c= (Converter)c.clone();
			}
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return new AcopGraphParameters(
				getSelectorGUI().getConnectionParameters(), 
				getGraphCustomizerPanel().getColor(),
				getGraphCustomizerPanel().getGraphFFT(),
				getGraphCustomizerPanel().getGraphStyle(),
				getGraphCustomizerPanel().getGraphMode(),
				getGraphCustomizerPanel().getGraphWidth(),
				getGraphCustomizerPanel().isTrend(),
				getGraphCustomizerPanel().getTrendLength(),
				c);
		
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSettingsPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopGraphParameters> getSettingsPanel() {
		return getGraphCustomizerPanel();
	}
	
	private GraphCustomizerPanel getGraphCustomizerPanel() {
		if (graphCustomizerPanel == null) {
			graphCustomizerPanel = new GraphCustomizerPanel();
		}
		return graphCustomizerPanel;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getDisplayerParameters()
	 */
	@Override
	public AcopGraphParameters[] getDisplayerParameters() {
		AcopGraphParameters[] p = new AcopGraphParameters[displayerParameters.size()];
    	return displayerParameters.toArray(p);
	}
	
	/**
	 * Constructs new ChartConnectionCustomizer.
	 *
	 */
	public ChartConnectionCustomizer() {
		super();
		setConverterTabEnabled(true);
	}
}
