/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Color;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditorSupport;

import com.cosylab.gui.adapters.ConverterUtilities;

import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>PropertyEditor</code> for <code>AcopGraphParameters</code> property type.
 * 
 * @author Jaka Bobnar, Cosylab
 * @see AcopGraphParameters
 */
public class ChartSelectorPropertyEditor extends PropertyEditorSupport {

	private ChartConnectionCustomizer editor;

	/**
     * Creates new instance of SelectorPropertyEditor.
     */
	public ChartSelectorPropertyEditor() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
	public Component getCustomEditor() {
		if (editor == null) {
			editor = new ChartConnectionCustomizer();
			editor.setDisplayerParameters((AcopGraphParameters[]) getValue());
			editor.addPropertyChangeListener(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,
			        new PropertyChangeListener() {
				        public void propertyChange(PropertyChangeEvent evt) {
					        setValueSilently(editor.getDisplayerParameters());
				        }
			        });
		}
		return editor;
	}

	/* 
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
	public String getJavaInitializationString() {
		AcopGraphParameters[] params = (AcopGraphParameters[]) getValue();
		StringBuffer buffer = new StringBuffer();
		buffer.append("new de.desy.acop.displayers.tools.AcopGraphParameters[]{");
		ConnectionParameters cp; 
		Color color;
		if (params != null) {
			for (AcopGraphParameters p : params) {
				cp = p.getConnectionParameters();
				color = p.getColor();
				buffer.append("new de.desy.acop.displayers.tools.AcopGraphParameters(");
				buffer.append("new de.desy.acop.transport.ConnectionParameters(\""
				        + cp.getRemoteName()
				        + "\",de.desy.acop.transport.AccessMode."
				        + cp.getAccessMode() + ", " + cp.getAccessRate() + "," + cp.getPropertySize() + "," + SelectorUtilities.getExtendedDataString(cp) + "),");
				if (color!=null) {
					buffer.append("new java.awt.Color("+color.getRGB() + "),");
				} else {
					buffer.append("null,");
				}
				buffer.append(p.getFFT() + "," + p.getDrawStyle() + ","+p.getMode()+","+ p.getWidth() + "," + p.isTrend() + "," + p.getTrendLength());
				if (p.getConverter() != null){
					buffer.append(", " + ConverterUtilities.getInitializationString(p.getConverter()));
				}
				else {
					buffer.append(", null");
				}
				buffer.append("),");
				
			}
		}
		return buffer.toString() + "}";
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
	public boolean supportsCustomEditor() {
		return true;
	}
	
//	/*
//	 * (non-Javadoc)
//	 * @see java.beans.PropertyEditorSupport#setAsText(java.lang.String)
//	 */
//	@Override
//	public void setAsText(String text) {
//		if (text == null || text.trim().equals("")) setValue(null);
//		String[] values = text.split(";");
//		AcopGraphParameters[] params = new AcopGraphParameters[values.length];
//		String[] temp;
//		String[] cp;
//		for (int i = 0; i < values.length; i++) {
//			temp = values[i].split(":");
//			cp = temp[0].split(",");
//			params[i] = new AcopGraphParameters(new ConnectionParameters(cp[0], AccessMode
//			        .valueOf(cp[1]), Integer.parseInt(cp[2])), new Color(Integer.parseInt(temp[1].trim())),Integer.parseInt(temp[2].trim()), Integer.parseInt(temp[3].trim()), Integer.parseInt(temp[4].trim()), Integer.parseInt(temp[5].trim()), Boolean.parseBoolean(temp[6].trim()));
//		}
//		setValue(params);
//	} 
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getAsText()
	 */
	@Override
	public String getAsText() {
		AcopGraphParameters[] params = (AcopGraphParameters[]) getValue();	
	    if (params == null || params.length == 0) return "";
	    StringBuffer buffer = new StringBuffer();
	    for (AcopGraphParameters cp : params) {
	    	buffer.append(cp.toString() + ";");
	    }
	    return buffer.toString();
	}

	
	private void setValueSilently(Object value) {
		super.setValue(value);
	}
	/*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
     */
	@Override
	public void setValue(Object value) {
		super.setValue(value);
		if (editor != null) {
			editor.setDisplayerParameters((AcopGraphParameters[]) value);
		}
	}
}

/* __oOo__ */
