/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.beans.Beans;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.adapters.ConverterCustomizer;
import com.cosylab.gui.components.util.IconHelper;
import com.cosylab.gui.util.GUIUtilities;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>MultipleConnectionCustomizer</code> is customizer panel
 * that enables selection of multiple <code>AcopDisplayerParameters</code>.
 * This component is intended to be used with <code>MultipleAcopDisplayer</code>s.
 * <p>
 * The component constis of the parameters selector and customization user
 * interface where the displayer parameters cen be constructed and then added
 * to the displayer. The left side of the panel shows the list of all 
 * displayer parameters on the displayer that this customizer belongs to.
 * </p>  
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 * @param <T> AcopDisplayerParameters that are being handled by this class
 * @see MultipleAcopDisplayer
 */
public abstract class MultipleConnectionCustomizer<T extends AcopDisplayerParameters> extends JPanel implements Customizer {


	private static final long serialVersionUID = 4596748128425127898L;
	
	private MultipleAcopDisplayer<T> disp;
	
	private SelectorGUI selectorGUI;
	private JPanel selectorPanel;
	private JList<T> selectionList;
	private DefaultListModel<T> listModel;
	private JScrollPane selectionScroll;
	private JLabel selectedLabel;
	private JButton removeButton;
	private JTabbedPane tabPane;
	private ConverterCustomizer converterCustomizer;
	
	private JButton addButton;
	private JButton applyButton;
	private JPanel converterP;
	
	protected T selectedParameters;
		
	protected List<T> displayerParameters = new ArrayList<>();
	
	private ExecutorService executor = Executors.newSingleThreadExecutor();

	/**
	 * Constructs a new MultipleConnectionCustomizer.
	 *
	 */
	public MultipleConnectionCustomizer() {
		super();
		initialize();
		
	}
	
	private void initialize() {
		this.setLayout(new GridBagLayout());
		this.add(getSelectorPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
		
	}
	
	private JPanel getSelectorPanel() {
		if (selectorPanel == null) {
			selectorPanel = new JPanel(new GridBagLayout());
			selectedLabel = new JLabel("Selected connections: ");
			GUIUtilities.setFixedSize(selectedLabel,new Dimension(200,21));
			selectorPanel.add(selectedLabel, new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,0,0,0), 0,0));		
			selectorPanel.add(getSelectionScroll(), new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0,0,0,0), 0,0));
			selectorPanel.add(getRemoveButton(), new GridBagConstraints(0,2,1,1,1,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10,0,0,0), 0,0));
			selectorPanel.add(getCustomizerTabbedPane(), new GridBagConstraints(1,0,1,2,0,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,11,0,0),0,0));
			if (Beans.isDesignTime()) {
				JPanel p = new JPanel(new GridBagLayout());
				p.add(getAddButton(), new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,0,0,5),0,0));
				p.add(getApplyButton() , new GridBagConstraints(1,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(00,5,0,0),0,0));
				selectorPanel.add(p, new GridBagConstraints(1,2,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10,11,0,0),0,0));
			} else {
				selectorPanel.add(getAddButton(), new GridBagConstraints(1,2,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10,0,0,0),0,0));
			}
		
		}
		return selectorPanel;
	}
	
	private JButton getApplyButton() {
		if (applyButton == null) {
			applyButton = new JButton("Apply");
			applyButton.setEnabled(false);
			applyButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (selectedParameters != null) {
						if (disp != null) {
		        			disp.removeDisplayerParameters(selectedParameters);
		        		} else {
	        				removeDisplayerParameters(selectedParameters);
		        		}
						T param = generateAndAddDisplayerParameters();
						getSelectionList().setSelectedValue(param, true);
		    		    getSettingsPanel().setParameters(param);
					}
				}
			});
		}
		return applyButton;
	}
	
	private JButton getRemoveButton() {
		if (removeButton == null) {
	        removeButton = new JButton("Remove", IconHelper.createIcon("icons/navigation/Minus16.gif"));
	        removeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
	        		List<T> selected = getSelectionList().getSelectedValuesList();
	        		if (disp != null) {
	        			for (T p: selected) {
	        				disp.removeDisplayerParameters(p);
	        			}
	        		} else {
	        			for (T p: selected) {
	        				removeDisplayerParameters(p);
	        			}
	        		}
	        	
	        	}
	        });
	        removeButton.setEnabled(false);
	        
        }

        return removeButton;
	}
	
	protected JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton("Add", IconHelper.createIcon("icons/navigation/Plus16.gif"));
			addButton.addActionListener(new ActionListener() {
	        	public void actionPerformed(ActionEvent e) {
	        		executor.execute(new Runnable(){
	        			@Override
	        			public void run() {
	        				T param = generateAndAddDisplayerParameters();
	    	        		getSettingsPanel().setParameters(param);	        				
	        			}
	        		});
	        	}
	        });
			addButton.setEnabled(false);
	        
        }

        return addButton;
	}
	
	private JScrollPane getSelectionScroll() {
		if (selectionScroll == null) {
	        selectionScroll = new JScrollPane(getSelectionList());
        }
        return selectionScroll;
    }
	
	protected ListCellRenderer<Object> getListRenderer() {
		return new DefaultListCellRenderer();
	}
	
	protected JList<T> getSelectionList() {
		if (selectionList == null) {
	        selectionList = new JList<>();
	        selectionList.setPreferredSize(new Dimension(200,200));
	        selectionList.setModel(getModel());
	        selectionList.setCellRenderer(getListRenderer());
	        selectionList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
	        		if (getSelectionList().getSelectedIndex() >= 0) {
	        			getRemoveButton().setEnabled(true);
	        		} else {
	        			getRemoveButton().setEnabled(false);
	        		}
	        		
	        		int sel = selectionList.getSelectedIndex();
	        		if (sel >= 0) {
	        			T parameters = getModel().getElementAt(sel);
	        			setSelectedParameters(parameters);
	        			getSelectorGUI().setConnectionParameters(parameters.getConnectionParameters());
	        			getSettingsPanel().setParameters(parameters);
	        		} else {
	        			setSelectedParameters(null);
	        		}
	        	}
	        });
	        selectionList.addKeyListener(new KeyAdapter() {
				@Override
	        	public void keyPressed(KeyEvent e) {
	        		if (e.getKeyCode() == KeyEvent.VK_DELETE) {
	        			List<T> selected = selectionList.getSelectedValuesList();
	        			if (disp != null) {
		        			for (T p: selected) {
		        				disp.removeDisplayerParameters(p);
		        			}
		        		} else {
		        			for (T p: selected) {
		        				removeDisplayerParameters(p);
		        			}
		        		}
	        		}
	        	}
	        });
        }
		return selectionList;
    }
	
	private DefaultListModel<T> getModel() {
		if (listModel == null) {
	        listModel = new DefaultListModel<>();
        }

        return listModel;
	}
	

	protected JTabbedPane getCustomizerTabbedPane() {
		if (tabPane == null) {
			tabPane = new JTabbedPane();
			JPanel selectorP = new JPanel(new GridBagLayout());
			selectorP.add(getSelectorGUI(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
			JPanel customizerP = new JPanel(new GridBagLayout());
			customizerP.add(getSettingsPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
			converterP = new JPanel(new GridBagLayout());
			converterP.add(getConverterPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
			tabPane.addTab("Connection", selectorP);
			tabPane.addTab("Style", customizerP);
			tabPane.addTab("Converter", converterP);
		}
		
		return tabPane;
	}
	
	protected ConverterCustomizer getConverterPanel() {
		if (converterCustomizer == null) {
			converterCustomizer = new ConverterCustomizer();
			converterCustomizer.setPreferredSize(new Dimension(200,200));
		}

		return converterCustomizer;
	}

	protected abstract MultipleDisplayerAbstractSettingsPanel<T> getSettingsPanel();
		
	protected SelectorGUI getSelectorGUI() {
		if (selectorGUI == null) {
			selectorGUI = new SelectorGUI();
			selectorGUI.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					getAddButton().setEnabled(SelectorUtilities.isConnectionParametersValid(selectorGUI.getConnectionParameters()));
				}
			});
			selectorGUI.addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent e) {
					getAddButton().setEnabled(SelectorUtilities.isConnectionParametersValid(selectorGUI.getConnectionParameters()));
				}
			});
	    }
	    return selectorGUI;
    }
	
	protected abstract T generateParameters();
	
	private T generateAndAddDisplayerParameters() {
		T param = generateParameters();
		addDisplayerParameters(param);
		return param;
	}
	
	private void addDisplayerParameters(T agp) {
		if (agp == null || displayerParameters.contains(agp)) return;
		T[] old = getDisplayerParameters();
		getModel().addElement(agp);
		displayerParameters.add(agp);
				
	    if (disp != null) {
		    try {
                disp.addDisplayerParameters(agp);
            } catch (CommonException e1) {
            	getSelectorGUI().handleException(e1);
            } catch (PropertyVetoException e1) {
            	getSelectorGUI().handleException(e1);
            }
	    }
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,old,getDisplayerParameters());
	}
	
	private void removeDisplayerParameters(T agp) {
		if (agp == null || !displayerParameters.contains(agp)) return;
		T[] old = getDisplayerParameters();
		getModel().removeElement(agp);
		displayerParameters.remove(agp);
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,old,getDisplayerParameters());
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	public void setObject(Object bean) {
		disp = (MultipleAcopDisplayer<T>)bean;
		getSelectorGUI().setObject(bean);
		setDisplayerParameters(disp.getDisplayerParameters());
		disp.addPropertyChangeListener(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, new PropertyChangeListener(){

			public void propertyChange(PropertyChangeEvent evt) {
				if (!MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) 
					return;
				setDisplayerParameters(disp.getDisplayerParameters());
			}
		});     
    }

	/**
	 * Returns an array of displayer parameters containing all displayer parameters.
	 * (use displayerParameters vector and cast elements to T)
     * @return displayerParameters
     */
    public abstract T[] getDisplayerParameters();

	/**
	 * Sets displayer parameters to this customizer. This parameters will be displayed
	 * as parameters set on the displayer and will be listed in the left list.
	 * 
     * @param cps selected parameters
     */
    public void setDisplayerParameters(T[] parameters) {
    	if (parameters == null) return;
		T[] old = getDisplayerParameters();
		ArrayList<T> toBeRemoved = new ArrayList<>();
		
		if (displayerParameters != null) {
			for(T cp : displayerParameters) {
				toBeRemoved.add(cp);
			}
		}
		
		ArrayList<T> toBeAdded = new ArrayList<>();
	
		for (T p : parameters) {
			if (toBeRemoved.contains(p)) {
				toBeRemoved.remove(p);
			} else {
				toBeAdded.add(p);
			}
		}
		
		for (T p : toBeRemoved) {
			getModel().removeElement(p);
			displayerParameters.remove(p);
		}
		
		for (T p : toBeAdded) {
			if (!displayerParameters.contains(p)) {
				getModel().addElement(p);
				displayerParameters.add(p);
			}
		}
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,old,getDisplayerParameters());
		
    }
    
    protected void setSelectedParameters(T selected) {
    	this.selectedParameters = selected;
    	if (this.selectedParameters != null || SelectorUtilities.isConnectionParametersValid(getSelectorGUI().getConnectionParameters())) {
    		getApplyButton().setEnabled(true);
    		getAddButton().setEnabled(true);
    		if (selectedParameters != null && selectedParameters.getConverter()!=null) {
    			Converter c = selectedParameters.getConverter();
				try {
					c= (Converter)c.clone();
				} catch (CloneNotSupportedException e) {
					e.printStackTrace();
				}
    			getConverterPanel().setConverter(c);
    		} else {
    			getConverterPanel().setConverter(null);
    		}
    	} else {
    		getApplyButton().setEnabled(false);
    		getAddButton().setEnabled(false);
    		getConverterPanel().setConverter(null);
    	} 		
    }
    
    /**
     * Enables/disables the converter customizer tab.
     * 
     * @param enabled true if converter tab should be enabled
     */
    public void setConverterTabEnabled(boolean enabled){
    	for (int i = 0; i < tabPane.getComponentCount(); i++) if (tabPane != null && tabPane.getComponentAt(i).equals(converterP)){
    		tabPane.setEnabledAt(i, enabled);
    		break;
    	}
    }
    
    /**
     * This method will select the given ConnectionParameters in the combo box browsers.
     * 
     * @param p
     */
    public void applyConnectionParameters(ConnectionParameters p) {
    	getSelectorGUI().setConnectionParameters(p);
    }
    
    protected MultipleAcopDisplayer<T> getDisplayer() {
    	return disp;
    }
    
}


/* __oOo__ */
