/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.beans.Beans;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.swing.AbstractListModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.cosylab.gui.components.NumberField;
import com.cosylab.gui.components.util.IconHelper;
import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.util.DisplayerParametersSelectorDialog;
import com.cosylab.gui.util.GUIUtilities;
import com.cosylab.util.ListenerList;

import de.desy.acop.displayers.tools.ConnectionParametersReceiver;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.AccessMode;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>SelectorListGUI</code> is an instance of SelectorGUI which
 * instead of combo boxes uses lists displaying multiple values at the
 * same time.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see SelectorGUI
 */
public class SelectorListGUI extends JPanel implements Customizer {

	private static final long serialVersionUID = -5435798671352845158L;

	private static class AcopListModel extends AbstractListModel<String> implements ListSelectionModel {
		private static final long serialVersionUID = 399329824023675134L;
		protected String[] elements= new String[0];
		protected int selectedIndex = -1;
		private ListenerList selectionListeners = new ListenerList(ListSelectionListener.class);

	    /**
	     * Constructs an empty DefaultComboBoxModel object.
	     */
	    public AcopListModel() {
	    	
	    }
	    
	    @Override
	    public int getSize() {
	        return elements.length;
	    }

	    @Override
	    public String getElementAt(int index) {
	        if ( index >= 0 && index < elements.length)
	            return elements[index];
	        else
	            return null;
	    }
	    
	    public void setElements(String[] obj) {
	    	if (obj==elements || (obj==null && elements==null)) {
	    		return;
	    	}
	    	removeAllElements();
	    	if (obj != null) {
	    		elements=obj;
		    	fireIntervalAdded(this,0, obj.length-1);
		    	if (obj.length > 0) {
		    		
		    	}
	    	}
	    }
	    
	    public void reinstateLeadSelectedIndex() {
	    	if (selectedIndex < 0) {
    			setLeadSelectionIndex(0);
    		}
	    }
	    
	    /**
	     * Empties the list.
	     */
	    protected void removeAllElements() {
	        if ( elements.length > 0 ) {
	            int firstIndex = 0;
	            int lastIndex = elements.length - 1;
	            elements=new String[0];
	            fireIntervalRemoved(this, firstIndex, lastIndex);
	        }
	    }		
	    
	    public int getLeadSelectionIndex() {
			return selectedIndex;
		}

	    public void setLeadSelectionIndex(int index) {
	    	if (index == selectedIndex) return;
	    	int lower = Math.min(selectedIndex, index);
	    	int higher = Math.max(selectedIndex, index);
	    	selectedIndex = index;
	    	fireSelectionChanged(lower, higher);
	    }
	    
	    public void setSelectedValue(Object o) {
	    	for (int i = 0; i < elements.length; i++) {
	    		if (elements[i].equals(o)) { 
	    			setLeadSelectionIndex(i);
	    			return;
	    		}
	    	}
	    }
	    
	    public void clearSelection() {
	    	setLeadSelectionIndex(-1);
	    }

	    public boolean isSelectionEmpty() {
	    	return selectedIndex < 0;
	    }

		public void addSelectionInterval(int index0, int index1) {
			setLeadSelectionIndex(index0);
        }

		public int getAnchorSelectionIndex() {
	        return selectedIndex;
        }

		public int getMaxSelectionIndex() {
	        return selectedIndex;
        }

		public int getMinSelectionIndex() {
	        return selectedIndex;
        }

		public int getSelectionMode() {
	        return SINGLE_SELECTION;
	        
        }

		public boolean getValueIsAdjusting() {
	        return false;
        }

		public void insertIndexInterval(int index, int length, boolean before) {
			//
        }

		public boolean isSelectedIndex(int index) {
	        return index == selectedIndex;
        }

		public void removeIndexInterval(int index0, int index1) {
			setLeadSelectionIndex(-1);
        }

		public void removeSelectionInterval(int index0, int index1) {
			setLeadSelectionIndex(-1);
        }

		public void setAnchorSelectionIndex(int index) {
	        setLeadSelectionIndex(index);
        }

		public void setSelectionInterval(int index0, int index1) {
			setLeadSelectionIndex(index0);
        }

		public void setSelectionMode(int selectionMode) {
	        // 	        
        }

		public void setValueIsAdjusting(boolean valueIsAdjusting) {
	        // 	        
        }

		public void addListSelectionListener(ListSelectionListener x) {
			selectionListeners.add(x);
        }

		public void removeListSelectionListener(ListSelectionListener x) {
	        selectionListeners.remove(x);
        }
		
		protected void fireSelectionChanged(int firstIndex, int lastIndex) {
			if (firstIndex < 0) firstIndex = 0;
			if (lastIndex < 0) lastIndex = 0;
			ListSelectionListener[] listeners = (ListSelectionListener[]) selectionListeners.toArray();
			ListSelectionEvent e = new ListSelectionEvent(this, firstIndex, lastIndex, false);

			for (ListSelectionListener l : listeners) {
				l.valueChanged(e);
			}
			
		}
		
	}
	
	private Object disp;
	private boolean initialized = false;
	
	private JComboBox<String> accessProtocolCombo;
	private JList<String> deviceContextList;
	private JList<String> subSystemList;
	private JList<String> deviceServerList;
	private JList<String> deviceNameList;
	private JList<String> propertyNameList;
	private JScrollPane deviceContextScroll;
	private JScrollPane subSystemScroll;
	private JScrollPane deviceServerScroll;
	private JScrollPane deviceNameScroll;
	private JScrollPane propertyNameScroll;
	private JPanel listPanel;
	private JScrollPane listPanelScroll; 
	private AcopComboBoxModel<String> accessProtocolModel;
	private AcopListModel deviceContextModel;
	private AcopListModel subSystemModel;
	private AcopListModel deviceServerModel;
	private AcopListModel deviceNameModel;
	private AcopListModel propertyNameModel;
	private NumberField accessRateField;
	private JLabel accessProtocolLabel;
	private JLabel deviceContextLabel;
	private JLabel subSystemLabel;
	private JLabel deviceServerLabel;
	private JLabel deviceNameLabel;
	private JLabel propertyNameLabel;
	private JLabel accessRateLabel;
	private JLabel selectionLabel;
	private JLabel descriptionLabel;
	private JLabel accessModeLabel;
	private JComboBox<AccessMode> accessModeCombo;
	private JCheckBox showStockProperties;
	
	private ConnectionParameters connectionParameters;
	private volatile Selector selector; 
	
	private ListenerList selectorGUIListeners = new ListenerList(SelectorGUIListener.class);
	
	private PropertyChangeListener selectorPropertyListener;
	private QueryListener selectorQueryListener;
	private boolean settingFromSelector = false;
	
	private Object selectedContext;
	private Object selectedServer;
	private Object selectedDevice;
	private Object selectedProperty;
	
	/**
	 * Constructs a new SelectorListGUI.
	 *
	 */
	public SelectorListGUI() {
		super();
		initialize();
	}
	
	private void initialize() {
		accessProtocolModel= new AcopComboBoxModel<>(String.class);
		deviceContextModel= new AcopListModel();
		subSystemModel = new AcopListModel();
		deviceServerModel= new AcopListModel();
		deviceNameModel= new AcopListModel();
		propertyNameModel = new AcopListModel();
		setSelector(new Selector());
		
		setLayout(new GridBagLayout());
		setFocusable(false);
		accessProtocolLabel = new JLabel("Access protocol");
		GUIUtilities.setFixedSize(accessProtocolLabel,new Dimension(120,21));
		accessProtocolLabel.setFocusable(false);
		add(accessProtocolLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,3,1,1), 0,0));
		add(getAccessProtocolCombo(), new GridBagConstraints(1,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
		add(getShowStockPropertiesCheckBox(), new GridBagConstraints(2,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
		
		add(getListPanelScroll(), new GridBagConstraints(0,1,3,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,1,1,1), 0,0));
		
		accessRateLabel = new JLabel("Access rate");
		GUIUtilities.setFixedSize(accessRateLabel,new Dimension(120,21));
		accessRateLabel.setFocusable(false);
		add(accessRateLabel, new GridBagConstraints(0,4,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,3,1,1), 0,0));
		add(getAccessRateField(), new GridBagConstraints(1,4,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
		
		accessModeLabel = new JLabel("Access mode");
		GUIUtilities.setFixedSize(accessModeLabel,new Dimension(120,21));
		accessModeLabel.setFocusable(false);
		add(accessModeLabel, new GridBagConstraints(0,5,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,3,1,1), 0,0));
		add(getAccessModeCombo(), new GridBagConstraints(1,5,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,1,1,1), 0,0));
					
		add(getSelectionLabel(), new GridBagConstraints(0,6,3,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,3,1,1),0,0));
		
		if (!Beans.isDesignTime()) {
			add(getDescriptionLabel(), new GridBagConstraints(0,7,3,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,3,1,1),0,0));
		}
		
		setCombosEnabled(true);

		
		initialized = true;
	}
	
	private void applySettings() {
		
		try {
			String remoteName = getSelector().getRemoteName();
			ConnectionParameters old = connectionParameters;
			int accessRate = getAccessRateField().getValue().intValue();
			connectionParameters = new ConnectionParameters(remoteName,AccessMode.POLL, accessRate);
			firePropertyChange(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY,old,getConnectionParameters());
			getListPanel().revalidate();
        } catch (Exception e) {
            e.printStackTrace();
        } 
	}
	
	private JPanel getListPanel() {
		if (listPanel == null) {
			listPanel = new JPanel(new GridBagLayout());
			listPanel.setFocusCycleRoot(true);
			
			JPanel labelPanel = new JPanel(new GridLayout(1,5));
			JPanel lists = new JPanel(new GridLayout(1,5));
			deviceContextLabel = new JLabel("Device context");
			GUIUtilities.setFixedSize(deviceContextLabel,new Dimension(120,21));
			deviceContextLabel.setFocusable(false);
			labelPanel.add(deviceContextLabel);
			deviceContextScroll = new JScrollPane(getDeviceContextList());
			deviceContextScroll.setFocusable(false);
			lists.add(deviceContextScroll);
			
			subSystemLabel = new JLabel("Device subsystem");
			GUIUtilities.setFixedSize(subSystemLabel,new Dimension(120,21));
			subSystemLabel.setFocusable(false);
			labelPanel.add(subSystemLabel);
			subSystemScroll = new JScrollPane(getSubSystemList());
			subSystemScroll.setFocusable(false);
			lists.add(subSystemScroll);
						
			deviceServerLabel = new JLabel("Device server");
			GUIUtilities.setFixedSize(deviceServerLabel,new Dimension(120,21));
			deviceServerLabel.setFocusable(false);
			labelPanel.add(deviceServerLabel);
			deviceServerScroll = new JScrollPane(getDeviceServerList());
			deviceServerScroll.setFocusable(false);
			lists.add(deviceServerScroll);
			
			deviceNameLabel = new JLabel("Device name");
			GUIUtilities.setFixedSize(deviceNameLabel,new Dimension(120,21));
			deviceNameLabel.setFocusable(false);
			labelPanel.add(deviceNameLabel);
			deviceNameScroll = new JScrollPane(getDeviceNameList());
			deviceNameScroll.setFocusable(false);
			lists.add(deviceNameScroll);
			
			propertyNameLabel = new JLabel("Property name");
			GUIUtilities.setFixedSize(propertyNameLabel,new Dimension(120,21));
			propertyNameLabel.setFocusable(false);
			labelPanel.add(propertyNameLabel);
			propertyNameScroll = new JScrollPane(getPropertyNameList());
			propertyNameScroll.setFocusable(false);
			lists.add(propertyNameScroll);
			
			listPanel.add(labelPanel, new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0,0,0,0),1,1));
			listPanel.add(lists, new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.NORTH, GridBagConstraints.BOTH, new Insets(0,0,0,0),1,1));
			
		}
		
		GUIUtilities.setFixedSize(listPanel,new Dimension(765,100));
		return listPanel;
	}
	
	private JComboBox<AccessMode> getAccessModeCombo() {
		if (accessModeCombo == null) {
			accessModeCombo = new JComboBox<>(AccessMode.values());
			accessModeCombo.removeItem(AccessMode.WRITE);
			accessModeCombo.removeItem(AccessMode.WRITE_CONNECT);
			accessModeCombo.setSelectedItem(AccessMode.POLL);
		    GUIUtilities.setFixedSize(accessModeCombo,new Dimension(120, 21));
			accessModeCombo.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					if (settingFromSelector) return;
					refreshSelectionLabel();
					getSelector().setAccessMode((AccessMode)accessModeCombo.getSelectedItem());
				}
			});
		}
		return accessModeCombo;
	}
	
	private JScrollPane getListPanelScroll() {
		if (listPanelScroll == null) {
	        listPanelScroll = new JScrollPane(getListPanel());
			listPanelScroll.setFocusable(false);
			listPanelScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        }

        return listPanelScroll;
	}
	
	private JLabel getSelectionLabel() {
		if (selectionLabel == null) {
			selectionLabel = new JLabel();
			selectionLabel.setFocusable(false);
			selectionLabel.setIcon(IconHelper.createIcon("icons/development/ApplicationDeploy16.gif"));
			selectionLabel.addMouseMotionListener(new MouseMotionAdapter() {
				
				@Override
				public void mouseDragged(MouseEvent e) {
					applySettings();
					selectionLabel.getTransferHandler().exportAsDrag(selectionLabel,e,TransferHandler.COPY);
				}
			
			});
					
			selectionLabel.setTransferHandler(new TransferHandler() {

				private static final long serialVersionUID = 1658117801619157639L;
				private DisplayerParametersSelectorDialog dialog;
				@Override
				public boolean importData(JComponent comp, Transferable t) {
					try {
						if (t.isDataFlavorSupported(DataFlavor.stringFlavor)) {
							if (getConnectionParameters() == null) return false;
							String input = t.getTransferData(DataFlavor.stringFlavor).toString();
							StringTokenizer st = new StringTokenizer(input, "\n");
							ArrayList<ConnectionParameters> al = new ArrayList<>(0);
							while (st.hasMoreTokens()) {
								al.add(new ConnectionParameters(
										st.nextToken(),	AccessMode.POLL,
										1000));
							}
							ConnectionParameters[] cp = al.toArray(new ConnectionParameters[]{});
							int idx = selectByDialog(getSelectionLabel(), cp);
							if (idx < 0) return false;
							setConnectionParameters(cp[idx]);
							return true;
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
					return false;
				}

				@Override
				protected Transferable createTransferable(JComponent c) {
					return new Transferable() {
					
						public Object getTransferData(DataFlavor flavor)
								throws UnsupportedFlavorException, IOException {
							if (flavor.isFlavorTextType()) {
								return getConnectionParameters().getRemoteName();
							}
							throw new UnsupportedFlavorException(flavor);
						}
					
						public boolean isDataFlavorSupported(DataFlavor flavor) {
							return flavor.isFlavorTextType();
						}
					
						public DataFlavor[] getTransferDataFlavors() {
							return new DataFlavor[]{DataFlavor.stringFlavor};
						}
					};
				}

				@Override
				public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
					for (int i = 0; i < transferFlavors.length; i++) {
						if (transferFlavors[i].isFlavorTextType()) {
							return true;
						}
					}
					return false;
				}
				@Override
				public int getSourceActions(JComponent c) {
					return TransferHandler.COPY;
				}
				
				protected int selectByDialog(Component c, ConnectionParameters[] dp) {
					if (dialog == null) {
						dialog = new DisplayerParametersSelectorDialog(c, "Adding Source", "Select Source:");
					}
					String[] s= new String[dp.length];
					for (int i = 0; i < s.length; i++) {
						s[i]=dp[i].getRemoteName();
					}
					return dialog.showSelectionDialog(c,s);
				}
			});
			
			refreshSelectionLabel();
		}
		return selectionLabel;
	}
	
	private JLabel getDescriptionLabel() {
		if (descriptionLabel == null) {
	        descriptionLabel = new JLabel();
	        descriptionLabel.setFocusable(false);
        }

        return descriptionLabel;
	}
	
	private JComboBox<String> getAccessProtocolCombo() {
		if (accessProtocolCombo == null) {
			accessProtocolCombo = new JComboBox<>();
			accessProtocolCombo.setModel(accessProtocolModel);
			accessProtocolModel.setElements(new String[]{Selector.PROTOCOL_TINE});
			accessProtocolModel.setSelectedItem(null);
			GUIUtilities.setFixedSize(accessProtocolCombo,new Dimension(120, 21));
			accessProtocolCombo.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					if (settingFromSelector) return;
					getExecutorService().execute(new Runnable(){
						public void run() {
							getSelector().setSelectedProtocol(accessProtocolModel.getSelectedItem());
							getDeviceContextList().requestFocus();							
						}
					});
                }
			});
			accessProtocolCombo.addKeyListener(new KeyAdapter() {
				@Override
				public void keyPressed(KeyEvent e1) {
					final KeyEvent e = e1;
					getExecutorService().execute(new Runnable(){
						public void run() {
							if (e.getKeyCode() == KeyEvent.VK_RIGHT || e.getKeyCode() == KeyEvent.VK_ENTER) {
								getSelector().setSelectedContext(deviceContextList.getSelectedValue());
								getDeviceContextList().requestFocus();
							} else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
								getSelector().setSelectedProtocol(null);
								getAccessProtocolCombo().requestFocus();
							}
						}
					});
				}
			});
		}
		return accessProtocolCombo;
	}
	
	private JList<String> getDeviceContextList() {
		if (deviceContextList == null) {
			deviceContextList = new JList<>();
			deviceContextList.setModel(deviceContextModel);
			deviceContextList.setCellRenderer(new CellRenderer("context"));
			deviceContextList.setSelectionModel(deviceContextModel);
			deviceContextList.addKeyListener(new KeyAdapter() {
				@Override
				public void keyPressed(KeyEvent e1) {
					final KeyEvent e = e1;
					getExecutorService().execute(new Runnable(){
						public void run() {
							if (e.getKeyCode() == KeyEvent.VK_RIGHT || e.getKeyCode() == KeyEvent.VK_ENTER) {
								getSelector().setSelectedContext(deviceContextList.getSelectedValue());
								getDeviceServerList().requestFocus();
							}
		
						}
					});
				}
			});
			
			deviceContextList.addMouseListener(new MouseAdapter() {
				@Override
	        	public void mousePressed(MouseEvent e) {
	        		getExecutorService().execute(new Runnable(){
	        			public void run() {
	        				getSelector().setSelectedContext(deviceContextList.getSelectedValue());
	        			}
	        		});
                }
			});
			deviceContextList.addFocusListener(new FocusAdapter(){
				@Override
				public void focusGained(FocusEvent e) {
					if (getSelector().getSelectedContext() == null && deviceContextModel.getSize() > 0) {
						deviceContextList.setSelectedValue(selectedContext, true);
						deviceContextModel.reinstateLeadSelectedIndex();
					}							
				}
			});
		}
		return deviceContextList;
	}
	
	private JList<String> getSubSystemList() {
		if (subSystemList == null) {
			subSystemList = new JList<>();
//			Set<String> subSysName = Selector.SUB_SYSTEM.keySet();
//			String[] subSys = new String[subSysName.size()];
//			subSys = subSysName.toArray(subSys);
//			subSystemModel.setElements(subSys);
			subSystemList.setModel(subSystemModel);
			subSystemList.setCellRenderer(new CellRenderer("subSystem"));
			subSystemList.setSelectionModel(subSystemModel);
			subSystemList.setFocusable(false);
			subSystemList.addKeyListener(new KeyAdapter() {
				@Override
				public void keyPressed(KeyEvent e1) {
					final KeyEvent e = e1;
					getExecutorService().execute(new Runnable(){
						public void run() {
							if (e.getKeyCode() == KeyEvent.VK_RIGHT || e.getKeyCode() == KeyEvent.VK_ENTER) {
								getSelector().setSelectedSubSystem(subSystemList.getSelectedValue());
								getDeviceServerList().requestFocus();
							} else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
								selectedContext = getDeviceContextList().getSelectedValue();
								getSelector().setSelectedContext(null);
								getDeviceContextList().requestFocus();
							}

						}
					});
				}
			});
			subSystemList.addMouseListener(new MouseAdapter() {
				@Override
	        	public void mousePressed(MouseEvent e) {
	        		getExecutorService().execute(new Runnable(){
	        			public void run() {
	        				getSelector().setSelectedSubSystem(subSystemList.getSelectedValue());
	        			}
	        		});
                }
			});
			subSystemList.addFocusListener(new FocusAdapter(){
				@Override
				public void focusGained(FocusEvent e) {
					if (getSelector().getSelectedSubSystem() == null && subSystemModel.getSize() > 0) {
						subSystemList.setSelectedIndex(0);
					}							
				}
			});
		}
		return subSystemList;
	}
	
	private JList<String> getDeviceServerList() {
		if (deviceServerList == null) {
			deviceServerList = new JList<>();
			deviceServerList.setModel(deviceServerModel);
			deviceServerList.setCellRenderer(new CellRenderer("server"));
			deviceServerList.setSelectionModel(deviceServerModel);
			deviceServerList.addKeyListener(new KeyAdapter() {
				@Override
				public void keyPressed(KeyEvent e1) {
					final KeyEvent e = e1;
					getExecutorService().execute(new Runnable(){
						public void run() {
							if (e.getKeyCode() == KeyEvent.VK_RIGHT || e.getKeyCode() == KeyEvent.VK_ENTER) {
								getSelector().setSelectedServer(deviceServerList.getSelectedValue());
								getDeviceNameList().requestFocus();
			                } else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
			                	selectedContext = getDeviceContextList().getSelectedValue();
								getSelector().setSelectedContext(null);
								getDeviceContextList().requestFocus();
							}							
						}
					});
				}
			});
			deviceServerList.addMouseListener(new MouseAdapter() {
				@Override
	        	public void mousePressed(MouseEvent e) {
	        		getExecutorService().execute(new Runnable(){
	        			public void run() {
	        				getSelector().setSelectedServer(deviceServerList.getSelectedValue());	        				
	        			}
	        		});
                }
			});
			deviceServerList.addFocusListener(new FocusAdapter(){
				@Override
				public void focusGained(FocusEvent e) {
					if (deviceServerModel.getSize() < 0 || getSelector().getSelectedContext() == null) {
						getDeviceServerList().requestFocus();
					} else if (getSelector().getSelectedServer() == null && deviceServerModel.getSize() > 0) {
						deviceServerList.setSelectedValue(selectedServer, true);
						deviceServerModel.reinstateLeadSelectedIndex();
					}							
				}
			});
		}
		return deviceServerList;
	}
	
	private JList<String> getDeviceNameList() {
		if (deviceNameList == null) {
			deviceNameList = new JList<>();
			deviceNameList.setModel(deviceNameModel);
			deviceNameList.setCellRenderer(new CellRenderer("device"));
			deviceNameList.setSelectionModel(deviceNameModel);
			deviceNameList.addKeyListener(new KeyAdapter() {
				@Override
				public void keyPressed(KeyEvent e1) {
					final KeyEvent e = e1;
					getExecutorService().execute(new Runnable(){
						public void run() {
							if (e.getKeyCode() == KeyEvent.VK_RIGHT || e.getKeyCode() == KeyEvent.VK_ENTER) {
								getSelector().setSelectedDevice(deviceNameList.getSelectedValue());
								getPropertyNameList().requestFocus();
			                } else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
			                	selectedServer = getDeviceServerList().getSelectedValue();
								getSelector().setSelectedServer(null);
								getDeviceServerList().requestFocus();
							}							
						}
					});
				}
			});
			deviceNameList.addMouseListener(new MouseAdapter() {
				@Override
	        	public void mousePressed(MouseEvent e) {
	        		getExecutorService().execute(new Runnable(){
	        			public void run() {
	        				getSelector().setSelectedDevice(deviceNameList.getSelectedValue());	        				
	        			}
	        		});
                }
			});
			
			deviceNameList.addFocusListener(new FocusAdapter(){
				@Override
				public void focusGained(FocusEvent e) {
					if (deviceNameModel.getSize() < 0 || getSelector().getSelectedServer() == null) {
						getDeviceServerList().requestFocus();
					} else if (getSelector().getSelectedDevice() == null && deviceNameModel.getSize() > 0) {
						deviceNameList.setSelectedValue(selectedDevice, true);
						deviceNameModel.reinstateLeadSelectedIndex();
					}							
				}
			});
		}
		return deviceNameList;
	}
	
	private JList<String> getPropertyNameList() {
		if (propertyNameList == null) {
			propertyNameList = new JList<>();
			propertyNameList.setModel(propertyNameModel);
			propertyNameList.setCellRenderer(new CellRenderer("property"));
			propertyNameList.setSelectionModel(propertyNameModel);
			propertyNameModel.addListSelectionListener(new ListSelectionListener(){
				public void valueChanged(ListSelectionEvent e) {
					selectedProperty = propertyNameList.getSelectedValue();					
				}
			});
			propertyNameList.addKeyListener(new KeyAdapter() {
				@Override
				public void keyPressed(KeyEvent e1) {
					final KeyEvent e = e1;
					getExecutorService().execute(new Runnable(){
						public void run() {
							if (e.getKeyCode() == KeyEvent.VK_RIGHT || e.getKeyCode() == KeyEvent.VK_ENTER) {
								getSelector().setSelectedProperty(propertyNameList.getSelectedValue());
								fireSelectorGUIEvent(true);
							} else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
								selectedDevice = getDeviceNameList().getSelectedValue();
								getSelector().setSelectedProperty(null);
								getSelector().setSelectedDevice(null);
								getDeviceNameList().requestFocus();
							}							
						}
					});
				}
			});
			
			propertyNameList.addMouseListener(new MouseAdapter() {
				@Override
	        	public void mousePressed(MouseEvent e) {
	        		getExecutorService().execute(new Runnable(){
	        			public void run() {
	        				getSelector().setSelectedProperty(propertyNameList.getSelectedValue());	        				
	        			}
	        		});
                }
			});
			
			propertyNameList.addFocusListener(new FocusAdapter(){
				@Override
				public void focusGained(FocusEvent e) {
					if (getSelector().getSelectedProperty() == null && propertyNameModel.getSize() > 0) {
						propertyNameList.setSelectedValue(selectedProperty, true);
						propertyNameModel.reinstateLeadSelectedIndex();
					}
					if (getSelector().getSelectedDevice() == null && getSelector().getSelectedProperty() == null) {
						getDeviceNameList().requestFocus();
					}							
				}
			});
		}
		return propertyNameList;
	}
	
	private JCheckBox getShowStockPropertiesCheckBox() {
		if (showStockProperties == null) {
			showStockProperties = new JCheckBox("Show stock properties");
			showStockProperties.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					getExecutorService().execute(new Runnable() {
						public void run() {
							getSelector().setShowStockProperties(showStockProperties.isSelected());
						}
					});
				}
			});
		}
		return showStockProperties;
	}
	
	private NumberField getAccessRateField() {
		if (accessRateField == null) {
			accessRateField = new NumberField();
			accessRateField.setNumberType(Integer.class);
			accessRateField.setFormat("%d");
			GUIUtilities.setFixedSize(accessRateField,new Dimension(121, 21));
			accessRateField.setText(String.valueOf(1000));
			accessRateField.addPropertyChangeListener("value", new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					getExecutorService().execute(new Runnable(){
						public void run() {
							refreshSelectionLabel();
							getSelector().setAccessRate(accessRateField.getValue().intValue());							
						}
					});
				}
			});
		}
		return accessRateField;
	}
	
	private void refreshSelectionLabel() {
		applySettings();
		String text = connectionParameters.toString();
		getSelectionLabel().setText(text);
		getSelectionLabel().setToolTipText(text);
	}
	
	private void update(final ConnectionParameters params) {
		if (params == null || params.equals(connectionParameters)) return;
		ConnectionParameters old = connectionParameters;
		connectionParameters = params;
		getExecutorService().execute(new Runnable() {
			public void run() {
				getSelector().setSelectedProtocol(params.getAccessProtocol());
				getSelector().setSelectedContext(params.getDeviceContext());
				getSelector().setSelectedServer(params.getDeviceGroup());
				getSelector().setSelectedDevice(params.getDeviceName());
				getSelector().setSelectedProperty(params.getDeviceProperty());
				getSelector().setSelectedDevice(params.getDeviceName());
			}
		});
		getSelector().setAccessRate(params.getAccessRate());
		firePropertyChange(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY,old,connectionParameters);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		if (bean instanceof MultipleAcopDisplayer) {
			disp = bean;
		} else if (bean instanceof ConnectionParametersReceiver) {
			disp = bean;
			update(((ConnectionParametersReceiver)disp).getConnectionParameters());
			((ConnectionParametersReceiver)disp).addPropertyChangeListener(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY, new PropertyChangeListener(){

				public void propertyChange(PropertyChangeEvent evt) {
					if (!ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) return;
					update(((ConnectionParametersReceiver)disp).getConnectionParameters());
				}
				
			});  
		} else {
			throw new IllegalArgumentException("Only ConnectionParametersReceiver and" +
				" MultipleAcopDisplayer can use this Customizer.");
		}
		
		      
    }

	/**
	 * Returns the selected ConnectionParameters.
	 * 
     * @return the parameters
     */
    public ConnectionParameters getConnectionParameters() {
    	return connectionParameters;
    }

	/**
	 * Sets the ConnectionParameters and applies the values to the UI.
	 * 
     * @param parameters the parameters to set
     */
    public void setConnectionParameters(ConnectionParameters parameters) {
    	if (this.connectionParameters != null && this.connectionParameters.equals(parameters)) return;
		update(parameters);
    }

    private PropertyChangeListener getSelectorPropertyListener() {
        if (selectorPropertyListener == null) {
            selectorPropertyListener = evt -> SwingUtilities.invokeLater(() -> {
                if (getSelector() == null) return;
                if (!getPropertyNameList().isEnabled()) return;
                String process = evt.getPropertyName();
                String name = evt.getPropertyName();
                if ("accessRate".equals(process)) {
                    getAccessRateField().setValue(getSelector().getAccessRate());
                    refreshSelectionLabel();
                    getAccessRateField().requestFocus();
                    return;
                } else if ("accessMode".equals(name)) {
                    getAccessModeCombo().setSelectedItem(getSelector().getAccessMode());
                    refreshSelectionLabel();
                    getAccessModeCombo().requestFocus();
                    return;
                }
                if ("selectedProtocol".equals(name)) {
                    accessProtocolModel.setSelectedItem(getSelector().getSelectedProtocol());
                } else if ("selectedContext".equals(name)) {
                    deviceContextModel.setSelectedValue(getSelector().getSelectedContext());
                } else if ("selectedSubSystem".equals(name)) {
                    subSystemModel.setSelectedValue(getSelector().getSelectedSubSystem());
                } else if ("selectedServer".equals(name)) {
                    deviceServerModel.setSelectedValue(getSelector().getSelectedServer());
                } else if ("selectedDevice".equals(name)) {
                    deviceNameModel.setSelectedValue(getSelector().getSelectedDevice());
                } else if ("selectedProperty".equals(name)) {
                    propertyNameModel.setSelectedValue(getSelector().getSelectedProperty());
                } else if ("availableContexts".equals(name)) {
                    deviceContextModel.setElements(getSelector().getAvailableContexts());
                } else if ("availableSubsystems".equals(name)) {
                    subSystemModel.setElements(getSelector().getAvailableSubsystems());
                } else if ("availableServers".equals(name)) {
                    deviceServerModel.setElements(getSelector().getAvailableServers());
                } else if ("availableDevices".equals(name)) {
                    deviceNameModel.setElements(getSelector().getAvailableDevices());
                } else if ("availableProperties".equals(name)) {
                    propertyNameModel.setElements(getSelector().getAvailableProperties());
                } else if ("showStockProperties".equals(name)) {
                    getShowStockPropertiesCheckBox().setSelected(getSelector().isShowStockProperties());
                }
                refreshSelectionLabel();
                if (!Beans.isDesignTime()) {
                    if ("selectedProperty".equals(evt.getPropertyName())) {
                        if (evt.getNewValue() == null) {
                            getDescriptionLabel().setText("");
                            getDescriptionLabel().setToolTipText("");
                            return;
                        }
                        if (evt.getNewValue() != evt.getOldValue()) {
                            if (SelectorUtilities.isConnectionParametersValid(connectionParameters)) {
                                String text = SelectorUtilities.getInfo(connectionParameters);
                                getDescriptionLabel().setText(text);
                                getDescriptionLabel().setToolTipText(text);
                            }
                        }
                    }
                }
            });
        }
        return selectorPropertyListener;
    }
    
    private QueryListener getSelectorQueryListener() {
    	if (selectorQueryListener == null) {
    		selectorQueryListener = new QueryListener() {
				public void queryFinished(final QueryEvent e) {
					fillUpValues();
					adjustScrollBar(e.getProcess());
					refreshSelectionLabel();
					setCombosEnabled(true);
				}
				public void queryStarted(QueryEvent e) {
				    setCombosEnabled(false);
				}
			};
    	}
    	return selectorQueryListener;
    }
	
    protected void setSelector(Selector selector) {
    	if (this.selector != null && this.selector.equals(selector)) return;
    	if (this.selector != null) {
	    	this.selector.removePropertyChangeListener(getSelectorPropertyListener());
	    	this.selector.removeQueryListener(getSelectorQueryListener());
    	}
    	Selector old = this.selector;
    	this.selector = selector;
    	if (this.selector != null) {
	    	this.selector.addPropertyChangeListener(getSelectorPropertyListener());
	    	this.selector.addQueryListener(getSelectorQueryListener());
    	}
    	fillUpValues();
    	refreshSelectionLabel();
    	firePropertyChange("selector", old, this.selector);
    }
    
	protected Selector getSelector() {
        return selector;
    }
	
	private void adjustScrollBar(String process) {
		int width = 0;
		int extent = 0;
		if (initialized) {
			width = getListPanelScroll().getHorizontalScrollBar().getMaximum()/5;
			extent = getListPanelScroll().getHorizontalScrollBar().getModel().getExtent();
		}
		
		if ("protocol".equals(process)) {
			getListPanelScroll().getHorizontalScrollBar().setValue(0);
		} else if ("context".equals(process)) {
			getListPanelScroll().getHorizontalScrollBar().setValue(2*width-extent);
		} else if ("subSystem".equals(process)) {
			getListPanelScroll().getHorizontalScrollBar().setValue(3*width-extent);
		} else if ("server".equals(process)) {
			getListPanelScroll().getHorizontalScrollBar().setValue(4*width-extent);
		} else if ("device".equals(process)) {
			getListPanelScroll().getHorizontalScrollBar().setValue(5*width-extent);
		} else if ("property".equals(process)) {
			getListPanelScroll().getHorizontalScrollBar().setValue(5*width);
		}
	} 
	
	private void fillUpValues() {
	    SwingUtilities.invokeLater(() -> {
    		settingFromSelector = true;
    		deviceContextModel.setElements(getSelector().getAvailableContexts());
    		subSystemModel.setElements(getSelector().getAvailableSubsystems());
        	deviceServerModel.setElements(getSelector().getAvailableServers());
        	deviceNameModel.setElements(getSelector().getAvailableDevices());
        	propertyNameModel.setElements(getSelector().getAvailableProperties());
        	setSelectedValues();
        	settingFromSelector = false;
	    });
	}
	
	private void setSelectedValues() {
		getAccessRateField().setValue(getSelector().getAccessRate());
		getAccessModeCombo().setSelectedItem(getSelector().getAccessMode());
		if (getSelector().getSelectedProtocol() != null) {
			if (!getSelector().getSelectedProtocol().equals(getAccessProtocolCombo().getSelectedItem())
			        && getAccessProtocolCombo().getModel().getSize() != 0)
				getAccessProtocolCombo().setSelectedItem(getSelector().getSelectedProtocol());
		} else {
			if (getAccessProtocolCombo().getSelectedItem() != null)
				getAccessProtocolCombo().setSelectedItem(null);
		}
		getShowStockPropertiesCheckBox().setSelected(getSelector().isShowStockProperties());
		if (getSelector().getSelectedContext() != null) {
			if (!getSelector().getSelectedContext().equals(getDeviceContextList().getSelectedValue())
			        && getDeviceContextList().getModel().getSize() != 0)
				getDeviceContextList().setSelectedValue(getSelector().getSelectedContext(), true);
		} else {
			getDeviceContextList().setSelectedValue(selectedContext, true);
			deviceContextModel.reinstateLeadSelectedIndex();
		}
		
		if (getSelector().getSelectedSubSystem() != null) {
			if (!getSelector().getSelectedSubSystem().equals(getSubSystemList().getSelectedValue())
			        && getSubSystemList().getModel().getSize() != 0)
				getSubSystemList().setSelectedValue(getSelector().getSelectedSubSystem(), true);
		} else {
			getSubSystemList().setSelectedIndex(0);
			subSystemModel.reinstateLeadSelectedIndex();
		}
    	
		if (getSelector().getSelectedServer() != null) {
			if (!getSelector().getSelectedServer().equals(getDeviceServerList().getSelectedValue())
			        && getDeviceServerList().getModel().getSize() != 0)
				getDeviceServerList().setSelectedValue(getSelector().getSelectedServer(), true);
		} else {
			getDeviceServerList().setSelectedValue(selectedServer, true);
			deviceServerModel.reinstateLeadSelectedIndex();
		}
		
		if (getSelector().getSelectedDevice() != null) {
			if (!getSelector().getSelectedDevice().equals(getDeviceNameList().getSelectedValue())
			        && getDeviceNameList().getModel().getSize() != 0)
				getDeviceNameList().setSelectedValue(getSelector().getSelectedDevice(), true);
		} else {
			getDeviceNameList().setSelectedValue(selectedDevice, true);
			deviceNameModel.reinstateLeadSelectedIndex();
		}
    	
		if (getSelector().getSelectedProperty() != null) {
			if (!getSelector().getSelectedProperty().equals(getPropertyNameList().getSelectedValue())
			        && getPropertyNameList().getModel().getSize() != 0)
				getPropertyNameList().setSelectedValue(getSelector().getSelectedProperty(), true);
		} else {
			getPropertyNameList().setSelectedValue(selectedProperty, true);
			propertyNameModel.reinstateLeadSelectedIndex();
		}
   	
	}
	
	private void setCombosEnabled(boolean enabled) {
		getAccessProtocolCombo().setEnabled(enabled);
	    getDeviceContextList().setEnabled(enabled);
	    getSubSystemList().setEnabled(enabled);
	    getDeviceServerList().setEnabled(enabled);
	    getDeviceNameList().setEnabled(enabled);
	    getPropertyNameList().setEnabled(enabled);
	}
	
	/**
	 * Registers a SelectorGUIListener with this component.
	 * 
	 * @param l listener to be added
	 */
	public void addSelectorGUIListener(SelectorGUIListener l) {
		selectorGUIListeners.add(l);
	}
	
	/**
	 * Removes a SelectorGUIListener.
	 * 
	 * @param l listener to be removed
	 */
	public void removeSelectorGUIListener(SelectorGUIListener l) {
		selectorGUIListeners.remove(l);
	}
	
	protected void fireSelectorGUIEvent(boolean selectionComplete) {
		SelectorGUIListener[] listeners = (SelectorGUIListener[]) selectorGUIListeners.toArray();
		SelectorGUIEvent e = new SelectorGUIEvent(this, selectionComplete);
		for (SelectorGUIListener listener : listeners) {
			listener.selectionChanged(e);
		}
	}
	
	public static void main(String[] args) {
		try {
			
			SelectorListGUI s = new SelectorListGUI();
			RunnerHelper.runComponent(s,500,500);
						
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private class CellRenderer extends DefaultListCellRenderer {

		private static final long serialVersionUID = -1255512223908432074L;
	
		private String process;
		public CellRenderer(String process) {
			super();
			this.process = process;
		}
		
		@Override
		public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
	        Component c = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			if (!(c instanceof JLabel)) return c;
	        if (value == null) value = "";
	        JLabel component = (JLabel)c;
	        component.setText(" " + value.toString());
			
			String selectedValue = null;
			if ("context".equals(process)) {
				selectedValue = getSelector().getSelectedContext();
			} else if ("subSystem".equals(process)) {
				selectedValue = getSelector().getSelectedSubSystem();
			} else if ("server".equals(process)) {
				selectedValue = getSelector().getSelectedServer();
			} else if ("device".equals(process)) {
				selectedValue = getSelector().getSelectedDevice();
			} else if ("property".equals(process)) {
				selectedValue = getSelector().getSelectedProperty();
			}
			if (value.equals(selectedValue)) {
				component.setFont(new Font(getFont().getName(), Font.BOLD, getFont().getSize()));
			} else {
				component.setFont(new Font(getFont().getName(), Font.PLAIN, getFont().getSize()));
			}
			
			return component;
	        
        }
		
	}
	
	private ExecutorService executorService;
	private synchronized ExecutorService getExecutorService() {
		if (executorService == null) {
//			executorService = Executors.newSingleThreadExecutor();
			executorService = new  ThreadPoolExecutor(1, 1, 0, TimeUnit.NANOSECONDS,
		              new LinkedBlockingQueue<Runnable>()){
				@Override
				protected void afterExecute(Runnable r, Throwable t) {
					super.afterExecute(r, t);
					if (t != null) {
						t.printStackTrace();
					}
				}
			};
			((ThreadPoolExecutor)executorService).setRejectedExecutionHandler(new RejectedExecutionHandler(){
				public void rejectedExecution(Runnable r, ThreadPoolExecutor executor) {
					//ignore
				}
			});
        }
        return executorService;
	}
	
	synchronized void setExecutorService(ExecutorService service) {
		if (executorService != null) executorService.shutdown();
		this.executorService = service;
	}

}
/* __oOo__ */
