/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;
import java.awt.Container;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;

import com.cosylab.gui.components.util.RunnerHelper;

import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>SelectorPropertyEditor</code> is a <code>PropertyEditor</code>
 * for <code>ConnectionParameters</code>. 
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see ConnectionParameters
 */
public class SelectorPropertyEditor extends PropertyEditorSupport implements PropertyEditor {

	private ConnectionCustomizer editor;
	
	/**
	 * Creates new instance of SelectorPropertyEditor.
	 */
	public SelectorPropertyEditor() {
		super();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
	public Component getCustomEditor() {
		if (editor == null) {
			editor = new ConnectionCustomizer();
			editor.setConnectButtonVisible(false);
			editor.setConnectionParameters((ConnectionParameters)getValue());
			editor.addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					setValueSilently(editor.getConnectionParameters());
				}
			});
		}

		return editor;
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
	public String getJavaInitializationString() {
		ConnectionParameters param = (ConnectionParameters) getValue();
		return "new de.desy.acop.transport.ConnectionParameters(\""+param.getRemoteName()+
				"\",de.desy.acop.transport.AccessMode."+param.getAccessMode()+","+param.getAccessRate()+"," + param.getPropertySize() + ","+SelectorUtilities.getExtendedDataString(param)+")";
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
	public boolean supportsCustomEditor() {
	    return true;
    }
	
//	public void setAsText(String text) {
//		String[] values = text.split(",");
//		setValue(new ConnectionParameters(values[0], AccessMode.valueOf(values[1]),Integer.parseInt(values[2])));
//    }
	
	private void setValueSilently(Object value) {
		super.setValue(value);
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
	 */
	@Override
	public void setValue(Object value) {
		super.setValue(value);
		if (editor!=null) {
			editor.setConnectionParameters((ConnectionParameters)value);
		}
	}
	
	public static void main(String[] args) {
		try {
			
			SelectorPropertyEditor e= new SelectorPropertyEditor();
			e.addPropertyChangeListener(new PropertyChangeListener() {
			
				public void propertyChange(PropertyChangeEvent evt) {
					System.out.println("E: "+evt.getPropertyName()+" "+evt.getNewValue());
				}
			
			});
			
			Component c= e.getCustomEditor();
			
			RunnerHelper.runComponent((Container)c,300,300);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}


/* __oOo__ */
