/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.selector;

import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import com.cosylab.gui.adapters.Converter;

import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.table.TableColumnType;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>TableConnectionCustomizer</code> is a connection customizer for 
 * multiple displayers which handles <code>AcopTableParameters</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see MultipleAcopDisplayer
 * @see AcopTableParameters
 */
public class TableConnectionCustomizer extends MultipleConnectionCustomizer<AcopTableParameters> {


	private static final long serialVersionUID = 4596748128425127898L;
	private TableCustomizerPanel tableCustomizerPanel;
	private SelectorGUI selector;
	
	protected class AcopTableCellRenderer extends DefaultListCellRenderer {
		
		private static final long serialVersionUID = 7937185460891053942L;

		/*
		 * (non-Javadoc)
		 * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
		 */
		@Override
		public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			if (value instanceof AcopTableParameters) {
				value = ((AcopTableParameters)value).getConnectionParameters() + "; " + ((AcopTableParameters)value).getColumnType() + (((AcopTableParameters)value).getConverter() != null ? "; Converter" : "");
			}
			
			return super.getListCellRendererComponent(list, value, index, isSelected,
					cellHasFocus);
		}
	}
	
	public TableConnectionCustomizer() {
		super();
		setConverterTabEnabled(true);
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getListRenderer()
	 */
	@Override
	protected ListCellRenderer<Object> getListRenderer() {
		return new AcopTableCellRenderer();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#generateParameters()
	 */
	@Override
	protected AcopTableParameters generateParameters() {
		Converter c=null;
		try {
			c= getConverterPanel().getConverter();
			if (c!=null) {
				c= (Converter)c.clone();
			}
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return new AcopTableParameters(
				getSelectorGUI().getConnectionParameters(),
				getTableCustomizerPanel().getType(),
				c,
				getTableCustomizerPanel().getName()); 
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSettingsPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopTableParameters> getSettingsPanel() {
		return getTableCustomizerPanel();
	}
	
	private TableCustomizerPanel getTableCustomizerPanel() {
		if (tableCustomizerPanel == null) {
			tableCustomizerPanel = new TableCustomizerPanel();
		}
		return tableCustomizerPanel;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSelectorGUI()
	 */
	@Override
	protected SelectorGUI getSelectorGUI() {
		if (selector == null) {
			selector = super.getSelectorGUI();
			selector.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					ConnectionParameters parameters = selector.getConnectionParameters();
					if (parameters.getDeviceName().contains("*") ||
							parameters.getDeviceProperty().contains("*")) {
						getTableCustomizerPanel().setType(TableColumnType.STRING);
					}
					getTableCustomizerPanel().setName(parameters.getRemoteName());
				}
			});
		}
		
	    return super.getSelectorGUI();
    }
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getDisplayerParameters()
	 */
	@Override
	public AcopTableParameters[] getDisplayerParameters() {
		AcopTableParameters[] p = new AcopTableParameters[displayerParameters.size()];
    	return displayerParameters.toArray(p);
	}  
}


/* __oOo__ */
