/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.table;

import com.cosylab.gui.components.table.cells.PatternCell;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.AcopTable;

/**
 * Table column for presenting numerical values in binary form.
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopBitTableColumn extends AcopTableColumn implements DoubleSeqConsumer {

	@SuppressWarnings("rawtypes")
	public static final Class[] SUPPORTED_CONSUMER_TYPES = {
		DoubleSeqConsumer.class
		};
	public static final String[] SUPPORTED_CHARACTERISTICS = {};
		
	protected double[] rows;
	
	/**
	 * Constructs new AcopBitTableColumn.
	 *
	 */
	public AcopBitTableColumn() {
		super();
	}
	
	/**
	 * Constructs new AcopBitTableColumn.
	 * 
	 * @param table parent table
	 * @param uniqueName unique name of the column
	 * @param shortName short name of the column
	 */
	public AcopBitTableColumn(AcopTable table, String uniqueName, String shortName) {
		super(table, uniqueName, shortName);
		rows = new double[0];
		cells = new PatternCell[0];
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getColumnClass()
	 */
	@Override
	public Class<?> getColumnClass() {
		return PatternCell.class; //String.class;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopNumericTableColumn#getValue(int)
	 */
	@Override
	public Object getValue(int rowIndex) {
		if (rows == null || rowIndex >= rows.length || rowIndex < 0) return null;
		if (cells[rowIndex] == null) {
			cells[rowIndex] = new PatternCell(displayerParameters.getName()+"[" + rowIndex+"]", this, (long)rows[rowIndex], null);
			cells[rowIndex].putAllCharacteristics(characteristics);
		} else {
			cells[rowIndex].setValue((long)rows[rowIndex]);
		}
		setCharacteristicsOnCell(cells[rowIndex]);
		return cells[rowIndex];
	}	
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleSeqConsumer#updateValue(long, double[])
	 */
	public void updateValue(long timestamp, double[] value) throws CommonException {
		rows = value;
		if (cells.length != value.length) {
			cells = new PatternCell[value.length];
		}
		fireColumnChanged();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics() {
		return DisplayerUtilities.combineCharacteristics(DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS,
			    SUPPORTED_CHARACTERISTICS);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	@SuppressWarnings("unchecked")
	public Class<DataConsumer>[] getSupportedConsumerTypes() {
		return SUPPORTED_CONSUMER_TYPES;
	}

}
