/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.table;

import com.cosylab.gui.components.table.cells.StringCell;
import com.cosylab.gui.components.table.cells.TableCell;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.StringSeqConsumer;

import de.desy.acop.displayers.AcopTable;

/**
 * TableColumn for presenting string values. 
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopStringTableColumn extends AcopTableColumn implements StringSeqConsumer {
	
	@SuppressWarnings("rawtypes")
	public static final Class[] SUPPORTED_CONSUMER_TYPES = {
		StringSeqConsumer.class
		};
	public static final String[] SUPPORTED_CHARACTERISTICS = {};
	
	private String[] rows;
	private DataState state = new DataState(DataState.NORMAL);
	
	/**
	 * Constructs new AcopStringTableColumn.
	 *
	 */
	public AcopStringTableColumn() {
		super();
	}
	
	/**
	 * Constructs new AcopStringTableColumn.
	 * 
	 * @param table parent table
	 * @param uniqueName unique name of the column
	 * @param shortName short name of the column
	 */
	public AcopStringTableColumn(AcopTable table, String uniqueName, String shortName) {
		super(table, uniqueName, shortName);
		rows = new String[0];
		cells= new StringCell[0];
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getColumnClass()
	 */
	@Override
	public Class<?> getColumnClass() {
		return StringCell.class;
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics() {
		return DisplayerUtilities.combineCharacteristics(DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS,
			    SUPPORTED_CHARACTERISTICS);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	@SuppressWarnings("unchecked")
	public Class<DataConsumer>[] getSupportedConsumerTypes() {
		return SUPPORTED_CONSUMER_TYPES;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getValue(int)
	 */	
	@Override
	public Object getValue(int rowIndex) {
		if (rows == null || rowIndex >= rows.length || rowIndex < 0) return null;
		if (cells[rowIndex] == null) {
			cells[rowIndex] = new StringCell(displayerParameters.getName()+"[" + rowIndex+"]", this, rows[rowIndex], null);
			cells[rowIndex].putAllCharacteristics(characteristics);
			cells[rowIndex].putCharacteristic(TableCell.SUSPENDED, false);
			cells[rowIndex].putCharacteristic(TableCell.ALARM, false);
			cells[rowIndex].putCharacteristic(TableCell.WARNING, false);
			cells[rowIndex].putCharacteristic(TableCell.TIMEOUT, false);
			cells[rowIndex].putCharacteristic(TableCell.ERROR, false);
		} else {
			cells[rowIndex].setValue(rows[rowIndex]);
		}
		setCharacteristicsOnCell(cells[rowIndex]);
		return cells[rowIndex];
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.displayers.StringSeqConsumer#updateValue(long, java.lang.String[])
	 */
	public void updateValue(long timestamp, String[] names) {
		rows = names;	
		if (cells.length != names.length) {
			cells = new StringCell[names.length];
		}
		fireColumnChanged();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getDataState()
	 */
	@Override
	public DataState getDataState() {
		return state;
	}
}
