/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.table;

import java.util.EventObject;

/**
 * <code>TableColumnEvent</code> is an event triggered by <code>AcopTableColumn</code>
 * and notifies the listeners that there has been a change in the data of the column.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see TableColumnListener
 * @see AcopTableColumn
 */
public class TableColumnEvent extends EventObject {

	private static final long serialVersionUID = -2287102515781647264L;
	private int rowIndex = -1;
	
	/**
	 * Constructs new TableColumnEvent.
	 * 
	 * @param source the source column
	 */
	public TableColumnEvent(AcopTableColumn source) {
		super(source);
	}
	
	/**
	 * Constructs new TableColumnEvent.
	 * 
	 * @param source the source column
	 * @param rowIndex index of the row which changed
	 */
	public TableColumnEvent(AcopTableColumn source, int rowIndex) {
		super(source);
		this.rowIndex = rowIndex;
	}
	
	/**
	 * Returns the index of the row, which triggered the event.
	 * If returned value is -1, the change had an effect on all rows.
	 * 
	 * @return the row index
	 */
	public int getRowIndex() {
		return rowIndex;
	}
	
	/**
	 * Returns the column that was the source of the event.
	 * 
	 * @return the source
	 */
	public AcopTableColumn getColumn() {
		return (AcopTableColumn)getSource();
	}

	
	

}
