/**
 * File: ConnectionInfo.java
 * Date: Jul 27, 2010
 */
package de.desy.acop.displayers.tarantula;


/**
 * Information about the connection. 
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ConnectionInfo.java 40186 2010-11-03 14:12:06Z walla $
 */
final class ConnectionInfo {
	
	private static final String UNKNOWN = "UNKNOWN";
	public static final ConnectionInfo UNKNOWN_CONNECTION_INFO = new ConnectionInfo();

	private final String responsible;
	private final String location;
	private final String fecName;
	private final String eqmName;
	private final String os;
	private final String tineVersion;
	private final String hardware;
	private final String description;
	private final String ipAddress;
	private final String hostname;
	private final int portOffset;

	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * Constructor of the connection info instance.
	 * 
x	 * @param fecName FEC (Front End Computer) name
	 * @param eqmName Equipment name
	 * @param os Operation system
	 * @param responsible Who is responsible for the server
	 * @param location Where is the server located
	 * @param version Version of the server
	 * @param hardware Hardware interface
	 * @param description Short description of the entry
	 * @param hostName Host name
	 * @param hostAddress Numerical host address
	 * @param portOffset Port offset
	 */
	ConnectionInfo(
			final String fecName, 
			final String eqmName, 
			final String os, 
			final String responsible, 
			final String location, 
			final String version, 
			final String hardware,
			final String description,
			final String hostName,
			final String hostAddress,
			final int portOffset) {
		this.fecName = fecName;
		this.eqmName = eqmName;
		
		this.ipAddress = hostAddress;
		this.hostname = hostName;
		this.portOffset = portOffset;
		
		this.os = os;
		this.responsible = responsible;
		this.location = location;
		this.tineVersion = version;
		this.hardware = hardware;
		this.description = description;
	}
	
	private ConnectionInfo() {
		this.responsible = UNKNOWN;
		this.location = UNKNOWN;
		this.fecName = UNKNOWN;
		this.eqmName = UNKNOWN;
		this.os = UNKNOWN;
		this.tineVersion = UNKNOWN;
		this.hardware = UNKNOWN;
		this.description = UNKNOWN;
		this.ipAddress = UNKNOWN;
		this.hostname = UNKNOWN;
		this.portOffset = 0;
	}

	// --- INSTANCE METHODS ----------------------------------------------------

	String getResponsible() { return this.responsible; }
	String getLocation() { return this.location; }
	String getFecName() { return this.fecName; }
	String getEqmName() { return this.eqmName; }
	String getOs() { return this.os; }
	String getTineVersion() { return this.tineVersion; }
	String getHardware() { return this.hardware; }
	String getDescription() { return this.description; }
	String getIpAddress() { return this.ipAddress; }
	String getHostName() { return this.hostname; }
	int getPortOffset() { return this.portOffset; }
}
/* end of file */