package de.desy.acop.displayers.tarantula;

/**
 * <p>Status enumeration, mostly used to provide setter and getter method on the
 * nodes (see {@link NodeConnection}). This methods uses this enumeration for 
 * their status information.</p>
 * <p>Their currently also exists three special enumeration which are used for special
 * purpose: {@link #ASCENDING_WARNING}, {@link #STATUS_UNKNOWN}, and {@link #INTERNAL_ERROR}.
 * 
 * <p><b>ATTENTION</b>: This code depends heavily on the TINE library. For 
 * further version this enumeration should deal with the link status in a more
 * independent way then it is realized now. Currently it depends on the special 
 * underlying connection protocol.</p>
 *  
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: EStatusLink.java 40186 2010-11-03 14:12:06Z walla $
 */
enum EStatusLink {
	/** Links are established and connections are OK. */
	SUCCESS, 
	/** Links are established and connections failed. */
	FAILURE, 
	/** Links aren't established because of a blacklisted link (link doesn't exist) . */
	FAILURE_BLACKLISTED, 
	/** Links aren't established because of an unresolved address (address doesn't exist). */
	FAILURE_UNRESOLVED_ADDRESS, 
	/** Links aren't established because of an invalid data reference (provided data on link differs). */
	FAILURE_INVALID_DATA_REFERENCE, 
	/** Links aren't established because of an invalid data reference (provided data on link differs). */
	FAILURE_ILLEGAL_PROPERTY,
	
	/** Links is pending (no TINE link status is returned due to a pending (German: schwebend} link). */
	PENDING,

	/** Link is established but the status is currently unknown. */
	STATUS_UNKNOWN,

	/** Ascending warning, reflecting that one of the ascending connections 
	 * returns a failure code (i.e. a TINE code number unequal to 0). */
	ASCENDING_WARNING,
	
	/** Internal error occurred. */
	INTERNAL_ERROR;
}
/* end of file */