package de.desy.acop.displayers.tarantula;

import java.awt.Font;
import java.util.Date;
import java.util.logging.Logger;

import de.desy.tine.definitions.TErrorList;

/**
 * Root node.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: NodeRoot.java 63096 2014-11-14 16:36:43Z bobnar $
 */
final class NodeRoot extends ANode {
	
	// --- CLASS FIELDS --------------------------------------------------------

	private static final Logger log = Utility.getLogger(NodeRoot.class);
	private static final long serialVersionUID = 7034295155376280340L;
	public static final Font FONT_ROOT = new Font("Arial", Font.BOLD, 14);
	public static final String ROOT = "ROOT";
	private static final String UNKNOWN_CONNECTION_INFO_STR = "UNKOWN CONNECTION INFO";

	// --- INSTANCE FIELDS -----------------------------------------------------

	private EStatusRoot status;
	private Date updateTime;
	private ENodeCondition condition;

	// --- CONSTRUCTOR ---------------------------------------------------------

	/**
	 * Constructor.
	 * 
	 * @param status Initializing status of the root node.
	 */
	NodeRoot(final EStatusRoot status) {
		this.status = status;
		this.updateTime = new Date();
		this.condition = ENodeCondition.ORDINARY_NODE;
	}
	
	// --- INSTANCE METHODS ---------------------------------------------------

	void setStatus(final EStatusRoot status) { this.status = status; }
	Date getUpdateTime()        { return this.updateTime; }
	EStatusRoot getStatus()     { return this.status; }
	
    @Override final String         getFullAddress() { return NodeRoot.ROOT; }
	@Override final ConnectionInfo getInfo()        { return ConnectionInfo.UNKNOWN_CONNECTION_INFO; }
	@Override final String         getInfoString()  { return NodeRoot.UNKNOWN_CONNECTION_INFO_STR; }
	/** Currently returns a {@code null}. */
	@Override final String         getStatusLink()  { return null; }
	/** Always returns the value from {@link TErrorList#getStatusString(int)} with {@link TErrorList#success)}. */
    @Override final int            getStatusLinkCode() { return TErrorList.success; }
    @Override final void           setCondition(final ENodeCondition condition) { this.condition = condition; }
    @Override final ENodeCondition getCondition() { return this.condition; }
	
	void refreshTree(final EStatusRoot status) {
		switch(status) {
		case ROOT_NON_CONNECTION_FAILED:
		case ROOT_ONLY_FAILED_CONNECTIONS:
			this.status = status;
			this.updateTime = new Date();
			break;
			
		case ROOT_REFRESH_CONNECTIONS:
			this.status = status;
			break;
			
		case ROOT_REFRESH_FINISHED:
		case ROOT_TIME_EXCEEDED:
			this.status = status;
			this.updateTime = new Date();
			break;
			
		default:
			log.warning("unknown refresh status on root node: " + status + " -> only " + EStatusRoot.ROOT_REFRESH_CONNECTIONS + " or " + EStatusRoot.ROOT_REFRESH_FINISHED + " are allowed!");
		}
	}

	@Override
    public String toString() {
		return NodeRoot.ROOT;
	}
}
/* end of file */