/**
 * File: TineException.java
 * Date: Sep 20, 2010
 */
package de.desy.acop.displayers.tarantula;

import de.desy.tine.definitions.TErrorList;

/**
 * Special exception instance of the TINE protocol stack. Returns the error code
 * and transformed error code as a string instance from the failure.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: TineException.java 40186 2010-11-03 14:12:06Z walla $
 */
class TineException extends ConnectionException {

	// --- CLASS FIELDS --------------------------------------------------------
	
	private static final long serialVersionUID = -4471923846094123839L;
	
	// --- CONSTRUCTOR -----------------------------------------------------
	
	/**
	 * Constructor.
	 * 
	 * @param errorCode Error code.
	 */
	TineException(final int errorCode) {
		super(errorCode);
	}
	
	/**
	 * Constructor.
	 * 
	 * @param errorCode Error code.
	 * @param fullAddress Full address.
	 */
	TineException(final int errorCode, final String fullAddress) { 
		super(errorCode, fullAddress);
	}
	
	/**
	 * Constructor.
	 * 
	 * @param errorCode Error code.
	 * @param fullAddress Full address.
	 * @param throwable Throwable.
	 */
	TineException(final int errorCode, final String fullAddress, final Throwable throwable) { 
		super(errorCode, fullAddress, throwable); 
	}
	
	/**
	 * Constructor.
	 * 
	 * @param errorCode Error code.
	 * @param throwable Throwable.
	 */
	TineException(final int errorCode, final Throwable throwable) { 
		super(errorCode, throwable); 
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	@Override String getError() { return TErrorList.getErrorString(this.getErrorCode()); }
	@Override String getErrorWithCode() { return TErrorList.getErrorString(this.getErrorCode()) + " (" + this.getErrorCode() + ')'; }
}
/* end of file */