package de.desy.acop.displayers.tarantula;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.logging.ConsoleHandler;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.border.TitledBorder;

import com.cosylab.gui.components.util.IconHelper;

import de.desy.tine.definitions.TErrorList;

/**
 * Miscellaneous static helper methods and fields.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: Utility.java 63096 2014-11-14 16:36:43Z bobnar $
 */
final class Utility {

	public static final Color LIGHT_BLUE = new Color(200,220,255);
	private static final Font FONT_BORDER_TITLE = new Font("Arial", Font.BOLD, 10);

	final static Icon ROOT_ICON_16                               = IconHelper.createIcon("icons/tarantula/Root16.png");
	final static Icon ROOT_ICON_DISABLE_16                       = IconHelper.createIcon("icons/tarantula/Root16.png");
	
	final static Icon WAIT_ICON_16                               = IconHelper.createIcon("icons/tarantula/Wait16.png"); 
	final static Icon WAIT_ICON_DISABLE_16                       = IconHelper.createIcon("icons/tarantula/Wait16.png"); 

	// -> OK
	final static Icon SUCCESS_NODE_ICON_16                       = IconHelper.createIcon("icons/tarantula/SuccessNode16.png");
	final static Icon SUCCESS_NODE_ICON_DISABLE_16               = IconHelper.createIcon("icons/tarantula/SuccessNode16.png");

	// -> ?
	final static Icon SUCCESS_IS_ASCENDING_NODE_ICON_16          = IconHelper.createIcon("icons/tarantula/SuccessIsAscendingNode16.png");
	final static Icon SUCCESS_IS_ASCENDING_NODE_ICON_DISABLE_16  = IconHelper.createIcon("icons/tarantula/SuccessIsAscendingNode16.png");

	// -> -|
	final static Icon SUCCESS_IS_STOP_NODE_ICON_16               = IconHelper.createIcon("icons/tarantula/SuccessIsStopNode16.png");
	final static Icon SUCCESS_IS_STOP_NODE_ICON_DISABLE_16       = IconHelper.createIcon("icons/tarantula/SuccessIsStopNode16.png");

	// -> -O
	final static Icon SUCCESS_IS_CYCLE_LOOP_ICON_16              = IconHelper.createIcon("icons/tarantula/SuccessIsCycleLoopNode16.png");
	final static Icon SUCCESS_IS_CYCLE_LOOP_ICON_DISABLE_16      = IconHelper.createIcon("icons/tarantula/SuccessIsCycleLoopNode16.png");

	// -> [!]
	final static Icon SUCCESS_META_PROTOCOL_FAILURE_ICON_16      = IconHelper.createIcon("icons/tarantula/SuccessMetaProtocolFailureNode16.png");
	final static Icon SUCCESS_META_PROTOCOL_FAILURE_DISABLE_16   = IconHelper.createIcon("icons/tarantula/SuccessMetaProtocolFailureNode16.png");
	
	// -> [!]
	final static Icon SUCCESS_IGNORE_SERVER_ICON_16              = IconHelper.createIcon("icons/tarantula/SuccessButIgnoreServer16.png");
	final static Icon SUCCESS_IGNORE_SERVER_ICON_DISABLE_16      = IconHelper.createIcon("icons/tarantula/SuccessButIgnoreServer16.png");
	
	final static Icon FAILURE_CONNECTION_ICON_16                 = IconHelper.createIcon("icons/tarantula/FailureConnectionNode16.png");
	final static Icon FAILURE_CONNECTION_ICON_DISABLE_16         = IconHelper.createIcon("icons/tarantula/FailureConnectionNode16.png");
	
	final static Icon FAILURE_ICON_16                            = IconHelper.createIcon("icons/tarantula/FailedLink16.png");
	final static Icon FAILURE_ICON_DISABLE_16                    = IconHelper.createIcon("icons/tarantula/FailedLink16.png");
	
	final static Icon FAILURE_ICON_32                            = IconHelper.createIcon("icons/tarantula/FailedLink32.png");
	final static Icon FAILURE_ICON_DISABLE_32                    = IconHelper.createIcon("icons/tarantula/FailedLink32.png");
	
	final static Icon WARNING_ICON_16                            = IconHelper.createIcon("icons/tarantula/WarningLink16.png");
	final static Icon WARNING_ICON_DISABLE_16                    = IconHelper.createIcon("icons/tarantula/WarningLink16.png");
	
	final static Icon WARNING_ICON_32                            = IconHelper.createIcon("icons/tarantula/WarningLink32.png");
	final static Icon WARNING_ICON_DISABLE_32                    = IconHelper.createIcon("icons/tarantula/WarningLink32.png");
	
	final static Icon TIME_EXCEEDED_16                           = IconHelper.createIcon("icons/tarantula/TimeExceeded16.png");
	final static Icon TIME_EXCEEDED_DISABLE_16                   = IconHelper.createIcon("icons/tarantula/TimeExceeded16.png");
	
	final static Icon INTERNAL_ERROR_ICON_16                     = IconHelper.createIcon("icons/tarantula/InternalError16.png");
	final static Icon INTERNAL_ERROR_ICON_DISABLE_16             = IconHelper.createIcon("icons/tarantula/InternalError16.png");
	
	final static Icon SPECIAL_FAILURE_ICON_16                    = IconHelper.createIcon("icons/tarantula/SpecialFailedLink16.png");
	final static Icon SPECIAL_FAILURE_ICON_DISABLE_16            = IconHelper.createIcon("icons/tarantula/SpecialFailedLink16.png");
	
	final static Icon SPECIAL_FAILURE_ICON_32                    = IconHelper.createIcon("icons/tarantula/SpecialFailedLink32.png");
	final static Icon SPECIAL_FAILURE_ICON_DISABLE_32            = IconHelper.createIcon("icons/tarantula/SpecialFailedLink32.png");
	
	final static String EMPTY_STRING = "";

	// --- CLASS METHODS ---------------------------------------------------
	
	static Icon getStatusIcon(final EStatusLink status, final boolean isEnabled) {
		switch(status) {
			case SUCCESS:
				return isEnabled ? SUCCESS_NODE_ICON_16 : SUCCESS_NODE_ICON_DISABLE_16;
			case FAILURE:
				return isEnabled ? FAILURE_ICON_16 : FAILURE_ICON_DISABLE_16;
			case FAILURE_BLACKLISTED:
			case FAILURE_UNRESOLVED_ADDRESS:
			case FAILURE_ILLEGAL_PROPERTY:
			case FAILURE_INVALID_DATA_REFERENCE:
				return isEnabled ? SPECIAL_FAILURE_ICON_16 : SPECIAL_FAILURE_ICON_DISABLE_16;
			case ASCENDING_WARNING:
			case INTERNAL_ERROR:
			case STATUS_UNKNOWN:
			case PENDING:
				return isEnabled ? WARNING_ICON_16 : WARNING_ICON_DISABLE_16;
		}
		
		return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
	}
	
	static Icon getStatusIcon(final EStatusLink status, final ENodeCondition condition, final boolean isEnabled) {
		switch(status) {
			case SUCCESS:
				switch(condition) {
				case STOP_NODE:
					return isEnabled ? SUCCESS_IS_STOP_NODE_ICON_16 : SUCCESS_IS_STOP_NODE_ICON_DISABLE_16;
				case CYCLE_LOOP_NODE:
					return isEnabled ? SUCCESS_IS_CYCLE_LOOP_ICON_16 : SUCCESS_IS_CYCLE_LOOP_ICON_DISABLE_16;
				case DEPTH_EXCEEDED:
					return isEnabled ? SUCCESS_IS_ASCENDING_NODE_ICON_16 : SUCCESS_IS_ASCENDING_NODE_ICON_DISABLE_16;
				case TIME_EXCEEDED:
					return isEnabled ? TIME_EXCEEDED_16 : TIME_EXCEEDED_DISABLE_16;
				case ORDINARY_NODE:
					return isEnabled ? SUCCESS_NODE_ICON_16 : SUCCESS_NODE_ICON_DISABLE_16;
				case IGNORE_SERVER:
					return isEnabled ? SUCCESS_IGNORE_SERVER_ICON_16 : SUCCESS_IGNORE_SERVER_ICON_DISABLE_16;
				case FAILURE_CONNECTION:
					return isEnabled ? FAILURE_CONNECTION_ICON_16 : FAILURE_CONNECTION_ICON_DISABLE_16;
				default:
					return isEnabled ? SUCCESS_NODE_ICON_16 : SUCCESS_NODE_ICON_DISABLE_16;
				}
			case FAILURE:
				switch(condition) {
				case FAILURE_INTERNAL:
					return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
				default:
					return isEnabled ? FAILURE_ICON_16 : FAILURE_ICON_DISABLE_16;
				}
			case FAILURE_ILLEGAL_PROPERTY:
				switch(condition) {
				case FAILURE_INTERNAL:
					return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
				default:
					return isEnabled ? SUCCESS_META_PROTOCOL_FAILURE_ICON_16 : SUCCESS_META_PROTOCOL_FAILURE_DISABLE_16;
				}
			case FAILURE_BLACKLISTED:
				switch(condition) {
				case FAILURE_INTERNAL:
					return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
				default:
					return isEnabled ? SUCCESS_META_PROTOCOL_FAILURE_ICON_16 : SUCCESS_META_PROTOCOL_FAILURE_DISABLE_16;
				}
			case FAILURE_UNRESOLVED_ADDRESS:
				switch(condition) {
				case FAILURE_INTERNAL:
					return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
				default:
					return isEnabled ? SUCCESS_META_PROTOCOL_FAILURE_ICON_16 : SUCCESS_META_PROTOCOL_FAILURE_DISABLE_16;
				}
			case FAILURE_INVALID_DATA_REFERENCE:
				switch(condition) {
				case FAILURE_INTERNAL:
					return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
				default:
					return isEnabled ? SUCCESS_META_PROTOCOL_FAILURE_ICON_16 : SUCCESS_META_PROTOCOL_FAILURE_DISABLE_16;
				}
			case ASCENDING_WARNING:
			case STATUS_UNKNOWN:
			case INTERNAL_ERROR:
			case PENDING:
				return isEnabled ? WARNING_ICON_16 : WARNING_ICON_DISABLE_16;
		}
		
		return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
	}
	
	static Icon getRootIcon(final EStatusRoot status, final boolean isEnabled) {
		switch(status) {
		    case ROOT:
			case ROOT_REFRESH_FINISHED:
				return isEnabled ? ROOT_ICON_16 : ROOT_ICON_DISABLE_16;
			case ROOT_REFRESH_CONNECTIONS:
			case ROOT_REFRESH_PRESS_BUTTON_TO_REFRESH:
			case ROOT_REFRESH_WAIT_FOR_REFRESH:
				return isEnabled ? WAIT_ICON_16 : WAIT_ICON_DISABLE_16;
			case ROOT_TIME_EXCEEDED:
				return isEnabled ? TIME_EXCEEDED_16 : TIME_EXCEEDED_DISABLE_16;
			case ROOT_ONLY_FAILED_CONNECTIONS:
				return isEnabled ? SPECIAL_FAILURE_ICON_16: SPECIAL_FAILURE_ICON_DISABLE_16;
			case ROOT_NON_CONNECTION_FAILED:
				return isEnabled ? WARNING_ICON_16: WARNING_ICON_DISABLE_16;
		}
		
		return isEnabled ? INTERNAL_ERROR_ICON_16 : INTERNAL_ERROR_ICON_DISABLE_16;
	}
	
	static String getInfo(final NodeConnection node) {
		final StringBuilder builder = new StringBuilder();
		
		switch(node.getStatus()) {
		
		case FAILURE:
		case FAILURE_BLACKLISTED:
		case FAILURE_ILLEGAL_PROPERTY:
		case FAILURE_INVALID_DATA_REFERENCE:
		case FAILURE_UNRESOLVED_ADDRESS:
			if (node.getStatusLinkCode() != TErrorList.success) {
				builder.append("<p><center>");
				builder.append("The following error occurs on this connection:<br>").append(node.getStatusLink());
				if (node.getStatusLinkCode() == TErrorList.illegal_property 
						|| node.getStatusLinkCode() == TErrorList.link_blacklisted)
					builder.append("<br><br>This error can happen on older TINE releases,<br>for example version 3.31. In this case the TINE<br>protocol don't allow to fetch meta information<br>for connected server links.");
				builder.append("</center></p><br>");
			}
			break;
			
		default:
			switch(node.getCondition()) {
			
			case FAILURE_CONNECTION:
				if (node.getStatusLinkCode() != TErrorList.success) {
					builder.append("<p><center>");
					builder.append("The following error occurs on this connection:<br>").append(node.getStatusLink());
					if (node.getStatusLinkCode() == TErrorList.illegal_property 
							|| node.getStatusLinkCode() == TErrorList.link_blacklisted)
						builder.append("<br><br>This error can happen on older TINE releases,<br>for example version 3.31. In this case the TINE<br>protocol don't allow to fetch meta information<br>for connected server links.");
					builder.append("</center></p><br>");
				}
				break;
				
			case ORDINARY_NODE:
				if (node.getStatusLinkCode() != TErrorList.success) {
					builder.append("<p><center>");
					builder.append("The following error occurs on this connection:<br>").append(node.getStatusLink());
					builder.append("</center></p><br>");
				}
				break;
			
			case CYCLE_LOOP_NODE:
				builder.append("<p><center>");
				builder.append("Cycle loop of connections exists!");
				builder.append("</center></p><br>");
				break;
			
			case DEPTH_EXCEEDED:
				builder.append("<p><center>");
				builder.append("There exists more child links underneath this connection!<br>Enlarge behind the label of '").append(TarantulaPanel.STR_STRUCTURE_DEPTH).append("' the measurement<br>of the tree level depth.");
				builder.append("</center></p><br>");
				break;
			
			case STOP_NODE:
				builder.append("<p><center>");
				builder.append("This is the final end link! There are no<br>more links underneath this connection.");
				builder.append("</center></p><br>");
				break;
			
			case TIME_EXCEEDED:
				builder.append("<p><center>");
				builder.append("Time has exceeded! Enlarge the maximum loop time<br>").append("behind the label '").append(TarantulaPanel.STR_MAX_LOOP_TIME).append("'.");
				builder.append("</center></p><br>");
				break;
				
			case IGNORE_SERVER:
				builder.append("<p><center>");
				builder.append("The special server of '").append(node.getServer()).append("' is ignored!");
				builder.append("</center></p><br>");
				break;
	
			case FAILURE_INTERNAL:
				builder.append("<p><center>");
				builder.append("An internal error occurred! Connection parameters:<br>").append("Address: ").append(node.getFullAddress()).append("<br>").append("Status: ").append(node.getStatusLink()).append("<br>").append("Status flag: ").append(node.getStatus());
				if (TarantulaPanel.getVersion() != null) {
					final Version version = TarantulaPanel.getVersion();
					builder.append("<br><br>Contact the delveloper ").append(version.developers()[0]).append(" (").append(version.group()).append("), phone ").append(version.phones()[0]).append(" at DESY, Germany!");
				}
				else
					builder.append("<br><br>Contact a developer from MCS1 at DESY, Germany!");
				builder.append("</center></p><br>");
				break;
			
			default:
				builder.append("<p><center>");
				builder.append("DEFAULT! Connection parameters:<br>").append("Address: ").append(node.getFullAddress()).append("<br>").append("Status: ").append(node.getStatusLink()).append("<br>").append("Status flag: ").append(node.getStatus());
				builder.append("</center></p><br>");
				break;
			}
		}
		builder.append("<center>").append(node.getInfoString()).append("</center>");
		return TineConnection.includeHTMLTag(builder.toString());
	}
	
	static final DateFormat DATE_FORMAT = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM, Locale.GERMAN);

	/**
	 * Gets the special logger for the class.
	 * @param clazz Class.
	 * @return Logger.
	 */
	static Logger getLogger(final Class<?> clazz) {
		
		final Logger log = Logger.getLogger(clazz.getName());
	    log.setUseParentHandlers(false);

	    final Handler handler = new ConsoleHandler();
	    handler.setFormatter(new Formatter() {
	    	
	    	@Override public String format(LogRecord record) {
	    		if (record.getThrown() == null)
	    			return "[" + record.getLevel() + "] "
	      				+ Utility.DATE_FORMAT.format(new Date(record.getMillis())) + ", "
	      				+ record.getSourceClassName().substring(record.getSourceClassName().lastIndexOf('.') + 1) + '.'
	      				+ record.getSourceMethodName() + ": "
	      				+ record.getMessage() + "\n";
	    		
	    	  	final StringBuilder builder = new StringBuilder();
	    	  	builder.append('[').append(record.getLevel()).append("] ")
	    	  		.append(Utility.DATE_FORMAT.format(new Date(record.getMillis()))).append(", ")
					.append(record.getSourceClassName().substring(record.getSourceClassName().lastIndexOf('.') + 1)).append('.')
					.append(record.getSourceMethodName()).append(": ")
	    	  		.append(record.getMessage()).append('\n');

	    	  	Throwable cause = record.getThrown();
				builder.append("exception ").append(cause.getClass().getName()).append(": ").append(cause.getMessage()).append('\n');
				while (cause.getCause() != null) {
    				cause = cause.getCause();
    				builder.append("cause by ").append(cause.getClass().getName()).append(": ").append(cause.getMessage()).append('\n');
				}
	    	  	for(final StackTraceElement trace: cause.getStackTrace())
	    	  		builder.append(trace.getClassName()).append('.').append(trace.getMethodName()).append(" (").append(trace.getLineNumber()).append(")\n");
	    		
	    	  	return builder.toString();
	      }
	    });
	    handler.setLevel(Level.FINEST);
	    log.addHandler(handler);
	    
	    return log;
	}

	/**
	 * Added a border to the component. If the title is {@code null} or empty
	 * then a norm line border is drawn. The title can also be {@code null} in 
	 * which cases this method draws a simple line border around the component.
	 * 
	 * @param title Title
	 * @param component Component. If the component is not set an 
	 * 		{@code IllegalArgumentException} is thrown.
	 */
	static void addBorder(final String title, final JComponent component)
	{
		if (component == null)
			throw new IllegalArgumentException("Component is null -> cannot set the border.");

		if (title == null || title.isEmpty())
			addBorder(component);
		else
			component.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(LIGHT_BLUE), 
					title , 
					TitledBorder.DEFAULT_JUSTIFICATION, 
					TitledBorder.DEFAULT_POSITION, 
					FONT_BORDER_TITLE, 
					Color.BLACK));
	}
	
	/**
	 * Adds a new line border around the component. 
	 * 
	 * @param component Component.
	 */
	public static void addBorder(final JComponent component) {
		component.setBorder(BorderFactory.createLineBorder(LIGHT_BLUE));
	}

	/**
	 * Centers the child component inside the rectangle. If one the arguments
	 * is {@code null} no centering is performed and the method will exit
	 * immediately. If during the calculation the result is a negative number
	 * then 0 is taken as the preference argument. 
	 * 
	 * @param parent Parent rectangle.
	 * @param child Child component.
	 */
	static void centerComponent(final Rectangle parent, final Component child)
	{
		if (parent != null && child != null)
			child.setLocation(
					Math.max(0, ((int) (parent.getX() + 
						((parent.getWidth() - child.getSize().width) / 2)))),
					Math.max(0, ((int) (parent.getY() + 
						((parent.getHeight() - child.getSize().height) / 2)))));
	}
	
	/**
	 * Centers the child component inside the dimension of the area. If one 
	 * of the arguments is {@code null} no centering is performed and the 
	 * method will exit immediately. If during the calculation the result is
	 * a negative number then 0 is taken as the preference argument. 
	 * Internally calls the {@link #centerComponent(Rectangle, Component)}
	 * 
	 * @param parent Parent rectangle.
	 * @param child Child component.
	 * @see #centerComponent(Rectangle, Component)
	 */
	static void centerComponent(final Dimension dim, final Component child) {
		if (dim != null && child != null)
			centerComponent(new Rectangle(dim), child);
	}
}
/* end of file */