/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import java.awt.datatransfer.DataFlavor;
import java.awt.event.MouseEvent;
import java.beans.PropertyVetoException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.util.CommonException;

/**
 * TransferHandler which operates with <code>MultipleAcopDisplayer</code>
 * that handles with <code>AcopGraphParameters</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class AcopChartTransferHandler extends GraphParametersTransferHandler{
	
	private static final long serialVersionUID = 4277524441131938702L;

	/**
	 * Constructs a new AcopChartTransferHandler.
	 * 
	 * @param comp
	 * @param reciveEnabled
	 * @param exportEnabled
	 * @param f
	 * @param resetFlavors
	 * @see GraphParametersTransferHandler#GraphParametersTransferHandler(JComponent, boolean, boolean, DataFlavor[], boolean)
	 */
	public AcopChartTransferHandler(MultipleAcopDisplayer<AcopGraphParameters> comp, boolean reciveEnabled, boolean exportEnabled, DataFlavor[] f, boolean resetFlavors) {
		super((JComponent)comp, reciveEnabled, exportEnabled, f, resetFlavors);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#isMouseDragGesture(java.awt.event.MouseEvent)
	 */
	@Override
	protected boolean isMouseDragGesture(MouseEvent e) {
		AcopGraphParameters[] agp = exportMultipleDisplayerParameters();
		boolean ok = false;
		for(AcopGraphParameters p : agp) {
			if (p.getConnectionParameters() != null) ok = true;
		}
		
		return super.isMouseDragGesture(e) && 
			(e.isControlDown() && ok);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.GraphParametersTransferHandler#exportMultipleDisplayerParameters()
	 */
	@SuppressWarnings("unchecked")
	@Override
	protected AcopGraphParameters[] exportMultipleDisplayerParameters() {
		return ((MultipleAcopDisplayer<AcopGraphParameters>)displayer).getDisplayerParameters();
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#setParametersToDisplayer(java.lang.Object, com.cosylab.gui.displayers.DisplayerParameters)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean setParametersToDisplayer(Object disp, DisplayerParameters parameters) throws PropertyVetoException {
		try {
			if (parameters instanceof AcopGraphParameters) {
				((MultipleAcopDisplayer<AcopGraphParameters>)displayer).addDisplayerParameters((AcopGraphParameters) parameters);
				return true;
			}
		} catch (CommonException e) {
			e.printStackTrace();
		}
		return false;
	}
}
