/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.displayers.DoubleDisplayer;
import com.cosylab.gui.util.DefaultDisplayerParametersTransferHandler;
import com.cosylab.gui.util.DisplayerParametersFlavor;
import com.cosylab.introspection.BeanIntrospector;

import de.desy.acop.displayers.selector.ChannelSelector;
import de.desy.acop.displayers.selector.SelectorUtilities;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>AcopDisplayerTransferHandler</code> is a TransferHandler, which can handle
 * <code>AcopDisplayer</code>s and deals with <code>AcopDisplayerParametersFlavor</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class AcopDisplayerTransferHandler extends DefaultDisplayerParametersTransferHandler {

	private static final long serialVersionUID = -8218250806158424887L;

	/**
	 * Constructs new AcopDisplayerTransferHandler and associates it with
	 * the supplied displayer.
	 * 
	 * @param comp
	 */
	public AcopDisplayerTransferHandler(AcopDisplayer comp) {
		super(comp, true, true, new DataFlavor[]{AcopDisplayerParametersFlavor.FLAVOR,
				DisplayerParametersFlavor.FLAVOR,ConnectionParametersFlavor.FLAVOR, 
				DataFlavor.stringFlavor}, true);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.DefaultDisplayerParametersTransferHandler#exportDisplayerParameters()
	 */
	@Override
	protected DisplayerParameters exportDisplayerParameters() {
		AcopDisplayer disp = (AcopDisplayer)displayer; 
		ConnectionParameters parameters = disp.getConnectionParameters();
		int index = disp.getArrayIndex();
		if (disp instanceof DoubleDisplayer) {
			return new AcopDisplayerParameters(parameters, index, 
					((DoubleDisplayer)disp).getMinimum(),((DoubleDisplayer)disp).getMaximum(),
					((DoubleDisplayer)disp).getUnits(),((DoubleDisplayer)disp).getFormat(),disp.getConverter());
		}
		else return new AcopDisplayerParameters(parameters, index, disp.getConverter());
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor instanceof AcopDisplayerParametersFlavor) {
			return new AcopDisplayerParameters[]{
					(AcopDisplayerParameters) exportDisplayerParameters()};
		} else if (flavor instanceof ConnectionParametersFlavor) {
			return new ConnectionParameters[]{
					((AcopDisplayerParameters) exportDisplayerParameters()).getConnectionParameters()};
		} else if (flavor.isFlavorTextType()) {
			return ((AcopDisplayerParameters) exportDisplayerParameters()).getConnectionParameters().getRemoteName();
		} else {
			return super.getTransferData(flavor);
		}
		
//		throw new UnsupportedFlavorException(flavor);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopDisplayerParametersFlavor.FLAVOR)) {
				Object[] drop = (Object[]) t.getTransferData(AcopDisplayerParametersFlavor.FLAVOR);
				DisplayerParameters[] dp = null;
				dp = castParameters(drop);
				if (dp!=null) {
					return importDisplayerParameters(dp);
				}
			} else if (t.isDataFlavorSupported(ConnectionParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(ConnectionParametersFlavor.FLAVOR);
				
				if (drop != null) {
					ConnectionParameters[] cp = null;
					if (drop instanceof ConnectionParameters) {
						cp = new ConnectionParameters[]{(ConnectionParameters)drop};
					} else if (drop instanceof ConnectionParameters[]) {
						cp = (ConnectionParameters[])drop;
					} else {
						return super.importData(comp, t);
					}
					
					AcopDisplayerParameters[] param = new AcopDisplayerParameters[cp.length];
					for (int i = 0; i < param.length; i++) {
						param[i] = new AcopDisplayerParameters(cp[i]);
					}
					
					return importDisplayerParameters(param);
				}
			}
			return super.importData(comp, t);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
		
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#importString(java.lang.String)
	 */
	@Override
	protected boolean importString(String string) throws PropertyVetoException {
		if (string.indexOf('\n') > 0) {
			return false;
		}
		try {
			ConnectionParameters cp = new ConnectionParameters(string);
			return importDisplayerParameters(new AcopDisplayerParameters[]{new AcopDisplayerParameters(cp)});
		} catch (IllegalArgumentException e) {
			return false;
		}
		
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#castParameters(java.lang.Object[])
	 */
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopDisplayerParameters[]) {
			return (AcopDisplayerParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1) {
			if (parameters[0] instanceof AcopDisplayerParameters[]) {
				return (AcopDisplayerParameters[]) parameters[0];
			} else if (parameters[0] instanceof AcopDisplayerParameters) {
				return new AcopDisplayerParameters[] {(AcopDisplayerParameters)parameters[0]};
			}
		}
		return super.castParameters(parameters);
	}

	protected int selectChannelIndex(ConnectionParameters cp) {
		int length = SelectorUtilities.getSequenceLength(cp);
		if (length==1) {
			return 1;
		}
		return ChannelSelector.selectChannelDialog((Component) displayer, length, ((AcopDisplayer)displayer).getArrayIndex());
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#setParametersToDisplayer(java.lang.Object, com.cosylab.gui.displayers.DisplayerParameters)
	 */
	@Override
	public boolean setParametersToDisplayer(Object disp, DisplayerParameters parameters) throws PropertyVetoException {
		if (!(parameters instanceof AcopDisplayerParameters)) 
			super.setParametersToDisplayer(disp, parameters);
				
		String[] n= parameters.getNames();
		Object[] o= parameters.getValues();
		
		ConnectionParameters cp = null;
		int index = -1;
		for (int i = 0; i < n.length ; i++) {
			if (AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY.equals(n[i])) {
				cp = (ConnectionParameters) o[i];
				if (SelectorUtilities.isChannel(cp)) {
					index = 0;
				} else {
					index = selectChannelIndex(cp);
				}
			}
		}
		try {
			BeanIntrospector.setPropertyValue(disp,AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,cp);
			BeanIntrospector.setPropertyValue(disp, "arrayIndex", index);
		} catch (Exception e) {
			throw new PropertyVetoException("Property '" +AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY +"' not set: "+e.getMessage(),new PropertyChangeEvent(disp,AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,null,cp));
		}
		
		for (int i = 0; i < n.length; i++) {
			if (!AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY.equals(n[i]) && o[i]!=null && (!(o[i] instanceof Double) || !((Double)o[i]).isNaN())) {
				try {
					BeanIntrospector.setPropertyValue(disp,n[i],o[i]);
				} catch(NoSuchMethodException e) {
					//do nothing
				} catch (Exception e) {
					throw new PropertyVetoException("Property '"+n[i]+"' not set: "+e.getMessage(),new PropertyChangeEvent(disp,n[i],null,o[i]));
				}
			}
		}
		return true;
	}
}
