/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import de.desy.acop.displayers.AcopTrendChart;
import de.desy.acop.displayers.chart.HistoryParameters;

/**
 * <code>AcopGraphHistoryParameters</code> is an extension of the 
 * <code>AcopGrapParameters</code> which can in addition holds also
 * <code>HistoryParameters</code>. This parameters are inteded to be used
 * in combinataion with <code>AcopTrendChart</code>.
 * 
 * @author Tilen Kusterle, Cosylab
 * @see AcopTrendChart
 * @see HistoryParameters
 */
public class AcopGraphHistoryParameters extends AcopGraphParameters {

	
	private static final long serialVersionUID = 7485604445974438082L;

	/**
	 * Constructs new AcopGraphhistoryParameters including all parameters
	 * from the <code>acopGraphParameters</code> and the supplied
	 * <code>HistoryParameters</code>.
	 * 
	 * @param acopGraphParameters parameters holder
	 * @param historyParameters history connection point
	 */
	public AcopGraphHistoryParameters(AcopGraphParameters acopGraphParameters, HistoryParameters historyParameters) {
		super(
				acopGraphParameters.getConnectionParameters(),
				acopGraphParameters.getArrayIndex(),
				acopGraphParameters.getMinimum(),
				acopGraphParameters.getMaximum(),
				acopGraphParameters.getUnits(),
				acopGraphParameters.getFormat(),
				acopGraphParameters.getColor(),
				acopGraphParameters.getFFT(),
				acopGraphParameters.getDrawStyle(),
				acopGraphParameters.getMode(),
				acopGraphParameters.getWidth(),
				acopGraphParameters.isTrend(),
				acopGraphParameters.getTrendLength(),
				acopGraphParameters.getConverter()
				);
		values.put("historyParameters", historyParameters);
	}
	
	/**
	 * Constructs a new AcopGraphHistoryParameters with null HistoryParameters.
	 * 
	 * @param acopGraphParameters
	 */
	public AcopGraphHistoryParameters(AcopGraphParameters acopGraphParameters) {
		this(acopGraphParameters, null);
	}

	/**
	 * Returns the HistoryParameters of this parameters.
	 * 
	 * @return the historyParameters
	 */
	public HistoryParameters getHistoryParameters() {
		return (HistoryParameters) values.get("historyParameters");
	}

	/**
	 * Sets the HistoryParameters to this parameters container.
	 * 
	 * @param historyParameters the historyParameters to set
	 */
	public void setHistoryParameters(HistoryParameters historyParameters) {
		values.put("historyParameters", historyParameters);
	}

}
