/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JPanel;

import com.cosylab.gui.InfoDialog;
import com.cosylab.gui.displayers.Displayer;

import de.desy.acop.displayers.info.AcopDebugPanel;
import de.desy.acop.displayers.info.CharacteristicsPanel;
import de.desy.acop.displayers.info.ConnectionPanel;
import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.adapters.AdapterFactoryService;
import de.desy.tine.client.TLinkFactory;

/**
 * <code>AcopInfoDialog</code> is an extension of <code>InfoDialog</code>
 * which in addition to existing data also shows connection details and 
 * characteristics of property that is connected with the displayer that this
 * dialog is attached to.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class AcopInfoDialog extends InfoDialog {
	
	private static final long serialVersionUID = 513076582058739594L;
	
	private ConnectionPanel connectionPanel;
	private CharacteristicsPanel characteristicsPanel;
	private AcopDebugPanel acopDebugPanel;
	private JPanel debugCompositionPanel;
		
	private PropertyChangeListener propertyListener;
		
	/**
	 * Constructs a new AcopInfoDialog.
	 *
	 */
	public AcopInfoDialog() {
		this(null);
	}
	
	/**
	 * Constructs a new AcopInfoDialog and associates it with the given displyer.
	 * 
	 * @param displayer
	 */
	public AcopInfoDialog(AcopDisplayer displayer) {
		super(displayer);
		setSize(500,400);
		this.addWindowListener(new WindowAdapter(){
			@Override
			public void windowActivated(WindowEvent e) {
				getAcopDebugPanel().setDebug(TLinkFactory.getInstance().getDebugLevel());
			}
		});
		this.addPanel(getConnectionPanel(), "Connection", 0, true);
		this.addPanel(getCharacteristicsPanel(), "Prop Info");
		this.addPanel(getDebugCompositionPanel(), "Debug", 1, false);
	}
	
	private ConnectionPanel getConnectionPanel() {
		if (connectionPanel == null) {
			connectionPanel = new ConnectionPanel();
		}
		return connectionPanel;
	}
	
	private CharacteristicsPanel getCharacteristicsPanel() {
		if (characteristicsPanel == null) {
			characteristicsPanel = new CharacteristicsPanel();
		}
		return characteristicsPanel;
	}
	
	private AcopDebugPanel getAcopDebugPanel() {
		if (acopDebugPanel == null) {
			acopDebugPanel = new AcopDebugPanel();
		}
		return acopDebugPanel;
	}
	
	private JPanel getDebugCompositionPanel() {
		if (debugCompositionPanel == null) {
			debugCompositionPanel = new JPanel(new GridBagLayout());
			debugCompositionPanel.add(getAcopDebugPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,1,11),0,0));
			debugCompositionPanel.add(getDebugPanel(), new GridBagConstraints(0,1,1,1,1,0.1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(1,11,11,11),0,0));
		}
		return debugCompositionPanel;		
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.InfoDialog#setDisplayer(com.cosylab.gui.displayers.Displayer)
	 */
	@Override
	public void setDisplayer(Displayer displayer) {
		
		if (getDisplayer() != null && !getDisplayer().equals(displayer)) {
			getDisplayer().removePropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,getPropertyListener());
			if (displayer != null)
				displayer.addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,getPropertyListener());
		} else if (getDisplayer() == null && displayer != null) {
			displayer.addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,getPropertyListener());
		}

		super.setDisplayer(displayer);
		if (getDisplayer() != null) {
			setTitle("Info for "+getDisplayer().getClass().getSimpleName());
			getConnectionPanel().setConnectionParameters(((AcopDisplayer)getDisplayer()).getConnectionParameters());
			getAcopDebugPanel().setConnectionParameters(((AcopDisplayer)getDisplayer()).getConnectionParameters());
			try {
				getCharacteristicsPanel().setCharacteristics(AdapterFactoryService.getInstance().getAdapterFactory().getCharacteristics(((AcopDisplayer)getDisplayer()).getConnectionParameters()));
			} catch (ConnectionFailed e) {
				e.printStackTrace();
			}
		} else {
			setTitle("Info dialog");
			getConnectionPanel().setConnectionParameters(null);
			getAcopDebugPanel().setConnectionParameters(null);
			getCharacteristicsPanel().setCharacteristics(null);
		}
		getAcopDebugPanel().setDebug(TLinkFactory.getInstance().getDebugLevel());
		
	}
	
	private PropertyChangeListener getPropertyListener() {
		if (propertyListener == null) {
			propertyListener = new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) {
						getConnectionPanel().setConnectionParameters(((AcopDisplayer)getDisplayer()).getConnectionParameters());
						getAcopDebugPanel().setConnectionParameters(((AcopDisplayer)getDisplayer()).getConnectionParameters());
						try {
							getCharacteristicsPanel().setCharacteristics(AdapterFactoryService.getInstance().getAdapterFactory().getCharacteristics(((AcopDisplayer)getDisplayer()).getConnectionParameters()));
						} catch (ConnectionFailed e) {
							e.printStackTrace();
						}
					}
				}
			};
		}
		return propertyListener;
	}
	
	public static void main(String[] args) {
		new AcopInfoDialog().setVisible(true);
	}
	
}
