/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.util.DisplayerParametersFlavor;

import de.desy.acop.displayers.AcopTable;
import de.desy.acop.displayers.selector.AcopTableParametersDialog;
import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.table.AcopTableParametersFlavor;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>AcopTableTransferHandler</code> is an extension of TransferHandler
 * than can handle <code>AcopTableParameters</code>.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopTableParameters
 */
public class AcopTableTransferHandler extends AcopMultipleDisplayerParametersTransferHandler{
	
	private static final long serialVersionUID = 4277524441131938702L;
	private AcopTableParametersDialog tableDialog;

	public AcopTableTransferHandler(AcopTable comp, boolean reciveEnabled, boolean exportEnabled, DataFlavor[] f, boolean resetFlavors) {
		super(comp, reciveEnabled, exportEnabled, f, resetFlavors);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor.equals(AcopTableParametersFlavor.FLAVOR)) {
			return exportDisplayerParameters();
		} else if (flavor instanceof DisplayerParametersFlavor) {
			return exportDisplayerParameters();
		} else if (flavor instanceof ConnectionParametersFlavor) {
			return ((AcopTableParameters)exportDisplayerParameters()).getConnectionParameters();
		} else if (flavor.isFlavorTextType()){
			return ((AcopTableParameters)exportDisplayerParameters()).getConnectionParameters().toString();
		} else {
			return super.getTransferData(flavor);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AbstractMDPTransferHandler#exportDisplayerParameters()
	 */
	@Override
	protected DisplayerParameters exportDisplayerParameters() {
		return ((AcopTable)displayer).getSelectedParameters();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	protected AcopTableParameters convertParameters(AcopDisplayerParameters adp) {
		if (tableDialog == null) {
			tableDialog = new AcopTableParametersDialog(displayer);
		}
		return tableDialog.showDialog(displayer,adp);	
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	@Override
	protected AcopTableParameters convertParameters(ConnectionParameters adp) {
		if (tableDialog == null) {
			tableDialog = new AcopTableParametersDialog(displayer);
		}
		return tableDialog.showDialog(displayer,adp);	
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#castParameters(java.lang.Object[])
	 */
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopTableParameters[]) {
			return (AcopTableParameters[])parameters.getClass().cast(parameters);
		} else if (parameters instanceof AcopDisplayerParameters[]) {
			return (AcopDisplayerParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1) {
			if (parameters[0] instanceof AcopTableParameters) {
				return new AcopTableParameters[] {(AcopTableParameters)parameters[0]};
			} else if (parameters[0] instanceof AcopDisplayerParameters) {
				return new AcopDisplayerParameters[] {(AcopDisplayerParameters)parameters[0]};
			}
		} 
		return super.castParameters(parameters);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopTableParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(AcopTableParametersFlavor.FLAVOR);
				
				if (!(drop instanceof AcopTableParameters[])) {
					return false;
				}
				
				AcopTableParameters[] p = (AcopTableParameters[])drop;
				for (int i = 0; i < p.length; i++) {
					setParametersToDisplayer(displayer,p[i]);
				}
				return true;
			} 			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (super.importData(comp,t)) {
			return true;
		}		

		return false;
	}
	
}
