/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import java.util.EventObject;

import de.desy.acop.displayers.AcopTrendChart;

/**
 * <code>AcopTrendChartEvent</code> is an event triggered be 
 * <code>AcopTrendChart</code> and is listened by the <code>AcopTrendChartListener</code>.
 * 
 * @author Tilen Kusterle, Cosylab
 * @see AcopTrendChart
 */
public class AcopTrendChartEvent extends EventObject {
	
	private static final long serialVersionUID = 1L;
	
	public static final String TIMEOUT = "Timeout";
	public static final String NO_DATA = "No data";

	public static enum EventType {OPERATION_STARTED,MULTIPLE_OPERATION_STARTED,OPERATION_ENDED,MULTIPLE_OPERATION_ENDED,ERROR,PROGRESS}
	
	private EventType type;
	private String message;
	private AcopGraphHistoryParameters currentElement;
	private Throwable throwable;
	
	
	public AcopTrendChartEvent(Object source) {
		super(source);
	}
	
	public AcopTrendChartEvent(Object source, EventType type, String message) {
		this(source, type, message, null, null);
	}
	
	public AcopTrendChartEvent(Object source, EventType type, String message, AcopGraphHistoryParameters currentElement) {
		this(source, type, message, currentElement, null);
	}
	
	public AcopTrendChartEvent(Object source, EventType type, Throwable throwable) {
		this(source, type, null, null, throwable);
	}
	
	/**
	 * @param source
	 * @param type
	 * @param message
	 * @param currentElement
	 * @param throwable
	 */
	public AcopTrendChartEvent(Object source, EventType type, String message, AcopGraphHistoryParameters currentElement, Throwable throwable) {
		super(source);
		this.type = type;
		this.message = message;
		this.currentElement = currentElement;
		this.throwable = throwable;
	}


	/**
	 * @return the currentElement
	 */
	public AcopGraphHistoryParameters getCurrentElement() {
		return currentElement;
	}


	/**
	 * @return the message
	 */
	public String getMessage() {
		return message;
	}


	/**
	 * @return the type
	 */
	public EventType getType() {
		return type;
	}

	/**
	 * @return the exception
	 */
	public Throwable getThrowable() {
		return throwable;
	}
	
}
