package de.desy.acop.displayers.tools;

import java.util.Locale;

import com.cosylab.util.PrintfFormat;

public class AcopUtilities {
	
	private static PrintfFormat normalF = new PrintfFormat(Locale.UK, "%4.2f");
	private static PrintfFormat extendedF = new PrintfFormat(Locale.UK, "%4.6f");
	private static PrintfFormat normalSciF = new PrintfFormat(Locale.UK, "%4.2E2");
	private static PrintfFormat extendedSciF = new PrintfFormat(Locale.UK, "%4.6E2");
	private static PrintfFormat superExtendedF = new PrintfFormat(Locale.UK, "%4.7f");
	private static PrintfFormat superExtendedSciF = new PrintfFormat(Locale.UK, "%4.7E2");
	private static PrintfFormat hexF = new PrintfFormat(Locale.UK, "%x");
	
	
	public static void main(String[] args) {
		String s = AcopUtilities.formatValue(0.009999999776482582,false, true);
		System.out.println(s);
		
					
//		double value = 6.32321E5;
//		String s = formatValue(value,true);
//		int index = s.indexOf("E");
//		
//		double first = Double.parseDouble(s.substring(0,index));
//		int second = Integer.parseInt(s.substring(index+1));
//		int part = second%3;
//		int rest = second/3;
//		if (part != 0) {
//			if (second > 0 || part == -1) {
//    			first *= Math.pow(10,part);
//    			s = formatValue(first,true) + "E" + (rest*3);
//    		} else {
//    			if (part == -2) {
//    				first *= Math.pow(10,3+part);
//        			s = formatValue(first,true)  + "E" + ((rest-1)*3);
//    			} else {
//    				first *= Math.pow(10,part);
//        			s = formatValue(first,true)  + "E" + ((rest)*3);
//    			}    			
//    		}
//		}
//		System.out.println(s);
		
		
	}
	
	/**
	 * Formats the value that its exponential part is always in the power of 3
	 * @param value the value to be formatted
	 * @param extraDigits
	 * @return
	 */
	public static String formatValueInPowerOf3(double value, boolean extraDigits) {
		String s = formatValue(value,extraDigits);
		int index = s.indexOf("E");
		if (index < 0) return s;
		double first = Double.parseDouble(s.substring(0,index));
		int second = Integer.parseInt(s.substring(index+1));
		int part = second%3;
		int rest = second/3;
		if (part != 0) {
			if (second > 0 || part == -1) {
    			first *= Math.pow(10,part);
    			s = formatValue(first,extraDigits) + "E" + (rest*3);
    		} else {
    			if (part == -2) {
    				first *= Math.pow(10,3+part);
        			s = formatValue(first,extraDigits)  + "E" + ((rest-1)*3);
    			} else {
    				first *= Math.pow(10,part);
        			s = formatValue(first,extraDigits)  + "E" + ((rest)*3);
    			}    			
    		}
		}
		return s;
	}
	
	/**
	 * Formats a value according to the given extra digits parameter.
	 * 
	 * @param value
	 * @param extraDigits
	 * @return
	 */
	public static String formatValue(double value, boolean extraDigits) {
		return formatValue(value,extraDigits,false);
	}
	/**
	 * Formats the value according the given extra digits parameter. Value is 
	 * also checked if it should be displayed in scientific format.
	 * 
	 * @param value the value to format
	 * @param extraDigits if extra decimal digits should be used
	 * @param superExtraDigits if even more digits should be used
	 * @return
	 */
	public static String formatValue(double value, boolean extraDigits, boolean superExtraDigits) {
		String str = null;
		if (extraDigits) {
			if (isValueScientific(value)) {
				if (superExtraDigits) {
					str = superExtendedSciF.sprintf(value);
				} else {
					str = extendedSciF.sprintf(value);
				}
			} else {
    			if (superExtraDigits) {
    				str = superExtendedF.sprintf(value);
    			} else {
    				str = extendedF.sprintf(value);
    			}
			}
		} else {
			if (isValueScientific(value)) {
				str = normalSciF.sprintf(value);
			} else {
				str = normalF.sprintf(value);
			}
		}
		
		if (str != null && str.indexOf("E+") >= 0) {
			int idx = str.indexOf("E+");
			str = str.substring(0,idx+1) + str.substring(idx+2);
		} 
		
		//bugfix for sprintf: in some weird cases, empty characters are inserted (e.g. 0.009999999776482582)
		char[] chars = str.trim().toCharArray();
		for (int i = 0; i < chars.length; i++) {
			if (chars[i] == 0) {
				chars[i] = '0';
			}
		}
		return new String(chars);
	}
	
	/**
	 * Formats the value to hex string (0xValue).
	 * 
	 * @param value the value
	 * @return the hex string
	 */
	public static String formatHex(long value) {
		return "0x" + hexF.sprintf(value);
	}
	
	/**
	 * Checks if the value is in the range to be displayed in a scientific
	 * format (exponential).
	 * 
	 * @param val
	 * @return
	 */
	public static boolean isValueScientific(double val) {
		return (Math.abs(val) < 0.01 || Math.abs(val) > 9999) && val != 0.;
	}
	
}
