package de.desy.acop.displayers.tools;

import com.cosylab.util.NumericStringComparator;

/**
 * 
 * <code>CaseInsensitiveComparator</code> is a case insensitive comparator, which extends
 * the numeric string comparator and uses a backslash as a separator to avoid problems with
 * dots and commas.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class CaseInsensitiveComparator extends NumericStringComparator {
	
	private static CaseInsensitiveComparator defaultInstance = new CaseInsensitiveComparator();
	
	
	public static CaseInsensitiveComparator getDefault() {
		return defaultInstance;
	}
	
	public CaseInsensitiveComparator() {
		super(new char[]{'\n','-'}, '\\');
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.util.NumericStringComparator#compare(java.lang.Object, java.lang.Object)
	 */
	@Override
	public int compare(Object o1, Object o2) {
		String s1 = String.valueOf(o1).trim();
		String s2 = String.valueOf(o2).trim();
		return super.compare(s1.toUpperCase(),s2.toUpperCase());
	}
}
