/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.displayers.tools;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;

import de.desy.acop.displayers.selector.AcopGraphParametersDialog;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>GraphParametersTransferHandler</code> is an extension of 
 * TransferHandler which can handle <code>AcopGraphParameters</code>
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @see AcopGraphParameters
 */
public abstract class GraphParametersTransferHandler extends AbstractMDPTransferHandler{
	
	private static final long serialVersionUID = 4277524441131938702L;
	private AcopGraphParametersDialog graphDialog;

	/**
	 * Constructs new GraphParametersTransferHandler.
	 * 
	 * @param comp
	 * @param reciveEnabled
	 * @param exportEnabled
	 * @param f 
	 * @param resetFlavors
	 * @see AbstractMDPTransferHandler#AbstractMDPTransferHandler(JComponent, boolean, boolean, DataFlavor[], boolean)
	 */
	public GraphParametersTransferHandler(JComponent comp, boolean reciveEnabled, boolean exportEnabled, DataFlavor[] f, boolean resetFlavors) {
		super(comp, reciveEnabled, exportEnabled, f, resetFlavors);
	}
	
	@Override
	protected boolean importString(String string) throws PropertyVetoException {
		ConnectionParameters cp = new ConnectionParameters(string);
		return importDisplayerParameters(new AcopGraphParameters[]{new AcopGraphParameters(cp)});
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor instanceof AcopGraphParametersFlavor){
			DisplayerParameters[] p = exportMultipleDisplayerParameters();
			return p;
		} else {
			return super.getTransferData(flavor);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	protected AcopDisplayerParameters convertParameters(AcopDisplayerParameters adp) {
		if (graphDialog == null) {
			graphDialog = new AcopGraphParametersDialog(displayer);
		}
		AcopGraphParameters p = graphDialog.showDialog(displayer,adp);
		
		return p;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	@Override
	protected AcopDisplayerParameters convertParameters(ConnectionParameters adp) {
		if (graphDialog == null) {
			graphDialog = new AcopGraphParametersDialog(displayer);
		}
		return graphDialog.showDialog(displayer,adp);	
	}
	
	/**
	 * Override to provide conversion from AcopGraphParameters to higher level 
	 * parameters.
	 * @param agp
	 * @return
	 */
	protected AcopDisplayerParameters convertParameters(AcopGraphParameters agp) {
		return agp;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#castParameters(java.lang.Object[])
	 */
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopGraphParameters[]) {
			return (AcopGraphParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1 && parameters[0] instanceof AcopGraphParameters) {
			return new AcopGraphParameters[] {(AcopGraphParameters)parameters[0]};
		} 
		return super.castParameters(parameters);
	}
		
	/* (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopGraphParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(AcopGraphParametersFlavor.FLAVOR);
				
				if (!(drop instanceof AcopGraphParameters[])) {
					return false;
				}
				
				AcopGraphParameters[] p = (AcopGraphParameters[])drop;
				for (int i = 0; i < p.length; i++) {
					AcopDisplayerParameters a = convertParameters(p[i]);
					if (a!=null) {
						importDisplayerParameters(new AcopDisplayerParameters[]{a});
					}
				}
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (super.importData(comp,t)) {
			return true;
		}		
		return false;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AbstractMDPTransferHandler#exportMultipleDisplayerParameters()
	 */
	@Override
	protected abstract AcopDisplayerParameters[] exportMultipleDisplayerParameters();
	
}
