package de.desy.acop.displayers.tools;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Thread factory to be used with applications. 
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class TINEThreadFactory implements ThreadFactory {
    private static final AtomicInteger poolNumber = new AtomicInteger(1);
    private final ThreadGroup group;
    private final AtomicInteger threadNumber = new AtomicInteger(1);
    private final String namePrefix;
    private final boolean lowPriority;
    private final boolean deamon;

    /**
     * Constructs a new factory with the given name.
     * @param name
     * @param lowPriortiy should the threads from this factory have priority set to low
     * @param deamon if true or created threads are deamons
     */
    public TINEThreadFactory(String name, boolean lowPriority, boolean deamon) {
        group = Thread.currentThread().getThreadGroup();
        namePrefix = name + "-" +
                      poolNumber.getAndIncrement() +
                     "-thread-";
        this.lowPriority = lowPriority;
        this.deamon = deamon;
    }
    
    /**
     * Constructs a new factory with the given name.
     * @param name
     */
    public TINEThreadFactory(String name) {
    	this(name, false, false);
    }

    /*
     * (non-Javadoc)
     * @see java.util.concurrent.ThreadFactory#newThread(java.lang.Runnable)
     */
    public Thread newThread(Runnable r) {
        Thread t = new Thread(group, r,
                              namePrefix + threadNumber.getAndIncrement(),
                              0);
        t.setDaemon(deamon);
            
        if (t.getPriority() != Thread.NORM_PRIORITY)
            t.setPriority(Thread.NORM_PRIORITY);
        if (lowPriority) {
        	t.setPriority(Thread.MIN_PRIORITY);
        }
        return t;
    }
}
