/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Date;
import java.util.Vector;
import javax.swing.event.EventListenerList;
import de.desy.acop.chart.Acop;
import de.desy.acop.displayers.tools.ConnectionParametersReceiver;

public class AcopTransport implements ConnectionParametersReceiver
{
  private ConnectionParameters connectionParameters = null;
  public String getConnectionInformation(String deviceContext,String deviceGroup,String deviceName, String deviceProperty)
  {
    AcopTransportPlug atp;
    if ((atp = atf.getProtocol(connectionParameters.getAccessProtocol())) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getTargetInformation(deviceContext,deviceGroup,deviceName,deviceProperty);
  }
  public String getConnectionInformation(int hLink)
  {
    if (hLink >= 0)
    {
      AcopTransportRequest a;
      for (int i=0; i<acopLinkTable.size(); i++)
      {
        a = acopLinkTable.elementAt(i);
        if (a.getLinkIdentifier() == hLink)
        {
          return a.getDeviceInfo();
        }
      }
    }
    return null;
  }
  protected String acopConfig, Status;
  /**
   * @brief The grouped flag
   * @return the isGrouped state of this transport
   */
  public boolean isGrouped;
  protected int GroupIndex, ReceiveQueueDepth;
  public int getReceiveQueueDepth()
  {
    return ReceiveQueueDepth;
  }
  protected double TimeStamp;
  protected int systemStamp;
  protected int userStamp;
  
  public int getUserStamp() {
	return userStamp;
  }
  public int getSystemStamp() {
    	return systemStamp;
  }
  public double getDataTimeStamp()
  {
    return TimeStamp;
  }
  public double getDataTimeStamp(int hLink) 
  { 
    return ((double)(getDateStamp(hLink)).getTime()/1000);
  }
  protected Date dateStamp;
  public Date getDateStamp()
  {
    return dateStamp;
  }
  public Date getDateStamp(int hLink) 
  { 
    if (hLink >= 0)
    {
      AcopTransportRequest a;
      for (int i=0; i<acopLinkTable.size(); i++)
      {
        a = acopLinkTable.elementAt(i);
        if (a.getLinkIdentifier() == hLink)
        {
          return a.getDataTimeStamp();
        }
      }
    }
    return dateStamp;
  }
  // private Acop parentAcop;
  protected AcopTransportFactory atf;
  // protected AcopTransportRequest atr = new AcopTransportRequest();
  protected final Vector<AcopTransportRequest> acopLinkTable = new Vector<>();
  public Vector<AcopTransportRequest> getAcopLinkTable()
  {
    return acopLinkTable;
  }
  protected int lastUpdatedLink = -1, lastUpdatedDataSize = 0, lastUpdatedDataSizeEx = 0;
  protected EventListenerList listeners = new EventListenerList();
  public boolean verbose = false;
  public class AcopCallback implements AcopTransportCallback
  {
    public void callback(int linkHandle, int statusCode)
    {
      int linkTableId, identifier, i;
      AcopTransportRequest a;
      if ((linkTableId = getAcopLinkTableIdFromCallbackId(linkHandle)) < 0)
      {
        if (verbose) System.out.println("Link " + linkHandle + " not allocated");
        return;
      }
      if ((a =  acopLinkTable.elementAt(linkTableId)) == null)
      {
        if (verbose) System.out.println("Link " + linkHandle + " no entry in link table!");
        return;
      }
      boolean pending = false;
      getData(a.getData(), a.getDataEx(), a.getLinkIdentifier());
      identifier = a.getLinkIdentifier();
      if (isGrouped)
      {
        for (i = 0; i < acopLinkTable.size(); i++)
        {
          if (i == linkTableId) (acopLinkTable.elementAt(i)).pending = false;
          if (( acopLinkTable.elementAt(i)).pending) pending = true;
        }
        if (pending) return;
      }
      else
      {
        a.pending = false;
      }
      fireAcopTransportEvent(identifier, statusCode);
      if (isGrouped)
      {
        for (i = 0; i < acopLinkTable.size(); i++)
          (acopLinkTable.elementAt(i)).pending = true;
      }
      else
      {
        a.pending = true;
      }
      // TODO check this out (does it work ?)
      if (a.accessMode.isReadMode() || a.accessMode.isWriteMode())
      {
       // closeLink(a.getLinkHandle());
      }
    }
    private void fireAcopTransportEvent(int identifier, int statusCode)
    {
      AcopTransportEvent ev = new AcopTransportEvent(AcopTransport.this, identifier, statusCode);
      AcopTransportListener[] l = listeners.getListeners(AcopTransportListener.class);
      for (int i = 0; i < l.length; i++)
      {
        try
        {
          l[i].dataEventReceived(ev);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
      }
    }
  }
  protected AcopTransportCallback atc = new AcopCallback();
  public int addAcopTransportProtocol(AcopTransportPlug plug)
  {
    return atf.addProtocol(plug);
  }
  public AcopTransport()
  {
    // {{INIT_CONTROLS
    // parentAcop = acop;
    acopConfig = "STANDARD";
    Status = "";
    ReceiveQueueDepth = 100;
    TimeStamp = 0;
    isGrouped = false;
    GroupIndex = 0;
    atf = AcopTransportFactory.getInstance();
    // }}
  }
  // {{DECLARE_CONTROLS
  protected AcopTransportRequest getAcopTransportRequest(AcopTransportRequest atr)
  {
    AcopTransportRequest a;
    for (int i = 0; i < acopLinkTable.size(); i++)
    {
      a = acopLinkTable.elementAt(i);
      if (a.compareTo(atr)) return a;
    }
    a = atr.cloneTransportRequest();
    acopLinkTable.addElement(a);
    return a;
  }
  protected AcopTransportRequest getAcopTransportRequest(int hLink)
  { 
    AcopTransportRequest atr;
    if (hLink >= 0)
    { // TODO: maybe use a hash map? or maybe not (never more than a few links in the table) 
      for (int i = 0; i < acopLinkTable.size(); i++)
      {
        atr = acopLinkTable.elementAt(i);
        if (atr.getLinkIdentifier() == hLink)
        {
          return atr;
        }
      }
    }
    return null;
  }
  public int getAcopLinkTableId(AcopTransportRequest atr)
  {
    AcopTransportRequest a;
    for (int i = 0; i < acopLinkTable.size(); i++)
    {
      a = acopLinkTable.elementAt(i);
      if (a.compareTo(atr))
      {
        //TODO update the inputdata set here ?
        return i;
      }
    }
    return -1;
  }
  public int getAcopLinkTableId(int hLink)
  {
    AcopTransportRequest a;
    for (int i = 0; i < acopLinkTable.size(); i++)
    {
      a = acopLinkTable.elementAt(i);
      if (a.getLinkIdentifier() == hLink) return i;
    }
    return -1;
  }
  public int getAcopLinkTableIdFromCallbackId(int hLinkCallback)
  {
    AcopTransportRequest a;
    for (int i = 0; i < acopLinkTable.size(); i++)
    {
      a = acopLinkTable.elementAt(i);
      if (a.getLinkHandle() == hLinkCallback) return i;
    }
    return -1;
  }
  protected int getFreeIdentifier()
  {
    AcopTransportRequest a;
    int identifier = 0;
    for (int i = 0; i < acopLinkTable.size(); i++)
    {
      a = acopLinkTable.elementAt(i);
      if (a.getLinkIdentifier() == identifier) identifier++;
    }
    return identifier;
  }
  public void halt()
  {
    atf.halt();
  }
  /**
   * @brief Establishes and executes a synchronous data link using the Device
   *        Properties assigned to the control. Use Execute for synchronous data
   *        acquisition. Fill in the DeviceContext DeviceGroup, DeviceName, and
   *        DeviceProperty bean property parameters, along with the
   *        accessProtocol, accessMode, and accessRate bean property parameters.
   *        This method blocks until either the transfer is complete or a
   *        timeout has occurred (given by the accessRate setting). Asynchronous
   *        READ and WRITE calls can be effected by using the AttachLink()
   *        method and handling the results inside the corresponding receive
   *        event.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. Pass 'null' if no
   *          return data is desired.
   * @param ArraySizeEx
   *          is the data array size associated with the DataEx object. This may
   *          be smaller than the actual size of the DataEx object, but NOT
   *          bigger.
   * @return 0 if successful otherwise a positive return code or -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data and DataEx objects above must refer to arrays of data types
   *       (which are passed by reference) and not single values (which are
   *       passed by value). So even if passing a single value, it must be
   *       declared as an array of 1 element!
   * @note The only allowed values of 'accessMode' are "READ" and "WRITE" when
   *       using the Execute() method. \b Example: \include eg_acopExecute1.java
   */
  public int execute(Object Data, int ArraySize, Object DataEx, int ArraySizeEx)
  {
    if (getObjectSize(Data) < ArraySize || getObjectSize(DataEx) < ArraySizeEx)
    {
      Status = "Data object smaller than size given";
      return -1;
    }
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.setFromParameters(connectionParameters);
    atr.setData(Data);
    atr.setArraySize(ArraySize);
    atr.setDataEx(DataEx);
    atr.setArraySizeEx(ArraySizeEx);
    atr.isUnboundToData = false;
    if ((atp = atf.getProtocol(connectionParameters.getAccessProtocol())) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    int rc = atp.handleRequest(atr);
    lastUpdatedDataSize = atr.getDataSize();
    lastUpdatedDataSizeEx = atr.getDataSizeEx();
    setTimeStamp(atr.getDataTimeStamp());
    setUserStamp(atr.getUserStamp());
    setSystemStamp(atr.getSystemStamp());
    Status = atr.status;
    return rc;
  }
  /**
   * @brief Establishes and executes a synchronous data link using the Device
   *        Properties assigned to the control. Use Execute for synchronous data
   *        acquisition. Fill in the DeviceContext DeviceGroup, DeviceName, and
   *        DeviceProperty bean property parameters, along with the
   *        accessProtocol, accessMode, and accessRate bean property parameters.
   *        This method blocks until either the transfer is complete or a
   *        timeout has occurred (given by the accessRate setting). Asynchronous
   *        READ and WRITE calls can be effected by using the AttachLink()
   *        method and handling the results inside the corresponding receive
   *        event.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. Pass 'null' if no
   *          return data is desired.
   * @return 0 if successful otherwise a positive return code or -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data and DataEx objects above must refer to arrays of data types
   *       (which are passed by reference) and not single values (which are
   *       passed by value). So even if passing a single value, it must be
   *       declared as an array of 1 element! The sizes of the arrays are in
   *       turn used in establishing the link when using this form of the
   *       Execute() method.
   * @note The only allowed values of 'accessMode' are "READ" and "WRITE" when
   *       using the Execute() method. \b Example: \include
   *       eg_acopExecute1b.java
   */
  public int execute(Object Data, Object DataEx)
  {
    return execute(Data, getObjectSize(Data), DataEx, getObjectSize(DataEx));
  }
  /**
   * @brief Establishes and executes a synchronous data link using the Device
   *        Properties assigned to the control. Use Execute for synchronous data
   *        acquisition. Fill in the DeviceContext DeviceGroup, DeviceName, and
   *        DeviceProperty bean property parameters, along with the
   *        accessProtocol, accessMode, and accessRate bean property parameters.
   *        This method blocks until either the transfer is complete or a
   *        timeout has occurred (given by the accessRate setting). Asynchronous
   *        READ and WRITE calls can be effected by using the AttachLink()
   *        method and handling the results inside the corresponding receive
   *        event.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @return 0 if successful otherwise a positive return code or -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element! The size of the array is in turn used in
   *       establishing the link when using this form of the Execute() method.
   * @note The only allowed values of 'accessMode' are "READ" and "WRITE" when
   *       using the Execute() method. \b Example: \include
   *       eg_acopExecute1c.java
   */
  public int execute(Object Data)
  {
    return execute(Data, getObjectSize(Data), null, 0);
  }
  /**
   * @brief Establishes and executes a synchronous data link using the Device
   *        Properties assigned to the control. Use Execute for synchronous data
   *        acquisition. Fill in the DeviceContext DeviceGroup, DeviceName, and
   *        DeviceProperty bean property parameters, along with the
   *        accessProtocol, accessMode, and accessRate bean property parameters.
   *        This method blocks until either the transfer is complete or a
   *        timeout has occurred (given by the accessRate setting). Asynchronous
   *        READ and WRITE calls can be effected by using the AttachLink()
   *        method and handling the results inside the corresponding receive
   *        event.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @return 0 if successful otherwise a positive return code or -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element!
   * @note The only allowed values of 'accessMode' are "READ" and "WRITE" when
   *       using the Execute() method. \b Example: \include
   *       eg_acopExecute1d.java
   */
  public int execute(Object Data, int ArraySize)
  {
    return execute(Data, ArraySize, null, 0);
  }
  // TODO: the only difference between AttachLink() and OpenLink() seems to be
  // atr.isUnboundToData
  // Execute() is also of the same ilk. Maybe put the meat into one method only?
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control and Attaches the data objects passed. Use
   *        AttachLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data transfer link and attach the returning data to
   *        the Data object passed. When new data arrive or the link status has
   *        changed, the chart receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. Pass 'null' if no
   *          return data is desired.
   * @param ArraySizeEx
   *          is the data array size associated with the DataEx object. This may
   *          be smaller than the actual size of the DataEx object, but NOT
   *          bigger.
   * @param identifier
   *          is a user-specified integer identifier which is used to identify
   *          the link to the calling application. Following a receive() event,
   *          the application can call getLinkId() to retrieve this number.
   *          Other forms of AttachLink() will use the returned link handle as
   *          and identifier.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data and DataEx objects above must refer to arrays of data types
   *       (which are passed by reference) and not single values (which are
   *       passed by value). So even if passing a single value, it must be
   *       declared as an array of 1 element! \b Example: \include
   *       eg_acopAttach1f.java
   */
  public int attachLink(Object Data, int ArraySize, Object DataEx, int ArraySizeEx, int identifier)
  {
    if (getObjectSize(Data) < ArraySize || getObjectSize(DataEx) < ArraySizeEx)
    {
      Status = "Data object smaller than size given";
      return -1;
    }
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.ATTACH;
    atr.setFromParameters(connectionParameters);
    atr.setData(Data);
    atr.setArraySize(ArraySize);
    atr.setDataEx(DataEx);
    atr.setArraySizeEx(ArraySizeEx);
    atr.atc = atc;
    atr.isUnboundToData = false;
    if (atr.accessMode == null)
    {
      Status = "access mode not set";
      return -1;      
    }
    if ((atp = atf.getProtocol(connectionParameters.getAccessProtocol())) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    boolean isSingleLink = (atr.accessMode.isReadMode() || atr.accessMode.isWriteMode());
    if (!isSingleLink && getAcopLinkTableId(atr) >= 0) return atr.getLinkIdentifier();
    if (atp.handleRequest(atr) < 0)
    {
      Status = atr.status;
      return -1;
    }
    if (identifier == -1) identifier = getFreeIdentifier();
    atr.setLinkIdentifier(identifier);
    // getAcopTransportRequest(atr);
    atr = getAcopTransportRequest(atr);
    atr.pending = true;
    Status = atr.status;
    // AcopLinkTableId(atr.getLinkIdentifier());
    return identifier;
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control and Attaches the data objects passed. Use
   *        AttachLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data transfer link and attach the returning data to
   *        the Data object passed. When new data arrive or the link status has
   *        changed, the chart receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. Pass 'null' if no
   *          return data is desired.
   * @param ArraySizeEx
   *          is the data array size associated with the DataEx object. This may
   *          be smaller than the actual size of the DataEx object, but NOT
   *          bigger.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data and DataEx objects above must refer to arrays of data types
   *       (which are passed by reference) and not single values (which are
   *       passed by value). So even if passing a single value, it must be
   *       declared as an array of 1 element! \b Example: \include
   *       eg_acopAttach1.java
   */
  public int attachLink(Object Data, int ArraySize, Object DataEx, int ArraySizeEx)
  {
    return attachLink(Data, ArraySize, DataEx, ArraySizeEx, -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control and Attaches the data objects passed. Use
   *        AttachLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data tranfer link and attach the returning data to
   *        the Data object passed. When new data arrive or the link status has
   *        changed, the de.desy.acop.chart receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. Pass 'null' if no
   *          return data is desired.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data and DataEx objects above must refer to arrays of data types
   *       (which are passed by reference) and not single values (which are
   *       passed by value). So even if passing a single value, it must be
   *       declared as an array of 1 element! The sizes of the arrays are in
   *       turn used in establishing the link when using this form of the
   *       AttachLink() method. \b Example: \include eg_acopAttach1b.java
   */
  public int attachLink(Object Data, Object DataEx)
  {
    return attachLink(Data, getObjectSize(Data), DataEx, getObjectSize(DataEx), -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control and Attaches the data object passed. Use
   *        AttachLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data tranfer link and attach the returning data to
   *        the Data object passed. When new data arrive or the link status has
   *        changed, the de.desy.acop.chart receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element! The size of the array is in turn used in
   *       establishing the link when using this form of the AttachLink()
   *       method. \b Example: \include eg_acopAttach1c.java
   */
  public int attachLink(Object Data)
  {
    return attachLink(Data, getObjectSize(Data), null, 0, -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control and Attaches the data object passed. Use
   *        AttachLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data tranfer link and attach the returning data to
   *        the Data object passed. When new data arrive or the link status has
   *        changed, the de.desy.acop.chart receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element! \b Example: \include eg_acopAttach1d.java
   */
  public int attachLink(Object Data, int ArraySize)
  {
    return attachLink(Data, ArraySize, null, 0, -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control and Attaches the data object passed. Use
   *        AttachLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data tranfer link and attach the returning data to
   *        the Data object passed. When new data arrive or the link status has
   *        changed, the de.desy.acop.chart receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @param identifier
   *          is a user-specified integer identifier which is used to identify
   *          the link to the calling application. Following a receive() event,
   *          the application can call getLinkId() to retrieve this number.
   *          Other forms of AttachLink() will use the returned link handle as
   *          and identifier.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element! \b Example: \include eg_acopAttach1e.java
   */
  public int attachLink(Object Data, int ArraySize, int identifier)
  {
    return attachLink(Data, ArraySize, null, 0, identifier);
  }
  /**
   * @brief Sets the link identifier to the value specified. Establishing a data
   *        link by using either AttachLink() or OpenLink() will assign a link
   *        identifier to the associated link when successful. This link
   *        identifier (or link 'handle') will be set prior to event
   *        notification via the Receive() event. One can set the link
   *        identifier to a desired value either in the original call to
   *        AttachLink() or OpenLink() or at a later time by calling this
   *        method.
   * @param currentLinkIdentifier
   *          is the current link identifier of the data link to which the new
   *          link identifier is to be assigned. If not originally set by the
   *          initial call, it is the value returned by the initial call to
   *          AttachLink() or OpenLink().
   * @param linkIdentifier
   *          is the desired link identifier to be associated with the data link
   *          defined by linkHandle.
   * @return The link identifier associated with the data link. If
   *         de.desy.acop.chart is unable to set the link identifier to the new
   *         value, it remains unchanged. If successful, the new link handle
   *         will be set to the assigned link identifier. \b Example: \include
   *         eg_acopSetLinkIdentifier.java
   */
  public int openLink(Object Data, int ArraySize, Object DataEx, int ArraySizeEx, int identifier)
  {
    if (getObjectSize(Data) < ArraySize || getObjectSize(DataEx) < ArraySizeEx)
    {
      Status = "Data object smaller than size given";
      return -1;
    }
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.ATTACH;
    atr.setFromParameters(connectionParameters);
    atr.setData(Data);
    atr.setArraySize(ArraySize);
    atr.setDataEx(DataEx);
    atr.setArraySizeEx(ArraySizeEx);
    atr.atc = atc;
    atr.isUnboundToData = true;
    if ((atp = atf.getProtocol(connectionParameters.getAccessProtocol())) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    if (getAcopLinkTableId(atr) >= 0) return atr.getLinkIdentifier();
    if (atp.handleRequest(atr) < 0)
    {
      Status = atr.status;
      return -1;
    }
    if (identifier == -1) identifier = getFreeIdentifier();
    atr.setLinkIdentifier(identifier);
    getAcopTransportRequest(atr);
    atr.pending = true;
    Status = atr.status;
    getAcopLinkTableId(atr.getLinkIdentifier());
    return identifier;
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control, but does not Attach the data objects
   *        passed. Use OpenLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data tranfer link. The data returned from a server
   *        is not bound to the Data object passed in the call and must be
   *        obtained by an independent call to the GetData() method. When new
   *        data arrive or the link status has changed, the de.desy.acop.chart
   *        receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. Pass 'null' if no
   *          return data is desired.
   * @param ArraySizeEx
   *          is the data array size associated with the DataEx object. This may
   *          be smaller than the actual size of the DataEx object, but NOT
   *          bigger.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data and DataEx objects above must refer to arrays of data types
   *       (which are passed by reference) and not single values (which are
   *       passed by value). So even if passing a single value, it must be
   *       declared as an array of 1 element!
   */
  public int openLink(Object Data, int ArraySize, Object DataEx, int ArraySizeEx)
  {
    return openLink(Data, ArraySize, DataEx, ArraySizeEx, -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control, but does not Attach the data objects
   *        passed. Use OpenLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data tranfer link. The data returned from a server
   *        is not bound to the Data object passed in the call and must be
   *        obtained by an independent call to the GetData() method. When new
   *        data arrive or the link status has changed, the de.desy.acop.chart
   *        receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. Pass 'null' if no
   *          return data is desired.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data and DataEx objects above must refer to arrays of data types
   *       (which are passed by reference) and not single values (which are
   *       passed by value). So even if passing a single value, it must be
   *       declared as an array of 1 element! The size of the array is in turn
   *       used in establishing the link when using this form of the OpenLink()
   *       method.
   */
  public int openLink(Object Data, Object DataEx)
  {
    return openLink(Data, getObjectSize(Data), DataEx, getObjectSize(DataEx), -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control, but does not Attach the data object passed.
   *        Use OpenLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data tranfer link. The data returned from a server
   *        is not bound to the Data object passed in the call and must be
   *        obtained by an independent call to the GetData() method. When new
   *        data arrive or the link status has changed, the de.desy.acop.chart
   *        receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element! The size of the array is in turn used in
   *       establishing the link when using this form of the OpenLink() method.
   */
  public int openLink(Object Data)
  {
    return openLink(Data, getObjectSize(Data), null, 0, -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control, but does not Attach the data object passed.
   *        Use OpenLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data transfer link. The data returned from a server
   *        is not bound to the Data object passed in the call and must be
   *        obtained by an independent call to the GetData() method. When new
   *        data arrive or the link status has changed, the chart
   *        receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element!
   */
  public int openLink(Object Data, int ArraySize)
  {
    return openLink(Data, ArraySize, null, 0, -1);
  }
  /**
   * @brief Establishes an asynchronous data link using the Device Properties
   *        assigned to the control, but does not Attach the data object passed.
   *        Use OpenLink for asynchronous data acquisition. Fill in the
   *        DeviceContext DeviceGroup, DeviceName, and DeviceProperty bean
   *        property parameters, along with the accessProtocol, accessMode, and
   *        accessRate bean property parameters. Call this method to establish
   *        the asynchronous data transfer link. The data returned from a server
   *        is not bound to the Data object passed in the call and must be
   *        obtained by an independent call to the GetData() method. When new
   *        data arrive or the link status has changed, the chart
   *        receive event will be fired.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. Pass 'null' if no return
   *          data is desired.
   * @param ArraySize
   *          is the data array size associated with the Data object. This may
   *          be smaller than the actual size of the Data object, but NOT
   *          bigger.
   * @param identifier
   *          is a user-specified integer identifier which is used to identify
   *          the link to the calling application. Following a receive() event,
   *          the application can call getLinkId() to retrieve this number.
   *          Other forms of AttachLink() will use the returned link handle as
   *          and identifier.
   * @return A positive link handle if successful otherwise -1, in which case
   *         the getStatus() method can be queried to determine the cause of
   *         failure.
   * @note The Data object above must refer to an array of data types (which is
   *       passed by reference) and not a single value (which is passed by
   *       value). So even if passing a single value, it must be declared as an
   *       array of 1 element!
   */
  public int openLink(Object Data, int ArraySize, int identifier)
  {
    return openLink(Data, ArraySize, null, 0, identifier);
  }
  public String getStatus(int hLink)
  {
    AcopTransportRequest atr = getAcopTransportRequest(hLink);
    return (atr != null) ? atr.getStatus() : "link " + hLink + " not active";
  }
  public int getStatusCode(int hLink)
  {
    AcopTransportRequest atr = getAcopTransportRequest(hLink);
    return (atr != null) ? atr.getStatusCode() : -1;
  }
  /**
   * @brief Closes an existing data link. Using CloseLink()
   *        with a '-1' input parameter will close all data links associated
   *        with the acop bean.
   * @param identifier
   *          is the link handle of the acop data link which is to
   *          be terminated. If identifier == -1 then all associated links will
   *          be terminated.
   * @return 0 if successful otherwise -1, in which case the getStatus() method
   *         can be queried to determine the cause of failure. \b Example:
   *         \include eg_acopCloseLink1.java
   */
  public int closeLink(int hLink) // hLink is the identifier !
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    AcopTransportRequest a;
    int i;
    atr.accessMethod = AcopTransportAccessMethods.CLOSE;
    if (hLink >= 0)
    { //TODO: maybe use a hash map or something ? 
      //(maybe not: the number of links is almost always small!) 
      for (i = 0; i < acopLinkTable.size(); i++)
      {
        a = acopLinkTable.elementAt(i);
        if (a.getLinkIdentifier() == hLink)
        {
          atr.setLinkHandle(a.getLinkHandle());
          atr.accessProtocol = a.accessProtocol;
          if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
          {
            Status = "Protocol not available";
            return -1;
          }
          atp.handleRequest(atr);
          acopLinkTable.remove(i--); // backup !
        }
      }
    }
    else
    {
      for (i = 0; i < acopLinkTable.size(); i++)
      {
        a = acopLinkTable.elementAt(i);
        atr.setLinkHandle(a.getLinkHandle());
        atr.accessProtocol = a.accessProtocol;
        if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
        {
          Status = "Protocol not available";
          continue;
        }
        atp.handleRequest(atr);
      }
      acopLinkTable.removeAllElements();
    }
    Status = atr.status;
    return 0;
  }
  /**
   * @brief Retrieves the data associated with the given link handle into the
   *        data objects passed. This method is frequently used in conjunction
   *        with the OpenLink() method of establishing a data link, in which
   *        case the incoming data is not bound to a data object.
   * @param Data
   *          is the primary data object and refers to the data object to hold
   *          the output data returned from a server. This must be a reference
   *          to the same type of object passed in the original call to
   *          AttachLink() or OpenLink().
   * @param DataEx
   *          is the extended data object and refers to the data object to which
   *          holds the input data to be sent to a server. This must be a
   *          reference to the same type of object passed in the original call
   *          to AttachLink() or OpenLink().
   * @param hLink
   *          is the link handle of the data link for which the data is to be
   *          retrieved. If hLink < = then hLink = 0 is assumed.
   * @return 0 if successful otherwise -1, in which case the getStatus() method
   *         can be queried to determine the cause of failure.
   */
  public int getData(Object Data, Object DataEx, int hLink)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    AcopTransportRequest a;
    int i;
    if (hLink < 0) hLink = 0;
    if ((i = getAcopLinkTableId(hLink)) < 0)
    {
      Status = "Link not allocated";
      return -1;
    }
    a = acopLinkTable.elementAt(i);
    atr.accessMethod = AcopTransportAccessMethods.GETDATA;
    atr.setData(Data);
    atr.setArraySize(getObjectSize(Data));
    atr.setDataEx(DataEx);
    atr.setArraySizeEx(getObjectSize(DataEx));
    atr.setLinkHandle(a.getLinkHandle());
    atr.accessProtocol = a.accessProtocol;
    atr.isUnboundToData = a.isUnboundToData;
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    int rc = atp.handleRequest(atr);
    lastUpdatedLink = hLink;
    lastUpdatedDataSize = atr.getDataSize();
    lastUpdatedDataSizeEx = atr.getDataSizeEx();
    setTimeStamp(atr.getDataTimeStamp());
    setSystemStamp(atr.getSystemStamp());
    setUserStamp(atr.getUserStamp());
    a.setDataTimeStamp(atr.getDataTimeStamp());
    a.setSystemStamp(atr.getSystemStamp());
    a.setUserStamp(atr.getUserStamp());
    Status = atr.status;
    return rc;
  }
  public int acquireDisplayProperties(Acop a, String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    return atp.acquireDisplayProperties(a, deviceContext, deviceGroup, deviceName, deviceProperty);
  }
  public double[] getXAxis(Object data)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getXAxis(data);
  }
  public double[] getYAxis(Object data)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getYAxis(data);
  }
  public String[] getXAxisChannelNames(Object data)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    String[] names = atp.getXAxisChannelNames(data);
    if (names == null) names = getDeviceNames();
    return names;
  }
  public int getObjectSize(Object data)
  {
    // AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    if ((atp = atf.getProtocol(this.connectionParameters.getAccessProtocol())) == null)
    {
      Status = "Protocol not available";
      return -1;
    }
    return atp.getObjectSize(data);
  }
  /**
   * @brief Gives the current data size of the data objects associated with the
   *        given link handle. This method provides an analogy with the
   *        equivalent method in the ACOP ActiveX control, where the arguments
   *        are explicitly passed by reference. In the ACOP java bean, the
   *        overloaded methods GetDataSize(int identifier) and GetDataSizeEx(int
   *        identifier) are preferred.
   * @param DataSize
   *          is a reference to the current data size of primary Data object
   *          attached to the data link. As this is passed by reference, this
   *          must be an array of 1 integer. A 'null' can be passed if you are
   *          not interested in this value.
   * @param DataSizeEx
   *          is a reference to the current data size of extended Data object
   *          attached to the data link. As this is passed by reference, this
   *          must be an array of 1 integer. A 'null' can be passed if you are
   *          not interested in this value.
   * @param identifier
   *          is the link handle of the data link for which
   *          the current data sizes are desired.
   * @return 0 if successful otherwise -1, in which case the getStatus() method
   *         can be queried to determine the cause of failure. \b Example:
   *         \include eg_acopGetDataSize1.java
   */
  public int getDataSize(int[] DataSize, int[] DataSizeEx, int hLink)
  {
    AcopTransportRequest a;
    int i;
    int[] dsize = new int[1], dsizeEx = new int[1];
    if (hLink < 0)
    { // synchronous link no long in tables, do the best we can :
      dsize[0] = lastUpdatedDataSize;
      dsizeEx[0] = lastUpdatedDataSizeEx;
    }
    else
    {
      if ((i = getAcopLinkTableId(hLink)) < 0)
      {
        Status = "Link not allocated";
        return -1;
      }
      a = acopLinkTable.elementAt(i);
      dsize[0] = a.getDataSize();
      dsizeEx[0] = a.getDataSizeEx();
    }
    if (DataSize != null) DataSize[0] = dsize[0];
    if (DataSizeEx != null) DataSizeEx[0] = dsizeEx[0];
    return 0;
  }
  /**
   * @brief Queries for the available device contexts according to current
   *        setting of the accessProtocol
   * @return A string array containing the device contexts
   */
  public String[] getDeviceContexts()
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.setFromParameters(connectionParameters);
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceContexts();
  }
  /**
   * @brief Queries for the available device groups according to current setting
   *        of the accessProtocol
   * @param deviceContext
   *          is the device context for which the device groups are desired
   * @param deviceGroup
   *          is the device group (sub-string + wildcards) for which the device
   *          groups are desired. 'null' is equivalent to "*".
   * @param deviceName
   *          is the device name for which the device groups are desired. This
   *          parameter should typically passed as 'null'.
   * @param deviceProperty
   *          is the device property for which the device groups are desired.
   *          This parameter should typically passed as 'null'.
   * @return A string array containing the device groups
   */
  public String[] getDeviceGroups(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.setFromParameters(connectionParameters);
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceGroups(deviceContext, deviceGroup, deviceName, deviceProperty);
  }
  /**
   * @brief Queries for the available device groups according to current setting
   *        of the accessProtocol and Device parameters
   * @return A string array containing the device groups
   */
  public String[] getDeviceGroups()
  {
    return getDeviceGroups(getDeviceContext(), getDeviceGroup(), getDeviceName(), getDeviceProperty());
  }
  /**
   * @brief Queries for the available device names according to current setting
   *        of the accessProtocol
   * @param deviceContext
   *          is the device context for which the device names are desired
   * @param deviceGroup
   *          is the device group for which the device names are desired.
   * @param deviceName
   *          is the device name (sub-string + wildcards) for which the device
   *          names are desired. 'null' is equivalent to "*".
   * @param deviceProperty
   *          is the device property for which the device names are desired.
   * @return A string array containing the device names
   */
  public String[] getDeviceNames(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.setFromParameters(connectionParameters);
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceNames(deviceContext, deviceGroup, deviceName, deviceProperty);
  }
  /**
   * @brief Queries for the available device names according to current setting
   *        of the accessProtocol and Device parameters
   * @return A string array containing the device names
   */
  public String[] getDeviceNames()
  {
    return getDeviceNames(getDeviceContext(), getDeviceGroup(), getDeviceName(), getDeviceProperty());
  }
  public java.lang.String toTimeString(int TimeStamp)
  {
    Date d = new Date((TimeStamp * 1000));
    return d.toString();
  }
  public java.lang.String toTimeString(double TimeStamp)
  {
    Date d = new Date((long) (TimeStamp * 1000));
    return d.toString();
  }
  public java.lang.String toTimeString(Date TimeStamp)
  {
    return TimeStamp.toString();
  }
  public void setReceiveQueueDepth(int ReceiveQueueDepth)
  {
    this.ReceiveQueueDepth = ReceiveQueueDepth;
  }
  public void setTimeStamp(int TimeStamp)
  {
    this.TimeStamp = TimeStamp;
    this.dateStamp = new Date(((long) TimeStamp) * 1000);
  }
  public void setTimeStamp(Date TimeStamp)
  {
    this.dateStamp = TimeStamp;
    if (TimeStamp != null) this.TimeStamp = (double) (TimeStamp.getTime()) / 1000;
  }
  public void setTimeStamp(double TimeStamp)
  {
    this.TimeStamp = TimeStamp;
    this.dateStamp = new Date((long) (TimeStamp * 1000));
  }
  
  public void setSystemStamp(int stamp)
  {
    this.systemStamp = stamp;
  }
  public void setUserStamp(int stamp)
  {
    this.userStamp = stamp;
  }
  /**
   * @brief The current data status string
   * @return The current data status string
   */
  public java.lang.String getStatus()
  {
    return Status;
  }
  public int getGroupIndex()
  {
    return isGrouped ? GroupIndex : -1;
  }
  /**
   * @brief Sets the grouped flag
   * @param isGrouped
   *          determines whether all data links associated with this
   *          acop are to fire one single event (isGrouped = true)
   *          when all link states have changed or not (isGrouped = false)
   */
  public void setGrouped(boolean Grouped)
  {
    this.isGrouped = Grouped;
  }
  public void setDeviceContext(java.lang.String deviceContext)
  {
    if (connectionParameters != null) {
	  setConnectionParameters(connectionParameters.deriveWithDeviceContext(deviceContext));
    } else {
    	setConnectionParameters(new ConnectionParameters(null, deviceContext, null, null, null, null, 1000));
    }
  }
  public String getDeviceContext()
  {
    return connectionParameters.getDeviceContext();
  }
  public void setDeviceGroup(java.lang.String deviceGroup)
  {
	  if (connectionParameters != null) {
		  setConnectionParameters(connectionParameters.deriveWithDeviceGroup(deviceGroup));
	  } else {
	      setConnectionParameters(new ConnectionParameters(null, null, deviceGroup, null, null, null, 1000));
	  }
  }
  public String getDeviceGroup()
  {
    return connectionParameters.getDeviceGroup();
  }
  public void setDeviceName(java.lang.String deviceName)
  {
	  if (connectionParameters != null) {
		  setConnectionParameters(connectionParameters.deriveWithDeviceName(deviceName));
	  } else {
		 setConnectionParameters(new ConnectionParameters(null, null, null, deviceName, null, null, 1000));
	  }
  }
  public String getDeviceName()
  {
    return connectionParameters.getDeviceName();
  }
  public void setDeviceProperty(java.lang.String deviceProperty)
  {
	  if (connectionParameters != null) {
		  setConnectionParameters(connectionParameters.deriveWithDeviceProperty(deviceProperty));
	  } else {
		  setConnectionParameters(new ConnectionParameters(null, null, null, null, deviceProperty, null, 1000));
	  }
  }
  public String getDeviceProperty()
  {
    return connectionParameters.getDeviceProperty();
  }
  public void setAccessMode(AccessMode am)
  {
	  if (connectionParameters != null) {
		  setConnectionParameters(connectionParameters.deriveWith(am));
	  } else {
		  setConnectionParameters(new ConnectionParameters(null, null, null, null, null, am, 1000));
	  }
  }
  public AccessMode getAccessMode()
  {
    return connectionParameters.getAccessMode();
  }
  public void setAccessProtocol(java.lang.String accessProtocol)
  {
	  if (connectionParameters != null) {
		  setConnectionParameters(connectionParameters.deriveWithAccessProtocol(accessProtocol));
	  } else {
		  setConnectionParameters(new ConnectionParameters(accessProtocol, null, null, null, null, null, 1000));
	  }
  }
  public String getAccessProtocol()
  {
    return connectionParameters.getAccessProtocol();
  }
  public void setAccessRate(String accessRate)
  {
	  if (connectionParameters != null) {
		  setConnectionParameters(connectionParameters.deriveWithAccessRate(Integer.parseInt(accessRate)));
	  } else {
		  setConnectionParameters(new ConnectionParameters(null, null, null, null, null, null, Integer.parseInt(accessRate)));
	  }
  }
  public String getAccessRate()
  {
    return Integer.toString(connectionParameters.getAccessRate());
  }
  public void setPropertySize(String propertySize)
  {
	  if (connectionParameters != null) {
		  setConnectionParameters(connectionParameters.deriveWithPropertySize(Integer.parseInt(propertySize)));
	  } else {
		  setConnectionParameters(new ConnectionParameters(null, null, null, null, null, null, 1000, Integer.parseInt(propertySize)));
	  }
  }
  public String getPropertySize() {
	  return Integer.toString(connectionParameters.getPropertySize());
  }
  public void setAcopConfig(java.lang.String AcopConfig)
  {
    this.acopConfig = AcopConfig;
  }
  public String getAcopConfig()
  {
    return acopConfig;
  }
  public int setLinkIdentifier(int currentLinkIdentifier, int linkIdentifier)
  {
    AcopTransportRequest a;
    int identifier = currentLinkIdentifier;
    Vector<AcopTransportRequest> lnkTbl = getAcopLinkTable();
    for (int i = 0; i < lnkTbl.size(); i++)
    {
      if ((a = lnkTbl.elementAt(i)) == null) continue;
      if (a.getLinkIdentifier() != currentLinkIdentifier) continue;
      a.setLinkIdentifier(linkIdentifier);
      identifier = linkIdentifier;
    }
    return identifier;
  }
  /**
   * @brief Returns the current data size of the primary data object associated
   *        with the given link handle.
   * @param identifier
   *          is the link handle of the data link for which
   *          the current data sizes are desired.
   * @return The data size of the returned data if successful otherwise -1, in
   *         which case the getStatus() method can be queried to determine the
   *         cause of failure. \b Example: \include eg_acopGetDataSize2.java
   */
  public int getDataSize(int identifier)
  {
    int[] dsize = new int[1];
    int rc = getDataSize(dsize, null, identifier);
    if (rc < 0) return rc;
    return dsize[0];
  }
  /**
   * @brief Returns the current data size of the primary data object.
   * @return The data size of the returned data if successful otherwise -1, in
   *         which case the getStatus() method can be queried to determine the
   *         cause of failure. \b Example: \include eg_acopGetDataSize3.java
   */
  public int getDataSize()
  {
    int[] dsize = new int[1];
    int rc = getDataSize(dsize, null, -1);
    if (rc < 0) return rc;
    return dsize[0];
  }
  /**
   * @brief Returns the current data size of the extended data object associated
   *        with the given link handle.
   * @param identifier
   *          is the link handle of the data link for which
   *          the current data sizes are desired.
   * @return The data size of the sent data if successful otherwise -1, in which
   *         case the getStatus() method can be queried to determine the cause
   *         of failure.
   */
  public int getDataSizeEx(int identifier)
  {
    int[] dsize = new int[1];
    int rc = getDataSize(null, dsize, identifier);
    if (rc < 0) return rc;
    return dsize[0];
  }
  /**
   * @brief Returns the current data size of the extended data object.
   * @return The data size of the sent data if successful otherwise -1, in which
   *         case the getStatus() method can be queried to determine the cause
   *         of failure.
   */
  public int getDataSizeEx()
  {
    int[] dsize = new int[1];
    int rc = getDataSize(null, dsize, -1);
    if (rc < 0) return rc;
    return dsize[0];
  }
  /**
   * @brief The current data time stamp as a Date object
   * @return The current data time stamp as a Date object
   */
  public Date getTimeStamp()
  {
    return getDateStamp();
  }
  /**
   * @brief The grouped flag
   * @return the isGrouped state of this acop
   */
  public boolean getGrouped()
  {
    return isGrouped;
  }
  /**
   * @brief Queries for the available device properties according to current
   *        setting of the accessProtocol and Device parameters
   * @return A string array containing the device properties
   */
  public String[] getDeviceProperties()
  {
    return getDeviceProperties(getDeviceContext(), getDeviceGroup(), getDeviceName(), getDeviceProperty());
  }
  /**
   * @brief Queries for the available device properties according to current
   *        setting of the accessProtocol
   * @param deviceContext
   *          is the device context for which the device properties are desired
   * @param deviceGroup
   *          is the device group for which the device properties are desired.
   * @param deviceName
   *          is the device name for which the device properties are desired.
   * @param deviceProperty
   *          is the device property (sub-string + wildcards) for which the
   *          device properties are desired. 'null' is equivalent to "*".
   * @return A string array containing the device properties
   */
  public String[] getDeviceProperties(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    AcopTransportPlug atp;
    atr.accessMethod = AcopTransportAccessMethods.EXECUTE;
    atr.setFromParameters(connectionParameters);
    if ((atp = atf.getProtocol(atr.accessProtocol)) == null)
    {
      Status = "Protocol not available";
      return null;
    }
    return atp.getDeviceProperties(deviceContext, deviceGroup, deviceName, deviceProperty);
  }
  /**
   * @brief Queries for the available display setting according to current
   *        setting of the accessProtocol This method uses the query mechanisms
   *        of the underlying transport to ascertain the maximum and minimum
   *        display settings and axis labels. The resulting 'Y' axis label will
   *        be the units associated with the queried property. The maximum and
   *        minimum settings for the 'Y' axis will be the range associated with
   *        the queried property. The resulting 'X' axis label will be the
   *        description associated with the queried property. The maximum and
   *        minimum settings for the 'X' axis will be 0 to the array size of the
   *        property queried, if the property corresponds to a multi-channel
   *        display.
   * @param deviceContext
   *          is the device context for which the device properties are desired
   * @param deviceGroup
   *          is the device group for which the device properties are desired.
   * @param deviceName
   *          is the device name for which the device properties are desired.
   * @param deviceProperty
   *          is the device property (sub-string + wildcards) for which the
   *          device properties are desired. 'null' is equivalent to "*".
   * @return 0 if successful otherwise -1.
   */
  public int acquireDisplayProperties(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    return acquireDisplayProperties(null, deviceContext, deviceGroup, deviceName, deviceProperty);
  }
  /**
   * @brief Queries for the available display setting according to current
   *        setting of the accessProtocol and display parameters. This method
   *        uses the query mechanisms of the underlying transport to ascertain
   *        the maximum and minimum display settings and axis labels. The
   *        resulting 'Y' axis label will be the units associated with the
   *        queried property. The maximum and minimum settings for the 'Y' axis
   *        will be the range associated with the queried property. The
   *        resulting 'X' axis label will be the description associated with the
   *        queried property. The maximum and minimum settings for the 'X' axis
   *        will be 0 to the array size of the property queried, if the property
   *        corresponds to a multi-channel display.
   * @return 0 if successful otherwise -1.
   */
  public int acquireDisplayProperties()
  {
    return acquireDisplayProperties(null, getDeviceContext(), getDeviceGroup(), getDeviceName(),
        getDeviceProperty());
  }
  public void addAcopTransportListener(AcopTransportListener l)
  {
    listeners.add(AcopTransportListener.class, l);
  }
  public void removeAcopTransportListener(AcopTransportListener l)
  {
    listeners.remove(AcopTransportListener.class, l);
  }
  /**
   * Returns the connectionParameters.
   * 
   * @return Returns the connectionParameters.
   */
  public ConnectionParameters getConnectionParameters()
  {
    return connectionParameters;
  }
  /**
   * Sets new connectionParameters.
   * 
   * @param connectionParameters
   *          The connectionParameters to set.
   */
  public void setConnectionParameters(ConnectionParameters connectionParameters)
  {
	if ((this.connectionParameters != null && this.connectionParameters.equals(connectionParameters)) ||
		  this.connectionParameters == null && connectionParameters == null) return;
    ConnectionParameters oldValue = this.connectionParameters;
    this.connectionParameters = connectionParameters;
    firePropertyChange(CONNECTION_PARAMETERS_PROPERTY, oldValue, this.connectionParameters);
  }
  
  private PropertyChangeSupport propertyChangeSupport;
  
  protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
	  if (propertyChangeSupport == null) return;
	  propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
  }
  
  public void addPropertyChangeListener(PropertyChangeListener listener) {
	if (propertyChangeSupport == null) {
		propertyChangeSupport = new PropertyChangeSupport(this);
	}
	propertyChangeSupport.addPropertyChangeListener(listener);	
  }
  
  public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
	if (propertyChangeSupport == null) {
		propertyChangeSupport = new PropertyChangeSupport(this);
	}
	propertyChangeSupport.addPropertyChangeListener(propertyName, listener);	
  }

  public void removePropertyChangeListener(PropertyChangeListener listener) {
	if (propertyChangeSupport == null) return;
	propertyChangeSupport.removePropertyChangeListener(listener);
  }
  public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
	if (propertyChangeSupport == null) return;
	propertyChangeSupport.removePropertyChangeListener(propertyName, listener);	
  }
}
