/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport;
import java.util.Date;

/**
 * @version 1.0
 * @author
 */
public class AcopTransportRequest
{
  private String emptyString = new String("");
  public String deviceContext = emptyString;
  public String deviceGroup = emptyString;
  public String deviceName = emptyString;
  public String deviceProperty = emptyString;
  private Object data;
  private Object dataEx;
  private int arraySize;
  private int arraySizeEx;
  private int dataSize;
  private int dataSizeEx;
  private Date dataTimeStamp;
  private int systemStamp;
  private int userStamp;
  protected boolean pending;
  public boolean isUnboundToData;
  public int statusCode;
  public String status = emptyString;
  public AccessMode accessMode;
  public String accessRate = emptyString;
  public String accessProtocol = emptyString;
  public String deviceInfo = emptyString;
  public int accessMethod;
  private int linkHandle;
  private int linkIdentifier;
  public AcopTransportCallback atc;
  public int getLinkHandle() { return linkHandle; }
  public int getLinkIdentifier() { return linkIdentifier; }
  public void setLinkIdentifier(int id) { linkIdentifier = id; }
  public void setLinkHandle(int id) { linkHandle = id; }
  public boolean compareTo(AcopTransportRequest atr)
  {
    if (atr.accessMode.compareTo(accessMode) == 0 &&
        atr.accessRate.compareTo(accessRate) == 0 &&
        atr.accessProtocol.compareTo(accessProtocol) == 0 &&
        atr.arraySize == arraySize &&
        atr.arraySizeEx == arraySizeEx &&
        atr.atc == atc &&
        atr.data == data &&
        atr.dataEx == dataEx &&
        atr.deviceContext.compareTo(deviceContext) == 0 &&
        atr.deviceGroup.compareTo(deviceGroup) == 0 &&
        atr.deviceName.compareTo(deviceName) == 0 &&
        atr.deviceProperty.compareTo(deviceProperty) == 0 &&
        atr.systemStamp == systemStamp &&
        atr.userStamp == userStamp) return true;

    return false;         
  }
  public AcopTransportRequest cloneTransportRequest()
  {
    AcopTransportRequest atr = new AcopTransportRequest();
    atr.accessMethod = accessMethod;
    atr.accessMode = accessMode;
    atr.accessRate = new String(accessRate);
    atr.accessProtocol = new String(accessProtocol);
    atr.isUnboundToData = isUnboundToData;
    atr.arraySize = arraySize;
    atr.arraySizeEx = arraySizeEx;
    atr.atc = atc;
    atr.data = data;
    atr.dataEx = dataEx;
    atr.deviceContext = new String(deviceContext);
    atr.deviceGroup = new String(deviceGroup);
    atr.deviceName = new String(deviceName);
    atr.deviceProperty = new String(deviceProperty);
    atr.deviceInfo = new String(deviceInfo);
    atr.linkHandle = linkHandle;
    atr.linkIdentifier = linkIdentifier;
    return atr;    
  }
  public int getArraySize() { return arraySize; }
  public void setArraySize(int arraySize) { this.arraySize = arraySize; }
  public int getArraySizeEx() { return arraySizeEx; }
  public void setArraySizeEx(int arraySizeEx) { this.arraySizeEx = arraySizeEx; }
  public Object getData() { return data; }
  public void setData(Object data) { this.data = data; }
  public Object getDataEx() { return dataEx; }
  public void setDataEx(Object dataEx) { this.dataEx = dataEx; }
  public int getDataSize() { return dataSize; }
  public void setDataSize(int dataSize) { this.dataSize = dataSize; }
  public int getDataSizeEx() { return dataSizeEx; }
  public void setDataSizeEx(int dataSizeEx) { this.dataSizeEx = dataSizeEx; }
  public Date getDataTimeStamp() { return dataTimeStamp; }
  public void setDataTimeStamp(Date dataTimeStamp) { this.dataTimeStamp = dataTimeStamp; }
  public String getStatus() { return status; }
  public void setStatus(String status) { this.status = status; }
  public int getStatusCode() { return statusCode; }
  public void setStatusCode(int statusCode) { this.statusCode = statusCode; }
  public String getDeviceInfo() { return deviceInfo; }
  public void setDeviceInfo(String deviceInfo) { this.deviceInfo = deviceInfo; }
  public void setSystemStamp(int systemStamp) {	this.systemStamp = systemStamp;}
  public int getSystemStamp() {return systemStamp;}
  public void setUserStamp(int userStamp) {this.userStamp = userStamp;}
  public int getUserStamp() {return userStamp;}

  /**
   * Sets this object with values from connection parameters object.
   * @param connectionParameters values to be set
   */
  public void setFromParameters(ConnectionParameters connectionParameters) {
	    accessMode = connectionParameters.getAccessMode();
	    accessRate = Integer.toString(connectionParameters.getAccessRate());
	    accessProtocol = connectionParameters.getAccessProtocol();
	    deviceContext = connectionParameters.getDeviceContext();
	    deviceGroup = connectionParameters.getDeviceGroup();
	    deviceName = connectionParameters.getDeviceName();
	    deviceProperty = connectionParameters.getDeviceProperty();
  }
}
