/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport;

import com.cosylab.util.CommonException;


/**
 * This exception signals that connection to remote system has failed for some reason.
 * 
 * @author Igor Kriznar (igor.kriznarATcosylab.com)
 *
 */
public class ConnectionFailed extends CommonException {
	
	private static final long serialVersionUID = 1L;

	/**
	 * Creates new instance of ConnectionFailed.
	 */
	public ConnectionFailed() {
		super("","Connection failed for unknown reason.");
	}
	/**
	 * Creates new instance of ConnectionFailed.
	 */
	public ConnectionFailed(ConnectionParameters p) {
		super(p,"Connection to '"+p.getRemoteName()+"' failed for unknown reason.");
	}
	
	/**
	 * Creates new instance of ConnectionFailed.
	 */
	public ConnectionFailed(ConnectionParameters p, Throwable e) {
		super(p,"Connection to '"+p.getRemoteName()+"' failed for unknown reason.",e);
	}
	
	/**
	 * Creates new instance of ConnectionFailed.
	 */
	public ConnectionFailed(String message,Throwable t) {
		super("",message,t);
	}
}
