/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport.adapters;


import java.util.Map;

import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.ConnectionParameters;


/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public interface AdapterFactory
{
	/**
	 * Returns data source for parameters and increases use count for that data
	 * source for 1. If adapter does not already exists, it is created.
	 *
	 * @param param parameters for the data source
	 *
	 * @return data source 
	 */
	public AcopTransportDataSource createDataSource(ConnectionParameters param) throws ConnectionFailed;

	/**
	 * Returns adapter assotiated with the parameters, does not increase use
	 * count. If no data source is associated, then create is called.
	 *
	 * @param param parameters for the data source
	 * @param characteristics the preloaded characteristics for the datasource
	 *
	 * @return 
	 */
	public AcopTransportDataSource getDataSource(ConnectionParameters param, Map<String,Object> characteristics) throws ConnectionFailed;

	/**
	 * Returns data source for parameters and increases use count for that data
	 * source for 1. If adapter does not already exists, it is created.
	 *
	 * @param param parameters for the data source
	 * @param characteristics the preloaded characteristics for the datasource
	 *
	 * @return data source 
	 */
	public AcopTransportDataSource createDataSource(ConnectionParameters param, Map<String,Object> characteristics) throws ConnectionFailed;

	/**
	 * Returns adapter assotiated with the parameters, does not increase use
	 * count. If no data source is associated, then create is called.
	 *
	 * @param param parameters for the data source
	 *
	 * @return 
	 */
	public AcopTransportDataSource getDataSource(ConnectionParameters param) throws ConnectionFailed;

	/**
	 * Decreases use count on data source. If use count reaches 0, then destroys
	 * data source and releases all handles to the data source.
	 *
	 * @param ds data source to be destroyed
	 */
	public void releaseDataSource(AcopTransportDataSource ds);

	/**
	 * Decreases use count on data source associated with parameters. 
	 * If use count reaches 0, then destroys data source and releases all handles 
	 * to the data source.
	 * 
	 * @param param 
	 * @return released DataSource 
	 */
	public AcopTransportDataSource releaseDataSource(ConnectionParameters param);
	
	/**
	 * Destroys all data source and clears all cache information.
	 */
	public void clear();
	
	/**
	 * Method returns the characteristics for specified parameters. Characteristics should
	 * be read from cache (from DataSource) if DataSource associated with these parameters 
	 * exists. If not, characteristics should be obtained by other means. 
	 * @param param
	 * @return
	 * @throws ConnectionFailed
	 * 
	 */
	public Map<String, Object> getCharacteristics(ConnectionParameters param) throws ConnectionFailed;
}

/* __oOo__ */
