package de.desy.acop.transport.simulator;

import java.util.Random;
import java.util.Date;
import de.desy.acop.chart.Acop;
import de.desy.acop.transport.AcopTransportAccessMethods;
import de.desy.acop.transport.AcopTransportCallback;
import de.desy.acop.transport.AcopTransportPlug;
import de.desy.acop.transport.AcopTransportRequest;

/**
 * @version 1.0
 * @author
 */
public class AcopTransportSimulator implements AcopTransportPlug
{
  private AcopTransportCallback atc;
  private String[] AcopSimulatorString = new String[1];
  /**
   * @see AcopPack.acopTransportPlug#handleRequest(acopTransportRequest)
   */
  public int handleRequest(AcopTransportRequest atr)
  {
    switch (atr.accessMethod)
    {
      case AcopTransportAccessMethods.EXECUTE:
        if (atr.accessMode.compareToIgnoreCase("READ") != 0
            && atr.accessMode.compareToIgnoreCase("WRITE") != 0)
        {
          atr.status = "illegal mode (READ or WRITE required)";
          return -1;
        }
        if (atr.getData() != null)
        {
          int len = atr.getArraySize();
          atr.statusCode = simulateData(atr.getData(), len);
          atr.setDataSize(len); // just set it to the request
          atr.status = "Success";
          return 0;
        }
        break;
      case AcopTransportAccessMethods.ATTACH:
        atr.setLinkHandle(getLinkHandle());
        atc = atr.atc;
        return 0;
      case AcopTransportAccessMethods.CLOSE:
        return rmvLinkHandle(atr.getLinkHandle());
      case AcopTransportAccessMethods.GETDATA:
        if (atr.getData() != null)
        {
          int len = atr.getArraySize();
          atr.statusCode = simulateData(atr.getData(), len);
          atr.setDataTimeStamp(ast.timer);
          atr.setDataSize(len); // just set it to the request
          atr.status = "Success";
          return 0;
        }
        break;
      case AcopTransportAccessMethods.SHUTDOWN:
        active = false;
        terminate = true;
        break;
      default:
        return -1;
    }
    return 0;
  }
  /**
   * @see AcopPack.acopTransportPlug#getPlugName()
   */
  public String getPlugName()
  {
    return "Simulate";
  }
  public int getObjectSize(Object data)
  {
    if (data == null) return 0;
    if (data instanceof double[]) return ((double[]) data).length;
    if (data instanceof float[]) return ((float[]) data).length;
    if (data instanceof byte[]) return ((byte[]) data).length;
    if (data instanceof short[]) return ((short[]) data).length;
    if (data instanceof int[]) return ((int[]) data).length;
    if (data instanceof String) return ((String) data).length();
    if (data instanceof StringBuffer) return ((StringBuffer) data).length();
    if (data instanceof char[]) return ((char[]) data).length;
    return 0;
  }
  public double[] getYAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[]) data;
      double[] y;
      y = new double[siz];
      if (data instanceof float[])
        for (int i = 0; i < siz; i++)
          y[i] = ((float[]) data)[i];
      else if (data instanceof byte[])
        for (int i = 0; i < siz; i++)
          y[i] = ((byte[]) data)[i];
      else if (data instanceof short[])
        for (int i = 0; i < siz; i++)
          y[i] = ((short[]) data)[i];
      else if (data instanceof int[]) for (int i = 0; i < siz; i++)
        y[i] = ((int[]) data)[i];
      return y;
    }
    return null;
  }
  public double[] getXAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[]) data;
      double[] x;
      x = new double[siz];
      if (data instanceof float[])
        for (int i = 0; i < siz; i++)
          x[i] = ((float[]) data)[i];
      else if (data instanceof byte[])
        for (int i = 0; i < siz; i++)
          x[i] = ((byte[]) data)[i];
      else if (data instanceof short[])
        for (int i = 0; i < siz; i++)
          x[i] = ((short[]) data)[i];
      else if (data instanceof int[]) for (int i = 0; i < siz; i++)
        x[i] = ((int[]) data)[i];
      return x;
    }
    return null;
  }
  private int numberOfLinks = 0;
  private int getLinkHandle()
  {
    return numberOfLinks++;
  }
  private int rmvLinkHandle(int hLink)
  {
    return numberOfLinks > 0 ? numberOfLinks-- : 0;
  }
  private static final int simulatedDataArraySize = 512;
  private float[] simulatedDataArray = new float[simulatedDataArraySize];
  private int simulateData(Object data, int datalength)
  {
    int i, len;
    if (data instanceof double[])
    {
      len = ((double[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((double[]) data)[i] = (double) simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    else if (data instanceof float[])
    {
      len = ((float[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((float[]) data)[i] = simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    else if (data instanceof int[])
    {
      len = ((int[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((int[]) data)[i] = (int) simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    else if (data instanceof short[])
    {
      len = ((short[]) data).length;
      if (datalength > 0 && datalength < len) len = datalength;
      datalength = len;
      for (i = 0; i < len; i++)
        ((short[]) data)[i] = (short) simulatedDataArray[i % simulatedDataArraySize];
      return 0;
    }
    return -1;
  }
  private Random acopSimulatorRandom = new Random();
  private void updateSimulatedData()
  {
    for (int i = 0; i < simulatedDataArraySize; i++)
    {
      simulatedDataArray[i] = (float) (100 * Math.sin(i * 6.2832 / simulatedDataArraySize) + acopSimulatorRandom
          .nextFloat() * 10);
    }
  }
  private boolean active = false;
  private boolean terminate = false;
  private acopSimulatorThread ast;
  public AcopTransportSimulator()
  {
    active = true;
    ast = new acopSimulatorThread();
    ast.start();
    updateSimulatedData();
  }
  private class acopSimulatorThread extends Thread
  {
    boolean isWaiting = false;
    protected Date timer = new Date();
    public synchronized void run()
    {
      while (active)
      {
        if (terminate) break;
        try
        {
          isWaiting = true;
          sleep(100);
          timer.setTime(timer.getTime() + 100);
          isWaiting = false;
          updateSimulatedData();
          if (numberOfLinks > 0) // don't worry about link tables in the
                                  // simulator
          {
            for (int i = 0; i < numberOfLinks; i++)
              if (atc != null)
              {
                atc.callback(i, 0);
              }
          }
        }
        catch (InterruptedException e)
        {
          System.out.println(" InterruptedException : " + e);
        }
      }
    }
  }
  public int acquireDisplayProperties(Acop a, String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    a.setYMax(100);
    a.setYMin(-100);
    a.setXMax(512);
    a.setXMin(0);
    a.setYAxisLabel("Amplitude");
    a.setXAxisLabel("Simulated data");
    a.setGraphStyle(0);
    return 0;
  }
  public String[] getDeviceContexts()
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
  public String[] getDeviceGroups(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
  public String[] getDeviceNames(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
  public String[] getDeviceProperties(String deviceContext, String deviceGroup, String deviceName,
      String deviceProperty)
  {
    if (AcopSimulatorString[0] == null)
    {
      AcopSimulatorString[0] = new String("TEST");
    }
    return AcopSimulatorString;
  }
}
