/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport.tine;

import java.util.Date;
import java.util.Vector;

import de.desy.acop.chart.Acop;
import de.desy.acop.transport.AcopTransportAccessMethods;
import de.desy.acop.transport.AcopTransportPlug;
import de.desy.acop.transport.AcopTransportRequest;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.*;
import de.desy.tine.queryUtils.*;
import de.desy.tine.structUtils.TTaggedStructure;
import de.desy.tine.types.*;

/**
 * @version 	1.0
 * @author
 */
@SuppressWarnings("deprecation")
public class AcopTransportTine implements AcopTransportPlug
{
  private AcopTransportTineCallback tcb;
  private Vector<TLink> linkTable = new Vector<>();
  private int addLink(TLink tl)
  {
    for (TLink lnk : linkTable)
    {
      if (lnk.isClosed()) 
      {
        int idx = linkTable.indexOf(lnk);
        linkTable.set(idx, tl);
        return idx;
      }
    }
    linkTable.addElement(tl);
    return linkTable.indexOf(tl);
  }
  @SuppressWarnings("unused")
  private int getTableId(TLink tl)
  {
    return linkTable.indexOf(tl);
  }  
  private int getTableId(int linkId)
  {
    if (linkId >= 0 && linkId < linkTable.size() && linkTable.elementAt(linkId).callbackId == linkId)
    { // this will hit most of the time ...
      return linkId;
    }
    for (TLink lnk : linkTable)
    {
      if (lnk.callbackId == linkId) return linkTable.indexOf(lnk);
    }
    return -1;
  } 
  private TLink getTableElement(int linkId)
  {
    try
    {
      int id = getTableId(linkId);
      if (id >= 0) return linkTable.elementAt(id);
    }
    catch (Exception any) {} // sometimes there's a concurrency issue
    return null;
  }
  @SuppressWarnings("unused")
  private int rmvLink(TLink tl)
  {
    linkTable.removeElement(tl);
    return 0;
  }
  private String deviceName;
  private String makeDeviceName(AcopTransportRequest atr)
  {
    if (atr.deviceContext != null && atr.deviceContext.length() > 0)
    {
      return new String("/" + atr.deviceContext + 
                           "/" + atr.deviceGroup + 
                           "/" + atr.deviceName);
    }
    else
    {
      return new String(atr.deviceGroup + "/" + atr.deviceName);
    }
  }
  private int makeAccessRate(AcopTransportRequest atr)
  {
    int rate = (new Integer(atr.accessRate)).intValue();
    if (rate == 0) rate = 1000;
    return rate;
  }
  private short makeAccessMode(AcopTransportRequest atr)
  {
    short m = 0;
    String tmp = atr.accessMode.toString();
    if (tmp.indexOf("READ") >= 0) m |= TMode.CM_SINGLE;
    if (tmp.indexOf("WRITE") >= 0) m |= TMode.CM_SINGLE;
    if (tmp.indexOf("POLL") >= 0) m |= TMode.CM_POLL;
    if (tmp.indexOf("REFRESH") >= 0) m |= TMode.CM_REFRESH;
    if (tmp.indexOf("EVENT") >= 0) m |= TMode.CM_EVENT;
    if (tmp.indexOf("CONNECT") >= 0) m |= TMode.CM_CONNECT;
    if (tmp.indexOf("NETWORK") >= 0) m |= TMode.CM_NETWORK;
    if (tmp.indexOf("BROADCAST") >= 0) m |= TMode.CM_BCAST;
    if (tmp.indexOf("MULTICAST") >= 0) m |= TMode.CM_MCAST;
    if (tmp.indexOf("RECEIVE") >= 0) m |= TMode.CM_GLOBAL;
    if (tmp.indexOf("GLOBAL") >= 0) m |= TMode.CM_GLOBAL;
    if (tmp.indexOf("WAIT") >= 0) m |= TMode.CM_WAIT;
    if (tmp.indexOf("USE_ON_ERROR") >= 0) m |= TMode.CM_WAIT;
    return m;
  }
  public int acquireDisplayProperties(Acop a,String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    //PropertyQueryEx[] p = TQuery.getDevicePropertyInformation(deviceContext, deviceGroup, deviceName, deviceProperty);
    TPropertyQuery[] p = TQuery.getPropertyInformation(deviceContext, deviceGroup, deviceName, deviceProperty);
    if (p != null)
    {
      a.setYMax(p[0].prpMaxValue);
      a.setYMin(p[0].prpMinValue);
      a.setXMax(p[0].prpSize);
      a.setXMin(0);
      a.setYAxisLabel(p[0].prpUnits);
      a.setXAxisLabel(p[0].prpDescription);
      switch (p[0].prpGraphType) 
      {
        case 0: a.setGraphStyle(0); break;// lin-lin
        case 1: a.setGraphStyle(1); break;// log-lin
      }
      return 0;
    }
    return -1;
  }
  /**
   * @see AcopPack.acopTransportPlug#handleRequest(acopTransportRequest)
   */
  public int handleRequest(AcopTransportRequest atr)
  {
    TDataType dout, din;
    short access;
    TLink tl = null;

    switch (atr.accessMethod)
    {
      case AcopTransportAccessMethods.EXECUTE:
        if (!atr.accessMode.isReadMode() && !atr.accessMode.isWriteMode())
        {
          atr.status = "illegal mode (READ or WRITE required)";
          atr.setDataTimeStamp(new Date());
          atr.setSystemStamp(0);
          atr.setUserStamp(0);
          return -1;
        }
        deviceName = makeDeviceName(atr);
        dout = toTDataType(atr.getData(),atr.getArraySize(),!atr.isUnboundToData);
        if (dout == null) dout = new TDataType();
        din = toTDataType(atr.getDataEx(),atr.getArraySizeEx(),!atr.isUnboundToData);
        if (din == null) din = new TDataType();
        access = TAccess.CA_READ;
        if (atr.accessMode.isWriteMode()) access |= TAccess.CA_WRITE;
        try
        {
          tl = new TLink(deviceName,atr.deviceProperty,dout,din,access);
          atr.statusCode = tl.execute(makeAccessRate(atr));
          atr.setDataSize(dout.getLastDataSize());
          if (atr.statusCode > 0) atr.status = tl.getLastError();
          else atr.status = "Success";
          tl.close();
          if (atr.getDataTimeStamp() == null) atr.setDataTimeStamp(new Date());
          atr.getDataTimeStamp().setTime(dout.getTimeStamp());
          atr.setSystemStamp(dout.getSystemDataStamp());
          atr.setUserStamp(dout.getUserDataStamp());
          return atr.statusCode;
        }
        catch (Exception e)
        {
          atr.status = e.getMessage();
          if (tl != null) tl.close();
          return -1;
        }
      case AcopTransportAccessMethods.ATTACH:
        deviceName = makeDeviceName(atr);
        dout = toTDataType(atr.getData(),atr.getArraySize(),!atr.isUnboundToData);
        din = toTDataType(atr.getDataEx(),atr.getArraySizeEx(),!atr.isUnboundToData);
        access = TAccess.CA_READ;
        if (atr.accessMode.isWriteMode()) access |= TAccess.CA_WRITE;
        try
        {
          tl = new TLink(deviceName,atr.deviceProperty,dout,din,access);
          tcb = new AcopTransportTineCallback(atr.atc);
          int hLnk = addLink(tl);
          int lid = tl.attach(makeAccessMode(atr),tcb,makeAccessRate(atr),hLnk);
          if (lid < 0)
          {
            atr.statusCode = -lid;
            atr.status = tl.getError(atr.statusCode);
            return -1;
          }
          atr.setLinkHandle(hLnk);
          return hLnk; //atr.getLinkHandle();
        }
        catch (Exception e)
        {
          atr.statusCode = -1;
          atr.status = e.getMessage();
          return -1;
        }
      case AcopTransportAccessMethods.CLOSE:
        if ((tl=getTableElement(atr.getLinkHandle())) != null)
        {
          tl.close();
          //linkTable.remove(tl);
        }
        break;
      case AcopTransportAccessMethods.GETDATA: 
        if ((tl=getTableElement(atr.getLinkHandle())) == null)
        {
          atr.statusCode = -1;  
          atr.status = "Invalid Link Index";
          return -1;
        }
        atr.statusCode = tl.getLinkStatus();  
        if (atr.statusCode > 0) atr.status = tl.getLastError();
        else atr.status = "Success";
        if (atr.getDataTimeStamp() == null) atr.setDataTimeStamp(new Date());        
        atr.getDataTimeStamp().setTime(tl.getLastTimeStamp());
        if (tl.getOutputDataObject() != null) 
        {
          atr.setSystemStamp(tl.getOutputDataObject().getSystemDataStamp());
          atr.setUserStamp(tl.getOutputDataObject().getUserDataStamp());
        }        
        if (atr.isUnboundToData)
        {
          if (atr.getData() != null)
          {
            dout = tl.getOutputDataObject();
            dout.getData(atr.getData());
          }
          if (atr.getDataEx() != null)
          {
            din = tl.getInputDataObject();
            din.getData(atr.getDataEx());
          }
        }
        break;
      case AcopTransportAccessMethods.SHUTDOWN:
        for (TLink lnk : linkTable) lnk.close();
        break;
      case AcopTransportAccessMethods.GETINFO:
        if ((tl=getTableElement(atr.getLinkHandle())) == null)
        {
          atr.statusCode = -1;  
          atr.status = "Invalid Link Index";
          return -1;
        }
        atr.setDeviceInfo(TQuery.getModuleAddressInfo(tl.getDeviceServer(), tl.getContext()));
        break;
      default: 
        return -1;
    }
		return 0;
  }
  /**
   * @see AcopPack.acopTransportPlug#getPlugName()
   */
  public String getPlugName()
  {
    return "TINE";
  }
  public double[] getYAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[])data;
      double[] y;
      if (data instanceof SPECTRUM)
      { 
        siz = ((SPECTRUM)data).length;
        y = new double[siz];
        for (int i=0; i<siz; i++) y[i] = ((SPECTRUM)data).d_spect_array[i];
        return y;
      }
      if (data instanceof GSPECTRUM)
      { 
        siz = ((GSPECTRUM)data).length;
        y = new double[siz];
        for (int i=0; i<siz; i++) y[i] = ((GSPECTRUM)data).d_spect_array[i];
        return y;
      }
      y = new double[siz];
      if (data instanceof float[]) for (int i=0; i<siz; i++) y[i] = ((float[])data)[i];
      else if (data instanceof byte[]) for (int i=0; i<siz; i++) y[i] = ((byte[])data)[i];
      else if (data instanceof short[]) for (int i=0; i<siz; i++) y[i] = ((short[])data)[i];
      else if (data instanceof int[]) for (int i=0; i<siz; i++) y[i] = ((int[])data)[i];
      else if (data instanceof long[]) for (int i=0; i<siz; i++) y[i] = ((long[])data)[i];
      else if (data instanceof FLTINT[]) for (int i=0; i<siz; i++) y[i] = ((FLTINT[])data)[i] != null ? ((FLTINT[])data)[i].fval : 0;
      else if (data instanceof POINT[]) for (int i=0; i<siz; i++) y[i] = ((POINT[])data)[i] != null ? ((POINT[])data)[i].f2val : 0;
      else if (data instanceof FLTFLT[]) for (int i=0; i<siz; i++) y[i] = ((FLTFLT[])data)[i] != null ? ((FLTFLT[])data)[i].f1val : 0;
      else if (data instanceof DBLINT[]) for (int i=0; i<siz; i++) y[i] = ((DBLINT[])data)[i] != null ? ((DBLINT[])data)[i].dval : 0;
      else if (data instanceof INTINT[]) for (int i=0; i<siz; i++) y[i] = ((INTINT[])data)[i] != null ? ((INTINT[])data)[i].i1val : 0;
      else if (data instanceof DBLDBL[]) for (int i=0; i<siz; i++) y[i] = ((DBLDBL[])data)[i] != null ? ((DBLDBL[])data)[i].d1val : 0;
      else if (data instanceof TDS[]) for (int i=0; i<siz; i++) y[i] = ((TDS[])data)[i] != null ? ((TDS[])data)[i].dval : 0;
      else if (data instanceof INTFLTINT[]) for (int i=0; i<siz; i++) y[i] = ((INTFLTINT[])data)[i] != null ? ((INTFLTINT[])data)[i].fval : 0;
      else if (data instanceof FLTFLTINT[]) for (int i=0; i<siz; i++) y[i] = ((FLTFLTINT[])data)[i] != null ? ((FLTFLTINT[])data)[i].f1val : 0;
      else if (data instanceof FLTINTINT[]) for (int i=0; i<siz; i++) y[i] = ((FLTINTINT[])data)[i] != null ? ((FLTINTINT[])data)[i].f1val : 0;
      else if (data instanceof INTINTINT[]) for (int i=0; i<siz; i++) y[i] = ((INTINTINT[])data)[i] != null ? ((INTINTINT[])data)[i].i1val : 0;
      else if (data instanceof INTINTINTINT[]) for (int i=0; i<siz; i++) y[i] = ((INTINTINTINT[])data)[i] != null ? ((INTINTINTINT[])data)[i].i1val : 0;
      else if (data instanceof NAME16I[]) for (int i=0; i<siz; i++) y[i] = ((NAME16I[])data)[i] != null ? ((NAME16I[])data)[i].ival : 0;
      else if (data instanceof NAME32I[]) for (int i=0; i<siz; i++) y[i] = ((NAME32I[])data)[i] != null ? ((NAME32I[])data)[i].ival : 0;
      else if (data instanceof NAME64I[]) for (int i=0; i<siz; i++) y[i] = ((NAME64I[])data)[i] != null ? ((NAME64I[])data)[i].ival : 0;
      else if (data instanceof NAME16FI[]) for (int i=0; i<siz; i++) y[i] = ((NAME16FI[])data)[i] != null ? ((NAME16FI[])data)[i].fval : 0;
      else if (data instanceof NAME16II[]) for (int i=0; i<siz; i++) y[i] = ((NAME16II[])data)[i] != null ? ((NAME16II[])data)[i].i1val : 0;
      else if (data instanceof NAME16DBLDBL[]) for (int i=0; i<siz; i++) y[i] = ((NAME16DBLDBL[])data)[i] != null ? ((NAME16DBLDBL[])data)[i].d1val : 0;
      else if (data instanceof NAME32DBLDBL[]) for (int i=0; i<siz; i++) y[i] = ((NAME32DBLDBL[])data)[i] != null ? ((NAME32DBLDBL[])data)[i].d1val : 0;
      else if (data instanceof NAME64DBLDBL[]) for (int i=0; i<siz; i++) y[i] = ((NAME64DBLDBL[])data)[i] != null ? ((NAME64DBLDBL[])data)[i].d1val : 0;
      else if (data instanceof USTRING[]) for (int i=0; i<siz; i++) y[i] = ((USTRING[])data)[i] != null ? ((USTRING[])data)[i].f1val : 0;
      else if (data instanceof UNAME[]) for (int i=0; i<siz; i++) y[i] = ((UNAME[])data)[i] != null ? ((UNAME[])data)[i].f2val : 0;
      else if (data instanceof Object[]) 
      {
        if ((((Object[])data)[0]) instanceof Double) for (int i=0; i<siz; i++) y[i] =  ((Double)(((Object[])data)[i])).doubleValue();
        else if ((((Object[])data)[0]) instanceof Float) for (int i=0; i<siz; i++) y[i] =  ((Float)(((Object[])data)[i])).floatValue();
        else if ((((Object[])data)[0]) instanceof Short) for (int i=0; i<siz; i++) y[i] =  ((Short)(((Object[])data)[i])).shortValue();
        else if ((((Object[])data)[0]) instanceof Integer) for (int i=0; i<siz; i++) y[i] =  ((Integer)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof Long) for (int i=0; i<siz; i++) y[i] =  ((Long)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof Byte) for (int i=0; i<siz; i++) y[i] =  ((Byte)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof FLTINT) for (int i=0; i<siz; i++) y[i] =  ((FLTINT)(((Object[])data)[i])).fval;
        else if ((((Object[])data)[0]) instanceof POINT) for (int i=0; i<siz; i++) y[i] =  ((POINT)(((Object[])data)[i])).f2val;
        else if ((((Object[])data)[0]) instanceof FLTFLT) for (int i=0; i<siz; i++) y[i] =  ((FLTFLT)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof INTINT) for (int i=0; i<siz; i++) y[i] =  ((INTINT)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof DBLDBL) for (int i=0; i<siz; i++) y[i] =  ((DBLDBL)(((Object[])data)[i])).d1val;
        else if ((((Object[])data)[0]) instanceof TDS) for (int i=0; i<siz; i++) y[i] =  ((TDS)(((Object[])data)[i])).dval;
        else if ((((Object[])data)[0]) instanceof INTFLTINT) for (int i=0; i<siz; i++) y[i] =  ((INTFLTINT)(((Object[])data)[i])).fval;
        else if ((((Object[])data)[0]) instanceof FLTFLTINT) for (int i=0; i<siz; i++) y[i] =  ((FLTFLTINT)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof FLTINTINT) for (int i=0; i<siz; i++) y[i] =  ((FLTINTINT)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof INTINTINT) for (int i=0; i<siz; i++) y[i] =  ((INTINTINT)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof INTINTINTINT) for (int i=0; i<siz; i++) y[i] =  ((INTINTINTINT)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof NAME16I) for (int i=0; i<siz; i++) y[i] =  ((NAME16I)(((Object[])data)[i])).ival;
        else if ((((Object[])data)[0]) instanceof NAME32I) for (int i=0; i<siz; i++) y[i] =  ((NAME32I)(((Object[])data)[i])).ival;
        else if ((((Object[])data)[0]) instanceof NAME64I) for (int i=0; i<siz; i++) y[i] =  ((NAME64I)(((Object[])data)[i])).ival;
        else if ((((Object[])data)[0]) instanceof NAME16FI) for (int i=0; i<siz; i++) y[i] =  ((NAME16FI)(((Object[])data)[i])).fval;
        else if ((((Object[])data)[0]) instanceof NAME16II) for (int i=0; i<siz; i++) y[i] =  ((NAME16II)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof NAME16DBLDBL) for (int i=0; i<siz; i++) y[i] =  ((NAME16DBLDBL)(((Object[])data)[i])).d1val;
        else if ((((Object[])data)[0]) instanceof NAME32DBLDBL) for (int i=0; i<siz; i++) y[i] =  ((NAME32DBLDBL)(((Object[])data)[i])).d1val;
        else if ((((Object[])data)[0]) instanceof NAME64DBLDBL) for (int i=0; i<siz; i++) y[i] =  ((NAME64DBLDBL)(((Object[])data)[i])).d1val;
        else if ((((Object[])data)[0]) instanceof USTRING) for (int i=0; i<siz; i++) y[i] =  ((USTRING)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof UNAME) for (int i=0; i<siz; i++) y[i] =  ((UNAME)(((Object[])data)[i])).f2val;
      }
      return y;
    }
    return null;    
  }
  public double[] getXAxis(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      if (data instanceof double[]) return (double[])data;
      double[] x;
      if (data instanceof SPECTRUM)
      { 
        siz = ((SPECTRUM)data).length;
        x = new double[siz];
        for (int i=0; i<siz; i++) 
          x[i] = ((SPECTRUM)data).s_start + i * ((SPECTRUM)data).s_inc;
        return x;
      }
      if (data instanceof GSPECTRUM)
      { 
        siz = ((GSPECTRUM)data).length;
        x = new double[siz];
        for (int i=0; i<siz; i++) 
          x[i] = ((GSPECTRUM)data).s_start + i * ((GSPECTRUM)data).s_inc;
        return x;
      }
      x = new double[siz];
      if (data instanceof float[]) for (int i=0; i<siz; i++) x[i] = ((float[])data)[i];
      else if (data instanceof byte[]) for (int i=0; i<siz; i++) x[i] = ((byte[])data)[i];
      else if (data instanceof short[]) for (int i=0; i<siz; i++) x[i] = ((short[])data)[i];
      else if (data instanceof int[]) for (int i=0; i<siz; i++) x[i] = ((int[])data)[i];
      else if (data instanceof long[]) for (int i=0; i<siz; i++) x[i] = ((long[])data)[i];
      else if (data instanceof FLTINT[]) for (int i=0; i<siz; i++) x[i] = ((FLTINT[])data)[i] != null ? ((FLTINT[])data)[i].ival : 0;
      else if (data instanceof POINT[]) for (int i=0; i<siz; i++) x[i] = ((POINT[])data)[i] != null ? ((POINT[])data)[i].f1val :  0;
      else if (data instanceof FLTFLT[]) for (int i=0; i<siz; i++) x[i] = ((FLTFLT[])data)[i] != null ? ((FLTFLT[])data)[i].f2val : 0;
      else if (data instanceof DBLINT[]) for (int i=0; i<siz; i++) x[i] = ((DBLINT[])data)[i] != null ? ((DBLINT[])data)[i].ival : 0;
      else if (data instanceof INTINT[]) for (int i=0; i<siz; i++) x[i] = ((INTINT[])data)[i] != null ? ((INTINT[])data)[i].i2val : 0;
      else if (data instanceof DBLDBL[]) for (int i=0; i<siz; i++) x[i] = ((DBLDBL[])data)[i] != null ? ((DBLDBL[])data)[i].d2val : 0;
      else if (data instanceof TDS[]) for (int i=0; i<siz; i++) x[i] = ((TDS[])data)[i] != null ? ((TDS[])data)[i].tval : 0;
      else if (data instanceof INTFLTINT[]) for (int i=0; i<siz; i++) x[i] = ((INTFLTINT[])data)[i] != null ? ((INTFLTINT[])data)[i].i1val : 0;
      else if (data instanceof FLTFLTINT[]) for (int i=0; i<siz; i++) x[i] = ((FLTFLTINT[])data)[i] != null ? ((FLTFLTINT[])data)[i].f2val : 0;
      else if (data instanceof FLTINTINT[]) for (int i=0; i<siz; i++) x[i] = ((FLTINTINT[])data)[i] != null ? ((FLTINTINT[])data)[i].i1val : 0;
      else if (data instanceof INTINTINT[]) for (int i=0; i<siz; i++) x[i] = ((INTINTINT[])data)[i] != null ? ((INTINTINT[])data)[i].i2val : 0;
      else if (data instanceof INTINTINTINT[]) for (int i=0; i<siz; i++) x[i] = ((INTINTINTINT[])data)[i] != null ? ((INTINTINTINT[])data)[i].i2val : 0;
      else if (data instanceof NAME16I[]) for (int i=0; i<siz; i++) x[i] = ((NAME16I[])data)[i] != null ? i : 0;
      else if (data instanceof NAME32I[]) for (int i=0; i<siz; i++) x[i] = ((NAME32I[])data)[i] != null ? i : 0;
      else if (data instanceof NAME64I[]) for (int i=0; i<siz; i++) x[i] = ((NAME64I[])data)[i] != null ? i : 0;
      else if (data instanceof NAME16FI[]) for (int i=0; i<siz; i++) x[i] = ((NAME16FI[])data)[i] != null ? ((NAME16FI[])data)[i].ival : 0;
      else if (data instanceof NAME16II[]) for (int i=0; i<siz; i++) x[i] = ((NAME16II[])data)[i] != null ? ((NAME16II[])data)[i].i2val : 0;
      else if (data instanceof NAME16DBLDBL[]) for (int i=0; i<siz; i++) x[i] = ((NAME16DBLDBL[])data)[i] != null ? ((NAME16DBLDBL[])data)[i].d2val : 0;
      else if (data instanceof NAME32DBLDBL[]) for (int i=0; i<siz; i++) x[i] = ((NAME32DBLDBL[])data)[i] != null ? ((NAME32DBLDBL[])data)[i].d2val : 0;
      else if (data instanceof NAME64DBLDBL[]) for (int i=0; i<siz; i++) x[i] = ((NAME64DBLDBL[])data)[i] != null ? ((NAME64DBLDBL[])data)[i].d2val : 0;
      else if (data instanceof USTRING[]) for (int i=0; i<siz; i++) x[i] = ((USTRING[])data)[i] != null ? ((USTRING[])data)[i].f2val : 0;
      else if (data instanceof UNAME[]) for (int i=0; i<siz; i++) x[i] = ((UNAME[])data)[i] != null ? ((UNAME[])data)[i].f1val : 0;
      else if (data instanceof Object[]) 
      {
        if ((((Object[])data)[0]) instanceof Double) for (int i=0; i<siz; i++) x[i] =  ((Double)(((Object[])data)[i])).doubleValue();
        else if ((((Object[])data)[0]) instanceof Float) for (int i=0; i<siz; i++) x[i] =  ((Float)(((Object[])data)[i])).floatValue();
        else if ((((Object[])data)[0]) instanceof Short) for (int i=0; i<siz; i++) x[i] =  ((Short)(((Object[])data)[i])).shortValue();
        else if ((((Object[])data)[0]) instanceof Integer) for (int i=0; i<siz; i++) x[i] =  ((Integer)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof Long) for (int i=0; i<siz; i++) x[i] =  ((Long)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof Byte) for (int i=0; i<siz; i++) x[i] =  ((Byte)(((Object[])data)[i])).intValue();
        else if ((((Object[])data)[0]) instanceof FLTINT) for (int i=0; i<siz; i++) x[i] =  ((FLTINT)(((Object[])data)[i])).ival;
        else if ((((Object[])data)[0]) instanceof POINT) for (int i=0; i<siz; i++) x[i] =  ((POINT)(((Object[])data)[i])).f1val;
        else if ((((Object[])data)[0]) instanceof FLTFLT) for (int i=0; i<siz; i++) x[i] =  ((FLTFLT)(((Object[])data)[i])).f2val;
        else if ((((Object[])data)[0]) instanceof INTINT) for (int i=0; i<siz; i++) x[i] =  ((INTINT)(((Object[])data)[i])).i2val;
        else if ((((Object[])data)[0]) instanceof DBLDBL) for (int i=0; i<siz; i++) x[i] =  ((DBLDBL)(((Object[])data)[i])).d2val;
        else if ((((Object[])data)[0]) instanceof TDS) for (int i=0; i<siz; i++) x[i] =  ((TDS)(((Object[])data)[i])).tval;
        else if ((((Object[])data)[0]) instanceof INTFLTINT) for (int i=0; i<siz; i++) x[i] =  ((INTFLTINT)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof FLTFLTINT) for (int i=0; i<siz; i++) x[i] =  ((FLTFLTINT)(((Object[])data)[i])).f2val;
        else if ((((Object[])data)[0]) instanceof FLTINTINT) for (int i=0; i<siz; i++) x[i] =  ((FLTINTINT)(((Object[])data)[i])).i1val;
        else if ((((Object[])data)[0]) instanceof INTINTINT) for (int i=0; i<siz; i++) x[i] =  ((INTINTINT)(((Object[])data)[i])).i2val;
        else if ((((Object[])data)[0]) instanceof INTINTINTINT) for (int i=0; i<siz; i++) x[i] =  ((INTINTINTINT)(((Object[])data)[i])).i2val;
        else if ((((Object[])data)[0]) instanceof NAME16I) for (int i=0; i<siz; i++) x[i] =  i;
        else if ((((Object[])data)[0]) instanceof NAME32I) for (int i=0; i<siz; i++) x[i] =  i;
        else if ((((Object[])data)[0]) instanceof NAME64I) for (int i=0; i<siz; i++) x[i] =  i;
        else if ((((Object[])data)[0]) instanceof NAME16FI) for (int i=0; i<siz; i++) x[i] =  ((NAME16FI)(((Object[])data)[i])).ival;
        else if ((((Object[])data)[0]) instanceof NAME16II) for (int i=0; i<siz; i++) x[i] =  ((NAME16II)(((Object[])data)[i])).i2val;
        else if ((((Object[])data)[0]) instanceof NAME16DBLDBL) for (int i=0; i<siz; i++) x[i] =  ((NAME16DBLDBL)(((Object[])data)[i])).d2val;
        else if ((((Object[])data)[0]) instanceof NAME32DBLDBL) for (int i=0; i<siz; i++) x[i] =  ((NAME32DBLDBL)(((Object[])data)[i])).d2val;
        else if ((((Object[])data)[0]) instanceof NAME64DBLDBL) for (int i=0; i<siz; i++) x[i] =  ((NAME64DBLDBL)(((Object[])data)[i])).d2val;
        else if ((((Object[])data)[0]) instanceof USTRING) for (int i=0; i<siz; i++) x[i] =  ((USTRING)(((Object[])data)[i])).f2val;
        else if ((((Object[])data)[0]) instanceof UNAME) for (int i=0; i<siz; i++) x[i] =  ((UNAME)(((Object[])data)[i])).f3val;
      }
      return x;
    }
    return null;    
  }
  public String[] getXAxisChannelNames(Object data)
  {
    int siz = getObjectSize(data);
    if (siz > 0)
    {
      String[] s;
      s = new String[siz];
      if (data instanceof NAME8[]) for (int i=0; i<siz; i++) s[i] = ((NAME16I[])data)[i] != null ? ((NAME8[])data)[i].getName() : "";
      if (data instanceof NAME16[]) for (int i=0; i<siz; i++) s[i] = ((NAME16I[])data)[i] != null ? ((NAME16[])data)[i].getName() : "";
      else if (data instanceof NAME32[]) for (int i=0; i<siz; i++) s[i] = ((NAME32I[])data)[i] != null ? ((NAME32[])data)[i].getName() : "";
      else if (data instanceof NAME48[]) for (int i=0; i<siz; i++) s[i] = ((NAME32I[])data)[i] != null ? ((NAME48[])data)[i].getName() : "";
      else if (data instanceof NAME64[]) for (int i=0; i<siz; i++) s[i] = ((NAME64I[])data)[i] != null ? ((NAME64[])data)[i].getName() : "";
      else if (data instanceof NAME16I[]) for (int i=0; i<siz; i++) s[i] = ((NAME16I[])data)[i] != null ? ((NAME16I[])data)[i].getName() : "";
      else if (data instanceof NAME32I[]) for (int i=0; i<siz; i++) s[i] = ((NAME32I[])data)[i] != null ? ((NAME32I[])data)[i].getName() : "";
      else if (data instanceof NAME64I[]) for (int i=0; i<siz; i++) s[i] = ((NAME64I[])data)[i] != null ? ((NAME64I[])data)[i].getName() : "";
      else if (data instanceof NAME16FI[]) for (int i=0; i<siz; i++) s[i] = ((NAME16FI[])data)[i] != null ? ((NAME16FI[])data)[i].getName() : "";
      else if (data instanceof NAME16II[]) for (int i=0; i<siz; i++) s[i] = ((NAME16II[])data)[i] != null ? ((NAME16II[])data)[i].getName() : "";
      else if (data instanceof NAME16DBLDBL[]) for (int i=0; i<siz; i++) s[i] = ((NAME16DBLDBL[])data)[i] != null ? ((NAME16DBLDBL[])data)[i].getName() : "";
      else if (data instanceof NAME32DBLDBL[]) for (int i=0; i<siz; i++) s[i] = ((NAME32DBLDBL[])data)[i] != null ? ((NAME32DBLDBL[])data)[i].getName() : "";
      else if (data instanceof NAME64DBLDBL[]) for (int i=0; i<siz; i++) s[i] = ((NAME64DBLDBL[])data)[i] != null ? ((NAME64DBLDBL[])data)[i].getName() : "";
      else if (data instanceof USTRING[]) for (int i=0; i<siz; i++) s[i] = ((USTRING[])data)[i] != null ? ((USTRING[])data)[i].getString() : "";
      else if (data instanceof UNAME[]) for (int i=0; i<siz; i++) s[i] = ((UNAME[])data)[i] != null ? ((UNAME[])data)[i].getString() : "";
      else if (data instanceof Object[]) 
      {
        if ((((Object[])data)[0]) instanceof NAME8) for (int i=0; i<siz; i++) s[i] =  ((NAME8)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME16) for (int i=0; i<siz; i++) s[i] =  ((NAME16)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME32) for (int i=0; i<siz; i++) s[i] =  ((NAME32)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME48) for (int i=0; i<siz; i++) s[i] =  ((NAME48)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME64) for (int i=0; i<siz; i++) s[i] =  ((NAME64)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME16I) for (int i=0; i<siz; i++) s[i] =  ((NAME16I)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME32I) for (int i=0; i<siz; i++) s[i] =  ((NAME32I)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME64I) for (int i=0; i<siz; i++) s[i] =  ((NAME64I)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME16FI) for (int i=0; i<siz; i++) s[i] =  ((NAME16FI)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME16II) for (int i=0; i<siz; i++) s[i] =  ((NAME16II)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME16DBLDBL) for (int i=0; i<siz; i++) s[i] =  ((NAME16DBLDBL)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME32DBLDBL) for (int i=0; i<siz; i++) s[i] =  ((NAME32DBLDBL)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof NAME64DBLDBL) for (int i=0; i<siz; i++) s[i] =  ((NAME64DBLDBL)(((Object[])data)[i])).getName();
        else if ((((Object[])data)[0]) instanceof USTRING) for (int i=0; i<siz; i++) s[i] =  ((USTRING)(((Object[])data)[i])).getString();
        else if ((((Object[])data)[0]) instanceof UNAME) for (int i=0; i<siz; i++) s[i] =  ((UNAME)(((Object[])data)[i])).getString();
        else return null;
      }
      else return null;
      return s;
    }
    return null;    
  }
  public int getObjectSize(Object data)
  {
    if (data == null) return 0;
    if (data instanceof char[]) return ((char[])data).length;
    if (data instanceof TTaggedStructure[]) return ((TTaggedStructure[])data).length;
    if (data instanceof TTaggedStructure) return 1;
    return TDataType.getObjectSize(data);
  }
  private TDataType toTDataType(Object data, int length, boolean isBound)
  {
    if (data == null) return null;  // return null if Object is null !
    if (data instanceof double[])
    {
      return isBound ? new TDataType((double[])data, length) : new TDataType(((double[])data).length,TFormat.CF_DOUBLE);
    }
    if (data instanceof float[])
    {
      return isBound ? new TDataType((float[])data, length) : new TDataType(length,TFormat.CF_FLOAT);
    }
    if (data instanceof int[])
    {
      return isBound ? new TDataType((int[])data, length) : new TDataType(((int[])data).length,TFormat.CF_LONG);
    }
    if (data instanceof short[])
    {
      return isBound ? new TDataType((short[])data, length) : new TDataType(((short[])data).length,TFormat.CF_SHORT);
    }
    if (data instanceof byte[])
    {
      return isBound ? new TDataType((byte[])data, length) : new TDataType(((byte[])data).length,TFormat.CF_BYTE);
    }
    if (data instanceof long[])
    {
      return isBound ? new TDataType((long[])data, length) : new TDataType(((long[])data).length,TFormat.CF_DLONG);
    }
    if (data instanceof boolean[])
    {
      return isBound ? new TDataType((boolean[])data, length) : new TDataType(((boolean[])data).length,TFormat.CF_BOOLEAN);
    }
    if (data instanceof TCompoundDataObject[])
    {
      return isBound ? new TDataType((TCompoundDataObject[])data, length) : new TDataType(((TCompoundDataObject[])data).length,((TCompoundDataObject[])data)[0].getFormat());
    } 
    if (data instanceof SPECTRUM)
    {
      return isBound ? new TDataType((SPECTRUM)data) : new TDataType(((SPECTRUM)data).length,TFormat.CF_SPECTRUM);
    }
    if (data instanceof GSPECTRUM)
    {
      return isBound ? new TDataType((GSPECTRUM)data) : new TDataType(((GSPECTRUM)data).length,TFormat.CF_GSPECTRUM);
    }
    if (data instanceof String)
    {
      return isBound ? new TDataType((String)data, length) : new TDataType(((String[])data).length,TFormat.CF_TEXT);
    }
    if (data instanceof StringBuffer)
    {
      return isBound ? new TDataType((StringBuffer)data, length) : new TDataType(((StringBuffer)data).length(),TFormat.CF_TEXT);
    }
    if (data instanceof char[])
    {
      return isBound ? new TDataType((char[])data, length) : new TDataType(((char[])data).length,TFormat.CF_TEXT);
    }
    if (data instanceof TTaggedStructure)
    {
      return isBound ? new TDataType((TTaggedStructure)data) : new TDataType(((TTaggedStructure)data).getSizeInBytes(),TFormat.CF_STRUCT);
    }
    if (data instanceof TTaggedStructure[])
    {
      return isBound ? new TDataType((TTaggedStructure[])data) : new TDataType(((TTaggedStructure[])data)[0].getSizeInBytes()*((TTaggedStructure[])data).length,TFormat.CF_STRUCT);
    }
    return null;
  }
  public String[] getDeviceContexts() 
  {
    return TQuery.getContexts();
  }
  public String[] getDeviceGroups(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    return TQuery.getDeviceServers(deviceContext);
  }
  public String[] getDeviceNames(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    return TQuery.getDeviceNames(deviceContext,deviceGroup,deviceProperty);
  }
  public String[] getDeviceProperties(String deviceContext,String deviceGroup,String deviceName,String deviceProperty)
  {
    return TQuery.getDeviceProperties(deviceContext,deviceGroup,deviceName);
  }
  public String getTargetInformation(String deviceContext, String deviceGroup, String deviceName, String deviceProperty)
  {
    return TQuery.getModuleAddressInfo(deviceGroup, deviceContext);
  }
}
