package de.desy.acop.video;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;

import javax.swing.AbstractAction;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.util.PopupManageable;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.components.util.ScreenCapturer;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.tools.ConnectionParametersReceiver;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.video.sw.ImageDisplayer;
import de.desy.video.sw.TineHandler;
import de.desy.video.sw.ImageDisplayer.ColorMap;


public class AcopVideo extends JPanel implements ConnectionParametersReceiver, PopupManageable{

	private static final long serialVersionUID = 1L;
	
	private ImageDisplayer imageDisplayer;
	
	private TineHandler tineHandler;
	private ConnectionParameters parameters;
	private AbstractCustomizerPanel customizer;
	private PopupManager popupManager;
	private AcopVideoInfoDialog infoDialog;
	
	private boolean aoiZoom = false;
	private boolean keepAspectRatio = false;
	private boolean overlayEnabled = false;
	private boolean falseColorSwitchingEnabled = true;
	private ColorMap colorMap = ColorMap.GRAYSCALE;	
		
	public AcopVideo() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.add(getImageDisplayer(), BorderLayout.CENTER);
		getImageDisplayer().addMouseListener(getPopupManager().getMouseHook());
		this.addMouseListener(getPopupManager().getMouseHook());
	}
	
	private ImageDisplayer getImageDisplayer() {
		if (imageDisplayer == null) {
			imageDisplayer = new ImageDisplayer();
			imageDisplayer.addPropertyChangeListener(new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					String propertyName = evt.getPropertyName();
					if ("AOIZoom".equals(propertyName)) {
						setAOIZoom(getImageDisplayer().isAOIZoom());
					} else if ("keepAspectRatioEnabled".equals(propertyName)) {
						setKeepAspectRatio(getImageDisplayer().isKeepAspectRatioEnabled());
					} else if ("overlayEnabled".equals(propertyName)) {
						setOverlayEnabled(getImageDisplayer().isOverlayEnabled());
					} else if ("falseColorSwitchingEnabled".equals(propertyName)) {
						setFalseColorSwitchingEnabled(getImageDisplayer().isFalseColorSwitchingEnabled());
					} else if ("colorMap".equals(propertyName)) {
						setColorMap(getImageDisplayer().getColorMap());
					}
				}
			});
		}
		return imageDisplayer;
	}
		
	public void setAOIZoom(boolean zoom) {
		if (this.aoiZoom == zoom) return;
		this.aoiZoom = zoom;
		getImageDisplayer().setAOIZoom(zoom);
		firePropertyChange("AOIZoom", !aoiZoom, aoiZoom);
	}
	
	public boolean isAOIZoom() {
		return aoiZoom;
	}
	
	public void setKeepAspectRatio(boolean keepAspectRatio) {
		if (this.keepAspectRatio == keepAspectRatio) return;
		this.keepAspectRatio = keepAspectRatio;
		getImageDisplayer().setKeepAspectRatioEnabled(keepAspectRatio);
		firePropertyChange("keepAspectRatio", !keepAspectRatio, keepAspectRatio);
	}
	
	public boolean isKeepAspectRatio() {
		return keepAspectRatio;
	}
	
	public void setOverlayEnabled(boolean overlayEnabled) {
		if (this.overlayEnabled == overlayEnabled) return;
		this.overlayEnabled = overlayEnabled;
		getImageDisplayer().setOverlayEnabled(overlayEnabled);
		firePropertyChange("overlayEnabled", !overlayEnabled, overlayEnabled);
	}
	
	public boolean isOverlayEnabled() {
		return overlayEnabled;
	}
	
	public void setColorMap(ColorMap colorMap) {
		if (this.colorMap == colorMap) return;
		ColorMap oldValue = this.colorMap;
		this.colorMap = colorMap;
		getImageDisplayer().setColorMap(this.colorMap);
		firePropertyChange("colorMap", oldValue, this.colorMap);
	}
	
	public ColorMap getColorMap() {
		return colorMap;
	}
	
	private TineHandler getTineHandler() {
		if (tineHandler == null) {
			tineHandler = new TineHandler(getImageDisplayer());
		}
		return tineHandler; 
	}
	
	private void setFalseColorSwitchingEnabled(boolean falseColorSwitchingEnabled) {
		if (this.falseColorSwitchingEnabled == falseColorSwitchingEnabled) return;
		this.falseColorSwitchingEnabled = falseColorSwitchingEnabled;
		firePropertyChange("falseColorSwitchingEnabled", !falseColorSwitchingEnabled, falseColorSwitchingEnabled);
	}
	
	public boolean isFalseColorSwitchingEnabled() {
		return falseColorSwitchingEnabled;
	}	
		
	public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
		}
		return customizer;
	}
	
	public PopupManager getPopupManager() {
		if (popupManager == null) {
			popupManager = new PopupManager(this, true);
			popupManager.addAction(new AbstractAction("Preferences...") {
				private static final long serialVersionUID = 1L;
				public void actionPerformed(ActionEvent e)
				{
					getCustomizer().showDialog();
				}
			});
			
			popupManager.addAction(new AbstractAction("Capture screen..."){
				private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent e){
				   	ScreenCapturer sc = new ScreenCapturer(getImageDisplayer());
					sc.showScreenDialog();
				}
			});
			
			popupManager.addAction(new AbstractAction("Info...") {
				private static final long serialVersionUID = 1L;
				public void actionPerformed(ActionEvent e)
				{
					getInfoDialog().setVisible(true);
					getInfoDialog().toFront();
				}
			});
			//disable this, in order for popup to be drawn on the awt canvas
			JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		}
		return popupManager;
	}
	
	private AcopVideoInfoDialog getInfoDialog() {
		if (infoDialog == null) {
			infoDialog = new AcopVideoInfoDialog(this);
		}
		return infoDialog;
	}

	public ConnectionParameters getConnectionParameters() {
		return parameters;
	}

	public void setConnectionParameters(ConnectionParameters cp) throws CommonException, PropertyVetoException {
		if ((this.parameters != null && this.parameters.equals(cp)) ||
				(this.parameters == null && cp == null)) return;
		
		ConnectionParameters oldValue = this.parameters;
		this.parameters = cp;
		stop();
		firePropertyChange("connectionParameters", oldValue, this.parameters);
	}
	
	private String getRemoteName(ConnectionParameters p) {
		if (p == null) return null;
		String remoteName = "/"+p.getDeviceContext() +"/" +p.getDeviceGroup()+"/";
		if (p.getDeviceName() != null && p.getDeviceName().length() != 0) {
			remoteName += p.getDeviceName();
		}
		return remoteName;
	}
	
	public void start() {
		if (parameters == null) return;
		final String remoteName = getRemoteName(parameters);
		if (remoteName == null || remoteName.trim().length()==0) return;
		setEnabled(false);
		new Thread(new Runnable(){
			public void run() {
				try {					
					getTineHandler().openLink(remoteName, parameters.getAccessMode(), parameters.getAccessRate());
				} catch (Exception e) {
					JOptionPane.showMessageDialog(AcopVideo.this, "Error connecting to the address '" + remoteName+"'.\n" + e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				} finally {
					setEnabled(true);
				}
			}
		}).start();
	}
	
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		getImageDisplayer().setEnabled(enabled);
	}
	
	public void stop() {
		getTineHandler().closeLink();
	}
	
	public boolean saveAsPNGImage(String fileNamePath) {
		return getImageDisplayer().saveAsPNGImage(fileNamePath);
	}
	
	public static void main(String[] args) {
		RunnerHelper.runComponent(new AcopVideo(), 500,500);
	}
	
}
