package de.desy.acop.video;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler;
import com.cosylab.gui.util.DisplayerParametersFlavor;

import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.displayers.tools.AcopDisplayerParametersFlavor;
import de.desy.acop.displayers.tools.ConnectionParametersFlavor;
import de.desy.acop.transport.ConnectionParameters;

/**
 * 
 * <code>AcopVideoTransferHandler</code> is the transfer handler employed by the
 * AcopVideo.
 * 
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 * @author <a href="mailto:david.melkumyan@desy.de">David Melkumyan</a>
 * 
 */
public class AcopVideoTransferHandler extends AbstractDisplayerParametersTransferHandler {

	private static final long serialVersionUID = 1L;

	/**
	 * Constructs a new AcopVideoTransferHandler associated with the given
	 * AcopVideo.
	 * 
	 * @param comp
	 */
	public AcopVideoTransferHandler(AcopVideo comp) {
		super(comp, true, true, new DataFlavor[] { AcopVideoParametersFlavor.FLAVOR, AcopDisplayerParametersFlavor.FLAVOR, DisplayerParametersFlavor.FLAVOR,
				ConnectionParametersFlavor.FLAVOR, DataFlavor.stringFlavor }, true);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.cosylab.gui.util.DefaultDisplayerParametersTransferHandler#
	 * exportDisplayerParameters()
	 */
	@Override
	protected DisplayerParameters exportDisplayerParameters() {
		AcopVideo disp = (AcopVideo) displayer;
		return new AcopVideoParameters(disp.getConnectionParameters(), //
				disp.isAOIZoom(), //
				disp.getImageZoom(), //
				disp.getImageOrientation(), //
				disp.getOverlayState(), //
				disp.isKeepAspectRatio(), //  
				disp.isHistogramEqualisation(), //
				disp.getHistogramMin(), //
				disp.getHistogramMax(), //
				disp.getColorMap(), //
				disp.getGridColor(), //
				disp.getGridMode(), //
				disp.getGridSize(), //
				disp.isGridVisible(), //
				disp.isShowDroppedFramesNumber(), //
				disp.isDisplayClutHeader());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java
	 * .awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
		if (flavor instanceof AcopDisplayerParametersFlavor) {
			return new AcopDisplayerParameters[] { (AcopDisplayerParameters) exportDisplayerParameters() };
		} else if (flavor instanceof ConnectionParametersFlavor) {
			return new ConnectionParameters[] { ((AcopDisplayerParameters) exportDisplayerParameters()).getConnectionParameters() };
		} else if (flavor.isFlavorTextType()) {
			return ((AcopDisplayerParameters) exportDisplayerParameters()).getConnectionParameters().getRemoteName();
		} else {
			return super.getTransferData(flavor);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#importData
	 * (javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopDisplayerParametersFlavor.FLAVOR)) {
				Object[] drop = (Object[]) t.getTransferData(AcopDisplayerParametersFlavor.FLAVOR);
				DisplayerParameters[] dp = null;
				dp = castParameters(drop);
				if (dp != null) {
					return importDisplayerParameters(dp);
				}
			} else if (t.isDataFlavorSupported(ConnectionParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(ConnectionParametersFlavor.FLAVOR);

				if (drop != null) {
					ConnectionParameters[] cp = null;
					if (drop instanceof ConnectionParameters) {
						cp = new ConnectionParameters[] { (ConnectionParameters) drop };
					} else if (drop instanceof ConnectionParameters[]) {
						cp = (ConnectionParameters[]) drop;
					} else {
						return super.importData(comp, t);
					}

					AcopVideoParameters[] param = new AcopVideoParameters[cp.length];
					for (int i = 0; i < param.length; i++) {
						param[i] = new AcopVideoParameters(cp[i]);
					}

					return importDisplayerParameters(param);
				}
			}
			return super.importData(comp, t);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#importString
	 * (java.lang.String)
	 */
	@Override
	protected boolean importString(String string) throws PropertyVetoException {
		if (string.indexOf('\n') > 0) {
			return false;
		}
		try {
			ConnectionParameters cp = new ConnectionParameters(string);
			return importDisplayerParameters(new AcopVideoParameters[] { new AcopVideoParameters(cp) });
		} catch (IllegalArgumentException e) {
			return false;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#
	 * castParameters(java.lang.Object[])
	 */
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopVideoParameters[]) {
			return (AcopVideoParameters[]) parameters.getClass().cast(parameters);
		} else if (parameters.length == 1) {
			if (parameters[0] instanceof AcopVideoParameters[]) {
				return (AcopVideoParameters[]) parameters[0];
			} else if (parameters[0] instanceof AcopVideoParameters) {
				return new AcopVideoParameters[] { (AcopVideoParameters) parameters[0] };
			}
		}
		return super.castParameters(parameters);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.cosylab.gui.util.AbstractDisplayerParametersTransferHandler#
	 * setParametersToDisplayer(java.lang.Object,
	 * com.cosylab.gui.displayers.DisplayerParameters)
	 */
	@Override
	public boolean setParametersToDisplayer(Object disp, DisplayerParameters parameters) throws PropertyVetoException {
		((AcopVideo) disp).stop();
		boolean result = super.setParametersToDisplayer(disp, parameters);
		((AcopVideo) disp).start();
		return result;
	}

}
