package de.desy.acop.video;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.Arrays;

import com.cosylab.gui.util.DisplayerParametersFlavor;

import de.desy.acop.displayers.tools.AcopDisplayerParametersFlavor;
import de.desy.acop.displayers.tools.ConnectionParametersFlavor;

/**
 * 
 * <code>AcopVideoTransferable</code> is the implementation of
 * the Transferable which is used by the AcopVideo. It allows
 * transferring of the AcopVideoParameters.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class AcopVideoTransferable implements Transferable {

	private static final DataFlavor[] flavors = new DataFlavor[]{AcopVideoParametersFlavor.FLAVOR,
		AcopDisplayerParametersFlavor.FLAVOR, DisplayerParametersFlavor.FLAVOR,
		ConnectionParametersFlavor.FLAVOR, DataFlavor.stringFlavor};
	
	
	private AcopVideoParameters parameters;
	
	/**
	 * Constructs a new AcopVideoTransferable.
	 * @param parameters
	 */
	public AcopVideoTransferable(AcopVideoParameters parameters) {
		this.parameters = parameters;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.datatransfer.Transferable#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
		if (flavor == AcopVideoParametersFlavor.FLAVOR) {
			return parameters;
		} else if (flavor == AcopDisplayerParametersFlavor.FLAVOR) {
			return parameters;
		} else if (flavor == DisplayerParametersFlavor.FLAVOR) {
			return parameters;
		} else if (flavor == ConnectionParametersFlavor.FLAVOR) {
			return parameters.getConnectionParameters();
		} else if (flavor == DataFlavor.stringFlavor) {
			if (parameters.getConnectionParameters() == null) return null;
			return parameters.getConnectionParameters().getRemoteName();
		}
		throw new UnsupportedFlavorException(flavor);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
	 */
	public DataFlavor[] getTransferDataFlavors() {
		return flavors;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(java.awt.datatransfer.DataFlavor)
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return Arrays.asList(flavors).contains(flavor);
	}
	
}
