/**
 *
 */
package de.desy.acop.video;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.DecimalFormat;
import java.util.stream.IntStream;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import de.desy.acop.video.analysis.AnalysisImage;
import de.desy.acop.video.analysis.gaussian.AImage;
import de.desy.acop.video.analysis.gaussian.AbstractGaussianImage;
import de.desy.acop.video.analysis.interferometer.DIFMImage;
import de.desy.acop.video.analysis.interferometer.IFMImage;
import de.desy.acop.video.displayer.ImageOrientation;

/**
 * </code>NumericDataDisplayer</code> displays numerical image analysis data in form of a table.
 *
 * @author Tilen Kusterle, Cosylab
 */
public class NumericDataDisplayer extends JPanel {

    private static class TableModel extends DefaultTableModel {

        private static final long serialVersionUID = 1161377304568263895L;

        public TableModel(int rows, int columns) {
            super(rows,columns);
        }

        @Override
        public boolean isCellEditable(int row, int column) {
            return false;
        }
    }

    private static final long serialVersionUID = 1L;
    private final DecimalFormat df = new DecimalFormat("#0.0");
    private final DecimalFormat hdf = new DecimalFormat("#0.0000");
    private String type = "AISr4";
    private TableModel tmGaussian3C;
    private TableModel tmGaussian2C;
    private JPanel gaussianPanel;
    private TableModel tmInterferometer2C;
    private JPanel interferometerPanel;
    private TableModel tmDoubleInterferometer3C;
    private TableModel tmDoubleInterferometer2C;
    private JPanel doubleInterferometerPanel;
    private TableModel tmGaussian2D;
    private JPanel gaussian2DPanel;
    private ImageOrientation orientation = ImageOrientation.NoChange;

    private class CellRenderer extends DefaultTableCellRenderer {

        private static final long serialVersionUID = 1L;
        private int[] headerRows;

        public CellRenderer(int[] headerRows) {
            this.headerRows = headerRows;
        }

        /*
         * (non-Javadoc)
         * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable,
         * java.lang.Object, boolean, boolean, int, int)
         */
        @Override
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,
                int row, int column) {
            JLabel label = (JLabel)super.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,column);
            for (int i = 0; i < headerRows.length; i++) {
                if (headerRows[i] == row) {
                    label.setHorizontalAlignment(SwingConstants.CENTER);
                    label.setFont(label.getFont().deriveFont(Font.BOLD));
                    return label;
                }
            }
            if (column == 0) label.setFont(label.getFont().deriveFont(Font.ITALIC));
            label.setHorizontalAlignment(SwingConstants.LEADING);
            return label;
        }
    }

    /**
     * Constructs this </code>NumericDataDisplayer</code>..
     */
    public NumericDataDisplayer() {
        super(new GridBagLayout());
        add(getGaussianPanel(),new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,
                new Insets(0,0,0,0),0,0));
        add(getGaussian2DPanel(),new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,
                new Insets(0,0,0,0),0,0));
        add(getInterferometerPanel(),new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,
                GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
        add(getDoubleInterferometerPanel(),new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,
                GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
    }

    public void setImageOrientation(ImageOrientation orientation) {
        this.orientation = orientation;
    }

    private JPanel getInterferometerPanel() {
        if (interferometerPanel == null) {
            interferometerPanel = new JPanel(new GridBagLayout());
            tmInterferometer2C = new TableModel(16,2);
            tmInterferometer2C.setValueAt("Projection",0,0);
            tmInterferometer2C.setValueAt("Value",0,1);
            tmInterferometer2C.setValueAt("Background",1,0);
            tmInterferometer2C.setValueAt("Amplitude",2,0);
            tmInterferometer2C.setValueAt("Asinc",3,0);
            tmInterferometer2C.setValueAt("PSinc",4,0);
            tmInterferometer2C.setValueAt("ACos",5,0);
            tmInterferometer2C.setValueAt("PCos",6,0);
            tmInterferometer2C.setValueAt("Vis",7,0);
            tmInterferometer2C.setValueAt("IMAGE DATA",8,0);
            tmInterferometer2C.setValueAt("VALUE",8,1);
            tmInterferometer2C.setValueAt("width [pixels]",9,0);
            tmInterferometer2C.setValueAt("height [pixels]",10,0);
            tmInterferometer2C.setValueAt("ROI X [pixels]",11,0);
            tmInterferometer2C.setValueAt("ROI Y [pixels]",12,0);
            tmInterferometer2C.setValueAt("ROI width [pixels]",13,0);
            tmInterferometer2C.setValueAt("ROI height [pixels]",14,0);
            tmInterferometer2C.setValueAt("threshold",15,0);
            JTable interferometerTable = new JTable(tmInterferometer2C);
            interferometerTable.setDefaultRenderer(Object.class,new CellRenderer(new int[]{0,8}));
            interferometerPanel.add(interferometerTable,new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.CENTER,
                    GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
            interferometerPanel.setBorder(new LineBorder(Color.BLACK));
        }
        return interferometerPanel;
    }

    private JPanel getDoubleInterferometerPanel() {
        if (doubleInterferometerPanel == null) {
            doubleInterferometerPanel = new JPanel(new GridBagLayout());
            tmDoubleInterferometer3C = new TableModel(8,3);
            tmDoubleInterferometer3C.setValueAt("Projection",0,0);
            tmDoubleInterferometer3C.setValueAt("Horizontal",0,1);
            tmDoubleInterferometer3C.setValueAt("Vertical",0,2);
            tmDoubleInterferometer3C.setValueAt("Background",1,0);
            tmDoubleInterferometer3C.setValueAt("Amplitude",2,0);
            tmDoubleInterferometer3C.setValueAt("Asinc",3,0);
            tmDoubleInterferometer3C.setValueAt("PSinc",4,0);
            tmDoubleInterferometer3C.setValueAt("ACos",5,0);
            tmDoubleInterferometer3C.setValueAt("PCos",6,0);
            tmDoubleInterferometer3C.setValueAt("Vis",7,0);
            tmDoubleInterferometer2C = new TableModel(8,2);
            tmDoubleInterferometer2C.setValueAt("IMAGE DATA",0,0);
            tmDoubleInterferometer2C.setValueAt("VALUE",0,1);
            tmDoubleInterferometer2C.setValueAt("width [pixels]",1,0);
            tmDoubleInterferometer2C.setValueAt("height [pixels]",2,0);
            tmDoubleInterferometer2C.setValueAt("ROI X [pixels]",3,0);
            tmDoubleInterferometer2C.setValueAt("ROI Y [pixels]",4,0);
            tmDoubleInterferometer2C.setValueAt("ROI width [pixels]",5,0);
            tmDoubleInterferometer2C.setValueAt("ROI height [pixels]",6,0);
            tmDoubleInterferometer2C.setValueAt("threshold",7,0);
            JTable interferometerTable3C = new JTable(tmDoubleInterferometer3C);
            interferometerTable3C.setDefaultRenderer(Object.class,new CellRenderer(new int[]{0}));
            doubleInterferometerPanel.add(interferometerTable3C,new GridBagConstraints(0,0,1,1,1,0,
                    GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
            JTable interferometerTable2C = new JTable(tmDoubleInterferometer2C);
            interferometerTable2C.setDefaultRenderer(Object.class,new CellRenderer(new int[]{0}));
            doubleInterferometerPanel.add(interferometerTable2C,new GridBagConstraints(0,1,1,1,1,0,
                    GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
            doubleInterferometerPanel.setBorder(new LineBorder(Color.BLACK));
        }
        return doubleInterferometerPanel;
    }

    private JPanel getGaussianPanel() {
        if (gaussianPanel == null) {
            gaussianPanel = new JPanel(new GridBagLayout());
            tmGaussian3C = new TableModel(6,3);
            tmGaussian3C.setValueAt("XY PROJECTION",0,0);
            tmGaussian3C.setValueAt("X VALUE (stat./fit)",0,1);
            tmGaussian3C.setValueAt("Y VALUE (stat./fit)",0,2);
            tmGaussian3C.setValueAt("mean [pixels]",1,0);
            tmGaussian3C.setValueAt("STD [pixels]",2,0);
            tmGaussian3C.setValueAt("amplitude",3,0);
            tmGaussian3C.setValueAt("constant",4,0);
            tmGaussian3C.setValueAt("slope",5,0);
            JTable gaussianTable3C = new JTable(tmGaussian3C);
            gaussianTable3C.setDefaultRenderer(Object.class,new CellRenderer(new int[]{0}));
            gaussianPanel.add(gaussianTable3C,new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.CENTER,
                    GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
            tmGaussian2C = new TableModel(14,2);
            tmGaussian2C.setValueAt("2D DATA",0,0);
            tmGaussian2C.setValueAt("VALUE",0,1);
            tmGaussian2C.setValueAt("STD A [pixels]",1,0);
            tmGaussian2C.setValueAt("STD B [pixels]",2,0);
            tmGaussian2C.setValueAt("rotation [degrees]",3,0);
            tmGaussian2C.setValueAt("amplitude",4,0);
            tmGaussian2C.setValueAt("minimum",5,0);
            tmGaussian2C.setValueAt("IMAGE DATA",6,0);
            tmGaussian2C.setValueAt("VALUE",6,1);
            tmGaussian2C.setValueAt("width [pixels]",7,0);
            tmGaussian2C.setValueAt("height [pixels]",8,0);
            tmGaussian2C.setValueAt("ROI X [pixels]",9,0);
            tmGaussian2C.setValueAt("ROI Y [pixels]",10,0);
            tmGaussian2C.setValueAt("ROI width [pixels]",11,0);
            tmGaussian2C.setValueAt("ROI height [pixels]",12,0);
            tmGaussian2C.setValueAt("threshold",13,0);
            JTable gaussianTable2C = new JTable(tmGaussian2C);
            gaussianTable2C.setDefaultRenderer(Object.class,new CellRenderer(new int[]{0,6}));
            gaussianPanel.add(gaussianTable2C,new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.CENTER,
                    GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
            gaussianPanel.setBorder(new LineBorder(Color.BLACK));
        }
        return gaussianPanel;
    }

    private JPanel getGaussian2DPanel() {
        if (gaussian2DPanel == null) {
            gaussian2DPanel = new JPanel(new GridBagLayout());
            tmGaussian2D = new TableModel(15,2);
            tmGaussian2D.setValueAt("DATA",0,0);
            tmGaussian2D.setValueAt("VALUE",0,1);
            tmGaussian2D.setValueAt("X Mean [pixels]",1,0);
            tmGaussian2D.setValueAt("Y Mean [pixels]",2,0);
            tmGaussian2D.setValueAt("STD X [pixels]",3,0);
            tmGaussian2D.setValueAt("STD Y [pixels]",4,0);
            tmGaussian2D.setValueAt("rotation [deg]",5,0);
            tmGaussian2D.setValueAt("amplitude",6,0);
            tmGaussian2D.setValueAt("constant",7,0);
            tmGaussian2D.setValueAt("Image width [pixels]",8,0);
            tmGaussian2D.setValueAt("Image height [pixels]",9,0);
            tmGaussian2D.setValueAt("ROI X [pixels]",10,0);
            tmGaussian2D.setValueAt("ROI Y [pixels]",11,0);
            tmGaussian2D.setValueAt("ROI width [pixels]",12,0);
            tmGaussian2D.setValueAt("ROI height [pixels]",13,0);
            tmGaussian2D.setValueAt("threshold",14,0);
            JTable gaussianTable2D = new JTable(tmGaussian2D);
            gaussianTable2D.setDefaultRenderer(Object.class,new CellRenderer(new int[]{0}));
            gaussian2DPanel.add(gaussianTable2D,new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.CENTER,
                    GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
            gaussian2DPanel.setBorder(new LineBorder(Color.BLACK));
        }
        return gaussian2DPanel;
    }

    private void restructure(AnalysisImage image) {
        type = image.getName();
        getGaussianPanel().setVisible(image instanceof AImage);
        getGaussian2DPanel().setVisible(image instanceof AbstractGaussianImage);
        getInterferometerPanel().setVisible(image instanceof IFMImage);
        getDoubleInterferometerPanel().setVisible(image instanceof DIFMImage);
    }

    /**
     * Sets table data.
     */
    public void setData(AnalysisImage aImage) {
        if (aImage == null) {
            clearData();
            return;
        }
        if (this.type == null || !this.type.equals(aImage.getName())) {
            restructure(aImage);
        }
        if (aImage instanceof AImage) {
            setGaussian((AImage)aImage);
        } else if (aImage instanceof IFMImage) {
            setInterferometer((IFMImage)aImage);
        } else if (aImage instanceof DIFMImage) {
            setDoubleInterferometer((DIFMImage)aImage);
        } else if (aImage instanceof AbstractGaussianImage) {
            setGaussian2D((AbstractGaussianImage)aImage);
        }
    }

    private void setDoubleInterferometer(DIFMImage image) {
        if (orientation == ImageOrientation.NoChange) {
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantX()),1,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeX()),2,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincX()),3,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPsincX()),4,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosX()),5,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPcosX()),6,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisX()),7,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantY()),1,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeY()),2,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincY()),3,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPsincY()),4,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosY()),5,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPcosY()),6,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisY()),7,2);
            tmDoubleInterferometer2C.setValueAt(image.getImageW(),1,1);
            tmDoubleInterferometer2C.setValueAt(image.getImageH(),2,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiX(),3,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiY(),4,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiW(),5,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiH(),6,1);
        } else if (orientation == ImageOrientation.RotationBy90CW) {
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantY()),1,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeY()),2,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincY()),3,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiH() - image.getPsincY()),4,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosY()),5,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiH() - image.getPcosY()),6,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisY()),7,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantX()),1,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeX()),2,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincX()),3,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPsincX()),4,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosX()),5,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPcosX()),6,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisX()),7,2);
            tmDoubleInterferometer2C.setValueAt(image.getImageH(),1,1);
            tmDoubleInterferometer2C.setValueAt(image.getImageW(),2,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiY(),3,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiX(),4,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiH(),5,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiW(),6,1);
        } else if (orientation == ImageOrientation.RotationBy180) {
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantX()),1,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeX()),2,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincX()),3,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiW() - image.getPsincX()),4,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosX()),5,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiW() - image.getPcosX()),6,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisX()),7,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantY()),1,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeY()),2,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincY()),3,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiH() - image.getPsincY()),4,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosY()),5,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiH() - image.getPcosY()),6,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisY()),7,2);
            tmDoubleInterferometer2C.setValueAt(image.getImageW(),1,1);
            tmDoubleInterferometer2C.setValueAt(image.getImageH(),2,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiX(),3,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiY(),4,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiW(),5,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiH(),6,1);
        } else if (orientation == ImageOrientation.RotationBy90CCW) {
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantY()),1,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeY()),2,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincY()),3,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPsincY()),4,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosY()),5,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getPcosY()),6,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisY()),7,1);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getConstantX()),1,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAmplitudeX()),2,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAsincX()),3,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiW() - image.getPsincX()),4,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getAcosX()),5,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getRoiW() - image.getPcosX()),6,2);
            tmDoubleInterferometer3C.setValueAt(hdf.format(image.getVisX()),7,2);
            tmDoubleInterferometer2C.setValueAt(image.getImageH(),1,1);
            tmDoubleInterferometer2C.setValueAt(image.getImageW(),2,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiY(),3,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiX(),4,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiH(),5,1);
            tmDoubleInterferometer2C.setValueAt(image.getRoiW(),6,1);
        }

        tmDoubleInterferometer2C.setValueAt(df.format(image.getThreshold()),7,1);
    }

    private void setInterferometer(IFMImage image) {
        if (orientation == ImageOrientation.NoChange) {
            tmInterferometer2C.setValueAt(hdf.format(image.getPsincY()),4,1);
            tmInterferometer2C.setValueAt(hdf.format(image.getPcosY()),6,1);
            tmInterferometer2C.setValueAt(image.getImageW(),9,1);
            tmInterferometer2C.setValueAt(image.getImageH(),10,1);
            tmInterferometer2C.setValueAt(image.getRoiX(),11,1);
            tmInterferometer2C.setValueAt(image.getRoiY(),12,1);
            tmInterferometer2C.setValueAt(image.getRoiW(),13,1);
            tmInterferometer2C.setValueAt(image.getRoiH(),14,1);
        } else if (orientation == ImageOrientation.RotationBy90CW) {
            tmInterferometer2C.setValueAt(
                    hdf.format((image.isVertical() ? image.getRoiW() : image.getRoiH()) - image.getPsincY()),4,1);
            tmInterferometer2C.setValueAt(
                    hdf.format((image.isVertical() ? image.getRoiW() : image.getRoiH()) - image.getPcosY()),6,1);
            tmInterferometer2C.setValueAt(image.getImageH(),9,1);
            tmInterferometer2C.setValueAt(image.getImageW(),10,1);
            tmInterferometer2C.setValueAt(image.getRoiY(),11,1);
            tmInterferometer2C.setValueAt(image.getRoiX(),12,1);
            tmInterferometer2C.setValueAt(image.getRoiH(),13,1);
            tmInterferometer2C.setValueAt(image.getRoiW(),14,1);
        } else if (orientation == ImageOrientation.RotationBy180) {
            tmInterferometer2C.setValueAt(
                    hdf.format((image.isVertical() ? image.getRoiH() : image.getRoiW()) - image.getPsincY()),4,1);
            tmInterferometer2C.setValueAt(
                    hdf.format((image.isVertical() ? image.getRoiH() : image.getRoiW()) - image.getPcosY()),6,1);
            tmInterferometer2C.setValueAt(image.getImageW(),9,1);
            tmInterferometer2C.setValueAt(image.getImageH(),10,1);
            tmInterferometer2C.setValueAt(image.getRoiX(),11,1);
            tmInterferometer2C.setValueAt(image.getRoiY(),12,1);
            tmInterferometer2C.setValueAt(image.getRoiW(),13,1);
            tmInterferometer2C.setValueAt(image.getRoiH(),14,1);
        } else if (orientation == ImageOrientation.RotationBy90CCW) {
            tmInterferometer2C.setValueAt(
                    hdf.format((image.isVertical() ? image.getRoiW() : image.getRoiH()) - image.getPsincY()),4,1);
            tmInterferometer2C.setValueAt(
                    hdf.format((image.isVertical() ? image.getRoiW() : image.getRoiH()) - image.getPcosY()),6,1);
            tmInterferometer2C.setValueAt(image.getImageH(),1,1);
            tmInterferometer2C.setValueAt(image.getImageW(),2,1);
            tmInterferometer2C.setValueAt(image.getRoiY(),11,1);
            tmInterferometer2C.setValueAt(image.getRoiX(),12,1);
            tmInterferometer2C.setValueAt(image.getRoiH(),13,1);
            tmInterferometer2C.setValueAt(image.getRoiW(),14,1);
        }
        tmInterferometer2C.setValueAt(hdf.format(image.getConstantY()),1,1);
        tmInterferometer2C.setValueAt(hdf.format(image.getAmplitudeY()),2,1);
        tmInterferometer2C.setValueAt(hdf.format(image.getAsincY()),3,1);
        tmInterferometer2C.setValueAt(hdf.format(image.getAcosY()),5,1);
        tmInterferometer2C.setValueAt(hdf.format(image.getVisY()),7,1);

        tmInterferometer2C.setValueAt(df.format(image.getThreshold()),15,1);
    }

    private void setGaussian(AImage image) {
        if (orientation == ImageOrientation.NoChange) {
            tmGaussian3C.setValueAt(df.format(image.getMeanX()) + " / " + df.format(image.getMeanXFit()),1,1);
            tmGaussian3C.setValueAt(df.format(image.getMeanY()) + " / " + df.format(image.getMeanYFit()),1,2);
            tmGaussian3C.setValueAt(df.format(image.getStdX()) + " / " + df.format(image.getStdXFit()),2,1);
            tmGaussian3C.setValueAt(df.format(image.getStdY()) + " / " + df.format(image.getStdYFit()),2,2);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeX()) + " / " + df.format(image.getAmplitudeXFit()),3,1);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeY()) + " / " + df.format(image.getAmplitudeYFit()),3,2);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstX()) + " / " + df.format(image.getConstXFit()),4,1);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstY()) + " / " + df.format(image.getConstYFit()),4,2);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeXFit()),5,1);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeYFit()),5,2);
            tmGaussian2C.setValueAt(image.getImageW(),7,1);
            tmGaussian2C.setValueAt(image.getImageH(),8,1);
            tmGaussian2C.setValueAt(image.getRoiX(),9,1);
            tmGaussian2C.setValueAt(image.getRoiY(),10,1);
            tmGaussian2C.setValueAt(image.getRoiW(),11,1);
            tmGaussian2C.setValueAt(image.getRoiH(),12,1);
        } else if (orientation == ImageOrientation.RotationBy90CW) {
            tmGaussian3C.setValueAt(df.format(image.getImageH() - image.getMeanY()) + " / "
                    + df.format(image.getImageH() - image.getMeanYFit()),1,1);
            tmGaussian3C.setValueAt(df.format(image.getMeanX()) + " / " + df.format(image.getMeanXFit()),1,2);
            tmGaussian3C.setValueAt(df.format(image.getStdY()) + " / " + df.format(image.getStdYFit()),2,1);
            tmGaussian3C.setValueAt(df.format(image.getStdX()) + " / " + df.format(image.getStdXFit()),2,2);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeY()) + " / " + df.format(image.getAmplitudeYFit()),3,1);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeX()) + " / " + df.format(image.getAmplitudeXFit()),3,2);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstY()) + " / " + df.format(image.getConstYFit()),4,1);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstX()) + " / " + df.format(image.getConstXFit()),4,2);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeYFit()),5,1);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeXFit()),5,2);
            tmGaussian2C.setValueAt(image.getImageH(),7,1);
            tmGaussian2C.setValueAt(image.getImageW(),8,1);
            tmGaussian2C.setValueAt(image.getRoiY(),9,1);
            tmGaussian2C.setValueAt(image.getRoiX(),10,1);
            tmGaussian2C.setValueAt(image.getRoiH(),11,1);
            tmGaussian2C.setValueAt(image.getRoiW(),12,1);
        } else if (orientation == ImageOrientation.RotationBy180) {
            tmGaussian3C.setValueAt(df.format(image.getImageW() - image.getMeanX()) + " / "
                    + df.format(image.getImageW() - image.getMeanXFit()),1,1);
            tmGaussian3C.setValueAt(df.format(image.getImageH() - image.getMeanY()) + " / "
                    + df.format(image.getImageH() - image.getMeanYFit()),1,2);
            tmGaussian3C.setValueAt(df.format(image.getStdX()) + " / " + df.format(image.getStdXFit()),2,1);
            tmGaussian3C.setValueAt(df.format(image.getStdY()) + " / " + df.format(image.getStdYFit()),2,2);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeX()) + " / " + df.format(image.getAmplitudeXFit()),3,1);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeY()) + " / " + df.format(image.getAmplitudeYFit()),3,2);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstX()) + " / " + df.format(image.getConstXFit()),4,1);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstY()) + " / " + df.format(image.getConstYFit()),4,2);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeXFit()),5,1);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeYFit()),5,2);
            tmGaussian2C.setValueAt(image.getImageW(),7,1);
            tmGaussian2C.setValueAt(image.getImageH(),8,1);
            tmGaussian2C.setValueAt(image.getRoiX(),9,1);
            tmGaussian2C.setValueAt(image.getRoiY(),10,1);
            tmGaussian2C.setValueAt(image.getRoiW(),11,1);
            tmGaussian2C.setValueAt(image.getRoiH(),12,1);
        } else if (orientation == ImageOrientation.RotationBy90CCW) {
            tmGaussian3C.setValueAt(df.format(image.getMeanY()) + " / " + df.format(image.getMeanYFit()),1,1);
            tmGaussian3C.setValueAt(df.format(image.getImageW() - image.getMeanX()) + " / "
                    + df.format(image.getImageW() - image.getMeanXFit()),1,2);
            tmGaussian3C.setValueAt(df.format(image.getStdY()) + " / " + df.format(image.getStdYFit()),2,1);
            tmGaussian3C.setValueAt(df.format(image.getStdX()) + " / " + df.format(image.getStdXFit()),2,2);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeY()) + " / " + df.format(image.getAmplitudeYFit()),3,1);
            tmGaussian3C.setValueAt(
                    df.format(image.getSideViewAmplitudeX()) + " / " + df.format(image.getAmplitudeXFit()),3,2);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstY()) + " / " + df.format(image.getConstYFit()),4,1);
            tmGaussian3C.setValueAt(df.format(image.getSideViewConstX()) + " / " + df.format(image.getConstXFit()),4,2);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeYFit()),5,1);
            tmGaussian3C.setValueAt(df.format(0) + " / " + df.format(image.getSlopeXFit()),5,2);
            tmGaussian2C.setValueAt(image.getImageH(),7,1);
            tmGaussian2C.setValueAt(image.getImageW(),8,1);
            tmGaussian2C.setValueAt(image.getRoiY(),9,1);
            tmGaussian2C.setValueAt(image.getRoiX(),10,1);
            tmGaussian2C.setValueAt(image.getRoiH(),11,1);
            tmGaussian2C.setValueAt(image.getRoiW(),12,1);
        }
        tmGaussian2C.setValueAt(df.format(image.getStdA()),1,1);
        tmGaussian2C.setValueAt(df.format(image.getStdB()),2,1);
        tmGaussian2C.setValueAt(df.format(image.getAngle2D()),3,1);
        tmGaussian2C.setValueAt(df.format(image.getAmplitude2D()),4,1);
        tmGaussian2C.setValueAt(df.format(image.getConst2D()),5,1);

        tmGaussian2C.setValueAt(df.format(image.getThreshold()),13,1);
    }

    private void setGaussian2D(AbstractGaussianImage image) {
        tmGaussian2D.setValueAt(df.format(image.getMeanX()),1,1);
        tmGaussian2D.setValueAt(df.format(image.getMeanY()),2,1);
        tmGaussian2D.setValueAt(df.format(image.getStdX()),3,1);
        tmGaussian2D.setValueAt(df.format(image.getStdY()),4,1);
        tmGaussian2D.setValueAt(df.format(image.getAngle2D()),5,1);
        tmGaussian2D.setValueAt(df.format(image.getAmplitude2D()),6,1);
        tmGaussian2D.setValueAt(df.format(image.getConst2D()),7,1);
        tmGaussian2D.setValueAt(image.getImageW(),8,1);
        tmGaussian2D.setValueAt(image.getImageH(),9,1);
        tmGaussian2D.setValueAt(image.getRoiX(),10,1);
        tmGaussian2D.setValueAt(image.getRoiY(),11,1);
        tmGaussian2D.setValueAt(image.getRoiW(),12,1);
        tmGaussian2D.setValueAt(image.getRoiH(),13,1);
        tmGaussian2D.setValueAt(df.format(image.getThreshold()),14,1);
    }

    /**
     * Clears table contents.
     */
    public void clearData() {
        IntStream.range(1,6).forEach(i -> tmGaussian3C.setValueAt(null,i,1));
        IntStream.range(1,6).forEach(i -> tmGaussian3C.setValueAt(null,i,2));
        IntStream.range(1,14).forEach(i -> tmGaussian2C.setValueAt(null,i,1));
        IntStream.range(1,16).forEach(i -> tmInterferometer2C.setValueAt(null,i,1));
        IntStream.range(1,8).forEach(i -> tmDoubleInterferometer3C.setValueAt(null,i,1));
        IntStream.range(1,8).forEach(i -> tmDoubleInterferometer3C.setValueAt(null,i,2));
        IntStream.range(1,8).forEach(i -> tmDoubleInterferometer2C.setValueAt(null,i,1));
        IntStream.range(1,14).forEach(i -> tmGaussian2D.setValueAt(null,i,1));
    }
}
