package de.desy.acop.video.analysis.interferometer;

import de.desy.acop.video.analysis.AnalysisData;

/**
 * 
 * <code>InterferometerAnalysisData</code> is a container for the interferometer analysis data,
 * which is required by the graphics components to properly draw the interferometer analysis
 * fit.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class InterferometerAnalysisData extends AnalysisData {

	public final double constant;
	public final double amplitude;
	public final double asinc;
	public final double psinc;
	public final double acos;
	public final double pcos;
	public final double vis;
	public final boolean isInterferometerVertical;
	public final boolean isVertical;
	public final boolean useBesselFunction;
	
	/**
	 * 
	 * @param profile the profile points
	 * @param roiStart region of interest starting index
	 * @param roiLength region of interest length
	 * @param interferometerVertical true if the interferometer is oriented vertically
	 * @param useBesselFunction true to use bessel function or false to use trigonometric 
	 */
	public InterferometerAnalysisData(double[] profile, int roiStart, int roiLength, boolean interferometerVertical,
	        boolean useBesselFunction) {
		this(profile, roiStart, roiLength,false, Double.NaN,Double.NaN,Double.NaN,Double.NaN,
				Double.NaN,Double.NaN,Double.NaN,false,interferometerVertical,useBesselFunction);
	}
	
	/**
	 * 
	 * @param profile the profile points
	 * @param roiStart region of interest starting index
	 * @param roiLength region of interest length
	 * @param drawFit true if the fit is drawn
	 * @param constant the interferometer fit parameter
	 * @param amplitude the interferometer fit parameter
	 * @param asinc the interferometer fit parameter
	 * @param psinc the interferometer fit parameter
	 * @param acos the interferometer fit parameter
	 * @param pcos the interferometer fit parameter
	 * @param vis the interferometer fit parameter
	 * @param isVertical true if this data is for vertical direction 
	 * @param interferometerVertical true if the interferometer is oriented vertically 
	 * @param useBesselFunction true to use bessel function or false to use trigonometric 
	 */
	public InterferometerAnalysisData(double[] profile, int roiStart, int roiLength, boolean drawFit, 
			double constant, double amplitude,
			double asinc, double psinc, double acos, double pcos, double vis, boolean isVertical,
			boolean interferometerVertical, boolean useBesselFunction) {
		super(profile,roiStart,roiLength,drawFit);
		this.constant = constant;
		this.amplitude = amplitude;
		this.asinc = asinc;
		this.psinc = psinc;
		this.acos = acos;
		this.pcos = pcos;
		this.vis = vis;
		this.isVertical = isVertical;
		this.isInterferometerVertical = interferometerVertical;
		this.useBesselFunction = useBesselFunction;
	}

	
}
