package de.desy.acop.video.analysis.interferometer;

import de.desy.tine.types.IMAGE;

/**
 * <code>SingleBesselIFMImage</code> is the single side analysis image structure which has been fitted with bessel 
 * functions.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class SingleBesselIFMImage extends IFMImage {

    private static final String NAME = "SBIFISr4";

    /**
     * Construct new image. 
     */
    public SingleBesselIFMImage() {
        super(NAME);
    }
    
    /**
     * Copy the image.
     * 
     * @param image the source
     */
    public SingleBesselIFMImage(SingleBesselIFMImage image) {
        this(image.getImage(),image.getRoiX(),image.getRoiY(),image.getRoiW(),image.getRoiH(),image.getRoi2X(),
                image.getRoi2Y(),image.getRoi2W(),image.getRoi2H(),image.getThreshold(),image.getSideViewX(),
                image.getSideViewY(),image.getConstantY(),image.getAmplitudeY(),image.getAsincY(),image.getPsincY(),
                image.getAcosY(),image.getPcosY(),image.getVisY(),
                image.getThresholdPoints(),image.isCalculateThreshold(),image.isPerformFit(),
                image.isPerformSmoothing(),image.isVertical());
    }

    /**
     * @see DIFMImage
     */
    public SingleBesselIFMImage(IMAGE image, int roiX, int roiY, int roiW, int roiH, int roi2X, int roi2Y, int roi2W,
            int roi2H, double threshold, double[] sideViewX, double[] sideViewY, double constantY, double amplitudeY,
            double asincY, double psincY, double visY, double acosY, double pcosY, int thresholdPoints,
            boolean calculateThreshold, boolean performFit, boolean performSmoothing, boolean vertical) {
        super(NAME,image,roiX,roiY,roiW,roiH,roi2X,roi2Y,roi2W,roi2H,threshold,sideViewX,sideViewY,constantY,amplitudeY,
                asincY,psincY,visY,acosY,pcosY,thresholdPoints,calculateThreshold,performFit,performSmoothing,vertical);
    }
}
