package de.desy.acop.video.displayer;

import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.awt.image.IndexColorModel;

import javax.swing.JComponent;

/**
 * Color Look-Up Panel
 * 
 * @author David Melkumyan, DESY Zeuthen
 * @note mdavid: painting can be improved
 */
public class ColorLookupPanel extends JComponent {

	private static final long serialVersionUID = 31031976L;

	/**
	 * @see ColorLookupTable
	 */
	private ImageCLUT clut;

	/**
	 * Constructs a color map swing component
	 * 
	 * @param clut
	 */
	public ColorLookupPanel(ImageCLUT clut) {
		this.clut = clut;
	}

	/**
	 * @return color lookup table
	 */
	public ImageCLUT getCLUT() {
		return clut;
	}

	/**
	 * @param clut
	 *            - color lookup table
	 */
	public void setCLUT(ImageCLUT clut) {
		this.clut = clut;
	}

	@Override
	public void paintComponent(Graphics g) {

		FontMetrics fm = g.getFontMetrics();
		Insets insets = new Insets(fm.getHeight() + 2, 8, 4, 8);

		int w = getWidth() - insets.left - insets.right;
		int h = getHeight() - insets.top - insets.bottom;

		Rectangle rect = new Rectangle(insets.left - 1, insets.top - 1, w + 1, h + 1);
		g.drawRect(rect.x, rect.y, rect.width, rect.height);

		if (clut != null) {
			ColorMap cm = clut.getColorMap();
			if (!cm.equals(ColorMap.NONE)) {
				int pixelSize = clut.getPixelSize();
				IndexColorModel icm = cm.getLUT(pixelSize);
				BufferedImage image = new BufferedImage(icm.getMapSize(), 1, BufferedImage.TYPE_INT_RGB);
				icm.getRGBs(((DataBufferInt) image.getRaster().getDataBuffer()).getData());
				g.drawImage(image, insets.left, insets.top, w, h, this);

				int x0, x1 = insets.left - 1;
				float offset = w / (float) pixelSize;
				for (int index = 0; index < pixelSize; index++) {
					x0 = x1;
					x1 += Math.round(offset);
					g.drawLine(x0, insets.top - 2, x0, insets.top + h);
					String s = String.valueOf(index);
					g.drawString(s, x0 + (x1 - x0 - fm.stringWidth(s)) / 2, 12);
				}
				return;
			}
		}
		String s = "no color lookup table";
		g.drawString(s, (int) (rect.getCenterX() - fm.stringWidth(s) / 2), 12);
	}

}
