package de.desy.acop.video.displayer;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * To resize an image in the ImageDisplayer pane
 * 
 * @author mdavid
 * 
 */
public enum ImageZoom {
	/**
	 * The image automatically resizes to fit the dimensions of the
	 * ImageDisplayer
	 */
	AUTO("Auto"),

	/**
	 * Zoom at 50%
	 */
	HALF("50%"),

	/**
	 * Zoom at 100%
	 */
	NORMAL("100%"),

	/**
	 * Zoom at 200%
	 */
	DOUBLE("200%");

	/**
	 * Image zoom description
	 */
	private String descr;

	private static Map<String, ImageZoom> map = new LinkedHashMap<>(values().length);
	static {
		for (ImageZoom e : values())
			if (map.put(e.descr, e) != null)
				throw new InternalError("duplicate description: " + e.descr);
	}

	public static String[] descriptions() {
		return map.keySet().toArray(new String[map.size()]);
	}

	/**
	 * Sets a description of the zoom mode
	 * 
	 * @param description
	 */
	private ImageZoom(String description) {
		descr = description;
	}

	/**
	 * @return Description of the zoom mode
	 */
	public String getDescription() {
		return descr;
	}

	/**
	 * @param desc
	 *            - description
	 * @return returns ImageZoom value by description
	 * @throws IllegalArgumentException
	 */
	public static ImageZoom getValue(String desc) throws IllegalArgumentException {
		ImageZoom e = map.get(desc);
		if (e == null)
			throw new IllegalArgumentException("non-existing image zoom desc: " + desc);
		return e;
	}
}
