package de.desy.acop.video.displayer;

/**
 * 
 * <code>LinkData</code> provides the timing information about the current image update.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class LinkData {
    
    private final String linkName;
    private final long timestamp;
    private final int systemstamp;
    private final int userstamp;
    
    /**
     * Constructs a new link data.
     * 
     * @param linkName the name of the channel that the image belongs to
     * @param timestamp the timestamp in utc
     * @param systemstamp the system stamp
     * @param userstamp the user stamp
     */
    public LinkData(String linkName, long timestamp, int systemstamp, int userstamp) {
        this.linkName = linkName;
        this.timestamp = timestamp;
        this.systemstamp = systemstamp;
        this.userstamp = userstamp;
    }
    
    
    /**
     * @return the name of the channel
     */
    public String getLinkName() {
        return linkName;
    }
    
    /**
     * @return the timestamp in UTC
     */
    public long getTimestamp() {
        return timestamp;
    }
    
    
    /**
     * @return the system stamp
     */
    public int getSystemstamp() {
        return systemstamp;
    }
    
    /**
     * @return the user stamp
     */
    public int getUserstamp() {
        return userstamp;
    }
}
