/*
 * Created on Nov 8, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.alarms;

/**
 * An alarm.
 * An alarm is a flag which indicates an error or undesired
 * state in a device. It is valid for one update period.
 * Every device has a set of pre-defined alarms. It can set
 * each alarm once.
 * An alarm has a name and a description, a key (identifier)
 * and a severity. The key must be unique for a device type.
 * @author jwlg
 */
public class Alarm implements Comparable<Alarm>{
    
    private final int key;
    private final Severity severity;
    private final String name;
    private final String description;
    
    /**
     * Constructor.
     * @param key An identifier which is unique within a device type. 
     * @param severity The severity.
     * @param name The name of the alarm. 
     * @param description The description.
     */
    public Alarm(int key, Severity severity, String name, String description) {
        this.key = key;
        this.severity = severity;
        this.name = name;
        this.description = description;
    }
    
    /**
     * Returns the key.
     */
    public int getKey() {
        return key;
    }

    /**
     * Returns the description.
     */
    public String getDescription() {
        return description;
    }
    
    /**
     * Returns the severity.
     */
    public Severity getSeverity() {
        return severity;
    }

    /**
     * Returns the name.
     */
    public String getName() {
        return name;
    }

    public int compareTo(Alarm o) {
        int result = 0;
        if (this.key < o.key) {
            result =  -1;
        } else if (this.key > o.key) {
            result =  1;
        } else {
            result = 0;
        }
        return result;
    }

    @Override
    public int hashCode() {

        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((description == null) ? 0 : description.hashCode());
        result = PRIME * result + key;
        result = PRIME * result + ((name == null) ? 0 : name.hashCode());
        result = PRIME * result + severity.getValue();
        return result;
    }

    @Override
    public boolean equals(Object obj) {

        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final Alarm other = (Alarm) obj;
        if (description == null) {
            if (other.description != null)
                return false;
        } else if (!description.equals(other.description))
            return false;
        if (key != other.key)
            return false;
        if (severity.getValue() != other.severity.getValue())
            return false;
        if (name == null) {
            if (other.name != null)
                return false;
        } else if (!name.equals(other.name))
            return false;
        return true;
    }

    
}
