/*
 * Created on Nov 8, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.alarms;

import de.desy.tine.server.alarms.TAlarmDefinition;

/**
 * An Alarm Description Generator.
 * This class is used by EquipmentModule to generate
 * TAlarmDefinitions for TINE.
 * @author jwlg
 */
public class TineAlarmFactory {
    
    private int tineAlarmSystemID = 0;
    private String moduleDescription = "";

    /**
     * The Constructor.
     * @param tineAlarmSystemID The number which the Central Alarm Server uses for this module.
     * @param moduleDescription The module description (called Device Text in the Alarm Definition)
     */
    public TineAlarmFactory(int tineAlarmSystemID, String moduleDescription) {
        this.tineAlarmSystemID = tineAlarmSystemID;
        this.moduleDescription = moduleDescription;
    }

    /**
     * Returns a TINE alarm definition.
     * @param alarm An alarm.
     */
    public TAlarmDefinition getAlarmDefinition(Alarm alarm) {
        TAlarmDefinition result;
        result = new TAlarmDefinition(
            alarm.getName(),
            alarm.getDescription(),
            this.moduleDescription, 
            getTineCode(alarm),
            alarm.getSeverity().getTineLevel(),
            this.tineAlarmSystemID
        );
        return result;
    }
    
    /**
     * Adds 512 to the alarm key.
     * TINE allows values > 512 as alarm identifiers.
     * The device server numbers alarm identifiers
     * in a sequence starting at 1.
     * This method transforms an alarm identifier
     * into a valid TINE alarm identifier.
     * @param alarm The alarm.
     */
    public static int getTineCode(Alarm alarm) {
        return 512 + alarm.getKey();
    }

}
