/*
 * Created on Jan 16, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import java.util.LinkedList;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * The configuration section in the device server specification.
 * The configuration section contains information wich is needed
 * to set up and adjust the generated device server at runtime.
 * @author jwlg
 */
@XmlRootElement
public class CfgConfiguration {
    
    private CfgServer server = new CfgServer();
    private CfgLogging logging = new CfgLogging();
    private LinkedList<CfgDeviceGroup> deviceGroups = new LinkedList<CfgDeviceGroup>();
    private LinkedList<CfgDeviceGroup> controllerGroups = new LinkedList<CfgDeviceGroup>();
    
    /**
     * Returns the list of device groups.
     * The list of device groups specifies which device groups
     * of which device types the server knows.
     * In the device server specification, this section 
     * is named "modules" for historical reasons.
     * @see CfgDeviceGroup
     */
    @XmlElementWrapper(name="modules")
    @XmlElement(name="module")
    public LinkedList<CfgDeviceGroup> getDeviceGroups() {
        return deviceGroups;
    }

    /**
     * Sets the device groups.
     * This method is only needed for JAXB. It is not intended to be used
     * directly in programs.
     * @param modules The new list of device groups.
     */
    public void setDeviceGroups(LinkedList<CfgDeviceGroup> modules) {
        this.deviceGroups = modules;
    }
    
    /**
     * Returns the list of controller groups.
     * The list of controller groups specifies which controller groups
     * of which controller types the server knows.
     * In the device server specification, this section 
     * is named "modules" for historical reasons.
     * @see CfgDeviceGroup
     */
    @XmlElementWrapper(name="controllerGroups")
    @XmlElement(name="controllerGroup")
    public LinkedList<CfgDeviceGroup> getControllerGroups() {
        return controllerGroups;
    }

    /**
     * Sets the controller groups.
     * This method is only needed for JAXB. It is not intended to be used
     * directly in programs.
     * @param modules The new list of controller groups.
     */
    public void setControllerGroups(LinkedList<CfgDeviceGroup> modules) {
        this.controllerGroups = modules;
    }

    
    /**
     * Returns the server section.
     * @see CfgServer
     */
    @XmlElement
    public CfgServer getServer() {
        return server;
    }
    
    /**
     * Sets the server section.
     * This method is only needed for JAXB. It is not intended to be used
     * directly in programs.
     * @param server The new server section.
     */
    public void setServer(CfgServer server) {
        this.server = server;
    }
    
    /**
     * Returns the logging section.
     */
    @XmlElement
    public CfgLogging getLogging() {
        return logging;
    }

    /**
     * Sets the logging section.
     * This method is only needed for JAXB. It is not intended to be used
     * directly in programs.
     * @param logging The new logging section.
     */
    public void setLogging(CfgLogging logging) {
        this.logging = logging;
    }
    
}
