/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import java.util.LinkedList;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

/**
 * The definition section of a device server specification.
 * The definition section contains specifications of data 
 * types and device types.
 * @see CfgDeviceType
 * @see CfgDataTypes
 * @author jwlg
 */
@XmlRootElement
@XmlType(propOrder={"dataTypes","deviceTypes", "controllerTypes"}) 
public class CfgDefinition {
    
    private CfgDataTypes dataTypes = new CfgDataTypes();
    private LinkedList<CfgDeviceType> deviceTypes = new LinkedList<CfgDeviceType>();
    private LinkedList<CfgDeviceType> controllerTypes = new LinkedList<CfgDeviceType>();
    
    /**
     * Returns the list of data types.
     */
    @XmlElement
    public CfgDataTypes getDataTypes() {
        return dataTypes;
    }

    /**
     * Sets the list of data types.
     * This method is required by JAXB.
     * @param dataTypes The new list of data types.
     */
    public void setDataTypes(CfgDataTypes dataTypes) {
        this.dataTypes = dataTypes;
    }

    /**
     * Returns the list of device types.
     */
    @XmlElementWrapper(name="deviceTypes")
    @XmlElement(name="deviceType")
    public LinkedList<CfgDeviceType> getDeviceTypes() {
        return deviceTypes;
    }

    /**
     * Sets the list of device types.
     * @param deviceTypes The new list of device types.
     */
    public void setDeviceTypes(LinkedList<CfgDeviceType> deviceTypes) {
        this.deviceTypes = deviceTypes;
    }

    
    /**
     * Returns the list of controller types. 
     */
    @XmlElementWrapper(name="controllerTypes")
    @XmlElement(name="controllerType")
    public LinkedList<CfgDeviceType> getControllerTypes() {
        return controllerTypes;
    }

    /**
     * Sets the list of controller types.
     * Not intended for general used. Neded by JAXB.
     * @param controllerTypes List of controller types.
     */
    public void setControllerTypes(LinkedList<CfgDeviceType> controllerTypes) {
        this.controllerTypes = controllerTypes;
    }
    
}
