/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

/**
 * The device server specification.
 * This is the root element of a device server specification.
 * A device server specification contains a definition section,
 * a configuration section, a section with project settings and
 * a section with document information.
 * @author jwlg
 */
@XmlRootElement
@XmlType(propOrder= {"document", "project", "definition", "configuration"})
public class CfgDeviceServer {
    
    private CfgDocument document = new CfgDocument();
    private CfgDefinition definition = new CfgDefinition();
    private CfgConfiguration configuration = new CfgConfiguration();
    private CfgProject project = new CfgProject();
    
    /**
     * Default Constructor.
     */
    public CfgDeviceServer() {
    }

    /**
     * Constructor which specifies the name of the
     * main class.
     * @param mainClassName The name of the main class.
     */
    public CfgDeviceServer(String mainClassName) {
        project.setName(mainClassName);
    }
    
    /** 
     * Returns the document information
     */
    @XmlElement
    public CfgDocument getDocument() {
        return document;
    }

    /**
     * Sets the document information.
     * Only used by JAXB.
     * @param document The new document information.
     */
    public void setDocument(CfgDocument document) {
        this.document = document;
    }
    
    /**
     * Returns the project information.
     */
    @XmlElement
    public CfgProject getProject() {
        return project;
    }

    /**
     * Sets the project information.
     * Only used by JAXB.
     * @param project The project information.
     */
    public void setProject(CfgProject project) {
        this.project = project;
    }

    /**
     * Returns the configuration section.
     */
    @XmlElement
    public CfgConfiguration getConfiguration() {
        return configuration;
    }

    /**
     * Sets the configuration section.
     * Only used by JAXB.
     * @param configuration a configuration section.
     */
    public void setConfiguration(CfgConfiguration configuration) {
        this.configuration = configuration;
    }

    /**
     * Returns the definition section.
     */
    @XmlElement
    public CfgDefinition getDefinition() {
        return definition;
    }

    /**
     * Sets the definition section.
     * Only used by JAXB.
     * @param definition The definition section.
     */
    public void setDefinition(CfgDefinition definition) {
        this.definition = definition;
    }
    
    
    public boolean existsDeviceGroup(String groupName) {
        boolean result = false;
        for (CfgDeviceGroup group : getConfiguration().getDeviceGroups()) {
            if (group.getName().equals(groupName)) {
                result = true;
                break;
            }
        }
        return result;
    }
    
    public boolean existsControllerGroup(String groupName) {
        boolean result = false;
        for (CfgDeviceGroup group : getConfiguration().getControllerGroups()) {
            if (group.getName().equals(groupName)) {
                result = true;
                break;
            }
        }
        return result;
    }
    
    public boolean existsDeviceGroupIgnoreCase(String groupName) {
        boolean result = false;
        for (CfgDeviceGroup group : getConfiguration().getDeviceGroups()) {
            if (group.getName().equalsIgnoreCase(groupName)) {
                result = true;
                break;
            }
        }
        return result;
    }
    
    public boolean existsControllerGroupIgnoreCase(String groupName) {
        boolean result = false;
        for (CfgDeviceGroup group : getConfiguration().getControllerGroups()) {
            if (group.getName().equalsIgnoreCase(groupName)) {
                result = true;
                break;
            }
        }
        return result;
    }

    
    public boolean existsControllerType(String typeName) {
        boolean result = false;
        for (CfgDeviceType type : getDefinition().getControllerTypes()) {
            if (type.getName().equals(typeName)) {
                result = true;
                break;
            }
        }
        return result;

    }
    
    public boolean existsDeviceType(String typeName) {
        boolean result = false;
        for (CfgDeviceType type : getDefinition().getDeviceTypes()) {
            if (type.getName().equals(typeName)) {
                result = true;
                break;
            }
        }
        return result;
    }
    
    public boolean existsDeviceTypeIgnoreCase(String typeName) {
        boolean result = false;
        for (CfgDeviceType type : getDefinition().getDeviceTypes()) {
            if (type.getName().equalsIgnoreCase(typeName)) {
                result = true;
                break;
            }
        }
        return result;
    }
    
    public boolean existsControllerTypeIgnoreCase(String typeName) {
        boolean result = false;
        for (CfgDeviceType type : getDefinition().getControllerTypes()) {
            if (type.getName().equalsIgnoreCase(typeName)) {
                result = true;
                break;
            }
        }
        return result;

    }
    
    public CfgDeviceGroup findDeviceGroup(String groupName) {
        CfgDeviceGroup result = null;
        for (CfgDeviceGroup group : getConfiguration().getDeviceGroups()) {
            if (group.getName().equals(groupName)) {
                result = group;
                break;
            }
        }
        return result;
    }
    
    public CfgDeviceGroup findControllerGroup(String groupName) {
        CfgDeviceGroup result = null;
        for (CfgDeviceGroup group : getConfiguration().getControllerGroups()) {
            if (group.getName().equals(groupName)) {
                result = group;
                break;
            }
        }
        
        return result;
    }
    
    public CfgDeviceType findDeviceType(String typeName) {
        CfgDeviceType result = null;
        for (CfgDeviceType type : getDefinition().getDeviceTypes()) {
            if (type.getName().equals(typeName)) {
                result = type;
                break;
            }
        }
        return result;
    }
    
    public CfgDeviceType findControllerType(String typeName) {
        CfgDeviceType result = null;
        for (CfgDeviceType type : getDefinition().getControllerTypes()) {
            if (type.getName().equals(typeName)) {
                result = type;
                break;
            }
        }
        return result;
    }
    
    

    
}
