/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import java.util.LinkedList;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

/**
 * The device type specification.
 * 
 * A device type specifies the interface of a device. It consists
 * of properties, operations, and alarms. 
 * 
 * 
 * @author jwlg
 */
@XmlRootElement
@XmlType(propOrder= {"name", "managedDeviceType", "alarms", "properties", "operations"})
public class CfgDeviceType {
    
    /**
     * The No-DeviceType
     */
    @XmlTransient
    public static final CfgDeviceType NO_DEVICE_TYPE = new CfgDeviceType("None");
    
    private String name = "";
    private LinkedList<CfgAlarm> alarms = new LinkedList<CfgAlarm>();
    private LinkedList<CfgProperty> properties = new LinkedList<CfgProperty>();
    private LinkedList<CfgOperation> operations = new LinkedList<CfgOperation>();
    private String managedDeviceType = null;

    /**
     * The default constructor.
     */
    public CfgDeviceType() {
    }
    
    /**
     * A constructor which specifies the name.
     * @param name name of the device type.
     */
    public CfgDeviceType(String name) {
        this.name = name;
    }
    
    /**
     * Returns the list of alarms.
     */
    @XmlElementWrapper(name="alarms")
    @XmlElement(name="alarms")
    public LinkedList<CfgAlarm> getAlarms() {
        return alarms;
    }

    /**
     * Sets the list of alarms.
     * Only needed by JAXB.
     * @param alarms The new list of alarms.
     */
    public void setAlarms(LinkedList<CfgAlarm> alarms) {
        this.alarms = alarms;
    }
    
    /**
     * Returns the name of the device type.
     */
    @XmlElement
    @XmlID
    public String getName() {
        return name;
    }
    
    /**
     * Sets the name of the device type.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Returns the list of properties.
     */
    @XmlElementWrapper(name="properties")
    @XmlElement(name="property")
    public LinkedList<CfgProperty> getProperties() {
        return properties;
    }
    
    /**
     * Sets the list of properties.
     * Uesd by JAXB only.
     * @param properties The new list of properties.
     */
    public void setProperties(LinkedList<CfgProperty> properties) {
        this.properties = properties;
    }

    /**
     * Returns the list of operations.
     */
    @XmlElementWrapper(name="operations")
    @XmlElement(name="operation")
    public LinkedList<CfgOperation> getOperations() {
        return operations;
    }

    /**
     * Sets the list of operations.
     * Used by JAXB only.
     * @param operations The new list of operations.
     */
    public void setOperations(LinkedList<CfgOperation> operations) {
        this.operations = operations;
    }
    

    /**
     * Returns the managed device type.
     * 
     * A device type has a managed device type if
     * it is a controller type.
     * 
     */
    @XmlElement(name="managedDeviceType")
    public String getManagedDeviceType() {
        return managedDeviceType;
    }
    
    public void setManagedDeviceType(String managedDeviceType) {
        this.managedDeviceType = managedDeviceType;
    }


    @Override
    public int hashCode() {

        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((name == null) ? 0 : name.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {

        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final CfgDeviceType other = (CfgDeviceType) obj;
        if (name == null) {
            if (other.name != null)
                return false;
        } else if (!name.equals(other.name))
            return false;
        return true;
    }
    
    
    public CfgProperty findProperty(String propertyName) {
        CfgProperty result = null;
        for (CfgProperty property : properties) {
            if (property.getName().equals(propertyName)) {
                result = property;
                break;
            }
        }
        return result;
    } 
    
    
    public CfgOperation findOperation(String operationName) {
        CfgOperation result = null;
        for (CfgOperation operation : operations) {
            if (operation.getName().equals(operationName)) {
                result = operation;
                break;
            }
        }
        return result;
    }
    
    
    
}

