/*
 * Created on Mar 12, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Specification of local history parameters.
 * The TINE local history settings for a certain property.
 * For the meaning of the parameters, see the tine documentation.
 * @author jwlg
 * @see <a href="http://tine.desy.de">TINE documentation</a>
 * @see CfgProperty
 */
@XmlRootElement
public class CfgLocalArchive {
    
    private boolean enabled;
    private int  pollingRate = 1000;             /* polling rate in msec */
    private int  archiveRate = 10;               /* archive rate in sec */
    private int  shortTermStorageDepth = 1000;   /* for short term storage */
    private int  longTermStorageDepth = 10000;   /* for long term storage */
    private int  archiveHeartbeat = 600;         /* archive heartbeat in sec - maximum archiving interval */
    private double percentTolerance = 0.0f;       /* percent tolerance */
    private double absoluteTolerance = 0.0f;      /* absolute tolerance */
    private String remoteServer = null;          /* Remote Server Name */
    
    
    /**
     * Returns true if the local history is enabled.
     */
    @XmlElement
    public boolean isEnabled() {
        return enabled;
    }

    /**
     * Enables or disables the local history.
     * @param enabled True enables.
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    /**
     * Returns the absolute tolerance.
     */
    @XmlElement
    public double getAbsoluteTolerance() {
        return absoluteTolerance;
    }

    /**
     * Setws the absolute tolerance.
     * @param absoluteTolerance New value.
     */
    public void setAbsoluteTolerance(double absoluteTolerance) {
        this.absoluteTolerance = absoluteTolerance;
    }
    
    /**
     * Returns the archive heartbeat.
     */
    @XmlElement
    public int getArchiveHeartbeat() {
        return archiveHeartbeat;
    }
    
    /**
     * Sets the archive heartbeat.
     * @param archiveHeartbeat New value.
     */
    public void setArchiveHeartbeat(int archiveHeartbeat) {
        this.archiveHeartbeat = archiveHeartbeat;
    }
    
    /**
     * Returns the archive rate.
     */
    @XmlElement
    public int getArchiveRate() {
        return archiveRate;
    }
    
    /**
     * Sets the archive rate.
     * @param archiveRate New archive rate.
     */
    public void setArchiveRate(int archiveRate) {
        this.archiveRate = archiveRate;
    }
    
    /**
     * Returns the long term storage depth.
     */
    @XmlElement
    public int getLongTermStorageDepth() {
        return longTermStorageDepth;
    }
    
    /**
     * Sets the long term storage depth.
     * @param longTermStorageDepth New value.
     */
    public void setLongTermStorageDepth(int longTermStorageDepth) {
        this.longTermStorageDepth = longTermStorageDepth;
    }
    
    /**
     * Returns the tolerance in percent.
     */
    @XmlElement
    public double getPercentTolerance() {
        return percentTolerance;
    }
    
    /**
     * Sets the tolerance in percent.
     * @param percentTolerance New tolerance.
     */
    public void setPercentTolerance(double percentTolerance) {
        this.percentTolerance = percentTolerance;
    }
    
    /**
     * Returns the polling rate.
     */
    @XmlElement
    public int getPollingRate() {
        return pollingRate;
    }
    
    /**
     * Sets the polling rate.
     * @param pollingRate New polling rate.
     */
    public void setPollingRate(int pollingRate) {
        this.pollingRate = pollingRate;
    }
    
    /**
     * Returns the name of the remote server.
     */
    @XmlElement
    public String getRemoteServer() {
        return remoteServer;
    }
    
    
    /**
     * Sets the name of the remote server.
     * @param remoteServer
     */
    public void setRemoteServer(String remoteServer) {
        this.remoteServer = remoteServer;
    }
    
    /**
     * Returns true if a remote server is specified.
     */
    public boolean hasRemoteServer() {
        return (this.remoteServer != null);
    }
    
    /**
     * Returns the short term storage depth.
     */
    @XmlElement
    public int getShortTermStorageDepth() {
        return shortTermStorageDepth;
    }
    
    /**
     * Sets the short term storage depth.
     * @param shortTermStorageDepth
     */
    public void setShortTermStorageDepth(int shortTermStorageDepth) {
        this.shortTermStorageDepth = shortTermStorageDepth;
    }
    
    /**
     * Assigns another CfgLocalArchive.
     * Makes a deep copy.
     * @param other The CfgLocalArchive to be assigned.
     */
    public void assign(CfgLocalArchive other) {
        this.absoluteTolerance = other.absoluteTolerance;
        this.archiveHeartbeat = other.archiveHeartbeat;
        this.archiveRate = other.archiveRate;
        this.enabled = other.enabled;
        this.longTermStorageDepth = other.longTermStorageDepth;
        this.percentTolerance = other.percentTolerance;
        this.pollingRate = other.pollingRate;
        this.remoteServer = other.remoteServer;
        this.shortTermStorageDepth = other.shortTermStorageDepth;
    }
    
}
