/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

/**
 * Configuration of project properties.
 * @author jwlg
 */
@XmlRootElement
@XmlType(propOrder= {
    "name", 
    "directory", 
    "javaPackage", 
    "eclipseProject", 
    "mstApp"}) 
public class CfgProject {
    
    private String directory = "";
    private String name = "";
    
    private CfgPackage javaPackage = new CfgPackage();
    private CfgMstApp mstApp = new CfgMstApp();
    private CfgEclipseProject eclipseProject = new CfgEclipseProject();
    
    /**
     * Returns the project directory.
     */
    @XmlElement
    public String getDirectory() {
        return directory;
    }
    
    /**
     * Sets the project directory.
     * The project directory is the top directory under wich
     * files are saved and subdirectories are created.
     * @param directory The project directory.
     */
    public void setDirectory(String directory) {
        this.directory = directory;
    }
    
    /**
     * Returns the eclipse project settings.
     */
    @XmlElement
    public CfgEclipseProject getEclipseProject() {
        return eclipseProject;
    }
    
    /**
     * Sets the eclipse project section.
     * Required by JAXB.
     * @param eclipseProject
     */
    public void setEclipseProject(CfgEclipseProject eclipseProject) {
        this.eclipseProject = eclipseProject;
    }
    
    /**
     * Returns the project name.
     */
    @XmlElement(name="name")
    public String getName() {
        return name;
    }

    /**
     * Sets the project name.
     * The project name is also the name of the main class.
     * @param theName New project name.
     */
    public void setName(String theName) {
        this.name = theName;
    }
    
    /**
     * Returns the MstApp configuration section.
     */
    @XmlElement
    public CfgMstApp getMstApp() {
        return mstApp;
    }

    /**
     * Sets the MstApp configuration section.
     * Not for use. Required by JAXB.
     * @param mstApp New MstApp section.
     */
    public void setMstApp(CfgMstApp mstApp) {
        this.mstApp = mstApp;
    }
    
    /**
     * Returns the Java package.
     */
    @XmlElement
    public CfgPackage getJavaPackage() {
        return this.javaPackage;
    }
    
    /**
     * Sets the java package.
     * This is the package above the project (i.e. 
     * without the project name in it).
     * @param javaPackage New package.
     * @see #getPackageName
     */
    public void setJavaPackage(CfgPackage javaPackage) {
        this.javaPackage = javaPackage;
    }
    
    /** 
     * Returns the complete package name.
     * This is the package name of the java package plus the project name.
     */
    public String getPackageName() {
        return javaPackage.getPackageName() + "." + name.toLowerCase();
    }

}
