/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import de.desy.tine.definitions.TFormat;
import de.desy.tine.types.DBLDBL;
import de.desy.tine.types.DBLINT;
import de.desy.tine.types.FLTFLT;
import de.desy.tine.types.FLTFLTINT;
import de.desy.tine.types.FLTINT;
import de.desy.tine.types.FLTINTINT;
import de.desy.tine.types.INTFLTFLTFLT;
import de.desy.tine.types.INTFLTINT;
import de.desy.tine.types.INTINT;
import de.desy.tine.types.INTINTFLTFLT;
import de.desy.tine.types.INTINTINT;
import de.desy.tine.types.LNGINT;
import de.desy.tine.types.NAME16;
import de.desy.tine.types.NAME16DBLDBL;
import de.desy.tine.types.NAME16FI;
import de.desy.tine.types.NAME16I;
import de.desy.tine.types.NAME16II;
import de.desy.tine.types.NAME32;
import de.desy.tine.types.NAME32DBLDBL;
import de.desy.tine.types.NAME32I;
import de.desy.tine.types.NAME48;
import de.desy.tine.types.NAME48I;
import de.desy.tine.types.NAME64;
import de.desy.tine.types.NAME64DBLDBL;
import de.desy.tine.types.NAME64I;
import de.desy.tine.types.NAME8;
import de.desy.tine.types.TDS;
import de.desy.tine.types.USTRING;

/**
 * An enum which defines all available property types.
 * 
 * Each property type contains a class name, 
 * a value of a TINE constant, and the name of
 * the TINE constant.
 * 
 * @author jwlg
 */
public enum CfgPropertyType {
    
    VOID(void.class,  "CF_NULL", TFormat.CF_NULL),
    BYTE(byte.class,  "CF_BYTE", TFormat.CF_BYTE),
    SHORT(short.class,  "CF_SHORT", TFormat.CF_SHORT),
    INT(int.class, "CF_LONG", TFormat.CF_LONG),
    FLOAT(float.class, "CF_FLOAT", TFormat.CF_FLOAT),
    DOUBLE(double.class, "CF_DOUBLE", TFormat.CF_DOUBLE),
    BOOLEAN(boolean.class, "CF_BOOLEAN", TFormat.CF_BOOLEAN),
    STRING(String.class, "CF_NAME32", TFormat.CF_NAME32),
    
    DBLDBL(DBLDBL.class, "CF_DBLDBL", TFormat.CF_DBLDBL),
    DBLINT(DBLINT.class, "CF_DBLINT", TFormat.CF_DBLINT),
    
    FLTINT(FLTINT.class, "CF_FLTINT", TFormat.CF_FLTINT),
    FLTFLT(FLTFLT.class, "CF_FLTFLT", TFormat.CF_FLTFLT),
    FLTFLTINT(FLTFLTINT.class, "CF_FLTFLTINT", TFormat.CF_FLTFLTINT),
    FLTINTINT(FLTINTINT.class, "CF_FLTINTINT", TFormat.CF_FLTINTINT),
    
    INTFLTFLTFLT(INTFLTFLTFLT.class, "CF_INTFLTFLTFLT", TFormat.CF_INTFLTFLTFLT),
    INTFLTINT(INTFLTINT.class, "CF_INTFLTINT", TFormat.CF_INTFLTINT),
    INTINT(INTINT.class, "CF_INTINT", TFormat.CF_INTINT),
    INTINTFLTFLT(INTINTFLTFLT.class, "CF_INTINTFLTFLT", TFormat.CF_INTINTFLTFLT),
    INTINTINT(INTINTINT.class, "CF_INTINTINT", TFormat.CF_INTINTINT),
    LNGINT(LNGINT.class, "CF_LNGINT", TFormat.CF_LNGINT),

    NAME16(NAME16.class,"CF_NAME16", TFormat.CF_NAME16),
    NAME16DBLDBL(NAME16DBLDBL.class,"CF_NAME16DBLDBL", TFormat.CF_NAME16DBLDBL),
    NAME16FI(NAME16FI.class, "CF_NAME16FI", TFormat.CF_NAME16FI),
    NAME16I(NAME16I.class, "CF_NAME16I", TFormat.CF_NAME16I),
    NAME16II(NAME16II.class, "CF_NAME16II", TFormat.CF_NAME16II),
    
    NAME32(NAME32.class,"CF_NAME32", TFormat.CF_NAME32),
    NAME32DBLDBL(NAME32DBLDBL.class,"CF_NAME32DBLDBL", TFormat.CF_NAME32DBLDBL),
    NAME32I(NAME32I.class,"CF_NAME32I", TFormat.CF_NAME32I),

    NAME48(NAME48.class,"CF_NAME48", TFormat.CF_NAME48),
    NAME48I(NAME48I.class,"CF_NAME48I", TFormat.CF_NAME48I),

    NAME64(NAME64.class,"CF_NAME64", TFormat.CF_NAME64),
    NAME64I(NAME64I.class,"CF_NAME64I", TFormat.CF_NAME64I),
    NAME64DBLDBL(NAME64DBLDBL.class,"CF_NAME64DBLDBL", TFormat.CF_NAME64DBLDBL),

    NAME8(NAME8.class, "CF_NAME8", TFormat.CF_NAME8),
    //NAMEINT(NAMEINT.class, "CF_NAMEINT", TFormat.CF_NAMEINT),
    
    TDS(TDS.class, "CF_TDS", TFormat.CF_TDS),
    USTRING(USTRING.class, "CF_USTRING", TFormat.CF_USTRING);
        
    
    private String fullClassName = "";
    private String tineFormat = "";
    private String simpleClassName; /** derived from fullClassName */
    private String packageName; /** derived from fullClassName */
    private int tineKey = 0;
    
    /**
     * Constructor.
     * @param clazz
     * @param tineFormat
     * @param tineKey
     */
    private CfgPropertyType(Class clazz, String tineFormat, int tineKey) {
        initialize(clazz.getName(), tineFormat, tineKey);
    }
    
    /**
     * Initializer.
     * Used by constructors.
     */
    private void initialize(String aJavaClass, String tineFormatName, int tineKey) {
        this.tineFormat = tineFormatName;
        this.tineKey = tineKey;
        setFullClassName(aJavaClass);
    }
    
    /**
     * Sets the full class name.
     * The full class name consists of package name
     * and class name.
     * The simple class name and the package name
     * are derived from the full class name. Only 
     * the full class name can be set.
     * @param aFullJavaClassName Full class name.
     */
    public void setFullClassName(String aFullJavaClassName) {
        int lastPointPos = aFullJavaClassName.lastIndexOf('.');
        if (lastPointPos >= 0) {
            simpleClassName = aFullJavaClassName.substring(lastPointPos + 1); 
            packageName = aFullJavaClassName.substring(0,lastPointPos);
            fullClassName = aFullJavaClassName;
        } else {
            packageName = "";
            simpleClassName = aFullJavaClassName;
            fullClassName = aFullJavaClassName;
        }
    }

    /**
     * Returns the full class name.
     */
    public String getFullClassName() {
        return fullClassName;
    }

    /**
     * Returns the package name.
     * The package name is extracted from the 
     * full class name.
     */
    public String getPackageName() {
        return packageName;
    }

    /**
     * Returns the simple class name.
     * The simple class name is derived from the
     * full class name. It contains only the name
     * of the class.
     */
    public String getSimpleClassName() {
        return simpleClassName;
    }
    
    /**
     * Returns the TINE format string.
     */
    public String getTineFormat() {
        return tineFormat;
    }

    /**
     * Returns the TINE format key.
     */
    public int getTineFormatKey() {
        return tineKey;
    }
    /**
     * Returns true if the data type is a simple type.
     * Void is not regarded as a simple type.
     */
    public boolean isSimple() {
        return (fullClassName.equals(double.class.getName())
                || fullClassName.equals(float.class.getName()) 
                || fullClassName.equals(int.class.getName()) 
                || fullClassName.equals(short.class.getName()) 
                || fullClassName.equals(long.class.getName()) 
                || fullClassName.equals(boolean.class.getName()) 
                || fullClassName.equals(byte.class.getName())); 
    }
    
    /**
     * Returns true if the data type is a String.
     */
    public boolean isString() {
        return (fullClassName.equals(String.class.getName()));
    }
    
    /**
     * Returns true if the data type is boolean.
     * Void is not regarded as a simple type.
     */
   public boolean isBoolean() {
        return (fullClassName.equals(boolean.class.getName()));
    }
   
   /**
    * Returns true if the data type is boolean.
    * Void is not regarded as a simple type.
    */
    public boolean isVoid()
    {
        return (fullClassName.equals(void.class.getName()));
    }

    /**
     * Returns true if the data type is a TINE-specific type.
     */
    public boolean isTineType() {
        return (fullClassName.startsWith("de.desy.tine"));
    }

    /**
     * Returns true if the data type is a class from which
     * objects can be instantiated.
     */
    public boolean isObject() {
        return (!(isSimple()||isVoid()||isString()));
    }

    /**
     * Returns the simple class name of the wrapper class.
     * Returns the class name of the wrapper class 
     * (Double, Integer, ...) for simple data 
     * types (double, int, ...).
     * If the type is not simple data type, the 
     * method behaves like getSimpleClassName().
     */
    public String getWrapperClassName() {
        String result = null;
        if (fullClassName.equals(double.class.getName())) {
            result = Double.class.getSimpleName();
        } else if (fullClassName.equals(float.class.getName())) {
            result = Float.class.getSimpleName();
        } else if (fullClassName.equals(int.class.getName())) {
            result = Integer.class.getSimpleName();
        } else if (fullClassName.equals(short.class.getName())) {
            result = Short.class.getSimpleName();
        } else if (fullClassName.equals(long.class.getName())) {
            result = Long.class.getSimpleName();
        } else if (fullClassName.equals(boolean.class.getName())) {
            result = Boolean.class.getSimpleName();
        } else if (fullClassName.equals(byte.class.getName())) {
            result = Byte.class.getSimpleName();
        } else {
            result = getSimpleClassName();
        }
        return result;
    }
    
}
