/*
 * Created on Jun 5, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;


/**
 * A field of a user-defined structure.
 * A User-defined structure is described as a 
 * list of fields. Each field has a className,
 * a type and an array size. (=> Since 
 * user-defined structures
 * are derived from de.desy.tine.TTaggedStructure,
 * all members have to be arrays. A scalar is
 * seen as an array of size 1.)
 * 
 * @see CfgTypeDefinition
 * @author jwlg
 */
public class CfgTypeField {
    
    private String name = "";
    private CfgPropertyType type = CfgPropertyType.SHORT;
    private int arraySize = 1;
    
    /**
     * Default constructor.
     */
    public CfgTypeField() {
    }

    /**
     * Returns the array size.
     */
    public int getArraySize() {
        return arraySize;
    }

    /**
     * Sets the array size.
     * @param arraySize New array size.
     */
    public void setArraySize(int arraySize) {
        this.arraySize = arraySize;
    }
    
    /**
     * Returns the name.
     */
    public String getName() {
        return name;
    }

    /** 
     * Sets the name.
     * @param name New name.
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Returns the type.
     */
    public CfgPropertyType getType() {
        return type;
    }
    
    /**
     * Sets the type.
     * @param type New property type.
     */
    public void setType(CfgPropertyType type) {
        this.type = type;
    }

    @Override
    public int hashCode() {

        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + arraySize;
        result = PRIME * result + ((name == null) ? 0 : name.hashCode());
        result = PRIME * result + ((type == null) ? 0 : type.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {

        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final CfgTypeField other = (CfgTypeField) obj;
        if (arraySize != other.arraySize)
            return false;
        if (name == null) {
            if (other.name != null)
                return false;
        } else if (!name.equals(other.name))
            return false;
        if (type == null) {
            if (other.type != null)
                return false;
        } else if (!type.equals(other.type))
            return false;
        return true;
    }
    
    
}
