/*
 * Created on Jan 16, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.config;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * A class which provides verion handling.
 * A version consists of a major number, a minor number,
 * and a revision number. Versions can be compared.
 * 
 * @author jwlg
 */
@XmlRootElement
public class CfgVersion implements Comparable {
   
    private int major = 0;
    private int minor = 0;
    private int revision = 0;

    /**
     * Default constructor.
     * Initializes to version 0.0.0.
     *
     */
    public CfgVersion() {
    }
    
    /**
     * Constructor.
     * @param major Major number.
     * @param minor Minor number.
     * @param revision Revision number.
     */
    public CfgVersion(int major, int minor, int revision) {
        this.major = major;
        this.minor = minor;
        this.revision = revision;
    }

    /**
     * Returns the major number.
     */
    @XmlElement
    public int getMajor() {
        return major;
    }

    /**
     * Sets the major number.
     * @param major New major number.
     */
    public void setMajor(int major) {
        this.major = major;
    }

    /**
     * Returns the minor number.
     */
    @XmlElement
    public int getMinor() {
        return minor;
    }

    /**
     * Sets the minor number.
     * @param minor New number.
     */
    public void setMinor(int minor) {
        this.minor = minor;
    }

    /**
     * Returns the revision.
     */
    @XmlElement
    public int getRevision() {
        return revision;
    }

    /**
     * Sets the revision.
     * @param revision new revision.
     */
    public void setRevision(int revision) {
        this.revision = revision;
    }

    public String toString() {
        return major + "." + minor + "." + revision;
    }

    public boolean equals(CfgVersion other) {
        return ((this.major == other.major)
                && (this.minor == other.minor)
                && (this.revision == other.revision));
    }

    public boolean equals(Object obj) {
        if (obj instanceof CfgVersion) {
            return equals((CfgVersion)obj);
        } else {
            return false;
        }
    }

    public int compareTo(Object o) {
        CfgVersion other = (CfgVersion)o;  // Possibly throws ClassCastException
        int result = 0;
        if (this.major == other.major) {
            if (this.minor == other.minor) {
                if (this.revision == other.revision) {
                    result = 0;
                } else {
                    result = this.revision < other.revision ? -1 : 1;
                }
            } else {
                result = this.minor < other.minor ? -1 : 1;
            }
        } else {
            result = this.major < other.major ? -1 : 1;
        }
        return result;      
    }

}
