/*
 * Created on Sep 20, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.exceptions;


/**
 * A user-defined exception.
 * If the user is unhappy with the exception hierarchy 
 * supplied by the device server library, it is possible to use
 * a user-defined exception as well. There are three ways to use
 * user-defined exceptions:
 * <ul>
 * <li>Don't use them. Use the basic Exceptions and supply a specific
 *  message instead. This is the recommended way to specialize a
 *  pre-defined exception.</li>
 * <li>Use a user-defined message with a fixed ID. This is recommended
 * when the kind of error remains unspecified and only the message 
 * text counts.</li>
 * <li>Use a user-defined message with customized message text and ID.
 * This is only useful if clients need to interpret the ID.</>
 * </ul>
 * 
 * @author jwlg
 */
public class UserException extends DeviceServerException {

    private static final long serialVersionUID = -4106451051405661030L;
    private final int errorCode;
    
    /**
     * Default constructor.
     */
    public UserException() {
        super();
        this.errorCode = ErrorSpecs.USER_EXCEPTION.getKey();
    }
    
    /**
     * Constructor with a specific message.
     * @param message A message which overrides the default message.
     */
    public UserException(String message) {
        super(message);
        this.errorCode = ErrorSpecs.USER_EXCEPTION.getKey();
    }
    
    /**
     * Coinstructor.
     * Builds a User exception with a specified error code.
     * @param errorCode The error code.
     */
    public UserException(int errorCode) {
        super();
        this.errorCode = errorCode;
    }
    
    /**
     * Constructor.
     * Builds a user-defined exception with specified 
     * error code an message text.
     * @param errorCode The error code.
     * @param errorMessage The message text.
     */
    public UserException(int errorCode, String errorMessage) {
        super(errorMessage);
        this.errorCode = errorCode;
    }
    
    /**
     * Returns the error code.
     */
    public int getErrorKey() {
        return errorCode;
    }

}
