/*
 * Created on Mar 7, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.history;

import java.util.HashMap;

import javax.xml.bind.annotation.XmlRootElement;

/**
 * The History Map.
 * 
 * A history map is a mechanism which maps 
 * objects of the class HistoryMapEntry
 * uniquely to integers. 
 * @see #getIndexOf(HistoryMapEntry)
 * @author jwlg
 */
@XmlRootElement
public class HistoryMap {
    
    private HashMap<HistoryMapEntry, Integer> entries = new HashMap<HistoryMapEntry, Integer>();
    
    /**
     * Default Constructor.
     */
    public HistoryMap() {
    }

    /**
     * Returns the map of entries.
     * Not for general use. Needed only bu JAXB.
     */
    public HashMap<HistoryMapEntry, Integer> getEntries() {
        return entries;
    }
    
    /**
     * Assigns a map of entries.
     * Not for general use. Needed only bu JAXB.
     * @param entries New map of entries.
     */
    public void setEntries(HashMap<HistoryMapEntry, Integer> entries) {
        this.entries = entries;
    }

    /**
     * Returns a unique integer which is mapped to a HistoryMapEntry.
     * If this method is called for the first time with a HistoryMapEntry,
     * it creates a new integer (i.e. increments an internal counter) and
     * registers the entry in its map of entries. When it is called the
     * next time with the same entry, it finds the entry in its entry map
     * and returns the same integer.
     * @param entry
     */
    public int getIndexOf(HistoryMapEntry entry) {
        Integer result = entries.get(entry);
        if (result == null) {
            result = new Integer(entries.size() + 1);
            entries.put(entry, result);
        }
        return result.intValue();
    }
    
    /**
     * Wrapper for getIndexOf(HistoryMapEntry).
     * This adds or finds a HistoryMapEntry for property of a 
     * whole device group.
     * @param group Name of the device group.
     * @param property Name of the property.
     * @return The index.
     */
    public int getIndexOf(String group, String property) {
        return getIndexOf(new HistoryMapEntry(group, "", property));
    }
    
    /**
     * Wrapper for getIndexOf(HistoryMapEntry).
     * This adds or finds a HistoryMapEntry for a property of a 
     * single device in a device group.
     * @param group The device group.
     * @param device The device.
     * @param property The property.
     * @return The index.
     */
    public int getIndexOf(String group, String device, String property) {
        return getIndexOf(new HistoryMapEntry(group, device, property));
    }
    
}
