/*
 * Created on Sep 19, 2007
 *
 */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.io.File;

import org.apache.log4j.Logger;

import de.desy.mst.auxiliary.devl.deviceserverlib.initialization.controller.ControllerGroup;
import de.desy.mst.auxiliary.devl.deviceserverlib.initialization.controller.ControllerGroupReader;

/**
 * Builds an equipment module for a group of device controllers.
 * 
 * @author jwlg
 */
public class ControllerEquipmentModuleBuilder<D extends IBasicDevice, T extends ABasicController<D>> extends AEquipmentModuleBuilder<T> {

    private static Logger log = Logger.getLogger(ControllerEquipmentModuleBuilder.class);
    IControllerModuleFactory<T> moduleFactory;

    @Override
    public void createDeviceGroup() throws EquipmentModuleBuilderException {

        if (getDeviceServer() == null) {
            log.error("Cannot create device group. DeviceServer is not set.");
            throw new EquipmentModuleBuilderException();
        }
        if (getCfgDeviceGroup() == null) {
            log.error("Cannot create device group. CfgDeviceGRoup is not set.");
            throw new EquipmentModuleBuilderException();
        }
        if (moduleFactory == null) {
            log.error("Cannot create device group. Module factory is not set.");
            throw new EquipmentModuleBuilderException();
        }

        String fileName = "";

        DeviceGroup<T> deviceGroup = new DeviceGroup<T>();
        deviceGroup.setName(getCfgDeviceGroup().getName());
        ControllerDeviceFactory<D,T> deviceFactory = new ControllerDeviceFactory<D,T>();

        DirectorySet directorySet = getDeviceServer().getDirectorySet();

        String databaseDir = directorySet.getDatabaseDirectory();
        fileName = databaseDir + File.separator + "ControllerGroup." + getCfgDeviceGroup().getName() + ".xml";

        ControllerGroupReader reader = new ControllerGroupReader();
        try {
            ControllerGroup controllerGroup = reader.read(fileName);
            deviceFactory.setControllerGroupDefinition(controllerGroup);
            deviceFactory.setFactoryMethod(moduleFactory.createFactoryMethod());
            deviceFactory.setDeviceServer(getDeviceServer());
            deviceFactory.createDevices(deviceGroup);
            deviceGroup.setDeviceContext(getDeviceContext());
            super.setDeviceGroup(deviceGroup);
        } catch (Exception e) {
            log.error("Cannot read controller group definition from " + fileName + ": " + e, e);
        }

    }

    /**
     * Sets the module factory.
     * Also passes the module factory to the superclass.
     * @param moduleFactory A controller module factory.
     */
    public void setControllerModuleFactory(IControllerModuleFactory<T> moduleFactory) {
        this.moduleFactory = moduleFactory;
        super.setModuleFactory(moduleFactory);
    }

}
