/*
 * Created on Aug 3, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;



/**
 * The device context.
 * The device context makes it possible for a device
 * to access its device group as well as data from
 * the device server.
 * 
 * It provides an interface which hides the actual device
 * server from the device and makes only a selected set
 * of functionality available.
 *
 */
public class DeviceContext {
    
    GenericDeviceServer deviceServer = null;
    DeviceGroup deviceGroup = null;
    ACommonData commonData = null;
    
    /**
     * Constructor.
     * @param deviceServer The device server.
     * @param deviceGroup The device group.
     */
    public DeviceContext(
        GenericDeviceServer deviceServer,
        DeviceGroup deviceGroup) {
        this.deviceServer = deviceServer;
        this.deviceGroup = deviceGroup;
    }
    
    /**
     * Constructor.
     * The device group is set to null.
     * @param deviceServer The device server.
     */
    public DeviceContext(
        GenericDeviceServer deviceServer) {
        this.deviceServer = deviceServer;
        this.deviceGroup = null;
    }

    public DeviceContext() {
    }
    
    /**
     * Returns the device group.
     */
    public DeviceGroup getDeviceGroup() {
        return deviceGroup;
    }
    
    /**
     * Returns the device group.
     * @param name
     */
    public DeviceGroup getDeviceGroup(String name) {
        return deviceServer.getDeviceGroup(name);
    }
    
    /**
     * Returns the application name.
     */
    public String getApplicationName() {
        return deviceServer.getApplicationName();
    }

    /**
     * Returns the directory set which is used by the device server.
     */
    public DirectorySet getDirectorySet() {
        return deviceServer.getDirectorySet();
    }
    
    public ACommonData getCommonData() {
        return commonData;
    }
    
    void setCommonData(ACommonData commonData) {
        this.commonData = commonData;
    }
    
    void setDeviceGroup(DeviceGroup group) {
        deviceGroup = group;
    }
    
    void setDeviceServer(GenericDeviceServer server) {
        this.deviceServer = server;
    }
    
}
