/*
 * Created on Jul 31, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.server;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

/**
 * A reader for Java Properties (not TINE properties).
 * 
 * @author jwlg
 */
public class PropertyReader {
    
    private URL url;
   
    /**
     * Constructor with URL.
     * @param urlString The URL.
     * @throws MalformedURLException
     */
    public PropertyReader(String urlString) throws MalformedURLException {
        this.url = new URL(urlString);
    }
    
    /**
     * Constructor with URL.
     * @param url The URL of the properties file. 
     */
    public PropertyReader(URL url) {
        this.url = url;
    }
    
    /**
     * Reads and returns the properties.
     * @throws IOException
     */
    public Properties getProperties() throws IOException {
        InputStream is = this.url.openStream();
        Properties result = new Properties();
        result.load(is);
        is.close();
        return result;
    }

    /**
     * Static method which reads properties from an URL.
     * Simple to use.
     * @param urlString The URL.
     * @return A set of properties.
     * @throws IOException
     */
    public static Properties getPropertiesFromUrl(String urlString) throws IOException {
        return getPropertiesFromUrl(new URL(urlString));
    }
    
    /**
     * Static method which reads properties from an URL.
     * Simple to use.
     * @param url The URL.
     * @return A set of properties.
     * @throws IOException
     */
    public static Properties getPropertiesFromUrl(URL url) throws IOException {
        PropertyReader reader = new PropertyReader(url);
        return reader.getProperties();
    }

}
