/*
 * Created on Mar 12, 2007
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.tools;

import java.math.BigDecimal;
import java.math.MathContext;
import java.util.ArrayList;

/**
 * Generates quasi-logarithmic ranges in 1-2-5-10 patterns.
 * Example: getIntRange(0,3) returns {1,2,5,10,20,50,100,200,1000}
 * @author jwlg
 */
public class FakeLogRange {
    
    
    /**
     * Returns a range as an ArrayList of BigDecimals.
     * @param minExponent Start Exponent.
     * @param maxExponent EndExponent.
     */
    public static ArrayList<BigDecimal> getBigDecimalRange(int minExponent, int maxExponent) {
        
        ArrayList<BigDecimal> result = null; 
        MathContext mathContext = new MathContext(1);
        
        if (minExponent >= maxExponent) {
            return new ArrayList<BigDecimal>();
        }
        
        result = new ArrayList<BigDecimal>((maxExponent - minExponent) * 3 - 1);

        double startValue = 0.0;
        for (int i = minExponent; i < maxExponent; i++) {
            startValue = Math.pow(10.0, i);
            result.add(new BigDecimal( 1.0 * startValue, mathContext));
            result.add(new BigDecimal( 2.0 * startValue, mathContext));
            result.add(new BigDecimal( 5.0 * startValue, mathContext));
        }
        
        result.add(new BigDecimal(Math.pow(10.0, maxExponent),mathContext));
        return result;
    }
    
    /**
     * Returns a range as an array of double.
     * @param minExponent Start Exponent.
     * @param maxExponent EndExponent.
     */
    public static double[] getDoubleArray(int minExponent, int maxExponent) {
        
        ArrayList<BigDecimal> bigDecimalRange = null;
        double[] result =null;
        
        if (minExponent >= maxExponent) {
            return new double[0];
        }
        bigDecimalRange = getBigDecimalRange(minExponent, maxExponent);
        result = new double[bigDecimalRange.size()];
        
        for (int i = 0; i< bigDecimalRange.size(); i++) {
            result[i] = bigDecimalRange.get(i).doubleValue();
        }

        return result;
    }

    /**
     * Returns a range as an ArrayList of double.
     * @param minExponent Start Exponent.
     * @param maxExponent EndExponent.
     */
    public static ArrayList<Double> getDoubleRange(int minExponent, int maxExponent) {
        
        ArrayList<BigDecimal> bigDecimalRange = null;
        ArrayList<Double> result = null; 
        
        if (minExponent >= maxExponent) {
            return new ArrayList<Double>();
        }
        
        bigDecimalRange = getBigDecimalRange(minExponent, maxExponent);
        result = new ArrayList<Double>(bigDecimalRange.size());

        for (int i = 0; i < bigDecimalRange.size(); i++) {
            result.add(bigDecimalRange.get(i).doubleValue());
        }

        return result;

    }

    /**
     * Returns a range as an ArrayList of Integer.
     * @param minExponent Start Exponent.
     * @param maxExponent EndExponent.
     */
    public static ArrayList<Integer> getIntRange(int minExponent, int maxExponent) {
        ArrayList<Integer> result = null;

        if (minExponent >= maxExponent) {
            return new ArrayList<Integer>();
        }
        if (minExponent < 0) {
            return new ArrayList<Integer>();
        }

        result = new ArrayList<Integer>((maxExponent - minExponent) * 3 - 1);
        int startValue = 0;
        for (int i = minExponent; i < maxExponent; i++) {
            startValue = (int)Math.round(Math.pow(10.0, (double)i));
            result.add(startValue);
            result.add(startValue * 2);
            result.add(startValue * 5);
        }
        result.add((int)Math.round(Math.pow(10.0, maxExponent)));

        return result;

    }

    /**
     * Returns a range as an array of int.
     * @param minExponent Start Exponent.
     * @param maxExponent EndExponent.
     */

    public static int[] getIntArray(int minExponent, int maxExponent)  {
        int[] result = null;

        if (minExponent >= maxExponent) {
            return new int[0];
        }
        if (minExponent < 0) {
            return new int[0];
        }

        result = new int[(maxExponent - minExponent) * 3 - 1];

        int startValue = 0;
        int arrayIndex = 0;

        for (int i = minExponent; i < maxExponent; i++) {
            startValue = (int)Math.round(Math.pow(10.0, i));
            result[arrayIndex] = startValue;
            arrayIndex++;
            result[arrayIndex] = startValue * 2;
            arrayIndex++;
            result[arrayIndex] = startValue * 5;
            arrayIndex++;
        }
        result[arrayIndex] = (int)Math.round(Math.pow(10.0, maxExponent));

        return result;
    }

}
