/*
 * Created on Mar 15, 2006
 *
  */
package de.desy.mst.auxiliary.devl.deviceserverlib.tools;

import java.util.LinkedList;

import de.desy.mst.auxiliary.devl.deviceserverlib.config.CfgAlarm;
import de.desy.mst.auxiliary.devl.deviceserverlib.config.CfgDeviceGroup;
import de.desy.mst.auxiliary.devl.deviceserverlib.config.CfgDeviceType;
import de.desy.mst.auxiliary.devl.deviceserverlib.config.CfgOperation;
import de.desy.mst.auxiliary.devl.deviceserverlib.config.CfgProperty;

/**
 * A utility class which provides tools for device-server specific lists.
 * 
 * These are tools for property lists, alarm lists, operation lists and the like.
 * 
 * @author jwlg
 */
public class ListUtils {

    
    /**
     * Returns true if a property with a certain name exists in a list.
     * @param propertyList A property list.
     * @param propertyCfg The property of which the name is sought.
     */
    public static boolean existsPropertyName(LinkedList<CfgProperty> propertyList, CfgProperty propertyCfg)
    {
        return existsPropertyName(propertyList, propertyCfg.getName());
    }
    
    /**
     * Returns true if a property with a certain name exists in a list.
     * @param propertyList A property list.
     * @param propertyName The property name which is sought.
     */
    public static boolean existsPropertyName(LinkedList<CfgProperty> propertyList, String propertyName)
    {
        boolean result = false;
        for (CfgProperty element : propertyList) {
            if (element.getName().equalsIgnoreCase(propertyName)) {
                result = true;
                break;
            }
        }
        return result;
    }

    /**
     * Returns true if an  operation with a certain name exists in a list.
     * @param operationList An operation list.
     * @param operationName The operation name which is sought.
     */
    public static boolean existsOperationName(LinkedList<CfgOperation> operationList, String operationName)
    {
        boolean result = false;
        for (CfgOperation element : operationList) {
            if (element.getName().equalsIgnoreCase(operationName)) {
                result = true;
                break;
            }
        }
        return result;
    }

    /**
     * Returns true if an device group with a certain name exists in a list.
     * @param deviceGroupList A list of device groups.
     * @param deviceGroupCfg A device group of which the name is sought.
     */
    public static boolean existsModuleName(LinkedList<CfgDeviceGroup> deviceGroupList, CfgDeviceGroup deviceGroupCfg) {
        boolean result = false;
        for (CfgDeviceGroup element : deviceGroupList) {
            if (element.getName().equalsIgnoreCase(deviceGroupCfg.getName())) {
                result = true;
                break;
            }
        }
        return result;
       
    }

    /**
     * Returns true if a property or an operation with a given name 
     * exists in a device type.
     * @param deviceType The device type.
     * @param name The name which is sought.
     */
    public static boolean existsPropertyOrOperationName(CfgDeviceType deviceType, String name) {
        return ((existsPropertyName(deviceType.getProperties(), name))
                || (existsOperationName(deviceType.getOperations(), name)));
    }
               
 
    /**
     * Returns true if an alarm with a given name exists in an alarm list.
     * @param alarmList An alarm list.
     * @param alarmCfg An alarm of which the name is compared to the list elements.
     */
    public static boolean existsAlarmName(LinkedList<CfgAlarm> alarmList, CfgAlarm alarmCfg)
    {
        return existsAlarmName(alarmList, alarmCfg.getName());
    }

    /**
     * Returns true if an alarm with a given name exists in an alarm list.
     * @param alarmList An alarm list.
     * @param alarmName The alarm name which is sought.
     */
    public static boolean existsAlarmName(LinkedList<CfgAlarm> alarmList, String alarmName)
    {
        boolean result = false;
        for (CfgAlarm element : alarmList) {
            if (element.getName().equals(alarmName)) {
                result = true;
                break;
            }
        }
        return result;
    }

    
}
