package de.desy.tine.accesslayer;

import de.desy.tine.definitions.TErrorList;

/**
 *
 * <code>ChannelCallback</code> is the callback interface that receives updates when a channel
 * receives a new value or goes into an error state.
 *
 * @see ChannelCallbackAdapter
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public interface ChannelCallback {

    /**
     * An asynchronous write was attempted and failed with the given status.
     *
     * @param address the name of the property to which the values was written
     * @param value the value that was written
     * @param linkStatus the status (can be converted to string by {@link TErrorList#toString(int)}
     */
    public void writeFailed(String address, Object value, int linkStatus);

    /**
     * Asynchronouse write completed successfully.
     *
     * @param address the name of the property to which the values was written
     * @param value the value that was written
     */
    public void writeCompleted(String address, Object value);

    /**
     * The channel value was updated. The current value can be obtained by {@link Channel#getValue()},
     * but the value is not guaranteed to be the value that triggered this update. If another value
     * was received while this callback is being handled the {@link Channel#getValue()} will return
     * the new value.
     *
     * @param channel the channel which value was updated
     * @deprecated use {@link ChannelCallback2#updateValue(Channel, Object)} instead
     */
    @Deprecated
    public void updateValue(Channel channel);

    /**
     * The channel status was updated. The current status can be obtained by {@link Channel#getStatus()}.
     * If another update has been received while this callback is handled the {@link Channel#getStatus()}
     * will return the newest value.
     *
     * @param channel the channel which value was updated
     * @deprecated use {@link ChannelCallback2#updateErrorStatus(Channel, int)} instead
     */
    @Deprecated
    public void updateErrorStatus(Channel channel);

    /**
     * Called when the an error happened in the channels callback call.
     *
     * @param message message describing the situation
     * @param cause the cause of the error
     */
    public void channelError(String message, Throwable cause);
}
