package de.desy.tine.accesslayer;

import de.desy.tine.definitions.TErrorList;

/**
 * 
 * <code>ConnectionException</code> can be thrown when there was an error establishing the connection 
 * to the underlying system.
 *
 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
 *
 */
public class ConnectionException extends Exception {
    
    private static final long serialVersionUID = -7709275807796443410L;
    
    private final int errorStatus;
    
    /**
     * Constructs a new exception.
     * 
     * @param message the message describing what went wrong
     */
    public ConnectionException(String message) {
        this(message,0);
    }
    
    /**
     * Constructs a new exception.
     * 
     * @param message the message describing what went wrong
     * @param err the error code describing the type of the error 
     */
    public ConnectionException(String message, int err) {
        super(message);
        this.errorStatus = err;
    }
    
    /**
     * Constructs a new exception.
     * 
     * @param message the message describing what went wrong
     * @param the cause of this exception 
     */
    public ConnectionException(String message, Throwable cause) {
        super(message,cause);
        this.errorStatus = 0;
    }
    
    
    /**
     * @see TErrorList#toString(int)
     * @return the code of the error that caused the exception
     */
    public int getErrorStatus() {
        return errorStatus;
    }
}
