package de.desy.tine.accesslayer;

import java.util.Arrays;
import java.util.Date;

import de.desy.tine.definitions.TFormat;

class Demo {
    
    public static void main(String[] args) throws ConnectionException, Exception {
        ChannelFactory factory = ChannelFactory.getInstance();
        String address = "/TEST/SineServer/SineGen0/Amplitude.MAX";
        //String address = "/XFEL.TEST/CPS.DEV.CORR/CCY.4.B1/PS.RBV";
        //make a channel that periodically receives new values, using default data type (e.g. double)
        Channel channel = factory.getChannel(address,ConnectionMode.POLL,1000,new ChannelCallbackAdapter(){
            @Override
            public void updateValue(Channel channel) {
                //double value = ((double[])channel.getValue())[0];
                float value = (Float)channel.getValue();
                System.out.println("Fast channel: " + new Date((long)channel.getRawValue().getTimeStamp()*1000) + " " + value);
            }
        });
        
        //make another channel that receives new values with a different frequency
        Channel slowChannel = factory.getChannel(address,ConnectionMode.POLL,10000,new ChannelCallbackAdapter(){
            @Override
            public void updateValue(Channel channel) {
                double value = ((double[])channel.getValue())[0];
                System.out.println("Slow channel: " + new Date((long)channel.getRawValue().getTimeStamp()*1000) + " " + value);
            }
        });
        
        Thread.sleep(5000);
        
        //asynchronously set a new value
        channel.setValue(5.3f);
        
        //close the fast channel, to stop receiving events
        channel.stop();
        Thread.sleep(5000);
        //close the slow channel
        slowChannel.stop();
        
        //at this point the underlying tine link to the Amplitude property is still alive.
        //after 30 seconds (if there is no new channel for Amplitude property) the underlying tine link will be closed as well
        
        Channel nonExistingChannel = factory.getChannel("someBogusAddress",ConnectionMode.POLL,2000,new ChannelCallbackAdapter() {
            @Override
            public void updateValue(Channel channel) {
                System.err.println("Received a value from a non existing channel!?");
            }
            @Override
            public void updateErrorStatus(Channel channel) {
                System.err.println("Non existing channel connection status: " + channel.getStatus());
            }
        });
        
        Thread.sleep(10000);
        nonExistingChannel.stop();
        
        //create a channel that receives updates when the value changes, request an integer type and 5 different values
        TFormat format = TFormat.valueOf(TFormat.CF_INT32);
        int size = 5;
        Channel eventChannel = factory.getChannel(address,format,size,ConnectionMode.CHANGE,1000,new ChannelCallbackAdapter() {
            @Override
            public void updateValue(Channel channel) {
                int[] value = ((int[])channel.getValue());
                System.out.println("Event received: " + new Date((long)channel.getRawValue().getTimeStamp()*1000) + " " + Arrays.toString(value));
            }
        });
        
        Thread.sleep(2000);
        //read the last value that was received from the server
        int[] value = (int[])eventChannel.getValue();
        System.out.println("Last value: " + Arrays.toString(value));
        int[] val = new int[5];
        for (int i = 0; i < 5; i++) 
            val[i] = value[i]+1;
        eventChannel.setValue(val);
        
        Thread.sleep(2000);
        eventChannel.stop();
        
        //sometimes you just want to read a value once and forget about the connection stuff
        double[] synchronouslyReadValue = (double[])factory.getValue(address);
        System.out.println(Arrays.toString(synchronouslyReadValue));
        //or write the value
        synchronouslyReadValue[0] = Math.random()*1000;
        factory.setValue(address,synchronouslyReadValue);
        
        Thread.sleep(500000);
    }
}
