package de.desy.tine.alarmUtils;

import de.desy.tine.definitions.TFormat;
import de.desy.tine.definitions.TStrings;
import de.desy.tine.structUtils.TTaggedStructure;

public class CasAction extends TTaggedStructure {

	// CasAct
//  char email[128];
//  char ctx[CONTEXT_NAME_SIZE];
//  char srv[EXPORT_NAME_SIZE];
//  char dev[DEVICE_NAME_SIZE];
//  char prp[PROPERTY_NAME_SIZE];
//  char actionSignalString[128];
//  int actionSignalFormat;
//  int alarmCode;
	
	public void clear() {
		contextString = null;
		serverString = null;
		deviceString = null;
		propertyString = null;
		actionSignalString = null;
		emailString = null;
	}

	private char[] email = new char[128]; /* email */
	private String emailString;
	private char[] ctx = new char[TStrings.CONTEXT_NAME_SIZE]; /* calling context */
	private String contextString;
	private char[] srv = new char[TStrings.EXPORT_NAME_SIZE]; /* calling equipment module */
	private String serverString;
	private char[] prp = new char[TStrings.PROPERTY_NAME_SIZE]; /* calling property */
	private String propertyString;
	private char[] dev = new char[TStrings.DEVICE_NAME_SIZE]; /* calling device name */
	private String deviceString;
	private char[] actionSignal = new char[128]; /* action signal */
	private String actionSignalString;
	private int[] actSignalFormat = new int[1]; /* action signal format */
	private int[] alarmCode = new int[1]; /* alarm code */

	private void initStructure() {
		addField(email,"email");
		addField(ctx,"context");
		addField(srv,"server");
		addField(dev,"device");
		addField(prp,"property");
		addField(actionSignal,"actionSignalString");
		addField(actSignalFormat, "actionSignalFormat");
		addField(alarmCode, "alarmCode");
		initDone();
	}

	public CasAction() {
		super("CasAct");
		initStructure();
	}

	public CasAction(CasAction reference) {
		super("CasAct");
		initStructure();
		alarmCode[0] = reference.alarmCode[0];
		actSignalFormat[0] = reference.actSignalFormat[0];
		setActionSignal(reference.getActionSignal());
		setEmail(reference.getEmail());
		setContext(reference.getContext());
		setServer(reference.getServer());
		setDevice(reference.getDevice());
		setProperty(reference.getProperty());
	}

	public int getActionSignalFormat() {
		return TFormat.makeByteFormatCode(actSignalFormat[0]);
	}

	public void setActionSignalFormat(int value) {
		actSignalFormat[0] = TFormat.makeStdFormatCode(value);
	}

	public int getAlarmCode() {
		return alarmCode[0];
	}

	public void setAlarmCode(int value) {
		alarmCode[0] = value;
	}

	public String getContext() {
		if (contextString == null)
			contextString = new String(ctx).trim();
		return contextString;
	}

	public void setContext(String context) {
		pushChars(context,ctx);
		contextString = new String(context);
	}

	public String getServer() {
		if (serverString == null)
			serverString = new String(srv).trim();
		return serverString;
	}

	public void setServer(String server) {
		pushChars(server,srv);
		serverString = new String(server);
	}

	public String getDevice() {
		if (deviceString == null)
			deviceString = new String(dev).trim();
		return deviceString;
	}

	public void setDevice(String device) {
		pushChars(device,dev);
		deviceString = new String(device);
	}

	public String getProperty() {
		if (propertyString == null)
			propertyString = new String(prp).trim();
		return propertyString;
	}

	public void setProperty(String property) {
		pushChars(property,prp);
		propertyString = new String(property);
	}
	
	public String getActionSignal() {
		if (actionSignalString == null)
			actionSignalString = new String(actionSignal).trim();
		return actionSignalString;
	}

	public void setActionSignal(String actionSignal) {
		pushChars(actionSignal,this.actionSignal);
		actionSignalString = new String(actionSignal);
	}
	
	public String getEmail() {
		if (emailString == null)
			emailString = new String(email).trim();
		return emailString;
	}

	public void setEmail(String email) {
		pushChars(email,this.email);
		emailString = new String(email);
	}
}
