package de.desy.tine.bitfieldUtils;

import java.util.HashMap;

public class TBitfieldRegistry
{
  private static HashMap<String, TBitfield> bfRegistry = new HashMap<String, TBitfield>();
  private static String rmvDeviceName(String fullName)
  { // might pass fullName of the form /<context>/<server>/<device>
    if (fullName == null) return "this";
    String[] parts = fullName.split("/");
    if (parts.length == 1) return parts[0];
    if (fullName.startsWith("/")) // context given 
      return "/" + parts[1] + "/" + parts[2];
    return parts[0];
  }
  public static TBitfield[] getRegisteredBitfields()
  {
    return bfRegistry.values().toArray(new TBitfield[0]);
  }
  public static void add(TBitfield bitfield)
  {
    if (bitfield == null) return; //TODO: throw exception
    String srv = rmvDeviceName(bitfield.getSrvName());
    String key = srv + "/" +  bitfield.getName();
    if (!bfRegistry.containsKey(key))
    {
      bfRegistry.put(key, bitfield);
    }
  }
  public static TBitfield getBitfield(String context, String server, String tag)
  {
    if (tag == null) return null;
    if (context == null) context = "default";
    if (server == null) server = "this"; 
    return getBitfield("/"+context+"/"+server+"/"+tag);
  }
  public static TBitfield getBitfield(String server, String tag)
  {
    if (tag == null) return null;
    if (server == null) server = "this"; 
    return getBitfield("/"+server+"/"+tag);
  }
  public static TBitfield getBitfield(String name)
  {
    if (name == null) return null;
    if (bfRegistry.isEmpty()) return null;
    if (bfRegistry.containsKey(name))
    {
      return (TBitfield) bfRegistry.get(name);
    }
    boolean useAny = false;
    if (name.length() == 0 && bfRegistry.size() == 1)
    {
      useAny = true;
    }
    if (!name.contains("/"))
    { // naked bitfield tag name given
      String keytag = "/" + name;
      String[] keys = bfRegistry.keySet().toArray(new String[0]);
      if (keys == null) return null;
      for (String s : keys)
      {
        if (s.endsWith(keytag) || useAny)
        {
          return (TBitfield) bfRegistry.get(s);
        }
      }
    }
    return null;
  }
}
