package de.desy.tine.client;

import java.util.Iterator;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.definitions.TMode;
import de.desy.tine.headers.TSubReqHdr;
import de.desy.tine.server.logger.DbgLog;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.types.*;

public class TMcaLink implements TLinkCallback
{
  private long timeLinkEstablished = 0;
  private static TLinkFactory tlf = null;
  private static TLinkFactory getLinkFactory()
  {
    if (tlf == null) tlf = TLinkFactory.getInstance();
    return tlf;
  }
  private static String makeMcaLinkKey(TLink lnk)
  {
  	if (lnk == null) return "null";
  	return lnk.getFullDeviceNameAndProperty()+"@"+lnk.hashCode();  	
  }
  String ctx;
  String srv;
  String prp;
  String dev;
  int ndevices;
  ConcurrentHashMap<String, TLink>lnks = new ConcurrentHashMap<String,TLink>();
  //ConcurrentLinkedQueue <TLink>lnks = new ConcurrentLinkedQueue<TLink>();
  TLink parent;
  public TLink getParent() { return parent; }
  public static boolean revertOnLinkTimeout = true;
  private boolean isElevatedFormat = false;
  public boolean isElevatedFormat() { return isElevatedFormat; }
  public TMcaLink(TLink linkItem,int length)
  {
    ctx = new String(linkItem.getContext());
    srv = new String(linkItem.getDeviceServer());
    prp = new String(linkItem.getProperty());
    ndevices = length;
  }
  public TMcaLink(TLink linkItem, String devName, int length)
  {
  	this(linkItem,devName,length,false);
  }
  public TMcaLink(TLink linkItem, String devName, int length, boolean isElevatedFormat)
  {
  	boolean trace = TLinkFactory.traceKey != null && linkItem.isTraceLink();
  	ctx = new String(linkItem.getContext());
    srv = new String(linkItem.getDeviceServer());
    prp = new String(linkItem.getProperty());
    dev = new String(devName);
    linkItem.setMcaDevice(devName);
    ndevices = length;
    this.isElevatedFormat = isElevatedFormat;
    short dfmt = linkItem.dOutput.getFormat();
    if (isElevatedFormat) dfmt = TFormat.getElevatedMcaFormat(dfmt);
    TDataType dt = new TDataType(length,dfmt);
    if (TFormat.isBitfield(dfmt))
    {
      dt.setTag(linkItem.dOutput.getTag());
      dt.setField(linkItem.dOutput.getField());
      dt.setBitField(linkItem.dOutput.getBitField());
    }
    String tgt = "/"+ctx+"/"+srv+"/"+dev;
    try
    {
      parent = new TLink(tgt,prp,dt,linkItem.dInput,linkItem.devAccess);
    } catch (Exception ignore) {} // let the status code propagate
    if (parent == null)
    {
      if (trace)  linkItem.traceLink("TMcaLink", "could not instantiate MCA parent link");
      MsgLog.log("TMcaLink", "could not instantiate MCA parent link",0,null,0);
      return;
    }
    MsgLog.log("TMcaLink", "establish parent MCA Link (id "+parent.linkId+"): isBound = "+(parent.getBoundLink() == null ? "false":"true"),0,null,2);
    short mode = linkItem.getSubscription().mode;
    if (mode == TMode.CM_CANCEL) 
    { // this check should NOT be necessary
      MsgLog.log("TMcaLink", "would have instantiated MCA parent link in CANCEL mode",0,null,0);
    	mode = TMode.CM_TIMER; 
    }
    mode &= ~(TMode.CM_WAIT); // remove any WAIT that might be there ...
    parent.setMcaLink(this);
    int lid = parent.attach(mode, this, linkItem.getLinkPollingInterval());
    if (lid < 0)
    {
      MsgLog.log("TMcaLink", "could not create mca parent link : "+TErrorList.toString(-lid),-lid,null,0);
      parent.setMcaLink(null);
    }
    if (trace)  linkItem.traceLink("TMcaLink", "establish MCA parent link (link id "+lid+")");
    timeLinkEstablished = System.currentTimeMillis();
  }
  public void remove(TLink lnk)
  {
    synchronized(lnks) 
    { 
      String p = parent == null ? "parent unknown" : parent.getFullDeviceNameAndProperty();
      MsgLog.log("TMcaLink.remove", "remove "+lnk.getFullDeviceNameAndProperty()+" from McaLink "+p,TErrorList.property_is_mca,null,1);
      lnks.remove(makeMcaLinkKey(lnk));
      lnk.mcaRemovePending = false;
    }
  }
  public void add(TLink lnk)
  {
    if (TLinkFactory.isRunningAsServer())
    { // remove any link error alarms that might be lingering
      TLinkFactory tf = TLinkFactory.getInstance();
      if (tf.getAutoLinkErrorAlarms())
        tf.getEquipmentModuleFactory().clearFecLinkErrorAlarm(lnk);
    }                          
    synchronized(lnks) 
    {
      if (lnks.contains(lnk)) return;
      String p = parent == null ? "parent unknown" : parent.getFullDeviceNameAndProperty();
      String key = makeMcaLinkKey(lnk);
      MsgLog.log("TMcaLink.add", "add "+key+" to McaLink "+p,TErrorList.property_is_mca,null,1);
      lnks.put(key, lnk);
    }
  }
  public boolean getMcaValues(TLink lnk, TLink parent)
  {  	
    int idx = lnk.getMcaIndex() - 1;
    if (idx < 0 || idx >= parent.dOutput.dArrayLength)
    {
      if (TMode.isActive(lnk.getLinkMode()))
    	  MsgLog.log("TMcaLink.getMcaValues", parent.getFullDeviceNameAndProperty()+ " MCA index "+idx+" is invalid",TErrorList.invalid_index,null,0);
    	return false;
    }
    int sts = parent.linkStatus;
    if (TErrorList.hasData(parent.linkStatus))
    {
    	if (isElevatedFormat)
    	{
    		TDataType edt = new TDataType(1,parent.dOutput.dFormat);
    		edt.dataCopy(parent.dOutput, idx, true);
    		int cc = ((TCompoundDataObject[])edt.getDataObject())[0].getIntegerValue();
    		if ((cc & TErrorList.CE_SENDDATA) == 0) 
    	  { // returned status is an 'error status'
    			sts = cc; 
    	  }
    		if (TFormat.isNumberFormat(lnk.dOutput.dFormat))
    		{ // question: are the underlying bytes in dBuffers relevant ?
    			double dval = ((TCompoundDataObject[])edt.getDataObject())[0].getDoubleValue();
    			lnk.dOutput.putData(dval);
    		}
    		else if (TFormat.isNameInt(lnk.dOutput.dFormat))
    		{
    			String sval = ((NAMEINT[])edt.getDataObject())[0].getName();
    			lnk.dOutput.putData(sval);
    		}
    	}
    	else
    	{
    	  lnk.dOutput.dataCopy(parent.dOutput, idx, true);
    	}
    }
    lnk.hasNotifiedOnce = parent.hasNotifiedOnce;
    lnk.hasObtainedStatus = true;
    lnk.linkStatus = sts;
    lnk.linkStatusSource = parent.linkStatusSource;
    lnk.linkStatusStringBuffer = parent.linkStatusStringBuffer;
    lnk.dOutput.dCompletionLength = 1;
    lnk.dOutput.numblks = lnk.dOutput.blknum = lnk.dOutput.blksin = 1;
    lnk.dOutput.setDataTimeStamp(parent.dOutput.getDataTimeStamp());
    lnk.dOutput.setSystemDataStamp(parent.dOutput.getSystemDataStamp());
    lnk.dOutput.setUserDataStamp(parent.dOutput.getUserDataStamp());
    return true;  	
  }
  public void callback(TLink link)
  {
    //synchronized(lnks) 
    {
      boolean backout = false;
      boolean trace = false;
      if (link.linkStatus == TErrorList.reset_mca_property)
      {
        if (System.currentTimeMillis() > timeLinkEstablished + link.getLinkPollingInterval() + 1000)
        {          
          MsgLog.log("TMcaLink.callback","received reset mca property signal for "+
              link.getFullDeviceNameAndProperty(),link.linkStatus, null,1);
          backout = true;
        }
        else
        {
          MsgLog.log("TMcaLink.callback","ignore reset mca property signal for "+
              link.getFullDeviceNameAndProperty()+" due to recent reset",link.linkStatus, null,1); 
          link.linkStatus = 0;
        }
      }
      if (TErrorList.isTimeout(link.linkStatus) &&
          link.linkTimeouts > TLinkFactory.RETRY_THRESHOLD*5 &&
          revertOnLinkTimeout) backout = true;
      if (backout) synchronized(lnks)
      { // server seems to have disappeared => revert to original conditions
        MsgLog.log("TMcaLink","Mapped MCA Link " +  "/"+ctx+"/"+srv+"/"+dev+"["+prp+"] has gone down : revert to original conditions",TErrorList.link_error,null,0);
        TLinkFactory.rmvMcaLink(this);
        TSubReqHdr sub;
        for (TLink lnk : lnks.values())
        {
          trace = TLinkFactory.traceKey != null && lnk.isTraceLink(lnk.getFullDeviceNameAndProperty());
          if (trace) 
          {
            lnk.traceLink("TMcaLink.callback", "MCA element ("+ lnk.linkId+") is separating from parent");
            if (lnk.hasDependents()) 
              lnk.traceLink("TMcaLink.callback", "MCA element has dependents");
          }
          lnk.setMcaIndex(0);
          lnk.setMcaDevice(null);
          if (lnk.primary != null)
          {
            lnk.bindToParentLink(lnk.primary,null,null,lnk.linkId);
            if (trace) lnk.traceLink("TMcaLink.callback", "MCA element is now dependent on link "+lnk.primary.linkId);
          }
          else
          {
            lnk.setBoundLink(null);
            if ((sub=lnk.getSubscription()) == null) 
            {
              if (trace) lnk.traceLink("TMcaLink.callback", "has null subscription !");
              continue;
            }
            sub.mode = link.getSubscription().mode;
            sub.linkLastTime = 0;
            if (trace) lnk.traceLink("TMcaLink.callback", "mode changed to "+TMode.toString(sub.mode));
          }
        }
        if (link.dependents != null) link.dependents.clear();
        link.close();
//        TLinkFactory.rmvMcaLink(this);
        return;
      }
      boolean lookForEarlyNotification = link.needsEarlyNotification;
      link.needsEarlyNotification = false;
      TLink lnk = null;
      Iterator<TLink> itr = lnks.values().iterator(); //lnks.iterator(); 
      while (itr.hasNext())
      {
        lnk = itr.next();
        if (lookForEarlyNotification && !lnk.needsEarlyNotification) continue;
        if (TLinkFactory.debugLevel > 0 && lnk.needsEarlyNotification) 
          DbgLog.log("TMcaLink.callback", "early MCA element notification for "+lnk.getFullDeviceNameAndProperty());
        lnk.needsEarlyNotification = false;
        if (lnk.sub != null && lnk.sub.mode == TMode.CM_CANCEL)
        { // canceled in a callback ?
          itr.remove();
          continue;
        }
        trace = TLinkFactory.traceKey != null && lnk.isTraceLink();
        if (getMcaValues(lnk,link))
        {
          lnk.notifyPending = false;
          if (lnk.isGrouped())
          { 
            if (lnk.getGroup().getNumberPending() > 0) continue;
            if (TLinkFactory.debugLevel > 0) DbgLog.log("TMcaLink.callback","all members of group have updated");
            lnk.getGroup().reset();
          }
          lnk.notifyPending = true;
          if (TLinkFactory.debugLevel > 3) DbgLog.log("TMcaLink.callback", "fire callback for link "+lnk.getFullDeviceName()+" (id "+lnk.linkId+")");
          //TLinkFactory.getInstance().fireCallbackEvent("TMcaLink", lnk);        
          if (lnk.hasDependents())
          { // chained dependencies ? -> do them first
            if (trace) lnk.traceLink("TMcaLink.callback", "run thru dependency list");
            TLink xlnk;
            TDataType xlnkData;
            ConcurrentLinkedQueue<TLink> xlst = lnk.getDependents();
            Iterator<TLink> ditr = xlst.iterator();
            while (ditr.hasNext())
            {
              xlnk = ditr.next();
              if (xlnk == null || xlnk.sub == null) continue;
              if (!xlnk.active) continue;
              if ((xlnkData=xlnk.getOutputDataObject()) != null)
              {
                xlnkData.dataCopy(lnk.dOutput);
                xlnkData.setDataTimeStamp(lnk.dOutput.getDataTimeStamp());
              }                          
              xlnk.sub.linkLastTime = lnk.sub.linkLastTime;
              xlnk.linkStatus = lnk.linkStatus;
              xlnk.linkStatusSource = link.linkStatusSource;
              xlnk.linkStatusStringBuffer = link.linkStatusStringBuffer;
              if (trace) lnk.traceLink("TMcaLink.callback", "fire dependent callback for link "+xlnk.linkId);
              getLinkFactory().fireCallbackEvent("TMcaLink",xlnk);
            }
          }
          if (!lnk.isCancelledWithDependents())
          {
            if (trace) lnk.traceLink("TMcaLink.callback", "firing single element callback for link "+lnk.linkId);
            getLinkFactory().fireCallbackEvent("TMcaLink",lnk);
          }        
        }
        else
        { // index invalid ? => remove this 
          itr.remove();
        }
      }
    }
    // outside the 'synchronized' as 'remove()' also uses this ...
    for (TLink lnk : lnks.values()) if (lnk.mcaRemovePending) remove(lnk);
  }
  public int hashCode()
  {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((ctx == null) ? 0 : ctx.hashCode());
    result = prime * result + ((prp == null) ? 0 : prp.hashCode());
    result = prime * result + ((srv == null) ? 0 : srv.hashCode());
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    TMcaLink other = (TMcaLink) obj;
    if (ctx == null)
    {
      if (other.ctx != null) return false;
    }
    else if (ctx.compareToIgnoreCase(other.ctx) != 0) return false;
    if (prp == null)
    {
      if (other.prp != null) return false;
    }
    else if (prp.compareToIgnoreCase(other.prp) != 0) return false;
    if (srv == null)
    {
      if (other.srv != null) return false;
    }
    else if (srv.compareToIgnoreCase(other.srv) != 0) return false;
    return true;
  }
}
