package de.desy.tine.console;

import java.awt.BorderLayout;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;

import de.desy.tine.client.TLinkFactory;
import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.equipment.TEquipmentModuleFactory;
import de.desy.tine.server.logger.DbgLog;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.startup.TInitializer;
import de.desy.tine.startup.TInitializerFactory;

public class TConsole
{
  private PrintStream stdout;
  private PrintStream stderr;
  private PrintStream prevOut;
  private PrintStream prevErr;
  private JFrame consoleFrame;
  private JTextArea textArea;
  private JScrollPane scrollPane;
  private boolean isShowing = false;
  public boolean isShowing() { return isShowing; }
  private static TConsole instance = new TConsole();
  public static TConsole getInstance() { return instance; }
  private TConsole()
  {
    storeDefaultStreamReferences();
    createOutputStreams();
    createOutputWindow();
  }
  private boolean isSetLocally = false;
  public void show()
  {
    System.setOut(stdout);
    System.setErr(stderr);
    int dl = TLinkFactory.getOutputDebugLevel();
    isSetLocally = true;
    if (dl >= 0 && dl < dbgButton.length) dbgButton[dl].setSelected(true);
    consoleFrame.setVisible(true);
    isShowing = true;
  }
  public void hide()
  {
    restoreStreams();
    consoleFrame.setVisible(false);
    isShowing = false;
  }
  public void setDebugLevel(int debuglevel)
  {
    if (debuglevel >= 0 && debuglevel < dbgButton.length)
    {
      if (!isShowing) isSetLocally = false;
      dbgButton[debuglevel].setSelected(true);
    }
  }
  private static final int MAX_TEXT_WINDOW_SIZE = 65000;
  private static final int TEXT_WINDOW_CHOP_SIZE = 5000;
  private static int maxTextWindowSize = MAX_TEXT_WINDOW_SIZE;
  public static int getMaxTextWindowSize()
  {
    TLinkFactory.dbgPrint("current text window size: "+maxTextWindowSize+" characters");
    return maxTextWindowSize;
  }
  public static void setMaxTextWindowSize(int value)
  {
    if (value == 0) value = MAX_TEXT_WINDOW_SIZE;
    if (value < 2*TEXT_WINDOW_CHOP_SIZE) value = 2*TEXT_WINDOW_CHOP_SIZE;
    maxTextWindowSize = value;
    TLinkFactory.dbgPrint("current text window size: "+maxTextWindowSize+" characters");
  }
  private void createOutputStreams()
  {
    OutputStream out = new OutputStream()
    {
      public void write(final int b) throws IOException
      {
        String str = null;
        try
        {
          str = new String(new int[] {b}, 0, 1);
        }
        catch (IllegalArgumentException e)
        {
          str = "*";
        }
        final String s = str;
        SwingUtilities.invokeLater(new Runnable(){
            public void run() {
                textArea.append(s);
                if ( b == '\n')
                {
                  textArea.setCaretPosition(textArea.getDocument().getLength());
                  //scrollPane.getVerticalScrollBar().setValue(scrollPane.getVerticalScrollBar().getMaximum());
                  String txt = textArea.getText();
                  int len = txt.length();
                  if (len > maxTextWindowSize)
                  { // don't let this grow too big !
                   textArea.setText(txt.substring(TEXT_WINDOW_CHOP_SIZE, len));
                  }
                }
            }
        });
      }
    };
    stdout = new PrintStream(out);
    OutputStream err = new OutputStream()
    {
      public void write(final int b) throws IOException
      {
        final String str = new String(new int[] {b}, 0, 1);
        SwingUtilities.invokeLater(new Runnable(){
            public void run() {
                textArea.append(str);
                if ( b == '\n')
                  textArea.setCaretPosition(textArea.getDocument().getLength());
                  //scrollPane.getVerticalScrollBar().setValue(scrollPane.getVerticalScrollBar().getMaximum());
            }
        });
      }
    };
    stderr = new PrintStream(err);
  }
  private JButton clientsButton = null;
  JButton getClientsButton()
  {
    if (clientsButton == null)
    {
      clientsButton = new JButton("clients");
      clientsButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          //TEquipmentModuleFactory.getInstance().dumpClientTable();
          TLinkFactory.dumpClientTable();
        }
      });
    }
    return clientsButton;
  }
  private JButton contractsButton = null;
  JButton getContractsButton()
  {
    if (contractsButton == null)
    {
      contractsButton = new JButton("contracts");
      contractsButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          TLinkFactory.dumpContractTable();
        }
      });
    }
    return contractsButton;
  }
  private JButton connectionsButton = null;
  JButton getConnectionsButton()
  {
    if (connectionsButton == null)
    {
      connectionsButton = new JButton("connections");
      connectionsButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          TLinkFactory.dumpLinkTable();
        }
      });
    }
    return connectionsButton;
  }
  private JButton messagesButton = null;
  JButton getMessagesButton()
  {
    if (messagesButton == null)
    {
      messagesButton = new JButton("messages");
      messagesButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          MsgLog.MsgEntry[] msgs = MsgLog.getMessages(25);
          System.out.println("\nLast 25 Messages:\n");
          String msg;
          for (MsgLog.MsgEntry m: msgs)
          {
            msg = m.getMsg();
            if (m.getCode() != 0) msg += " ("+TErrorList.getErrorString(m.getCode())+")";
            System.out.println(TDataTime.toString(m.getTime())+" "+msg);
          }
        }
      });
    }
    return messagesButton;
  }
  private JButton cmdButton = null;
  JButton getFilterButton()
  {
    if (cmdButton == null)
    {
      cmdButton = new JButton("command:");
      cmdButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          String text = getCmdPane().getText();
          if (text == null || text.length() == 0) text = "help";
          TCommandList.parseCommand(text);
        }
      });
    }
    return cmdButton;
  }
  private JEditorPane cmdPane = null;
  JEditorPane getCmdPane()
  {
    if (cmdPane == null)
    {
      cmdPane = new JEditorPane();
      cmdPane.setBackground(SystemColor.control);
      //cmdPane.setSize(new Dimension(256,25));
      //cmdPane.setMaximumSize(new Dimension(256,25));
      cmdPane.setToolTipText("enter command (empty = help)");
      cmdPane.addKeyListener(new KeyListener()
      {
        public void keyPressed(KeyEvent e)
        {
        }
        public void keyReleased(KeyEvent e)
        {
        }
        public void keyTyped(KeyEvent e)
        {
          int idx;
          boolean keep = false;
          if (e.getKeyChar() == '\n')
          {
            String text = getCmdPane().getText().trim();
            if ((idx=text.indexOf('\n')) >= 0)
            {
              text = text.substring(0, idx)+text.substring(idx+1);
            }
            if (text.length() == 0) { text = "help"; keep = true; }
            TCommandList.parseCommand(text);
            if (!keep) getCmdPane().setText(text);
          }
        }
      });
    }
    return cmdPane;
  }
  private ButtonGroup dbgGroup = null;
  private JRadioButton[] dbgButton = null;
  private JRadioButton getDebugButton(final int index)
  {
    if (dbgButton == null)
    {
      dbgButton = new JRadioButton[5];
      dbgGroup = new ButtonGroup();
    }
    if (dbgButton[index] == null)
    {
      dbgButton[index] = new JRadioButton("" + (index > 0 ? new String(""+index) : "off"));
      dbgButton[index].addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          if (isSetLocally) TLinkFactory.setOutputDebugLevel(index);
          if (!isShowing) isSetLocally = false;
        }
      });
      dbgGroup.add(dbgButton[index]);
    }
    return dbgButton[index];
  }
  private JLabel dbgLabel;
  private JToolBar toolBar = null;
  private JToolBar getToolBar()
  {
    if (toolBar == null)
    {
      toolBar = new JToolBar();
      toolBar.add(getClientsButton());
      toolBar.add(getContractsButton());
      toolBar.add(getConnectionsButton());
      toolBar.add(getMessagesButton());
      dbgLabel = new JLabel();
      dbgLabel.setText("debug level:");
      toolBar.add(dbgLabel);
//      toolBar.add(getClientsButton());
      toolBar.add(getDebugButton(0));
      toolBar.add(getDebugButton(1));
      toolBar.add(getDebugButton(2));
      toolBar.add(getDebugButton(3));
      toolBar.add(getDebugButton(4));
      toolBar.add(getFilterButton());
      toolBar.add(getCmdPane());
    }
    return toolBar;
  }
  public void addWindowListener(WindowListener l)
  {
    if (consoleFrame != null) consoleFrame.addWindowListener(l);
  }
  public void removeWindowListener(WindowListener l)
  {
    if (consoleFrame != null) consoleFrame.removeWindowListener(l);
  }
  private OperatingSystemMXBean os = ManagementFactory.getOperatingSystemMXBean();
  private void createOutputWindow()
  {
    textArea = new JTextArea();
    textArea.setRows(10);
    String text = TLinkFactory.isRunningAsServer() ?
        TEquipmentModuleFactory.getInstance().getFecName() :
          "Application";
    if (text == null) text = "";
    TInitializer ti = TInitializerFactory.getInstance().getInitializer();
    consoleFrame = new JFrame(text + " Debugging console (OS: "+os.getName()+" "+os.getVersion()+"; TINE: "+ti.getVersion()+":"+ti.getBuildId()+")");
    consoleFrame.setLayout(new BorderLayout());
    scrollPane = new JScrollPane(textArea);
    consoleFrame.add(scrollPane, BorderLayout.CENTER);
    consoleFrame.addWindowListener(new ConsoleWindowListener());
    consoleFrame.setBounds(0, 0, 750, 750);
    consoleFrame.add(getToolBar(), BorderLayout.NORTH);
  }
  private void restoreStreams()
  {
    System.setOut(prevOut);
    System.setErr(prevErr);
    DbgLog.setFilter(null);
  }
  private void storeDefaultStreamReferences()
  {
    prevOut = System.out;
    prevErr = System.err;
  }
  private class ConsoleWindowListener implements WindowListener
  {
    public void windowOpened(WindowEvent e)
    {
    }
    public void windowClosing(WindowEvent e)
    {
      restoreStreams();
      isShowing = false;
      isSetLocally = false;
    }
    public void windowClosed(WindowEvent e)
    {
    }
    public void windowIconified(WindowEvent e)
    {
    }
    public void windowDeiconified(WindowEvent e)
    {
    }
    public void windowActivated(WindowEvent e)
    {
    }
    public void windowDeactivated(WindowEvent e)
    {
    }
  }
}
