package de.desy.tine.dataUtils;

/**
 * Utility class to perform common unsigned conversion algorithms.
 * 
 * @author David Melkumyan, DESY Zeuthen
 * @since 08.2009
 */
public class UnsignedUtils
{
  /**
   * Converts the specified int into an unsigned long.<br>
   * The least significant int (32 bits) of the long will be identical to the
   * int (32 bits) provided, and the most significant int (32 bits) of the long
   * will be zero.
   * 
   * @param input
   *          - the int to convert.
   * @return An unsigned long representing the specified int.
   */
  public static long toUnsignedInt(int input)
  {
    return input & 0xFFFFFFFFL;
  }
  /**
   * Converts the specified short into an unsigned integer.<br>
   * The least significant short (16 bits) of the integer will be identical to
   * the short (16 bits) provided, and the most significant 2 bytes (16 bits) of
   * the integer will be zero.
   * 
   * @param input
   *          - the short to convert.
   * @return An unsigned int representing the specified short.
   */
  public static int toUnsignedShort(short input)
  {
    return input & 0xFFFF;
  }
  /**
   * Converts the specified byte into an unsigned short.<br>
   * The least significant byte (8 bits) of the short will be identical to the
   * byte (8 bits) provided, and the most significant byte (8 bits) of the short
   * will be zero.
   * 
   * @param input
   *          - the byte to convert.
   * @return An unsigned short representing the specified byte.
   */
  public static short toUnsignedByte(byte input)
  {
    return (short) (input & 0xFF);
  }
}
