/*
 * Created on Feb 1, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.endianUtils;

/**
 * \internal 
 * 
 * New utilities for big endian / little endian conversion. This is an
 * Extension/Replacement for the class Swap. Adds swap utilities for int,long,
 * double, and float which don't need intermediate byte arrays. These utilities
 * are packed into a separate class, because they would disturb Swap (where long
 * means int) otherwise.
 * 
 * @author jwlg
 */
public class SwapUtil
{
  /**
   * Swaps a short integer byte-wise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static short swapShort(short value)
  {
    return (short) (((value >> 8) & 0xff) | ((value & 0xff) << 8));
  }
  /**
   * Swaps an integer byte-wise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static int swapInt(int value)
  {
    return Integer.reverseBytes(value);
  }
  /**
   * Swaps a long integer bytewise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static long swapLong(long value)
  {
    return Long.reverseBytes(value);
  }
  /**
   * Swaps a double bytewise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static double swapDouble(double value)
  {
    return Double.longBitsToDouble(swapLong(Double.doubleToLongBits(value)));
  }
  public static double swapDouble(long value)
  {
    return Double.longBitsToDouble(swapLong(value));
  }
  /**
   * Swaps a float bytewise.
   * 
   * @param value
   *          Input value.
   * @return Byte-swapped value.
   */
  public static float swapFloat(float value)
  {
    return Float.intBitsToFloat(swapInt(Float.floatToIntBits(value)));
  }
  public static float swapFloat(int value)
  {
    return Float.intBitsToFloat(swapInt(value));
  }
  /**
   * Unusable Constructor; pure utility class
   */
  private SwapUtil()
  {
  }
}