package de.desy.tine.eventArchive;

import java.util.Arrays;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.structUtils.TTaggedStructure;

public class DASQS extends TTaggedStructure
{ // DASQS
  public static final int READ = 0x01;
  public static final int WRITE = 0x02;
  public static final int MATCH = 0x04;
  public static final int WAIT = 0x08;
  public static final int SETNUM = 0x10;   /* acquire output data size for subsequent call */
  public static final int PREALLOC = 0x20;
  public static final int FORWARD = 0x40;   /* acquire input data for subsequent calls */
  public static final int SERIES = 0x80;
  public static final int MONITOR = 0x100;
  public static final int PERSIST = 0x200;  /* applicable in combo with SETNUM */
  public static final int SETDEV = 0x400;  /* acquire device list and use for subsequent calls */
  public static final int UNSETDEV = 0x800;  /* clear the setting of the acquired device list */
  public static final int DISABLE = 0x4000; /* disable this archive trigger step */
  public static String getAccessString(int access)
  {
    StringBuilder sb = new StringBuilder();
    if ((access & READ) != 0) sb.append("|READ");
    if ((access & WRITE) != 0) sb.append("|WRITE");
    if ((access & MATCH) != 0) sb.append("|MATCH");
    if ((access & WAIT) != 0) sb.append("|WAIT");
    if ((access & SETNUM) != 0) sb.append("|SETNUM");
    if ((access & PREALLOC) != 0) sb.append("|PREALLOC");
    if ((access & FORWARD) != 0) sb.append("|FORWARD");
    if ((access & SERIES) != 0) sb.append("|SERIES");
    if ((access & MONITOR) != 0) sb.append("|MONITOR");
    if ((access & PERSIST) != 0) sb.append("|PERSIST");
    if ((access & SETDEV) != 0) sb.append("|SETDEV");
    if ((access & UNSETDEV) != 0) sb.append("|UNSETDEV");
    if ((access & DISABLE) != 0) sb.append("|DISABLED");
    return sb.length() == 0 ? "" : sb.substring(1);
  }
  public String getAccessString()
  {
    return getAccessString(access[0]);
  }
  public static int getAccessCode(String accessString)
  {
    if (accessString == null || accessString.length() == 0) return 0;
    String as = accessString.toUpperCase();
    int ac = 0;
    if (as.contains("READ")) ac |= READ;
    if (as.contains("WRITE")) ac |= WRITE;
    if (as.contains("MATCH")) ac |= MATCH;
    if (as.contains("WAIT")) ac |= WAIT;
    if (as.contains("SETNUM")) ac |= SETNUM;
    if (as.contains("PREALLOC")) ac |= PREALLOC;
    if (as.contains("FORWARD")) ac |= FORWARD;
    if (as.contains("SERIES")) ac |= SERIES;
    if (as.contains("MONITOR")) ac |= MONITOR;
    if (as.contains("PERSIST")) ac |= PERSIST;
    if (as.contains("UNSETDEV")) ac |= UNSETDEV;
    else if (as.contains("SETDEV")) ac |= SETDEV;
    if (as.contains("DISABLE")) ac |= DISABLE;
    return ac;
  }
  public void setAccess(String value)
  {
    access[0] = getAccessCode(value);
  }
  public void clear()
  {
    ctxStr = null;
    srvStr = null;
    prpStr = null;
    devStr = null;
    srvArcStr = null;
    prpArcStr = null;
    infileStr = null;
    datTagStr = null;
    datTagInStr = null;
  }
  private char[] ctx = new char[TStrings.CONTEXT_NAME_SIZE];
  private String ctxStr = null;
  private char[] srv = new char[TStrings.EXPORT_NAME_SIZE];
  private String srvStr = null;
  private char[] prp = new char[TStrings.PROPERTY_NAME_SIZE];
  private String prpStr = null;
  private char[] dev = new char[TStrings.DEVICE_NAME_SIZE];
  private String devStr = null;
  private char[] srvArc = new char[TStrings.EXPORT_NAME_SIZE];
  private String srvArcStr = null;
  private char[] prpArc = new char[TStrings.PROPERTY_NAME_SIZE];
  private String prpArcStr = null;
  private char[] infile = new char[16];
  private String infileStr = null;
  private char[] datTag = new char[TStrings.TAG_NAME_SIZE];
  private String datTagStr = null;
  private char[] datTagIn = new char[TStrings.TAG_NAME_SIZE];
  private String datTagInStr = null;
  int[] datSiz = new int[1];
  int[] datFmt = new int[1];
  int[] datFmtIn = new int[1];
  int[] access = new int[1];
  int[] wait = new int[1];
  int[] timeout = new int[1];
  int[] value = new int[1];
  int[] mode = new int[1];
  int[] monitorRate = new int[1]; /* milliseconds */
  int[] monitorTime = new int[1]; /* seconds */
  float[] scale = new float[1];
  float[] shift = new float[1];
  private void initStructure()
  {
    addField(ctx,"ctx");
    addField(srv,"srv");
    addField(prp,"prp");
    addField(dev,"dev");
    addField(srvArc,"srvArc");
    addField(prpArc,"prpArc");
    addField(infile,"infile");
    addField(datTag,"datTag");
    addField(datTagIn,"datTagIn");
    addField(datSiz,"datSiz");
    addField(datFmt,"datFmt");
    addField(datFmtIn,"datFmtIn");
    addField(access,"access");
    addField(wait,"wait");
    addField(timeout,"timeout");
    addField(value,"value");
    addField(mode,"mode");
    addField(monitorRate,"monitorRate");
    addField(monitorTime,"monitorTime");
    addField(scale,"scale");
    addField(shift,"shift");    
    initDone();    
  }
  public DASQS()
  {
    super();
    initStructure();
  }
  public String getContext()
  {
    if (ctxStr == null) ctxStr = new String(ctx).trim();
    return ctxStr;
  }
  public void setContext(String newCtx)
  {
    pushChars(newCtx, ctx);
    ctxStr = new String(newCtx);
  }
  public String getServer()
  {
    if (srvStr == null) srvStr = new String(srv).trim();
    return srvStr;
  }
  public void setServer(String newSrv)
  {
    pushChars(newSrv, srv);
    srvStr = new String(newSrv);
  }
  public String getServerArchiveName()
  {
    if (srvArcStr == null) srvArcStr = new String(srvArc).trim();
    return srvArcStr;
  }
  public void setServerArchiveName(String newSrv)
  {
    pushChars(newSrv, srvArc);
    srvArcStr = new String(newSrv);
  }
  public String getProperty()
  {
    if (prpStr == null) prpStr = new String(prp).trim();
    return prpStr;
  }
  public void setProperty(String newPrp)
  {
    pushChars(newPrp, prp);
    prpStr = new String(newPrp);
  }
  public String getPropertyArchiveName()
  {
    if (prpArcStr == null) prpArcStr = new String(prpArc).trim();
    return prpArcStr;
  }
  public void setPropertyArchiveName(String newPrp)
  {
    pushChars(newPrp, prpArc);
    prpArcStr = new String(newPrp);
  }
  public String getDevice()
  {
    if (devStr == null) devStr = new String(dev).trim();
    return devStr;
  }
  public void setDevice(String newDev)
  {
    pushChars(newDev, dev);
    devStr = new String(newDev);
  }
  public String getInputFileName()
  {
    if (infileStr == null) infileStr = new String(infile).trim();
    return infileStr;
  }
  public void setInputFileName(String newName)
  {
    pushChars(newName, infile);
    infileStr = new String(newName);
  }
  public String getDataTag()
  {
    if (datTagStr == null) datTagStr = new String(datTag).trim();
    return datTagStr;
  }
  public void setDataTag(String newTag)
  {
    pushChars(newTag, datTag);
    datTagStr = new String(newTag);
  }
  public String getInputDataTag()
  {
    if (datTagInStr == null) datTagInStr = new String(datTagIn).trim();
    return datTagInStr;
  }
  public void setInputDataTag(String newTag)
  {
    pushChars(newTag, datTagIn);
    datTagInStr = new String(newTag);
  }
  public int getDataSize() { return datSiz[0]; }
  public void setDataSize(int value) { datSiz[0] = value; }
  public int getDataFormat() { return datFmt[0]; }
  public void setDataFormat(int value) { datFmt[0] = value; }
  public int getInputDataFormat() { return datFmtIn[0]; }
  public void setInputDataFormat(int value) { datFmtIn[0] = value; }
  public int getAccess() { return access[0]; }
  public void setAccess(int value) { access[0] = value; }
  public int getWait() { return wait[0]; }
  public void setWait(int value) { wait[0] = value; }
  public int getTimeout() { return timeout[0]; }
  public void setTimeout(int value) { timeout[0] = value; }
  public int getValue() { return value[0]; }
  public void setValue(int value) { this.value[0] = value; }
  public int getMode() { return mode[0]; }
  public void setMode(int value) { mode[0] = value; }
  public int getMonitorRate() { return monitorRate[0]; }
  public void setMonitorRate(int value) { monitorRate[0] = value; }
  public int getMonitorTime() { return monitorTime[0]; }
  public void setMonitorTime(int value) { monitorTime[0] = value; }
  public float getScale() { return scale[0]; }
  public void setScale(float value) { scale[0] = value; }
  public float getShift() { return shift[0]; }
  public void setShift(float value) { shift[0] = value; }  
}
