package de.desy.tine.eventUtils;

import de.desy.tine.client.*;
import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.*;
import java.io.*;
import java.util.Arrays;
import java.util.Date;

/**
 * Obtains information from the specified Archive Server
 */
public final class TEventArchive
{
  /**
   * @deprecated
   * @param ArchiveServerContext
   * @param DeviceServer
   * @param DeviceProperty
   * @param DeviceName
   * @param eventNumber
   * @param startTime
   * @param stopTime
   * @param data
   * @return
   */
  public static int getArchiveData(String archiveServerContext, String deviceServer, String deviceProperty,
      String deviceName, int eventNumber, int startTime, int stopTime, TDataType data)
  {
    adrb5 = new ADRB5();
    adrb5.deviceName = deviceName;
    adrb5.deviceServerName = deviceServer;
    adrb5.deviceProperty = deviceProperty;
    adrb5.starttime = startTime;
    adrb5.stoptime = stopTime;
    adrb5.eventNumber = eventNumber;
    d_adrb = new TDataType(adrb5.toByteArray(), "ADRB");
    if (arc != null) arc.cancel();
    String dev = new String("/" + archiveServerContext + "/" + "EVENTS");
    arc = new TLink(dev, "StoredData", data, d_adrb, (short) 1);
    statusCode = arc.execute(3000);
    status = arc.getLastError();
    arc.cancel();
    return statusCode;
  }
  public static int statusCode;
  public static String status = "undefined";
  private static int[] timeRange = new int[2];
  static int[] nevents = new int[1];
  static short[] ndservers = new short[1];
  static short[] naservers = new short[1];
  static short[] nadevices = new short[1];
  static short[] naproperties = new short[1];
  @SuppressWarnings("deprecation")
  static ADRB5 adrb5;
  static ADRBX adrbx;
  static ADRB adrb;
  static AIB aib;
  static PIB pib;
  static AEScontext aescon;
  static DHS dhs;
  static AES[] aes;
  static int[] evnlst;
  static NAME16[] aservers;
  static NAME16[] dservers;
  static NAME32[] dservers32;
  static NAME64[] dservers64;
  static NAME16[] adevices;
  static NAME64[] adevices64;
  static NAME16[] aproperties;
  static NAME64[] aproperties64;
  static NAME16I[] devevn;
  static TDataType d_null = new TDataType();
  static TDataType d_nevent;
  static TDataType d_range;
  static TDataType d_data;
  static TDataType d_adrb;
  static TDataType d_adrbx;
  static TDataType d_aib;
  static TDataType d_pib;
  static TDataType d_aescon;
  static TDataType d_aes;
  static TDataType d_dhs;
  static TDataType d_ndservers;
  static TDataType d_dservers;
  static TDataType d_nadevices;
  static TDataType d_adevices;
  static TDataType d_naproperties;
  static TDataType d_aproperties;
  static TDataType d_device;
  static TDataType d_devevn;
  static TDataType d_evn;
  static TDataType d_evnlst;
  static TLink arc, ah, ai, pi, nds, ds, nad, ad, nap, ap, nevn, evn;
  public static int getArchiveData(String eventTrigger, String archiveServerContext, String deviceContext,
      String deviceServer, String deviceProperty, String deviceName, int dataIndex, int eventNumber,
      int startTime, int stopTime, TDataType data)
  {
    adrb = new ADRB();
    adrb.deviceContext = deviceContext;
    adrb.deviceName = deviceName;
    adrb.deviceServerName = deviceServer;
    adrb.deviceProperty = deviceProperty;
    adrb.deviceTag = data.getTag();
    adrb.dataIndex = dataIndex;
    adrb.starttime = startTime;
    adrb.stoptime = stopTime;
    adrb.eventNumber = eventNumber;
    TDataType dt = new TDataType(adrb.toByteArray(), "ADRBr4");
    if (eventTrigger == null) eventTrigger = "";
    String dev = "/"+archiveServerContext+"/EVENTS/"+eventTrigger;
    arc = new TLink(dev, "StoredData", data, dt, TAccess.CA_READ);
    statusCode = arc.execute(3000);
    status = arc.getLastError();
    arc.close();
    return statusCode;
  }
  public static int getArchiveDataAndCloseSession(String eventTrigger, String archiveServerContext, String deviceContext,
      String deviceServer, String deviceProperty, String deviceName, int dataIndex, int eventNumber,
      int startTime, int stopTime, TDataType data)
  {
    int cc = getArchiveData(eventTrigger,archiveServerContext,deviceContext,deviceServer,deviceProperty,deviceName,dataIndex,eventNumber,startTime,stopTime,data);
    if (cc != 0) return cc;
    return closeArchiveSession(eventTrigger,archiveServerContext,eventNumber);
  }
  public static int closeArchiveSession(String eventTrigger,String archiveServerContext,int eventNumber)
  { // devName = trigger, input = event number
    if (eventTrigger == null || eventTrigger.length() == 0) return TErrorList.invalid_parameter;
    if (archiveServerContext == null || archiveServerContext.length() == 0)  return TErrorList.invalid_parameter;
    TDataType dt = new TDataType(new int[]{eventNumber});
    String dev = new String("/" + archiveServerContext + "/" + "EVENTS/" + eventTrigger);
    TLink lnk = new TLink(dev, "CloseArchive", null, dt, TAccess.CA_READ);
    return lnk.executeAndClose();
  }
  public static int getArchiveData(String eventTrigger, String archiveServerContext, String deviceContext,
      String deviceServer, String deviceProperty, String deviceName, int dataIndex, int eventNumber, TDataType data)
  {
    return getArchiveData(eventTrigger,archiveServerContext,
        deviceContext,deviceServer,deviceProperty,deviceName,
                          dataIndex,eventNumber,0,0,data);  
  }
  public static int getArchiveDataAndCloseSession(String eventTrigger, String archiveServerContext, String deviceContext,
      String deviceServer, String deviceProperty, String deviceName, int dataIndex, int eventNumber, TDataType data)
  {
    int cc = getArchiveData(eventTrigger,archiveServerContext,
        deviceContext,deviceServer,deviceProperty,deviceName,
                          dataIndex,eventNumber,0,0,data);
    if (cc != 0) return cc;
    return closeArchiveSession(eventTrigger,archiveServerContext,eventNumber);
  }
  public static DHS getArchiveHeader(String archiveServerContext, String deviceServer, String deviceProperty,
      String deviceName, int eventNumber, int startTime, int stopTime)
  {
    String ctx = archiveServerContext, srv = deviceServer;
    if (deviceServer.startsWith("/"))
    {
      String[] parts = deviceServer.split("/");
      if (parts.length > 2)
      {
        ctx = parts[1]; srv = parts[2];
      }
    }
    return getArchiveHeader(null,archiveServerContext,ctx,srv,deviceProperty,deviceName,0,eventNumber,startTime,stopTime);
  }
  public static DHS getArchiveHeader(String trigger, String archiveServerContext, String deviceServer, String deviceProperty,
      String deviceName, int eventNumber, int startTime, int stopTime)
  {
    String ctx = archiveServerContext, srv = deviceServer;
    if (deviceServer.startsWith("/"))
    {
      String[] parts = deviceServer.split("/");
      if (parts.length > 2)
      {
        ctx = parts[1]; srv = parts[2];
      }
    }
    return getArchiveHeader(trigger,archiveServerContext,ctx,srv,deviceProperty,deviceName,0,eventNumber,startTime,stopTime);
  }
  public static DHS getArchiveHeader(String trigger, String archiveServerContext,
      String deviceContext, String deviceServer, String deviceProperty, String DeviceName, int dataIndex,
      int eventNumber, int startTime, int stopTime)
  {
    dhs = new DHS();
    d_dhs = new TDataType(dhs.toByteArray(), "DHS");
    
    adrb = new ADRB();
    adrb.deviceContext = deviceContext;
    adrb.deviceName = DeviceName;
    adrb.deviceServerName = deviceServer;
    adrb.deviceProperty = deviceProperty;
    adrb.dataIndex = dataIndex;
    adrb.starttime = startTime;
    adrb.stoptime = stopTime;
    adrb.eventNumber = eventNumber;
    TDataType dt = new TDataType(adrb.toByteArray(), "ADRBr4");
    
    String dev = new String("/" + archiveServerContext + "/" + "EVENTS");
    if (trigger != null) dev += "/" + trigger;
    ah = new TLink(dev, "DataHeader", d_dhs, dt, (short) 1);
    statusCode = ah.execute(3000);
    status = ah.getLastError();
    ah.close();
    status = ah.getLastError();
    if (statusCode != 0) return null;
    dhs.toStruct();
    return dhs;
  }
  public static AIB getArchiveInfo(String archiveServerContext, String deviceServer, int eventNumber)
  {
    aescon = new AEScontext();
    aescon.deviceServerName = deviceServer;
    aescon.eventNumber = eventNumber;
    d_aescon = new TDataType(aescon.toByteArray(), "AESCON");
    aib = new AIB();
    d_aib = new TDataType(aib.toByteArray(), "AIB");
    if (ai != null) ai.close();
    String dev = new String("/" + archiveServerContext + "/" + "EVENTS");
    ai = new TLink(dev, "ArchiveInformation", d_aib, d_aescon, (short) 1);
    if ((statusCode = ai.execute(3000)) == 0)
    {
      aib.toStruct();
      ai.close();
      return aib;
    }
    status = ai.getLastError();
    ai.close();
    return null;
  }
  public static PIB getPropertyInfo(String archiveServerContext, String deviceServer, String deviceProperty,
      int eventNumber)
  {
    aescon = new AEScontext();
    aescon.deviceServerName = deviceServer;
    aescon.eventNumber = eventNumber;
    d_aescon = new TDataType(aescon.toByteArray(), "AESCON");
    pib = new PIB();
    d_pib = new TDataType(pib.toByteArray(), "PIB");
    if (pi != null) pi.close();
    String dev = new String("/" + archiveServerContext + "/" + "EVENTS");
    pi = new TLink(dev, "PROPINFO." + deviceProperty, d_pib, d_aescon, (short) 1);
    if ((statusCode = pi.execute(3000)) == 0)
    {
      pib.toStruct();
      pi.close();
      return pib;
    }
    status = pi.getLastError();
    pi.close();
    return null;
  }
  public static int getNumberOfDeviceServers(String context)
  {
    d_ndservers = new TDataType(ndservers);
    if (nds != null) nds.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    nds = new TLink(dev, "NumStoredServers", d_ndservers, d_null, TAccess.CA_READ);
    if ((statusCode = nds.execute(1000)) == 0)
    {
      nds.close();
      return ndservers[0];
    }
    status = nds.getLastError();
    nds.close();
    return -1;
  }
  /**
   * @deprecated use String[] getDeviceServers(String context,String trigger)
   * or String[] getDeviceServers(String context,String trigger,int eventTime)
   * @param context is the archive server context
   * @return
   */
  public static NAME16[] getDeviceServers(String context)
  {
    return getDeviceServers(context,"ALL",0,0);  
  }
  public static NAME16[] getDeviceServers_LEGACY(String context,String trigger)
  {
    return getDeviceServers(context,trigger,0,0);
  }
  /**
   * @deprecated use String[] getDeviceServers(String context,String trigger)
   * or String[] getDeviceServers(String context,String trigger,int eventTime)
   * @param context is the archive server context
   * @return
   */
  public static NAME16[] getDeviceServers(String context,int start, int stop)
  {
    return getDeviceServers(context,"ALL",start,stop);
  }
  /**
   * @deprecated use GetDeviceServers
   * @param context
   * @return
   */
  public static String[] pmGetDeviceServers(String context)
  {
    int nds = getNumberOfDeviceServers(context);
    if (nds <= 0) return null;
    dservers = new NAME16[nds];
    for (int i = 0; i < dservers.length; i++)
    {
      dservers[i] = new NAME16();
    }
    d_dservers = new TDataType(dservers);
    if (ds != null) ds.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    ds = new TLink(dev + "/ALL", "StoredServers", d_dservers, d_null, (short) 1);
    if ((statusCode = ds.execute(3000)) == 0)
    {
      ds.close();
      String[] dserversStr = new String[nds];
      for (int i = 0; i < dserversStr.length; i++)
        dserversStr[i] = dservers[i].toString();
      return dserversStr;
    }
    status = ds.getLastError();
    ds.close();
    return null;
  }
  /**
   * @deprecated use String[] getDeviceServers(String context,String trigger)
   * or String[] getDeviceServers(String context,String trigger,int eventTime)
   * @param context is the archive server context
   * @param trigger is the specific trigger used to access the device server list
   * @return
   */
  public static NAME16[] getDeviceServers(String context, String trigger, int start, int stop)
  {
    if (context == null || context.length() == 0) return null;
    if (trigger == null || trigger.length() == 0) trigger = "ALL";
    TDataType tdt;
    if (start > 0)
    {
      if (stop < start) stop = (int) (System.currentTimeMillis()) / 1000;
      timeRange[0] = start;
      timeRange[1] = stop;
      tdt = new TDataType(timeRange);
    }
    else
    {
      tdt = d_null;
    }
    int nds = getNumberOfDeviceServers(context);
    if (nds <= 0) return null;
    dservers = new NAME16[nds];
    for (int i = 0; i < dservers.length; i++)
      dservers[i] = new NAME16();
    d_dservers = new TDataType(dservers);
    if (ds != null) ds.close();
    String dev = new String("/" + context + "/EVENTS/"+trigger);
    ds = new TLink(dev, "StoredServers", d_dservers, tdt, (short) 1);
    if ((statusCode = ds.execute(3000)) == 0)
    {
      ds.close();
      return dservers;
    }
    status = ds.getLastError();
    ds.close();
    return null;
  }
  public static NAME64[] getDeviceServers64(String context, String trigger, int start, int stop)
  {
    if (context == null || context.length() == 0) return null;
    if (trigger == null || trigger.length() == 0) trigger = "ALL";
    TDataType tdt;
    if (start > 0)
    {
      if (stop < start) stop = (int) (System.currentTimeMillis()) / 1000;
      timeRange[0] = start;
      timeRange[1] = stop;
      tdt = new TDataType(timeRange);
    }
    else
    {
      tdt = d_null;
    }
    int nds = getNumberOfDeviceServers(context);
    if (nds <= 0) return null;
    dservers64 = new NAME64[nds];
    for (int i = 0; i < dservers64.length; i++) dservers64[i] = new NAME64();
    d_dservers = new TDataType(dservers64);
    if (ds != null) ds.close();
    String dev = new String("/" + context + "/EVENTS/"+trigger);
    ds = new TLink(dev, "StoredServers", d_dservers, tdt, (short) 1);
    if ((statusCode = ds.execute(3000)) == 0)
    {
      ds.close();
      return dservers64;
    }
    status = ds.getLastError();
    ds.close();
    return null;
  }
  private static int niceNumberQuerySize = 256;
  private static NAME64[] n64 = null;
  private static NAME64[] getName64Array()
  {
    if (n64 == null)
    {
      n64 = new NAME64[niceNumberQuerySize];
      for (int i = 0; i < n64.length; i++) n64[i] = new NAME64();
    }
    return n64;    
  }
  /**
   * @param context is the archive server context
   * @param trigger is the specific trigger used to access the device server list
   * @param eventTime is the specific event of interest.  If eventTime == 0 then the
   * canonical list of servers which are addressed for the specific trigger is returned.
   * If eventTime is non-zero and refers to a specific event, then the list of actual stored
   * servers is returned (which should be equivalent to the canonical server list under
   * most circumstances.
   *  
   * @return the list of device servers associated with the trigger and event time given.
   */
  public static String[] getDeviceServers(String context, String trigger, int eventTime)
  {
    if (context == null || context.length() == 0) return null;
    if (trigger == null || trigger.length() == 0) trigger = "ALL";
    TDataType tdt = eventTime > 0 ? new TDataType(new int[]{eventTime}) : null;
    NAME64[] srvnams = getName64Array();
    String dev = new String("/"+context+"/EVENTS/"+trigger);
    TLink lnk = new TLink(dev, "StoredServers", new TDataType(srvnams), tdt, TAccess.CA_READ);
    if ((statusCode=lnk.executeAndClose(3000)) != 0)
    {
      status = lnk.getLastError();
      return new String[]{""};
    }
    int n = lnk.getOutputDataObject().getCompletionLength();
    String[] srvs = new String[n];
    for (int i=0; i<n; i++) srvs[i] = srvnams[i].getName();
    return srvs;
  }
  /**
   * This method is an API break of its predecessor, which returned an array of
   * NAME16 objects.
   * @see getDeviceServers_LEGACY
   * 
   * @param context is the archive server context
   * @param trigger is the specific trigger used to access the device server list.
   *  
   * @return the list of device servers associated with the trigger.
   */
  public static String[] getDeviceServers(String context, String trigger)
  {
    return getDeviceServers(context,trigger,0);
  }
  public static String[] getDeviceServersAsString(String context, int start, int stop)
  {
    return getDeviceServersAsString(context,"ALL",start,stop);
  }
  public static String[] getDeviceServersAsString(String context, String trigger, int start, int stop)
  {
    return getDeviceServers(context,trigger,0);
  }
  public static String[] getDeviceServersAsFullString(String context, String trigger, int start, int stop)
  {
    return getDeviceServers(context,trigger,0);
  }
  public static String[] getDeviceServersAsString(String context)
  {
    return getDeviceServersAsString(context, 0, 0);
  }
  /**
   * @deprecated use GetDeviceServers
   * @param context
   * @return
   */
  public static String[] pmGetDeviceServers(String context, int start, int stop)
  {
    return pmGetDeviceServers(context,"ALL",start,stop);
  }
  /**
   * @deprecated use GetDeviceServers
   * @param context
   * @param trigger
   * @return
   */
  public static String[] pmGetDeviceServers(String context, String trigger,int start, int stop)
  {
    timeRange[0] = start;
    timeRange[1] = stop;
    int nds = getNumberOfDeviceServers(context);
    if (nds <= 0) return null;
    dservers32 = new NAME32[nds];
    for (int i = 0; i < dservers.length; i++) dservers32[i] = new NAME32();
    if (ds != null) ds.close();
    String dev = new String("/"+context+"/"+"EVENTS"+"/"+trigger);
    ds = new TLink(dev, "StoredServers", new TDataType(dservers32), new TDataType(timeRange), (short) 1);
    if ((statusCode = ds.execute(3000)) == 0)
    {
      ds.close();
      String[] dserversStr = new String[nds];
      for (int i = 0; i < dserversStr.length; i++)
        dserversStr[i] = dservers32[i].toString();
      return dserversStr;
    }
    status = ds.getLastError();
    ds.close();
    return null;
  }
  public static int getNumberOfDevices(String ArchiveServerContext)
  {
    d_nadevices = new TDataType(nadevices);
    String dev = new String("/" + ArchiveServerContext + "/" + "EVENTS");
    nad = new TLink(dev, "NumStoredDevices", d_nadevices, d_null, (short) 1);
    if ((statusCode = nad.execute(3000)) == 0)
    {
      nad.close();
      return nadevices[0];
    }
    status = nad.getLastError();
    nad.close();
    return -1;
  }
  public static int getNumberOfDevices(String ArchiveServerContext, String DeviceServer)
  {
    return getNumberOfDevices(ArchiveServerContext,DeviceServer,0);
  }
  public static int getNumberOfDevices(String ArchiveServerContext, String DeviceServer, int eventNumber)
  {
    if (eventNumber > 0)
    {
      int[] evn = new int[1];
      evn[0] = eventNumber;
      d_evn = new TDataType(evn);
    }
    else
    {
      d_evn = d_null;
    }
    String srv = DeviceServer;
    int idx = -1;
    //if (srv.startsWith("/")) srv = srv.substring(1);
    if (!srv.startsWith("/") && (idx=srv.indexOf(":")) > 0)
      srv = srv.substring(idx+1);
    if ((idx=srv.indexOf("[")) > 0) srv = srv.substring(0, idx);
    d_nadevices = new TDataType(nadevices);
    String dev = new String("/" + ArchiveServerContext + "/" + "EVENTS");
    nad = new TLink(dev+"/"+srv,"NumStoredDevices",d_nadevices,d_evn,TAccess.CA_READ);
    if ((statusCode = nad.executeAndClose(3000)) == 0)
    {
      return nadevices[0];
    }
    status = nad.getLastError();
    return -1;
  }
  public static NAME16[] getDevices(String ArchiveServerContext)
  {
    int nad = getNumberOfDevices(ArchiveServerContext);
    if (nad <= 0) return null;
    adevices = new NAME16[nad];
    for (int i = 0; i < adevices.length; i++)
    {
      adevices[i] = new NAME16();
    }
    d_adevices = new TDataType(adevices);
    if (ad != null) ad.close();
    String dev = new String("/" + ArchiveServerContext + "/" + "EVENTS");
    ad = new TLink(dev + "/ALL", "StoredDevices", d_adevices, d_null, (short) 1);
    if ((statusCode = ad.execute(3000)) == 0)
    {
      ad.close();
      return adevices;
    }
    status = ad.getLastError();
    ad.close();
    return null;
  }
  public static NAME16[] getDevices(String ArchiveServerContext, String DeviceServer)
  {
    int nad = getNumberOfDevices(ArchiveServerContext, DeviceServer);
    if (nad <= 0) return null;
    adevices = new NAME16[nad];
    for (int i = 0; i < adevices.length; i++)
    {
      adevices[i] = new NAME16();
    }
    d_adevices = new TDataType(adevices);
    if (ad != null) ad.close();
    String dev = new String("/" + ArchiveServerContext + "/" + "EVENTS");
    ad = new TLink(dev + "/" + DeviceServer, "StoredDevices", d_adevices, d_null, (short) 1);
    if ((statusCode = ad.execute(3000)) == 0)
    {
      ad.close();
      return adevices;
    }
    status = ad.getLastError();
    ad.close();
    return null;
  }
  public static NAME16[] getDevices(String ArchiveServerContext, String DeviceServer, int eventNumber)
  {
    TDataType tdt;
    int nad = getNumberOfDevices(ArchiveServerContext, DeviceServer);
    if (nad <= 0) return null;
    if (eventNumber > 0)
    {
      int[] evn = new int[1];
      evn[0] = eventNumber;
      d_evn = new TDataType(evn);
      tdt = d_evn;
    }
    else
    {
      tdt = d_null;
    }
    adevices = new NAME16[nad];
    for (int i = 0; i < adevices.length; i++)
    {
      adevices[i] = new NAME16();
    }
    d_adevices = new TDataType(adevices);
    if (ad != null) ad.close();
    String dev = new String("/" + ArchiveServerContext + "/" + "EVENTS");
    ad = new TLink(dev + "/" + DeviceServer, "StoredDevices", d_adevices, tdt, (short) 1);
    if ((statusCode = ad.execute(3000)) == 0)
    {
      ad.close();
      return adevices;
    }
    status = ad.getLastError();
    ad.close();
    return null;
  }
  public static NAME64[] getDevices64(String ArchiveServerContext, String DeviceServer, int eventNumber)
  {
    String srv = DeviceServer;
    if (srv.startsWith("/")) srv = srv.substring(1); 
    int nad = getNumberOfDevices(ArchiveServerContext, DeviceServer);
    if (nad <= 0) return null;
    TDataType tdt;
    if (eventNumber > 0)
    {
      int[] evn = new int[1];
      evn[0] = eventNumber;
      tdt = new TDataType(evn);
    }
    else
    {
      tdt = d_null;
    }
    adevices64 = new NAME64[nad];
    for (int i = 0; i < adevices64.length; i++)
    {
      adevices64[i] = new NAME64();
    }
    d_adevices = new TDataType(adevices64);
    String dev = new String("/"+ArchiveServerContext+"/"+"EVENTS");
    TLink lnk = new TLink(dev+"/"+DeviceServer,"StoredDevices",d_adevices,tdt,TAccess.CA_READ);
    if ((statusCode=lnk.executeAndClose(3000)) == 0)
    {
      return Arrays.copyOf(adevices64, d_adevices.getCompletionLength());
    }
    status = lnk.getLastError();
    return null;
  }
  public static String[] getDevicesAsString(String ArchiveServerContext, String DeviceServer, int eventNumber)
  {
    NAME64[] n = getDevices64(ArchiveServerContext, DeviceServer, eventNumber);
    if (n != null)
    {
      String[] devStringArray = new String[n.length];
      for (int i = 0; i < n.length; i++)
      {
        devStringArray[i] = new String(n[i].toString());
      }
      return devStringArray;
    }
    else
    {
      String[] nullStringArray = new String[1];
      nullStringArray[0] = new String("");
      return nullStringArray;
    }
  }
  public static String[] getDevicesAsString(String ArchiveTrigger, String ArchiveServerContext, String DeviceServer, String DeviceProperty,int eventNumber)
  {
    if (DeviceServer == null || ArchiveServerContext == null ) return new String[0];
    if (!DeviceServer.startsWith("/")) DeviceServer = "/"+ArchiveServerContext+"/"+DeviceServer;
    if (DeviceProperty != null) DeviceServer = DeviceServer + "["+DeviceProperty+"]";
    if (ArchiveTrigger != null) DeviceServer = ArchiveTrigger + ":" + DeviceServer;
    return getDevicesAsString(ArchiveServerContext,DeviceServer,eventNumber);
  }
  public static String[] getDevicesAsString(String ArchiveServerContext, String DeviceServer)
  {
    return getDevicesAsString(ArchiveServerContext, DeviceServer, 0);
  }
  public static int getNumberOfProperties(String ArchiveServerContext, String DeviceServer)
  {
    return getNumberOfProperties(ArchiveServerContext, DeviceServer, null, 0);
  }
  public static int getNumberOfProperties(String context, String trigger,
      String deviceServer, String device)
  {
    return getNumberOfProperties(context, trigger, deviceServer, device, 0);
  }
  public static int getNumberOfProperties(String context, String deviceServer, String device,
      int eventNumber)
  {
    return getNumberOfProperties(context, null, deviceServer, device, eventNumber);
  }
  public static int getNumberOfProperties(String context, String trigger,
      String deviceServer, String device, int eventNumber)
  { // @server: device name = target device server; input is only ever the trigger
    TDataType tdt = null;
    if (trigger != null)
    { // input is trigger
      tdt = new TDataType(new NAME64[]{new NAME64(trigger)});
    }
    String dev = "/"+context+"/"+"EVENTS"+"/"+deviceServer;
    TLink lnk = new TLink(dev,"NumStoredProperties",new TDataType(naproperties),tdt,TAccess.CA_READ);
    if ((statusCode = lnk.executeAndClose(3000)) != 0)
    {
      status = lnk.getLastError();      
      return -1;
    }
    return naproperties[0];
  }
  public static NAME16[] getProperties(String context, String deviceServer)
  {
    return getProperties(context, deviceServer, null, 0);
  }
  public static NAME16[] getProperties(String context, String trigger,
      String deviceServer, String device)
  {
    return getProperties(context, trigger, deviceServer, device, 0);
  }
  public static NAME16[] getProperties(String context, String deviceServer, String device, int eventNumber)
  {
    return getProperties(context, null, deviceServer, device, eventNumber);
  }
  public static NAME16[] getProperties(String context, String trigger,
      String deviceServer, String device, int eventNumber)
  {
    TDataType tdt;
    int nap = getNumberOfProperties(context, trigger, deviceServer, null);
    if (nap <= 0) return null;
    if (trigger != null)
    {
      NAME16[] dev = new NAME16[1];
      dev[0] = new NAME16(trigger);
      d_device = new TDataType(dev);
      tdt = d_device;
    }
    else if (eventNumber > 0)
    {
      NAME16I[] devevn = new NAME16I[1];
      devevn[0] = new NAME16I(device, eventNumber);
      d_devevn = new TDataType(devevn);
      tdt = d_devevn;
    }
    else if (device != null)
    {
      NAME16[] dev = new NAME16[1];
      dev[0] = new NAME16(device);
      d_device = new TDataType(dev);
      tdt = d_device;
    }
    else
    {
      tdt = d_null;
    }
    aproperties = new NAME16[nap];
    for (int i = 0; i < aproperties.length; i++)
    {
      aproperties[i] = new NAME16();
    }
    d_aproperties = new TDataType(aproperties);
    if (ap != null) ap.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    ap = new TLink(dev + "/" + deviceServer, "StoredProperties", d_aproperties, tdt, (short) 1);
    if ((statusCode = ap.execute(3000)) == 0)
    {
      ap.close();
      return aproperties;
    }
    status = ap.getLastError();
    ap.close();
    return null;
  }
  private static NAME64[] getProperties64(String context, String trigger,
      String deviceServer, String device, int eventNumber)
  {
    String dev = deviceServer;
    TDataType tdt = null;
    if (eventNumber > 0)
    {
      if (trigger == null || trigger.length() == 0)
      {
        status = TErrorList.getErrorString(TErrorList.invalid_parameter);
        return null;        
      }
      dev = trigger+":"+dev;
      if (device != null && device.length() > 0) dev += "/"+device;
      tdt = new TDataType(new int[]{eventNumber});
    }
    else
    {
      if (trigger != null)
      {
        tdt = new TDataType(new NAME64[]{new NAME64(trigger)});
      }      
    }
    dev = "/"+context+"/EVENTS/"+dev;
    int nap = getNumberOfProperties(context, trigger, deviceServer, null);
    if (nap <= 0) return null;
    aproperties64 = new NAME64[nap];
    for (int i = 0; i < aproperties64.length; i++) aproperties64[i] = new NAME64();
    TLink lnk = new TLink(dev, "StoredProperties", new TDataType(aproperties64), tdt, TAccess.CA_READ);
    if ((statusCode = lnk.executeAndClose(3000)) != 0)
    {
      status = lnk.getLastError();
      return null;
    }
    return aproperties64;
  }
  public static String[] getPropertiesAsString(String ArchiveServerContext, String DeviceServer,
      String Device, int eventNumber)
  {
    return getPropertiesAsString(ArchiveServerContext, null, DeviceServer, Device, eventNumber);
  }
  public static String[] getPropertiesAsString(String context, String trigger,
      String deviceServer, String device, int eventNumber)
  {
    NAME64[] n = getProperties64(context, trigger, deviceServer, device, eventNumber);
    if (n != null)
    {
      String[] devStringArray = new String[n.length];
      for (int i = 0; i < n.length; i++)
      {
        devStringArray[i] = n[i].getName(); // new String(n[i].toString());
      }
      return devStringArray;
    }
    return new String[]{""};
  }
  public static String[] getPropertiesAsString(String context, String trigger,
      String deviceServer, String device)
  {
    return getPropertiesAsString(context, trigger, deviceServer, device, 0);
  }
  public static String[] getPropertiesAsString(String context, String deviceServer)
  {
    return getPropertiesAsString(context, deviceServer, null, 0);
  }
  public static int getNumberOfEvents(String context, int start, int stop)
  {
    timeRange[0] = start;
    timeRange[1] = stop;
    d_nevent = new TDataType(nevents);
    d_range = new TDataType(timeRange);
    if (nevn != null) nevn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    nevn = new TLink(dev, "NEVENTS", d_nevent, d_range, (short) 1);
    if ((statusCode = nevn.execute(3000)) == 0)
    {
      nevn.close();
      return nevents[0];
    }
    status = nevn.getLastError();
    nevn.close();
    return -1;
  }
  public static int getNumberOfEvents(String context, String eventTrigger, int start, int stop)
  {
    timeRange[0] = start;
    timeRange[1] = stop;
    d_nevent = new TDataType(nevents);
    d_range = new TDataType(timeRange);
    if (nevn != null) nevn.close();
    String dev = "/" + context + "/EVENTS/" + eventTrigger;
    nevn = new TLink(dev, "NEvents", d_nevent, d_range, TAccess.CA_READ);
    if ((statusCode = nevn.execute(3000,true)) == 0)
    {
      nevn.close();
      return nevents[0];
    }
    status = nevn.getLastError();
    nevn.close();
    return -1;
  }  
  static int getNumberOfEvents()
  {
    return nevents[0];
  }
  public static AES[] getArchivedEvents(String context, int start, int stop)
  {
    int nevn = getNumberOfEvents(context, start, stop);
    if (nevn <= 0) return null;
    int nbytes = 0;
    aes = new AES[nevn];
    for (int i = 0; i < aes.length; i++)
    {
      aes[i] = new AES();
      nbytes += AES.sizeInBytes;
    }
    int siz = nbytes / nevn;
    byte[] bBuffer = new byte[nbytes];
    d_aes = new TDataType(bBuffer, "AES");
    timeRange[0] = start;
    timeRange[1] = stop;
    d_range = new TDataType(timeRange);
    if (evn != null) evn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    evn = new TLink(dev, "EVENTS", d_aes, d_range, (short) 1);
    if ((statusCode = evn.execute(3000)) == 0)
    {
      for (int i = 0, ptr = 0; i < nevn; i++)
      {
        byte[] b = aes[i].toByteArray();
        for (int n = 0; n < b.length; n++)
          b[n] = bBuffer[ptr + n];
        ptr += siz;
        aes[i].toStruct();
      }
      evn.close();
      return aes;
    }
    status = evn.getLastError();
    evn.close();
    return null;
  }
  public static int getNumberOfDeviceEvents(String context, String deviceServer)
  {
    NAME16[] dev = new NAME16[1];
    dev[0] = new NAME16(deviceServer);
    d_nevent = new TDataType(nevents);
    d_device = new TDataType(dev);
    if (nevn != null) nevn.close();
    String devsrv = new String("/" + context + "/" + "EVENTS");
    nevn = new TLink(devsrv + "/ALL", "NEVENTS", d_nevent, d_device, (short) 1);
    if ((statusCode = nevn.execute(3000)) == 0)
    {
      nevn.close();
      return nevents[0];
    }
    status = nevn.getLastError();
    nevn.close();
    return -1;
  }
  public static int getNumberOfDeviceEvents(String context, String deviceServer, int start, int stop)
  {
    d_nevent = new TDataType(nevents);
    timeRange[0] = start;
    timeRange[1] = stop;
    d_range = new TDataType(timeRange);
    if (nevn != null) nevn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    nevn = new TLink(dev + "/" + deviceServer, "NumDeviceEvents", d_nevent, d_range, TAccess.CA_READ);
    if ((statusCode = nevn.execute(3000)) == 0)
    {
      nevn.close();
      return nevents[0];
    }
    status = nevn.getLastError();
    nevn.close();
    return -1;
  }
  public static int getNumberOfArchivedDeviceEvents(String context, String deviceServer, int start, int stop)
  {
    d_nevent = new TDataType(nevents);
    timeRange[0] = start;
    timeRange[1] = stop;
    d_range = new TDataType(timeRange);
    if (nevn != null) nevn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    nevn = new TLink(dev + "/" + deviceServer, "NumDeviceEvents", d_nevent, d_range, TAccess.CA_READ);
    if ((statusCode = nevn.execute(3000)) == 0)
    {
      nevn.close();
      return nevents[0];
    }
    status = nevn.getLastError();
    nevn.close();
    return -1;
  }
  public static int getNumberOfArchivedDeviceEvents(String context, String deviceServer,String device, int start, int stop)
  { // work Device into this sometime ...
    NAME16I[] timeRangeDev = new NAME16I[2];
    d_nevent = new TDataType(nevents);
    timeRange[0] = start;
    timeRange[1] = stop;
    timeRangeDev[0] = new NAME16I(device, start);
    timeRangeDev[1] = new NAME16I("", stop);
    d_range = new TDataType(timeRangeDev);
    if (nevn != null) nevn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    nevn = new TLink(dev + "/" + deviceServer, "NumDeviceEvents", d_nevent, d_range, TAccess.CA_READ);
    if ((statusCode = nevn.execute(3000)) == 0)
    {
      nevn.close();
      return nevents[0];
    }
    status = nevn.getLastError();
    nevn.close();
    return -1;
  }
  public static AES[] getArchivedDeviceEvents(String context, String deviceServer)
  {
    int nevn = getNumberOfDeviceEvents(context, deviceServer);
    if (nevn <= 0) return null;
    int nbytes = 0;
    aes = new AES[nevn];
    for (int i = 0; i < aes.length; i++)
    {
      aes[i] = new AES();
      nbytes += AES.sizeInBytes;
    }
    int siz = nbytes / nevn;
    byte[] bBuffer = new byte[nbytes];
    d_aes = new TDataType(bBuffer, "AES");
    if (evn != null) evn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    evn = new TLink(dev + "/" + deviceServer, "DeviceEvents", d_aes, d_null, (short) 1);
    if ((statusCode = evn.execute(3000)) == 0)
    {
      for (int i = 0, ptr = 0; i < nevn; i++)
      {
        byte[] b = aes[i].toByteArray();
        for (int n = 0; n < b.length; n++)
          b[n] = bBuffer[ptr + n];
        ptr += siz;
        aes[i].toStruct();
      }
      evn.close();
      return aes;
    }
    status = evn.getLastError();
    evn.close();
    return null;
  }
  public static AES[] getArchivedDeviceEvents(String context, String deviceServer, int start,
      int stop)
  {
    int nevn = getNumberOfDeviceEvents(context, deviceServer);
    if (nevn <= 0) return null;
    int nbytes = 0;
    aes = new AES[nevn];
    for (int i = 0; i < aes.length; i++)
    {
      aes[i] = new AES();
      nbytes += AES.sizeInBytes;
    }
    int siz = nbytes / nevn;
    byte[] bBuffer = new byte[nbytes];
    d_aes = new TDataType(bBuffer, "AES");
    timeRange[0] = start;
    timeRange[1] = stop;
    d_range = new TDataType(timeRange);
    if (evn != null) evn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    evn = new TLink(dev + "/" + deviceServer, "DeviceEvents", d_aes, d_range, (short) 1);
    if ((statusCode = evn.execute(3000)) == 0)
    {
      for (int i = 0, ptr = 0; i < nevn; i++)
      {
        byte[] b = aes[i].toByteArray();
        for (int n = 0; n < b.length; n++)
          b[n] = bBuffer[ptr + n];
        ptr += siz;
        aes[i].toStruct();
      }
      evn.close();
      return aes;
    }
    status = evn.getLastError();
    evn.close();
    return null;
  }
  
  public static int[] getArchivedEventList(String context, String trigger, int start, int stop)
	  {
	  return getArchivedEventList(context,trigger,start,stop,5000);
	  }
  public static int[] getArchivedEventList(String context, String trigger, int start, int stop, int nEvents)
  {
    evnlst = new int[nEvents];
    d_evnlst = new TDataType(evnlst);
    timeRange[0] = start;
    timeRange[1] = stop;
    d_range = new TDataType(timeRange);
    if (evn != null) evn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    evn = new TLink(dev + "/" + trigger, "Events", d_evnlst, d_range, (short) 1);
    statusCode = evn.execute(1000);
    if (!TErrorList.hasData(statusCode))
    {
      status = evn.getLastError();
      evnlst = null;
    }
    evn.close();
    int n = d_evnlst.getCompletionLength();
    if (n == 0) return null;
    if (n < nEvents)
    {
      int[] lst = new int[n];
      System.arraycopy(evnlst, 0, lst, 0, n);
      evnlst = lst;
    }
    return evnlst;
  }
  public static int[] getArchivedDeviceEventList(String context, String deviceServer, int start, int stop)
  {
    int ndevn = getNumberOfArchivedDeviceEvents(context, deviceServer, start, stop);
    if (ndevn <= 0) return null;
    evnlst = new int[ndevn];
    d_evnlst = new TDataType(evnlst);
    timeRange[0] = start;
    timeRange[1] = stop;
    d_range = new TDataType(timeRange);
    if (evn != null) evn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    evn = new TLink(dev + "/" + deviceServer, "DeviceEvents", d_evnlst, d_range, (short) 1);
    statusCode = evn.execute(3000);
    if (!TErrorList.hasData(statusCode))
    {
      status = evn.getLastError();
      evnlst = null;
    }
    evn.close();
    return evnlst;
  }
  public static String[] getArchivedDeviceEventListAsString(String context, String deviceServer,
      int start, int stop)
  {
    int[] el = getArchivedDeviceEventList(context, deviceServer, start, stop);
    if (el != null)
    {
      Date d = new Date();
      String[] elStringArray = new String[el.length];
      for (int i = 0; i < el.length; i++)
      {
        d.setTime(((long) el[i]) * 1000);
        elStringArray[i] = new String(d.toString());
      }
      return elStringArray;
    }
    else
    {
      String[] nullStringArray = new String[1];
      nullStringArray[0] = new String("");
      return nullStringArray;
    }
  }
  public static AES[] getArchivedDeviceEvents(String context, String deviceServer,
      String Device, int start, int stop)
  {
    int nevn = getNumberOfDeviceEvents(context, deviceServer);
    if (nevn <= 0) return null;
    NAME16I[] devevn = new NAME16I[2];
    devevn[0] = new NAME16I(Device, start);
    devevn[1] = new NAME16I("", stop);
    d_devevn = new TDataType(devevn);
    int nbytes = 0;
    aes = new AES[nevn];
    for (int i = 0; i < aes.length; i++)
    {
      aes[i] = new AES();
      nbytes += AES.sizeInBytes;
    }
    int siz = nbytes / nevn;
    byte[] bBuffer = new byte[nbytes];
    d_aes = new TDataType(bBuffer, "AES");
    if (evn != null) evn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    evn = new TLink(dev + "/" + deviceServer, "DeviceEvents", d_aes, d_devevn, (short) 1);
    statusCode = evn.execute(3000);
    if (TErrorList.hasData(statusCode))
    {
      for (int i = 0, ptr = 0; i < nevn; i++)
      {
        byte[] b = aes[i].toByteArray();
        for (int n = 0; n < b.length; n++)
          b[n] = bBuffer[ptr + n];
        ptr += siz;
        aes[i].toStruct();
      }
      evn.close();
      return aes;
    }
    status = evn.getLastError();
    evn.close();
    return null;
  }
  public static int[] getArchivedDeviceEventList(String context, String deviceServer,
      String device, int start, int stop)
  {
    int nevn = getNumberOfArchivedDeviceEvents(context, deviceServer, device, start, stop);
    if (nevn <= 0) return null;
    NAME16I[] devevn = new NAME16I[2];
    devevn[0] = new NAME16I(device, start);
    devevn[1] = new NAME16I("", stop);
    d_devevn = new TDataType(devevn);
    evnlst = new int[nevn];
    d_evnlst = new TDataType(evnlst);
    if (evn != null) evn.close();
    String dev = new String("/" + context + "/" + "EVENTS");
    evn = new TLink(dev + "/" + deviceServer, "DeviceEvents", d_evnlst, d_devevn, (short) 1);
    statusCode = evn.execute(3000);
    if (TErrorList.hasData(statusCode))
    {
      evn.close();
      return evnlst;
    }
    status = evn.getLastError();
    evn.close();
    return null;
  }
  public static String[] getArchivedDeviceEventListAsString(String context, String deviceServer,
      String device, int start, int stop)
  {
    int[] el = getArchivedDeviceEventList(context, deviceServer, device, start, stop);
    if (el != null)
    {
      Date d = new Date();
      String[] elStringArray = new String[el.length];
      for (int i = 0; i < el.length; i++)
      {
        d.setTime(((long) el[i]) * 1000);
        elStringArray[i] = new String(d.toString());
      }
      return elStringArray;
    }
    else
    {
      String[] nullStringArray = new String[1];
      nullStringArray[0] = new String("");
      return nullStringArray;
    }
  }
  public static int removeArchiveData(String context, String trigger, int eventNumber)
  {
    int evn[] = new int[1];
    evn[0] = eventNumber;
    TDataType d_evnt = new TDataType(evn);
    TLink rmv;
    String dev = new String("/"+context+"/EVENTSTORE/"+trigger);
    rmv = new TLink(dev, "REMOVE", null, d_evnt, TAccess.CA_WRITE);
    statusCode = rmv.execute(1000);
    rmv.close();
    return statusCode;
  }
  public static String[] getEventArchiveServerContexts()
  {
    return TQuery.getContexts("EVENTS");
  }
  public static String[] getTransientRecorderConfigurationNames(String context)
  {
    if (context == null) return null;
    return TQuery.getDeviceNames(context, "EVENTS", "TRC.COMPONENTS");
  }
  public static TrgCmp[] getTransientRecorderComponents(String context, String config) throws IOException
  {
    if (context == null || context.length() == 0) return null;
    if (config == null || config.length() == 0) return null;
    TrgCmp[] tc = new TrgCmp[64];
    for (int i = 0; i < 64; i++)
      tc[i] = new TrgCmp();
    TDataType dout = new TDataType(tc);
    TLink lnk = new TLink("/" + context + "/EVENTS/" + config, "TRC.COMPONENTS", dout, null, TAccess.CA_READ);
    int cc = lnk.execute(500, true);
    lnk.close();
    if (cc != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    TrgCmp[] rtc = new TrgCmp[n];
    for (int i = 0; i < n; i++)
      rtc[i] = tc[i];
    return rtc;
  }
  public static TrgChn[] getTransientRecorderChannels(String context, String description) throws IOException
  {
    if (context == null || context.length() == 0) return null;
    if (description == null || description.length() == 0) return null;
    TrgChn[] tc = new TrgChn[64];
    for (int i = 0; i < 64; i++)
      tc[i] = new TrgChn();
    TDataType dout = new TDataType(tc);
    TLink lnk = new TLink("/" + context + "/EVENTS/" + description, "TRC.CHANNELS", dout, null,
        TAccess.CA_READ);
    int cc = lnk.execute(500, true);
    lnk.close();
    if (cc != 0) throw new IOException("link error " + lnk.getLastError());
    int n = dout.getCompletionLength();
    if (n == 0) return null;
    TrgChn[] rtc = new TrgChn[n];
    for (int i = 0; i < n; i++)
      rtc[i] = tc[i];
    return rtc;
  }
  public static String[] getArchiveServerContexts()
  {
    return TQuery.getContexts("EVENTS");
  }
  public static String[] getArchiveServerContexts(String importance)
  {
    return TQuery.getContexts("EVENTS",importance);
  }
  public static String[] getArchiveContexts(String context)
  {
    return TQuery.getDeviceNames(context, "EVENTSTORE");
  }
  public static String[] getArchiveContexts(String context, String server)
  {
    if (context == null || server == null) return null;
    NAME64[] n64 = new NAME64[100];
    TDataType tn64 = new TDataType(n64);
    TLink lnk = new TLink("/" + context + "/EVENTS/" + server, "StoredContexts", tn64, null,
        TAccess.CA_READ);
    int cc = lnk.executeAndClose();
    if (cc != 0) return null;
    int siz = tn64.getCompletionLength();
    if (siz == 0) return null;
    String[] nams = new String[siz];
    for (int i = 0; i < siz; i++)
      nams[i] = n64[i].getName();
    return nams;
  }
  public static String getArchiveComment(String context, String trigger, int eventNumber)
  {
    String cmt = "no comment";
    String dev = new String("/" + context + "/EVENTSTORE/" + trigger);
    int[] triggerTime = new int[1];
    triggerTime[0] = eventNumber;
    StringBuffer cmtbuf = new StringBuffer(1024);
    TLink tl = new TLink(dev, "COMMENT", new TDataType(cmtbuf), new TDataType(triggerTime), TAccess.CA_READ);
    int cc = tl.executeAndClose(500);
    if (cc == 0)
    {
      cmt = cmtbuf.toString().trim();
    }
    return cmt;
  }
  public static int sendComment(String context, String trigger, int eventNumber,
      String comment)
  {
    String dev = new String("/" + context + "/EVENTSTORE/" + trigger);
    int[] triggerTime = new int[1];
    triggerTime[0] = eventNumber;
    String cmt = new String(comment + " @" + triggerTime[0]);
    TLink tl = new TLink(dev, "ANNOTATE", new TDataType(), new TDataType(cmt), TAccess.CA_WRITE);
    return tl.executeAndClose(500);
  }
  /**
   * Sends the given trigger to the designated event archive server.
   * 
   * @param context is the context of the event archive server which should receive the trigger.
   * @param trigger is the trigger to send
   * @param comment is an event annotation which describes this triggered event.
   * @param triggerTime is the requested time of trigger to associate with this event.  If
   * '0' (the default) then the system obtains the current trigger time associated with 'now'.
   * @param triggerLevel is the desired trigger level. Default = 1 signals a normal start of
   * trigger.  The value '-1' indicates a pre-step which informs any other configured triggers
   * that the indicated trigger is firing and thereby giving other trigger scripts a chance
   * to participate in the on-going event.
   * @param rangeStart is a system stamp to be used at the initial value when accessing a
   * range of local history data. A value of '0' (default) implies that should the event 
   * involve local history data acquisition then a simple 'last 10' algorithm should be used.
   * A value less than zero indicates that the time stamp should be used in acquiring local
   * history data and that the range start value should be subtracted from the current time stamp
   * in establishing the initial local history access.  
   * @param rangeStop is a system stamp to be used at the final value when accessing a
   * range of local history data.
   * @param maxOverRange gives a maximum value (default = 1000) of successive calls to the
   * local history subsystem while traversing a range.  If this parameter is less than or equal
   * to zero then no range information will be sent with the trigger.
   * 
   * @return 0 upon success or a TINE error code indicating a problem issuing the trigger.
   */
  public static int sendTrigger(String context, String trigger, String comment,long triggerTime,int triggerLevel,int rangeStart,int rangeStop,int maxOverRange)
  {
    int[] tvals = new int[5];
    int[] tot = new int[1];
    String dev = new String("/" + context + "/EVENTSTORE/" + trigger);
    tvals[0] = triggerLevel == 0 ? 1 : triggerLevel;
    tvals[1] = (int)(triggerTime/1000);
    tvals[2] = rangeStart;
    tvals[3] = rangeStop;
    tvals[4] = maxOverRange;
    int minlen = triggerTime == 0 ? 1 : 2;
    int len = maxOverRange <= 0 ? minlen : 5;
    TLink tl = new TLink(dev, "TRIGGER", new TDataType(tot), new TDataType(tvals,len),TAccess.CA_WRITE);
    int cc = tl.executeAndClose(500);
    if (cc == 0)
    {
      String cmt = new String(comment + " @" + tot[0]);
      tl = new TLink(dev, "ANNOTATE", new TDataType(), new TDataType(cmt), TAccess.CA_WRITE);
      tl.executeAndClose(500);
    }
    return cc;
  }
  public static int sendTrigger(String context, String trigger, String comment,int rangeStart,int rangeStop,int maxOverRange)
  {
    return sendTrigger(context,trigger,comment,0,1,rangeStart,rangeStop,maxOverRange);
  }
  public static int sendTrigger(String context, String trigger, String comment,long triggerTime)
  {
    return sendTrigger(context,trigger,comment,triggerTime,1,0,0,-1);
  }
  /**
   * Sends the given trigger to the designated event archive server.  This is the simplest
   * form of sending an event trigger and is the de-facto standard for most triggers.
   * Triggers started via this method will not be able to receive any range information to
   * use should the trigger event involve accessing a sequence of data from a server's 
   * local history subsystem.  As few triggers require this, this form of firing an event
   * trigger is widely used.
   * 
   * @param context is the context of the event archive server which should receive the trigger.
   * @param trigger is the trigger to send
   * @param comment is an event annotation which describes this triggered event.
   * 
   * @return 0 upon success or a TINE error code indicating a problem issuing the trigger.
   */
  public static int sendTrigger(String context, String trigger, String comment)
  {
    return sendTrigger(context,trigger,comment,0,1,0,0,-1);
  }
  public static int stopTrigger(String context, String trigger)
  {
    int[] triggerLevel = new int[1];
    int[] triggerTime = new int[1];
    String dev = new String("/" + context + "/EVENTSTORE/" + trigger);
    triggerLevel[0] = 1;
    TLink tl = new TLink(dev, "STOP", new TDataType(triggerTime), new TDataType(triggerLevel),
        TAccess.CA_WRITE);
    int cc = tl.executeAndClose(500);
    return cc;
  }
}
