/*
 * Created on Feb 17, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.desy.tine.example.sine;
import de.desy.tine.server.devices.*;

/**
 * This is an example for a device. The device periodically produces an array of
 * values in its update() method. The device has properties which can be read
 * and set via set/get methods.
 * 
 * @author jwlg
 */
public class CoSineDevice extends TDevice 
{
  public static final int DATASIZE = 1024;
  public static final double PHASEDIFF = 2 * Math.PI / 100.0;
  private double frequency = 1;
  private double amplitude = 1;
  private double[] myData;
  private double phase = 0.0f;
  private double noise = 0.05;
  /**
   * \brief Simple constructor
   * 
   * Use this constructor if you intend on working with device (i.e. equipment)
   * numbers in your code and you use a standard <local-name>-devices.csv file
   * to register the exported device names.
   * 
   * @param newNumber is the device number to be assigned to this instance.  This
   * will be cross-checked inside the <local-name>-devices.csv to determine the
   * correpsonding device name.
   */
  public CoSineDevice(int newNumber)
  {
    super(newNumber);
    myData = new double[DATASIZE];
  }
  /**
   * \brief Complex constructor
   * 
   * Use this constructor if you 1) want to explicitly assign the device names inside
   * your code or 2) you are reading in this information via your own database 
   * (and not via <local-name>-devices.csv) and hence need to assign the device
   * names to device numbers in code.
   * 
   * @param newName
   * @param newNumber
   */
  public CoSineDevice(String newName,int newNumber)
  {
    super(newName,newNumber);
    myData = new double[DATASIZE];
  }
  /**
   * @return Returns the frequency.
   */
  public double getFrequency()
  {
    return frequency;
  }
  /**
   * @param frequency The frequency to set.
   */
  public void setFrequency(double frequency)
  {
    this.frequency = frequency;
  }
  /**
   * @return Returns the amplitude.
   */
  public double getAmplitude()
  {
    return amplitude;
  }
  /**
   * @param amplitude The amplitude to set.
   */
  public void setAmplitude(double amplitude)
  {
    this.amplitude = amplitude;
  }
  /**
   * @return sine function array
   */
  public double[] getSine()
  {
    return myData;
  }
  private void incrementPhase()
  {
    double phasediff = Math.PI / 100.0;
    phase += phasediff;
    if (phase + phasediff > 2.0 * Math.PI) phase = 0.0f;
  }
  /**
   * Called in the background task.
   * 
   * Note the calls to the local alarm server.
   * clearAlarm() increments the clear counter and is called when the routine is entered.
   * setAlarm() is called when an alarm state is detected.
   * This is just an example.  Alarms for 'value too high', etc. are better handled
   * through the use of an 'almwatch.csv' configuration file.   
   *
   */
  public void update()
  {
    clearAlarm(512);
    incrementPhase();
    for (int i = 0; i < 1024; i++)
      myData[i] = amplitude * Math.sin(phase + frequency * 6.28 * ((double) i / 1024.0)) + noise * Math.random();
    if (amplitude > 100) setAlarm(512);  // amplitude too high !
  }
  public double getPhase()
  {
    return phase;
  }
  public void setPhase(double phase)
  {
    this.phase = phase;
  }
  public double getNoise()
  {
    return noise;
  }
  public void setNoise(double noise)
  {
    this.noise = noise;
  }
}