//: TLink.java
package de.desy.tine.headers;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentLinkedQueue;

import de.desy.tine.addrUtils.*;
import de.desy.tine.client.*;
import de.desy.tine.console.TCommandItem;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.*;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.io.TPacket;
import de.desy.tine.server.connections.*;
import de.desy.tine.server.logger.*;
import de.desy.tine.server.properties.*;
import de.desy.tine.types.MDA;
// Subscription response header (supersedes TPHdr)
public class TSubRspHdr
{
  private static TLinkFactory tlf = null;
  private static TLinkFactory getLinkFactory()
  {
    if (tlf == null) tlf = TLinkFactory.getInstance();
    return tlf;
  }
  // header bytes:
  public int msgSizeInBytes; // this message size in bytes
  public short subId; // supplied by the consumer upon registration (con tbl id)
  public short retCode; // return code of the eqm call
  public short numblks; // total number of message blocks
  public short blknum; // block number of this message
  public int mtu; // maximum transport unit
  public short dataFormat; // data format of returned data
  public short counter; // subscription counter
  public short tineProtocol;// tine protocol level
  public short xferReason; // transfer reason
  public int starttime; // supplied by caller @ registration (always at byte 20)
  public short stsSize;    // size of error/status string space
  public short stsCode;    // status code of call
  public int dTimeStamp; // data time stamp, sec part
  public int dTimeStampUSEC; // data time stamp, usec part
  public int usrStamp;   // user-supplied data stamp
  public int sysStamp;   // systematic data stamp
  public int dataSize; // size of contract
  public int msgSizeInElements; // number of elements (of type format) returned in this message
  // end of header bytes
  static public final int hdrSizeInBytes = 56;
  static public final int hdrSizeInBytesR4 = (12*2 + 5*4); // Release 4 : 44 bytes
  static public final int hdrSizeInBytesR3 = (7*2 + 2*1 + 2*4); // Release 3 : 24 bytes
  private ByteArrayOutputStream stream;
  public byte[] toByteArray()
  {
    return stream != null ? stream.toByteArray() : null;
  }
  private byte[] bBuffer = new byte[TPacket.MAX_DATAGRAM_SIZE];
  private byte[] stsBuffer = new byte[TStrings.STATUS_SIZE];
  public static int getHeaderSizeInBytes(int protocol)
  {
    if (protocol < TTransport.legacyProtocolLevel) return hdrSizeInBytesR3;
    if (protocol == TTransport.legacyProtocolLevel) return hdrSizeInBytesR4;
    return hdrSizeInBytes;
  }
  // prepare out-going data (server side) ...
  private void prepare(int protocolLevel)
  {
    try
    {
      stream.reset();
      DataOutputStream ds = new DataOutputStream(stream);
      if (protocolLevel > TTransport.legacyProtocolLevel)
      { // modern : Release 5.xx
        ds.writeInt(Swap.Int(msgSizeInBytes));
        ds.writeShort(Swap.Short(subId));
        ds.writeShort(Swap.Short(retCode));
        ds.writeShort(Swap.Short(numblks));
        ds.writeShort(Swap.Short(blknum));
        ds.writeInt(Swap.Int(mtu));        
        ds.writeShort(Swap.Short(dataFormat));
        ds.writeShort(Swap.Short(counter));
        ds.writeShort(Swap.Short(tineProtocol));
        ds.writeShort(Swap.Short(xferReason));
        ds.writeInt(Swap.Int(starttime));
        ds.writeShort(Swap.Short(stsSize));
        ds.writeShort(Swap.Short(stsCode));      
        ds.writeInt(Swap.Int(dTimeStamp));
        ds.writeInt(Swap.Int(dTimeStampUSEC));
        ds.writeInt(Swap.Int(usrStamp));
        ds.writeInt(Swap.Int(sysStamp));        
        ds.writeInt(Swap.Int(dataSize));        
        ds.writeInt(Swap.Int(msgSizeInElements));        
      }
      else if (protocolLevel == TTransport.legacyProtocolLevel)
      { // Release 4.xx
        ds.writeShort(Swap.Short(UnsignedShort(msgSizeInBytes)));
        ds.writeShort(Swap.Short(subId));
        ds.writeShort(Swap.Short(retCode));
        ds.writeShort(Swap.Short(numblks));
        ds.writeShort(Swap.Short(blknum));
        ds.writeShort(Swap.Short(UnsignedShort(mtu)));
        ds.writeShort(Swap.Short(dataFormat));
        ds.writeShort(Swap.Short(counter));
        ds.writeShort(Swap.Short(tineProtocol));
        ds.writeShort(Swap.Short(xferReason));
        ds.writeInt(Swap.Int(starttime));
        ds.writeShort(Swap.Short(stsSize));
        ds.writeShort(Swap.Short(stsCode));      
        ds.writeInt(Swap.Int(dTimeStamp));
        ds.writeInt(Swap.Int(dTimeStampUSEC));
        ds.writeInt(Swap.Int(usrStamp));
        ds.writeInt(Swap.Int(sysStamp));                
      }
      else if (protocolLevel < TTransport.legacyProtocolLevel)
      { // Release 3.xx
        ds.writeShort(Swap.Short(UnsignedShort(msgSizeInBytes)));
        ds.writeShort(Swap.Short(subId));
        ds.writeShort(Swap.Short(retCode));
        ds.writeShort(Swap.Short(numblks));
        ds.writeShort(Swap.Short(blknum));
        ds.writeShort(Swap.Short(UnsignedShort(mtu)));
        ds.writeShort(Swap.Short(dTimeStampUSEC/1000));
        ds.writeByte(dataFormat);
        ds.writeByte(counter);
        ds.writeInt(Swap.Int(dTimeStamp));
        ds.writeInt(Swap.Int(starttime));        
      }
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("TPHdr.prepare",e.toString(),TErrorList.code_failure,e,0);
    }  
  }
  private String getStsSting(byte[] b,int offset,int length )
  {
    int len = 0;
    for (int i=0; i<length; i++) 
    {
      if (b[i+offset] == 0) break;
      len++;
    }
    return new String(b,offset,len).trim();
  }
  public static int UnsignedShort(short sval)
  {
    int ival = (int)sval;
    if (ival < 0) ival += 65536;
    return ival;
  }
  public static short UnsignedShort(int ival)
  {
    if (ival > 32767) ival -= 65536;
    return (short)ival;
  }
  private LinkedList<TLink> lnks = new LinkedList<TLink>();
  private static boolean lockout = false;
  public static void setLockOut(boolean value) { lockout = value; }
  public static void setLockOut(int value) { lockout = value != 0; }
  public static void setLockOut(String value)
  {
  	lockout = TCommandItem.getBoolean(value);
  	getLockOutAsString();
  }
  public static boolean getLockOut() { return lockout; }
  public static int getLockOutAsInt() { return lockout ? 1 : 0; }
  public static String getLockOutAsString() 
  {
  	DbgLog.log("getLockOut", "current lockout setting: "+lockout);
  	return lockout ? "true" : "false"; 
  }
  public boolean assertRedirectionValid(TLink lnk,String ctx,String srv,String dev,String prp)
  {
    if (lnk == null) return false;
    if (ctx != null && ctx.compareToIgnoreCase(lnk.cntName) != 0) return true;
    if (srv != null && srv.compareToIgnoreCase(lnk.expName) != 0) return true;
    if (dev != null && dev.compareToIgnoreCase(lnk.devName) != 0) return true;
    if (prp != null && prp.compareToIgnoreCase(lnk.devProperty) != 0) return true;
    return false;
  }
  private int getSubMsgSizeInBytes(DataInputStream ds,TRspHdr rsp) throws IOException
  {
    if (rsp.getTineProtocol() > TTransport.legacyProtocolLevel)
      return Swap.Int(ds.readInt());
    else if (!rsp.isJumbo())
      return TSubRspHdr.UnsignedShort(Swap.Short(ds.readShort()));
    else
      return rsp.getTotalSizeInBytes();
  }
  private int getSubMsgMtu(DataInputStream ds,TRspHdr rsp) throws IOException
  {
    if (rsp.getTineProtocol() > TTransport.legacyProtocolLevel)
      return Swap.Int(ds.readInt());
    else
      return TSubRspHdr.UnsignedShort(Swap.Short(ds.readShort()));
  }
  // prepare in-coming data (client side) ...
  // must examine the status code for 'illegal_protocol' !
  public TLink[] toStruct(InetAddress inetAddr, int port, byte[] databytes, int length, int transport)
  {
    lnks.clear();
    TLink lnk;
    int bytesread = 0, pktsread = 0, hdrSize = hdrSizeInBytesR4;
    int datasize, fmtsize, doutsize, extbytes, extsize, extptr;
    int len;
    byte[] d;
    boolean isLegacy = false;
    ByteArrayInputStream instream;
    long ltime = System.currentTimeMillis();
    
    TRspHdr rsp = new TRspHdr(databytes,transport == TTransport.STREAM);    
    try
    {
      instream = new ByteArrayInputStream(databytes);
      DataInputStream ds = new DataInputStream(instream);      
      // overall message length:
      len = rsp.getTotalSizeInBytes();      
      if (TLinkFactory.debugLevel > 1) DbgLog.log("TSubRspHdr","recv : " + length + " bytes (hdr " + len + " bytes)");
      if (len < length)
      {
        if (TLinkFactory.debugLevel > 0) DbgLog.log("TSubRspHdr","recv error : abort ! ");
        return null;
      }
      boolean needLinkWatchDogCycle = false;
      boolean applyStatusLater = false;
      boolean trace = false;
      bytesread = rsp.getHeaderSizeInBytes();
      if (ds.skipBytes(bytesread) != bytesread)
      {
        if (TLinkFactory.debugLevel > 0) DbgLog.log("TSubRspHdr","could not skip "+bytesread+" bytes!");
        return null;        
      }
      // hbytes: the number of header bytes read in at the top of a packed response 
      // (jumbos excluded because they are not packed)
      int hbytes = 20 + (rsp.getTineProtocol() <= TTransport.legacyProtocolLevel ? 4 : 8);
      while (bytesread < length)
      {
        msgSizeInBytes = getSubMsgSizeInBytes(ds,rsp);
        msgSizeInElements = 0; // as yet unknown
        if (msgSizeInBytes < TSubRspHdr.hdrSizeInBytesR3) 
        { // this is corrupt !
          break; 
        }
        subId = Swap.Short(ds.readShort());
        retCode = Swap.Short(ds.readShort());
        numblks = Swap.Short(ds.readShort());
        blknum = Swap.Short(ds.readShort());
        mtu = getSubMsgMtu(ds,rsp);
        ds.mark(msgSizeInBytes);        
        dataFormat = Swap.Short(ds.readShort());
        counter = Swap.Short(ds.readShort());
        tineProtocol = Swap.Short(ds.readShort());
        xferReason = Swap.Short(ds.readShort());
        starttime = Swap.Int(ds.readInt());
        if (retCode == TErrorList.illegal_protocol)
        { // if illegal_protocol then stop reading the stream and try to find the link via inet addr
          if ((lnk=tf.findLink(inetAddr,port)) == null)
          { // assume the worst and bail out ...
            if (TLinkFactory.debugLevel > 0) 
              DbgLog.log("TSubRspHdr","link for " + inetAddr.getHostAddress() + " not found!");
            break;
          }            
        }
        else
        { 
          if (starttime == TFecEntry.BCAST_ID)
          { // incoming multicast
            lnk = tf.findLink((int)subId,inetAddr,port);
          }
          else
          { // a 'normal' link
            lnk = tf.findTLink((int)subId,starttime);
          }
          if (lnk == null)
          { // one small hope (could be a packed multicast) 
            if (TLinkFactory.debugLevel > 2) DbgLog.log("TSubRspHdr","id " + subId + " not found!");
            bytesread += msgSizeInBytes;
            if (bytesread < length) 
            { // this might help ?
              //ds.skipBytes(msgSizeInBytes-hdrSizeInBytesR4);
              ds.skipBytes(msgSizeInBytes-hbytes);
              continue;
            }
            break; // assume the worst and bail out ...
          }
          trace = lnk.isTraceLink();
          if (lnk.getTineProtocol() != rsp.getTineProtocol())
          { // could happen due to a multicast network client mix-up
            String msg = "response protocol "+rsp.getTineProtocol()+" != link protocol "+lnk.getTineProtocol();
            if (trace) lnk.traceLink("TSubRspHdr", msg);
            if (TLinkFactory.debugLevel > 0) DbgLog.log("TSubRspHdr",lnk.getFullDeviceNameAndProperty()+" : "+msg);
            break;
          }
          if (lnk.getTineProtocol() >= TTransport.legacyProtocolLevel)
          { // normally true
            stsSize = Swap.Short(ds.readShort());
            stsCode = Swap.Short(ds.readShort());
            dTimeStamp = Swap.Int(ds.readInt());
            dTimeStampUSEC = Swap.Int(ds.readInt());
            usrStamp = Swap.Int(ds.readInt());
            sysStamp = Swap.Int(ds.readInt());
            hdrSize = hdrSizeInBytesR4;
            if (rsp.isJumbo()) hdrSize += 2; // need to append the legacy 2-byte 'header'
          }
          if (lnk.getTineProtocol() > TTransport.legacyProtocolLevel)
          { // modern protocol level (Release 5.xx)
            dataSize = Swap.Int(ds.readInt());
            msgSizeInElements = Swap.Int(ds.readInt());
            hdrSize = hdrSizeInBytes;
          }
        }
        if (numblks > lnk.dOutput.numblks)
        {
          lnk.dOutput.numblks = numblks;
          lnk.dOutput.reassignBuffers();
        }
        if (TLinkFactory.debugLevel > 1)
          DbgLog.log("TSubRspHdr","data : " + "bytes " + msgSizeInBytes + ", blknum " + blknum + " from " + numblks + " <" + retCode + ">");
        bytesread += msgSizeInBytes;
        if (lockout) continue; // simulate total timeout conditions
        if (lnk.getTineProtocol() == TTransport.legacyProtocolLevel-1)
        { // Release 3.xx ! go back to the mark
          ds.reset();
          dTimeStampUSEC = (int)Swap.Short(ds.readShort()) * 1000;
          dataFormat = (short)ds.readByte();
          counter = (short)ds.readByte();
          dTimeStamp = Swap.Int(ds.readInt());
          starttime = Swap.Int(ds.readInt());
          stsSize = retCode == 0 ? (short)0 : TStrings.STATUS_SHORTSIZE;
          stsCode = retCode;
          usrStamp = 0;
          sysStamp = 0;
          hdrSize = hdrSizeInBytesR3;
          isLegacy = true;
        }
        if (numblks == 1 && blknum != 1)
        { // trap a legacy bug !
          blknum = 1; 
        }
        lnk.dOutput.numblks = numblks;
        lnk.linkStatusSource = TLink.STATUS_REMOTE;
        applyStatusLater = TErrorList.isCoercive(retCode);
        if (lnk.linkStatus != TErrorList.link_blacklisted && !applyStatusLater) 
          lnk.linkStatus = retCode;
        if (lnk.getRenewalMultiplier() < 2 && counter > TContract.CTR_RENEWAL)
        { // learn from the server how this was set
          lnk.setRenewalMultiplier((short)(counter/TContract.CTR_RENEWAL));
        }
        lnk.linkCounter = counter;
        datasize = lnk.dOutput.dArrayLength * TFormat.formatSizeOf(dataFormat)
          + TFormat.getFormatHeaderSize(dataFormat);// size of incoming data in bytes
        doutsize = msgSizeInBytes - hdrSize; // size of data + msg in bytes
        extsize = 0; extptr = 0;
        pktsread++;
        boolean addbytes = false;
        TSubReqHdr sub = lnk.getSubscription();
        if (usrStamp < 0 && TErrorList.isMcaPropertyCoercive(retCode))
        { // trap this early
          lnk.linkStatus = retCode = TErrorList.illegal_device;
        }
        if (trace) lnk.traceLink("TSubRspHdr", "incoming with completion "+retCode);
        switch (retCode)
        { //check for coercion here ...
          case TErrorList.mcast_access_required:
            short mode = sub != null ? TMode.getBaseMode(sub.mode) : TMode.CM_CANCEL;
            switch (mode)
            { // strip off any CM_CONNECT, CM_STREAM, etc. ...
              case TMode.CM_TIMER:
              case TMode.CM_DATACHANGE:
              case TMode.CM_EVENT:
                sub.mode = (short)(mode | TMode.CM_NETWORK);
                break;
              default:
                lnk.close();
                break;
            }
            needLinkWatchDogCycle = true;
            break;
          case TErrorList.tcp_access_required:
            if (sub == null) break;
            sub.mode &= ~(TMode.CM_NETWORK);
            sub.mode |= TMode.CM_CONNECT;
            needLinkWatchDogCycle = true;
            break;            
          case TErrorList.call_redirection:
          case TErrorList.server_redirection:
            Arrays.fill(stsBuffer, (byte)0);
            d = stsBuffer;
            ds.read(d,0,stsSize); // redirection names
            int offset = 0;
            String ctx = null;
            String srv = null;
            String dev = null;
            String prp = null;
            if (isLegacy)
            { // redirection info squeezed into 32 chars
              if (d[offset] != 0) srv = getStsSting(d,offset,16);
              offset += 16;                
              if (d[offset] != 0) prp = getStsSting(d,offset,16);
            }
            else
            {
              if (d[offset] != 0) ctx = getStsSting(d,offset,TStrings.CONTEXT_NAME_SIZE);
              offset += TStrings.CONTEXT_NAME_SIZE;
              if (d[offset] != 0) srv = getStsSting(d,offset,TStrings.EXPORT_NAME_SIZE);
              offset += TStrings.EXPORT_NAME_SIZE;
              if (d[offset] != 0) dev = getStsSting(d,offset,TStrings.DEVICE_NAME_SIZE);
              offset += TStrings.DEVICE_NAME_SIZE;
              if (d[offset] != 0) prp = getStsSting(d,offset,TStrings.PROPERTY_NAME_SIZE);
            }
            if (!assertRedirectionValid(lnk,ctx,srv,dev,prp))
            {
              if (lnk.getRedirectionKey() != null)
              {
                MsgLog.log("TSubRspHdr", "redirection information already obtained for "+lnk.getFullDeviceNameAndProperty(),0,null,0);
                needLinkWatchDogCycle = true;
                break;                
              }
              MsgLog.log("TSubRspHdr", "redirection information for "+lnk.getFullDeviceNameAndProperty()+" is invalid ",0,null,0);
              lnk.linkStale = true;
              continue;
            }
            if (lnk.getRedirectionKey().length() == 0 && lnk.srvAddr.fecName.compareTo("GENS") == 0)
            { // update local group db cache
              getLinkFactory().addItemToGroupCache(lnk.cntName, lnk.expName, srv, lnk.devName);
            }
            getLinkFactory().addLinkToRedirectionList(lnk, ctx, srv, dev, prp);
            if (trace) lnk.traceLink("TPHdr", "redirect to /"+ctx+"/"+srv+"/"+dev+"["+prp+"]");
            if (ctx != null) lnk.cntName = ctx;
            if (srv != null) lnk.expName = srv;
            if (dev != null) lnk.devName = dev;
            if (prp != null) 
            {
              String meta = TMetaProperties.getMetaExtension(lnk.devProperty);
              if (meta != null && !TMetaProperties.isMetaProperty(prp)
                  && srv.compareToIgnoreCase("HISTORY") != 0) prp += meta; 
              lnk.devProperty = prp;              
            }
            lnk.rmvBucket();
            MsgLog.log("TSubRspHdr", "redirect "+lnk.getRedirectionKey()+" to "+lnk.getFullDeviceNameAndProperty(),0,null,1);
            needLinkWatchDogCycle = true;
            break;
          case TErrorList.has_bitfield_tag:
          case TErrorList.has_structure_tag:
            Arrays.fill(stsBuffer, (byte)0);
            d = stsBuffer;
            ds.read(d,0,TStrings.TAG_NAME_SIZE); // tag name
            String tag = getStsSting(d,0,TStrings.TAG_NAME_SIZE);//  new String(d).trim();
            lnk.dOutput.setTag(tag);
            needLinkWatchDogCycle = true;
            break;
          case TErrorList.property_is_mca_elevated:
          case TErrorList.property_is_mca:
            synchronized (TLinkFactory.mcaListMutex) {
            if (usrStamp < 0)
            {
              lnk.linkStatus = retCode = TErrorList.illegal_device;
              lnk.linkStale = true;
              break;
            }
            if (sysStamp < 0)
            { // gotta be robust !
              lnk.linkStatus = retCode = TErrorList.code_failure;
              lnk.linkStale = true;
              break;              
            }
            Arrays.fill(stsBuffer, (byte)0);
            d = stsBuffer;
            ds.read(d,0,stsSize); // is passed a device string ?
            String mcadevTag = getStsSting(d,0,8);
            if (mcadevTag.compareTo(TStrings.MCADEV_TAG) == 0)
            { // has been passed a device string
              dev = getStsSting(d,8,TStrings.DEVICE_NAME_SIZE);
            }
            else
            {
              dev = "#0";
            }
            TMcaLink mca = TLinkFactory.getMcaLinkForReuse(lnk.cntName,lnk.expName,dev,lnk.devProperty);
            if (lnk.dOutput.dFormat == TFormat.CF_DEFAULT) 
            {
              lnk.dOutput.dFormat = dataFormat;
              lnk.dOutput.setArrayLength(1);
              lnk.dOutput.setDataObject(1, dataFormat);
              lnk.dOutput.dCompletionLength = 1;
              //lnk.dOutput.blksin = 1; => kept = 0 => don't add to links that notify
              if (dataFormat == TFormat.CF_NULL) MsgLog.log("TPHdr","default format resolved to null format",retCode,null,0);
            }
            TLink plnk = null;
            if (mca == null)
            { // not yet known, make new (length given by system stamp)
            	boolean isElvFmt = (retCode == TErrorList.property_is_mca_elevated);
              mca = new TMcaLink(lnk,dev,sysStamp,isElvFmt);
              lnk.linkStatusSource = TLink.STATUS_LOCAL;
              lnk.linkStatus = mca.getParent().linkStatus;
              if (lnk.linkStatus == TErrorList.not_signalled || lnk.linkStatus < 0)
                lnk.linkStatus = 0;
              TLinkFactory.addMcaLink(mca);
              String msg = "attach to and create MCA parent "+mca.getParent().getFullDeviceNameAndProperty();
              if (trace) lnk.traceLink("TPHdr", msg);
              MsgLog.log("TSubRspHdr", msg,0,null,1);
            }
            else
            { // is known !
              lnk.linkStatusSource = TLink.STATUS_LOCAL;
              lnk.linkStatus = 0;
              plnk = mca.getParent();
              if (plnk == null)
              {
                lnk.linkStatus = TErrorList.code_failure;
                break;
              }
              lnk.linkStatus = plnk.linkStatus;
              lnk.hasObtainedStatus = plnk.hasNotifiedOnce;
              if (trace) lnk.traceLink("TPHdr", "attach to known MCA parent "+plnk.getFullDeviceNameAndProperty());
              int pollp = plnk.getLinkPollingInterval();
              int polld = lnk.getLinkPollingInterval();
              if (polld < pollp && plnk.isActive())
              { // faster rate required
                String msg = plnk.getFullDeviceNameAndProperty()+" change to faster polling interval: "+pollp+" -> "+polld+" msec";
                MsgLog.log("TSubRspHdr", msg,0,null,1);
                plnk.isLinkReassignment = true;
                plnk.attach(plnk.getSubscription().mode, mca, polld);
                plnk.linkStatus = 0;
                if (trace) lnk.traceLink("TSubRspHdr", "faster polling rate required for "+plnk.getFullDeviceNameAndProperty());
              }
            }
            if (mca != null)
            { // set the mcaIndex (from the user stamp)
              lnk.getSubscription().mode = TMode.CM_REGISTER;
              if (lnk.getMcaDevice() == null) lnk.setMcaDevice(dev);
              lnk.dOutput.setArrayLength(1);
              lnk.dOutput.dCompletionLength = 1;
              lnk.setBoundLink(mca.getParent());
              lnk.setMcaIndex(usrStamp+1);
              mca.add(lnk);
              String msg = "add "+lnk.getFullDeviceNameAndProperty()+" (id "+lnk.linkId+")" + " to mca parent "+mca.getParent().getFullDeviceNameAndProperty()+" (id "+mca.getParent().linkId+")";
              MsgLog.log("TSubRspHdr",msg,retCode,null,2);
              //if (TLinkFactory.debugLevel > 2) DbgLog.log("TSubRspHdr", "add "+lnk.getFullDeviceNameAndProperty()+" (id "+lnk.linkId+")" + " to mca parent "+mca.getParent().getFullDeviceNameAndProperty()+" (id "+mca.getParent().linkId+")");
              if (trace) lnk.traceLink("TSubRspHdr", "set MCA index to  "+usrStamp+1);
              if (lnk.hasDependents())
              { // re-form the dependents
                TLink tl;
                try
                {
                  Iterator<TLink> itr = lnk.getDependents().iterator();
                  while (itr.hasNext())
                  {
                    tl = itr.next();
                    tl.setMcaIndex(usrStamp+1);
                    tl.setMcaDevice(dev);
                    tl.dOutput.setArrayLength(1);
                    tl.dOutput.dCompletionLength = 1;
                    tl.setBoundLink(mca.getParent());
                    mca.add(tl);
                    msg = "set MCA index for doubly bound link to  "+usrStamp+1;
                    MsgLog.log("TSubRspHdr",msg,retCode,null,2);
                    if (trace) tl.traceLink("TSubRspHdr", msg);
                    itr.remove(); // remove it from the normal dependents list
                  }
                } 
                catch (Exception e) 
                { // just bail
                  MsgLog.log("TSubRspHdr.toStruct", "can't apply dependecies : "+e.getMessage(),TErrorList.code_failure,e,1);
                };                 
              }
            }
            if (plnk != null && plnk.hasNotifiedOnce)
            { // have to make sure the parent mca link gets notified !
              if (plnk.dOutput != null && plnk.dOutput.blksin == 0)
              { // likely due to DATACHANGE ACK. (a sendLinkRequest which doesn't send anything back)
                plnk.dOutput.blksin = plnk.dOutput.numblks;
                MsgLog.log("TSubRspHdr","reset MCA parent "+plnk.getFullDeviceNameAndProperty()+" output block counter",retCode,null,2);
              }
              plnk.hasNotifiedOnce = false;
              plnk.linkStale = true;
              plnk.needsEarlyNotification = true;
              lnk.needsEarlyNotification = true;
              lnks.add(plnk); // add him in the notifiers list ...
              String msg = "reset MCA parent "+plnk.getFullDeviceNameAndProperty()+" notification";
              MsgLog.log("TSubRspHdr",msg,retCode,null,2);
              if (trace) lnk.traceLink("TPHdr", msg);
            } }
            break; 
          case TErrorList.invalid_datarequest:
            lnk.canTimeout = false;
            if (lnk.dOutput.dArrayLength > sysStamp && lnk.dOutput.dFormat == usrStamp)
            {
              MsgLog.log("TPHdr", lnk.getFullDeviceNameAndProperty()+ " adjust data length "+lnk.dOutput.dArrayLength+" to "+sysStamp,retCode,null,1);
              lnk.dOutput.dArrayLength = sysStamp;
              needLinkWatchDogCycle = true;
            }
            else 
            {
              short f = TFormat.getFormatDataType((short)usrStamp);
              TLinkFactory.getInstance().addLinkToReLinkList(lnk, sysStamp, f);
              String fmt = TFormat.toString(lnk.dOutput.dFormat);
              MsgLog.log("TSubRspHdr", lnk.getFullDeviceNameAndProperty()+ " adjust data length and type from "+lnk.dOutput.dArrayLength+" "+fmt+
                  " to "+sysStamp+" "+TFormat.toString(f),retCode,null,1);              
              needLinkWatchDogCycle = true;              
            }
            break;
          case TErrorList.invalid_interval:
            MsgLog.log("TSubRspHdr", lnk.getFullDeviceNameAndProperty()+ " adjust polling interval from "+lnk.devTimeout+" to "+sysStamp,retCode,null,1);
            if (sub != null) sub.pollingInterval = sysStamp;
            lnk.devTimeout = sysStamp;
            needLinkWatchDogCycle = true;
            break;
          case TErrorList.reacquire_address:
            MsgLog.log("TSubRspHdr", lnk.getFullDeviceNameAndProperty()+ " received reacquire address",retCode,null,1);
            needLinkWatchDogCycle = true;
            break;            
          case TErrorList.illegal_protocol:
            needLinkWatchDogCycle = true; // watchdog thread will decrement it!
            break;
          case TErrorList.get_subscription_id:
            lnk.srvId = Swap.Short(ds.readShort());
            break;
          case TErrorList.non_existent_elem:
            lnk.linkInvalidCount++;
          default:
            // non-zero code
            lnk.linkStale = true;
            if (doutsize < stsSize && lnk.dOutput.blksin == 0)
            { // no accompanying status string ? 
              lnk.dOutput.bytesin = 0;
              continue;
            }
            if ((retCode & TErrorList.CE_SENDDATA) != TErrorList.CE_SENDDATA)
            { // no accompanying data
              datasize = 0;
            }
            extsize = 0;
            extbytes = doutsize*blknum - datasize;
            //TODO: this logic is faulty: datasize refers to original request
            if (blknum == numblks || extbytes > 0)
            { // either last or next to last block ...
              if (blknum == numblks)
              { // last block contains the last piece (or all) of the status string
                if (numblks == 1 && doutsize < stsSize) 
                { // how ?
                  continue;
                }
                extsize = doutsize < stsSize ? doutsize : stsSize;
                extptr = stsSize - extsize;
              }
              else if (extbytes > 0)
              { // this will occasionally be true (2nd to last block has some room)
                extsize = extbytes;
                extptr = 0;
              }
              doutsize -= extsize;
              lnk.dOutput.bytesin += extsize;
            }
            if (datasize == 0)
            { // as set above -> don't fall thru
              lnk.dOutput.blknum = blknum;
              lnk.dOutput.blksin = 1;
              lnk.dOutput.numblks = 1;
              break;
            }
          case 0:
            if (lnk.isMcaParent()) lnk.needsEarlyNotification = false;
            lnk.linkStale = true;
            // push this data set into the corresponding TDataType object
            int sizeInBytes = msgSizeInBytes-hdrSize-extsize;
            if (sizeInBytes > bBuffer.length) bBuffer = new byte[sizeInBytes+TPacket.MAX_DATAGRAM_SIZE];
            d = bBuffer;
            ds.read(d,0,sizeInBytes);
            if (lnk.dOutput.dFormat == TFormat.CF_DEFAULT) synchronized (lnk.dOutput)
            { // contract not yet re-formulated
              if (dataFormat == TFormat.CF_NULL) MsgLog.log("TSubRspHdr","default format resolved to null format",retCode,null,0);
              lnk.dOutput.dFormat = dataFormat;
              if ((fmtsize=TFormat.formatSizeOf(lnk.dOutput.dFormat)) < 1) continue;
              if (dataFormat == TFormat.CF_MDA)
              { // a very special case for CF_DEFAULT logic ...
                byte[] ba = new byte[4];
                System.arraycopy(d, MDA.mda_fmt_offset, ba, 0, 4);
                short fmt = (short)Swap.Int(ba);
                fmtsize = TFormat.formatSizeOf(fmt);
                lnk.dOutput.setTag(TFormat.toString(fmt));
              }
              boolean needDataSize =  (dataFormat != TFormat.CF_NULL && dataSize == 0); 
              if (lnk.getTineProtocol() <= TTransport.legacyProtocolLevel || needDataSize)
              { // worst case array length calculation :
                int ids = dataFormat == TFormat.CF_TEXT ? datasize : (doutsize/fmtsize);
                dataSize = ids * lnk.dOutput.numblks;
                dataSize -= TFormat.getFormatHeaderSize(dataFormat)/fmtsize;
              }              
              lnk.dOutput.dArrayLength = dataSize;
              if (dataFormat != TFormat.CF_STRUCT && lnk.dOutput.getDataObject() == null)
              {
                lnk.dOutput.setDataObject(lnk.dOutput.dArrayLength,lnk.dOutput.dFormat);
              }
              if (lnk.hasDependents())
              { // already has dependents ? (a jDDD specialty)
                TLink xlnk;
                ConcurrentLinkedQueue<TLink> xlst = lnk.getDependents();
                TDataType dout;
                Iterator<TLink> itr = xlst.iterator();
                while (itr.hasNext())                    
                {
                  if ((xlnk = itr.next()) == null) continue;
                  dout = xlnk.getOutputDataObject();
                  if (dout.dFormat != TFormat.CF_DEFAULT) continue;
                  int dlen = lnk.dOutput.dArrayLength;
                  if (xlnk.getMcaDevice() != null) dlen = 1;
                  if (lnk.dOutput.dFormat == TFormat.CF_MDA) dout.setTag(lnk.dOutput.getTag());
                  dout.setDataObject(dlen,lnk.dOutput.dFormat);
                  dout.dArrayLength = dlen;
                  dout.dFormat = lnk.dOutput.dFormat;
                  dout.dTimestamp = (long) dTimeStamp * 1000 + (long) (dTimeStampUSEC / 1000); 
                  dout.timestamp = dTimeStamp;
                  dout.timestampMSEC = dTimeStampUSEC / 1000;
                  dout.timestampUSEC = dTimeStampUSEC;
                  dout.usrDataStamp = usrStamp;
                  dout.sysDataStamp = sysStamp;
                  dout.xferReason = xferReason;
                }
              }
              // will be adjusted properly in InterpretIncomingData() from the factory
              lnk.adjustDefaultValues = true;
            }
            addbytes = lnk.dOutput.update(d,0,sizeInBytes,blknum,counter);
            break;
        }  
        if (extsize > 0)
        { // there is an accompanying status string
          d = new byte[extsize];
          ds.read(d,0,d.length);
          if (extptr + extsize <= TStrings.STATUS_SIZE)
          {
            System.arraycopy(d,0,lnk.linkStatusStringBuffer,extptr,extsize);
          }
          else
          {
            MsgLog.log("TSubRspHdr.toStruct", "status string size mismatch: "+extsize+" bytes at position "+extptr,TErrorList.remitted_data_lost,null,0);
          }
        }      
        if (applyStatusLater) lnk.linkStatus = retCode;
        if (lnk.linkStatus == TErrorList.link_blacklisted)
        { // link has been black-listed : overwrite any incoming status string
          d = TErrorList.getErrorString(TErrorList.link_blacklisted).getBytes();
          extsize = d.length; extptr = 0;
          if (extsize > lnk.linkStatusStringBuffer.length) extsize = lnk.linkStatusStringBuffer.length;
          System.arraycopy(d,0,lnk.linkStatusStringBuffer,extptr,extsize);
        }
        if (addbytes) lnk.dOutput.bytesin += doutsize;
        if (lnk.dOutput.blksin == lnk.dOutput.numblks)
        {
          // lnk.dOutput.bytesin = 0; < not here! reset in InterpretIncomingData().
          lnk.canTimeout = false;
          lnk.dOutput.resetBuffersReady();
          lnk.dOutput.timestamp = dTimeStamp;
          lnk.dOutput.timestampMSEC = dTimeStampUSEC / 1000;
          lnk.dOutput.timestampUSEC = dTimeStampUSEC;
          lnk.dOutput.dTimestamp = (long) dTimeStamp * 1000 + (long) (dTimeStampUSEC / 1000);
          lnk.dOutput.usrDataStamp = usrStamp;
          lnk.dOutput.sysDataStamp = sysStamp;
          lnk.dOutput.xferReason = xferReason;
          lnk.getSubscription().linkLastTime = ltime;
          lnks.add(lnk);
          if (TLinkFactory.debugLevel > 1) DbgLog.log("TSubRspHdr","adding link "+lnk.getFullDeviceNameAndProperty()+" to incoming links list");
        }
      }
      if (TLinkFactory.debugLevel > 1) DbgLog.log("TSubRspHdr",pktsread + " incoming contracts scanned");
      ds.close();
      instream.close();
      if (needLinkWatchDogCycle) TLinkFactory.watchdogCycle();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("TSubRspHdr.toStruct",e.toString(),TErrorList.code_failure,e,0);
    }
    return lnks.toArray(new TLink[0]);
  }                  
  TLinkFactory tf;
 /**
 * Attach the object to a client link factory ...
 * Creation date: (8/21/01 6:33:36 PM)
 */
  public TSubRspHdr(TLinkFactory tfactory) 
  {
    tf = tfactory;
  }
  // going out (server-side) ...
  public TSubRspHdr(int msgsize,TClientEntry clnTblEntry,TContractTable conTblEntry)
  {
    boolean isStream = TTransport.isStream(clnTblEntry.cln.inetProtocol);
    msgSizeInBytes = msgsize;
    tineProtocol = TTransport.thisProtocolLevel; 
    if (clnTblEntry != null && clnTblEntry.sts != null)
    {
      subId = clnTblEntry.sts.id;
      retCode = clnTblEntry.sts.statusCode;
      if (retCode != 0) stsSize = TStrings.STATUS_SIZE;
      numblks = clnTblEntry.sts.numblks;
      blknum = clnTblEntry.sts.blknum;
      mtu = clnTblEntry.sts.mtu;
      counter = clnTblEntry.sts.counter;
      starttime = (int)(clnTblEntry.sts.starttime/1000);
      xferReason = clnTblEntry.sts.Stale;
      stsCode = (short)TErrorList.getErrorCode(conTblEntry.compStatus);      
    }
    if (conTblEntry != null)
    {
      tineProtocol = clnTblEntry.cln.tineProtocol;
      dTimeStampUSEC = (int)((conTblEntry.dtimestamp%1000)*1000);
      dTimeStamp = (int)(conTblEntry.dtimestamp/1000);
      usrStamp = conTblEntry.usrstamp;
      sysStamp = conTblEntry.sysstamp;
      TContract con = conTblEntry.getContract();
      if (con != null) 
      {
        dataFormat = con.dataFormatOut;
        dataSize = con.dataSizeOut;
      }
    }
    stream = new ByteArrayOutputStream(hdrSizeInBytes);
    if (isStream) numblks = blknum = 1;
    prepare(tineProtocol);
  }
}