package de.desy.tine.histUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class DAQFltr extends TTaggedStructure
{
  public static final int STR_COMP_MATCHES = 0;
  public static final int STR_COMP_BEGINS = 1;
  public static final int STR_COMP_ENDS = 2;
  public static final int STR_COMP_CONTAINS = 3;  
  public void clear()
  {
    keyString = null;
    tagString = null;
    descString = null;
    sValidString = null;
  }
  private char[] tag = new char[64]; /**< the filter tag (option filter database entry) */
  private String tagString;
  private char[] key = new char[64]; /**< the database parameter keyword providing the data source */
  private String keyString;
  private char[] desc = new char[64]; /**< description of the filter */
  private String descString;
  private char[] sValid = new char[64]; /**< for string parameters: the comparison string */
  private String sValidString;
  private int[] mask = new int[1]; /**< bit position in the validity check */
  private int[] valid = new int[1]; /**< boolean value */
  private int[] active = new int[1]; /**< filter is being used by at least one record */
  private int[] sValidCmp = new int[1]; /**< for string parameters: one of STR_COMP_CONTAINS, STR_COMP_BEGINS, STR_COMP_ENDS, STR_COMP_MATCHES */
  private int[] fmtValidMin = new int[1]; /**< keyword format for min comparison */
  private int[] fmtValidMax = new int[1]; /**< keyword format for max comparison */
  private double[] fValidMin = new double[1]; /**< for number parameters: minimum valid parameter */
  private double[] fValidMax = new double[1]; /**< for number parameters: maximum valid parameter */
  private void initStructure()
  {
    addField(tag,"tag");
    addField(key,"key");
    addField(desc,"desc");
    addField(sValid,"sValid");
    addField(mask,"mask");
    addField(valid,"valid");
    addField(active,"active");
    addField(sValidCmp,"sValidCmp");
    addField(fmtValidMin,"fmtValidMin");
    addField(fmtValidMax,"fmtValidMax");
    addField(fValidMin,"fValidMin");
    addField(fValidMax,"fValidMax");
    initDone();    
  }
  public DAQFltr()
  {
    super();
    initStructure();
  }
  DAQFltr(DAQFltr reference)
  {
    super();
    initStructure();
    setKey(reference.getKey());
    setTag(reference.getTag());
    setDescription(reference.getDescription());
    setValidString(reference.getValidString());
    mask[0] = reference.mask[0];
    valid[0] = reference.valid[0];
    active[0] = reference.active[0];
    sValidCmp[0] = reference.sValidCmp[0];
    fmtValidMax[0] = reference.fmtValidMax[0];
    fmtValidMin[0] = reference.fmtValidMin[0];
    fValidMax[0] = reference.fValidMax[0];
    fValidMin[0] = reference.fValidMin[0];
  }
  public String getKey()
  {
    if (keyString == null) keyString = new String(key).trim();
    return keyString;
  }
  public void setKey(String newKey)
  {
    pushChars(newKey, key);
    keyString = new String(newKey);
  }
  public String getTag()
  {
    if (tagString == null) tagString = new String(tag).trim();
    return tagString;
  }
  public void setTag(String newTag)
  {
    pushChars(newTag, tag);
    tagString = new String(newTag);
  }
  public String getValidString()
  {
    if (sValidString == null) sValidString = new String(sValid).trim();
    return sValidString;
  }
  public void setValidString(String validString)
  {
    pushChars(validString, sValid);
    sValidString = new String(validString);
  }
  public String getDescription()
  {
    if (descString == null) descString = new String(desc).trim();
    return descString;
  }
  public void setDescription(String newDescription)
  {
    pushChars(newDescription, desc);
    descString = new String(newDescription);
  }
  public int getMask() { return mask[0]; }
  public void setMask(int value) { mask[0] = value; }
  public int getValid() { return valid[0]; }
  public void setValid(int value) { valid[0] = value; }
  public int getActive() { return active[0]; }
  public void setActive(int value) { active[0] = value; }
  public int getValidStringComparitor() { return sValidCmp[0]; }
  public void setValidStringComparitor(int value) { sValidCmp[0] = value; }
  public int getValidMinFormat() { return fmtValidMin[0]; }
  public void setValidMinFormat(int value) { fmtValidMin[0] = value; }
  public int getValidMaxFormat() { return fmtValidMax[0]; }
  public void setValidMaxFormat(int value) { fmtValidMax[0] = value; }
  public double getValidMinimum() { return fValidMin[0]; }
  public void setValidMinimum(double value) { fValidMin[0] = value; }
  public double getValidMaximum() { return fValidMax[0]; }
  public void setValidMaximum(double value) { fValidMax[0] = value; }
}
