package de.desy.tine.histUtils;

import de.desy.tine.definitions.*;
import de.desy.tine.structUtils.TTaggedStructure;

public class DAQPrm extends TTaggedStructure
{
  public void clear()
  {
    keyString = null;
    unitsString = null;
    descString = null;
    plotString = null;
    boundToString = null;
    subString = null;
    assocString = null;
  }
  private char[] key = new char[TStrings.PROPERTY_NAME_SIZE]; /* parameter name */
  private String keyString;
  private char[] units = new char[TStrings.UNITS_SIZE];       /* units */
  private String unitsString;
  private char[] desc = new char[TStrings.PROPERTY_DESC_SIZE]; /* description */
  private String descString;
  private char[] plot = new char[TStrings.UNITS_SIZE];        /* plot style (LOG, LIN, etc.) */
  private String plotString;
  private char[] boundTo = new char[TStrings.PROPERTY_NAME_SIZE]; /* bound to parameter */
  private String boundToString;
  private char[] sub = new char[TStrings.SUBSYSTEM_NAME_SIZE]; /* parameter subsystem */
  private String subString;
  private char[] assoc = new char[TStrings.PROPERTY_NAME_SIZE]; /* associate dev names to */
  private String assocString;
  private int[] fmt = new int[1];              /* allowable format */
  private int[] size = new int[1];             /* data array size assuming given format */
  private float[] scaleFactor = new float[1];
  private float[] scaleOffset = new float[1];
  private float[] maximum = new float[1];       /* maximum value for parameter */
  private float[] minimum = new float[1];       /* minimun value for parameter */
  private float[] aTolerance = new float[1];    /* absolute tolerance in units given */
  private float[] pTolerance = new float[1];    /* percent tolerance */
  private void initStructure()
  {
    addField(key,"key");
    addField(units,"units");
    addField(desc,"desc");
    addField(plot,"plot");
    addField(boundTo,"boundTo");
    addField(sub,"sub");
    addField(assoc,"assoc");
    addField(fmt,"fmt");
    addField(size,"size");
    addField(scaleFactor,"scaleFactor");
    addField(scaleOffset,"scaleOffset");
    addField(maximum,"maximum");
    addField(minimum,"minimum");
    addField(aTolerance,"aTolerance");
    addField(pTolerance,"pTolerance");
    initDone();    
  }
  public DAQPrm()
  {
    super();
    initStructure();
  }
  public DAQPrm(DAQPrm reference)
  {
    super();
    initStructure();
    setKey(reference.getKey());
    setUnits(reference.getUnits());
    setDescription(reference.getDescription());
    setPlotType(reference.getPlotType());
    setBoundTo(reference.getBoundTo());
    setAssociate(reference.getAssociate());
    setSubSystem(reference.getSubSystem());
    fmt[0] = reference.fmt[0];
    size[0] = reference.size[0];
    scaleFactor[0] = reference.scaleFactor[0];
    scaleOffset[0] = reference.scaleOffset[0];
    maximum[0] = reference.maximum[0];
    minimum[0] = reference.minimum[0];
    aTolerance[0] = reference.aTolerance[0];
    pTolerance[0] = reference.pTolerance[0];
  }
  public String getKey()
  {
    if (keyString == null) keyString = new String(key).trim();
    return keyString;
  }
  public void setKey(String newKey)
  {
    pushChars(newKey, key);
    keyString = new String(newKey);
  }
  public String getUnits()
  {
    if (unitsString == null) unitsString = new String(units).trim();
    return unitsString;
  }
  public void setUnits(String newUnits)
  {
    pushChars(newUnits, units);
    unitsString = new String(newUnits);
  }
  private void validateDescription()
  {
    for (int i=0; i<desc.length; i++)
    { // replay any stray commas ...
      if (desc[i] == ',') desc[i] = ' ';
    }
  }
  private String getStringFromAsciiChars(char[] ascii)
  {
    if (ascii == null || ascii.length == 0) return "";
    byte[] bytes = new byte[ascii.length];
    for (int i=0; i<ascii.length; i++) bytes[i] = (byte)ascii[i];
    String s = "";
    try { s = new String(bytes,"ISO-8859-1").trim(); } catch (Exception ignore) {};
    return s;
  }
  public String getDescription()
  {
    if (descString == null) 
    {
      validateDescription();
      //descString = new String(desc).trim();
      descString = getStringFromAsciiChars(desc);
    }
    return descString;
  }
  public void setDescription(String newDescription)
  {
    pushChars(newDescription, desc);
    validateDescription();
    //descString = new String(desc).trim();
    descString = getStringFromAsciiChars(desc);
  }
  public String getPlotType()
  {
    if (plotString == null) plotString = new String(plot).trim();
    return plotString;
  }
  public void setPlotType(String plotType)
  {
    pushChars(plotType, plot);
    plotString = new String(plotType);
  }
  public String getBoundTo()
  {
    if (boundToString == null) boundToString = new String(boundTo).trim();
    return boundToString;
  }
  public void setBoundTo(String newBoundTo)
  {
    pushChars(newBoundTo, boundTo);
    boundToString = new String(newBoundTo);
  }
  public String getSubSystem()
  {
    if (subString == null) subString = new String(sub).trim();
    return subString;
  }
  public void setSubSystem(String subsystem)
  {
    pushChars(subsystem, sub);
    subString = new String(subsystem);
  }
  public String getAssociate()
  {
    if (assocString == null) assocString = new String(assoc).trim();
    return assocString;
  }
  public void setAssociate(String associate)
  {
    pushChars(associate, assoc);
    assocString = new String(associate);
  }
  public int getSize() { return size[0]; }
  public void setSize(int value) { size[0] = value; }
  public int getFormat() { return TFormat.makeByteFormatCode(fmt[0]); }
  public void setFormat(int value) { fmt[0] = TFormat.makeStdFormatCode(value); }
  public float getScaleFactor() { return scaleFactor[0]; }
  public void setScaleFactor(float value) { scaleFactor[0] = value; }
  public float getScaleOffset() { return scaleOffset[0]; }
  public void setScaleOffset(float value) { scaleOffset[0] = value; }
  public float getMaximum() { return maximum[0]; }
  public void setMaximum(float value) { maximum[0] = value; }
  public float getMinimum() { return minimum[0]; }
  public void setMinimum(float value) { minimum[0] = value; }
  public float getAbsTolerance() { return aTolerance[0]; }
  public void setAbsTolerance(float value) { aTolerance[0] = value; }
  public float getPctTolerance() { return pTolerance[0]; }
  public void setPctTolerance(float value) { pTolerance[0] = value; }
  public void setFormat(String stringValue)
  {
    setFormat(TFormat.makeStdFormatCode(stringValue));
  }
  public String getToleranceAsString()
  {
    if (aTolerance[0] == 0 && pTolerance[0] == 0) return "0";
    float atol = Math.abs(aTolerance[0]);
    float ptol = Math.abs(pTolerance[0]);
    String tolstr = "";
    if (atol > 0) tolstr += atol;
    if (ptol > 0)
    {
      if (tolstr.length() > 0) tolstr += " + ";
      tolstr += ptol + "%";
    }
    return tolstr;   
  }
  public void setAbsTolerance(String stringValue)
  {
    aTolerance[0] = 0;
    int p = stringValue.indexOf('%');
    try
    {
      if (p < 0)
      {
        aTolerance[0] = Float.parseFloat(stringValue);
      }
      else
      {
        int p1 = stringValue.indexOf('+');
        if (p1 >= 0)
        {
          if (p1 < p)
          {
            aTolerance[0] = Float.parseFloat(stringValue.substring(0, p1));
          }
          else
          {
            aTolerance[0] = Float.parseFloat(stringValue.substring(p1+1));
          }
        }
      }
    } catch (Exception ignore) {}
  }
  public void setPctTolerance(String stringValue)
  {
    pTolerance[0] = 0;
    int p = stringValue.indexOf('%');
    try
    {
      if (p >= 0)
      {
        int p1 = stringValue.indexOf('+');
        if (p1 < 0)
        {
          pTolerance[0] = Float.parseFloat(stringValue.substring(0,p));        
        }
        else
        {
          if (p1 < p)
          {
            pTolerance[0] = Float.parseFloat(stringValue.substring(p1+1, p));
          }
          else
          {
            pTolerance[0] = Float.parseFloat(stringValue.substring(0,p));
          }
        }
      }
    } catch (Exception ignore) {}
  }
  public String getFormatAsString()
  {
    return TFormat.toString((short)fmt[0]);
  }
}
