package de.desy.tine.histUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class PrfCfgEx extends TTaggedStructure
{
  public static final int PRF_ALWAYS_VISIBLE = 0x01;
  public static final int PRF_NON_OPERATIONAL = 0x02;
  public static final int PRF_EXCLUDE_STATS = 0x04;
  public static final int PRF_PROBLEMS = 0x08;
  public static final int PRF_NOT_AVAILABLE = 0x10;
  public static final int PRF_WARMUP = 0x20;
  public void clear()
  {
    nameString = null;
    typeString = null;
    statesString = null;
    colorString = null;
    metric[0] = 0;
    flags[0] = 0;
  }
  private char[] name = new char[64];
  private String nameString;
  private char[] type = new char[16];
  private String typeString;
  private char[] statesBuffer = new char[512];
  private String statesString;
  private char[] color = new char[16];
  private String colorString;
  private int[] metric = new int[1];
  private int[] flags = new int[1];
  private int[] reserved1 = new int[1];
  private int[] reserved2 = new int[1];
  private void initStructure()
  {
    addField(name,"name");
    addField(type,"type");
    addField(statesBuffer,"states");
    addField(color,"color");
    addField(metric,"metric");
    addField(flags,"flags");
    addField(reserved1,"reserved1");
    addField(reserved2,"reserved2");
    initDone();
  }
  public PrfCfgEx()
  {
    super();
    initStructure();
  }
  public PrfCfgEx(PrfCfgEx reference)
  {
    super();
    initStructure();
    System.arraycopy(reference.name, 0, name, 0, 64);
    System.arraycopy(reference.type, 0, type, 0, 16);
    System.arraycopy(reference.statesBuffer, 0, statesBuffer, 0, 512);
    System.arraycopy(reference.color, 0, color, 0, 16);
    metric[0] = reference.metric[0];
    flags[0] = reference.flags[0];
    reserved1[0] = reference.reserved1[0];
    reserved2[0] = reference.reserved2[0];
  }
  public PrfCfgEx(PrfCfg reference)
  {
    super();
    initStructure();
    setProfileName(reference.getProfileName());
    setType(reference.getType());
    setStates(reference.getStates());
    setColor(reference.getColor());
    metric[0] = 0;
    flags[0] = 0;
    reserved1[0] = 0;
    reserved2[0] = 0;
  }
  public int getMetric() { return metric[0]; }
  public void setMetric(int metric)
  {
    this.metric[0] = metric;
  }
  public int getFlags() { return flags[0]; }
  public void setFlags(int flags)
  {
    this.flags[0] = flags;
  }
  public boolean isAlwaysVisible()
  { // display profile in legend even if no counts
    return ((flags[0] & PRF_ALWAYS_VISIBLE) == PRF_ALWAYS_VISIBLE);
  }
  public boolean isNonOperational()
  { // service mode
    return ((flags[0] & PRF_NON_OPERATIONAL) == PRF_NON_OPERATIONAL);
  }
  public boolean isExcludedInStatistics()
  { // availability ignores this profile 
    return ((flags[0] & PRF_EXCLUDE_STATS) == PRF_EXCLUDE_STATS);
  }  
  public boolean isProblemStatistics()
  { // e.g. 'problems' : corrections into this state => corrections in alarm state
    return ((flags[0] & PRF_PROBLEMS) == PRF_PROBLEMS);
  }
  public boolean isNotAvailableStatistics()
  { // e.g. 'problems', 'warming up' : machine is not available if in one of these states
    return ((flags[0] & PRF_NOT_AVAILABLE) == PRF_NOT_AVAILABLE);
  } 
  public boolean isWarmup()
  { // e.g. 'problems', 'warming up' : machine is not available if in one of these states
    return ((flags[0] & PRF_WARMUP) == PRF_WARMUP);
  } 
  public String getProfileName()
  {
    if (nameString == null) nameString = new String(name).trim();
    return nameString;
  }
  public void setProfileName(String name)
  {
    Util.pushChars(name,this.name);
    this.nameString = new String(name);
  }
  public String getType()
  {
    if (typeString == null) typeString = new String(type).trim();
    return typeString;
  }
  public void setType(String type)
  {
    Util.pushChars(type,this.type);
    this.typeString = new String(type);
  }
  public String getStatesString()
  {
    if (statesString == null) statesString = new String(statesBuffer).trim();
    return statesString;
  }
  public String[] getStates()
  {
    if (statesString == null) statesString = new String(statesBuffer).trim();
    return statesString.split(",");
  }
  public void setStates(String[] states) {
      StringBuilder sb = new StringBuilder(512);
      for (String s : states) {
          sb.append(s).append(',');
      }
      String st = sb.length() > 0 ? sb.substring(0,sb.length()-1) : "";
      Util.pushChars(st,this.statesBuffer);
      this.statesString = new String(st);
  }
  public String getColor()
  {
    if (colorString == null) colorString = new String(color).trim();
    return colorString;
  }
  public void setColor(String color) {
      Util.pushChars(color,this.color);
      this.colorString = new String(color);
  }

}
