package de.desy.tine.histUtils;

import java.util.Arrays;

import de.desy.tine.client.*;
import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.exceptions.TineRuntimeErrorException;
import de.desy.tine.queryUtils.TPropertyQuery;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.server.logger.DbgLog;
import de.desy.tine.server.properties.TMetaProperties;
import de.desy.tine.types.*;

public class TrendData implements TLinkCallback
{
  protected void finalize() throws Throwable
  { // is this of any use at all ?
    if (lnk != null) lnk.close();
  }
  private TLink lnk = null;
  private int lid = -1; // link id
  private boolean isActive = false;
  private TDataType dout;
  protected TCompoundDataObject[] data;
  protected TCompoundDataObject nulldata;
  protected THistorySource src;
  protected int length;
  private int depthInSeconds;
  @SuppressWarnings("unused")
  private int interval;
  private int trendFormat;
  private int dataFormat;
  private float aTolerance;
  private float pTolerance;
  private long start;
  private long lastAccess = 0;
  private static final int ALLOWED_DEADTIME = 600000; // 10 minutes in milliseconds
  public boolean IsActive() { return isActive; }
  public TCompoundDataObject[] getTrendData() { return data; }
  public void setTrendData(TCompoundDataObject[] values) 
  { synchronized (data) {
    if (values == null || values.length == 0 || values[0] == null) return;
    if (trendFormat != values[0].getFormat()) return;
    if (values.length != data.length) return;
    data = values; 
  }}
  public long getTrendStart() { return start; }
  protected void setTrendStart(long value) { if (value > 0) start = value; }
  public int getTrendFormat() { return trendFormat; }
  public int getDataFormat() { return dataFormat; }
  public TLink getLink() { return lnk; }
  public void touch()
  {
    lastAccess = System.currentTimeMillis();
  }
  public double getCurrentValue()
  {
    int cc = 0;
    //lastAccess = System.currentTimeMillis();
    if (dout == null) return 0.0;
    //if ((cc=lnk.executeAndListen(interval)) == 0)
    if ((cc=lnk.getLinkStatus()) == 0)
    {
      double[] v = new double[1];
      dout.getData(v);
      return v[0];
    }
    throw new TineRuntimeErrorException(cc, "cannot get current value!");
  }
  public double getCurrentTimestamp()
  {
    double ts = 0.0;
    if (dout != null) ts = TDataTime.getDataTimeStamp(dout.getDataTimeStamp());
    if (ts < 100) ts = TDataTime.getDataTimeStamp(); // 0 or strange result from data output
    return ts;
  }
  public int getCurrentSystemStamp()
  {
    if (dout == null) return 0;
    return dout.getSystemDataStamp();    
  }
  public int getDepthInSeconds() { return depthInSeconds; }
  protected void setDepthInSeconds(int value) { if (value > 0) depthInSeconds = value; }
  public int getDepthInPoints() { return data.length; }
  public TrendData(String context, String server, String device, String property, TCompoundDataObject[] data, int depthInSeconds,int interval)
  {
    if (data == null || data.length == 0 || depthInSeconds <= 0)
    {
      throw new TineRuntimeErrorException(TErrorList.invalid_parameter, "input arguments not valid!");    
    }
    if (data instanceof DBLTIME[])
    {
      trendFormat = TFormat.CF_DBLTIME;     
      nulldata = new DBLTIME();
    }
    else if (data instanceof DBLDBL[])
    {
      trendFormat = TFormat.CF_DBLDBL;
      nulldata = new DBLDBL();
    }
    else if (data instanceof DBLDBLDBL[])
    {
      trendFormat = TFormat.CF_DBLDBLDBL;      
      nulldata = new DBLDBLDBL();
    }
    else
    {
      throw new TineRuntimeErrorException(TErrorList.illegal_format, "type of given trend data object is not allowed!");
    }
    if (TMetaProperties.isMetaProperty(property))
    { // don't trend a meta-property !
      property = TMetaProperties.getTargetProperty(property);
    }
    this.data = data;
    this.depthInSeconds = depthInSeconds;
    this.interval = interval;
    this.start = System.currentTimeMillis()/1000 - depthInSeconds;
    String tgt = "/"+context+"/"+server+"/"+device;
    TPropertyQuery[] p = TQuery.getPropertyInformation(context,server,device,property);
    if (p == null)
    {
      throw new TineRuntimeErrorException(TErrorList.illegal_property, "can't get property information for "+tgt+"["+property+"]");
    }
    dataFormat = TFormat.makeByteFormatCode(p[0].prpFormat);
    String tag = p[0].prpTag;
    dout = new TDataType(1,(short)dataFormat,tag);
    lnk = new TLink(tgt,property,dout,null,TAccess.CA_READ);
    lastAccess = System.currentTimeMillis();
    isActive = true;
    if ((lid=lnk.attach(TMode.CM_TIMER, this, interval)) < 0)
    //if (TLinkFactory.getInstance().getListener(lnk,TMode.CM_TIMER,interval) == null)
    {
      isActive = false;
      throw new TineRuntimeErrorException(TErrorList.code_failure, "can't start listening link: "+TErrorList.toString(-lid));
    }
    double stop = TDataTime.getDataTimeStamp();
    double start = stop - (double)depthInSeconds;
    FLTFLT ff = THistory.getToleranceOverInterval(context,server,property,start,stop,1000);
    aTolerance = Math.abs(ff.f1val);
    pTolerance = Math.abs(ff.f2val);
    if (aTolerance == 0 && pTolerance == 0) pTolerance = 0.1F;
  }
  public void shift(int num)
  { // shift points
    if (length < 0 || num < 0) return;
    synchronized (data) 
    {
      for (int i=0; i<data.length-num && i<length-num; i++) data[i] = data[i+num];
      length -= num;    
    }
  }
  public void shift(long seconds)
  {
    if (length < 0 || seconds < 0) return;
    int n = 0;
    start += seconds;
    for (n=0; n<length && timeAt(n)<(double)start; n++);
    if (n > 0) shift(n);
  }
  public void append(TCompoundDataObject[] values)
  {
    if (values == null || values.length == 0)
    {
      throw new TineRuntimeErrorException(TErrorList.illegal_format, "no data to append!");      
    }
    if (values[0].getFormat() != trendFormat)
    {
      throw new TineRuntimeErrorException(TErrorList.illegal_format, "type of appended data object is not equal to stored data!");
    }
    if (values.length > data.length)
    {
      throw new TineRuntimeErrorException(TErrorList.illegal_data_size, "number of values to append is larger then the trend buffer!");      
    }
    if (values.length > data.length - length)
    { // no room ! => back up
      shift(values.length);
    }
    synchronized (data) 
    {
      for (int i=0; i<values.length; i++)
      {
        data[length] = values[i];
      }
      length += values.length;
    }
  }
  protected double valueAt(int index)
  { synchronized (data) {
    switch (trendFormat)
    {
      case TFormat.CF_DBLDBL:
        return ((DBLDBL[])data)[index].d1val;
      case TFormat.CF_DBLTIME:
        return ((DBLTIME[])data)[index].d1val;
      case TFormat.CF_DBLDBLDBL:
        return ((DBLTIME[])data)[index].d1val;
      default:
        break;
    }
    return 0.0;    
  } }
  protected double timeAt(int index)
  { synchronized (data) {
    return getttime(index);
  } }
  private double getttime(int index)
  { 
    switch (trendFormat)
    {
      case TFormat.CF_DBLDBL:
        return ((DBLDBL[])data)[index].d2val;
      case TFormat.CF_DBLTIME:
        return ((DBLTIME[])data)[index].d2val;
      case TFormat.CF_DBLDBLDBL:
        return ((DBLTIME[])data)[index].d2val;
      default:
        break;
    }
    return 0.0;
  }
  int indexAt(double time)
  { synchronized (data) {
    for (int i=0, n=length-1; i<length && n>0; i++, n--)
    {
      if (getttime(i) > time) return i;
      if (getttime(n) < time) return n; 
    }
    return length > 0 ? length-1 : 0;
  } }
  private double pruningThreshold = 0.8; // 80 %
  public void setPruningThreshold(double value)
  {
    if (value < 0) return; // not accepted;
    if (value > 100) value /= 100.0; // was thinking in percent ?
    if (value < 0.1) value = 0.1; // impose reasonable lower limit
    if (value > 0.95) value = 0.95; // impose reasonable upper limit
    pruningThreshold = value;
  }
  public double getPruningThreshold()
  {
    return pruningThreshold;
  }
  public void prune()
  { synchronized (data) {
    double spacing = ((double)depthInSeconds / (double)data.length);
    int i = 0, n = 1;
    double stop = getCurrentTimestamp();
    double start = stop - (double)depthInSeconds;
    // check to see if the older points are in range ...
    if (start > 0) for (i = 0; i < length && timeAt(i) < start; i++);
    if (i >= data.length)
    { // wants to start past the most recent data point? -> weird, weird, weird !
      i = 0;  // then prune this from the known beginning
    }
    data[0] = data[i];
    double v = data[i].getDoubleValue();
    double v0, delta;
    for (++i; i < length - 1; i++)
    {
      if (timeAt(i) < timeAt(n - 1) + spacing)
      {
        v0 = v;
        v = valueAt(i);
        delta = Math.abs(v0-v);
        if (aTolerance > 0 && delta < aTolerance * 5) continue;
        if (pTolerance > 0 && delta < v*pTolerance * 5) continue;
      }
      data[n] = data[i];
      n++;
    }
    for (; i < length; i++)
    {
      data[n] = data[i];
      n++;
    }
    length = n;
    if (length < data.length)
      Arrays.fill(data, length, data.length - 1, nulldata);
    return;    
  } }
  private void append()
  {
    try
    {
      double v = getCurrentValue();
      double t = getCurrentTimestamp();
      switch (trendFormat)
      {
        case TFormat.CF_DBLDBL:
          if (length < data.length)
          { // there's room
            DBLDBL[] dp = new DBLDBL[] { new DBLDBL(v,t)};
            append(dp);
          }
          break;
        case TFormat.CF_DBLTIME:
          if (length < data.length)
          { // there's room
            DBLTIME[] dp = new DBLTIME[] { new DBLTIME(v,t)};
            append(dp);
          }
          break;
        case TFormat.CF_DBLDBLDBL:
          if (length < data.length)
          { // there's room
            int s = getCurrentSystemStamp();
            DBLDBLDBL[] dp = new DBLDBLDBL[] { new DBLDBLDBL(v,t,s)};
            append(dp);
          }
          break;
      }
    }
    catch (Exception x)
    {
      DbgLog.log("TrendData.append()", x.getMessage());
    }
    // check pruning conditions ...
    int pt = (int)(pruningThreshold * 100.0);
    if (length > (data.length * pt) / 100)
    {
      prune();
    }
  }
  @Override
  public void callback(TLink link)
  {
    long thisAccess = System.currentTimeMillis();
    if (thisAccess > lastAccess + ALLOWED_DEADTIME)
    {
      link.close();
      isActive = false;
    }
    else
    {
      append();
    }
  }
}
