package de.desy.tine.io;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import de.desy.tine.client.TLinkFactory;
import de.desy.tine.definitions.*;
import de.desy.tine.server.equipment.*;
import de.desy.tine.server.logger.*;

public class DcsEvnPkt
{
  public static final int MAX_DATAGRAM_SIZE = 65535;
  public static final short DCSEVN_PORT = 15555;
  int tag;    /**< magic number */
  int grp;    /**< source multicast IP address */
  int sec;    /**< acquisition time (sec) */
  int usec;   /**< acquisition time (usec) */
  int evnId;  /**< event */
  int blkId;  /**< server block ID */
  short pktNum; /**< packet number */
  short numPkt; /**< number of packets in server block */
  short datLen; /**< length of data in the packet */
  short sts = -1;    /**< status */
  int totLen; /**< total length of data block */
  byte[] bufIn = new byte[TTransport.UDP_BUFFER_SIZE];
  public DatagramPacket dpIn = new DatagramPacket(bufIn, bufIn.length);
  int port = DCSEVN_PORT;
  InetAddress mcgrp;
  private MulticastSocket sck = null;
  private Thread thrd = new DcsEvnThread();
  private boolean isActive = false;
  private long timestamp;
  private String ipSrc = null;
  public String getIpSender() { return ipSrc; }
  public InetAddress getMulticastGroup() { return mcgrp; }
  public DcsEvnPkt(String ipAddr)
  {
    ipSrc = ipAddr;
    start(ipAddr,MAX_DATAGRAM_SIZE);
  }
  void InterpretIncomingDcsEvn(InetAddress addr, byte[] data, int length) throws IOException
  { // be optimistic ....
    ByteArrayInputStream dpStrm = new ByteArrayInputStream(data);
    DataInputStream ds = new DataInputStream(dpStrm);
    tag = ds.readInt();
    grp = ds.readInt();
    sec = ds.readInt();
    usec = ds.readInt();
    evnId = ds.readInt();
    blkId = ds.readInt();
    pktNum = ds.readShort();
    numPkt = ds.readShort();
    datLen = ds.readShort();
    sts = ds.readShort();
    totLen = ds.readInt();
    timestamp = System.currentTimeMillis();
  }
  public void setActive(boolean value)
  {
    isActive = value;
  }
  public int getEventId() { return evnId; }
  public int getStatus() { return sts; }
  public long getTimestamp() { return timestamp; }
  private void callback()
  {
    if (!TLinkFactory.isRunningAsServer()) return;
    int cycleNumber = TEquipmentModuleFactory.getCycleNumber();
    if (cycleNumber == evnId) return;
    cycleNumber = evnId;
    TCycleTrigger[] trgs = TEquipmentModuleFactory.getCycleTriggerList(); 
    for (int i=0; i<trgs.length; i++) trgs[i].update(cycleNumber, sts);
    TEquipmentModuleFactory.setSystemStamp(cycleNumber);    
  }
  private class DcsEvnThread extends Thread
  {
    DcsEvnThread()
    {
      setName("doocs event number receiver");
      setDaemon(true);
    }
    public synchronized void run()
    {
      while (isActive)
      {
        try
        {
          dpIn.setLength(bufIn.length); // reset the receive buffer length
          sck.receive(dpIn);
          //scan thru the incoming datagram
          InterpretIncomingDcsEvn(dpIn.getAddress(), dpIn.getData(), dpIn.getLength());
          callback();
          if (TLinkFactory.debugLevel > 2) DbgLog.log("DcsEvnPktThread","doocs event number update: "+evnId);
        }
        catch (IOException e) 
        { // should never happen ...
          if (TLinkFactory.debugLevel > 1) DbgLog.log("DcsEvnPktThread", "IO exception: "+e.getMessage());
        }
        catch (Exception e)
        {
          e.printStackTrace();
          MsgLog.log("DcsEvnPktThread", "unhandled exception "+e.toString(),TErrorList.code_failure,e,0);          
        }
      }
      try
      {
        sck.leaveGroup(mcgrp);
        sck.close();
      } catch (Exception ignore) {}
    }
  }
  public void start(String addr,int sndBufferSize)
  {
    try
    {
      String ip = TTransport.getMCastGroup(addr,true);
      mcgrp = InetAddress.getByName(ip);
      sck = new MulticastSocket(port);
      sck.joinGroup(mcgrp); 
      sck.setSoTimeout(0);
      sck.setReceiveBufferSize(sndBufferSize);
      isActive = true;
      thrd.start();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      MsgLog.log("DcsEvnPkt.start", "error: "+e.getMessage(),TErrorList.udp_socket_error,e,0);
    }
  } 
}
