/*
 * Created on Feb 6, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.io;

import java.io.*;
import java.net.*;

import de.desy.tine.client.TLinkFactory;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.logger.DbgLog;
import de.desy.tine.server.logger.MsgLog;

public class TBucket
{
  public final static int ALLOWED_IDLE_TIME = 300000;
  Socket sck; // TCP connection socket
  public Socket getSocket() { return sck; }
  public void setSocket(Socket socket) { sck = socket; }
  int ptr; // bucket pointer
  public int getBucketPointer() { return ptr; }
  public void setBucketPointer(int pointer) { ptr = pointer; }
  private int siz; // bucket target size
  private int capacity = TPacket.MAX_DATAGRAM_SIZE;
  public int getBucketSize() { return siz; }
  public void setBucketSize(int bucketSize) { siz = bucketSize; }
  private byte[] buf = null; // bucket buffer
  public byte[] getBucketBuffer() { return buf; }
  int port;
  int localPort;
  public int getBucketPort() { return port; }
  public int getLocalPort() { return sck != null ? sck.getLocalPort() : 0; }
  public void setBucketPort(int portAddr) { port = portAddr; }
  InetAddress iaddr;
  public InetAddress getBucketEndpoint() { return iaddr; }
  public void setBucketEndpoint(InetAddress inetAddr) { iaddr = inetAddr; }
  public boolean active = false;
  public boolean isDeactivating = false;
  int number;
  private long lastActive = System.currentTimeMillis(); // creation time
  public long getTimeLastActive() { return lastActive; }
  public void touch() { lastActive = System.currentTimeMillis(); }
  private Thread bckThrd;
  OutputStream os;
  InputStream is;
  public Thread getBucketThread() { return bckThrd; }
  public void setBucketThread(Thread thread) { bckThrd = thread; } 
  public InputStream getInputStream() { return is; }
  public void setInputStream(InputStream input) { is = input; }
  public OutputStream getOutputStream() { return os; }
  public void setOutputStream(OutputStream output) { os = output; }
  public void reconnect()
  {
    if (sck == null) return;
    try
    {
      sck.close();
      os.close();
      is.close();
      initBucket(iaddr, port, bckThrd, capacity);
      //sck.connect(new InetSocketAddress(iaddr,port));;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      MsgLog.log("TBucket.reconnect",e.toString(),TErrorList.code_failure,e,0);      
    }
  }
  public void setCapacity(int newBufferSize)
  {
    byte[] b=null;
    try
    {
      b = new byte[newBufferSize];
    }
    catch (OutOfMemoryError e)
    {
      MsgLog.log("TBucket.setCapacity", "not enough memory to allocate "+newBufferSize+" bytes",TErrorList.out_of_local_memory,e,0);
      return;
    }
    buf = b;
    capacity = newBufferSize;
  }
  public int getCapacity() { return capacity; }
  protected void initBucket(Socket socket,Thread thread,int capacity)
  {
    sck = socket;
    try
    {
      int bufsiz = TLinkFactory.getSckRcvBufferSize();
      if (bufsiz > 0x1000) 
      {
        sck.setReceiveBufferSize(bufsiz);
        MsgLog.log("TBucket.initBucket","set server socket receive buffer size to "+bufsiz,0,null,1);
      }
      os = socket.getOutputStream();  
      is = socket.getInputStream(); 
      iaddr = socket.getInetAddress();
      port = socket.getPort();
      bckThrd = thread;
      this.capacity = capacity;
      buf = new byte[capacity];
    }
    catch (Exception e)
    {
      e.printStackTrace();
      MsgLog.log("TBucket.initBucket",e.toString(),TErrorList.code_failure,e,0);
    }    
  }
  public TBucket() 
  { 
  }
  protected int initBucket(InetAddress inetAddr,int portAddr,Thread thread,int capacity)
  {
    try
    {
      int bufsiz = TLinkFactory.getSckRcvBufferSize();
      sck = new Socket(inetAddr,portAddr);
      if (thread != null) thread.setName("tcp port " + sck.getLocalPort());
      if (bufsiz > 0x1000) 
      {
        sck.setReceiveBufferSize(bufsiz);
        MsgLog.log("TBucket.initBucket","set client socket receive buffer size to "+bufsiz,0,null,1);
      }
      bufsiz = TLinkFactory.getSckSndBufferSize();
      if (bufsiz > 0x1000) 
      {
        sck.setSendBufferSize(bufsiz);
        MsgLog.log("TBucket.initBucket","set client socket send buffer size to "+bufsiz,0,null,1);
      }      
      os = sck.getOutputStream();  
      is = sck.getInputStream(); 
      iaddr = inetAddr;
      port = sck.getPort();
      bckThrd = thread;
      this.capacity = capacity;
      buf = new byte[capacity];
    }
    catch (IOException e)
    { // host not there ?
      if (TLinkFactory.debugLevel > 1) DbgLog.log("TBucket.initBucket", "cannot open connected socket");
      return TErrorList.tcp_connect_error;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      MsgLog.log("TBucket.initBucket",e.toString(),TErrorList.code_failure,e,0);
      return TErrorList.tcp_connect_error;
    }
    return 0;
  }
  public void activate()
  {
    if (bckThrd != null) bckThrd.start(); 
  }
}
