/*
 * Created on Feb 23, 2005
 *
  */
package de.desy.tine.io;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import de.desy.tine.endianUtils.SwapUtil;

/**
 * \internal
 * 
 * A DataInputStream with implicit byte swapping for numeric
 * data types.
 * @author jwlg
 */
public class TDataInputStream extends FilterInputStream implements DataInput {
  
  DataInputStream stream;

  /**
   * @param in
   */
  public TDataInputStream(InputStream in) {
    super(in);
    stream = new DataInputStream(in);
  }

  /**
   * @param in
   * @return
   * @throws java.io.IOException
   */
  public static String readUTF(DataInput in) throws IOException {
    return DataInputStream.readUTF(in);
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterInputStream#available()
   */
  public int available() throws IOException {
    return stream.available();
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterInputStream#close()
   */
  public void close() throws IOException {
    stream.close();
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterInputStream#read()
   */
  public int read() throws IOException {
    return stream.read();
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterInputStream#read(byte[])
   */
  public int read(byte[] b) throws IOException {
    return stream.read(b);
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterInputStream#read(byte[], int, int)
   */
  public int read(byte[] b, int off, int len) throws IOException {
    return stream.read(b, off, len);
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public boolean readBoolean() throws IOException {
    return stream.readBoolean();
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public byte readByte() throws IOException {
    return stream.readByte();
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public char readChar() throws IOException {
    return (char) SwapUtil.swapShort((short)stream.readChar());
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public double readDouble() throws IOException {
    // avoid NaN ! 
    //return SwapUtil.swapDouble(stream.readDouble());
    return SwapUtil.swapDouble(stream.readLong());
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public float readFloat() throws IOException {
    // avoid NaN ! 
    //return SwapUtil.swapFloat(stream.readFloat());
    return SwapUtil.swapFloat(stream.readInt());
  }
  /**
   * @param b
   * @throws java.io.IOException
   */
  public void readFully(byte[] b) throws IOException {
    stream.readFully(b);
  }
  /**
   * @param b
   * @param off
   * @param len
   * @throws java.io.IOException
   */
  public void readFully(byte[] b, int off, int len) throws IOException {
    stream.readFully(b, off, len);
  }
  /**
   * @return
   * @throws java.io.IOException
   */
  public int readInt() throws IOException {
    return SwapUtil.swapInt(stream.readInt());
  }
  /**
   * @return
   * @throws java.io.IOException
   * @deprecated
   */
  public String readLine() throws IOException {
    return stream.readLine();
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public long readLong() throws IOException {
    return SwapUtil.swapLong(stream.readLong());
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public short readShort() throws IOException {
    return SwapUtil.swapShort(stream.readShort());
  }
  /**
   * @return
   * @throws java.io.IOException
   */
  public int readUnsignedByte() throws IOException {
    return stream.readUnsignedByte();
  }
  /**
   * @return
   * @throws java.io.IOException
   */
  public int readUnsignedShort() throws IOException {
    return SwapUtil.swapInt(stream.readUnsignedShort());
  }
  
  /**
   * @return
   * @throws java.io.IOException
   */
  public String readUTF() throws IOException {
    return stream.readUTF();
  }
  /* (non-Javadoc)
   * @see java.io.FilterInputStream#reset()
   */
  public void reset() throws IOException {
    stream.reset();
  }
  /* (non-Javadoc)
   * @see java.io.FilterInputStream#skip(long)
   */
  public long skip(long n) throws IOException {
    return stream.skip(n);
  }
  /**
   * @param n
   * @return
   * @throws java.io.IOException
   */
  public int skipBytes(int n) throws IOException {
    return stream.skipBytes(n);
  }
}
