/*
 * Created on Feb 23, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.desy.tine.io;

import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import de.desy.tine.endianUtils.SwapUtil;


/**
 * \internal
 * 
 * A data output stream which performs byte swapping.
 * TDataOutputStream behaves like java.io.DataOutputStream,
 * but internally swaps bytes of numeric data types 
 *
 * @author jwlg
 * @see DataOutput
 * @see DataOutputStream
 * @see FilterOutputStream
 * 
 * TODO A direct implementation without DataOutputStream would
 * improve the performance. 
 */
public class TDataOutputStream extends FilterOutputStream implements DataOutput {
  
  private DataOutputStream stream = null;
  
 /**
   * @param out
   */
  public TDataOutputStream(OutputStream out) {
    super(out);
    stream = new DataOutputStream(out);
  }

  
  /* (non-Javadoc)
   * @see java.io.FilterOutputStream#close()
   */
  public void close() throws IOException {
    stream.close();
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterOutputStream#flush()
   */
  public void flush() throws IOException {
    stream.flush();
  }
  
  /**
   * @return
   */
  public int size() {
    return stream.size();
  }
  
  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return stream.toString();
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterOutputStream#write(byte[])
   */
  public void write(byte[] b) throws IOException {
    stream.write(b);
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterOutputStream#write(byte[], int, int)
   */
  public void write(byte[] b, int off, int len) throws IOException {
    stream.write(b, off, len);
  }
  
  /* (non-Javadoc)
   * @see java.io.FilterOutputStream#write(int)
   */
  public void write(int b) throws IOException {
    stream.write(b);
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeBoolean(boolean v) throws IOException {
    stream.writeBoolean(v);
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeByte(int v) throws IOException {
    stream.writeByte(v);
  }
  
  /**
   * @param s
   * @throws java.io.IOException
   */
  public void writeBytes(String s) throws IOException {
    stream.writeBytes(s);
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeChar(int v) throws IOException {
    stream.writeChar(SwapUtil.swapShort((short)v));
  }
  
  /**
   * @param s
   * @throws java.io.IOException
   */
  public void writeChars(String s) throws IOException {
    stream.writeChars(s);
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeDouble(double v) throws IOException {
    // avoid NaN !
    //stream.writeDouble(SwapUtil.swapDouble(v));
    stream.writeLong(SwapUtil.swapLong(Double.doubleToLongBits(v)));
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeFloat(float v) throws IOException {
    // avoid NaN !
    //stream.writeFloat(SwapUtil.swapFloat(v));
    stream.writeInt(SwapUtil.swapInt(Float.floatToIntBits(v)));
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeInt(int v) throws IOException {
    stream.writeInt(SwapUtil.swapInt(v));
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeLong(long v) throws IOException {
    stream.writeLong(SwapUtil.swapLong(v));
  }
  
  /**
   * @param v
   * @throws java.io.IOException
   */
  public void writeShort(int v) throws IOException {
    stream.writeShort(SwapUtil.swapShort((short)v));
  }
  
  /**
   * @param str
   * @throws java.io.IOException
   * TODO Unclear if swapping is necessary here!
   */
  public void writeUTF(String str) throws IOException {
    stream.writeUTF(str);
  }
}
