package de.desy.tine.queryUtils;

import java.io.ByteArrayOutputStream;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import de.desy.tine.definitions.TStrings;
import de.desy.tine.server.devices.TDevice;
import de.desy.tine.server.equipment.TEquipmentModule;
import de.desy.tine.stringUtils.StringToBytes;
import de.desy.tine.structUtils.TStructBase;

public class DeviceInfo extends TStructBase
{
  private String context = null;
  private String server = null;
  private String name = null;
  private String desc = null;      /**< pointer to device description (64-char) */
  private String loc = null;       /**< pointer to device location (64-char) */
  private float zpos;              /**< device z position */
  private int mask;                /**< device mask (doocs sys_mask) & offline bit */
  private int region;              /**< specific device region setting */
  private int number;              /**< assigned device number (slot in device table) */

  public static final int sizeInBytes = 
      TStrings.CONTEXT_NAME_SIZE +
      TStrings.EXPORT_NAME_SIZE +
      TStrings.DEVICE_NAME_SIZE +
      TStrings.DEVICE_DESC_SIZE +
      TStrings.DEVICE_LOCATION_SIZE +
      4 * 4; 
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public String getName()
  {
    return name;
  }
  public void setName(String name)
  {
    this.name = name;
  }
  public String getContext()
  {
    return context;
  }
  public void setContext(String context)
  {
    this.context = context;
  }
  public String getDescription()
  {
    return desc;
  }
  public void setDescription(String desc)
  {
    this.desc = desc;
  }
  public String getLocation()
  {
    return loc;
  }
  public void setLocation(String loc)
  {
    this.loc = loc;
  }
  public float getZposition()
  {
    return zpos;
  }
  public void setZposition(float zpos)
  {
    this.zpos = zpos;
  }
  public int getMask()
  {
    return mask;
  }
  public void setMask(int mask)
  {
    this.mask = mask;
  }
  public int getRegion()
  {
    return region;
  }
  public void setRegion(int region)
  {
    this.region = region;
  }
  public int getNumber()
  {
    return number;
  }
  public void setNumber(int number)
  {
    this.number = number;
  }
  public String toString()
  {
    StringBuilder sb = new StringBuilder();
    sb.append("context: "+context + System.lineSeparator());
    sb.append("server: "+server + System.lineSeparator());
    sb.append("name: "+name + System.lineSeparator());
    sb.append("description: "+desc + System.lineSeparator());
    sb.append("location: "+loc + System.lineSeparator());
    sb.append("region: "+region + System.lineSeparator());
    sb.append("mask: "+mask + System.lineSeparator());
    sb.append("z-position: "+zpos + System.lineSeparator());
    sb.append("number: "+number + System.lineSeparator());
    return sb.toString();
  }
  public String getServer()
  {
    return server;
  }
  public void setServer(String server)
  {
    this.server = server;
  }
  public DeviceInfo()
  {
    context = new String(new StringBuffer(TStrings.CONTEXT_NAME_SIZE));
    server = new String(new StringBuffer(TStrings.EXPORT_NAME_SIZE));
    name = new String(new StringBuffer(TStrings.DEVICE_NAME_SIZE));
    desc = new String(new StringBuffer(TStrings.DEVICE_DESC_SIZE));
    loc = new String(new StringBuffer(TStrings.DEVICE_LOCATION_SIZE));
    region = 0;
    mask = 0;
    zpos = 0.0F;
    number = -1;
  } 
  public DeviceInfo(TDevice device)
  {
    if (device != null)
    {
      TEquipmentModule eqm = device.getEqm();
      if (eqm != null)
      {
        context = eqm.getContext();
        server = eqm.getExportName();
      }
      name = device.getName();
      desc = device.getDescription();
      loc = device.getLocation();
      region = device.getRegionCode();
      mask = device.getMask();
      zpos = device.getZposition();
      number = device.getNumber();
    }
  } 
  @Override
  public void writeData(DataOutput dout) throws IOException
  {
    dout.write(StringToBytes.stringToFixedBytes(context, TStrings.CONTEXT_NAME_SIZE));
    dout.write(StringToBytes.stringToFixedBytes(server, TStrings.EXPORT_NAME_SIZE));
    dout.write(StringToBytes.stringToFixedBytes(name, TStrings.DEVICE_NAME_SIZE));
    dout.write(StringToBytes.stringToFixedBytes(desc, TStrings.DEVICE_DESC_SIZE));
    dout.write(StringToBytes.stringToFixedBytes(loc, TStrings.DEVICE_LOCATION_SIZE));
    dout.writeFloat(zpos);
    dout.writeInt(mask);
    dout.writeInt(region);
    dout.writeInt(number);
  }
  @Override
  public void readData(DataInput din) throws IOException
  {
    context = StringToBytes.readFixedString(din, TStrings.CONTEXT_NAME_SIZE);
    server = StringToBytes.readFixedString(din, TStrings.EXPORT_NAME_SIZE);
    name = StringToBytes.readFixedString(din, TStrings.DEVICE_NAME_SIZE);
    desc = StringToBytes.readFixedString(din, TStrings.DEVICE_DESC_SIZE);
    loc = StringToBytes.readFixedString(din, TStrings.DEVICE_LOCATION_SIZE);
    zpos = din.readFloat();
    mask = din.readInt();
    region = din.readInt();
    number = din.readInt();
  }
}
