package de.desy.tine.server.alarms;

import de.desy.tine.client.*;
import de.desy.tine.dataUtils.*;
import de.desy.tine.structUtils.*;
import de.desy.tine.definitions.*;

public class TAlarmMessageStructure extends TTaggedStructure
{
  private static TLinkFactory gLinkFactory = null;
  private static TLinkFactory getLinkFactory()
  {
    if (gLinkFactory == null) gLinkFactory = TLinkFactory.getInstance();
    return gLinkFactory;
  }
  private char[] server = new char[32];
  private String serverString;
  private char[] device = new char[64];
  private String deviceString;
  private char[] alarmTag = new char[32];
  private String alarmTagString;
  private int[] alarmCode = new int[1];
  private int[] timestamp = new int[1];
  private int[] timestampUSec = new int[1];
  private int[] starttime = new int[1];
  private int[] starttimeUSec = new int[1];
  private int[] alarmMask = new int[1];
  private byte[] alarmData = new byte[64];
  private byte[] alarmDataFormat = new byte[1];
  private byte[] alarmDataArraySize = new byte[1];
  private TDataType tAlarmData;
  private byte[] severity = new byte[1];
  private byte[] descriptor = new byte[1];
  private short[] alarmSystem = new short[1];
  private byte[] reserved = new byte[2];
  public int getAlarmCode() { return alarmCode[0]; }
  public int getAlarmSeverity() { return severity[0]; }
  public int getAlarmDescriptor() { return descriptor[0]; }
  public String getAlarmDescriptorAsString() 
  {
    return TAlarmDescriptor.toLongString(descriptor[0]); 
  }
  public int getAlarmSystem() { return alarmSystem[0]; }
  public long getTimeStamp() 
  { 
    return (timestamp[0]*1000) + (timestampUSec[0]/1000); 
  }
  public int getStartTime()
  { 
    return (starttime[0]*1000) + (starttimeUSec[0]/1000); 
  }
  public int getAlarmMask() { return alarmMask[0]; }
  public boolean hasData() { return alarmDataArraySize[0] > 0 ? true : false; }
  public TDataType getAlarmData()
  {
    if (tAlarmData == null)
    {
      int dsiz = alarmDataArraySize[0];
      short dfmt = (short)(alarmDataFormat[0]);
      tAlarmData = new TDataType(dsiz,dfmt);
      if (dsiz > 0 && dfmt != TFormat.CF_NULL)
      { // the data are in TINE (i.e. little-endian) byte order!
        tAlarmData.pushBytes(alarmData);
        getLinkFactory().fillinIncomingData(tAlarmData);
      }
    }
    return tAlarmData;
  }
  public String getServer()
  {
    if (serverString == null) serverString = new String(server).trim();
    return serverString;
  }
  public String getDevice()
  {
    if (deviceString == null) deviceString = new String(device).trim();
    return deviceString;
  }
  public String getAlarmTag()
  {
    if (alarmTagString == null) alarmTagString = new String(alarmTag).trim();
    return alarmTagString;
  }
  public TAlarmMessageStructure()
  {
    super("AMSr4");
    addField(server,"server");
    addField(device,"device");
    addField(alarmTag,"alarmTag");
    addField(alarmCode,"alarmCode");
    addField(timestamp,"timestamp");
    addField(timestampUSec,"timestampUSec");
    addField(starttime,"starttime");
    addField(starttimeUSec,"starttimeUSec");
    addField(alarmMask,"alarmMask");
    addField(alarmData,"alarmData");
    addField(alarmDataFormat,"alarmDataFormat");
    addField(alarmDataArraySize,"alarmDataArraySize");
    addField(severity,"severity");
    addField(descriptor,"descriptor");
    addField(alarmSystem,"alarmSystem");
    addField(reserved,"reserved");
    initDone();  
  }
}
