/*
 * Created on Nov 9, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.connections;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Iterator;
import java.util.concurrent.ConcurrentLinkedQueue;

import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.headers.*;
import de.desy.tine.server.equipment.*;
import de.desy.tine.stringUtils.*;
import de.desy.tine.structUtils.*;

/**
 * \internal
 * 
 * TODO: separate the contract table management from the query logic
 * @author duval
 */
public class TContractTable extends TStructBase
{
  // below should be a linked list (ArrayList ?)
  public ConcurrentLinkedQueue<TClientEntry> clt = new ConcurrentLinkedQueue<TClientEntry>();
  public int nclients; /* number of clients for this contract */
  public int contractID; /* local contract ID */
  private TContract contract; /* see above */
  public int pollingRate; /* in mseconds, smallest number wins ... */
  public long lasttime; /* most recent time the data polled */
  public long lastmarked; /* last time the data were marked STALE */
  public long starttime; /* start time of the current call */
  public long dtimestamp; /* data timestamp */
  public int sysstamp; // system stamp (cycle number?)
  public int usrstamp; // user data stamp
  // is there a thread-safe problem with the data out buffer ?
  public TDataType dout; /* prepared output DTYPE */
  public TDataType drb; // readback output DTYPE, i.e. the result of the call
  public TDataType din; /* prepared input DTYPE */
  public int dataSizeOut; /* output data size < contract.dataSize */
  public int pending; /* pending flag for DeliverData() vs ProduceData() */
  public int delivered; // delivery flag for signals 
  public boolean expired; /* expired flag for contract removal */
  public boolean isRepeat = false; // signal whether the call is being repeated (from return = not_ready)
  public boolean isStale = false; // signal whether the contract should be marked stale upon entering doScheduler
  public boolean isProcessing = false;
  public int compStatus; /* completion status */
  public int returnCode; /* contract completion return code */
  public int delays; /* delayed scheduling due to pending */
  public int misses; /* missed scheduling (i.e. > 2x polling rate) */
  public String compString; /* completion string (redirection ?) */
  private String key;
  public String getKey() 
  { 
  	return key == null ? "(contract empty)" : key; 
  }
  public TEquipmentModule eqm; /* associated export element */
  public static final int nice_query_size = 50;
  public TEquipmentModuleFactory.ExecutionThread executeThread = null;
  public boolean runInSeparateThread = false;
  public TContractTable() {}
  public TContractTable(TContract con)
  {
    setContract(con);  	
  }
  public void setContract(TContract con)
  {
  	contract = con;
  	key = contract.eqmName+"/"+contract.eqmDeviceName+"["+contract.eqmProperty+"]";
  }
  public TContract getContract() { return contract; }
  public void runInSeparateThread()
  {
  	runInSeparateThread = true;
  	//executeThread = TEquipmentModuleFactory.getInstance().new ExecutionThread(this);
  }
  public boolean isRegisteredProperty = false;
//NEED the equals to work on the object reference !!!!
//  public boolean equals(TContractTable reference)
//  {
//    if (reference == null) return false;
//    return reference.contract.equals(contract);
//  }
  private static TStructDescription myStructDsc; // = new TStructDescription("CONQS");  //("XCTR");
  static
  {
    createQueryStruct();
  }
  public void setStale(byte staleFlag)
  {
    TClientEntry tce = null;
    Iterator<TClientEntry> li = clt.iterator();
    while (li.hasNext())
    {
      tce = (TClientEntry)li.next();
      tce.sts.Stale = staleFlag;
    }
  }
  public int renegotiateContract(short statusCode)
  {
    TClientEntry tce = null;
    int datasize = 0, ds;
    boolean addsts = false;
    int sts = TErrorList.getErrorCode(statusCode);
    if (sts == TErrorList.not_signalled || sts == TErrorList.not_ready) sts = 0;
    // get datasize from drb (read-back data set)
    datasize = TFormat.isAdjustableLength(drb.dFormat) ? drb.getSizeInBytes() : drb.getDataSize();
    
    if (sts != 0) addsts = true; //datasize += TStrings.STATUS_SIZE; //TErrorList.ERROR_SIZE;    
    Iterator<TClientEntry> li = clt.iterator();
    while (li.hasNext())
    {
      ds = datasize;
      tce = (TClientEntry) li.next();
      if (addsts) ds += tce.cln.tineProtocol < 6 ? TStrings.STATUS_SHORTSIZE : TStrings.STATUS_SIZE; 
      tce.sts.BytesRemaining = ds;
      tce.sts.BytesTotal = ds;
      tce.sts.nextDataSize = tce.sts.BytesRemaining;
      if (tce.sts.BytesRemaining > tce.sts.mtu) tce.sts.nextDataSize = tce.sts.mtu;
      tce.sts.numblks = (short) (ds / tce.sts.mtu + (ds % tce.sts.mtu > 0 ? 1 : 0));
      if (datasize == 0) tce.sts.numblks = 1;
    }
    return 0;
  }
  /**
   * Creates a tagged structure description for TContractTable;
   * TODO: move this off to the tagged struct
   */
  private static void createQueryStruct()
  {
    if (!TStructRegistry.contains("CONQS"))
    {
      new TContractStruct();
    }
    if (!TStructRegistry.contains("CTQSr4"))
    {
      new TContractEntry();
    }
  }
  /**
   * Returns tagged structure description.
   * 
   * @return
   */
  public static TStructDescription getStructDescription()
  {
    return myStructDsc;
  }
  /*
   * (non-Javadoc)
   * 
   * @see de.desy.tine.client.TStructIfc#readData(java.io.DataInput)
   */
  public void readData(DataInput din) throws IOException
  {
    // TODO Not implemented yet.
  }
  /*
   * (non-Javadoc)
   * 
   * @see de.desy.tine.client.TStructIfc#writeData(java.io.DataOutput)
   */
  //TODO: move this off to the tagged struct !
  //TODO: trap legacy vs modern !
  public void writeData(DataOutput dout) throws IOException
  {
    dout.write(StringToBytes.stringToFixedBytes(contract.eqmProperty, TStrings.PROPERTY_NAME_SHORTSIZE));
    dout.write(StringToBytes.stringToFixedBytes(contract.eqmDeviceName, TStrings.DEVICE_NAME_SHORTSIZE));
    dout.write(StringToBytes.stringToFixedBytes(contract.eqmName,TStrings.EQM_NAME_SIZE));
    dout.writeInt(contract.dataSizeIn);
    dout.writeInt(contract.dataSizeOut);
    dout.writeByte(contract.hEqmName);
    dout.writeByte(contract.dataAccess);
    dout.writeByte(contract.dataFormatIn);
    dout.writeByte(contract.dataFormatOut);
    dout.write(StringToBytes.stringToFixedBytes(contract.dataTagIn, TStrings.TAG_NAME_SHORTSIZE));
    dout.write(StringToBytes.stringToFixedBytes(contract.dataTagOut, TStrings.TAG_NAME_SHORTSIZE));
    dout.writeInt(this.pollingRate);
    dout.writeInt(this.clt.size());
  }
  public void writeData(String tag,DataOutput dout) throws IOException
  {
    int plen = TStrings.PROPERTY_NAME_SIZE;
    int dlen = TStrings.DEVICE_NAME_SIZE;
    int tlen = TStrings.TAG_NAME_SIZE;
    if (tag == null) tag = "CTQSr4";
    if (tag.compareTo("CONQS") == 0)
    { // legacy
      if (!TStructRegistry.contains("CONQS")) new TContractStruct();
      plen = TStrings.PROPERTY_NAME_SHORTSIZE;
      dlen = TStrings.DEVICE_NAME_SHORTSIZE;
      tlen = TStrings.TAG_NAME_SHORTSIZE;
    }
    else if (tag.compareTo("CTQSr4") == 0)
    { // modern
      if (!TStructRegistry.contains("CTQSr4")) new TContractEntry();
    }
    else 
    { // TODO: ? (throw exception ?)
      return;
    }

    dout.write(StringToBytes.stringToFixedBytes(contract.eqmProperty, plen));
    dout.write(StringToBytes.stringToFixedBytes(contract.eqmDeviceName, dlen));
    dout.write(StringToBytes.stringToFixedBytes(contract.eqmName,TStrings.EQM_NAME_SIZE));
    dout.writeInt(contract.dataSizeIn);
    dout.writeInt(contract.dataSizeOut);
    dout.writeByte(contract.hEqmName);
    dout.writeByte(contract.dataAccess);
    dout.writeByte(contract.dataFormatIn);
    dout.writeByte(contract.dataFormatOut);
    dout.write(StringToBytes.stringToFixedBytes(contract.dataTagIn, tlen));
    dout.write(StringToBytes.stringToFixedBytes(contract.dataTagOut, tlen));
    dout.writeInt(this.pollingRate);
    dout.writeInt(this.clt.size());
  }
  public String toString()
  {
    String s = contract.toString();
    s = s.concat("\ncontract id : "+contractID);
    s = s.concat("\nnumber of clients : "+nclients+" (list size "+clt.size()+")");
    s = s.concat("\nlast completion code : "+returnCode+" ("+TErrorList.getErrorString(returnCode)+")");
    s = s.concat("\nlast link status : "+compStatus+" ("+TErrorList.getErrorString(compStatus)+")");
    s = s.concat("\nnumber clients pending delivery : "+pending);
    s = s.concat("\ncontract expired : "+expired);
    return s;
  }
}
