package de.desy.tine.server.equipment;


/**
 * Server side cycle trigger methods
 * A cycle trigger interface.
 * 
 * The update() method will be called upon incoming 
 * cycle triggers (or system error).  The current cycle number will be passed 
 * along with the status (usually 0). 
 * 
 * @author duval
 *
 */
public interface TCycleTrigger
{
  /**
   * Is processed following an incoming cycle number event.
   * 
   * If a CYCLER exists and a TCycleTrigger is in use, then
   * its update routine will be called following the trigger
   * event associated with the incoming cycle number or a link
   * timeout notification.
   * 
   * By default a CYCLER is a server called "CYCLER" in the same context as the
   * initializing device server and produces a keyword (property) called
   * "CycleNumber".  This gives rise to the Cycle Key: 
   * "/<context>/CYCLER/CycleNumber". If it is known a priori, that the 
   * CYCLER exists in another context or uses another server name or property
   * name, the cycle key can be set via API (setCyclerNumberKey() in
   * TEquipmentModule) or via environment (set TINE_CYCLE_KEY=\<key\>).
   * 
   * @param cycleNumber is the last arrived cycle number
   * @param status is the cycle link status
   */
  void update(int cycleNumber,int status);
}
