package de.desy.tine.server.equipment;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.LinkedList;
import de.desy.tine.csvUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.server.histories.*;
import de.desy.tine.server.logger.*;
import de.desy.tine.startup.TInitializerFactory;

public class THistoryHomeManifest implements RowHandler
{
  private static String crlf = "\n";
  private static TEquipmentModuleFactory tf = TEquipmentModuleFactory.getInstance();
  private static final String mfHdr = "FecName, LocalName, Port, Index, StdFiles, Property, Device, DataLength, Format, LongDepth, BigEndian";
  private static LinkedList<THistoryHomeManifest> mfLst = new LinkedList<THistoryHomeManifest>();
  public static boolean hasUpdated = false;
  public String fec;
  public String eqm;
  public String prp;
  public String dev;
  public String fmtStr;
  public int port;
  public int idx;
  public int stdFiles;
  public int siz;
  public int depth;
  public String end; // endianness (1 = big, 0 = little)
  public String getEntryAsString()
  {
    String rec = fec+", "+eqm+", "+port+", "+idx+", "+
                 stdFiles+", "+prp+", "+dev+", "+siz+", "+fmtStr+", "+depth+", "+end;
    
    return rec;
  }
  THistoryHomeManifest()
  {
    fec = ""; eqm = ""; prp = ""; dev = ""; fmtStr = "";
  }
  THistoryHomeManifest(THistoryHomeManifest reference)
  {
    fec = new String(reference.fec).trim();
    eqm = new String(reference.eqm).trim();
    prp = new String(reference.prp).trim();
    dev = new String(reference.dev).trim();
    fmtStr = new String(reference.fmtStr).trim();
    port = reference.port;
    idx = reference.idx;
    stdFiles = reference.stdFiles;
    siz = reference.siz;
    depth = reference.depth;
    end = reference.end;
  }

  private static String mfFilePath = null;
  public static String getFilePath() 
  { 
    if (mfFilePath == null)
    {
      mfFilePath = TInitializerFactory.getInstance().getInitializer().getHistoryHome();
      if (File.separatorChar == '\\') crlf = "\r\n";
      File csvw = new File(mfFilePath);
      if (!csvw.exists()) csvw.mkdirs();
    }
    return new String(mfFilePath); 
  }
  private static void hstTableToManifest(BufferedWriter mfout,TEquipmentModule em)
  {
    String recStr;
    THistoryRecord[] hsts = (THistoryRecord[])em.gLclHstList.toArray(new THistoryRecord[0]);
    if (hsts == null) return;
    try
    {
      int dep;
      for (int r=0; r<hsts.length; r++)
      {
        if ((dep=hsts[r].getHistorySpecification().getDepthLong()) < 1) continue;
        recStr = tf.getFecName()+", "+em.getLocalName()+", "+tf.getPort()+", "+hsts[r].getRcdIdx()+", "+
                 (hsts[r].getUseStandardFileSet() ? "1" :"0")+", "+hsts[r].getPrp()+", "+hsts[r].getDev()+", "+
                 hsts[r].getArraySize()+", "+TFormat.toString(hsts[r].getFmt())+", "+dep+", TRUE";
        mfout.write(recStr + crlf);
      }
    }
    catch (Exception e)
    {
      MsgLog.log("THistoryHomeManifest.hstTableToManifest", "update manifest: "+e.getMessage(),TErrorList.code_failure,e,0);
    }
  }
  public static synchronized int update()
  { // update the manifest for this equipment module
    String mffp = getFilePath();
    String mfFileName = mffp + File.separator + "historymf.csv";
    String mfTmpName = mffp + File.separator + "historymf.csv.tmp";
    File mft = new File(mfTmpName);
    if (mft.exists() && mft.lastModified() > System.currentTimeMillis() - 300000)
    { // file more recent than the last 5 minutes -> some other client app is updating ?
      return TErrorList.operation_busy;
    }
    if (mfLst != null) mfLst.clear();    
    csvColumn[] mfCols = new csvColumn[11];
    THistoryHomeManifest mfRows = new THistoryHomeManifest();
    mfCols[0] = new csvColumn("LOCAL_NAME","",new StringFieldHandler(mfRows,"eqm"));
    mfCols[1] = new csvColumn("FEC_NAME","",new StringFieldHandler(mfRows,"fec"));
    mfCols[2] = new csvColumn("PORT","",new IntFieldHandler(mfRows,"port"));    
    mfCols[3] = new csvColumn("INDEX","",new IntFieldHandler(mfRows,"idx"));    
    mfCols[4] = new csvColumn("STD_FILES","",new IntFieldHandler(mfRows,"stdFiles"));    
    mfCols[5] = new csvColumn("PROPERTY","",new StringFieldHandler(mfRows,"prp"));
    mfCols[6] = new csvColumn("DEVICE","",new StringFieldHandler(mfRows,"dev"));
    mfCols[7] = new csvColumn("DATA_LENGTH","",new IntFieldHandler(mfRows,"siz"));    
    mfCols[8] = new csvColumn("FORMAT","",new StringFieldHandler(mfRows,"fmtStr"));
    mfCols[9] = new csvColumn("LONG_DEPTH","",new IntFieldHandler(mfRows,"depth"));    
    mfCols[10] = new csvColumn("BIG_ENDIAN","FALSE",new StringFieldHandler(mfRows,"end"));    
    // open it
    csv mfFile = new csv(mfFileName);
    // read it
    int rc = mfFile.readFile(mfCols,mfRows);
    // close it
    mfFile.close();
    
    TEquipmentModule[] em = tf.getEquipmentModuleTable();
    if (em == null) return TErrorList.code_failure;
    BufferedWriter mfout;
    if (rc != 0)
    { // create a new manifest
      mft.delete();
      File mf = new File(mffp);
      mf.mkdirs();
      try
      {
        mfout = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(mfFileName)));
        mfout.write(mfHdr.trim() + crlf);
        // loop over the eqms
        for (int i=0; i<em.length; i++)
        { 
          hstTableToManifest(mfout,em[i]);
        }        
        mfout.close();
        return 0; // we're finished
      }
      catch (Exception e)
      {
        MsgLog.log("THistoryHomeManifest","cannot update FEC manifest !",TErrorList.file_error,e,0);
        return TErrorList.file_error;
      }
    }
    try
    {
      mfout = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(mfTmpName)));
      int n = mfLst.size();
      int m;
      THistoryRecord hr;
      THistoryHomeManifest thm;
      mfout.write(mfHdr.trim() + crlf);
      for (int i=0; i<n; i++)    
      {
        thm = mfLst.get(i);
        if (thm.fec.length() == 0) continue;
        if (thm.eqm.length() == 0) continue;
        for (m=0; m<em.length; m++)
        {
          if (thm.eqm.compareToIgnoreCase(em[m].getLocalName()) == 0 && thm.port == tf.getPort()) break;
        }
        if (m < em.length) continue; // found myself in the manifest 
        mfout.write(thm.getEntryAsString() + crlf);
        if ((hr=tf.getHistoryFromRecordIndex(thm.idx)) != null)
        { // looks like a collision !
          TFecLog.log(TineLogLevel.WARN,"HIST: WARNING -> record "+thm.idx+" ("+hr.getEqm().getLocalName()+"/"+hr.getDev()+"["+hr.getPrp()+"]"+
              "appears to conflict with local history of "+thm.eqm+"/"+thm.dev+"["+thm.prp+"]"+
              "on FEC "+thm.fec);          
        }
      }
      // now add my entries at the end ....
      for (int i=0; i<em.length; i++)
      { 
        hstTableToManifest(mfout,em[i]);
      }        
      // ....
      mfout.close();
      File mf = new File(mfFileName);
      mf.delete();
      mf = mft;      
      mf.renameTo(new File(mfFileName));
    }
    catch (Exception e)
    {
      MsgLog.log("THistoryHomeManifest", e.toString(),TErrorList.code_failure,e,1);      
    }
    return 0;
  }
  
  @Override
  public int process(int index)
  {
    if (mfLst == null) mfLst = new LinkedList<THistoryHomeManifest>();
    mfLst.add(new THistoryHomeManifest(this));
    return 0;
  }

}
