package de.desy.tine.server.equipment;

import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TMode;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.server.logger.TFecLog;
import de.desy.tine.types.NAME64;

public class TStateChange implements TLinkCallback
{
  private int last_cc = TErrorList.not_initialized;
  private int to_counter = 0;
  private String prevState = "";
  private String thisState = "";
  private NAME64[] dStateChange = new NAME64[1]; 
  private TDataType dtStateChange = new TDataType(dStateChange);
  private TLink lnk;
  public int getTriggerStatus() { return last_cc; }
  public String getCurrentState() { return thisState; }
  public String getPreviousState() { return prevState; }
  public TStateChange(String context,String key)
  {
    if (context == null) 
    {
      MsgLog.log("TStateChange","State information unavailable for null context ",TErrorList.argument_list_error,null,0);
      return;
    }
    String dev = "/"+context+"/GLOBALS/keyword";
    try
    {
      last_cc = TErrorList.address_unknown;
      lnk = new TLink(dev,key,dtStateChange,null,TAccess.CA_READ);
      lnk.attach(TMode.CM_RECEIVE, this, 1000);
      last_cc = lnk.linkStatus;
    }
    catch (Exception e)
    {
      MsgLog.log("TStateChange","State information unavailable for context "+context,last_cc,e,0);
    }
  }
  public void callback(TLink link)
  {
    int cc = link.getLinkStatus();
    if (cc != last_cc)
    {
      last_cc = cc;
      MsgLog.log("TStateTriggerCallback","state trigger : " + link.getLastError(),cc,null,1);
    }
    if (cc != 0) 
    {
      if (to_counter++ < 5) return;
      thisState = "unavailable";
    }
    else
    {
      to_counter = 0;
    }
    thisState = dStateChange[0].getName();
    if (thisState.compareToIgnoreCase(prevState) == 0) return;
    TFecLog.log("detected state change from "+prevState+" to "+thisState);
    String ctx = link.getContext();
    String ectx = null;
    TStateChangeTrigger sct = null;
    TEquipmentModule[] eTbl = TEquipmentModuleFactory.getInstance().getEquipmentModuleTable();
    for (int i=0; i<eTbl.length; i++)
    {
      if ((ectx=eTbl[i].getContext()) == null) continue;
      if (ctx.compareToIgnoreCase(ectx) == 0)
      {
        if ((sct=eTbl[i].getStateChangeTrigger()) != null)
          sct.update(prevState,thisState);
      }
    }
    prevState = thisState;
  }    
}
