package de.desy.tine.server.equipment;


/**
 * Server side state-change trigger methods
 * A state-change trigger interface.
 * 
 * The update() method will be called upon a detected 
 * state-change.  The current and previous states will be passed. 
 * Upon any I/O errors the current state will be marked as "unavailable".
 * 
 * @author duval
 *
 */
public interface TStateChangeTrigger
{
  /**
   * Is processed following a detected state change.
   * 
   * If a GLOBALS server exists and delivers the system state variable, then
   * the state information is available and can be acquired and examined.
   * 
   * 
   * @param previousState is the prior system state
   * @param thisState is the current system state
   */
  void update(String previousState,String thisState);
}
