package de.desy.tine.server.histories;

import de.desy.tine.structUtils.TTaggedStructure;

public class DSUMMARY extends TTaggedStructure
{
  public static final int sizeInBytes = 48; // 4 doubles, 4 integers
  double[] ave = new double[1]; /**< data average over range */
  double[] dev = new double[1]; /**< standard deviation */
  double[] max = new double[1]; /**< data maximum */
  double[] min = new double[1]; /**< data minimum */
  int[] ts = new int[1];  /**< centered timestamp */
  int[] tMax = new int[1];/**< timestamp maximum */
  int[] tMin = new int[1];/**< timestamp minimum */
  int[] cnt = new int[1]; /**< number of data points in interval */
  
  private void initStructure()
  {
    addField(ave,"average");
    addField(dev,"deviation");
    addField(max,"maximum");
    addField(min,"minimum");
    addField(ts,"timeAve");
    addField(tMax,"timeMax");
    addField(tMin,"timeMin");
    addField(cnt,"counts");
    initDone();      
  }
  public DSUMMARY()
  {
    super("DSUMMARY");
    initStructure();
  }
  public DSUMMARY(double ave,double dev,double max,double min,
      long ts, long tsMax, long tsMin, int cnt)
  {
    super("DSUMMARY");
    initStructure();
    this.ave[0] = ave;
    this.dev[0] = dev;
    this.max[0] = max;
    this.min[0] = min;
    this.ts[0] = (int)(ts/1000);
    this.tMax[0] = (int)(tsMax/1000);
    this.tMin[0] = (int)(tsMin/1000);
    this.cnt[0] = cnt;
  }
  public double getAverage() { return ave[0]; }
  public double getDeviation() { return dev[0]; }
  public double getMax() { return max[0]; }
  public double getMin() { return min[0]; }
  public double[] getSummary()
  {
    double[] ds = new double[4];
    ds[0] = ave[0];
    ds[1] = dev[0];
    ds[2] = max[0];
    ds[3] = min[0];
    return ds;
  }
}
