package de.desy.tine.server.logger;

import de.desy.tine.client.TLinkFactory;
import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.definitions.TStrings;
import de.desy.tine.server.equipment.TEquipmentModuleFactory;
import de.desy.tine.structUtils.TTaggedStructure;

public class Clog extends TTaggedStructure
{
  private static TLinkFactory tlf = null;
  private static TLinkFactory getLinkFactory()
  {
    if (tlf == null) tlf = TLinkFactory.getInstance();
    return tlf;
  }
  private static TEquipmentModuleFactory temf = null;
  private static TEquipmentModuleFactory getModuleFactory()
  {
    if (temf == null) temf = TEquipmentModuleFactory.getInstance();
    return temf;
  } 
  private char[] ctx = new char[TStrings.CONTEXT_NAME_SIZE];
  private String ctxString;
  private char[] tag = new char[32];
  private String tagString;
  private char[] logger = new char[32];
  private String loggerString;
  private char[] txt = new char[256];
  private String txtString;
  private int[] pri = new int[1];
  private int[] sts = new int[1];
  private double[] timeSent = new double[1];
  private double[] timeLogged = new double[1];
  public void reset()
  {
    ctxString = null;
    tagString = null;
    loggerString = null;
    txtString = null;
  }
  public String getContext()
  {
    if (ctxString == null) ctxString = new String(ctx).trim();
    return ctxString;
  }
  public String getTag()
  {
    if (tagString == null) tagString = new String(tag).trim();
    return tagString;
  }
  public String getLogger()
  {
    if (loggerString == null) loggerString = new String(logger).trim();
    return loggerString;
  }
  public String getText()
  {
    if (txtString == null) txtString = new String(txt).trim();
    return txtString;
  }
  public int getPriority() { return pri[0]; }
  public int getStatus() { return sts[0]; }
  public long getTimeSent() 
  { 
    return TDataTime.getLongDataTimeStamp(timeSent[0]); 
  }
  public long getTimeLogged() 
  { 
    return TDataTime.getLongDataTimeStamp(timeLogged[0]); 
  }
  
  private void initStructure()
  {
    addField(ctx,"context");
    addField(tag,"tag");
    addField(logger,"logger");
    addField(txt,"text");
    addField(pri,"priority");
    addField(sts,"status");
    addField(timeSent,"timeSent");
    addField(timeLogged,"timeLogged");
    initDone();    
  }
  Clog()
  {
    super("CLOG");
    initStructure();
  }
  Clog(String text,String context,String tag,String caller,int priority,int status)
  {
    super("CLOG");
    pushChars(context,ctx);
    pushChars(tag,this.tag);
    pushChars(caller,logger);
    pushChars(text,txt);
    pri[0] = priority;
    sts[0] = status;
    timeSent[0] = TDataTime.getDataTimeStamp();
    initStructure();
  }
  Clog(String text)
  {
    super("CLOG");
    if (TLinkFactory.isRunningAsServer())
    {
      pushChars(getModuleFactory().getFecContext(),ctx);
      pushChars(getModuleFactory().getFecSubsystem(),this.tag);
    }
    else
    {
      pushChars("DEFAULT",ctx);
      pushChars("",this.tag);      
    }
    pushChars(getLinkFactory().getUserName(),logger);
    pushChars(text,txt);
    pri[0] = ClsLog.ClogPriority.CLOG_PRIORITY_USEFUL.ordinal();
    sts[0] = ClsLog.ClogStatus.CLOG_STATUS_INFO.ordinal();
    timeSent[0] = TDataTime.getDataTimeStamp();
    initStructure();
  }
  Clog(Clog reference)
  {
    this(reference.getText(),reference.getContext(),reference.getTag(),
        reference.getLogger(),reference.getPriority(),reference.getStatus());
    this.timeLogged[0] = reference.timeLogged[0];
    this.timeSent[0] = reference.timeSent[0];
  }
}
