
package de.desy.tine.server.logger;

import de.desy.tine.client.TLinkFactory;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.equipment.TEquipmentModuleFactory;
import de.desy.tine.startup.TInitializer;
import de.desy.tine.startup.TInitializerFactory;
import java.io.File;
import java.io.RandomAccessFile;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

/**
 *
 * @author szczesny
 */
public class TCommandLog {
    
  private static TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
  private static FileHandler cmdlogHandler = null;
  private static String tag = null;
  private static final String cmdLogFileName = "commands.log.0";
  private static int debugLevel = 0;
  
  private static boolean hasLoggingError = false;
  public static boolean hasLoggingError() { return hasLoggingError; }
  
  public static void close()
  {
  	if (cmdlogHandler != null) 
  	{
  		cmdlogHandler.close();
  		cmdlogHandler =  null;
  	}
  }
  
  public static int log(String text)
  {
    if (!TLinkFactory.isRunningAsServer()) return TErrorList.not_running;
    if (tag == null) tag = TEquipmentModuleFactory.getInstance().getFecName();
    return log(tag,text);
  }
  
  public static int log(String tag, String text)
  {
    if (cmdlogHandler == null) 
    {
      cmdlogHandler = initializer.getCmdLogResource();
      if (cmdlogHandler == null)
      { // still null !!!
        DbgLog.log("TCommandLog","cannot write to log file : no resource !");
        TEquipmentModuleFactory.getInstance().setFileErrorAlarm(cmdLogFileName);
        hasLoggingError = true;
        return -1;
      }
      cmdlogHandler.setFormatter(new TFecLogFormatter());
    }
    String txt = "[" + tag + "] : " + text;
    try
    {
      
      LogRecord lr = new LogRecord(Level.INFO,txt);
      cmdlogHandler.publish(lr);
      hasLoggingError = false;
    }
    catch (OutOfMemoryError e)
    {
      e.printStackTrace();      
    }
    catch (Exception e)
    {
      DbgLog.log("TCommandLog","cannot write to log file : " + e.toString());
      TEquipmentModuleFactory.getInstance().setFileErrorAlarm(cmdLogFileName);
      hasLoggingError = true;
    }
    // if (TLinkFactory.debugLevel > 0) DbgLog.log("TCommandLog",txt); << approx. same info also goes to the console from TFecLog. 
    return 0;
  }
  public static String[] getFiles()
  {
    return getFiles(initializer.getLogHome());
  }
  public static String[] getFiles(String filePath)
  {
    File path = new File(filePath);
    File[] rawList = path.listFiles();
    String[] files = new String[rawList.length];
    for (int i=0; i<rawList.length; i++)
    {
      files[i] = rawList[i].getName();      
    }
    return files;
  }
  public static String getLines(int maxNumberOfLines)
  {
    return getLines(cmdLogFileName,maxNumberOfLines);
  }
  public static boolean fileExists(String fileName)
  {
    String fn = null;
    if (fileName == null || fileName.length() == 0) return false;
    if (fileName.indexOf('/') == -1 && fileName.indexOf('\\') == -1 )
    { // just the file name given => look in the default log directory
      fn = initializer.getLogHome() + "/" + fileName;
    }
    else
    { // full path given
      fn = fileName;
    }
    return new File(fn).exists();
  }
  public static int writeTextFile(String fileName,char[] fileText)
  {
    String fn = null;
    try
    {
      if (fileName.indexOf('/') == -1 && fileName.indexOf('\\') == -1 )
      { // just the file name given => look in the default log directory
        fn = initializer.getLogHome() + "/" + fileName;
      }
      else
      { // full path given
        fn = fileName;
      }
      RandomAccessFile raf = new RandomAccessFile(fn,"rw");
      raf.write(new String(fileText).getBytes());
      raf.close();
    }
    catch (Exception e)
    {
      DbgLog.log("TCommandLog",e.toString());
      return TErrorList.file_error;
    }
    return 0;
  }
  public static boolean isAbsolutePath(String filename)
  {
    if (filename == null || filename.length() == 0) return false;
    if (filename.startsWith("/")) return true;
    if (filename.startsWith("\\\\")) return true;
    if (filename.startsWith(":\\", 1)) return true;
    if (filename.startsWith(":/",1)) return true; 
    return false;
  }
  public static String getLines(String fileName,int maxNumberOfLines)
  {
    String fn = null;
    StringBuffer sb = null;
    String text=null;
    try
    {
      if (fileName.indexOf('/') == -1 && fileName.indexOf('\\') == -1 )
      { // just the file name given => look in the default log directory
        fn = initializer.getLogHome() + "/" + fileName;
      }
      else
      { // full path given
        fn = fileName;
      }
      RandomAccessFile raf = new RandomAccessFile(fn,"r");
      int flen = (int)raf.length();
      int fpos = flen - maxNumberOfLines*80;
      if (fpos < 0) fpos = 0;
      raf.seek(fpos);
      sb = new StringBuffer(flen-fpos);
      while ((text = raf.readLine()) != null)
      {
        sb.append(text);
        sb.append("\n");
      }
      raf.close();
    }
    catch (Exception e)
    {
      DbgLog.log("TCommandLog",e.toString());
    }
    if (sb == null) sb = new StringBuffer("file unavailable");
    text = sb.toString();
    if (debugLevel > 0) DbgLog.log("TCommandLog",text);    
    return text;
  }
}
