/*
 * Created on Dec 15, 2004
 * CHANGES 
 * 2005-02-15 JW Made non-static
 */
package de.desy.tine.server.properties;
import java.util.*;

import de.desy.tine.definitions.TFormat;

// TODO Is ID needed at all?
// TODO If not, can this class be replaced by a plain HashSet?

/**
 * \internal
 * 
 * @author duval
 *
 */
public final class TMetaProperties
{
  public final class TMetaProperty
  {
    private String tag;
    private int id;
    private boolean isEmbedded;
    public String getTag() { return tag; }
    public int getId() { return id; }
    public TMetaProperty(String extension,int index)
    {
      tag = new String(extension);
      id = index;
      isEmbedded = false;
    }
    public TMetaProperty(String extension,int index,boolean embedded)
    {
      tag = new String(extension);
      id = index;
      isEmbedded = embedded;
    }
  }
  private  static LinkedList<TMetaProperty> mp = new LinkedList<TMetaProperty>();
  
  public static final int EGU_ID = 1;
  public static final int HST_ID = 2;
  public static final int NAM_ID = 3;
  public static final int DSC_ID = 4;
  public static final int BIT_ID = 5;
  public static final int MSK_ID = 6;
  public static final int GAT_ID = 7;
  public static final int XEGU_ID = 8;
  public static final int URL_ID = 9;
  public static final int MAX_ID = 10;
  public static final int MIN_ID = 11;
  public static final int XMAX_ID = 12;
  public static final int XMIN_ID = 13;
  public static final int DMASK_ID = 14;
  public static final int ONLINE_ID = 15;
  public static final int RBMAX_ID = 16;
  public static final int RBMIN_ID = 17;
  public static final int RBAVE_ID = 18;
  public static final int RBSUMMARY_ID = 19;
  
  public static final int GATE_NONE = 0;
  public static final int GATE_BOOLEAN = 1;
  public static final int GATE_FILTER = 2;
  public static final int GATE_ARRAY = 4;
  
  public static final int EGU_FLTR = 0x01;
  public static final int HST_FLTR = 0x02;
  public static final int DSC_FLTR = 0x04;
  public static final int MAX_FLTR = 0x08;
  public static final int XEGU_FLTR = 0x10;
  public static final int XMAX_FLTR = 0x20;
  public static final int ONL_FLTR = 0x40;
 
  public static TMetaProperties getInstance()
  {
    if (instance == null) instance = new TMetaProperties();
    return instance;
  }
  private static TMetaProperties instance = new TMetaProperties();

  private TMetaProperties()
  {
    mp.add(new TMetaProperty(".AR",HST_ID));
    mp.add(new TMetaProperty(".ARC",HST_ID));
    mp.add(new TMetaProperty(".ARCH",HST_ID));
    mp.add(new TMetaProperty(".EGU",EGU_ID));
    mp.add(new TMetaProperty(".HST",HST_ID));
    mp.add(new TMetaProperty(".HIST",HST_ID));
    mp.add(new TMetaProperty(".HST@",HST_ID));
    mp.add(new TMetaProperty(".HIST@",HST_ID));
    mp.add(new TMetaProperty(".NAM",NAM_ID));
    mp.add(new TMetaProperty(".DSC",DSC_ID));
    mp.add(new TMetaProperty(".DESC",DSC_ID));
    mp.add(new TMetaProperty(".BIT.",BIT_ID,true));
    mp.add(new TMetaProperty(".MASK.",MSK_ID,true));
    mp.add(new TMetaProperty(".GATE.",GAT_ID,true));
    mp.add(new TMetaProperty(".XEGU",XEGU_ID));
    mp.add(new TMetaProperty(".URL",URL_ID));
    mp.add(new TMetaProperty(".MAX",MAX_ID));
    mp.add(new TMetaProperty(".MIN",MIN_ID));
    mp.add(new TMetaProperty(".XMAX",XMAX_ID));
    mp.add(new TMetaProperty(".XMIN",XMIN_ID));
    mp.add(new TMetaProperty(".DMASK.",DMASK_ID,true));
    mp.add(new TMetaProperty(".ONLINE",ONLINE_ID));      
    mp.add(new TMetaProperty(".RBMAX",RBMAX_ID));    
    mp.add(new TMetaProperty(".RBMIN",RBMIN_ID));    
    mp.add(new TMetaProperty(".RBAVE",RBAVE_ID));    
    mp.add(new TMetaProperty(".RBSUMMARY",RBSUMMARY_ID));    
  }
  public String[] getMetaPropertyDecorations()
  {
    if (instance == null) instance = new TMetaProperties();
    int len = mp.size();
    if (len == 0) return null;
    String[] lst = new String[len];
    for (int i=0; i<len; i++)
    {
      lst[i] = mp.get(i).getTag();
    }
    return lst;    
  }
  public static int getGateType(String property)
  {
    switch (getIndex(property))
    {
      case BIT_ID:
      case GAT_ID:
        return GATE_BOOLEAN;
      case MSK_ID:
        return GATE_FILTER;
      case DMASK_ID:
      case ONLINE_ID:
        return GATE_ARRAY;
    }    
    return GATE_NONE;
  }
  public static int getGate(String property)
  {
    int idx, bit, gate = 0;
    try
    {
      if (property.endsWith(".NAM"))
      { // trap these early
        return 0;
      }
      if (property.endsWith(".ONLINE")) 
      {
        return -1;
      }
      if ((idx=property.indexOf(".BIT.")) > 0)
      {
        bit = Integer.parseInt(property.substring(idx+5));
        gate = 0x01 << bit;
      }
      else if ((idx=property.indexOf(".MASK.")) > 0 ||
               (idx=property.indexOf(".GATE.")) > 0)
      {
        String istr = property.substring(idx+6);
        if (istr.startsWith("0x") || istr.startsWith("0X"))
          gate = Integer.parseInt(istr,16);
        else
          gate = Integer.parseInt(istr);
      }
      else if ((idx=property.indexOf(".DMASK.")) > 0)
      {
        String istr = property.substring(idx+7);
        if (istr.startsWith("0x") || istr.startsWith("0X"))
          gate = Integer.parseInt(istr,16);
        else
          gate = Integer.parseInt(istr);
      }
    }
    catch (Exception ignore) {}
    return gate;
  }
  public static String getTargetProperty(String property)
  {
    String ext;
    int idx = 0, edx;
    TMetaProperty tmp;
    Iterator<TMetaProperty> it = mp.iterator();
    while (it.hasNext())
    {
      tmp = it.next();
      ext = tmp.getTag();
      if ((idx=property.indexOf(ext)) > 0)
      {
        if (property.endsWith(ext) || tmp.isEmbedded)
        {
          if (ext.compareToIgnoreCase(".NAM") == 0)
          { // special cases
            if ((edx=property.indexOf(".DMASK.")) > 0) idx = edx;
            if ((edx=property.indexOf(".ONLINE")) > 0) idx = edx;
          }
          String p = property.substring(0,idx);
          if ((edx=p.indexOf(".RB")) > 0 && tmp.id == HST_ID)
          {
            if (isMetaProperty(p)) idx = edx;
            p = p.substring(0,idx);
          }
          return p;
        }
      }
    }
    return property;
  }
  public static int getIndex(String property)
  {
    String ext;
    TMetaProperty tmp;
    Iterator<TMetaProperty> it = mp.iterator();
    while (it.hasNext())
    {
      tmp = it.next();
      ext = tmp.getTag();
      if (property.indexOf(ext) > 0)
      {
        if (property.endsWith(ext) || tmp.isEmbedded) return tmp.getId();
      }
    }
    return -1;
  }
  public static boolean isStaticMetaProperty(String property)
  {
    int idx = getIndex(property);
    switch (idx)
    {
      case EGU_ID:
      case NAM_ID:
      case DSC_ID:
      case XEGU_ID:
      case URL_ID:
      case MAX_ID:
      case MIN_ID:
        return true;
      default:
        break;
    }
    return false; 
  }
  public static boolean isPQSMetaProperty(String property)
  {
    int idx = getIndex(property);
    switch (idx)
    {
      case EGU_ID:
      case DSC_ID:
      case XEGU_ID:
      case MAX_ID:
      case MIN_ID:
        return true;
      default:
        break;
    }
    return false; 
  }
  public static boolean isMetaProperty(String property)
  {
    if (getIndex(property) < 0) return false;
    return true; 
  }
  public static String getMetaExtension(String property)
  {
    String ext;
    TMetaProperty tmp;
    Iterator<TMetaProperty> it = mp.iterator();
    while (it.hasNext())
    {
      tmp = it.next();
      ext = tmp.getTag();
      if (property.indexOf(ext) > 0)
      {
        if (property.endsWith(ext) || tmp.isEmbedded) return tmp.getTag();
      }
    }
    return null;
  }
  public static boolean isStatic(String property)
  {
    if (property == null) return false;
    if (property.endsWith(".EGU")) return true;
    if (property.endsWith(".DESC")) return true;
    if (property.endsWith(".DSC")) return true;
    if (property.endsWith(".URL")) return true;
    if (property.endsWith(".MAX")) return true;
    if (property.endsWith(".MIN")) return true;
    if (property.endsWith(".XMAX")) return true;
    if (property.endsWith(".XMIN")) return true;
    return false;
  }
  public static String getSuggestedDescription(String property)
  {
    if (property.endsWith(".AR") || 
        property.endsWith(".ARC") ||
        property.endsWith(".ARCH")) return "central archived data for targeted device over time range";
    if (property.endsWith(".AR@") || 
        property.endsWith(".ARC@") || 
        property.endsWith(".ARCH@")) return "central archived data snapshot at given time";
    if (property.endsWith(".HIST") || 
        property.endsWith(".HST")) return "local archived data for targeted device over time range";
    if (property.endsWith(".HIST@") ||
        property.endsWith(".HST@")) return "local archived data snapshot at given time";
    if (property.endsWith(".EGU")) return "units and max/min settings";
    if (property.endsWith(".XEGU")) return "x-axis units and max/min settings (spectrum arrays)";
    if (property.endsWith(".NAM")) return "device/keyword names associated with property";
    if (property.endsWith(".DESC") || 
        property.endsWith(".DSC")) return "description of property";
    if (property.contains(".BIT.")) return "value of specified bit (integer types only)";
    if (property.contains(".MASK.")) return "masked data value  (integers only)";
    if (property.contains(".GATE.")) return "boolean result of applied mask(integers only)";
    if (property.endsWith(".URL")) return "URL for additional information";
    if (property.endsWith(".MAX")) return "maximum value";
    if (property.endsWith(".MIN")) return "minimum value";
    if (property.endsWith(".XMAX")) return "horizontal maximum value";
    if (property.endsWith(".XMIN")) return "horizontal minimum value";
    if (property.contains(".DMASK.")) return "multi-channel array gated against the device mask";
    if (property.contains(".ONLINE")) return "on line status";
    if (property.endsWith(".RBAVE")) return "average of history ring buffer values over sample interval";
    if (property.endsWith(".RBMIN")) return "min of history ring buffer values over sample interval";
    if (property.endsWith(".RBMAX")) return "max of history ring buffer values over sample interval";
    if (property.endsWith(".RBSUMMARY")) return "summary of history ring buffer values over sample interval";
    return "unknown meta property";
  }
  public static short getSuggestedFormat(String property)
  {
    if (property.endsWith(".AR")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".ARC")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".ARCH")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".AR@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".ARC@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".ARCH@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".HIST")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".HST")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".HIST@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".HST@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".EGU")) return TFormat.CF_USTRING;
    if (property.endsWith(".XEGU")) return TFormat.CF_USTRING;
    if (property.endsWith(".NAM")) return TFormat.CF_NAME64;
    if (property.endsWith(".DESC")) return TFormat.CF_NAME64;
    if (property.endsWith(".DSC")) return TFormat.CF_NAME64;
    if (property.contains(".BIT.")) return TFormat.CF_INT32;
    if (property.contains(".MASK.")) return TFormat.CF_INT32;
    if (property.contains(".GATE.")) return TFormat.CF_INT32;
    if (property.endsWith(".URL")) return TFormat.CF_TEXT;
    if (property.endsWith(".MAX")) return TFormat.CF_FLOAT;
    if (property.endsWith(".MIN")) return TFormat.CF_FLOAT;
    if (property.endsWith(".XMAX")) return TFormat.CF_FLOAT;
    if (property.endsWith(".XMIN")) return TFormat.CF_FLOAT;
    if (property.contains(".DMASK.")) return TFormat.CF_FLOAT;
    if (property.contains(".ONLINE")) return TFormat.CF_INT32;
    if (property.endsWith(".RBAVE")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".RBMIN")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".RBMAX")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".RBSUMMARY")) return TFormat.CF_DOUBLE;
    return TFormat.CF_NULL;
  }
  public static int getSuggestedDataSize(String property)
  {
    if (property.endsWith(".AR")) return 1000;
    if (property.endsWith(".ARC")) return 1000;
    if (property.endsWith(".ARCH")) return 1000;
    if (property.endsWith(".HIST")) return 1000;
    if (property.endsWith(".HST")) return 1000;
    if (property.endsWith(".NAM")) return 1000;
    if (property.endsWith(".AR@")) return 1000;
    if (property.endsWith(".ARC@")) return 0;
    if (property.endsWith(".ARCH@")) return 0;
    if (property.endsWith(".HIST@")) return 0;
    if (property.endsWith(".HST@")) return 0;
    if (property.endsWith(".URL")) return 80;
    if (property.endsWith(".RBAVE")) return 1;
    if (property.endsWith(".RBMIN")) return 1;
    if (property.endsWith(".RBMAX")) return 1;
    if (property.endsWith(".RBSUMMARY")) return 4;
    return 1;
  }
  public static boolean isHistorySnapshot(String property)
  {
    return (property.endsWith(".HIST@") || property.endsWith(".HST@"));
  }
  public static boolean isCentralArchive(String property)
  {
    return (property.endsWith(".ARCH") || 
            property.endsWith(".ARC") || 
            property.endsWith(".AR"));
  }
}
