/*
 * Created on Nov 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 * CHANGES
 * 2005-02-10 JW Constructor made public
 * 2005-02-10 JW * New Constructor
 *               * egu,xegu renamed to yRange,xRange 
 *               * units made private; get/set methods added.
 * 				 * description and redirection javaized
 *               * description renamed to "text", to avoid things like property.getDescription().getDescription()
 * 				 * array-properties javaized
 * 13.7.05 * PD parseComplexString() 
 */
package de.desy.tine.server.properties;

import de.desy.tine.definitions.TArrayType;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TGraphType;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.stringUtils.Str;

/**
 * Provides meta information about an exported property
 * 
 * @author duval 
 */
public class TPropertyDescription
{
  private short arrayType = TArrayType.AT_UNKNOWN;
  private int arrayRowLength = 1;
  private int arrayNumRows = 1;
  private String text = "description not given";
  private String url = "none";
  private TPropertyEGU yRange = new TPropertyEGU();;
  private TPropertyEGU xRange = new TPropertyEGU();;
  private String redirection;
  /**
   * Constructs a TPropertyDescription
   * 
   * This constructor only establishes the text description of the property
   * 
   * @param prpDescription is a string description of the property.
   * 
   */
  public TPropertyDescription(String prpDescription)
  {
    if (prpDescription != null && prpDescription.length() != 0)  
      text = Str.toAscii(prpDescription);
  }
  /**
   * Constructs a TPropertyDescription
   * 
   * @param newArrayType is the array type of the property
   * @param newArrayRowLength is the row length (for multiply dimensioned properties)
   * @param newNumRows is the number of rows (for multiply dimensioned properties)
   */
  public TPropertyDescription(short newArrayType, int newArrayRowLength, int newNumRows)
  {
    this.arrayType = newArrayType;
    this.arrayRowLength = newArrayRowLength;
    this.arrayNumRows = newNumRows;
  }
  /**
   * Constructs a TPropertyDescription
   * 
   * @param prpDescription is a string description of the property.
   * @param newArrayType is the array type of the property
   * @param newArrayRowLength is the row length (for multiply dimensioned properties)
   * @param newNumRows is the number of rows (for multiply dimensioned properties)
   */
  public TPropertyDescription(String prpDescription,short newArrayType, int newArrayRowLength, int newNumRows)
  {
    if (prpDescription != null && prpDescription.length() != 0)  
      text = Str.toAscii(prpDescription);
    this.arrayType = newArrayType;
    this.arrayRowLength = newArrayRowLength;
    this.arrayNumRows = newNumRows;
  }
  /**
   * Constructs a TPropertyDescription
   * 
   * @param prpDescription is a string description of the property.
   * @param yRange is the (nominal) vertical axis engineering units and range object
   * @param xRange is the horizontal axis engineering units and range object
   * @param newArrayType is the array type of the property
   * @param newArrayRowLength is the row length (for multiply dimensioned properties)
   * @param newNumRows is the number of rows (for multiply dimensioned properties)
   */
  public TPropertyDescription(String prpDescription,TPropertyEGU yRange,TPropertyEGU xRange,short newArrayType, int newArrayRowLength, int newNumRows)
  {
    if (prpDescription != null && prpDescription.length() != 0)  
      text = Str.toAscii(prpDescription);
    this.arrayType = newArrayType;
    this.arrayRowLength = newArrayRowLength;
    this.arrayNumRows = newNumRows;
    this.yRange = new TPropertyEGU(yRange);
    this.xRange = new TPropertyEGU(xRange);
  }
  /**
   * Constructs a TPropertyDescription for a property representing a 
   * scalar attribute
   * 
   * 
   * @param prpDescription is a string description of the property.
   * @param units is the engineering units
   * @param minValue is the minimum value
   * @param maxValue is the maximum value
   */
  public TPropertyDescription(String prpDescription,String units,float minValue,float maxValue)
  {
    if (prpDescription != null && prpDescription.length() != 0)  
      text = Str.toAscii(prpDescription);
    this.arrayType = TArrayType.AT_SCALAR;
    this.arrayRowLength = 1;
    this.arrayNumRows = 1;
    this.yRange = new TPropertyEGU(units,minValue,maxValue);
  }
  /**
   * Constructs a default TPropertyDescription
   * 
   */
  public TPropertyDescription() {}
  /**
   * @return Returns the xUnits.
   */
  public TPropertyEGU getXRange()
  {
    return xRange;
  }
  /**
   * @param egu
   *          The horizontal egu to set.
   */
  public void setXRange(TPropertyEGU egu)
  {
    if (egu == null) return;
    // set array type to TRACE if still unset. 
    if (egu.getMaxValue() != egu.getMinValue() &&
        arrayType == 0) arrayType = TArrayType.AT_TRACE;
    xRange = egu;
  }
  public void setXRange(String parseableString)
  {
    TPropertyEGU xegu = parseEGUfromString(parseableString);
    if (xegu == null) return;
    setXRange(xegu);
  } 
  /**
   * @return Returns the yUnits.
   */
  public TPropertyEGU getYRange()
  {
    return yRange;
  }
  /**
   * @param egu
   *          The vertical egu to set.
   */
  public void setYRange(TPropertyEGU egu)
  {
    if (egu == null) return;
    yRange = egu;
  }
  private TPropertyEGU parseEGUfromString(String parseableString)
  {
    TPropertyEGU egu = new TPropertyEGU();
    int mid, stop, i, k;
    try
    {
      stop = parseableString.length();
      mid = stop;
      if ((i=parseableString.indexOf('!')) != -1)
      {
        egu.setGraphType(TGraphType.getType(parseableString.substring(i+1,stop)));
        mid = i;
      }
      if ((i=parseableString.indexOf(':')) != -1) 
      {
        if ((k=parseableString.substring(i+1,mid).indexOf(' ')) == -1) k = mid; else k += i + 1;
        //parseableString.substring(0,i)
        egu.setMinValue(Float.parseFloat(parseableString.substring(0,i)));
        egu.setMaxValue(Float.parseFloat(parseableString.substring(i+1,k)));
        if (k != mid) egu.setUnits(parseableString.substring(k+1,mid));
      }
    }
    catch (Exception e)
    {
      MsgLog.log("TPropertyDescription.parseEGUfromString", e.getMessage(),TErrorList.parsing_error,e,0);
    }
    return egu;
  }
  public void setYRange(String parseableString)
  {
    TPropertyEGU yegu = parseEGUfromString(parseableString);
    if (yegu == null) return;
    setYRange(yegu);
  }
  /**
   * @return Returns the description.
   */
  public String getText()
  {
    return text;
  }
  /**
   * @param description
   *          The description text to set.
   */
  public void setText(String description)
  {
    if(description != null) this.text = Str.toAscii(description);
  }
  public String getUrl() { return url; }
  public void setUrl(String newUrl)
  {
    if (url != null) url = newUrl; 
  }
  /**
   * @return Returns the redirection.
   */
  public String getRedirection()
  {
    return redirection;
  }
  /**
   * @param redirection
   *          The redirection to set.
   */
  public void setRedirection(String redirection)
  {
    this.redirection = redirection;
  }
  /**
   * @return Returns the arrayNumRows.
   */
  public int getArrayNumRows()
  {
    return arrayNumRows;
  }
  /**
   * @param arrayNumRows
   *          The arrayNumRows to set.
   */
  public void setArrayNumRows(int arrayNumRows)
  {
    this.arrayNumRows = arrayNumRows;
  }
  /**
   * @return Returns the arrayRowLength.
   */
  public int getArrayRowLength()
  {
    return arrayRowLength;
  }
  /**
   * @param arrayRowLength
   *          The arrayRowLength to set.
   */
  public void setArrayRowLength(int arrayRowLength)
  {
    this.arrayRowLength = arrayRowLength;
  }
  /**
   * @return Returns the arrayType.
   */
  public short getArrayType()
  {
    return arrayType;
  }
  /**
   * @param arrayType
   *          The arrayType to set.
   */
  public void setArrayType(short arrayType)
  {
    this.arrayType = arrayType;
  }
  /**
   * @param parseableDescription
   *    A property description string of the form (legacy) 
   *    "[ymin:ymax yegu][xmin:xmax xegu]<description>"
   *    or (modern)
   *    "[vscale=ymin:ymax yegu][hscale=xmin:xmax xegu][vplot=lin/log][url=...]
   *     [desc=<description>]"
   */
  public void parseComplexString(String parseableDescription)
  {
    int i;
    if (parseableDescription == null) return;
    String[] toks = parseableDescription.split("]");
    String hsstr=null, vsstr=null, hplstr=null, vplstr=null, urlstr=null, dscstr=null;
    String c1=null, c2=null;
    try
    {
      for (i=0; i<toks.length; i++)
      {
        if (!toks[i].startsWith("["))
        {
          if (dscstr == null) dscstr = toks[i];
          toks[i] = "";
          continue;
        }
        toks[i] = toks[i].substring(1);
      }
      String[] parts;
      for (i=0; i<toks.length; i++)
      {
        if (toks[i].length() == 0) continue;
        parts = toks[i].split("=");
        if (parts.length < 1) continue;
        if (parts.length == 1)
        { // legacy style ...
          if (c1 == null) c1 = parts[0];
          else if (c2 == null) c2 = parts[0];
          continue;
        }
        if (parts[0].trim().compareToIgnoreCase("vscale")== 0 ||
            parts[0].trim().compareToIgnoreCase("scale")== 0 )
        {
          vsstr = parts[1].trim();
        }
        if (parts[0].trim().compareToIgnoreCase("hscale")== 0 ||
            parts[0].trim().compareToIgnoreCase("range")== 0 )
        {
          hsstr = parts[1].trim();
        }
        if (parts[0].trim().compareToIgnoreCase("url")== 0)
        {
          urlstr = parts[1].trim();
        }
        if (parts[0].trim().compareToIgnoreCase("desc")== 0)
        {
          dscstr = parts[1].trim();
        }
        if (parts[0].trim().compareToIgnoreCase("vplot")== 0 ||
            parts[0].trim().compareToIgnoreCase("plot")== 0)
        {
          vplstr = parts[1].trim();
        }
        if (parts[0].trim().compareToIgnoreCase("hplot")== 0)
        {
          hplstr = parts[1].trim();
        }
      }
      if (vsstr == null) vsstr = c1;
      if (hsstr == null) hsstr = c2;
      
      if (vsstr != null) setYRange(vsstr);
      if (hsstr != null) setXRange(hsstr);
      if (vplstr != null) getYRange().setGraphType(vplstr);
      if (hplstr != null) getXRange().setGraphType(hplstr);
      if (urlstr != null) setUrl(urlstr);
      if (dscstr != null) setText(dscstr);
    }
    catch (Exception e)
    {
      MsgLog.log("TPropertyDescription.parseComplexString", "paring error",TErrorList.parsing_error,e,0);
    }
  }
}