package de.desy.tine.stringUtils;

public class Str
{
  public static boolean isTrue(String s)
  {
    if (s == null) return false;
    if (s.compareToIgnoreCase("TRUE") == 0) return true;
    if (s.compareToIgnoreCase("YES") == 0) return true;
    if (s.compareToIgnoreCase("ON") == 0) return true;
    return false;
  }
  public static String getenv(String envKey)
  {
    if (envKey == null) return null;
    String lcEnvKey = envKey.toLowerCase();
    String ucEnvKey = envKey.toUpperCase();
    String env = null;
    if (lcEnvKey.contains("_"))
    { // given as environment variable: prepare system property:
      String[] pts = lcEnvKey.split("_");
      lcEnvKey = pts[0];
      for (int i=1; i<pts.length; i++) lcEnvKey += "."+pts[i];
    }
    if (ucEnvKey.contains("."))
    { // given as system property: prepare environment variable
      String[] pts = ucEnvKey.split("\\.");
      ucEnvKey = pts[0];
      for (int i=1; i<pts.length; i++) ucEnvKey += "_"+pts[i];
    }
    if ((env=System.getProperty(lcEnvKey)) != null) return env;
    return System.getenv(ucEnvKey);
  }
  public static boolean envIsTrue(String envKey)
  {
    return isTrue(getenv(envKey));
  }
  public static int atoi(String s)
  {
    if (s == null || s.length() == 0) return 0;
    int v = 0;
    try { v = Integer.parseInt(s); return v;} catch (Exception any) {};
    try { v = (int)Float.parseFloat(s); return v;} catch (Exception any) {};
    char[] chs = s.toCharArray();
    int i;
    for (i=0; i<chs.length; i++)
    {
      if (!Character.isDigit(chs[i])) break;
    }
    if (i > 0)
    {
      s = new String(chs,0,i);
      try { v = Integer.parseInt(s); return v;} catch (Exception any) {};
    }
    return v;
  }
  public static String trim(String s)
  {
    if (s == null || s.length() == 0) return "";
    int idx = s.indexOf(0);
    if (idx > 0) s = s.substring(0,idx);
    return s.trim();
  }
  public static String toAscii(String s)
  {
    if (s.chars().allMatch(c -> c < 128)) return s;
    int slen = s.length();
    char[] chars = new char[slen*2]; 
    System.arraycopy(s.toCharArray(), 0, chars, 0, slen);
    toAscii(chars);
    return new String(chars);
  }
  public static void toAscii(char[] chars)
  {
    if (chars == null || chars.length == 0) return;
    char c;
    boolean isascii = false, isutf8 = false;
    StringBuilder sb = new StringBuilder();
    for (int i=0; i<chars.length; i++)
    {
      c = chars[i];
      isascii = (short)c < 128;
      isutf8 = ((short)c == 0xc3);
      if (isascii)
      {
        sb.append(c);
      }
      else if (isutf8)
      {
        i++;
        c = chars[i];
        switch ((short)c)
        {
          case 0x84:
            sb.append('A');
            sb.append('E');
            break;
          case 0xa4:
            sb.append('a');
            sb.append('e');
            break;
          case 0x96:
            sb.append('O');
            sb.append('E');
            break;
          case 0xb6:
            sb.append('o');
            sb.append('e');
            break;
          case 0x9c:
            sb.append('U');
            sb.append('E');
            break;
          case 0xbc:
            sb.append('u');
            sb.append('e');
            break;
          case 0x9f:
            sb.append('s');
            sb.append('s');
            break;
          default:
            sb.append(c);
            break;
        }
      }
      else
      { /* assume it's windows-1252 */
        switch ((short)c)
        {
          case 0xc4:
            sb.append('A');
            sb.append('E');
            break;
          case 0xe4:
            sb.append('a');
            sb.append('e');
            break;
          case 0xd6:
            sb.append('O');
            sb.append('E');
            break;
          case 0xf6:
            sb.append('o');
            sb.append('e');
            break;
          case 0xdc:
            sb.append('U');
            sb.append('E');
            break;
          case 0xfc:
            sb.append('u');
            sb.append('e');
            break;
          case 0xdf:
            sb.append('s');
            sb.append('s');
            break;
          default:
            sb.append(c);
            break;
        }
      }
    }
    char[] nchars = sb.toString().toCharArray();
    System.arraycopy(nchars, 0, chars, 0, chars.length);
  }
}
