/*
 * Created on May 14, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package de.desy.tine.structUtils;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

import de.desy.tine.client.TLinkFactory;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.server.properties.TStockProperties;
import de.desy.tine.types.*;

/**
 * \internal
 * 
 * Global struct registry This class provides static methods which give
 *        access to a global tag structure registry.
 */
public final class TStructRegistry
{
  private static HashSet<String> sysKnownTags = new HashSet<String>();
  private static HashSet<String> acqOnceTags = new HashSet<String>();
  static {
    sysKnownTags.add("PRPQSr4"); // all servers
    sysKnownTags.add("CTQSr4");
    sysKnownTags.add("CLNQS");
    sysKnownTags.add("CONTBLr4");
    sysKnownTags.add("WRACCTBL");
    sysKnownTags.add("HQS");
    sysKnownTags.add("AMSr4");
    sysKnownTags.add("AMS");
    sysKnownTags.add("ADSr4");
    sysKnownTags.add("ADS");
    sysKnownTags.add("AWSr4");
    sysKnownTags.add("PQS");
    sysKnownTags.add("PQSX");
    sysKnownTags.add("XPQS");
    sysKnownTags.add("AQS");
    sysKnownTags.add("CONQS");
    sysKnownTags.add("SrvSetQy");
    sysKnownTags.add("HRSr4");
    sysKnownTags.add("XAMS");  // alarm servers
    sysKnownTags.add("XAMSr4");
    sysKnownTags.add("HstCfg"); // archive servers
    sysKnownTags.add("HstView");
    sysKnownTags.add("McaCfg");
    sysKnownTags.add("DAQFltr");
    sysKnownTags.add("DAQPrm");
    sysKnownTags.add("DAQRec");
    sysKnownTags.add("DclProc"); // state servers
    sysKnownTags.add("DclState"); // state servers
    sysKnownTags.add("PrfCfg");    
    sysKnownTags.add("DASQS"); // event servers
    sysKnownTags.add("ARBQS");
    sysKnownTags.add("TrgChn");
    sysKnownTags.add("TrgCmp");
    sysKnownTags.add("PIB");
    sysKnownTags.add("EVNDHDR");
    sysKnownTags.add("DHS");
    sysKnownTags.add("AIB");
    sysKnownTags.add("Filt");
    sysKnownTags.add("DEVQSr5");

//  new HstCfg(); // known within tine.jar
//  new McaCfg(); // known within tine.jar
//  new PrfCfg(); // known within tine.jar
//  new HstView();// known within tine.jar
//  new DAQPrm(); // known within tine.jar
//  new DAQRec(); // known within tine.jar
//  new DAQFltr();// known within tine.jar
    acqOnceTags.add("XAMS");  // alarm servers
    acqOnceTags.add("XAMSr4");
    acqOnceTags.add("HstCfg"); // archive servers
    acqOnceTags.add("HstView");
    acqOnceTags.add("McaCfg");
    acqOnceTags.add("DAQFltr");
    acqOnceTags.add("DAQPrm");
    acqOnceTags.add("DAQRec");
    acqOnceTags.add("DclProc"); // state servers
    acqOnceTags.add("DclState"); // state servers
    acqOnceTags.add("PrfCfg");    
    acqOnceTags.add("DASQS"); // event servers
    acqOnceTags.add("ARBQS");
    acqOnceTags.add("TrgChn");
    acqOnceTags.add("TrgCmp");
    acqOnceTags.add("PIB");
    acqOnceTags.add("EVNDHDR");
    acqOnceTags.add("DHS");
    acqOnceTags.add("AIB");
  }
  public static void addToSystem(String tag)
  {
    LinkedList<TStructDescription> ll = acqRegistry.get(tag);
    if (ll == null) 
    {
      ll = new LinkedList<TStructDescription>();
      acqRegistry.put(tag,ll);
    }    
  }
  /**
   * basic registry for 'system' defined or known structures
   */
  private static HashMap<String, TStructDescription> sysRegistry = new HashMap<String, TStructDescription>();
  /**
   * compound registry for 'acquired' or learned structures
   */
  private static HashMap<String, LinkedList<TStructDescription>> acqRegistry = new HashMap<String, LinkedList<TStructDescription>>();
  public static int numRegisteredStructs = 0;
  /**
   * @deprecated
   * @param tag
   * @return
   */
  public static NAME16II[] getRegisteredStructure(String tag)
  {
    NAME16II[] rv = null;
    TStructDescription dsc = get(tag);
    if (dsc != null)
    {
      rv = dsc.toNameIntIntArray();
    }
    return rv;
  }
  public static NAME64DBLDBL[] getStructureRegistration(String tag)
  {
    NAME64DBLDBL[] rv = null;
    TStructDescription dsc = get(tag);
    if (dsc != null)
    {
      rv = dsc.toNameDblDblArray();
    }
    return rv;
  }
  public static NAME64DBLDBL[] getStructureRegistration(String tag,String key)
  {
    NAME64DBLDBL[] rv = null;
    TStructDescription dsc = get(tag,key);
    if (dsc != null)
    {
      rv = dsc.toNameDblDblArray();
    }
    return rv;
  }
  /**
   * Registers an element of a tagged structure If a data acquisition
   *        needs to make use of user-defined structures, it is strongly
   *        suggested to register such structures at both the server and client
   *        ends. This entails passing the structure information at
   *        initialization time and giving the structure a 'tag' (i.e. a
   *        16-character name). Structure information is passed by calling the
   *        fill() method for each element of the structure in succession. When
   *        the entire structure has been registered the fill() method should be
   *        call once more with a a TFormat.CF_NULL data type, which effectively
   *        closes the structure registration.
   * @param tag
   *          is the (up to) 16 character structure tag name
   * @param fmt
   *          is the tine Data type appropriate to the structure element
   * @param siz
   *          is the array size of the structure element \b Example: \include
   *          eg_struct1.java
   */
  public static void fill(String tag, short fmt, int siz)
  { // no server key list 
    TStructDescription sd = get(tag);
    if (sd == null)
    {
      sd = new TStructDescription(tag);
      sysRegistry.put(sd.getTagName(), sd);
      sd.beginDefinition();
    }
    if ((fmt % 512) == TFormat.CF_NULL) // registration finished
    {
      sd.setArraySize((siz));
      sd.endDefinition();
    }
    else
    {
      sd.addField(fmt, siz);
    }
  }
  public static void fill(String tag, String name, short fmt, int siz)
  {
    TStructDescription sd = get(tag);
    if (sd == null)
    {
      sd = new TStructDescription(tag);
      sysRegistry.put(sd.getTagName(), sd);
      sd.beginDefinition();
    }
    if ((fmt % 512) == TFormat.CF_NULL) // registration finished
    {
      sd.setArraySize((siz));
      sd.endDefinition();
    }
    else
    {
      sd.addField(name, fmt, siz);
    }
  }
  public static void clear()
  {
    synchronized (acqRegistry)
    {
      if (acqRegistry.isEmpty()) return;
      Iterator<String> it = acqRegistry.keySet().iterator();
      String tag;
      int n = 0;
      while (it.hasNext())
      {
        tag = it.next();
        if (sysKnownTags.contains(tag)) continue;
        it.remove();
        n++;
      }
      if (n > 0) MsgLog.log("TStructRegistry.clear", "clear the acquired structure registry ("+n+" items)",0,null,0);
    }
  }
  public static void assignServerKey(TStructDescription sd,String key)
  {
    if (sd == null) return; 
    if (key == null) return; 
    String tag = sd.getTagName();
    if (sysKnownTags.contains(tag)) return; // leave in system registry
    if (sysRegistry.containsKey(tag)) sysRegistry.remove(tag);
    synchronized (acqRegistry)
    {
      LinkedList<TStructDescription> ll = acqRegistry.get(tag);
      if (ll == null) 
      {
        ll = new LinkedList<TStructDescription>();
        acqRegistry.put(tag, ll);
      }
      sd.addSrvKey(key);
      ll.add(sd);
    }
  }
  public static void assignServerKey(String tag,String key)
  {
    if (sysKnownTags.contains(tag)) return; // leave in system registry
    if (key != null) return; // this method can't work => leave in system registry
    //if (key == null) return; // leave in system registry
    TStructDescription sd = sysRegistry.get(tag);
    if (sd == null) return;
    //sysRegistry.remove(tag);
    synchronized (acqRegistry)
    {
      LinkedList<TStructDescription> ll = acqRegistry.get(tag);
      if (ll == null) 
      {
        ll = new LinkedList<TStructDescription>();
        acqRegistry.put(tag, ll);
      }
      sd.addSrvKey(key);
      ll.add(sd);
    }
  }
  public static void assignServerKey(String tag,String context,String server)
  {
    if (context == null || server == null) return;
    assignServerKey(tag, "/"+context+"/"+server);
  }  
  /**
   * Converts a byte array to a string. This method uses the description of a
   * tagged structure to interpret the contents of a given byte array. It
   * assumes that the byte array contains an arbitrary number of elements of the
   * type described by the given structure.
   * 
   * @param tag is the (up to) 16 character structure tag name
   * @param data is a byte array reference to the tagged structure's data.
   * 
   * @see de.desy.tine.client.TStructDescription.converteBytesToString();
   */
  public static String toString(String tag, byte[] data)
  {
    if (data == null) return "";
    return toString(tag,data,data.length);
  }
  /**
   * Converts a byte array to a string. This method uses the description of a
   * tagged structure to interpret the contents of a given byte array. It
   * assumes that the byte array contains an arbitrary number of elements of the
   * type described by the given structure.
   * 
   * @param tag is the (up to) 16 character structure tag name
   * @param data is a byte array reference to the tagged structure's data.
   * @param length is the desired array length to convert to string
   * 
   * @see de.desy.tine.client.TStructDescription.converteBytesToString();
   */
  public static String toString(String tag, byte[] data,int length)
  {
    return toString(get(tag),data,length);
  }
  /**
   * Converts a byte array to a string. This method uses the description of a
   * tagged structure to interpret the contents of a given byte array. It
   * assumes that the byte array contains an arbitrary number of elements of the
   * type described by the given structure.
   * 
   * @param tag is the (up to) 16 character structure tag name
   * @param key is the target server 'key' ("/context/server") 
   * identifying where the structure is known to be valid 
   * @param data is a byte array reference to the tagged structure's data.
   * @param length is the desired array length to convert to string
   * @see de.desy.tine.client.TStructDescription.converteBytesToString();
   */
  public static String toString(String tag, String key, byte[] data,int length)
  {
    return toString(get(tag,key),data,length);
  }
  private static String toString(TStructDescription tsd, byte[] data,int length)
  {
    String rv = "";
    StringBuffer buffer = new StringBuffer();
    int nStructs;
    if (tsd == null)
    {
      rv = new String("Structure " + " not registered");
    }
    else if ((data == null) || (data.length == 0))
    {
      rv = "";
    }
    else
    { // Tag exists, data valid
      nStructs = length / tsd.getRawLength();
      for (int i = 0; i < nStructs; i++)
      {
        buffer.append(tsd.convertBytesToString(data, i * tsd.getRawLength()));
      }
      rv = buffer.toString();
    }
    return rv;
  }  
  public static int getSizeInBytes(String tag)
  {
    TStructDescription s = get(tag);
    return (s == null) ? -1 : s.getRawLength();
  }
  public static int getSizeInBytes(String tag,String key)
  {
    if (key == null) return getSizeInBytes(tag);
    TStructDescription s = get(tag,key);
    return (s == null) ? -1 : s.getRawLength();
  }
  public static int getSizeInBytes(String tag,String context,String server)
  {
    String key = context == null ? null : "/"+context+"/"+server;
    return getSizeInBytes(tag,key);
  }
  public static void dumpRegisteredStructureTags()
  {
    String[] tags = getRegisteredStructureTags();
    TLinkFactory.dbgPrint("Registered structures:");
    for (String t : tags)  TLinkFactory.dbgPrint("  "+t);
  }
  public static String[] getRegisteredStructureTags()
  {
    return getRegisteredStructureTags(true,true);
  }
  @SuppressWarnings("unchecked")
  public static String[] getRegisteredStructureTags(boolean includeKeys,boolean includeSystem)
  {
    String[] lst = null;
    try
    {
      int n= 0;
      synchronized (acqRegistry)
      {
        Collection<LinkedList<TStructDescription>> acqsdll = acqRegistry.values();
        Object[] ll = acqsdll.toArray();
        TStructDescription[] acqsd = new TStructDescription[ll.length];
        for (Object l : ll)
        {
          acqsd[n++] = ((LinkedList<TStructDescription>)l).get(0);
        }
        n = acqsd.length;
        TStructDescription[] syssd = null;
        if (includeSystem)
        {
          syssd = sysRegistry.values().toArray(new TStructDescription[0]);
          n += syssd.length;      
        }
        lst = new String[n];
        n = 0; 
        if (syssd != null) for (TStructDescription sd : syssd)
        {
          lst[n++] = includeKeys ? sd.getDecoratedTagName(true) : sd.getTagName();
        }
        for (TStructDescription sd : acqsd)
        {
          lst[n++] = includeKeys ? sd.getDecoratedTagName(true) : sd.getTagName();      
        }
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    return lst;
  }
  /**
   * @see TStructRegistry.add()
   */
  public static synchronized void add(TStructDescription newStruct)
  {
    if (newStruct == null) return;
    String tag = newStruct.getTagName();
    if (!contains(tag))
    {
      sysRegistry.put(tag, newStruct);
      assignServerKey(tag, null);
    }
  }
  public static synchronized void add(TStructDescription newStruct,String key)
  {
    if (newStruct == null) return;
    String tag = newStruct.getTagName();
    if (!contains(tag,key))
    {
      assignServerKey(newStruct,key);
    }
  }
  public static synchronized void add(TStructDescription newStruct,String context,String server)
  {
    if (newStruct == null) return;
    if (context == null || server == null) return;
    assignServerKey(newStruct,"/"+context+"/"+server);
  }
  /**
   * @see TStructRegistry.get()
   */
  public static TStructDescription get(String structName)
  {
    TStructDescription tsd = sysRegistry.get(structName);
    if (tsd != null) return tsd; // no key list (yet)
    synchronized (acqRegistry)
    {
      LinkedList<TStructDescription> ll = acqRegistry.get(structName);
      if (ll == null) return null;
      return ll.getFirst();
    }
  }
  public static TStructDescription get(String structName,String key)
  {
    TStructDescription tsd;
    synchronized (acqRegistry)
    {
      LinkedList<TStructDescription> ll = acqRegistry.get(structName);
      if (ll == null || ll.size() == 0|| key == null) 
      {
        return get(structName);
      }
      Iterator<TStructDescription> itr = ll.iterator();
      while (itr.hasNext())
      {
        tsd = itr.next();
        if (tsd.containsSrvKey(key)) return tsd;
      }
    }
    return null;
  }
  public static TStructDescription get(String structName,String context,String server)
  {
    if (context == null || server == null) return get(structName);
    return get(structName,"/"+context+"/"+server);
  }
  /**
   * @see TStructRegistry.contains()
   */
  public static boolean contains(String structName)
  {
    if (sysRegistry.containsKey(structName)) return true;
    synchronized (acqRegistry)
    {
      return acqRegistry.containsKey(structName);
    }
  }
  public static boolean contains(String structName,String key)
  {
    if (key == null) return contains(structName);
    if (sysRegistry.containsKey(structName)) return true;
    if (sysKnownTags.contains(structName))
    { // system known but not yet in the registry (client-side call?)
      if (structName.equals("AMSr4")) 
      { 
        TStockProperties.addAMStoStructReg();
        return true; 
      }
      if (structName.equals("ADSr4")) 
      { 
        TStockProperties.addADStoStructReg();
        return true; 
      }
    }
    synchronized (acqRegistry)
    {
      LinkedList<TStructDescription> ll = acqRegistry.get(structName);
      if (ll == null) return false;
      Iterator<TStructDescription> itr = ll.iterator();
      TStructDescription tsd;
      while (itr.hasNext())
      {
        tsd = itr.next();
        if (tsd.containsSrvKey(key)) return true;
      }
      return false;
    }
  }
  public static boolean acquireOnce(String structName)
  {
    return acqOnceTags.contains(structName);
  }
  public static void acquiredOnce(String structName)
  {
    acqOnceTags.remove(structName);
  }
  public static boolean contains(String structName,String context,String server)
  {
    if (acqOnceTags.contains(structName))
    {
      if (TQuery.AcquireAndRegisterStructInfo(context, server, structName) == 0)
        acqOnceTags.remove(structName);
    }
    if (context == null || server == null) return contains(structName);
    return contains(structName,"/"+context+"/"+server);
  }
}