package de.desy.tine.traceUtils;

import de.desy.tine.structUtils.TTaggedStructure;

public class TTraceHeader extends TTaggedStructure
{
  private char[] nam = new char[64];
  private String namString = null;
  private char[] dsc = new char[256];
  private String dscString = null;
  private int[] fmt = new int[1];
  private int[] siz = new int[1];
  private int[] preTrigger = new int[1]; // trigger position in array
  // STANDARD Scaling for the Time Axis: ScaleX * array-index + OffsetX
  private float[] scaleX = new float[1]; // (sec)
  private float[] offsetX  = new float[1]; // (sec)
  private char[] unitsX = new char[16]; // Units for X-axis ("normally" "s", "sec" or "seconds")
  private String unitsXString = null;
  private float[] plotMaxY = new float[1];// Default maximum value for Y scale
  private float[] plotMinY = new float[1];// Default minimum value for Y scale
  private char[] unitsY = new char[16]; // Units for Y-axis 
  private String unitsYString = null;
  private byte[] reserved = new byte[256];
  private String charsToString(char[] c)
  {
    boolean done = false;
    for (int i=0; i<c.length; i++)
    {
      if (c[i] != 0 && !done) continue;
      done = true;
      c[i] = 0;
    }
    return new String(c).trim();
  }
  private void initStructure()
  {
    addField(nam,"deviceName");
    addField(dsc,"deviceDesc");
    addField(fmt,"dataFormat");
    addField(siz,"arraySize");
    addField(preTrigger,"preTrigger");
    addField(scaleX,"scaleX");
    addField(offsetX,"offsetX");
    addField(unitsX,"unitsX");
    addField(plotMaxY,"plotMaxY");
    addField(plotMinY,"plotMinY");
    addField(unitsY,"unitsY");
    addField(reserved,"reserved");
    initDone();    
  }
  TTraceHeader()
  {
    super("TraceHS");
    initStructure();
  }
  public String getChannelName()
  {
    if (namString == null) namString = charsToString(nam);
    return namString;
  }
  public String getChannelDescription()
  {
    if (dscString == null) dscString = charsToString(dsc);
    return dscString;
  }
  public int getPreTrigger()
  {
    return preTrigger[0];
  }
  public float getXScaleFactor()
  {
    return scaleX[0];
  }
  public float getXOffset()
  {
    return offsetX[0];
  }
  public String getXUnits()
  {
    if (unitsXString == null) 
    {      
      unitsXString = charsToString(unitsX); //new String(unitsX).trim();
    }
    return unitsXString;
  }
  public float getYPlotMaxmimum()
  {
    return plotMaxY[0];
  }
  public float getYPlotMinimum()
  {
    return plotMinY[0];
  }
  public String getYUnits()
  {
    if (unitsYString == null) unitsYString = charsToString(unitsY);
    return unitsYString;
  }
  public int getTraceLength()
  {
    return siz[0];
  }
  public short getDataFormat()
  {
    return (short)fmt[0];
  }
}
