package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public class FLTFLT implements TCompoundDataObject
{
  public float f1val;
  public float f2val;
  public double getDoubleValue() { return (double)f1val; }
  public int getIntegerValue() { return (int)f2val; }
  public static final int sizeInBytes = 8;
  public static final short dFormat = TFormat.CF_FLTFLT;
  public void setValues(float newF1Value, float newF2Value)
  {
    f1val = newF1Value; f2val = newF2Value; 
  }
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public FLTFLT(FLTFLT instance) {f1val = instance.f1val; f2val = instance.f2val; } 
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof FLTFLT)
    {
      FLTFLT instance = (FLTFLT)from;
      f1val = instance.f1val; f2val = instance.f2val;
    }
  }
  public FLTFLT() {f1val = 0; f2val = 0; } 
  public FLTFLT(float f) { f1val = f; f2val = 0; }  
  public FLTFLT(float f1,float f2) { f1val = f1; f2val = f2; } 
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
      return "["+ f1val + separator + f2val + "]";
  }
  public String getCsvString()
  {
    return f1val+" "+f2val;
  }
  public FLTFLT newInstance()
  {
    return new FLTFLT();
  }
  public FLTFLT clone()
  {
    return new FLTFLT(this);
  }
  public boolean equals(FLTFLT instance)
  {
    if (instance.f1val != f1val) return false;
    if (instance.f2val != f2val) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] b = Swap.Float(f1val);
      ds.write(b,0,4);
      b = Swap.Float(f2val);
      ds.write(b,0,4);
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTFLT",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[4];
      ds.read(b,0,4);
      f1val = Swap.Float(b);
      ds.read(b,0,4);
      f2val = Swap.Float(b);
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTFLT",e.getMessage(),TErrorList.code_failure,e,0);        
    }  
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(f1val);
    result = PRIME * result + Float.floatToIntBits(f2val);
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final FLTFLT other = (FLTFLT) obj;
    return equals(other);
  }    
}
