package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public final class FLTFLTINT implements TCompoundDataObject
{
  public float f1val;
  public float f2val;
  public int ival;
  public double getDoubleValue() { return (double)f1val; }
  public int getIntegerValue() { return (int)ival; }
 public static final int sizeInBytes = 12;
  public static final short dFormat = TFormat.CF_FLTFLTINT;
  public void setValues(float newF1Value, float newF2Value, int newIValue)
  {
    ival = newIValue; f1val = newF1Value; f2val = newF2Value;
  }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public FLTFLTINT() { f1val = 0; f2val = 0; ival = 0; }  
  public FLTFLTINT(float f) { f1val = f; f2val = 0; ival = 0; }  
  public FLTFLTINT(int i) { f1val = 0; f2val = 0; ival = i; }  
  public FLTFLTINT(float f1,float f2,int i) { f1val = f1; f2val = f2; ival = i; }  
  public FLTFLTINT(FLTFLTINT instance) { f1val = instance.f1val; f2val = instance.f2val; ival = instance.ival; }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof FLTFLTINT)
    {
      FLTFLTINT instance = (FLTFLTINT)from;
      f1val = instance.f1val; f2val = instance.f2val; ival = instance.ival; 
    }
  } 
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  @Override
  public String toString()
  {
      return "["+ f1val + separator + f2val + separator + ival +"]";
  }
  public String getCsvString()
  {
    return f1val+" "+f2val+" "+ival;
  }  
  public FLTFLTINT newInstance()
  {
    return new FLTFLTINT();
  }
  public FLTFLTINT clone()
  {
    return new FLTFLTINT(this);
  } 
  public boolean equals(FLTFLTINT instance)
  {
    if (instance.ival != ival) return false;
    if (instance.f1val != f1val) return false;
    if (instance.f2val != f2val) return false;
    return true;
  }
 public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] b = Swap.Float(f1val);
      ds.write(b,0,4);
      b = Swap.Float(f2val);
      ds.write(b,0,4);
      ds.writeInt(Swap.Int(ival));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTFLTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[4];
      ds.read(b,0,4);
      f1val = Swap.Float(b);
      ds.read(b,0,4);
      f2val = Swap.Float(b);
      ival = Swap.Int(ds.readInt());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTFLTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }  
  }
  @Override
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(f1val);
    result = PRIME * result + Float.floatToIntBits(f2val);
    result = PRIME * result + ival;
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final FLTFLTINT other = (FLTFLTINT) obj;
    return equals(other);
  }    
}