package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public final class FLTINT implements TCompoundDataObject
{
  public float fval;
  public int ival;
  public double getDoubleValue() { return (double)fval; }
  public int getIntegerValue() { return (int)ival; }
  public static final int sizeInBytes = 8;
  public static final short dFormat = TFormat.CF_FLTINT;
  public int getSizeInBytes() { return sizeInBytes; }
  public void setValues(float newFValue, int newIValue)
  {
    ival = newIValue; fval = newFValue;
  }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public FLTINT() {fval = 0; ival = 0; }  
  public FLTINT(float f) { fval = f; ival = 0; }  
  public FLTINT(int i) { ival = i; }  
  public FLTINT(float f,int i) { fval = f; ival = i; }  
  public FLTINT(FLTINT instance) { fval = instance.fval; ival = instance.ival; }
  
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
   public String toString()
  {
      return "["+ fval + separator + ival + "]";
  }
   public String getCsvString()
   {
     return fval+" "+ival;
   }  
   public FLTINT newInstance()
   {
     return new FLTINT();
   }
   public FLTINT clone()
   {
     return new FLTINT(this);
   }
  
  public boolean equals(FLTINT instance)
  {
    if (instance.ival != ival) return false;
    if (instance.fval != fval) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] b = Swap.Float(fval);
      ds.write(b,0,4);
      ds.writeInt(Swap.Int(ival));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[4];
      ds.read(b,0,4);
      fval = Swap.Float(b);
      ival = Swap.Int(ds.readInt());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }	 
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(fval);
    result = PRIME * result + ival;
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final FLTINT other = (FLTINT) obj;
    return equals(other);
  }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof FLTINT)
    {
      FLTINT instance = (FLTINT)from;
      fval = instance.fval; ival = instance.ival;
    }
  }
}
