package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public final class FLTINTINT implements TCompoundDataObject
{
  public float f1val;
  public int i1val;
  public int i2val;
  public double getDoubleValue() { return (double)f1val; }
  public int getIntegerValue() { return (int)i1val; }
  public static final int sizeInBytes = 12;
  public static final short dFormat = TFormat.CF_FLTINTINT;
  public void setValues(float newFValue, int newI1Value, int newI2Value)
  {
    f1val = newFValue; i1val = newI1Value; i2val = newI2Value;
  }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public FLTINTINT() { f1val = 0; i1val = 0; i2val = 0; }  
  public FLTINTINT(float f) { f1val = f; i1val = 0; i2val = 0; }  
  public FLTINTINT(int i) { f1val = 0; i1val = i; i2val = 0; }  
  public FLTINTINT(float f1,int i1,int i2) { f1val = f1; i1val = i1; i2val = i2; }  
  public FLTINTINT(FLTINTINT instance) { f1val = instance.f1val; i1val = instance.i1val; i2val = instance.i2val; }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof FLTINTINT)
    {
      FLTINTINT instance = (FLTINTINT)from;
      f1val = instance.f1val; i1val = instance.i1val; i2val = instance.i2val;
    }
  }
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  @Override
  public String toString()
  {
      return "["+ f1val + separator + i1val + separator + i2val +"]";
  }
  public String getCsvString()
  {
    return f1val+" "+i1val+" "+i2val;
  }  
  public FLTINTINT newInstance()
  {
    return new FLTINTINT();
  }
  public FLTINTINT clone()
  {
    return new FLTINTINT(this);
  }
  public boolean equals(FLTINTINT instance)
  {
    if (instance.i1val != i1val) return false;
    if (instance.i2val != i2val) return false;
    if (instance.f1val != f1val) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] b = Swap.Float(f1val);
      ds.write(b,0,4);
      ds.writeInt(Swap.Int(i1val));
      ds.writeInt(Swap.Int(i2val));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTINTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[4];
      ds.read(b,0,4);
      f1val = Swap.Float(b);
      i1val = Swap.Int(ds.readInt());
      i2val = Swap.Int(ds.readInt());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("FLTINTINT",e.getMessage(),TErrorList.code_failure,e,0);        
    }  
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(f1val);
    result = PRIME * result + i1val;
    result = PRIME * result + i2val;
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final FLTINTINT other = (FLTINTINT) obj;
    return equals(other);
  }    
}