package de.desy.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.*;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.structUtils.TTaggedStructure;

public class HISTORY implements TCompoundDataObject
{
  public static final int HDR_SIZE = 16;
  private double timestamp;
  private int sysstamp;
  private int usrstamp;
  private short fmt = TFormat.CF_NULL;
  private String tag = null;
  private Object hDataObject = null;
  private Object hStructObject = null;
  private int sizeInBytes;
  private int dataLength = 1;
  private byte[] hdrbytes = new byte[HDR_SIZE];
  public TDataType getDataObject()
  {
    TDataType td = null;
    if (hDataObject != null)
    {
      td = new TDataType(1,fmt);
      td.setDataTimeStamp(timestamp);
      td.sysDataStamp = sysstamp;
      td.usrDataStamp = usrstamp;
      td.putData(hDataObject);
      td.dCompletionLength = 1;
    }
    else if (hStructObject != null)
    {
      td = new TDataType((TTaggedStructure[])hStructObject);
      td.setDataTimeStamp(timestamp);
      td.sysDataStamp = sysstamp;
      td.usrDataStamp = usrstamp;
      td.dCompletionLength = 1;
    }
    return td;
  }
  public HISTORY()
  {
    timestamp = 0.0;
    sysstamp = 0;
    usrstamp = 0;
    fmt = TFormat.CF_NULL;
    tag = "";
    hDataObject = null;
    hStructObject = null;
    sizeInBytes = TFormat.getHistoryHeaderSize();
    
  }
  public HISTORY(TDataType d)
  {
    if (d == null) return;
    timestamp = (double)(d.dTimestamp/1000)+((double)(d.dTimestamp%1000)/1000.0);
    sysstamp = d.sysDataStamp;
    usrstamp = d.usrDataStamp;
    fmt = d.dFormat;
    tag = fmt == TFormat.CF_STRUCT ? d.getTag() : TFormat.toString(fmt);
    hDataObject = d.getDataObject();
    hStructObject = d.getStructObject();
    dataLength = d.dArrayLength;
    sizeInBytes = TFormat.formatSizeOf(fmt) * d.dArrayLength +
                  TFormat.getFormatHeaderSize(fmt) +
                  TFormat.getHistoryHeaderSize();
    hByteArray = new byte[sizeInBytes];
  }
  public HISTORY(double time,short svalue,int sstmp,int ustmp)
  {
    timestamp = time;
    sysstamp = sstmp;
    usrstamp = ustmp;
    fmt = TFormat.CF_INT16;
    tag = TFormat.toString(fmt);
    hDataObject = new short[1];
    ((short[])hDataObject)[0] = svalue;
    sizeInBytes = TFormat.formatSizeOf(fmt) + TFormat.getHistoryHeaderSize();
    hByteArray = new byte[sizeInBytes];
  }
  public HISTORY(double time,int ivalue,int sstmp,int ustmp)
  {
    timestamp = time;
    sysstamp = sstmp;
    usrstamp = ustmp;
    fmt = TFormat.CF_INT32;
    tag = TFormat.toString(fmt);
    hDataObject = new int[1];
    ((int[])hDataObject)[0] = ivalue;
    sizeInBytes = TFormat.formatSizeOf(fmt) + TFormat.getHistoryHeaderSize();
    hByteArray = new byte[sizeInBytes];
  }
  public HISTORY(double time,float fvalue,int sstmp,int ustmp)
  {
    timestamp = time;
    sysstamp = sstmp;
    usrstamp = ustmp;
    fmt = TFormat.CF_FLOAT;
    tag = TFormat.toString(fmt);
    hDataObject = new float[1];
    ((float[])hDataObject)[0] = fvalue;
    sizeInBytes = TFormat.formatSizeOf(fmt) + TFormat.getHistoryHeaderSize();
    hByteArray = new byte[sizeInBytes-TFormat.getHistoryHeaderSize()];
  }
  public HISTORY(double time,double dvalue,int sstmp,int ustmp)
  {
    timestamp = time;
    sysstamp = sstmp;
    usrstamp = ustmp;
    fmt = TFormat.CF_DOUBLE;
    tag = TFormat.toString(fmt);
    hDataObject = new double[1];
    ((double[])hDataObject)[0] = dvalue;
    sizeInBytes = TFormat.formatSizeOf(fmt) + TFormat.getHistoryHeaderSize();
    hByteArray = new byte[sizeInBytes];
  }
  public HISTORY(double time,TCompoundDataObject value,int sstmp,int ustmp)
  {
    timestamp = time;
    sysstamp = sstmp;
    usrstamp = ustmp;
    fmt = value.getFormat();
    tag = TFormat.toString(fmt);
    hDataObject = new TCompoundDataObject[1];
    ((TCompoundDataObject[])hDataObject)[0] = value;
    sizeInBytes = TFormat.formatSizeOf(fmt) + TFormat.getHistoryHeaderSize();
    hByteArray = new byte[sizeInBytes];
  }
  public HISTORY(double time,TTaggedStructure value,int sstmp,int ustmp)
  {
    timestamp = time;
    sysstamp = sstmp;
    usrstamp = ustmp;
    fmt = TFormat.CF_STRUCT;
    tag = value.getName();
    hStructObject = new TTaggedStructure[1];
    ((TTaggedStructure[])hStructObject)[0] = value;
    sizeInBytes = TFormat.formatSizeOf(fmt) + TFormat.getHistoryHeaderSize();
    hByteArray = new byte[sizeInBytes];
  }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof HISTORY)
    {
    }
  }  
  public HISTORY newInstance()
  {
    return new HISTORY();
  }
  public HISTORY clone()
  {
    return new HISTORY(this.getDataObject());
  }
  private byte[] getObjectBytes()
  {
    if (hDataObject != null)
    {
      switch (fmt)
      {
        case TFormat.CF_CHAR:
          return (new String((char[])hDataObject).getBytes());
        case TFormat.CF_INT16:
          return Swap.ShortToBytes(((short[])hDataObject)[0]);
        case TFormat.CF_INT32:
          return Swap.IntToBytes(((int[])hDataObject)[0]);
        case TFormat.CF_FLOAT:
          return Swap.Float(((float[])hDataObject)[0]);
        case TFormat.CF_DOUBLE:
          return Swap.Double(((double[])hDataObject)[0]);
        case TFormat.CF_IMAGE:
          return ((IMAGE)hDataObject).toByteArray();
        case TFormat.CF_STRING:
          return ((String[])hDataObject)[0].getBytes();
        case TFormat.CF_KEYVALUE:
          return ((KEYVALUE[])hDataObject)[0].toString().getBytes();
        default:
          return ((TCompoundDataObject[])hDataObject)[0].toByteArray();
      }
    }
    if (hStructObject != null)
    {
      return ((TTaggedStructure[])hStructObject)[0].toByteArray();      
    }
    return null;
  }
  private void putObjectBytes(DataInputStream dis)
  {
    try
    {
      if (hDataObject != null)
      {
        switch (fmt)
        {
          case TFormat.CF_BYTE:
            break;
          case TFormat.CF_CHAR:
            for (int i=0; i < dataLength; i++) 
            {
              ((char[])hDataObject)[i] = (char)dis.readByte();
            }
            break;
          case TFormat.CF_UINT16:
          case TFormat.CF_INT16:
            ((short[])hDataObject)[0] = Swap.Short(dis.readShort());
            break;
          case TFormat.CF_BOOLEAN:
          case TFormat.CF_UINT32:
          case TFormat.CF_INT32:
            ((int[])hDataObject)[0] = Swap.Int(dis.readInt());
            break;
          case TFormat.CF_INT64:
            ((long[])hDataObject)[0] = Swap.Long(dis.readLong());
            break;
          case TFormat.CF_FLOAT:
            dis.read(hdrbytes, 0, 4);
            ((float[])hDataObject)[0] = Swap.Float(hdrbytes);
            break;
          case TFormat.CF_DOUBLE:
            dis.read(hdrbytes, 0, 8);
            ((double[])hDataObject)[0] = Swap.Double(hdrbytes);
            break;
          case TFormat.CF_IMAGE:
            if (hDataObject.getClass().isArray())
              ((IMAGE[])hDataObject)[0].read(dis);
            else
              ((IMAGE)hDataObject).read(dis);
            break;
          case TFormat.CF_KEYVALUE:
            StringBuffer sb;
            byte b;
            sb = new StringBuffer();
            while ((b=dis.readByte()) != 0) sb.append((char)b);
            ((KEYVALUE[])hDataObject)[0].putString(sb.toString());
            break;
          case TFormat.CF_STRING:
            sb = new StringBuffer();
            while ((b=dis.readByte()) != 0) sb.append((char)b);
            ((String[])hDataObject)[0] = sb.toString();
            break;
          default:
            byte[] ba = new byte[sizeInBytes-TFormat.getHistoryHeaderSize()];
            dis.read(ba,0,ba.length);
            ((TCompoundDataObject[])hDataObject)[0].setByteArray(ba);
            ((TCompoundDataObject[])hDataObject)[0].toStruct();
            break;
        }
        return;
      }
      if (hStructObject != null)
      {
        byte[] ba = new byte[sizeInBytes-TFormat.getHistoryHeaderSize()];
        ((TTaggedStructure[])hStructObject)[0].toStruct(ba);
      }
    }
    catch (Exception e)
    {
      MsgLog.log("putObjectBytes","exception "+e.getMessage(),TErrorList.code_failure,e,0);
    }
  }
  private byte[] hByteArray = null;
  public byte[] getByteArray()
  {
    return hByteArray;
  }
  public short getFormat()
  {
    return TFormat.CF_HISTORY;
  }
  public short getCarriedFormat()
  {
    return fmt;
  }
  public int getSizeInBytes()
  {
    return sizeInBytes;
  }
  public void setByteArray(byte[] byteArray)
  {
    hByteArray = byteArray;
  }
  ByteArrayOutputStream dBuffer;
  public byte[] toByteArray()
  {
    try
    {
      dBuffer = new ByteArrayOutputStream(sizeInBytes);
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.write(Swap.Double(timestamp));
      ds.writeInt(Swap.Int(sysstamp));
      ds.writeInt(Swap.Int(usrstamp));
      ds.write(getObjectBytes());
      ds.close();
      return dBuffer.toByteArray();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("HISTORY",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    return null;
  }
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream bais = new ByteArrayInputStream(hByteArray);
      DataInputStream dis = new DataInputStream(bais);
      dis.read(hdrbytes,0,8);
      timestamp = Swap.Double(hdrbytes);
      sysstamp = Swap.Int(dis.readInt());
      usrstamp = Swap.Int(dis.readInt());
      putObjectBytes(dis);
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("HISTORY",e.getMessage(),TErrorList.code_failure,e,0);        
    }
  }
  public String getTag()
  {
    return tag;
  }
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
    TDataType tdt = getDataObject();
    tdt.setArrayDelimiter(" ");
    String s = "value: "+tdt.toString()+"\n";
    s += "timestamp: "+TDataTime.toString(timestamp)+"\n";
    s += "sysstamp: "+sysstamp+"\n";
    s += "usrstamp: "+usrstamp+"\n";    
    return s;
  }
  public String getCsvString()
  {
    return "[HISTORY Data]";
  }  
  public double getDoubleValue() { return (double)0; }
  public int getIntegerValue() { return (int)0; }
}
