package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;

public final class NAME16FI implements TCompoundDataObject
{
  public String name;
  public int ival;
  public float fval;
  public double getDoubleValue() { return (double)fval; }
  public int getIntegerValue() { return (int)ival; }
  public static final int sizeInBytes = 24;
  public static final short dFormat = TFormat.CF_NAME16FI;
  public void setValues(String newName, float newFValue, int newIValue)
  {
    name = newName; ival = newIValue; fval = newFValue;
  }
  public void setName(String newName) { name = newName; }
  public String getName() { return name; }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public NAME16FI() {name = new String(new StringBuffer(16)); ival = 0; fval =0;}  
  public NAME16FI(String Name) { name = Name; ival = 0; fval = 0;}  
  public NAME16FI(String Name,int i) { name = Name; ival = i; fval = 0;}  
  public NAME16FI(String Name,float f) { name = Name; ival = 0; fval = f;}  
  public NAME16FI(String Name,int i,float f) { name = Name; ival = i; fval = f;}  
  public NAME16FI(String Name,float f, int i) { name = Name; ival = i; fval = f;}  
  public NAME16FI(NAME16FI instance) 
  { 
    name = instance.name == null ? new String(new StringBuffer(16)) : new String(instance.name); 
    ival = instance.ival; fval = instance.fval; 
  }
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof NAME16FI)
    {
      NAME16FI instance = (NAME16FI)from;
      name = instance.name == null ? new String(new StringBuffer(16)) : new String(instance.name); 
      ival = instance.ival; fval = instance.fval; 
    }
  }
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
      return "["+ name + separator +  fval + separator + ival + "]";
  }
  public String getCsvString()
  {
    String tname = name.trim();
    if (tname.contains(",") || tname.contains(" ") ||
        tname.contains("\t") || tname.contains("\n"))
    {
      return "\""+tname+" "+fval+" "+ival+"\"";
    }
    return tname + " "+fval+" "+ival;
  }
  public NAME16FI newInstance()
  {
    return new NAME16FI();
  }
  public NAME16FI clone()
  {
    return new NAME16FI(this);
  }          
  public boolean equals(NAME16FI instance)
  {
    if (instance.name == null || name == null) return false;
    if (instance.name.compareTo(name) != 0) return false;
    if (instance.fval != fval) return false;
    if (instance.ival != ival) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    if (name == null) name = new String(new StringBuffer(16));
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] bname = name.getBytes();
      byte[] b = new byte[16];
      for (int i=0; i<16 && i<bname.length; i++) b[i] = bname[i];
      ds.write(b,0,b.length);
      byte[] fb = Swap.Float(fval);
      ds.write(fb,0,4);
      ds.writeInt(Swap.Int(ival));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAME16DBLDBL","could not prepare byte stream",TErrorList.code_failure,e,0);
      return null;
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[sizeInBytes];
      ds.read(b,0,16);
      int len = 0; for (int i=0; i<16 && b[i] != 0; i++, len++);
      name = new String(b,0,len);
      byte[] fb = new byte[4];
      ds.read(fb,0,4);
      fval = Swap.Float(fb);
      ival = Swap.Int(ds.readInt());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAME16FI","could not prepare byte stream",TErrorList.code_failure,e,0);
    }  
  }
  @Override
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(fval);
    result = PRIME * result + ival;
    result = PRIME * result + ((name == null) ? 0 : name.hashCode());
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final NAME16FI other = (NAME16FI) obj;
    if (Float.floatToIntBits(fval) != Float.floatToIntBits(other.fval)) return false;
    return equals(other);
  }  
}