package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;

public final class NAME64DBLDBL implements TCompoundDataObject
{
  public String name;
  public double d1val;
  public double d2val;
  public double getDoubleValue() { return (double)d1val; }
  public int getIntegerValue() { return (int)d2val; }
  public static final int sizeInBytes = 80;
  public static final short dFormat = TFormat.CF_NAME64DBLDBL;
  public void setValues(String newName, double newD1Value, double newD2Value)
  {
    name = newName; d1val = newD1Value; d2val = newD2Value;
  }
  public void setName(String newName) { name = newName; }
  public String getName() { return name; }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  public byte[] hByteArray;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public NAME64DBLDBL() {name = new String(new StringBuffer(64)); d1val = 0; d2val = 0;}  
  public NAME64DBLDBL(String Name) { name = Name; d1val = 0; d2val = 0;}  
  public NAME64DBLDBL(String Name,double d) { name = Name; d1val = d; d2val = 0;}  
  public NAME64DBLDBL(String Name,double d1,double d2) { name = Name; d1val = d1; d2val = d2;}  
  public NAME64DBLDBL(String Name,int d1,int d2) { name = Name; d1val = (double)d1; d2val = (double)d2;}  
  public NAME64DBLDBL(String Name,float d1,int d2) { name = Name; d1val = (double)d1; d2val = (double)d2;}  
  public NAME64DBLDBL(NAME64DBLDBL instance) 
  { 
    name = instance.name == null ? new String(new StringBuffer(64)) : new String(instance.name); 
    d1val = instance.d1val; d2val = instance.d2val; 
  }
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof NAME64DBLDBL)
    {
      NAME64DBLDBL instance = (NAME64DBLDBL)from;
      name = instance.name;
      d1val = instance.d1val; d2val = instance.d2val;  
    }
  }
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
      return "["+ name + separator + d1val + separator + d2val + "]";
  }
  public String getCsvString()
  {
    String tname = name.trim();
    if (tname.contains(",") || tname.contains(" ") ||
        tname.contains("\t") || tname.contains("\n"))
    {
      return "\""+tname+" "+d1val+" "+d2val+"\"";
    }
    return tname + " "+d1val+" "+d2val;
  }
  public NAME64DBLDBL newInstance()
  {
    return new NAME64DBLDBL();
  }
  public NAME64DBLDBL clone()
  {
    return new NAME64DBLDBL(this);
  }              
  public boolean equals(NAME64DBLDBL instance)
  {
    if (instance.name == null || name == null) return false;
    if (instance.name.compareTo(name) != 0) return false;
    if (instance.d1val != d1val) return false;
    if (instance.d2val != d2val) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    if (name == null) name = new String(new StringBuffer(64));
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] bname = name.getBytes();
      byte[] b = new byte[64];
      for (int i=0; i<64 && i<bname.length; i++) b[i] = bname[i];
      ds.write(b,0,b.length);
      byte[] db = Swap.Double(d1val);
      ds.write(db,0,8);
      db = Swap.Double(d2val);
      ds.write(db,0,8);
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAME64DBLDBL","could not prepare byte stream",TErrorList.code_failure,e,0);
      return null;
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      byte[] b = new byte[sizeInBytes];
      ds.read(b,0,64);
      int len = 0; for (int i=0; i<64 && b[i] != 0; i++, len++);
      name = new String(b,0,len);
      byte[] db = new byte[8];
      ds.read(db,0,8);
      d1val = Swap.Double(db);
      ds.read(db,0,8);
      d2val = Swap.Double(db);
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("NAME64DBLDBL","could not prepare byte stream",TErrorList.code_failure,e,0);
    }  
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    long temp;
    temp = Double.doubleToLongBits(d1val);
    result = PRIME * result + (int) (temp ^ (temp >>> 32));
    temp = Double.doubleToLongBits(d2val);
    result = PRIME * result + (int) (temp ^ (temp >>> 32));
    result = PRIME * result + ((name == null) ? 0 : name.hashCode());
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final NAME64DBLDBL other = (NAME64DBLDBL) obj;
    return equals(other);
  }  
}