/*
 * Created on Jan 31, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.types;

/**
 * interface for most TINE complex data types
 * 
 * @author duval
 *
 */
public interface TCompoundDataObject
{
  /**
   * Active method to convert the relevant data fields of this compound data 
   * type into the underlying byte stream (data being sent).  This primarily involves byte swapping 
   * from big-endian to little-endian.  Used systematically by TDataType() and other
   * routines.
   *
   * @return a reference to the underlying bytestream (as a courtesy) 
   */
  public byte[] toByteArray();
  /**
   * Active method to convert the underlying byte stream to the relevant data fields of
   * this compound data type. (data being received).  This primarily involved swapping from
   * little-endian to big-endian.  Used systematically by TDataType() and other routines.
   * @return
   */
  public void toStruct();
  /**
   * Passive method that simple returns the reference to the underlying bytestream
   * Used by TDataType(), TTaggedStruct, and THistoryRecord.
   * @return a reference to the underlying bytestream (if any)
   */
  public byte[] getByteArray();
  /**
   * Sets the underlying bytestream to that give
   * Used only by the HISTORY data type for manipulating compound data types.
   * @param byteArray a new reference to an underlying bytestream
   */
  public void setByteArray(byte[] byteArray);
  /**
   * Used by TDataType, TTaggedStruct and other routines.
   * @return the size in bytes of the of this compound data type
   */
  public int getSizeInBytes();
  /**
   * Used by TDataType, TTaggedStruct and other routines.
   * @return the format code of this compound data type
   */
  public short getFormat();
  /**
   * Used by TDataType, TTaggedStruct and other routines.
   * @return the data tag of this compound data type
   */
  public String getTag();
  /**
   * Used only by the THistoryRecord class in retrieving the primary double floating
   * point value for this compound data object (if available).  If the compound data
   * object has no double value, then a '0' is returned.
   * @return the canonical double value part of this compound data object.  
   */
  public double getDoubleValue();
  /**
   * Used only by the THistoryRecord class in retrieving the primary 32-bit integer
   * value for this compound data object (if available).  If the compound data
   * object has no integer value, then a '0' is returned.
   * @return the canonical integer value part of this compound data object.  
   */
  public int getIntegerValue();  
  /**
   * Used by TDataType.toCsvString() to get a valid parse-able string to 
   * of the objects data contents to supply to csv files.
   */
  public String getCsvString();
  public TCompoundDataObject clone();
  public void copy(TCompoundDataObject from);
  public TCompoundDataObject newInstance();
  /**
   * sets the field separator token used the the toString() method
   * @param value
   */
  public void setFieldSeparator(String value);
}
