package de.desy.tine.types;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
public final class UNAME implements TCompoundDataObject
{
  public int ival;
  public float f1val;
  public float f2val;
  public float f3val;
  public String str;
  public double getDoubleValue() { return (double)f1val; }
  public int getIntegerValue() { return (int)ival; }
  public static final int sizeInBytes = 32;
  public static final short dFormat = TFormat.CF_UNAME;
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
    return "["+ ival + separator + f1val + separator + f2val + separator + f3val + separator + str + "]";
  }
  public String getCsvString()
  {
    String tstr = str.trim();
    if (tstr.contains(",") || tstr.contains(" ") ||
        tstr.contains("\t") || tstr.contains("\n"))
    {
      return "\""+ival+" "+f1val+" "+f2val+" "+f3val+" "+tstr+"\""; 
    }
    return ival+" "+f1val+" "+f2val+" "+f3val+" "+tstr;
  }  
  public void setValues(int newIValue, float newF1Value, float newF2Value, float newF3Value, String newString)
  {
    ival = newIValue; f1val = newF1Value; f2val = newF2Value; f3val = newF3Value;
    str = newString;
  }
  public void setString(String newString) { str = newString; }
  public String getString() { return str; }
  public int getSizeInBytes() { return sizeInBytes; }
  public short getFormat() { return dFormat; }    
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer;
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public byte[] hByteArray;
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public UNAME() {f1val = 0; f2val = 0; ival = 0; f3val  = 0; str = "";}  
  public UNAME(float f1,String s) {f1val = f1; f2val = 0; ival = 0; f3val  = 0; str = s;}  
  public UNAME(float f1, float f2,String s) {f1val = f1; f2val = f2; ival = 0; f3val  = 0; str = s;}  
  public UNAME(int i,String s) {f1val = 0; f2val = 0; ival = i; f3val  = 0; str = s;}  
  public UNAME(int i,float f1,float f2,float f3,String s) {f1val = f1; f2val = f2; ival = i; f3val  = f3; str = s;}  
  public UNAME(UNAME instance) 
  {
    f1val = instance.f1val; f2val = instance.f2val; 
    ival = instance.ival; f3val  = instance.f3val; 
    str = new String(instance.str);
  }  
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof UNAME)
    {
      UNAME instance = (UNAME)from;
      f1val = instance.f1val; f2val = instance.f2val; 
      ival = instance.ival; f3val  = instance.f3val; 
      str = new String(instance.str);
    }
  }
  public UNAME newInstance()
  {
    return new UNAME();
  }
  public UNAME clone()
  {
    return new UNAME(this);
  }              
  public boolean equals(UNAME instance)
  {
    if (instance.ival != ival) return false;
    if (instance.f1val != f1val) return false;
    if (instance.f2val != f2val) return false;
    if (instance.f3val != f3val) return false;
    if (instance.str == null || str == null) return false;
    if (instance.str.compareTo(str)!= 0) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer = new ByteArrayOutputStream(sizeInBytes);
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] b;
      ds.writeInt(Swap.Int(ival));
      b = Swap.Float(f1val); ds.write(b,0,4);
      b = Swap.Float(f2val); ds.write(b,0,4);
      b = Swap.Float(f3val); ds.write(b,0,4);
      b = str.getBytes();
      int len = b.length;
      if (len < 16)
      {
        ds.write(str.getBytes(),0,len);
        b = new byte[16];
        ds.write(b,0,16-len);
      }
      else
      {
        ds.write(str.getBytes(),0,16);        
      }
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("UNAME.toByteArray",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }    
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      ival = Swap.Int(ds.readInt());
      byte[] b = new byte[80];
      ds.read(b,0,4); f1val = Swap.Float(b);
      ds.read(b,0,4); f2val = Swap.Float(b);
      ds.read(b,0,4); f3val = Swap.Float(b);
      ds.read(b,0,16);
      int i;
      for (i=0; i<16 && b[i] != 0; i++);
      str = (new String(b)).substring(0,i);
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("UNAME.toStruct",e.getMessage(),TErrorList.code_failure,e,0);        
    }	 
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(f1val);
    result = PRIME * result + Float.floatToIntBits(f2val);
    result = PRIME * result + Float.floatToIntBits(f3val);
    result = PRIME * result + ival;
    result = PRIME * result + ((str == null) ? 0 : str.hashCode());
    return result;
  }
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final UNAME other = (UNAME) obj;
    return equals(other);
  }    
}