package de.desy.tine.types;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;

public final class USTRING implements TCompoundDataObject
{
  public int ival;
  public float f1val;
  public float f2val;
  public int tm;
  public String str;
  public double getDoubleValue() { return (double)f1val; }
  public int getIntegerValue() { return (int)ival; }
  public static final int sizeInBytes = 96;
  public static final short dFormat = TFormat.CF_USTRING;
  protected String separator = ", ";
  public void setFieldSeparator(String value)
  {
    if (value != null) separator = value;
  }
  public String toString()
  {
    return "["+ ival + separator + f1val + separator + f2val + separator + tm + separator + str.trim() + "]";
  }
  public String getCsvString()
  {
    String tstr = str.trim();
    if (tstr.contains(",") || tstr.contains(" ") ||
        tstr.contains("\t") || tstr.contains("\n"))
    {
      return "\""+ival+" "+f1val+" "+f2val+" "+tm+" "+tstr+"\""; 
    }
    return ival+" "+f1val+" "+f2val+" "+tm+" "+tstr;
  }  
  public void setValues(int newIValue, float newF1Value, float newF2Value, int newTMValue, String newString)
  {
    ival = newIValue;
    f1val = newF1Value;
    f2val = newF2Value;
    tm = newTMValue;
    str = newString;
  }
  public void setString(String newString)
  {
    str = newString;
  }
  public String getString()
  {
    return str;
  }
  public int getSizeInBytes()
  {
    return sizeInBytes;
  }
  public short getFormat()
  {
    return dFormat;
  }
  public String getTag() { return null; }
  ByteArrayOutputStream dBuffer;
  public byte[] hByteArray;
  public byte[] getByteArray() 
  { 
    return hByteArray == null ? toByteArray() : hByteArray; 
  }
  public void setByteArray(byte[] byteArray) { hByteArray = byteArray; }
  public USTRING()
  {
    f1val = 0;
    f2val = 0;
    ival = 0;
    tm = 0;
    str = "";
  }
  public USTRING(float f1, String s)
  {
    f1val = f1;
    f2val = 0;
    ival = 0;
    tm = 0;
    str = s;
  }
  public USTRING(float f1, float f2, String s)
  {
    f1val = f1;
    f2val = f2;
    ival = 0;
    tm = 0;
    str = s;
  }
  public USTRING(int i, String s)
  {
    f1val = 0;
    f2val = 0;
    ival = i;
    tm = 0;
    str = s;
  }
  public USTRING(String s)
  {
    f1val = 0;
    f2val = 0;
    ival = 0;
    tm = 0;
    str = s;
  }
  public USTRING(int i, float f1, float f2, int tm, String s)
  {
    f1val = f1;
    f2val = f2;
    ival = i;
    this.tm = tm;
    str = s;
  }
  public USTRING(USTRING instance)
  {
    f1val = instance.f1val;
    f2val = instance.f2val;
    ival = instance.ival;
    tm = instance.tm;
    str = new String(instance.str);
  }
  @Override
  public void copy(TCompoundDataObject from)
  {
    if (from instanceof USTRING)
    {
      USTRING instance = (USTRING)from;
      f1val = instance.f1val;
      f2val = instance.f2val;
      ival = instance.ival;
      tm = instance.tm;
      str = new String(instance.str);
    }
  }
  public boolean equals(USTRING instance)
  {
    if (instance.ival != ival) return false;
    if (instance.f1val != f1val) return false;
    if (instance.f2val != f2val) return false;
    if (instance.tm != tm) return false;
    if (instance.str == null || str == null) return false;
    if (instance.str.compareTo(str)!= 0) return false;
    return true;
  }
  public byte[] toByteArray()
  {
    try
    {
      dBuffer = new ByteArrayOutputStream(sizeInBytes);
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] b;
      ds.writeInt(Swap.Int(ival));
      b = Swap.Float(f1val);
      ds.write(b, 0, 4);
      b = Swap.Float(f2val);
      ds.write(b, 0, 4);
      ds.writeInt(Swap.Int(tm));
      b = str.getBytes();
      int len = b.length;
      if (len < 80)
      {
        ds.write(b);
        b = new byte[80];
        ds.write(b, 0, 80 - len);
      }
      else
      {
        ds.write(b, 0, 80);
      }
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("USTRING.toByteArray",e.getMessage(),TErrorList.code_failure,e,0);        
    }
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      ival = Swap.Int(ds.readInt());
      byte[] b = new byte[80];
      ds.read(b, 0, 4);
      f1val = Swap.Float(b);
      ds.read(b, 0, 4);
      f2val = Swap.Float(b);
      tm = Swap.Int(ds.readInt());
      ds.read(b, 0, 80);
      str = new String(b);
      //int siz = 0;
      //while (b[siz] != 0 && siz < 80) siz++;
      //str = siz > 0 ? new String(b,0,siz) : "";
      //for (i = 0; i < 80 && b[i] != 0; i++);
      //str = (new String(b)).substring(0, i);
      ds.close();
      dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("USTRING.toStruct",e.getMessage(),TErrorList.code_failure,e,0);        
    }
  }
  public int hashCode()
  {
    final int PRIME = 31;
    int result = 1;
    result = PRIME * result + Float.floatToIntBits(f1val);
    result = PRIME * result + Float.floatToIntBits(f2val);
    result = PRIME * result + ival;
    result = PRIME * result + ((str == null) ? 0 : str.hashCode());
    result = PRIME * result + tm;
    return result;
  }
  public USTRING newInstance()
  {
    return new USTRING();
  }
  public USTRING clone()
  {
    return new USTRING(this);
  }              
  public boolean equals(Object obj)
  {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    final USTRING other = (USTRING) obj;
    return equals(other);
  }
}