package de.desy.video.sw;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.Transparency;
//import java.awt.Toolkit;
import javax.swing.ImageIcon;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.HeadlessException;

import java.io.*;
import javax.imageio.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

//import de.desy.tine.client.*;
//import de.desy.tine.dataUtils.*;
//import de.desy.tine.definitions.*;
import de.desy.tine.types.*;


public final class CBasicPNGWriterV3 {

	// This method returns a buffered image with the contents of an image
	public static BufferedImage toBufferedImage(Image image) 
	{
		if (image instanceof BufferedImage) 
		{
			return (BufferedImage)image;
		}

		// This code ensures that all the pixels in the image are loaded
		image = new ImageIcon(image).getImage();

		// Determine if the image has transparent pixels; for this method's
		// implementation, see e661 Determining If an Image Has Transparent Pixels
		boolean hasAlpha = false;

		// Create a buffered image with a format that's compatible with the screen
		BufferedImage bimage = null;
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		
		try 
		{
			// Determine the type of transparency of the new buffered image
			int transparency = Transparency.OPAQUE;
			if (hasAlpha) 
			{
				transparency = Transparency.BITMASK;
			}

			// Create the buffered image
			GraphicsDevice gs = ge.getDefaultScreenDevice();
			GraphicsConfiguration gc = gs.getDefaultConfiguration();
			bimage = gc.createCompatibleImage(
					image.getWidth(null), image.getHeight(null), transparency);
		} 
		catch (HeadlessException e) 
		{
			// The system does not have a screen
		}

		if (bimage == null) 
		{
			// Create a buffered image using the default color model
			int type = BufferedImage.TYPE_INT_RGB;
			if (hasAlpha) 
			{
				type = BufferedImage.TYPE_INT_ARGB;
			}
			bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
		}

		// Copy image to buffered image
		Graphics g = bimage.createGraphics();

		// Paint the image onto the buffered image
		g.drawImage(image, 0, 0, null);
		g.dispose();

		return bimage;
	}

	public static boolean write( String aFileName, Image aImg, CVideoHeader3 aHdr)
	{
		return write(new File(aFileName), aImg, aHdr);
	}

	public static boolean write( File aFile, Image aImg, CVideoHeader3 aHdr)
	{
		try 
		{
			ImageIO.write(toBufferedImage(aImg), "png", aFile);
			
			
			// add .txt header afterwards
			String wholename = aFile.getAbsolutePath();
			wholename += ".txt";
			
		    Writer output = null;
		    try {
		      //use buffering
		      //FileWriter always assumes default encoding is OK!
		      File txtF = new File(wholename);
		      txtF.createNewFile();
		      
		      String n = System.getProperty("line.separator");
		      DateFormat dfmt = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
		      
		      output = new BufferedWriter( new FileWriter(txtF) );

		      IMAGE.SourceHeader srcH = aHdr.sourceHeader;
		      IMAGE.FrameHeader  frmH = aHdr.frameHeader;
		      
		      output.write("SourceHeader."+n);
		      output.write(" .version             "+srcH.versionTag+n);
		      output.write(" .timestamp           "+dfmt.format(aHdr.getTimestampAsDate())+n);
		      output.write(" .cameraport          \""+srcH.cameraPortName+"\""+n);
		      output.write(" .total_length        "+srcH.totalLength+n);
		      
		      output.write(" FrameHeader"+n);
		      output.write("       .sourceWidth         	"+frmH.sourceWidth+n);
		      output.write("       .sourceHeight        	"+frmH.sourceHeight+n);
		      output.write("       .aoiWidth     			"+frmH.aoiWidth+n);
		      output.write("       .aoiHeight    			"+frmH.aoiHeight+n);
		      output.write("       .xStart       			"+frmH.xStart+n);
		      output.write("       .yStart       			"+frmH.yStart+n);
		      output.write("       .bytesPerPixel    		"+frmH.bytesPerPixel+n);
		      output.write("       .effectiveBitsPerPixel 	"+frmH.effectiveBitsPerPixel+n);
		      output.write("       .horizontalBinning   	"+frmH.horizontalBinning+n);
		      output.write("       .verticalBinning  		"+frmH.verticalBinning+n);
		      output.write("       .sourceFormat    		"+CVideoHeader3.formatToString(frmH.sourceFormat)+n);
		      output.write("       .currentFormat    		"+CVideoHeader3.formatToString(frmH.imageFormat)+n);
		      output.write("       .frameNumber   			"+frmH.frameNumber+n);
		      output.write("       .eventNumer    			"+frmH.eventNumber+n);		      
		      output.write("       .xScale       			"+frmH.xScale+n);
		      output.write("       .yScale       			"+frmH.yScale+n);
		      output.write("       .imageRotation   		"+frmH.imageRotation+n);
		      output.write("       .imageFlags   			"+CVideoHeader3.flagsToString(frmH.imageFlags)+n);
		      output.write("       .appendedFrameSize 		"+frmH.appendedFrameSize+n);
		      output.write("Header END"+n);
		    }
		    catch( IOException ex1 )
		    {
		    	return false;
		    }
		    finally 
		    {
		      //flush and close both "output" and its underlying FileWriter
		      if (output != null) output.close();
		    }
						
		}
		catch( IOException ex )
		{
			return false;
		}
		
		return true;
	}
	
}