package de.desy.video.sw;

import java.util.Date;
//import java.lang.Float;
import de.desy.tine.types.*;
import de.desy.tine.types.IMAGE.*;

public final class CVideoHeader3 {

	private IMAGE m_intHdr = null; // image header from TINE
	public FrameHeader frameHeader = null;
	public SourceHeader sourceHeader = null;
	
	public static final int HDRSIZE=188;
	
	//private final long MAX_INT=0x7FFFFFFF; // 2415919103

	public static final long CF_IMAGE_MAGIC_01 = 0x33565356; // VSV3
	public static final long CF_IMAGE_MAGIC_02 = 0x38303032; // 2008

	public static final int CF_IMAGE_INITIAL_VERSION	= 0x1;
	public static final int CF_IMAGE_VERSION			= CF_IMAGE_INITIAL_VERSION;
	public static final int CF_IMAGE_MAX_VERSION		= CF_IMAGE_INITIAL_VERSION;

	
	public static final long CF_IMAGE_FORMAT_GRAY		= 0;
	public static final long CF_IMAGE_FORMAT_RGB		= 1;
	public static final long CF_IMAGE_FORMAT_RGBA		= 2;
	public static final long CF_IMAGE_FORMAT_RGB15		= 3;
	public static final long CF_IMAGE_FORMAT_RGB16		= 4;
	public static final long CF_IMAGE_FORMAT_JPEG		= 5;
	public static final long CF_IMAGE_FORMAT_TIFF		= 6;
	public static final long CF_IMAGE_FORMAT_YUV411		= 7;
	public static final long CF_IMAGE_FORMAT_YUV422		= 8;
	public static final long CF_IMAGE_FORMAT_YUV444		= 9;
	public static final long CF_IMAGE_FORMAT_HUFFYUV	= 10;
	public static final long CF_IMAGE_FORMAT_BMP		= 11;
	public static final long CF_IMAGE_FORMAT_BAYER		= 12;
	
	public static final long CF_IMAGE_FORMAT_FIRST		= CF_IMAGE_FORMAT_GRAY;
	public static final long CF_IMAGE_FORMAT_LAST		= CF_IMAGE_FORMAT_BAYER;
	
	public static final long CF_IMAGE_FLAG_LITTLE_ENDIAN_BYTE_ORDER	= 0x1;
	public static final long CF_IMAGE_FLAG_IMAGE_LOSSLESS			= 0x2;
	public static final long CF_IMAGE_FLAG_HORIZONTAL_FLIP_180		= 0x4;
	public static final long CF_IMAGE_FLAG_VERTICAL_FLIP_180			= 0x8;

	
	public CVideoHeader3()
	{
		m_intHdr = new IMAGE();
	
		frameHeader = m_intHdr.getFrameHeader();
		sourceHeader = m_intHdr.getSourceHeader();
		
		init();
	}

	public CVideoHeader3( IMAGE img )
	{	
		m_intHdr = img;
		sourceHeader = img.getSourceHeader();
		frameHeader = img.getFrameHeader();
	}
	
	public IMAGE getImage() {
		return m_intHdr;
	}
	
	public static final String flagsToString(long aFlags)
	{
		int flags = (int) aFlags;
		
		String retString = "";
		
		if ((flags&CF_IMAGE_FLAG_LITTLE_ENDIAN_BYTE_ORDER) != 0) 
			retString += "LITTLEENDIAN"; 
		else
			retString += "BIGENDIAN";			
		
		if ((flags&CF_IMAGE_FLAG_IMAGE_LOSSLESS) != 0) 
			retString += " LOSSLESS"; 
		else
			retString += " LOSSY";			
		
		if ((flags&CF_IMAGE_FLAG_HORIZONTAL_FLIP_180) != 0) 
			retString += " HORIZONTAL_FLIP_180"; 

		if ((flags&CF_IMAGE_FLAG_VERTICAL_FLIP_180) != 0) 
			retString += " VERTICAL_FLIP_180"; 
	
		return retString;
	}
	
	public final Date getTimestampAsDate()
	{
		long l_timestamp = (sourceHeader.timestampSeconds*1000L)+(sourceHeader.timestampMicroseconds/1000L); 
	
		return new Date(l_timestamp);
	}
	
	public static final String formatToString(long aformat)
	{
		switch( ((int) aformat) )
		{
			case ((int)CF_IMAGE_FORMAT_GRAY):
				return("IMAGE_FORMAT_GRAY");
	
			case ((int)CF_IMAGE_FORMAT_RGB):
				return("IMAGE_FORMAT_RGB");
				
			case ((int)CF_IMAGE_FORMAT_RGBA):
				return("IMAGE_FORMAT_RGBA");

			case ((int)CF_IMAGE_FORMAT_RGB15):
				return("IMAGE_FORMAT_RGB15");

			case ((int)CF_IMAGE_FORMAT_RGB16):
				return("IMAGE_FORMAT_RGB16");

			case ((int)CF_IMAGE_FORMAT_JPEG):
				return("IMAGE_FORMAT_JPEG");

			case ((int)CF_IMAGE_FORMAT_TIFF):
				return("IMAGE_FORMAT_TIFF");

			case ((int)CF_IMAGE_FORMAT_YUV411):
				return("IMAGE_FORMAT_YUV411");

			case ((int)CF_IMAGE_FORMAT_YUV422):
				return("IMAGE_FORMAT_YUV422");

			case ((int)CF_IMAGE_FORMAT_YUV444):
				return("IMAGE_FORMAT_YUV444");

			case ((int)CF_IMAGE_FORMAT_HUFFYUV):
				return("IMAGE_FORMAT_HUFFYUV");

			case ((int)CF_IMAGE_FORMAT_BMP):
				return("IMAGE_FORMAT_BMP");

			case ((int)CF_IMAGE_FORMAT_BAYER):
				return("IMAGE_FORMAT_BAYER");

			default:
				return("Unknown!");
		}
	}
	
	public int getAppendedWidth()
	{
		if (frameHeader.aoiWidth != -1) 
			return frameHeader.aoiWidth;
		else
			return frameHeader.sourceWidth; 
	}
	
	public int getAppendedHeight()
	{
		if (frameHeader.aoiHeight != -1) 
			return frameHeader.aoiHeight;
		else
			return frameHeader.sourceHeight;
	}
	
	public void init()
	{
		sourceHeader.versionTag = CF_IMAGE_VERSION;
		sourceHeader.totalLength = HDRSIZE;
/*
		timestamp = null;
		name_of_cameraport="";

		calc_appended_width=-1;
		calc_appended_height=-1;
		
		imH_width = -1;
		imH_height = -1;
		imH_aoi_width = -1;
		imH_aoi_height = -1;
		imH_x_start = 0;
		imH_y_start = 0;
		imH_bypp = -1;
		imH_ebitpp = -1;
		imH_vbin = -1;
		imH_hbin = -1;
		imH_source_format = CF_IMAGE_FORMAT_GRAY;
		imH_image_format = CF_IMAGE_FORMAT_GRAY;
		
		imH_framenumber = -1;
		imH_eventnumber = -1;
		
		imH_scale_x = -1.0;
		imH_scale_y = -1.0;
		imH_image_rotation = 0.0;
		imH_fspare2 = -1.0;
		imH_fspare3 = -1.0;
		imH_fspare4 = -1.0;

		imH_image_flags = 0; // big endian byte order
		imH_ispare2 = -1;
		imH_ispare3 = -1;
		imH_ispare4 = -1;
		imH_length = 0;
		*/
	}
}

